@@ -1,15 +1,30 @@
 /*
   ==============================================================================
 
-    ToggleLightComponent.h
-    Created: 11 Feb 2015 9:56:51am
-    Author:  Felix Faire
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
+
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
+
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
+
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
+
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
+
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef TOGGLELIGHTCOMPONENT_H_INCLUDED
-#define TOGGLELIGHTCOMPONENT_H_INCLUDED
+#pragma once
 
 /**
     This class represents one of the individual lights in our grid.
@@ -25,12 +40,10 @@ public:
 
     void paint (Graphics& g) override
     {
-        g.fillAll (Colours::black);
-
         // Only shows the red ellipse when the button is on.
         if (isOn)
         {
-            g.setColour (Colours::red);
+            g.setColour (getLookAndFeel().findColour (Slider::thumbColourId));
             g.fillEllipse (getLocalBounds().toFloat());
         }
     }
@@ -48,6 +61,3 @@ private:
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ToggleLightComponent)
 };
-
-
-#endif  // TOGGLELIGHTCOMPONENT_H_INCLUDED
