@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -44,7 +46,7 @@ public:
         s.setJustification (Justification::centred);
         s.setWordWrap (AttributedString::none);
         s.append ("Drag Me!");
-        s.setColour (Colours::white);
+        s.setColour (findColour (TextButton::textColourOffId));
         s.draw (g, area);
     }
 
@@ -119,7 +121,6 @@ struct BallComponent  : public Component
 
 //==============================================================================
 class AnimationDemo  : public Component,
-                       private Button::Listener,
                        private Timer
 {
 public:
@@ -130,10 +131,10 @@ public:
 
         for (int i = 11; --i >= 0;)
         {
-            Button* b = createButton();
+            auto* b = createButton();
             componentsToAnimate.add (b);
             addAndMakeVisible (b);
-            b->addListener (this);
+            b->onClick = [this] { triggerAnimation(); };
         }
 
         addAndMakeVisible (ballGenerator);
@@ -147,7 +148,7 @@ public:
         for (int i = 0; i < componentsToAnimate.size(); ++i)
         {
             const int newIndex = (i + 3) % componentsToAnimate.size();
-            const float angle = newIndex * 2.0f * float_Pi / componentsToAnimate.size();
+            const float angle = newIndex * MathConstants<float>::twoPi / componentsToAnimate.size();
             const float radius = getWidth() * 0.35f;
 
             Rectangle<int> r (getWidth()  / 2 + (int) (radius * std::sin (angle)) - 50,
@@ -168,7 +169,7 @@ public:
 
     void paint (Graphics& g) override
     {
-        fillStandardDemoBackground (g);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground));
     }
 
 private:
@@ -251,12 +252,12 @@ private:
         return b;
     }
 
-    void buttonClicked (Button*) override
+    void triggerAnimation()
     {
         for (int i = 0; i < componentsToAnimate.size(); ++i)
         {
             const int newIndex = (i + 3 * cycleCount) % componentsToAnimate.size();
-            const float angle = newIndex * 2.0f * float_Pi / componentsToAnimate.size();
+            const float angle = newIndex * MathConstants<float>::twoPi / componentsToAnimate.size();
             const float radius = getWidth() * 0.35f;
 
             Rectangle<int> r (getWidth()  / 2 + (int) (radius * std::sin (angle)) - 50,
