@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -45,17 +47,17 @@ public:
 
     void paint (Graphics& g) override
     {
-        fillStandardDemoBackground (g);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground));
     }
 
     void resized() override
     {
-        Rectangle<int> r (getLocalBounds().reduced (4));
+        auto r = getLocalBounds().reduced (4);
 
         int buttonHeight = 22;
 
-        Rectangle<int> columns (r.removeFromTop (buttonHeight * 4));
-        Rectangle<int> col (columns.removeFromLeft (200));
+        auto columns = r.removeFromTop (buttonHeight * 4);
+        auto col = columns.removeFromLeft (200);
 
         animatePosition.setBounds (col.removeFromTop (buttonHeight));
         animateRotation.setBounds (col.removeFromTop (buttonHeight));
@@ -108,7 +110,7 @@ public:
         AffineTransform t;
 
         if (controls.animateRotation.getToggleState())
-            t = t.rotated (rotation.getValue() * float_Pi * 2.0f);
+            t = t.rotated (rotation.getValue() * MathConstants<float>::twoPi);
 
         if (controls.animateSize.getToggleState())
             t = t.scaled (0.3f + size.getValue() * 2.0f);
@@ -218,7 +220,7 @@ public:
 
         AffineTransform transform (AffineTransform::translation (clipImage.getWidth() / -2.0f,
                                                                  clipImage.getHeight() / -2.0f)
-                                   .rotated (clipImageAngle.getValue() * float_Pi * 2.0f)
+                                   .rotated (clipImageAngle.getValue() * MathConstants<float>::twoPi)
                                    .scaled (2.0f + clipImageSize.getValue() * 3.0f)
                                    .translated (getWidth() * 0.5f,
                                                 getHeight() * 0.5f));
@@ -483,9 +485,7 @@ public:
             if (svgDrawable != nullptr)
             {
                 // to make our icon the right size, we'll set its bounding box to the size and position that we want.
-                svgDrawable->setBoundingBox (RelativeParallelogram (Point<float> (-100, -100),
-                                                                    Point<float> (100, -100),
-                                                                    Point<float> (-100, 100)));
+                svgDrawable->setBoundingBox ({ -100.0f, -100.0f, 200.0f, 200.0f });
             }
         }
     }
@@ -571,7 +571,7 @@ public:
 
     void paint (Graphics& g) override
     {
-        g.fillCheckerBoard (getLocalBounds(), 48, 48,
+        g.fillCheckerBoard (getLocalBounds().toFloat(), 48.0f, 48.0f,
                             Colours::lightgrey, Colours::white);
     }
 
@@ -630,7 +630,6 @@ public:
         addAndMakeVisible (listBox);
         listBox.setModel (this);
         listBox.selectRow (0);
-        listBox.setColour (ListBox::backgroundColourId, Colour::greyLevel (0.9f));
     }
 
     void resized()
@@ -645,14 +644,13 @@ public:
 
     void paintListBoxItem (int rowNumber, Graphics& g, int width, int height, bool rowIsSelected)
     {
-        Component* demo = demos [rowNumber];
-
-        if (demo != nullptr)
+        if (auto* demo = demos[rowNumber])
         {
             if (rowIsSelected)
-                g.fillAll (findColour (TextEditor::highlightColourId));
+                g.fillAll (Colour::contrasting (findColour (ListBox::textColourId),
+                                                findColour (ListBox::backgroundColourId)));
 
-            g.setColour (Colours::black);
+            g.setColour (findColour (ListBox::textColourId));
             g.setFont (14.0f);
             g.drawFittedText (demo->getName(), 8, 0, width - 10, height, Justification::centredLeft, 2);
         }
@@ -693,7 +691,7 @@ public:
 
     void resized() override
     {
-        Rectangle<int> area (getLocalBounds());
+        auto area = getLocalBounds();
         controllersComponent.setBounds (area.removeFromBottom (150));
         testList.setBounds (area.removeFromRight (150));
         demoHolder.setBounds (area);
