@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -30,8 +32,7 @@
     flag is used to promt the user to save the note when it is closed.
  */
 class Note    : public Component,
-                public FileBasedDocument,
-                private TextEditor::Listener
+                public FileBasedDocument
 {
 public:
     Note (const String& name, const String& contents)
@@ -48,12 +49,7 @@ public:
         editor.setReturnKeyStartsNewLine (true);
         editor.getTextValue().referTo (textValueObject);
         addAndMakeVisible (editor);
-        editor.addListener (this);
-    }
-
-    ~Note()
-    {
-        editor.removeListener (this);
+        editor.onTextChange = [this] { changed(); };
     }
 
     void resized() override
@@ -86,7 +82,7 @@ public:
     File getLastDocumentOpened() override
     {
         // not interested in this for now
-        return File();
+        return {};
     }
 
     void setLastDocumentOpened (const File& /*file*/) override
@@ -105,17 +101,11 @@ private:
     Value textValueObject;
     TextEditor editor;
 
-    void textEditorTextChanged (TextEditor& ed) override
+    void lookAndFeelChanged() override
     {
-        // let our FileBasedDocument know we've changed
-        if (&ed == &editor)
-            changed();
+        editor.applyFontToAllText (editor.getFont());
     }
 
-    void textEditorReturnKeyPressed (TextEditor&) override {}
-    void textEditorEscapeKeyPressed (TextEditor&) override {}
-    void textEditorFocusLost (TextEditor&) override {}
-
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Note)
 };
 
@@ -140,6 +130,8 @@ public:
        #if JUCE_MODAL_LOOPS_PERMITTED
         if (Note* note = dynamic_cast<Note*> (component))
             return note->saveIfNeededAndUserAgrees() != FileBasedDocument::failedToWriteToFile;
+       #else
+        ignoreUnused (component);
        #endif
 
         return true;
@@ -154,8 +146,7 @@ private:
     By default this will look for notes saved to the desktop and load them up.
  */
 class MDIDemo   : public Component,
-                  public FileDragAndDropTarget,
-                  private Button::Listener
+                  public FileDragAndDropTarget
 {
 public:
     MDIDemo()
@@ -164,11 +155,11 @@ public:
 
         showInTabsButton.setButtonText ("Show with tabs");
         showInTabsButton.setToggleState (false, dontSendNotification);
-        showInTabsButton.addListener (this);
+        showInTabsButton.onClick = [this] { updateLayoutMode(); };
         addAndMakeVisible (showInTabsButton);
 
         addNoteButton.setButtonText ("Create a new note");
-        addNoteButton.addListener (this);
+        addNoteButton.onClick = [this] { addNote (String ("Note ") + String (multiDocumentPanel.getNumDocuments() + 1), "Hello World!"); };
         addAndMakeVisible (addNoteButton);
 
         addAndMakeVisible (multiDocumentPanel);
@@ -179,15 +170,9 @@ public:
         addExistingNotes();
     }
 
-    ~MDIDemo()
-    {
-        addNoteButton.removeListener (this);
-        showInTabsButton.removeListener (this);
-    }
-
     void paint (Graphics& g) override
     {
-        fillStandardDemoBackground (g);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground));
     }
 
     void resized() override
@@ -217,11 +202,9 @@ public:
 
     void createNotesForFiles (const Array<File>& files)
     {
-        for (int i = 0; i < files.size(); ++i)
+        for (auto& file : files)
         {
-            const File file (files[i]);
-
-            String content = file.loadFileAsString();
+            auto content = file.loadFileAsString();
 
             if (content.length() > 20000)
                 content = "Too long!";
@@ -251,17 +234,8 @@ private:
 
     void addExistingNotes()
     {
-        Array<File> files;
-        File::getSpecialLocation (File::userDesktopDirectory).findChildFiles (files, File::findFiles, false, "*.jnote");
-        createNotesForFiles (files);
-    }
-
-    void buttonClicked (Button* b) override
-    {
-        if (b == &showInTabsButton)
-            updateLayoutMode();
-        else if (b == &addNoteButton)
-            addNote (String ("Note ") + String (multiDocumentPanel.getNumDocuments() + 1), "Hello World!");
+        createNotesForFiles (File::getSpecialLocation (File::userDesktopDirectory)
+                                .findChildFiles (File::findFiles, false, "*.jnote"));
     }
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MDIDemo)
