@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -42,7 +44,8 @@ public:
 
         if (isPositiveAndBelow (row, midiMessageList.size()))
         {
-            g.setColour (Colours::black);
+            g.setColour (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::defaultText,
+                                                 Colours::black));
 
             const MidiMessage& message = midiMessageList.getReference (row);
             double time = message.getTimeStamp();
@@ -65,7 +68,6 @@ private:
 
 //==============================================================================
 class MidiDemo  : public Component,
-                  private ComboBox::Listener,
                   private MidiInputCallback,
                   private MidiKeyboardStateListener,
                   private AsyncUpdater
@@ -73,7 +75,6 @@ class MidiDemo  : public Component,
 public:
     MidiDemo()
         : deviceManager (MainAppWindow::getSharedAudioDeviceManager()),
-          lastInputIndex (0), isAddingFromMidiInput (false),
           keyboardComponent (keyboardState, MidiKeyboardComponent::horizontalKeyboard),
           midiLogListBoxModel (midiMessageList)
     {
@@ -86,9 +87,10 @@ public:
 
         addAndMakeVisible (midiInputList);
         midiInputList.setTextWhenNoChoicesAvailable ("No MIDI Inputs Enabled");
-        const StringArray midiInputs (MidiInput::getDevices());
+
+        auto midiInputs = MidiInput::getDevices();
         midiInputList.addItemList (midiInputs, 1);
-        midiInputList.addListener (this);
+        midiInputList.onChange = [this] { setMidiInput (midiInputList.getSelectedItemIndex()); };
 
         // find the first enabled device and use that by default
         for (int i = 0; i < midiInputs.size(); ++i)
@@ -113,32 +115,29 @@ public:
         addAndMakeVisible (midiOutputList);
         midiOutputList.setTextWhenNoChoicesAvailable ("No MIDI Output Enabled");
         midiOutputList.addItemList (MidiOutput::getDevices(), 1);
-        midiOutputList.addListener (this);
+        midiOutputList.onChange = [this] { setMidiOutput (midiOutputList.getSelectedItemIndex()); };
 
         addAndMakeVisible (keyboardComponent);
         keyboardState.addListener (this);
 
         addAndMakeVisible (messageListBox);
         messageListBox.setModel (&midiLogListBoxModel);
-        messageListBox.setColour (ListBox::backgroundColourId, Colour (0x32ffffff));
-        messageListBox.setColour (ListBox::outlineColourId, Colours::black);
     }
 
     ~MidiDemo()
     {
         keyboardState.removeListener (this);
         deviceManager.removeMidiInputCallback (MidiInput::getDevices()[midiInputList.getSelectedItemIndex()], this);
-        midiInputList.removeListener (this);
     }
 
     void paint (Graphics& g) override
     {
-        fillStandardDemoBackground (g);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground));
     }
 
     void resized() override
     {
-        Rectangle<int> area (getLocalBounds());
+        auto area = getLocalBounds();
         midiInputList.setBounds (area.removeFromTop (36).removeFromRight (getWidth() - 150).reduced (8));
         midiOutputList.setBounds (area.removeFromTop (36).removeFromRight (getWidth() - 150).reduced (8));
         keyboardComponent.setBounds (area.removeFromTop (80).reduced(8));
@@ -149,8 +148,8 @@ private:
     AudioDeviceManager& deviceManager;
     ComboBox midiInputList, midiOutputList;
     Label midiInputListLabel, midiOutputListLabel;
-    int lastInputIndex;
-    bool isAddingFromMidiInput;
+    int lastInputIndex = 0;
+    bool isAddingFromMidiInput = false;
 
     MidiKeyboardState keyboardState;
     MidiKeyboardComponent keyboardComponent;
@@ -164,11 +163,11 @@ private:
     /** Starts listening to a MIDI input device, enabling it if necessary. */
     void setMidiInput (int index)
     {
-        const StringArray list (MidiInput::getDevices());
+        auto list = MidiInput::getDevices();
 
         deviceManager.removeMidiInputCallback (list[lastInputIndex], this);
 
-        const String newInput (list[index]);
+        auto newInput = list[index];
 
         if (! deviceManager.isMidiInputEnabled (newInput))
             deviceManager.setMidiInputEnabled (newInput, true);
@@ -182,7 +181,7 @@ private:
     //==============================================================================
     void setMidiOutput (int index)
     {
-        currentMidiOutput = nullptr;
+        currentMidiOutput.reset();
 
         if (MidiOutput::getDevices() [index].isNotEmpty())
         {
@@ -191,12 +190,6 @@ private:
         }
     }
 
-    void comboBoxChanged (ComboBox* box) override
-    {
-        if (box == &midiInputList)    setMidiInput  (midiInputList.getSelectedItemIndex());
-        if (box == &midiOutputList)   setMidiOutput (midiOutputList.getSelectedItemIndex());
-    }
-
     // These methods handle callbacks from the midi device + on-screen keyboard..
     void handleIncomingMidiMessage (MidiInput*, const MidiMessage& message) override
     {
@@ -209,7 +202,7 @@ private:
     {
         if (! isAddingFromMidiInput)
         {
-            MidiMessage m (MidiMessage::noteOn (midiChannel, midiNoteNumber, velocity));
+            auto m = MidiMessage::noteOn (midiChannel, midiNoteNumber, velocity);
             m.setTimeStamp (Time::getMillisecondCounterHiRes() * 0.001);
             postMessageToList (m);
         }
@@ -219,7 +212,7 @@ private:
     {
         if (! isAddingFromMidiInput)
         {
-            MidiMessage m (MidiMessage::noteOff (midiChannel, midiNoteNumber, velocity));
+            auto m = MidiMessage::noteOff (midiChannel, midiNoteNumber, velocity);
             m.setTimeStamp (Time::getMillisecondCounterHiRes() * 0.001);
             postMessageToList (m);
         }
