@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -27,8 +29,6 @@
 
 //==============================================================================
 class NetworkingDemo   : public Component,
-                         private Button::Listener,
-                         private TextEditor::Listener,
                          private Thread
 {
 public:
@@ -40,26 +40,26 @@ public:
 
         addAndMakeVisible (urlBox);
         urlBox.setText ("https://www.google.com");
-        urlBox.addListener (this);
+        urlBox.onReturnKey = [this] { fetchButton.triggerClick(); };
 
         addAndMakeVisible (fetchButton);
         fetchButton.setButtonText ("Download URL Contents");
-        fetchButton.addListener (this);
+        fetchButton.onClick = [this] { startThread(); };
 
         addAndMakeVisible (resultsBox);
     }
 
     void paint (Graphics& g) override
     {
-        fillStandardDemoBackground (g);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground));
     }
 
     void resized() override
     {
-        Rectangle<int> area (getLocalBounds());
+        auto area = getLocalBounds();
 
         {
-            Rectangle<int> topArea (area.removeFromTop (40));
+            auto topArea = area.removeFromTop (40);
             fetchButton.setBounds (topArea.removeFromRight (180).reduced (8));
             urlBox.setBounds (topArea.reduced (8));
         }
@@ -105,15 +105,9 @@ private:
     CodeDocument resultsDocument;
     CodeEditorComponent resultsBox;
 
-    void buttonClicked (Button* button) override
+    void lookAndFeelChanged() override
     {
-        if (button == &fetchButton)
-            startThread();
-    }
-
-    void textEditorReturnKeyPressed (TextEditor&) override
-    {
-        fetchButton.triggerClick();
+        urlBox.applyFontToAllText (urlBox.getFont());
     }
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (NetworkingDemo)
