@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -51,10 +53,10 @@ struct OpenGLDemoClasses
     {
         Attributes (OpenGLContext& openGLContext, OpenGLShaderProgram& shader)
         {
-            position      = createAttribute (openGLContext, shader, "position");
-            normal        = createAttribute (openGLContext, shader, "normal");
-            sourceColour  = createAttribute (openGLContext, shader, "sourceColour");
-            texureCoordIn = createAttribute (openGLContext, shader, "texureCoordIn");
+            position       = createAttribute (openGLContext, shader, "position");
+            normal         = createAttribute (openGLContext, shader, "normal");
+            sourceColour   = createAttribute (openGLContext, shader, "sourceColour");
+            textureCoordIn = createAttribute (openGLContext, shader, "textureCoordIn");
         }
 
         void enable (OpenGLContext& openGLContext)
@@ -77,22 +79,22 @@ struct OpenGLDemoClasses
                 openGLContext.extensions.glEnableVertexAttribArray (sourceColour->attributeID);
             }
 
-            if (texureCoordIn != nullptr)
+            if (textureCoordIn != nullptr)
             {
-                openGLContext.extensions.glVertexAttribPointer (texureCoordIn->attributeID, 2, GL_FLOAT, GL_FALSE, sizeof (Vertex), (GLvoid*) (sizeof (float) * 10));
-                openGLContext.extensions.glEnableVertexAttribArray (texureCoordIn->attributeID);
+                openGLContext.extensions.glVertexAttribPointer (textureCoordIn->attributeID, 2, GL_FLOAT, GL_FALSE, sizeof (Vertex), (GLvoid*) (sizeof (float) * 10));
+                openGLContext.extensions.glEnableVertexAttribArray (textureCoordIn->attributeID);
             }
         }
 
         void disable (OpenGLContext& openGLContext)
         {
-            if (position != nullptr)       openGLContext.extensions.glDisableVertexAttribArray (position->attributeID);
-            if (normal != nullptr)         openGLContext.extensions.glDisableVertexAttribArray (normal->attributeID);
-            if (sourceColour != nullptr)   openGLContext.extensions.glDisableVertexAttribArray (sourceColour->attributeID);
-            if (texureCoordIn != nullptr)  openGLContext.extensions.glDisableVertexAttribArray (texureCoordIn->attributeID);
+            if (position != nullptr)        openGLContext.extensions.glDisableVertexAttribArray (position->attributeID);
+            if (normal != nullptr)          openGLContext.extensions.glDisableVertexAttribArray (normal->attributeID);
+            if (sourceColour != nullptr)    openGLContext.extensions.glDisableVertexAttribArray (sourceColour->attributeID);
+            if (textureCoordIn != nullptr)  openGLContext.extensions.glDisableVertexAttribArray (textureCoordIn->attributeID);
         }
 
-        ScopedPointer<OpenGLShaderProgram::Attribute> position, normal, sourceColour, texureCoordIn;
+        ScopedPointer<OpenGLShaderProgram::Attribute> position, normal, sourceColour, textureCoordIn;
 
     private:
         static OpenGLShaderProgram::Attribute* createAttribute (OpenGLContext& openGLContext,
@@ -142,20 +144,18 @@ struct OpenGLDemoClasses
         Shape (OpenGLContext& openGLContext)
         {
             if (shapeFile.load (BinaryData::teapot_obj).wasOk())
-                for (int i = 0; i < shapeFile.shapes.size(); ++i)
-                    vertexBuffers.add (new VertexBuffer (openGLContext, *shapeFile.shapes.getUnchecked(i)));
-
+                for (auto* s : shapeFile.shapes)
+                    vertexBuffers.add (new VertexBuffer (openGLContext, *s));
         }
 
         void draw (OpenGLContext& openGLContext, Attributes& attributes)
         {
-            for (int i = 0; i < vertexBuffers.size(); ++i)
+            for (auto* vertexBuffer : vertexBuffers)
             {
-                VertexBuffer& vertexBuffer = *vertexBuffers.getUnchecked (i);
-                vertexBuffer.bind();
+                vertexBuffer->bind();
 
                 attributes.enable (openGLContext);
-                glDrawElements (GL_TRIANGLES, vertexBuffer.numIndices, GL_UNSIGNED_INT, 0);
+                glDrawElements (GL_TRIANGLES, vertexBuffer->numIndices, GL_UNSIGNED_INT, 0);
                 attributes.disable (openGLContext);
             }
         }
@@ -329,22 +329,17 @@ struct OpenGLDemoClasses
     */
     class DemoControlsOverlay  : public Component,
                                  private CodeDocument::Listener,
-                                 private ComboBox::Listener,
                                  private Slider::Listener,
-                                 private Button::Listener,
                                  private Timer
     {
     public:
         DemoControlsOverlay (OpenGLDemo& d)
             : demo (d),
               vertexEditorComp (vertexDocument, nullptr),
-              fragmentEditorComp (fragmentDocument, nullptr),
-              tabbedComp (TabbedButtonBar::TabsAtLeft),
-              showBackgroundToggle ("Draw 2D graphics in background")
+              fragmentEditorComp (fragmentDocument, nullptr)
         {
             addAndMakeVisible (statusLabel);
             statusLabel.setJustificationType (Justification::topLeft);
-            statusLabel.setColour (Label::textColourId, Colours::black);
             statusLabel.setFont (Font (14.0f));
 
             addAndMakeVisible (sizeSlider);
@@ -365,18 +360,13 @@ struct OpenGLDemoClasses
             speedLabel.attachToComponent (&speedSlider, true);
 
             addAndMakeVisible (showBackgroundToggle);
-            showBackgroundToggle.addListener (this);
-
-            Colour editorBackground (Colours::white.withAlpha (0.6f));
+            showBackgroundToggle.onClick = [this] { demo.doBackgroundDrawing = showBackgroundToggle.getToggleState(); };
 
             addAndMakeVisible (tabbedComp);
             tabbedComp.setTabBarDepth (25);
             tabbedComp.setColour (TabbedButtonBar::tabTextColourId, Colours::grey);
-            tabbedComp.addTab ("Vertex", editorBackground, &vertexEditorComp, false);
-            tabbedComp.addTab ("Fragment", editorBackground, &fragmentEditorComp, false);
-
-            vertexEditorComp.setColour (CodeEditorComponent::backgroundColourId, editorBackground);
-            fragmentEditorComp.setColour (CodeEditorComponent::backgroundColourId, editorBackground);
+            tabbedComp.addTab ("Vertex", Colours::transparentBlack, &vertexEditorComp, false);
+            tabbedComp.addTab ("Fragment", Colours::transparentBlack, &fragmentEditorComp, false);
 
             vertexDocument.addListener (this);
             fragmentDocument.addListener (this);
@@ -387,13 +377,13 @@ struct OpenGLDemoClasses
             textures.add (new DynamicTexture());
 
             addAndMakeVisible (textureBox);
-            textureBox.addListener (this);
+            textureBox.onChange = [this] { selectTexture (textureBox.getSelectedId()); };
             updateTexturesList();
 
             addAndMakeVisible (presetBox);
-            presetBox.addListener (this);
+            presetBox.onChange = [this] { selectPreset (presetBox.getSelectedItemIndex()); };
 
-            Array<ShaderPreset> presets (getPresets());
+            auto presets = getPresets();
             StringArray presetNames;
 
             for (int i = 0; i < presets.size(); ++i)
@@ -406,6 +396,8 @@ struct OpenGLDemoClasses
             addAndMakeVisible (textureLabel);
             textureLabel.setText ("Texture:", dontSendNotification);
             textureLabel.attachToComponent (&textureBox, true);
+
+            lookAndFeelChanged();
         }
 
         void initialise()
@@ -419,11 +411,11 @@ struct OpenGLDemoClasses
 
         void resized() override
         {
-            Rectangle<int> area (getLocalBounds().reduced (4));
+            auto area = getLocalBounds().reduced (4);
 
-            Rectangle<int> top (area.removeFromTop (75));
+            auto top = area.removeFromTop (75);
 
-            Rectangle<int> sliders (top.removeFromRight (area.getWidth() / 2));
+            auto sliders = top.removeFromRight (area.getWidth() / 2);
             showBackgroundToggle.setBounds (sliders.removeFromBottom (25));
             speedSlider.setBounds (sliders.removeFromBottom (25));
             sizeSlider.setBounds (sliders.removeFromBottom (25));
@@ -431,9 +423,9 @@ struct OpenGLDemoClasses
             top.removeFromRight (70);
             statusLabel.setBounds (top);
 
-            Rectangle<int> shaderArea (area.removeFromBottom (area.getHeight() / 2));
+            auto shaderArea = area.removeFromBottom (area.getHeight() / 2);
 
-            Rectangle<int> presets (shaderArea.removeFromTop (25));
+            auto presets = shaderArea.removeFromTop (25);
             presets.removeFromLeft (100);
             presetBox.setBounds (presets.removeFromLeft (150));
             presets.removeFromLeft (100);
@@ -527,11 +519,6 @@ struct OpenGLDemoClasses
             demo.rotationSpeed = (float) speedSlider.getValue();
         }
 
-        void buttonClicked (Button*) override
-        {
-            demo.doBackgroundDrawing = showBackgroundToggle.getToggleState();
-        }
-
         enum { shaderLinkDelay = 500 };
 
         void codeDocumentTextInserted (const String& /*newText*/, int /*insertIndex*/) override
@@ -551,12 +538,16 @@ struct OpenGLDemoClasses
                                    fragmentDocument.getAllContent());
         }
 
-        void comboBoxChanged (ComboBox* box) override
+        void lookAndFeelChanged() override
         {
-            if (box == &presetBox)
-                selectPreset (presetBox.getSelectedItemIndex());
-            else if (box == &textureBox)
-                selectTexture (textureBox.getSelectedId());
+            auto editorBackground = getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground,
+                                                            Colours::white);
+
+            for (int i = tabbedComp.getNumTabs(); i >= 0; --i)
+                tabbedComp.setTabBackgroundColour (i, editorBackground);
+
+            vertexEditorComp.setColour (CodeEditorComponent::backgroundColourId, editorBackground);
+            fragmentEditorComp.setColour (CodeEditorComponent::backgroundColourId, editorBackground);
         }
 
         OpenGLDemo& demo;
@@ -565,13 +556,13 @@ struct OpenGLDemoClasses
 
         CodeDocument vertexDocument, fragmentDocument;
         CodeEditorComponent vertexEditorComp, fragmentEditorComp;
-        TabbedComponent tabbedComp;
+        TabbedComponent tabbedComp { TabbedButtonBar::TabsAtLeft };
 
         ComboBox presetBox, textureBox;
         Label presetLabel, textureLabel;
 
         Slider speedSlider, sizeSlider;
-        ToggleButton showBackgroundToggle;
+        ToggleButton showBackgroundToggle { "Draw 2D graphics in background" };
 
         OwnedArray<DemoTexture> textures;
 
@@ -583,15 +574,13 @@ struct OpenGLDemoClasses
         it implements the OpenGLRenderer callback so that it can do real GL work.
     */
     class OpenGLDemo  : public Component,
-                        private OpenGLRenderer
+                        private OpenGLRenderer,
+                        private AsyncUpdater
     {
     public:
         OpenGLDemo()
-            : doBackgroundDrawing (false),
-              scale (0.5f), rotationSpeed (0.0f), rotation (0.0f),
-              textureToUse (nullptr), lastTexture (nullptr)
         {
-            if (MainAppWindow* mw = MainAppWindow::getMainAppWindow())
+            if (auto* mw = MainAppWindow::getMainAppWindow())
                 mw->setRenderingEngine (0);
 
             setOpaque (true);
@@ -631,10 +620,10 @@ struct OpenGLDemoClasses
 
         void freeAllContextObjects()
         {
-            shape = nullptr;
-            shader = nullptr;
-            attributes = nullptr;
-            uniforms = nullptr;
+            shape.reset();
+            shader.reset();
+            attributes.reset();
+            uniforms.reset();
             texture.release();
         }
 
@@ -644,8 +633,10 @@ struct OpenGLDemoClasses
         {
             jassert (OpenGLHelpers::isContextActive());
 
-            const float desktopScale = (float) openGLContext.getRenderingScale();
-            OpenGLHelpers::clear (Colours::lightblue);
+            auto desktopScale = (float) openGLContext.getRenderingScale();
+
+            OpenGLHelpers::clear (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground,
+                                                          Colours::lightblue));
 
             if (textureToUse != nullptr)
                 if (! textureToUse->applyTo (texture))
@@ -705,17 +696,18 @@ struct OpenGLDemoClasses
 
         Matrix3D<float> getProjectionMatrix() const
         {
-            float w = 1.0f / (scale + 0.1f);
-            float h = w * getLocalBounds().toFloat().getAspectRatio (false);
+            auto w = 1.0f / (scale + 0.1f);
+            auto h = w * getLocalBounds().toFloat().getAspectRatio (false);
+
             return Matrix3D<float>::fromFrustum (-w, w, -h, h, 4.0f, 30.0f);
         }
 
         Matrix3D<float> getViewMatrix() const
         {
-            Matrix3D<float> viewMatrix = draggableOrientation.getRotationMatrix()
-                                            * Vector3D<float> (0.0f, 1.0f, -10.0f);
+            auto viewMatrix = draggableOrientation.getRotationMatrix()
+                                 * Vector3D<float> (0.0f, 1.0f, -10.0f);
 
-            Matrix3D<float> rotationMatrix = viewMatrix.rotated (Vector3D<float> (rotation, rotation, -0.3f));
+            auto rotationMatrix = Matrix3D<float>::rotation ({ rotation, rotation, -0.3f });
 
             return rotationMatrix * viewMatrix;
         }
@@ -740,11 +732,16 @@ struct OpenGLDemoClasses
         }
 
         Draggable3DOrientation draggableOrientation;
-        bool doBackgroundDrawing;
-        float scale, rotationSpeed;
+        bool doBackgroundDrawing = false;
+        float scale = 0.5f, rotationSpeed = 0;
         BouncingNumber bouncingNumber;
 
     private:
+        void handleAsyncUpdate() override
+        {
+            controlsOverlay->statusLabel.setText (statusText, dontSendNotification);
+        }
+
         void drawBackground2DStuff (float desktopScale)
         {
             // Create an OpenGLGraphicsContext that will draw into this GL window..
@@ -784,7 +781,7 @@ struct OpenGLDemoClasses
 
         ScopedPointer<DemoControlsOverlay> controlsOverlay;
 
-        float rotation;
+        float rotation = 0;
 
         ScopedPointer<OpenGLShaderProgram> shader;
         ScopedPointer<Shape> shape;
@@ -792,9 +789,10 @@ struct OpenGLDemoClasses
         ScopedPointer<Uniforms> uniforms;
 
         OpenGLTexture texture;
-        DemoTexture* textureToUse, *lastTexture;
+        DemoTexture* textureToUse = nullptr;
+        DemoTexture* lastTexture = nullptr;
 
-        String newVertexShader, newFragmentShader;
+        String newVertexShader, newFragmentShader, statusText;
 
         struct BackgroundStar
         {
@@ -809,15 +807,14 @@ struct OpenGLDemoClasses
             if (newVertexShader.isNotEmpty() || newFragmentShader.isNotEmpty())
             {
                 ScopedPointer<OpenGLShaderProgram> newShader (new OpenGLShaderProgram (openGLContext));
-                String statusText;
 
                 if (newShader->addVertexShader (OpenGLHelpers::translateVertexShaderToV3 (newVertexShader))
                       && newShader->addFragmentShader (OpenGLHelpers::translateFragmentShaderToV3 (newFragmentShader))
                       && newShader->link())
                 {
-                    shape = nullptr;
-                    attributes = nullptr;
-                    uniforms = nullptr;
+                    shape.reset();
+                    attributes.reset();
+                    uniforms.reset();
 
                     shader = newShader;
                     shader->use();
@@ -833,11 +830,10 @@ struct OpenGLDemoClasses
                     statusText = newShader->getLastError();
                 }
 
-                controlsOverlay->statusLabel.setText (statusText, dontSendNotification);
+                triggerAsyncUpdate();
 
-
-                newVertexShader = String();
-                newFragmentShader = String();
+                newVertexShader = {};
+                newFragmentShader = {};
             }
         }
 
@@ -869,7 +865,7 @@ struct OpenGLDemoClasses
                 "attribute vec4 position;\n"
                 "attribute vec4 normal;\n"
                 "attribute vec4 sourceColour;\n"
-                "attribute vec2 texureCoordIn;\n"
+                "attribute vec2 textureCoordIn;\n"
                 "\n"
                 "uniform mat4 projectionMatrix;\n"
                 "uniform mat4 viewMatrix;\n"
@@ -882,7 +878,7 @@ struct OpenGLDemoClasses
                 "void main()\n"
                 "{\n"
                 "    destinationColour = sourceColour;\n"
-                "    textureCoordOut = texureCoordIn;\n"
+                "    textureCoordOut = textureCoordIn;\n"
                 "\n"
                 "    vec4 light = viewMatrix * lightPosition;\n"
                 "    lightIntensity = dot (light, normal);\n"
@@ -922,7 +918,7 @@ struct OpenGLDemoClasses
                 SHADER_DEMO_HEADER
                 "attribute vec4 position;\n"
                 "attribute vec4 sourceColour;\n"
-                "attribute vec2 texureCoordIn;\n"
+                "attribute vec2 textureCoordIn;\n"
                 "\n"
                 "uniform mat4 projectionMatrix;\n"
                 "uniform mat4 viewMatrix;\n"
@@ -933,7 +929,7 @@ struct OpenGLDemoClasses
                 "void main()\n"
                 "{\n"
                 "    destinationColour = sourceColour;\n"
-                "    textureCoordOut = texureCoordIn;\n"
+                "    textureCoordOut = textureCoordIn;\n"
                 "    gl_Position = projectionMatrix * viewMatrix * position;\n"
                 "}\n",
 
@@ -960,7 +956,7 @@ struct OpenGLDemoClasses
                 SHADER_DEMO_HEADER
                 "attribute vec4 position;\n"
                 "attribute vec4 sourceColour;\n"
-                "attribute vec2 texureCoordIn;\n"
+                "attribute vec2 textureCoordIn;\n"
                 "\n"
                 "uniform mat4 projectionMatrix;\n"
                 "uniform mat4 viewMatrix;\n"
@@ -971,7 +967,7 @@ struct OpenGLDemoClasses
                 "void main()\n"
                 "{\n"
                 "    destinationColour = sourceColour;\n"
-                "    textureCoordOut = texureCoordIn;\n"
+                "    textureCoordOut = textureCoordIn;\n"
                 "    gl_Position = projectionMatrix * viewMatrix * position;\n"
                 "}\n",
 
@@ -996,7 +992,7 @@ struct OpenGLDemoClasses
                 SHADER_DEMO_HEADER
                 "attribute vec4 position;\n"
                 "attribute vec4 sourceColour;\n"
-                "attribute vec2 texureCoordIn;\n"
+                "attribute vec2 textureCoordIn;\n"
                 "\n"
                 "uniform mat4 projectionMatrix;\n"
                 "uniform mat4 viewMatrix;\n"
@@ -1043,7 +1039,7 @@ struct OpenGLDemoClasses
 
                 SHADER_DEMO_HEADER
                 "attribute vec4 position;\n"
-                "attribute vec2 texureCoordIn;\n"
+                "attribute vec2 textureCoordIn;\n"
                 "\n"
                 "uniform mat4 projectionMatrix;\n"
                 "uniform mat4 viewMatrix;\n"
@@ -1052,7 +1048,7 @@ struct OpenGLDemoClasses
                 "\n"
                 "void main()\n"
                 "{\n"
-                "    textureCoordOut = texureCoordIn;\n"
+                "    textureCoordOut = textureCoordIn;\n"
                 "    gl_Position = projectionMatrix * viewMatrix * position;\n"
                 "}\n",
 
