@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -29,7 +31,6 @@
 //==============================================================================
 class OpenGL2DShaderDemo  : public Component,
                             private CodeDocument::Listener,
-                            private ComboBox::Listener,
                             private Timer
 {
 public:
@@ -38,15 +39,14 @@ public:
     {
         setOpaque (true);
 
-        if (MainAppWindow* mw = MainAppWindow::getMainAppWindow())
+        if (auto* mw = MainAppWindow::getMainAppWindow())
             mw->setOpenGLRenderingEngine();
 
         addAndMakeVisible (statusLabel);
         statusLabel.setJustificationType (Justification::topLeft);
-        statusLabel.setColour (Label::textColourId, Colours::black);
         statusLabel.setFont (Font (14.0f));
 
-        Array<ShaderPreset> presets (getPresets());
+        auto presets = getPresets();
         StringArray presetNames;
 
         for (int i = 0; i < presets.size(); ++i)
@@ -57,10 +57,8 @@ public:
         presetLabel.attachToComponent (&presetBox, true);
 
         addAndMakeVisible (presetBox);
-        presetBox.addListener (this);
+        presetBox.onChange = [this] { selectPreset (presetBox.getSelectedItemIndex()); };
 
-        Colour editorBackground (Colours::white.withAlpha (0.6f));
-        fragmentEditorComp.setColour (CodeEditorComponent::backgroundColourId, editorBackground);
         fragmentEditorComp.setOpaque (false);
         fragmentDocument.addListener (this);
         addAndMakeVisible (fragmentEditorComp);
@@ -70,16 +68,16 @@ public:
 
     ~OpenGL2DShaderDemo()
     {
-        shader = nullptr;
+        shader.reset();
     }
 
     void paint (Graphics& g) override
     {
-        g.fillCheckerBoard (getLocalBounds(), 48, 48, Colours::lightgrey, Colours::white);
+        g.fillCheckerBoard (getLocalBounds().toFloat(), 48.0f, 48.0f, Colours::lightgrey, Colours::white);
 
         if (shader == nullptr || shader->getFragmentShaderCode() != fragmentCode)
         {
-            shader = nullptr;
+            shader.reset();
 
             if (fragmentCode.isNotEmpty())
             {
@@ -90,7 +88,7 @@ public:
                 if (result.failed())
                 {
                     statusLabel.setText (result.getErrorMessage(), dontSendNotification);
-                    shader = nullptr;
+                    shader.reset();
                 }
             }
         }
@@ -153,11 +151,6 @@ private:
         repaint();
     }
 
-    void comboBoxChanged (ComboBox*) override
-    {
-        selectPreset (presetBox.getSelectedItemIndex());
-    }
-
     struct ShaderPreset
     {
         const char* name;
