@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -114,6 +116,7 @@ static String getAllSystemInfo()
       << "Operating system: " << SystemStats::getOperatingSystemName() << newLine
       << "Host name:        " << SystemStats::getComputerName() << newLine
       << "Device type:      " << SystemStats::getDeviceDescription() << newLine
+      << "Manufacturer:     " << SystemStats::getDeviceManufacturer() << newLine
       << "User logon name:  " << SystemStats::getLogonName() << newLine
       << "Full user name:   " << SystemStats::getFullUserName() << newLine
       << "User region:      " << SystemStats::getUserRegion() << newLine
@@ -122,9 +125,11 @@ static String getAllSystemInfo()
       << newLine;
 
     systemInfo
-      << "Number of CPUs:  " << SystemStats::getNumCpus() << newLine
+      << "Number of logical CPUs:  " << SystemStats::getNumCpus() << newLine
+      << "Number of physical CPUs: " << SystemStats::getNumPhysicalCpus() << newLine
       << "Memory size:     " << SystemStats::getMemorySizeInMegabytes() << " MB" << newLine
       << "CPU vendor:      " << SystemStats::getCpuVendor() << newLine
+      << "CPU model:       " << SystemStats::getCpuModel() << newLine
       << "CPU speed:       " << SystemStats::getCpuSpeedInMegaherz() << " MHz" << newLine
       << "CPU has MMX:     " << (SystemStats::hasMMX()    ? "yes" : "no") << newLine
       << "CPU has SSE:     " << (SystemStats::hasSSE()    ? "yes" : "no") << newLine
@@ -136,6 +141,7 @@ static String getAllSystemInfo()
       << "CPU has 3DNOW:   " << (SystemStats::has3DNow()  ? "yes" : "no") << newLine
       << "CPU has AVX:     " << (SystemStats::hasAVX()    ? "yes" : "no") << newLine
       << "CPU has AVX2:    " << (SystemStats::hasAVX2()   ? "yes" : "no") << newLine
+      << "CPU has Neon:    " << (SystemStats::hasNeon()   ? "yes" : "no") << newLine
       << newLine;
 
     systemInfo
@@ -186,7 +192,8 @@ public:
 
     void paint (Graphics& g) override
     {
-        g.fillAll (Colour::greyLevel (0.93f));
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground,
+                                           Colour::greyLevel (0.93f)));
     }
 
     void resized() override
@@ -197,6 +204,11 @@ public:
 private:
     TextEditor resultsBox;
 
+    void lookAndFeelChanged() override
+    {
+        resultsBox.applyFontToAllText (resultsBox.getFont());
+    }
+
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (SystemInfoDemo)
 };
 
