@@ -2,29 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
 #include "../JuceDemoHeader.h"
 
-#if JUCE_QUICKTIME || JUCE_DIRECTSHOW
+#if JUCE_MAC || (JUCE_WINDOWS && ! JUCE_MINGW)
 
 //==============================================================================
 // so that we can easily have two video windows each with a file browser, wrap this up as a class..
@@ -50,7 +52,7 @@ public:
         fileChooser.setCurrentFile (file, true);
     }
 
-    void paintOverChildren (Graphics& g)
+    void paintOverChildren (Graphics& g) override
     {
         if (isDragOver)
         {
@@ -59,28 +61,26 @@ public:
         }
     }
 
-    void resized()
+    void resized() override
     {
-        videoComp.setBoundsWithCorrectAspectRatio (Rectangle<int> (0, 0, getWidth(), getHeight() - 30),
-                                                   Justification::centred);
-        fileChooser.setBounds (0, getHeight() - 24, getWidth(), 24);
+        videoComp.setBounds (getLocalBounds().reduced (10));
     }
 
-    bool isInterestedInDragSource (const SourceDetails&)  { return true; }
+    bool isInterestedInDragSource (const SourceDetails&) override   { return true; }
 
-    void itemDragEnter (const SourceDetails&)
+    void itemDragEnter (const SourceDetails&) override
     {
         isDragOver = true;
         repaint();
     }
 
-    void itemDragExit (const SourceDetails&)
+    void itemDragExit (const SourceDetails&) override
     {
         isDragOver = false;
         repaint();
     }
 
-    void itemDropped (const SourceDetails& dragSourceDetails)
+    void itemDropped (const SourceDetails& dragSourceDetails) override
     {
         setFile (dragSourceDetails.description.toString());
         isDragOver = false;
@@ -88,11 +88,7 @@ public:
     }
 
 private:
-   #if JUCE_QUICKTIME
-    QuickTimeMovieComponent videoComp;
-   #elif JUCE_DIRECTSHOW
-    DirectShowComponent videoComp;
-   #endif
+    VideoComponent videoComp;
 
     bool isDragOver;
     FilenameComponent fileChooser;
@@ -100,11 +96,9 @@ private:
     void filenameComponentChanged (FilenameComponent*) override
     {
         // this is called when the user changes the filename in the file chooser box
-       #if JUCE_QUICKTIME
-        if (videoComp.loadMovie (fileChooser.getCurrentFile(), true))
-       #elif JUCE_DIRECTSHOW
-        if (videoComp.loadMovie (fileChooser.getCurrentFile()))
-       #endif
+        auto result = videoComp.load (fileChooser.getCurrentFile());
+
+        if (result.wasOk())
         {
             // loaded the file ok, so let's start it playing..
 
@@ -115,7 +109,7 @@ private:
         {
             AlertWindow::showMessageBoxAsync (AlertWindow::WarningIcon,
                                               "Couldn't load the file!",
-                                              r.getErrorMessage());
+                                              result.getErrorMessage());
         }
     }
 
@@ -125,7 +119,6 @@ private:
 //==============================================================================
 class VideoDemo   : public Component,
                     public DragAndDropContainer,
-                    private Button::Listener,
                     private FileBrowserListener
 {
 public:
@@ -142,7 +135,7 @@ public:
         directoryThread.startThread (1);
 
         fileTree.addListener (this);
-        fileTree.setColour (TreeView::backgroundColourId, Colours::lightgrey);
+        fileTree.setColour (FileTreeComponent::backgroundColourId, Colours::lightgrey.withAlpha (0.6f));
         addAndMakeVisible (fileTree);
 
         addAndMakeVisible (resizerBar);
@@ -150,8 +143,8 @@ public:
         loadLeftButton.setButtonText ("Load Left");
         loadRightButton.setButtonText ("Load Right");
 
-        loadLeftButton.addListener (this);
-        loadRightButton.addListener (this);
+        loadLeftButton .onClick = [this] { movieCompLeft .setFile (fileTree.getSelectedFile (0)); };
+        loadRightButton.onClick = [this] { movieCompRight.setFile (fileTree.getSelectedFile (0)); };
 
         addAndMakeVisible (loadLeftButton);
         addAndMakeVisible (loadRightButton);
@@ -174,14 +167,12 @@ public:
 
     ~VideoDemo()
     {
-        loadLeftButton.removeListener (this);
-        loadRightButton.removeListener (this);
         fileTree.removeListener (this);
     }
 
     void paint (Graphics& g) override
     {
-        fillStandardDemoBackground (g);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground));
     }
 
     void resized() override
@@ -220,14 +211,6 @@ private:
     TextButton loadLeftButton, loadRightButton;
     MovieComponentWithFileBrowser movieCompLeft, movieCompRight;
 
-    void buttonClicked (Button* button) override
-    {
-        if (button == &loadLeftButton)
-            movieCompLeft.setFile (fileTree.getSelectedFile (0));
-        else if (button == &loadRightButton)
-            movieCompRight.setFile (fileTree.getSelectedFile (0));
-    }
-
     void selectionChanged() override
     {
         // we're just going to update the drag description of out tree so that rows can be dragged onto the file players
