@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -66,33 +68,28 @@ private:
 
 
 //==============================================================================
-class WebBrowserDemo    : public Component,
-                          private TextEditor::Listener,
-                          private ButtonListener
+class WebBrowserDemo    : public Component
 {
 public:
     WebBrowserDemo()
-        : goButton ("Go", "Go to URL"),
-          backButton ("<<", "Back"),
-          forwardButton (">>", "Forward")
     {
         setOpaque (true);
 
         // Create an address box..
         addAndMakeVisible (addressTextBox);
         addressTextBox.setTextToShowWhenEmpty ("Enter a web address, e.g. https://www.juce.com", Colours::grey);
-        addressTextBox.addListener (this);
+        addressTextBox.onReturnKey = [this] { webView->goToURL (addressTextBox.getText()); };
 
         // create the actual browser component
         addAndMakeVisible (webView = new DemoBrowserComponent (addressTextBox));
 
         // add some buttons..
         addAndMakeVisible (goButton);
-        goButton.addListener (this);
+        goButton.onClick = [this] { webView->goToURL (addressTextBox.getText()); };
         addAndMakeVisible (backButton);
-        backButton.addListener (this);
+        backButton.onClick = [this] { webView->goBack(); };
         addAndMakeVisible (forwardButton);
-        forwardButton.addListener (this);
+        forwardButton.onClick = [this] { webView->goForward(); };
 
         // send the browser to a start page..
         webView->goToURL ("https://www.juce.com");
@@ -100,7 +97,8 @@ public:
 
     void paint (Graphics& g) override
     {
-        g.fillAll (Colours::grey);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground,
+                                           Colours::grey));
     }
 
     void resized() override
@@ -116,25 +114,14 @@ private:
     ScopedPointer<DemoBrowserComponent> webView;
 
     TextEditor addressTextBox;
-    TextButton goButton, backButton, forwardButton;
 
-    void textEditorTextChanged (TextEditor&) override             {}
-    void textEditorEscapeKeyPressed (TextEditor&) override        {}
-    void textEditorFocusLost (TextEditor&) override               {}
+    TextButton goButton      { "Go", "Go to URL" },
+               backButton    { "<<", "Back" },
+               forwardButton { ">>", "Forward" };
 
-    void textEditorReturnKeyPressed (TextEditor&) override
+    void lookAndFeelChanged() override
     {
-        webView->goToURL (addressTextBox.getText());
-    }
-
-    void buttonClicked (Button* b) override
-    {
-        if (b == &backButton)
-            webView->goBack();
-        else if (b == &forwardButton)
-            webView->goForward();
-        else if (b == &goButton)
-            webView->goToURL (addressTextBox.getText());
+        addressTextBox.applyFontToAllText (addressTextBox.getFont());
     }
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (WebBrowserDemo)
