@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -25,7 +27,7 @@
 #include "../JuceDemoHeader.h"
 
 
-static void showBubbleMessage (Component* targetComponent, const String& textToShow,
+static void showBubbleMessage (Component& targetComponent, const String& textToShow,
                                ScopedPointer<BubbleMessageComponent>& bmc)
 {
     bmc = new BubbleMessageComponent();
@@ -37,13 +39,14 @@ static void showBubbleMessage (Component* targetComponent, const String& textToS
     }
     else
     {
-        targetComponent->getTopLevelComponent()->addChildComponent (bmc);
+        targetComponent.getTopLevelComponent()->addChildComponent (bmc);
     }
 
     AttributedString text (textToShow);
     text.setJustification (Justification::centred);
+    text.setColour (targetComponent.findColour (TextButton::textColourOffId));
 
-    bmc->showAt (targetComponent, text, 2000, true, false);
+    bmc->showAt (&targetComponent, text, 2000, true, false);
 }
 
 //==============================================================================
@@ -102,10 +105,13 @@ struct SlidersPage  : public Component
         : hintLabel ("hint", "Try right-clicking on a slider for an options menu. \n\n"
                              "Also, holding down CTRL while dragging will turn on a slider's velocity-sensitive mode")
     {
+        Rectangle<int> layoutArea { 20, 20, 580, 430 };
+        auto sliderArea = layoutArea.removeFromTop (320);
+
         Slider* s = createSlider (false);
         s->setSliderStyle (Slider::LinearVertical);
         s->setTextBoxStyle (Slider::TextBoxBelow, false, 100, 20);
-        s->setBounds (10, 25, 70, 200);
+        s->setBounds (sliderArea.removeFromLeft (70));
         s->setDoubleClickReturnValue (true, 50.0); // double-clicking this slider will set it to 50.0
         s->setTextValueSuffix (" units");
 
@@ -114,68 +120,85 @@ struct SlidersPage  : public Component
         s->setVelocityBasedMode (true);
         s->setSkewFactor (0.5);
         s->setTextBoxStyle (Slider::TextBoxAbove, true, 100, 20);
-        s->setBounds (85, 25, 70, 200);
+        s->setBounds (sliderArea.removeFromLeft (70));
         s->setTextValueSuffix (" rels");
 
+        sliderArea.removeFromLeft (20);
+        auto horizonalSliderArea = sliderArea.removeFromLeft (180);
+
         s = createSlider (true);
         s->setSliderStyle (Slider::LinearHorizontal);
         s->setTextBoxStyle (Slider::TextBoxLeft, false, 80, 20);
-        s->setBounds (180, 35, 150, 20);
+        s->setBounds (horizonalSliderArea.removeFromTop (20));
 
         s = createSlider (false);
         s->setSliderStyle (Slider::LinearHorizontal);
         s->setTextBoxStyle (Slider::NoTextBox, false, 0, 0);
-        s->setBounds (180, 65, 150, 20);
-        s->setPopupDisplayEnabled (true, this);
+        horizonalSliderArea.removeFromTop (20);
+        s->setBounds (horizonalSliderArea.removeFromTop (20));
+        s->setPopupDisplayEnabled (true, false, this);
         s->setTextValueSuffix (" nuns required to change a lightbulb");
 
+        s = createSlider (false);
+        s->setSliderStyle (Slider::LinearHorizontal);
+        s->setTextBoxStyle (Slider::TextEntryBoxPosition::TextBoxAbove, false, 70, 20);
+        horizonalSliderArea.removeFromTop (20);
+        s->setBounds (horizonalSliderArea.removeFromTop (50));
+        s->setPopupDisplayEnabled (true, false, this);
+
         s = createSlider (false);
         s->setSliderStyle (Slider::IncDecButtons);
         s->setTextBoxStyle (Slider::TextBoxLeft, false, 50, 20);
-        s->setBounds (180, 105, 100, 20);
+        horizonalSliderArea.removeFromTop (20);
+        s->setBounds (horizonalSliderArea.removeFromTop (20));
         s->setIncDecButtonsMode (Slider::incDecButtonsDraggable_Vertical);
 
         s = createSlider (false);
         s->setSliderStyle (Slider::Rotary);
-        s->setRotaryParameters (float_Pi * 1.2f, float_Pi * 2.8f, false);
+        s->setRotaryParameters (MathConstants<float>::pi * 1.2f, MathConstants<float>::pi * 2.8f, false);
         s->setTextBoxStyle (Slider::TextBoxRight, false, 70, 20);
-        s->setBounds (190, 145, 120, 40);
+        horizonalSliderArea.removeFromTop (15);
+        s->setBounds (horizonalSliderArea.removeFromTop (70));
         s->setTextValueSuffix (" mm");
 
         s = createSlider (false);
         s->setSliderStyle (Slider::LinearBar);
-        s->setBounds (180, 195, 100, 30);
+        horizonalSliderArea.removeFromTop (10);
+        s->setBounds (horizonalSliderArea.removeFromTop (30));
         s->setTextValueSuffix (" gallons");
 
+        sliderArea.removeFromLeft (20);
+        auto twoValueSliderArea = sliderArea.removeFromLeft (180);
+
         s = createSlider (false);
         s->setSliderStyle (Slider::TwoValueHorizontal);
-        s->setBounds (360, 20, 160, 40);
-
-        s = createSlider (false);
-        s->setSliderStyle (Slider::TwoValueVertical);
-        s->setBounds (360, 110, 40, 160);
+        s->setBounds (twoValueSliderArea.removeFromTop (40));
 
         s = createSlider (false);
         s->setSliderStyle (Slider::ThreeValueHorizontal);
-        s->setBounds (360, 70, 160, 40);
+        s->setPopupDisplayEnabled (true, false, this);
+        twoValueSliderArea.removeFromTop (10);
+        s->setBounds (twoValueSliderArea.removeFromTop (40));
+
+        s = createSlider (false);
+        s->setSliderStyle (Slider::TwoValueVertical);
+        twoValueSliderArea.removeFromLeft (30);
+        s->setBounds (twoValueSliderArea.removeFromLeft (40));
 
         s = createSlider (false);
         s->setSliderStyle (Slider::ThreeValueVertical);
-        s->setBounds (440, 110, 40, 160);
+        s->setPopupDisplayEnabled (true, false, this);
+        twoValueSliderArea.removeFromLeft (30);
+        s->setBounds (twoValueSliderArea.removeFromLeft (40));
 
         s = createSlider (false);
         s->setSliderStyle (Slider::LinearBarVertical);
         s->setTextBoxStyle (Slider::NoTextBox, false, 0, 0);
-        s->setBounds (540, 35, 20, 230);
-        s->setPopupDisplayEnabled (true, this);
+        sliderArea.removeFromLeft (20);
+        s->setBounds (sliderArea.removeFromLeft (20));
+        s->setPopupDisplayEnabled (true, true, this);
         s->setTextValueSuffix (" mickles in a muckle");
 
-        for (int i = 7; i <= 10; ++i)
-        {
-            sliders.getUnchecked(i)->setTextBoxStyle (Slider::NoTextBox, false, 0, 0);
-            sliders.getUnchecked(i)->setPopupDisplayEnabled (true, this);
-        }
-
         /* Here, we'll create a Value object, and tell a bunch of our sliders to use it as their
            value source. By telling them all to share the same Value, they'll stay in sync with
            each other.
@@ -185,7 +208,7 @@ struct SlidersPage  : public Component
         */
         Value sharedValue;
         sharedValue = Random::getSystemRandom().nextDouble() * 100;
-        for (int i = 0; i < 7; ++i)
+        for (int i = 0; i < 8; ++i)
             sliders.getUnchecked(i)->getValueObject().referTo (sharedValue);
 
         // ..and now we'll do the same for all our min/max slider values..
@@ -193,13 +216,15 @@ struct SlidersPage  : public Component
         sharedValueMin = Random::getSystemRandom().nextDouble() * 40.0;
         sharedValueMax = Random::getSystemRandom().nextDouble() * 40.0 + 60.0;
 
-        for (int i = 7; i <= 10; ++i)
+        for (int i = 8; i <= 11; ++i)
         {
-            sliders.getUnchecked(i)->getMaxValueObject().referTo (sharedValueMax);
-            sliders.getUnchecked(i)->getMinValueObject().referTo (sharedValueMin);
+            auto* selectedSlider = sliders.getUnchecked(i);
+            selectedSlider->setTextBoxStyle (Slider::NoTextBox, false, 0, 0);
+            selectedSlider->getMaxValueObject().referTo (sharedValueMax);
+            selectedSlider->getMinValueObject().referTo (sharedValueMin);
         }
 
-        hintLabel.setBounds (20, 245, 350, 150);
+        hintLabel.setBounds (layoutArea);
         addAndMakeVisible (hintLabel);
     }
 
@@ -222,8 +247,7 @@ private:
 };
 
 //==============================================================================
-struct ButtonsPage   : public Component,
-                       public ButtonListener
+struct ButtonsPage   : public Component
 {
     ButtonsPage()
     {
@@ -278,6 +302,8 @@ struct ButtonsPage   : public Component,
 
             tb->setClickingTogglesState (true);
             tb->setRadioGroupId (34567);
+            tb->setColour (TextButton::textColourOffId, Colours::black);
+            tb->setColour (TextButton::textColourOnId, Colours::black);
             tb->setColour (TextButton::buttonColourId, Colours::white);
             tb->setColour (TextButton::buttonOnColourId, Colours::blueviolet.brighter());
 
@@ -325,58 +351,68 @@ struct ButtonsPage   : public Component,
         down.setImage (ImageCache::getFromMemory (BinaryData::juce_icon_png, BinaryData::juce_icon_pngSize));
         down.setOverlayColour (Colours::black.withAlpha (0.3f));
 
+        auto popupMessageCallback = [this]
+        {
+            if (auto* focused = Component::getCurrentlyFocusedComponent())
+                showBubbleMessage (*focused,
+                                   "This is a demo of the BubbleMessageComponent, which lets you pop up a message pointing "
+                                   "at a component or somewhere on the screen.\n\n"
+                                   "The message bubbles will disappear after a timeout period, or when the mouse is clicked.",
+                                   this->bubbleMessage);
+        };
+
         {
             // create an image-above-text button from these drawables..
-            DrawableButton* db = addToList (new DrawableButton ("Button 1", DrawableButton::ImageAboveTextLabel));
+            auto db = addToList (new DrawableButton ("Button 1", DrawableButton::ImageAboveTextLabel));
             db->setImages (&normal, &over, &down);
             db->setBounds (260, 60, 80, 80);
             db->setTooltip ("This is a DrawableButton with a label");
-            db->addListener (this);
+            db->onClick = popupMessageCallback;
         }
 
         {
             // create an image-only button from these drawables..
-            DrawableButton* db = addToList (new DrawableButton ("Button 2", DrawableButton::ImageFitted));
+            auto db = addToList (new DrawableButton ("Button 2", DrawableButton::ImageFitted));
             db->setImages (&normal, &over, &down);
             db->setClickingTogglesState (true);
             db->setBounds (370, 60, 80, 80);
             db->setTooltip ("This is an image-only DrawableButton");
-            db->addListener (this);
+            db->onClick = popupMessageCallback;
         }
 
         {
             // create an image-on-button-shape button from the same drawables..
-            DrawableButton* db = addToList (new DrawableButton ("Button 3", DrawableButton::ImageOnButtonBackground));
+            auto db = addToList (new DrawableButton ("Button 3", DrawableButton::ImageOnButtonBackground));
             db->setImages (&normal, 0, 0);
             db->setBounds (260, 160, 110, 25);
             db->setTooltip ("This is a DrawableButton on a standard button background");
-            db->addListener (this);
+            db->onClick = popupMessageCallback;
         }
 
         {
-            DrawableButton* db = addToList (new DrawableButton ("Button 4", DrawableButton::ImageOnButtonBackground));
+            auto db = addToList (new DrawableButton ("Button 4", DrawableButton::ImageOnButtonBackground));
             db->setImages (&normal, &over, &down);
             db->setClickingTogglesState (true);
             db->setColour (DrawableButton::backgroundColourId, Colours::white);
             db->setColour (DrawableButton::backgroundOnColourId, Colours::yellow);
             db->setBounds (400, 150, 50, 50);
             db->setTooltip ("This is a DrawableButton on a standard button background");
-            db->addListener (this);
+            db->onClick = popupMessageCallback;
         }
 
         {
-            ShapeButton* sb = addToList (new ShapeButton ("ShapeButton",
-                                                          getRandomDarkColour(),
-                                                          getRandomDarkColour(),
-                                                          getRandomDarkColour()));
+            auto sb = addToList (new ShapeButton ("ShapeButton",
+                                                  getRandomDarkColour(),
+                                                  getRandomDarkColour(),
+                                                  getRandomDarkColour()));
             sb->setShape (MainAppWindow::getJUCELogoPath(), false, true, false);
             sb->setBounds (260, 220, 200, 120);
         }
 
         {
-            ImageButton* ib = addToList (new ImageButton ("ImageButton"));
+            auto ib = addToList (new ImageButton ("ImageButton"));
 
-            Image juceImage = ImageCache::getFromMemory (BinaryData::juce_icon_png, BinaryData::juce_icon_pngSize);
+            auto juceImage = ImageCache::getFromMemory (BinaryData::juce_icon_png, BinaryData::juce_icon_pngSize);
 
             ib->setImages (true, true, true,
                            juceImage, 0.7f, Colours::transparentBlack,
@@ -403,15 +439,6 @@ private:
         return newComp;
     }
 
-    void buttonClicked (Button* button) override
-    {
-        showBubbleMessage (button,
-                           "This is a demo of the BubbleMessageComponent, which lets you pop up a message pointing "
-                           "at a component or somewhere on the screen.\n\n"
-                           "The message bubbles will disappear after a timeout period, or when the mouse is clicked.",
-                           bubbleMessage);
-    }
-
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ButtonsPage)
 };
 
@@ -442,22 +469,27 @@ struct MiscPage   : public Component
         comboBox.setSelectedId (1);
     }
 
+    void lookAndFeelChanged() override
+    {
+        textEditor1.applyFontToAllText (textEditor1.getFont());
+        textEditor2.applyFontToAllText (textEditor2.getFont());
+    }
+
     TextEditor textEditor1, textEditor2;
     ComboBox comboBox;
 };
 
 //==============================================================================
 class ToolbarDemoComp   : public Component,
-                          public SliderListener,
-                          public ButtonListener
+                          private Slider::Listener
 {
 public:
     ToolbarDemoComp()
-        : depthLabel (String(), "Toolbar depth:"),
-          infoLabel (String(), "As well as showing off toolbars, this demo illustrates how to store "
-                               "a set of SVG files in a Zip file, embed that in your application, and read "
-                               "them back in at runtime.\n\nThe icon images here are taken from the open-source "
-                                "Tango icon project."),
+        : depthLabel ({}, "Toolbar depth:"),
+          infoLabel ({}, "As well as showing off toolbars, this demo illustrates how to store "
+                         "a set of SVG files in a Zip file, embed that in your application, and read "
+                         "them back in at runtime.\n\nThe icon images here are taken from the open-source "
+                         "Tango icon project."),
           orientationButton ("Vertical/Horizontal"),
           customiseButton ("Customise...")
     {
@@ -484,12 +516,12 @@ public:
         depthLabel.attachToComponent (&depthSlider, false);
 
         addAndMakeVisible (orientationButton);
-        orientationButton.addListener (this);
+        orientationButton.onClick = [this] { toolbar.setVertical (! toolbar.isVertical()); resized(); };
         orientationButton.changeWidthToFitText (22);
         orientationButton.setTopLeftPosition (depthSlider.getX(), depthSlider.getBottom() + 20);
 
         addAndMakeVisible (customiseButton);
-        customiseButton.addListener (this);
+        customiseButton.onClick = [this] { toolbar.showCustomisationDialog (factory); };
         customiseButton.changeWidthToFitText (22);
         customiseButton.setTopLeftPosition (orientationButton.getRight() + 20, orientationButton.getY());
     }
@@ -509,19 +541,6 @@ public:
         resized();
     }
 
-    void buttonClicked (Button* button) override
-    {
-        if (button == &orientationButton)
-        {
-            toolbar.setVertical (! toolbar.isVertical());
-            resized();
-        }
-        else if (button == &customiseButton)
-        {
-            toolbar.showCustomisationDialog (factory);
-        }
-    }
-
 private:
     Toolbar toolbar;
     Slider depthSlider;
@@ -745,10 +764,12 @@ public:
     // This is overloaded from TableListBoxModel, and should fill in the background of the whole row
     void paintRowBackground (Graphics& g, int rowNumber, int /*width*/, int /*height*/, bool rowIsSelected) override
     {
+        const Colour alternateColour (getLookAndFeel().findColour (ListBox::backgroundColourId)
+                                                      .interpolatedWith (getLookAndFeel().findColour (ListBox::textColourId), 0.03f));
         if (rowIsSelected)
             g.fillAll (Colours::lightblue);
         else if (rowNumber % 2)
-            g.fillAll (Colour (0xffeeeeee));
+            g.fillAll (alternateColour);
     }
 
     // This is overloaded from TableListBoxModel, and must paint any cells that aren't using custom
@@ -756,7 +777,7 @@ public:
     void paintCell (Graphics& g, int rowNumber, int columnId,
                     int width, int height, bool /*rowIsSelected*/) override
     {
-        g.setColour (Colours::black);
+        g.setColour (getLookAndFeel().findColour (ListBox::textColourId));
         g.setFont (font);
 
         if (const XmlElement* rowElement = dataList->getChildElement (rowNumber))
@@ -766,7 +787,7 @@ public:
             g.drawText (text, 2, 0, width - 4, height, Justification::centredLeft, true);
         }
 
-        g.setColour (Colours::black.withAlpha (0.2f));
+        g.setColour (getLookAndFeel().findColour (ListBox::backgroundColourId));
         g.fillRect (width - 1, 0, 1, height);
     }
 
@@ -888,7 +909,6 @@ private:
         {
             // double click to edit the label text; single click handled below
             setEditable (false, true, false);
-            setColour (textColourId, Colours::black);
         }
 
         void mouseDown (const MouseEvent& event) override
@@ -912,17 +932,25 @@ private:
             setText (owner.getText(columnId, row), dontSendNotification);
         }
 
+        void paint (Graphics& g) override
+        {
+            auto& lf = getLookAndFeel();
+            if (! dynamic_cast<LookAndFeel_V4*> (&lf))
+                lf.setColour (textColourId, Colours::black);
+
+            Label::paint (g);
+        }
+
     private:
         TableDemoComponent& owner;
         int row, columnId;
+        Colour textColour;
     };
 
-
     //==============================================================================
     // This is a custom component containing a combo box, which we're going to put inside
     // our table's "rating" column.
-    class RatingColumnCustomComponent    : public Component,
-                                           private ComboBoxListener
+    class RatingColumnCustomComponent    : public Component
     {
     public:
         RatingColumnCustomComponent (TableDemoComponent& td)  : owner (td)
@@ -937,8 +965,7 @@ private:
             comboBox.addItem ("swingin", 6);
             comboBox.addItem ("wild", 7);
 
-            // when the combo is changed, we'll get a callback.
-            comboBox.addListener (this);
+            comboBox.onChange = [this] { owner.setRating (row, comboBox.getSelectedId()); };
             comboBox.setWantsKeyboardFocus (false);
         }
 
@@ -955,11 +982,6 @@ private:
             comboBox.setSelectedId (owner.getRating (row), dontSendNotification);
         }
 
-        void comboBoxChanged (ComboBox*) override
-        {
-            owner.setRating (row, comboBox.getSelectedId());
-        }
-
     private:
         TableDemoComponent& owner;
         ComboBox comboBox;
@@ -1015,7 +1037,7 @@ private:
                 return columnXml->getStringAttribute ("name");
         }
 
-        return String();
+        return {};
     }
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (TableDemoComponent)
@@ -1063,7 +1085,7 @@ private:
             if (rowIsSelected)
                 g.fillAll (Colours::lightblue);
 
-            g.setColour (Colours::black);
+            g.setColour (LookAndFeel::getDefaultLookAndFeel().findColour (Label::textColourId));
             g.setFont (height * 0.7f);
 
             g.drawText ("Draggable Thing #" + String (rowNumber + 1),
@@ -1110,14 +1132,14 @@ private:
                 g.drawRect (getLocalBounds(), 3);
             }
 
-            g.setColour (Colours::black);
+            g.setColour (getLookAndFeel().findColour (Label::textColourId));
             g.setFont (14.0f);
             g.drawFittedText (message, getLocalBounds().reduced (10, 0), Justification::centred, 4);
         }
 
         //==============================================================================
         // These methods implement the DragAndDropTarget interface, and allow our component
-        // to accept drag-and-drop of objects from other Juce components..
+        // to accept drag-and-drop of objects from other JUCE components..
 
         bool isInterestedInDragSource (const SourceDetails& /*dragSourceDetails*/) override
         {
@@ -1232,38 +1254,119 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (DragAndDropDemo)
 };
 
+//==============================================================================
+struct BurgerMenuHeader  : public Component
+{
+    BurgerMenuHeader()
+    {
+        static const unsigned char burgerMenuPathData[]
+            = { 110,109,0,0,128,64,0,0,32,65,108,0,0,224,65,0,0,32,65,98,254,212,232,65,0,0,32,65,0,0,240,65,252,
+                169,17,65,0,0,240,65,0,0,0,65,98,0,0,240,65,8,172,220,64,254,212,232,65,0,0,192,64,0,0,224,65,0,0,
+                192,64,108,0,0,128,64,0,0,192,64,98,16,88,57,64,0,0,192,64,0,0,0,64,8,172,220,64,0,0,0,64,0,0,0,65,
+                98,0,0,0,64,252,169,17,65,16,88,57,64,0,0,32,65,0,0,128,64,0,0,32,65,99,109,0,0,224,65,0,0,96,65,108,
+                0,0,128,64,0,0,96,65,98,16,88,57,64,0,0,96,65,0,0,0,64,4,86,110,65,0,0,0,64,0,0,128,65,98,0,0,0,64,
+                254,212,136,65,16,88,57,64,0,0,144,65,0,0,128,64,0,0,144,65,108,0,0,224,65,0,0,144,65,98,254,212,232,
+                65,0,0,144,65,0,0,240,65,254,212,136,65,0,0,240,65,0,0,128,65,98,0,0,240,65,4,86,110,65,254,212,232,
+                65,0,0,96,65,0,0,224,65,0,0,96,65,99,109,0,0,224,65,0,0,176,65,108,0,0,128,64,0,0,176,65,98,16,88,57,
+                64,0,0,176,65,0,0,0,64,2,43,183,65,0,0,0,64,0,0,192,65,98,0,0,0,64,254,212,200,65,16,88,57,64,0,0,208,
+                65,0,0,128,64,0,0,208,65,108,0,0,224,65,0,0,208,65,98,254,212,232,65,0,0,208,65,0,0,240,65,254,212,
+                200,65,0,0,240,65,0,0,192,65,98,0,0,240,65,2,43,183,65,254,212,232,65,0,0,176,65,0,0,224,65,0,0,176,
+                65,99,101,0,0 };
+
+        Path p;
+        p.loadPathFromData (burgerMenuPathData, sizeof (burgerMenuPathData));
+        burgerButton.setShape (p, true, true, false);
+
+        burgerButton.onClick = [this] { showOrHide(); };
+        addAndMakeVisible (burgerButton);
+    }
+
+    ~BurgerMenuHeader()
+    {
+        MainAppWindow::getSharedSidePanel().showOrHide (false);
+    }
+
+private:
+    void paint (Graphics& g) override
+    {
+        auto titleBarBackgroundColour = getLookAndFeel().findColour (ResizableWindow::backgroundColourId)
+                                                        .darker();
+
+        g.setColour (titleBarBackgroundColour);
+        g.fillRect (getLocalBounds());
+    }
+
+    void resized() override
+    {
+        auto r = getLocalBounds();
+
+        burgerButton.setBounds (r.removeFromRight (40).withSizeKeepingCentre (20, 20));
+
+        titleLabel.setFont (Font (getHeight() * 0.5f, Font::plain));
+        titleLabel.setBounds (r);
+    }
+
+    void showOrHide()
+    {
+        auto& panel = MainAppWindow::getSharedSidePanel();
+
+        panel.showOrHide (! panel.isPanelShowing());
+    }
+
+    Label titleLabel { "titleLabel", "JUCE Demo" };
+    ShapeButton burgerButton { "burgerButton", Colours::lightgrey, Colours::lightgrey, Colours::white };
+
+    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (BurgerMenuHeader)
+};
+
 //==============================================================================
 class MenusDemo : public Component,
                   public MenuBarModel,
-                  private Button::Listener
+                  public ChangeBroadcaster
 {
 public:
+    //==============================================================================
+    enum MenuBarPosition
+    {
+        window,
+        globalMenuBar,
+        burger
+    };
+
+    //==============================================================================
     MenusDemo()
     {
         addAndMakeVisible (menuBar = new MenuBarComponent (this));
 
         popupButton.setButtonText ("Show Popup Menu");
         popupButton.setTriggeredOnMouseDown (true);
-        popupButton.addListener (this);
+        popupButton.onClick = [this] { getDummyPopupMenu().showMenuAsync (PopupMenu::Options().withTargetComponent (&popupButton), nullptr); };
         addAndMakeVisible (popupButton);
+        addChildComponent (menuHeader);
 
         setApplicationCommandManagerToWatch (&MainAppWindow::getApplicationCommandManager());
     }
 
     ~MenusDemo()
     {
+        MainAppWindow::getSharedSidePanel().setContent (nullptr, false);
+
        #if JUCE_MAC
         MenuBarModel::setMacMainMenu (nullptr);
        #endif
         PopupMenu::dismissAllActiveMenus();
-
-        popupButton.removeListener (this);
     }
 
     void resized() override
     {
         Rectangle<int> area (getLocalBounds());
-        menuBar->setBounds (area.removeFromTop (LookAndFeel::getDefaultLookAndFeel().getDefaultMenuBarHeight()));
+
+        {
+            auto menuBarArea = area.removeFromTop (40);
+
+            menuBar->setBounds (menuBarArea.withHeight (LookAndFeel::getDefaultLookAndFeel().getDefaultMenuBarHeight()));
+            menuHeader.setBounds (menuBarArea);
+        }
 
         area.removeFromTop (20);
         area = area.removeFromTop (33);
@@ -1273,9 +1376,7 @@ public:
     //==============================================================================
     StringArray getMenuBarNames() override
     {
-        const char* const names[] = { "Demo", "Look-and-feel", "Tabs", "Misc", nullptr };
-
-        return StringArray (names);
+        return { "Demo", "Look-and-feel", "Menus", "Tabs", "Misc" };
     }
 
     PopupMenu getMenuForIndex (int menuIndex, const String& /*menuName*/) override
@@ -1296,13 +1397,18 @@ public:
             menu.addCommandItem (commandManager, MainAppWindow::useLookAndFeelV1);
             menu.addCommandItem (commandManager, MainAppWindow::useLookAndFeelV2);
             menu.addCommandItem (commandManager, MainAppWindow::useLookAndFeelV3);
+
+            PopupMenu v4SubMenu;
+            v4SubMenu.addCommandItem (commandManager, MainAppWindow::useLookAndFeelV4Dark);
+            v4SubMenu.addCommandItem (commandManager, MainAppWindow::useLookAndFeelV4Midnight);
+            v4SubMenu.addCommandItem (commandManager, MainAppWindow::useLookAndFeelV4Grey);
+            v4SubMenu.addCommandItem (commandManager, MainAppWindow::useLookAndFeelV4Light);
+
+            menu.addSubMenu ("Use LookAndFeel_V4", v4SubMenu);
+
             menu.addSeparator();
             menu.addCommandItem (commandManager, MainAppWindow::useNativeTitleBar);
 
-           #if JUCE_MAC
-            menu.addItem (6000, "Use Native Menu Bar");
-           #endif
-
            #if ! JUCE_LINUX
             menu.addCommandItem (commandManager, MainAppWindow::goToKioskMode);
            #endif
@@ -1321,6 +1427,14 @@ public:
             }
         }
         else if (menuIndex == 2)
+        {
+            menu.addItem (6000, "Inside Window",   true, menuBarPosition == window);
+           #if JUCE_MAC
+            menu.addItem (6001, "Global Menu Bar", true, menuBarPosition == globalMenuBar);
+           #endif
+            menu.addItem (6002, "Burger Menu",     true, menuBarPosition == burger);
+        }
+        else if (menuIndex == 3)
         {
             if (TabbedComponent* tabs = findParentComponentOfClass<TabbedComponent>())
             {
@@ -1330,7 +1444,7 @@ public:
                 menu.addItem (3003, "Tabs on Right",  true, tabs->getOrientation() == TabbedButtonBar::TabsAtRight);
             }
         }
-        else if (menuIndex == 3)
+        else if (menuIndex == 4)
         {
             return getDummyPopupMenu();
         }
@@ -1343,20 +1457,28 @@ public:
         // most of our menu items are invoked automatically as commands, but we can handle the
         // other special cases here..
 
-        if (menuItemID == 6000)
+        if (menuItemID >= 6000 && menuItemID < 7000)
         {
-           #if JUCE_MAC
-            if (MenuBarModel::getMacMainMenu() != nullptr)
+            auto newPosition = static_cast<MenuBarPosition> (menuItemID - 6000);
+
+            if (newPosition != menuBarPosition)
             {
-                MenuBarModel::setMacMainMenu (nullptr);
-                menuBar->setModel (this);
+                menuBarPosition = newPosition;
+
+                if (menuBarPosition != burger)
+                    MainAppWindow::getSharedSidePanel().showOrHide (false);
+
+               #if JUCE_MAC
+                MenuBarModel::setMacMainMenu (menuBarPosition == globalMenuBar ? this : nullptr);
+               #endif
+                menuBar->setModel     (menuBarPosition == window ? this : nullptr);
+                menuBar->setVisible   (menuBarPosition == window);
+                burgerMenu.setModel   (menuBarPosition == burger ? this : nullptr);
+                menuHeader.setVisible (menuBarPosition == burger);
+
+                MainAppWindow::getSharedSidePanel().setContent (menuBarPosition == burger ? &burgerMenu : nullptr, false);
+                menuItemsChanged();
             }
-            else
-            {
-                menuBar->setModel (nullptr);
-                MenuBarModel::setMacMainMenu (this);
-            }
-           #endif
         }
         else if (menuItemID >= 3000 && menuItemID <= 3003)
         {
@@ -1371,11 +1493,18 @@ public:
                 tabs->setOrientation (o);
             }
         }
+        else if (menuItemID >= 12298 && menuItemID <= 12305)
+        {
+            sendChangeMessage();
+        }
     }
 
 private:
     TextButton popupButton;
     ScopedPointer<MenuBarComponent> menuBar;
+    BurgerMenuComponent burgerMenu;
+    BurgerMenuHeader menuHeader;
+    MenuBarPosition menuBarPosition = window;
 
     PopupMenu getDummyPopupMenu()
     {
@@ -1388,33 +1517,29 @@ private:
         m.addCustomItem (5, new CustomMenuComponent());
         m.addSeparator();
 
-        for (int i = 0; i < 8; ++i)
+        if (menuBarPosition != burger)
         {
-            PopupMenu subMenu;
-
-            for (int s = 0; s < 8; ++s)
+            for (int i = 0; i < 8; ++i)
             {
-                PopupMenu subSubMenu;
+                PopupMenu subMenu;
 
-                for (int item = 0; item < 8; ++item)
-                    subSubMenu.addItem (1000 + (i * s * item), "Item " + String (item + 1));
+                for (int s = 0; s < 8; ++s)
+                {
+                    PopupMenu subSubMenu;
 
-                subMenu.addSubMenu ("Sub-sub menu " + String (s + 1), subSubMenu);
+                    for (int item = 0; item < 8; ++item)
+                        subSubMenu.addItem (1000 + (i * s * item), "Item " + String (item + 1));
+
+                    subMenu.addSubMenu ("Sub-sub menu " + String (s + 1), subSubMenu);
+                }
+
+                m.addSubMenu ("Sub menu " + String (i + 1), subMenu);
             }
-
-            m.addSubMenu ("Sub menu " + String (i + 1), subMenu);
         }
 
         return m;
     }
 
-    //==============================================================================
-    void buttonClicked (Button* button) override
-    {
-        if (button == &popupButton)
-            getDummyPopupMenu().showMenuAsync (PopupMenu::Options().withTargetComponent (&popupButton), nullptr);
-    }
-
     //==============================================================================
     class CustomMenuComponent   : public PopupMenu::CustomComponent,
                                   private Timer
@@ -1453,9 +1578,11 @@ private:
         void timerCallback() override
         {
             Random random;
-            blobPosition.setBounds ((float) random.nextInt (getWidth()),
-                                    (float) random.nextInt (getHeight()),
-                                    40.0f, 30.0f);
+
+            if (! getBounds().isEmpty())
+                blobPosition.setBounds ((float) random.nextInt (getWidth()),
+                                        (float) random.nextInt (getHeight()),
+                                        40.0f, 30.0f);
             repaint();
         }
 
@@ -1466,26 +1593,35 @@ private:
 };
 
 //==============================================================================
-class DemoTabbedComponent  : public TabbedComponent
+class DemoTabbedComponent  : public TabbedComponent,
+                             private ChangeListener
 {
 public:
     DemoTabbedComponent()
         : TabbedComponent (TabbedButtonBar::TabsAtTop)
     {
-        addTab ("Menus",            getRandomTabBackgroundColour(), new MenusDemo(),           true);
-        addTab ("Buttons",          getRandomTabBackgroundColour(), new ButtonsPage(),         true);
-        addTab ("Sliders",          getRandomTabBackgroundColour(), new SlidersPage(),         true);
-        addTab ("Toolbars",         getRandomTabBackgroundColour(), new ToolbarDemoComp(),     true);
-        addTab ("Misc",             getRandomTabBackgroundColour(), new MiscPage(),            true);
-        addTab ("Tables",           getRandomTabBackgroundColour(), new TableDemoComponent(),  true);
-        addTab ("Drag & Drop",      getRandomTabBackgroundColour(), new DragAndDropDemo(),     true);
+        // Register this class as a ChangeListener to the menus demo so we can update the tab colours when the LookAndFeel is changed
+        menusDemo = new MenusDemo();
+        menusDemo->addChangeListener (this);
+
+        const Colour c;
+        addTab ("Menus",       c, menusDemo,                true);
+        addTab ("Buttons",     c, new ButtonsPage(),        true);
+        addTab ("Sliders",     c, new SlidersPage(),        true);
+        addTab ("Toolbars",    c, new ToolbarDemoComp(),    true);
+        addTab ("Misc",        c, new MiscPage(),           true);
+        addTab ("Tables",      c, new TableDemoComponent(), true);
+        addTab ("Drag & Drop", c, new DragAndDropDemo(),    true);
+
+        updateTabColours();
 
         getTabbedButtonBar().getTabButton (5)->setExtraComponent (new CustomTabButton(), TabBarButton::afterText);
     }
 
-    static Colour getRandomTabBackgroundColour()
+    void changeListenerCallback (ChangeBroadcaster* source) override
     {
-        return Colour (Random::getSystemRandom().nextFloat(), 0.1f, 0.97f, 1.0f);
+        if (dynamic_cast<MenusDemo*> (source) != nullptr)
+            updateTabColours();
     }
 
     // This is a small star button that is put inside one of the tabs. You can
@@ -1509,7 +1645,7 @@ public:
 
         void mouseDown (const MouseEvent&) override
         {
-            showBubbleMessage (this,
+            showBubbleMessage (*this,
                                "This is a custom tab component\n"
                                "\n"
                                "You can use these to implement things like close-buttons "
@@ -1519,12 +1655,31 @@ public:
     private:
         ScopedPointer<BubbleMessageComponent> bubbleMessage;
     };
+
+private:
+    ScopedPointer<MenusDemo> menusDemo; //need to have keep a pointer around to register this class as a ChangeListener
+
+    void updateTabColours()
+    {
+        bool randomiseColours = ! dynamic_cast<LookAndFeel_V4*> (&LookAndFeel::getDefaultLookAndFeel());
+        for (int i = 0; i < getNumTabs(); ++i)
+        {
+            if (randomiseColours)
+                setTabBackgroundColour (i, Colour (Random::getSystemRandom().nextFloat(), 0.1f, 0.97f, 1.0f));
+            else
+                setTabBackgroundColour (i, getLookAndFeel().findColour (ResizableWindow::backgroundColourId));
+        }
+    }
+
+    void lookAndFeelChanged() override
+    {
+        updateTabColours();
+    }
 };
 
 //==============================================================================
-class WidgetsDemo   : public Component
+struct WidgetsDemo   : public Component
 {
-public:
     WidgetsDemo()
     {
         setOpaque (true);
@@ -1541,7 +1696,6 @@ public:
         tabs.setBounds (getLocalBounds().reduced (4));
     }
 
-private:
     DemoTabbedComponent tabs;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (WidgetsDemo)
