@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -185,8 +187,7 @@ private:
 };
 
 //==============================================================================
-class WindowsDemo   : public Component,
-                      private Button::Listener
+class WindowsDemo   : public Component
 {
 public:
     enum Windows
@@ -203,11 +204,11 @@ public:
 
         showWindowsButton.setButtonText ("Show Windows");
         addAndMakeVisible (showWindowsButton);
-        showWindowsButton.addListener (this);
+        showWindowsButton.onClick = [this] { showAllWindows(); };
 
         closeWindowsButton.setButtonText ("Close Windows");
         addAndMakeVisible (closeWindowsButton);
-        closeWindowsButton.addListener (this);
+        closeWindowsButton.onClick = [this] { closeAllWindows(); };
     }
 
     ~WindowsDemo()
@@ -222,14 +223,12 @@ public:
         }
 
         closeAllWindows();
-
-        closeWindowsButton.removeListener (this);
-        showWindowsButton.removeListener (this);
     }
 
     void paint (Graphics& g) override
     {
-        g.fillAll (Colours::grey);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground,
+                                           Colours::grey));
     }
 
     void resized() override
@@ -246,7 +245,7 @@ private:
     // Because in this demo the windows delete themselves, we'll use the
     // Component::SafePointer class to point to them, which automatically becomes
     // null when the component that it points to is deleted.
-    Array< Component::SafePointer<Component> > windows;
+    Array<Component::SafePointer<Component>> windows;
     TextButton showWindowsButton, closeWindowsButton;
     SafePointer<DialogWindow> dialogWindow;
 
@@ -338,14 +337,6 @@ private:
         balls->setVisible (true);
     }
 
-    void buttonClicked (Button* button) override
-    {
-        if (button == &showWindowsButton)
-            showAllWindows();
-        else if (button == &closeWindowsButton)
-            closeAllWindows();
-    }
-
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (WindowsDemo)
 };
 
