@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -206,7 +208,6 @@ private:
 
 //==============================================================================
 class StringsDemo   : public Component,
-                      private ComboBox::Listener,
                       private CodeDocument::Listener
 {
 public:
@@ -223,10 +224,17 @@ public:
         setOpaque (true);
 
         addAndMakeVisible (typeBox);
-        typeBox.addListener (this);
         typeBox.addItem ("XML", 1);
         typeBox.addItem ("JSON", 2);
 
+        typeBox.onChange = [this]
+        {
+            if (typeBox.getSelectedId() == 1)
+                reset (xml);
+            else
+                reset (json);
+        };
+
         comboBoxLabel.setText ("Database Type:", dontSendNotification);
         comboBoxLabel.attachToComponent (&typeBox, true);
 
@@ -254,7 +262,7 @@ public:
 
     void paint (Graphics& g) override
     {
-        fillStandardDemoBackground (g);
+        g.fillAll (getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::windowBackground));
     }
 
     void resized() override
@@ -294,14 +302,14 @@ private:
     {
         // clear the current tree
         resultsTree.setRootItem (nullptr);
-        rootItem = nullptr;
+        rootItem.reset();
 
         // try and parse the editor's contents
         switch (typeBox.getSelectedItemIndex())
         {
             case xml:           rootItem = rebuildXml();        break;
             case json:          rootItem = rebuildJson();       break;
-            default:            rootItem = nullptr;             break;
+            default:            rootItem.reset();               break;
         }
 
         // if we have a valid TreeViewItem hide any old error messages and set our TreeView to use it
@@ -316,7 +324,7 @@ private:
     /** Parses the editors contects as XML. */
     TreeViewItem* rebuildXml()
     {
-        parsedXml = nullptr;
+        parsedXml.reset();
 
         XmlDocument doc (codeDocument.getAllContent());
         parsedXml = doc.getDocumentElement();
@@ -362,17 +370,6 @@ private:
         }
     }
 
-    void comboBoxChanged (ComboBox* box) override
-    {
-        if (box == &typeBox)
-        {
-            if (typeBox.getSelectedId() == 1)
-                reset (xml);
-            else
-                reset (json);
-        }
-    }
-
     void codeDocumentTextInserted (const String&, int) override     { rebuildTree(); }
     void codeDocumentTextDeleted (int, int) override                { rebuildTree(); }
 
