@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -55,14 +57,13 @@ Array<JuceDemoTypeBase*>& JuceDemoTypeBase::getDemoTypeList()
 #if JUCE_WINDOWS || JUCE_LINUX || JUCE_MAC
 
 // Just add a simple icon to the Window system tray area or Mac menu bar..
-class DemoTaskbarComponent  : public SystemTrayIconComponent,
-                              private Timer
+struct DemoTaskbarComponent  : public SystemTrayIconComponent,
+                               private Timer
 {
-public:
     DemoTaskbarComponent()
     {
         setIconImage (ImageCache::getFromMemory (BinaryData::juce_icon_png, BinaryData::juce_icon_pngSize));
-        setIconTooltip ("Juce Demo App!");
+        setIconTooltip ("JUCE Demo App!");
     }
 
     void mouseDown (const MouseEvent&) override
@@ -83,13 +84,12 @@ public:
             JUCEApplication::getInstance()->systemRequestedQuit();
     }
 
-private:
     void timerCallback() override
     {
         stopTimer();
 
         PopupMenu m;
-        m.addItem (1, "Quit the Juce demo");
+        m.addItem (1, "Quit the JUCE demo");
 
         // It's always better to open menus asynchronously when possible.
         m.showMenuAsync (PopupMenu::Options(),
@@ -113,23 +113,35 @@ public:
         lookAndFeelV3.setColour (Label::textColourId, Colours::white);
         lookAndFeelV3.setColour (Label::textColourId, Colours::white);
         lookAndFeelV3.setColour (ToggleButton::textColourId, Colours::white);
-        LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV3);
+
+        LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV4);
 
         demoList.setModel (this);
-        demoList.setColour (ListBox::backgroundColourId, Colour::greyLevel (0.2f));
+        updateDemoListColours();
         demoList.selectRow (0);
-        addAndMakeVisible (demoList);
 
+        if (Desktop::getInstance().getMainMouseSource().isTouch())
+            demoList.getViewport()->setScrollOnDragEnabled (true);
+
+        addAndMakeVisible (demoList);
+        addAndMakeVisible (sidePanel);
+        sidePanel.setAlwaysOnTop (true);
+    }
+
+    ~ContentComponent()
+    {
+        // before deleting our lookandfeel object, make sure it's no longer in use
+        LookAndFeel::setDefaultLookAndFeel (nullptr);
     }
 
     void clearCurrentDemo()
     {
-        currentDemo = nullptr;
+        currentDemo.reset();
     }
 
     void resized() override
     {
-        Rectangle<int> r (getLocalBounds());
+        auto r = getLocalBounds();
 
         if (r.getWidth() > 600)
         {
@@ -156,9 +168,9 @@ public:
         if (rowIsSelected)
             g.fillAll (Colours::deepskyblue);
 
-        if (JuceDemoTypeBase* type = JuceDemoTypeBase::getDemoTypeList() [rowNumber])
+        if (auto* type = JuceDemoTypeBase::getDemoTypeList() [rowNumber])
         {
-            String name (type->name.trimCharactersAtStart ("0123456789").trimStart());
+            auto name = type->name.trimCharactersAtStart ("0123456789").trimStart();
 
             AttributedString a;
             a.setJustification (Justification::centredLeft);
@@ -176,10 +188,13 @@ public:
                     category << " ";
             }
 
-            if (category.isNotEmpty())
-                a.append (category, Font (10.0f), Colour::greyLevel (0.5f));
+            auto categoryColour = demoList.findColour (ListBox::outlineColourId);
+            auto nameColour     = demoList.findColour (ListBox::textColourId);
 
-            a.append (name, Font (13.0f), Colours::white.withAlpha (0.9f));
+            if (category.isNotEmpty())
+                a.append (category, Font (10.0f), categoryColour);
+
+            a.append (name, Font (13.0f), nameColour);
 
             a.draw (g, Rectangle<int> (width + 10, height).reduced (6, 0).toFloat());
         }
@@ -187,9 +202,9 @@ public:
 
     void selectedRowsChanged (int lastRowSelected) override
     {
-        if (JuceDemoTypeBase* selectedDemoType = JuceDemoTypeBase::getDemoTypeList() [lastRowSelected])
+        if (auto* selectedDemoType = JuceDemoTypeBase::getDemoTypeList() [lastRowSelected])
         {
-            currentDemo = nullptr;
+            currentDemo.reset();
             addAndMakeVisible (currentDemo = selectedDemoType->createComponent());
             currentDemo->setName (selectedDemoType->name);
             resized();
@@ -206,7 +221,7 @@ public:
         if (currentDemo == nullptr)
             return -1;
 
-        Array<JuceDemoTypeBase*>& demos (JuceDemoTypeBase::getDemoTypeList());
+        auto& demos = JuceDemoTypeBase::getDemoTypeList();
 
         for (int i = demos.size(); --i >= 0;)
             if (demos.getUnchecked (i)->name == currentDemo->getName())
@@ -217,10 +232,8 @@ public:
 
     void moveDemoPages (int numPagesToMove)
     {
-        const int newIndex = negativeAwareModulo (getCurrentPageIndex() + numPagesToMove,
-                                                  JuceDemoTypeBase::getDemoTypeList().size());
-        demoList.selectRow (newIndex);
-        // we have to go through our demo list here or it won't be updated to reflect the current demo
+        demoList.selectRow (negativeAwareModulo (getCurrentPageIndex() + numPagesToMove,
+                                                 JuceDemoTypeBase::getDemoTypeList().size()));
     }
 
     bool isShowingOpenGLDemo() const
@@ -235,13 +248,20 @@ public:
         return currentDemo != nullptr && currentDemo->getName().contains ("OpenGL 2D");
     }
 
+    SidePanel& getSharedSidePanel()
+    {
+        return sidePanel;
+    }
+
 private:
     ListBox demoList;
     ScopedPointer<Component> currentDemo;
+    SidePanel sidePanel {"Menu", 300, false};
 
     LookAndFeel_V1 lookAndFeelV1;
     LookAndFeel_V2 lookAndFeelV2;
     LookAndFeel_V3 lookAndFeelV3;
+    LookAndFeel_V4 lookAndFeelV4;
 
     //==============================================================================
     // The following methods implement the ApplicationCommandTarget interface, allowing
@@ -272,6 +292,10 @@ private:
                                   MainAppWindow::useLookAndFeelV1,
                                   MainAppWindow::useLookAndFeelV2,
                                   MainAppWindow::useLookAndFeelV3,
+                                  MainAppWindow::useLookAndFeelV4Dark,
+                                  MainAppWindow::useLookAndFeelV4Midnight,
+                                  MainAppWindow::useLookAndFeelV4Grey,
+                                  MainAppWindow::useLookAndFeelV4Light,
                                   MainAppWindow::toggleRepaintDebugging,
                                  #if ! JUCE_LINUX
                                   MainAppWindow::goToKioskMode,
@@ -285,7 +309,7 @@ private:
                                         MainAppWindow::renderingEngineTwo,
                                         MainAppWindow::renderingEngineThree };
 
-        StringArray renderingEngines (MainAppWindow::getMainAppWindow()->getRenderingEngines());
+        auto renderingEngines = MainAppWindow::getMainAppWindow()->getRenderingEngines();
         commands.addArray (engineIDs, renderingEngines.size());
     }
 
@@ -355,8 +379,8 @@ private:
             case MainAppWindow::renderingEngineTwo:
             case MainAppWindow::renderingEngineThree:
             {
-                MainAppWindow& mainWindow = *MainAppWindow::getMainAppWindow();
-                const StringArray engines (mainWindow.getRenderingEngines());
+                auto& mainWindow = *MainAppWindow::getMainAppWindow();
+                auto engines = mainWindow.getRenderingEngines();
                 const int index = commandID - MainAppWindow::renderingEngineOne;
 
                 result.setInfo ("Use " + engines[index], "Uses the " + engines[index] + " engine to render the UI", generalCategory, 0);
@@ -384,6 +408,27 @@ private:
                 result.setTicked (isLookAndFeelSelected<LookAndFeel_V3>());
                 break;
 
+            case MainAppWindow::useLookAndFeelV4Dark:
+                result.setInfo ("Use LookAndFeel_V4 Dark", String(), generalCategory, 0);
+                result.addDefaultKeypress ('k', ModifierKeys::commandModifier);
+                result.setTicked (isColourSchemeActive (LookAndFeel_V4::getDarkColourScheme()));
+                break;
+
+            case MainAppWindow::useLookAndFeelV4Midnight:
+                result.setInfo ("Use LookAndFeel_V4 Midnight", String(), generalCategory, 0);
+                result.setTicked (isColourSchemeActive (LookAndFeel_V4::getMidnightColourScheme()));
+                break;
+
+            case MainAppWindow::useLookAndFeelV4Grey:
+                result.setInfo ("Use LookAndFeel_V4 Grey", String(), generalCategory, 0);
+                result.setTicked (isColourSchemeActive (LookAndFeel_V4::getGreyColourScheme()));
+                break;
+
+            case MainAppWindow::useLookAndFeelV4Light:
+                result.setInfo ("Use LookAndFeel_V4 Light", String(), generalCategory, 0);
+                result.setTicked (isColourSchemeActive (LookAndFeel_V4::getLightColourScheme()));
+                break;
+
             case MainAppWindow::toggleRepaintDebugging:
                 result.setInfo ("Toggle repaint display", String(), generalCategory, 0);
                 result.addDefaultKeypress ('r', ModifierKeys());
@@ -396,8 +441,8 @@ private:
                 result.addDefaultKeypress ('n', ModifierKeys::commandModifier);
                 bool nativeTitlebar = false;
 
-                if (MainAppWindow* map = MainAppWindow::getMainAppWindow())
-                    nativeTitlebar = map->isUsingNativeTitleBar();
+                if (auto* mainWindow = MainAppWindow::getMainAppWindow())
+                    nativeTitlebar = mainWindow->isUsingNativeTitleBar();
 
                 result.setTicked (nativeTitlebar);
                 break;
@@ -418,63 +463,90 @@ private:
 
     bool perform (const InvocationInfo& info) override
     {
-        MainAppWindow* mainWindow = MainAppWindow::getMainAppWindow();
-
-        if (mainWindow == nullptr)
-            return true;
-
-        switch (info.commandID)
+        if (auto* mainWindow = MainAppWindow::getMainAppWindow())
         {
-            case MainAppWindow::showPreviousDemo:   moveDemoPages (-1); break;
-            case MainAppWindow::showNextDemo:       moveDemoPages ( 1); break;
-
-            case MainAppWindow::welcome:
-            case MainAppWindow::componentsAnimation:
-            case MainAppWindow::componentsDialogBoxes:
-            case MainAppWindow::componentsKeyMappings:
-            case MainAppWindow::componentsMDI:
-            case MainAppWindow::componentsPropertyEditors:
-            case MainAppWindow::componentsTransforms:
-            case MainAppWindow::componentsWebBrowsers:
-            case MainAppWindow::componentsWidgets:
-                demoList.selectRow (info.commandID - MainAppWindow::welcome);
-                break;
-
-            case MainAppWindow::renderingEngineOne:
-            case MainAppWindow::renderingEngineTwo:
-            case MainAppWindow::renderingEngineThree:
-                mainWindow->setRenderingEngine (info.commandID - MainAppWindow::renderingEngineOne);
-                break;
-
-            case MainAppWindow::useLookAndFeelV1:  LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV1); break;
-            case MainAppWindow::useLookAndFeelV2:  LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV2); break;
-            case MainAppWindow::useLookAndFeelV3:  LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV3); break;
-
-            case MainAppWindow::toggleRepaintDebugging:
-                juceDemoRepaintDebuggingActive = ! juceDemoRepaintDebuggingActive;
-                mainWindow->repaint();
-                break;
-
-            case MainAppWindow::useNativeTitleBar:
-                mainWindow->setUsingNativeTitleBar (! mainWindow->isUsingNativeTitleBar());
-                break;
-
-           #if ! JUCE_LINUX
-            case MainAppWindow::goToKioskMode:
-                {
-                    Desktop& desktop = Desktop::getInstance();
-
-                    if (desktop.getKioskModeComponent() == nullptr)
-                        desktop.setKioskModeComponent (getTopLevelComponent());
-                    else
-                        desktop.setKioskModeComponent (nullptr);
+            switch (info.commandID)
+            {
+                case MainAppWindow::showPreviousDemo:   moveDemoPages (-1); break;
+                case MainAppWindow::showNextDemo:       moveDemoPages ( 1); break;
 
+                case MainAppWindow::welcome:
+                case MainAppWindow::componentsAnimation:
+                case MainAppWindow::componentsDialogBoxes:
+                case MainAppWindow::componentsKeyMappings:
+                case MainAppWindow::componentsMDI:
+                case MainAppWindow::componentsPropertyEditors:
+                case MainAppWindow::componentsTransforms:
+                case MainAppWindow::componentsWebBrowsers:
+                case MainAppWindow::componentsWidgets:
+                    demoList.selectRow (info.commandID - MainAppWindow::welcome);
                     break;
-                }
-           #endif
 
-            default:
-                return false;
+                case MainAppWindow::renderingEngineOne:
+                case MainAppWindow::renderingEngineTwo:
+                case MainAppWindow::renderingEngineThree:
+                    mainWindow->setRenderingEngine (info.commandID - MainAppWindow::renderingEngineOne);
+                    break;
+
+                case MainAppWindow::useLookAndFeelV1:
+                    LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV1);
+                    updateDemoListColours();
+                    break;
+                case MainAppWindow::useLookAndFeelV2:
+                    LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV2);
+                    updateDemoListColours();
+                    break;
+                case MainAppWindow::useLookAndFeelV3:
+                    LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV3);
+                    updateDemoListColours();
+                    break;
+                case MainAppWindow::useLookAndFeelV4Dark:
+                    lookAndFeelV4.setColourScheme (LookAndFeel_V4::getDarkColourScheme());
+                    LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV4);
+                    updateDemoListColours();
+                    break;
+                case MainAppWindow::useLookAndFeelV4Midnight:
+                    lookAndFeelV4.setColourScheme (LookAndFeel_V4::getMidnightColourScheme());
+                    LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV4);
+                    updateDemoListColours();
+                    break;
+                case MainAppWindow::useLookAndFeelV4Grey:
+                    lookAndFeelV4.setColourScheme (LookAndFeel_V4::getGreyColourScheme());
+                    LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV4);
+                    updateDemoListColours();
+                    break;
+                case MainAppWindow::useLookAndFeelV4Light:
+                    lookAndFeelV4.setColourScheme (LookAndFeel_V4::getLightColourScheme());
+                    LookAndFeel::setDefaultLookAndFeel (&lookAndFeelV4);
+                    updateDemoListColours();
+                    break;
+
+                case MainAppWindow::toggleRepaintDebugging:
+                    juceDemoRepaintDebuggingActive = ! juceDemoRepaintDebuggingActive;
+                    mainWindow->repaint();
+                    break;
+
+                case MainAppWindow::useNativeTitleBar:
+                    mainWindow->setUsingNativeTitleBar (! mainWindow->isUsingNativeTitleBar());
+                    break;
+
+               #if ! JUCE_LINUX
+                case MainAppWindow::goToKioskMode:
+                    {
+                        auto& desktop = Desktop::getInstance();
+
+                        if (desktop.getKioskModeComponent() == nullptr)
+                            desktop.setKioskModeComponent (getTopLevelComponent());
+                        else
+                            desktop.setKioskModeComponent (nullptr);
+
+                        break;
+                    }
+               #endif
+
+                default:
+                    return false;
+            }
         }
 
         return true;
@@ -486,6 +558,28 @@ private:
         LookAndFeel& lf = getLookAndFeel();
         return typeid (LookAndFeelType) == typeid (lf);
     }
+
+    bool isColourSchemeActive (LookAndFeel_V4::ColourScheme scheme)
+    {
+        if (auto* v4 = dynamic_cast<LookAndFeel_V4*> (&LookAndFeel::getDefaultLookAndFeel()))
+            if (v4->getCurrentColourScheme() == scheme)
+                return true;
+
+        return false;
+    }
+
+
+    void updateDemoListColours()
+    {
+        demoList.setColour (ListBox::backgroundColourId,
+                            getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::widgetBackground, Colour::greyLevel (0.2f)));
+        demoList.setColour (ListBox::textColourId,
+                            getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::defaultText,
+                                                    Colours::white.withAlpha (0.9f)));
+        demoList.setColour (ListBox::outlineColourId,
+                            getUIColourIfAvailable (LookAndFeel_V4::ColourScheme::UIColour::defaultText,
+                                                    Colour::greyLevel (0.5f)).interpolatedWith (Colours::red, 0.4f));
+    }
 };
 
 //==============================================================================
@@ -494,7 +588,7 @@ static ScopedPointer<AudioDeviceManager> sharedAudioDeviceManager;
 
 MainAppWindow::MainAppWindow()
     : DocumentWindow (JUCEApplication::getInstance()->getApplicationName(),
-                      Colours::lightgrey,
+                      LookAndFeel::getDefaultLookAndFeel().findColour (ResizableWindow::backgroundColourId),
                       DocumentWindow::allButtons)
 {
     setUsingNativeTitleBar (true);
@@ -532,9 +626,9 @@ MainAppWindow::~MainAppWindow()
 {
     contentComponent->clearCurrentDemo();
     clearContentComponent();
-    contentComponent = nullptr;
-    applicationCommandManager = nullptr;
-    sharedAudioDeviceManager = nullptr;
+    contentComponent.reset();
+    applicationCommandManager.reset();
+    sharedAudioDeviceManager.reset();
 
    #if JUCE_OPENGL
     openGLContext.detach();
@@ -559,13 +653,13 @@ AudioDeviceManager& MainAppWindow::getSharedAudioDeviceManager()
     if (sharedAudioDeviceManager == nullptr)
     {
         sharedAudioDeviceManager = new AudioDeviceManager();
-        RuntimePermissions::request (RuntimePermissions::recordAudio, runtimPermissionsCallback);
+        RuntimePermissions::request (RuntimePermissions::recordAudio, runtimePermissionsCallback);
     }
 
     return *sharedAudioDeviceManager;
 }
 
-void MainAppWindow::runtimPermissionsCallback (bool wasGranted)
+void MainAppWindow::runtimePermissionsCallback (bool wasGranted)
 {
     int numInputChannels = wasGranted ? 2 : 0;
     sharedAudioDeviceManager->initialise (numInputChannels, 2, nullptr, true, String(), nullptr);
@@ -574,7 +668,7 @@ void MainAppWindow::runtimPermissionsCallback (bool wasGranted)
 MainAppWindow* MainAppWindow::getMainAppWindow()
 {
     for (int i = TopLevelWindow::getNumTopLevelWindows(); --i >= 0;)
-        if (MainAppWindow* maw = dynamic_cast<MainAppWindow*> (TopLevelWindow::getTopLevelWindow (i)))
+        if (auto* maw = dynamic_cast<MainAppWindow*> (TopLevelWindow::getTopLevelWindow (i)))
             return maw;
 
     return nullptr;
@@ -584,7 +678,8 @@ void MainAppWindow::handleAsyncUpdate()
 {
     // This registers all of our commands with the command manager but has to be done after the window has
     // been created so we can find the number of rendering engines available
-    ApplicationCommandManager& commandManager = MainAppWindow::getApplicationCommandManager();
+    auto& commandManager = MainAppWindow::getApplicationCommandManager();
+
     commandManager.registerAllCommandsForTarget (contentComponent);
     commandManager.registerAllCommandsForTarget (JUCEApplication::getInstance());
 }
@@ -597,7 +692,7 @@ void MainAppWindow::showMessageBubble (const String& text)
     AttributedString attString;
     attString.append (text, Font (15.0f));
 
-    currentBubbleMessage->showAt (Rectangle<int> (getLocalBounds().getCentreX(), 10, 1, 1),
+    currentBubbleMessage->showAt ({ getLocalBounds().getCentreX(), 10, 1, 1 },
                                   attString,
                                   500,  // numMillisecondsBeforeRemoving
                                   true,  // removeWhenMouseClicked
@@ -610,7 +705,7 @@ StringArray MainAppWindow::getRenderingEngines() const
 {
     StringArray renderingEngines;
 
-    if (ComponentPeer* peer = getPeer())
+    if (auto* peer = getPeer())
         renderingEngines = peer->getAvailableRenderingEngines();
 
    #if JUCE_OPENGL
@@ -636,7 +731,7 @@ void MainAppWindow::setRenderingEngine (int index)
     openGLContext.detach();
    #endif
 
-    if (ComponentPeer* peer = getPeer())
+    if (auto* peer = getPeer())
         peer->setCurrentRenderingEngine (index);
 }
 
@@ -652,12 +747,17 @@ int MainAppWindow::getActiveRenderingEngine() const
         return getRenderingEngines().indexOf (openGLRendererName);
    #endif
 
-    if (ComponentPeer* peer = getPeer())
+    if (auto* peer = getPeer())
         return peer->getCurrentRenderingEngine();
 
     return 0;
 }
 
+SidePanel& MainAppWindow::getSharedSidePanel()
+{
+    return getMainAppWindow()->contentComponent->getSharedSidePanel();
+}
+
 Path MainAppWindow::getJUCELogoPath()
 {
     return Drawable::parseSVGPath (
@@ -674,4 +774,4 @@ Path MainAppWindow::getJUCELogoPath()
         "M232.4,353c-8.1,0-15-6-15-15.8v-22.5h5.6v22.2c0,6.6,3.9,10.8,9.5,10.8c5.6,0,9.5-4.3,9.5-10.8v-22.2h5.6v22.5C247.5,347,240.5,353,232.4,353z"
         "M272,353c-10.8,0-19.5-8.6-19.5-19.3c0-10.8,8.8-19.3,19.5-19.3c4.8,0,9,1.6,12.3,4.4l-3.3,4.1c-3.4-2.4-5.7-3.2-8.9-3.2c-7.7,0-13.8,6.2-13.8,14.1c0,7.9,6.1,14.1,13.8,14.1c3.1,0,5.6-1,8.8-3.2l3.3,4.1C280.1,351.9,276.4,353,272,353z"
         "M290.4,352.5v-37.8h22.7v5H296v11.2h16.5v5H296v11.6h17.2v5H290.4z");
-};
+}
