@@ -2,28 +2,29 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef __MAINWINDOW_H_7DB41986__
-#define __MAINWINDOW_H_7DB41986__
+#pragma once
 
 class ContentComponent;
 
@@ -47,6 +48,8 @@ public:
     // (returns a shared AudioDeviceManager object that all the demos can use)
     static AudioDeviceManager& getSharedAudioDeviceManager();
 
+    static SidePanel& getSharedSidePanel();
+
     StringArray getRenderingEngines() const;
     int getActiveRenderingEngine() const;
     void setRenderingEngine (int index);
@@ -82,19 +85,22 @@ public:
         renderingEngineTwo          = 0x2301,
         renderingEngineThree        = 0x2302, // these must be contiguous!
 
-        useLookAndFeelV1            = 0x200b,
-        useLookAndFeelV2            = 0x200c,
-        useLookAndFeelV3            = 0x200d,
+        useLookAndFeelV1            = 0x300a,
+        useLookAndFeelV2            = 0x300b,
+        useLookAndFeelV3            = 0x300c,
+        useLookAndFeelV4Dark        = 0x300d,
+        useLookAndFeelV4Midnight    = 0x300e,
+        useLookAndFeelV4Grey        = 0x300f,
+        useLookAndFeelV4Light       = 0x3010,
 
-        toggleRepaintDebugging      = 0x200e,
+        toggleRepaintDebugging      = 0x201a,
 
         useNativeTitleBar           = 0x201d,
         goToKioskMode               = 0x200f
     };
 
 private:
-
-    static void runtimPermissionsCallback (bool wasGranted);
+    static void runtimePermissionsCallback (bool wasGranted);
 
     ScopedPointer<ContentComponent> contentComponent;
     ScopedPointer<Component> taskbarIcon;
@@ -113,6 +119,3 @@ private:
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MainAppWindow)
 };
-
-
-#endif  // __MAINWINDOW_H_7DB41986__
