@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
 
-#ifndef MPEDEMOSYNTHVOICE_H_INCLUDED
-#define MPEDEMOSYNTHVOICE_H_INCLUDED
+#pragma once
 
 
 class MPEDemoSynthVoice : public MPESynthesiserVoice
@@ -49,7 +50,7 @@ public:
 
         phase = 0.0;
         const double cyclesPerSample = frequency.getNextValue() / currentSampleRate;
-        phaseDelta = 2.0 * double_Pi * cyclesPerSample;
+        phaseDelta = MathConstants<double>::twoPi * cyclesPerSample;
 
         tailOff = 0.0;
     }
@@ -167,8 +168,8 @@ private:
         const float nextSample = float (amplitude * ((a1 * f1) + (a2 * f2)));
 
         const double cyclesPerSample = frequency.getNextValue() / currentSampleRate;
-        phaseDelta = 2.0 * double_Pi * cyclesPerSample;
-        phase = std::fmod (phase + phaseDelta, 2.0 * double_Pi);
+        phaseDelta = MathConstants<double>::twoPi * cyclesPerSample;
+        phase = std::fmod (phase + phaseDelta, MathConstants<double>::twoPi);
 
         return nextSample;
     }
@@ -181,6 +182,3 @@ private:
     const double maxLevelDb = 31.0f;
     const double smoothingLengthInSeconds = 0.01;
 };
-
-
-#endif  // MPEDEMOSYNTHVOICE_H_INCLUDED
