@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -72,13 +74,13 @@ public:
     void paintListBoxItem (int rowNumber, Graphics &g,
                            int width, int height, bool rowIsSelected) override
     {
+        const auto textColour = getLookAndFeel().findColour (ListBox::textColourId);
+
         if (rowIsSelected)
-            g.fillAll (Colours::lightblue);
-        else if (rowNumber % 2)
-            g.fillAll (Colour (0xffeeeeee));
+            g.fillAll (textColour.interpolatedWith (getLookAndFeel().findColour (ListBox::backgroundColourId), 0.5));
 
 
-        g.setColour (Colours::black);
+        g.setColour (textColour);
         g.setFont (height * 0.7f);
 
         if (isInput)
@@ -140,7 +142,7 @@ private:
 };
 
 //==============================================================================
-MainContentComponent::MainContentComponent ()
+MainContentComponent::MainContentComponent()
     : midiInputLabel ("Midi Input Label", "MIDI Input:"),
       midiOutputLabel ("Midi Output Label", "MIDI Output:"),
       incomingMidiLabel ("Incoming Midi Label", "Received MIDI messages:"),
@@ -209,9 +211,8 @@ MainContentComponent::~MainContentComponent()
 }
 
 //==============================================================================
-void MainContentComponent::paint (Graphics& g)
+void MainContentComponent::paint (Graphics&)
 {
-    g.fillAll (Colours::white);
 }
 
 //==============================================================================
@@ -340,7 +341,7 @@ void MainContentComponent::updateDeviceList (bool isInputDeviceList)
 }
 
 //==============================================================================
-void MainContentComponent::timerCallback ()
+void MainContentComponent::timerCallback()
 {
     updateDeviceList (true);
     updateDeviceList (false);
