@@ -1,48 +1,44 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   ------------------------------------------------------------------------------
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef OSCLOGLISTBOX_H_INCLUDED
-#define OSCLOGLISTBOX_H_INCLUDED
+#pragma once
 
 #include "../JuceLibraryCode/JuceHeader.h"
 
 //==============================================================================
-class OSCLogListBox    : public ListBox, private ListBoxModel, private AsyncUpdater
+class OSCLogListBox    : public ListBox,
+                         private ListBoxModel,
+                         private AsyncUpdater
 {
 public:
-
-    //==============================================================================
     OSCLogListBox()
     {
         setModel (this);
     }
 
-    //==============================================================================
     ~OSCLogListBox()
     {
     }
@@ -60,7 +56,7 @@ public:
 
         if (isPositiveAndBelow (row, oscLogList.size()))
         {
-            g.setColour (Colours::black);
+            g.setColour (Colours::white);
 
             g.drawText (oscLogList[row],
                         Rectangle<int> (width, height).reduced (4, 0),
@@ -80,8 +76,8 @@ public:
 
         if (! message.isEmpty())
         {
-            for (OSCArgument* arg = message.begin(); arg != message.end(); ++arg)
-                addOSCMessageArgument (*arg, level + 1);
+            for (auto& arg : message)
+                addOSCMessageArgument (arg, level + 1);
         }
 
         triggerAsyncUpdate();
@@ -96,12 +92,12 @@ public:
                         + "- osc bundle, time tag = "
                         + timeTag.toTime().toString (true, true, true, true));
 
-        for (OSCBundle::Element* element = bundle.begin(); element != bundle.end(); ++element)
+        for (auto& element : bundle)
         {
-            if (element->isMessage())
-                addOSCMessage (element->getMessage(), level + 1);
-            else if (element->isBundle())
-                addOSCBundle (element->getBundle(), level + 1);
+            if (element.isMessage())
+                addOSCMessage (element.getMessage(), level + 1);
+            else if (element.isBundle())
+                addOSCBundle (element.getBundle(), level + 1);
         }
 
         triggerAsyncUpdate();
@@ -131,13 +127,12 @@ public:
         else if (arg.isBlob())
         {
             typeAsString = "blob";
-            const MemoryBlock& blob = arg.getBlob();
-            valueAsString = String::fromUTF8( (const char*)blob.getData(), blob.getSize());
+            auto& blob = arg.getBlob();
+            valueAsString = String::fromUTF8 ((const char*) blob.getData(), (int) blob.getSize());
         }
         else
         {
             typeAsString = "(unknown)";
-            valueAsString = "";
         }
 
         oscLogList.add (getIndentationString (level + 1) + "- " + typeAsString.paddedRight(' ', 12) + valueAsString);
@@ -165,19 +160,13 @@ public:
     }
 
 private:
-
-    //==============================================================================
-    String getIndentationString (int level)
+    static String getIndentationString (int level)
     {
         return String().paddedRight (' ', 2 * level);
     }
 
-
     //==============================================================================
     StringArray oscLogList;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (OSCLogListBox)
 };
-
-
-#endif  // OSCLOGLISTBOX_H_INCLUDED
