@@ -1,24 +1,25 @@
 /*
   ==============================================================================
 
-   This file is part of the JUCE library - "Jules' Utility Class Extensions"
-   Copyright 2015 by ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-  ------------------------------------------------------------------------------
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   JUCE can be redistributed and/or modified under the terms of the GNU General
-   Public License (Version 2), as published by the Free Software Foundation.
-   A copy of the license is included in the JUCE distribution, or can be found
-   online at www.gnu.org/licenses.
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-  ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -175,7 +176,7 @@ private:
 
         if (CharacterFunctions::compareUpTo (CharPointer_ASCII (token), t, len) == 0)
         {
-            String::CharPointerType end = t + len;
+            auto end = t + len;
 
             if (end.isEmpty() || end.isWhitespace())
             {
@@ -248,7 +249,7 @@ private:
     };
 
     static Shape* parseFaceGroup (const Mesh& srcMesh,
-                                  const Array<Face>& faceGroup,
+                                  Array<Face>& faceGroup,
                                   const Material& material,
                                   const String& name)
     {
@@ -278,7 +279,7 @@ private:
 
         for (int lineNum = 0; lineNum < lines.size(); ++lineNum)
         {
-            String::CharPointerType l = lines[lineNum].getCharPointer().findEndOfWhitespace();
+            auto l = lines[lineNum].getCharPointer().findEndOfWhitespace();
 
             if (matchToken (l, "v"))    { mesh.vertices.add (parseVertex (l));            continue; }
             if (matchToken (l, "vn"))   { mesh.normals.add (parseVertex (l));             continue; }
@@ -340,7 +341,7 @@ private:
 
         for (int i = 0; i < lines.size(); ++i)
         {
-            String::CharPointerType l (lines[i].getCharPointer().findEndOfWhitespace());
+            auto l = lines[i].getCharPointer().findEndOfWhitespace();
 
             if (matchToken (l, "newmtl"))   { materials.add (material); material.name = String (l).trim(); continue; }
 
