@@ -1,12 +1,29 @@
 /*
   ==============================================================================
 
-   JUCE demo code - use at your own risk!
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
+
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
+
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
+
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
+
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
+
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-
 /*
     This component represents a horizontal vibrating musical string of fixed height
     and variable length. The string can be excited by calling stringPlucked().
@@ -27,8 +44,8 @@ public:
     //==============================================================================
     void stringPlucked (float pluckPositionRelative)
     {
-        amplitude = maxAmplitude * std::sin (pluckPositionRelative * float_Pi);
-        phase = float_Pi;
+        amplitude = maxAmplitude * std::sin (pluckPositionRelative * MathConstants<float>::pi);
+        phase = MathConstants<float>::pi;
     }
 
     //==============================================================================
@@ -40,7 +57,7 @@ public:
 
     Path generateStringPath() const
     {
-        const float y = height / 2.0f;
+        auto y = height / 2.0f;
 
         Path stringPath;
         stringPath.startNewSubPath (0, y);
@@ -70,8 +87,8 @@ public:
 
         phase += phaseStep;
 
-        if (phase > float_Pi)
-            phase -= 2.0f * float_Pi;
+        if (phase >= MathConstants<float>::twoPi)
+            phase -= MathConstants<float>::twoPi;
     }
 
 private:
