@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -50,8 +52,6 @@ public:
         appProperties = new ApplicationProperties();
         appProperties->setStorageParameters (options);
 
-        LookAndFeel::setDefaultLookAndFeel (&lookAndFeel);
-
         mainWindow = new MainHostWindow();
         mainWindow->setUsingNativeTitleBar (true);
 
@@ -93,8 +93,8 @@ public:
         }
 
         if (fileToOpen.existsAsFile())
-            if (GraphDocumentComponent* graph = mainWindow->getGraphEditor())
-                if (FilterGraph* ioGraph = graph->graph.get())
+            if (auto* graph = mainWindow->graphHolder.get())
+                if (auto* ioGraph = graph->graph.get())
                     ioGraph->loadFrom (fileToOpen, true);
     }
 
@@ -119,7 +119,6 @@ public:
 
     ApplicationCommandManager commandManager;
     ScopedPointer<ApplicationProperties> appProperties;
-    LookAndFeel_V3 lookAndFeel;
 
 private:
     ScopedPointer<MainHostWindow> mainWindow;
