@@ -2,41 +2,43 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#include "../jucer_Headers.h"
+#include "../Application/jucer_Headers.h"
 #include "jucer_PaintRoutine.h"
 #include "jucer_JucerDocument.h"
 #include "jucer_ObjectTypes.h"
-#include "paintelements/jucer_PaintElementUndoableAction.h"
-#include "paintelements/jucer_PaintElementPath.h"
-#include "paintelements/jucer_PaintElementImage.h"
-#include "paintelements/jucer_PaintElementGroup.h"
-#include "ui/jucer_JucerDocumentEditor.h"
-
+#include "PaintElements/jucer_PaintElementUndoableAction.h"
+#include "PaintElements/jucer_PaintElementPath.h"
+#include "PaintElements/jucer_PaintElementImage.h"
+#include "PaintElements/jucer_PaintElementGroup.h"
+#include "UI/jucer_JucerDocumentEditor.h"
+#include "../Application/jucer_Application.h"
 
 //==============================================================================
 PaintRoutine::PaintRoutine()
     : document (nullptr),
-      backgroundColour (Colours::white)
+      backgroundColour (ProjucerApplication::getApp().lookAndFeel.findColour (backgroundColourId))
 {
     clear();
 }
@@ -291,16 +293,9 @@ void PaintRoutine::copySelectedToClipboard()
 
     XmlElement clip (clipboardXmlTag);
 
-    for (int i = 0; i < elements.size(); ++i)
-    {
-        PaintElement* const pe = elements.getUnchecked(i);
-
+    for (auto* pe : elements)
         if (selectedElements.isSelected (pe))
-        {
-            XmlElement* const e = pe->createXml();
-            clip.addChildElement (e);
-        }
-    }
+            clip.addChildElement (pe->createXml());
 
     SystemClipboard::copyTextToClipboard (clip.createDocument ("", false, false));
 }
@@ -323,8 +318,8 @@ void PaintRoutine::paste()
 
 void PaintRoutine::deleteSelected()
 {
-    const SelectedItemSet <PaintElement*> temp1 (selectedElements);
-    const SelectedItemSet <PathPoint*> temp2 (selectedPoints);
+    const SelectedItemSet<PaintElement*> temp1 (selectedElements);
+    const SelectedItemSet<PathPoint*> temp2 (selectedPoints);
 
     if (temp2.getNumSelected() > 0)
     {
@@ -367,7 +362,7 @@ void PaintRoutine::selectAll()
 
 void PaintRoutine::selectedToFront()
 {
-    const SelectedItemSet <PaintElement*> temp (selectedElements);
+    const SelectedItemSet<PaintElement*> temp (selectedElements);
 
     for (int i = temp.getNumSelected(); --i >= 0;)
         elementToFront (temp.getSelectedItem(i), true);
@@ -375,12 +370,74 @@ void PaintRoutine::selectedToFront()
 
 void PaintRoutine::selectedToBack()
 {
-    const SelectedItemSet <PaintElement*> temp (selectedElements);
+    const SelectedItemSet<PaintElement*> temp (selectedElements);
 
     for (int i = 0; i < temp.getNumSelected(); ++i)
         elementToBack (temp.getSelectedItem(i), true);
 }
 
+void PaintRoutine::alignTop()
+{
+    if (selectedElements.getNumSelected() > 1)
+    {
+        auto* main = selectedElements.getSelectedItem (0);
+        auto yPos = main->getY();
+
+        for (auto* other : selectedElements)
+        {
+            if (other != main)
+                other->setPaintElementBoundsAndProperties (other, other->getBounds().withPosition (other->getX(),
+                                                                                                   yPos), main, true);
+        }
+    }
+}
+void PaintRoutine::alignRight()
+{
+    if (selectedElements.getNumSelected() > 1)
+    {
+        auto* main = selectedElements.getSelectedItem (0);
+        auto rightPos = main->getRight();
+
+        for (auto* other : selectedElements)
+        {
+            if (other != main)
+                other->setPaintElementBoundsAndProperties (other, other->getBounds().withPosition (rightPos - other->getWidth(),
+                                                                                                   other->getY()), main, true);
+        }
+    }
+}
+
+void PaintRoutine::alignBottom()
+{
+    if (selectedElements.getNumSelected() > 1)
+    {
+        auto* main = selectedElements.getSelectedItem (0);
+        auto bottomPos = main->getBottom();
+
+        for (auto* other : selectedElements)
+        {
+            if (other != main)
+                other->setPaintElementBoundsAndProperties (other, other->getBounds().withPosition (other->getX(),
+                                                                                                   bottomPos - other->getHeight()), main, true);
+        }
+    }
+}
+void PaintRoutine::alignLeft()
+{
+    if (selectedElements.getNumSelected() > 1)
+    {
+        auto* main = selectedElements.getSelectedItem (0);
+        auto xPos = main->getX();
+
+        for (auto* other : selectedElements)
+        {
+            if (other != main)
+                other->setPaintElementBoundsAndProperties (other, other->getBounds().withPosition (xPos,
+                                                                                                   other->getY()), main, true);
+        }
+    }
+}
+
 void PaintRoutine::groupSelected()
 {
     PaintElementGroup::groupSelected (this);
@@ -397,11 +454,9 @@ void PaintRoutine::ungroupSelected()
 
 void PaintRoutine::bringLostItemsBackOnScreen (const Rectangle<int>& parentArea)
 {
-    for (int i = 0; i < elements.size(); ++i)
+    for (auto* c : elements)
     {
-        PaintElement* const c = elements[i];
-
-        Rectangle<int> r (c->getCurrentBounds (parentArea));
+        auto r = c->getCurrentBounds (parentArea);
 
         if (! r.intersects (parentArea))
         {
@@ -413,11 +468,9 @@ void PaintRoutine::bringLostItemsBackOnScreen (const Rectangle<int>& parentArea)
 
 void PaintRoutine::startDragging (const Rectangle<int>& parentArea)
 {
-    for (int i = 0; i < elements.size(); ++i)
+    for (auto* c : elements)
     {
-        PaintElement* const c = elements[i];
-
-        Rectangle<int> r (c->getCurrentBounds (parentArea));
+        auto r = c->getCurrentBounds (parentArea);
 
         c->getProperties().set ("xDragStart", r.getX());
         c->getProperties().set ("yDragStart", r.getY());
@@ -472,8 +525,9 @@ void PaintRoutine::fillWithBackground (Graphics& g, const bool drawOpaqueBackgro
 {
     if ((! backgroundColour.isOpaque()) && drawOpaqueBackground)
     {
-        g.fillCheckerBoard (Rectangle<int> (0, 0, g.getClipBounds().getRight(), g.getClipBounds().getBottom()),
-                            50, 50,
+        g.fillCheckerBoard (Rectangle<float> ((float) g.getClipBounds().getRight(),
+                                              (float) g.getClipBounds().getBottom()),
+                            50.0f, 50.0f,
                             Colour (0xffdddddd).overlaidWith (backgroundColour),
                             Colour (0xffffffff).overlaidWith (backgroundColour));
     }
@@ -488,8 +542,8 @@ void PaintRoutine::drawElements (Graphics& g, const Rectangle<int>& relativeTo)
     Component temp;
     temp.setBounds (relativeTo);
 
-    for (int i = 0; i < elements.size(); ++i)
-        elements.getUnchecked (i)->draw (g, getDocument()->getComponentLayout(), relativeTo);
+    for (auto* e : elements)
+        e->draw (g, getDocument()->getComponentLayout(), relativeTo);
 }
 
 //==============================================================================
@@ -499,17 +553,16 @@ void PaintRoutine::dropImageAt (const File& f, int x, int y)
 
     if (d != nullptr)
     {
-        Rectangle<float> bounds (d->getDrawableBounds());
-        d = nullptr;
+        auto bounds = d->getDrawableBounds();
+        d.reset();
 
-        PaintElement* newElement
-            = addNewElement (ObjectTypes::createNewImageElement (this), -1, true);
+        auto* newElement = addNewElement (ObjectTypes::createNewImageElement (this), -1, true);
 
-        if (PaintElementImage* pei = dynamic_cast<PaintElementImage*> (newElement))
+        if (auto* pei = dynamic_cast<PaintElementImage*> (newElement))
         {
             String resourceName (getDocument()->getResources().findUniqueName (f.getFileName()));
 
-            if (const BinaryResources::BinaryResource* existingResource = getDocument()->getResources().getResourceForFile (f))
+            if (auto* existingResource = getDocument()->getResources().getResourceForFile (f))
             {
                 resourceName = existingResource->name;
             }
@@ -544,12 +597,11 @@ const char* PaintRoutine::xmlTagName = "BACKGROUND";
 
 XmlElement* PaintRoutine::createXml() const
 {
-    XmlElement* const xml = new XmlElement (xmlTagName);
-
+    auto* xml = new XmlElement (xmlTagName);
     xml->setAttribute ("backgroundColour", backgroundColour.toString());
 
-    for (int i = 0; i < elements.size(); ++i)
-        xml->addChildElement (elements.getUnchecked (i)->createXml());
+    for (auto* e : elements)
+        xml->addChildElement (e->createXml());
 
     return xml;
 }
@@ -563,7 +615,7 @@ bool PaintRoutine::loadFromXml (const XmlElement& xml)
         clear();
 
         forEachXmlChildElement (xml, e)
-            if (PaintElement* const newElement = ObjectTypes::createElementForXml (e, this))
+            if (auto* newElement = ObjectTypes::createElementForXml (e, this))
                 elements.add (newElement);
 
         return true;
@@ -577,6 +629,12 @@ void PaintRoutine::fillInGeneratedCode (GeneratedCode& code, String& paintMethod
     if (! backgroundColour.isTransparent())
         paintMethodCode << "g.fillAll (" << CodeHelpers::colourToCode (backgroundColour) << ");\n\n";
 
-    for (int i = 0; i < elements.size(); ++i)
-        elements[i]->fillInGeneratedCode (code, paintMethodCode);
+    for (auto* e : elements)
+        e->fillInGeneratedCode (code, paintMethodCode);
+}
+
+void PaintRoutine::applyCustomPaintSnippets (StringArray& snippets)
+{
+    for (auto* e : elements)
+        e->applyCustomPaintSnippets (snippets);
 }
