@@ -2,34 +2,34 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#include "../jucer_Headers.h"
+#include "../Application/jucer_Headers.h"
 #include "jucer_Module.h"
-#include "jucer_ProjectType.h"
-#include "../Project Saving/jucer_ProjectExporter.h"
-#include "../Project Saving/jucer_ProjectSaver.h"
-#include "../Project Saving/jucer_ProjectExport_XCode.h"
-
+#include "../ProjectSaving/jucer_ProjectSaver.h"
+#include "../ProjectSaving/jucer_ProjectExport_Xcode.h"
 
+//==============================================================================
 static String trimCommentCharsFromStartOfLine (const String& line)
 {
     return line.trimStart().trimCharactersAtStart ("*/").trimStart();
@@ -40,16 +40,16 @@ static var parseModuleDesc (const StringArray& lines)
     DynamicObject* o = new DynamicObject();
     var result (o);
 
-    for (int i = 0; i < lines.size(); ++i)
+    for (auto line : lines)
     {
-        String line = trimCommentCharsFromStartOfLine (lines[i]);
+        line = trimCommentCharsFromStartOfLine (line);
 
-        int colon = line.indexOfChar (':');
+        auto colon = line.indexOfChar (':');
 
         if (colon >= 0)
         {
-            String key = line.substring (0, colon).trim();
-            String value = line.substring (colon + 1).trim();
+            auto key = line.substring (0, colon).trim();
+            auto value = line.substring (colon + 1).trim();
 
             o->setProperty (key, value);
         }
@@ -81,7 +81,7 @@ static var parseModuleDesc (const File& header)
         }
     }
 
-    return var();
+    return {};
 }
 
 ModuleDescription::ModuleDescription (const File& folder)
@@ -92,22 +92,25 @@ ModuleDescription::ModuleDescription (const File& folder)
 
 File ModuleDescription::getHeader() const
 {
-    const char* extensions[] = { ".h", ".hpp", ".hxx" };
-
-    for (int i = 0; i < numElementsInArray (extensions); ++i)
+    if (moduleFolder != File())
     {
-        File header (moduleFolder.getChildFile (moduleFolder.getFileName() + extensions[i]));
+        const char* extensions[] = { ".h", ".hpp", ".hxx" };
 
-        if (header.existsAsFile())
-            return header;
+        for (auto e : extensions)
+        {
+            auto header = moduleFolder.getChildFile (moduleFolder.getFileName() + e);
+
+            if (header.existsAsFile())
+                return header;
+        }
     }
 
-    return File();
+    return {};
 }
 
 StringArray ModuleDescription::getDependencies() const
 {
-    StringArray deps = StringArray::fromTokens (moduleInfo ["dependencies"].toString(), " \t;,", "\"'");
+    auto deps = StringArray::fromTokens (moduleInfo ["dependencies"].toString(), " \t;,", "\"'");
     deps.trim();
     deps.removeEmptyStrings();
     return deps;
@@ -132,37 +135,27 @@ ModuleList& ModuleList::operator= (const ModuleList& other)
 
 const ModuleDescription* ModuleList::getModuleWithID (const String& moduleID) const
 {
-    for (int i = 0; i < modules.size(); ++i)
-    {
-        ModuleDescription* m = modules.getUnchecked(i);
-
+    for (auto* m : modules)
         if (m->getID() == moduleID)
             return m;
-    }
 
     return nullptr;
 }
 
-struct ModuleSorter
-{
-    static int compareElements (const ModuleDescription* m1, const ModuleDescription* m2)
-    {
-        return m1->getID().compareIgnoreCase (m2->getID());
-    }
-};
-
 void ModuleList::sort()
 {
-    ModuleSorter sorter;
-    modules.sort (sorter);
+    std::sort (modules.begin(), modules.end(), [] (const ModuleDescription* m1, const ModuleDescription* m2)
+    {
+        return m1->getID().compareIgnoreCase (m2->getID()) < 0;
+    });
 }
 
 StringArray ModuleList::getIDs() const
 {
     StringArray results;
 
-    for (int i = 0; i < modules.size(); ++i)
-        results.add (modules.getUnchecked(i)->getID());
+    for (auto* m : modules)
+        results.add (m->getID());
 
     results.sort (true);
     return results;
@@ -171,6 +164,7 @@ StringArray ModuleList::getIDs() const
 Result ModuleList::tryToAddModuleFromFolder (const File& path)
 {
     ModuleDescription m (path);
+
     if (m.isValid())
     {
         modules.add (new ModuleDescription (m));
@@ -184,7 +178,7 @@ Result ModuleList::addAllModulesInFolder (const File& path)
 {
     if (! tryToAddModuleFromFolder (path))
     {
-        const int subfolders = 2;
+        int subfolders = 2;
         return addAllModulesInSubfoldersRecursively (path, subfolders);
     }
 
@@ -197,7 +191,7 @@ Result ModuleList::addAllModulesInSubfoldersRecursively (const File& path, int d
     {
         for (DirectoryIterator iter (path, false, "*", File::findDirectories); iter.next();)
         {
-            const File& childPath = iter.getFile().getLinkedTarget();
+            auto childPath = iter.getFile().getLinkedTarget();
 
             if (! tryToAddModuleFromFolder (childPath))
                 addAllModulesInSubfoldersRecursively (childPath, depth - 1);
@@ -207,21 +201,71 @@ Result ModuleList::addAllModulesInSubfoldersRecursively (const File& path, int d
     return Result::ok();
 }
 
-static Array<File> getAllPossibleModulePaths (Project& project)
+static File getModuleFolderFromPathIfItExists (const String& path, const String& moduleID, const Project& project)
+{
+    if (path.isNotEmpty())
+    {
+        auto moduleFolder = project.resolveFilename (path);
+
+        if (moduleFolder.exists())
+        {
+            if (ModuleDescription (moduleFolder).getID() == moduleID)
+                return moduleFolder;
+
+            auto f = moduleFolder.getChildFile (moduleID);
+
+            if (ModuleDescription (f).getID() == moduleID)
+                return f;
+        }
+    }
+
+    return {};
+}
+
+static File getPathToSpecifiedModule (Project& project, StringRef moduleID)
+{
+    auto& modules = project.getModules();
+
+    if (! modules.shouldUseGlobalPath (moduleID))
+    {
+        for (Project::ExporterIterator exporter (project); exporter.next();)
+        {
+            if (! exporter->mayCompileOnCurrentOS())
+                continue;
+
+            auto path = getModuleFolderFromPathIfItExists (exporter->getPathForModuleString (moduleID), moduleID, project);
+
+            if (path != File())
+                return path;
+        }
+    }
+
+    return {};
+}
+
+static Array<File> getAllPossibleModulePathsFromExporters (Project& project)
 {
     StringArray paths;
 
     for (Project::ExporterIterator exporter (project); exporter.next();)
     {
-        for (int i = 0; i < project.getModules().getNumModules(); ++i)
+        auto& modules = project.getModules();
+        auto n = modules.getNumModules();
+
+        for (int i = 0; i < n; ++i)
         {
-            const String path (exporter->getPathForModuleString (project.getModules().getModuleID (i)));
+            auto id = modules.getModuleID (i);
+
+            if (modules.shouldUseGlobalPath (id))
+                continue;
+
+            auto path = exporter->getPathForModuleString (id);
 
             if (path.isNotEmpty())
                 paths.addIfNotAlreadyThere (path);
         }
 
-        String oldPath (exporter->getLegacyModulePath());
+        auto oldPath = exporter->getLegacyModulePath();
 
         if (oldPath.isNotEmpty())
             paths.addIfNotAlreadyThere (oldPath);
@@ -229,13 +273,13 @@ static Array<File> getAllPossibleModulePaths (Project& project)
 
     Array<File> files;
 
-    for (int i = 0; i < paths.size(); ++i)
+    for (auto& path : paths)
     {
-        const File f (project.resolveFilename (paths[i]));
+        auto f = project.resolveFilename (path);
 
         if (f.isDirectory())
         {
-            files.add (f);
+            files.addIfNotAlreadyThere (f);
 
             if (f.getChildFile ("modules").isDirectory())
                 files.addIfNotAlreadyThere (f.getChildFile ("modules"));
@@ -245,16 +289,14 @@ static Array<File> getAllPossibleModulePaths (Project& project)
     return files;
 }
 
-Result ModuleList::scanAllKnownFolders (Project& project)
+Result ModuleList::scanProjectExporterModulePaths (Project& project)
 {
     modules.clear();
     Result result (Result::ok());
 
-    const Array<File> modulePaths (getAllPossibleModulePaths (project));
-
-    for (int i = 0; i < modulePaths.size(); ++i)
+    for (auto& m : getAllPossibleModulePathsFromExporters (project))
     {
-        result = addAllModulesInFolder (modulePaths.getReference(i));
+        result = addAllModulesInFolder (m);
 
         if (result.failed())
             break;
@@ -264,6 +306,36 @@ Result ModuleList::scanAllKnownFolders (Project& project)
     return result;
 }
 
+void ModuleList::scanGlobalJuceModulePath()
+{
+    modules.clear();
+
+    auto& settings = getAppSettings();
+
+    auto path = settings.getStoredPath (Ids::defaultJuceModulePath).toString();
+
+    if (path.isNotEmpty())
+        addAllModulesInFolder ({ path });
+
+    sort();
+}
+
+void ModuleList::scanGlobalUserModulePath()
+{
+    modules.clear();
+
+    auto paths = StringArray::fromTokens (getAppSettings().getStoredPath (Ids::defaultUserModulePath).toString(), ";", {});
+
+    for (auto p : paths)
+    {
+        auto f = File::createFileWithoutCheckingPath (p.trim());
+        if (f.exists())
+            addAllModulesInFolder (f);
+    }
+
+    sort();
+}
+
 //==============================================================================
 LibraryModule::LibraryModule (const ModuleDescription& d)
     : moduleInfo (d)
@@ -273,16 +345,16 @@ LibraryModule::LibraryModule (const ModuleDescription& d)
 //==============================================================================
 void LibraryModule::writeIncludes (ProjectSaver& projectSaver, OutputStream& out)
 {
-    Project& project = projectSaver.project;
-    EnabledModuleList& modules = project.getModules();
+    auto& project = projectSaver.project;
+    auto& modules = project.getModules();
 
-    const String id (getID());
+    auto id = getID();
 
     if (modules.shouldCopyModuleFilesLocally (id).getValue())
     {
-        const File juceModuleFolder (moduleInfo.getFolder());
+        auto juceModuleFolder = moduleInfo.getFolder();
 
-        const File localModuleFolder (project.getLocalModuleFolder (id));
+        auto localModuleFolder = project.getLocalModuleFolder (id);
         localModuleFolder.createDirectory();
         projectSaver.copyFolder (juceModuleFolder, localModuleFolder);
     }
@@ -295,7 +367,7 @@ void LibraryModule::writeIncludes (ProjectSaver& projectSaver, OutputStream& out
 //==============================================================================
 static void parseAndAddLibs (StringArray& libList, const String& libs)
 {
-    libList.addTokens (libs, ", ", StringRef());
+    libList.addTokens (libs, ", ", {});
     libList.trim();
     libList.sort (false);
     libList.removeDuplicates (false);
@@ -303,52 +375,66 @@ static void parseAndAddLibs (StringArray& libList, const String& libs)
 
 void LibraryModule::addSettingsForModuleToExporter (ProjectExporter& exporter, ProjectSaver& projectSaver) const
 {
-    Project& project = exporter.getProject();
+    auto& project = exporter.getProject();
 
-    RelativePath modulePath = exporter.getModuleFolderRelativeToProject (getID());
+    auto moduleRelativePath = exporter.getModuleFolderRelativeToProject (getID());
 
-    exporter.addToExtraSearchPaths (modulePath.getParentDirectory());
+    exporter.addToExtraSearchPaths (moduleRelativePath.getParentDirectory());
 
-    const String extraInternalSearchPaths (moduleInfo.getExtraSearchPaths().trim());
+    String libDirPlatform;
+    if (exporter.isLinux())
+        libDirPlatform = "Linux";
+    else if (exporter.isCodeBlocks() && exporter.isWindows())
+        libDirPlatform = "MinGW";
+    else
+        libDirPlatform = exporter.getTargetFolder().getFileName();
+
+    auto libSubdirPath = moduleRelativePath.toUnixStyle() + "/libs/" + libDirPlatform;
+    auto moduleLibDir = File (project.getProjectFolder().getFullPathName() + "/" + libSubdirPath);
+
+    if (moduleLibDir.exists())
+        exporter.addToModuleLibPaths ({ libSubdirPath, moduleRelativePath.getRoot() });
+
+    auto extraInternalSearchPaths = moduleInfo.getExtraSearchPaths().trim();
 
     if (extraInternalSearchPaths.isNotEmpty())
     {
-        StringArray paths;
-        paths.addTokens (extraInternalSearchPaths, true);
+        auto paths = StringArray::fromTokens (extraInternalSearchPaths, true);
 
-        for (int i = 0; i < paths.size(); ++i)
-            exporter.addToExtraSearchPaths (modulePath.getChildFile (paths.getReference(i)));
+        for (auto& path : paths)
+            exporter.addToExtraSearchPaths (moduleRelativePath.getChildFile (path.unquoted()));
     }
 
     {
-        const String extraDefs (moduleInfo.getPreprocessorDefs().trim());
+        auto extraDefs = moduleInfo.getPreprocessorDefs().trim();
 
         if (extraDefs.isNotEmpty())
-            exporter.getExporterPreprocessorDefs() = exporter.getExporterPreprocessorDefsString() + "\n" + extraDefs;
+            exporter.getExporterPreprocessorDefsValue() = exporter.getExporterPreprocessorDefsString() + "\n" + extraDefs;
     }
 
     {
         Array<File> compiled;
+        auto& modules = project.getModules();
+        auto id = getID();
 
-        const File localModuleFolder = project.getModules().shouldCopyModuleFilesLocally (getID()).getValue()
-                                          ? project.getLocalModuleFolder (getID())
-                                          : moduleInfo.getFolder();
+        auto localModuleFolder = modules.shouldCopyModuleFilesLocally (id).getValue() ? project.getLocalModuleFolder (id)
+                                                                                      : moduleInfo.getFolder();
 
         findAndAddCompiledUnits (exporter, &projectSaver, compiled);
 
-        if (project.getModules().shouldShowAllModuleFilesInProject (getID()).getValue())
+        if (modules.shouldShowAllModuleFilesInProject (id).getValue())
             addBrowseableCode (exporter, compiled, localModuleFolder);
     }
 
     if (exporter.isXcode())
     {
-        XCodeProjectExporter& xcodeExporter = dynamic_cast<XCodeProjectExporter&> (exporter);
+        auto& xcodeExporter = dynamic_cast<XcodeProjectExporter&> (exporter);
 
         if (project.isAUPluginHost())
             xcodeExporter.xcodeFrameworks.addTokens (xcodeExporter.isOSX() ? "AudioUnit CoreAudioKit" : "CoreAudioKit", false);
 
-        const String frameworks (moduleInfo.moduleInfo [xcodeExporter.isOSX() ? "OSXFrameworks" : "iOSFrameworks"].toString());
-        xcodeExporter.xcodeFrameworks.addTokens (frameworks, ", ", StringRef());
+        auto frameworks = moduleInfo.moduleInfo [xcodeExporter.isOSX() ? "OSXFrameworks" : "iOSFrameworks"].toString();
+        xcodeExporter.xcodeFrameworks.addTokens (frameworks, ", ", {});
 
         parseAndAddLibs (xcodeExporter.xcodeLibs, moduleInfo.moduleInfo [exporter.isOSX() ? "OSXLibs" : "iOSLibs"].toString());
     }
@@ -357,15 +443,22 @@ void LibraryModule::addSettingsForModuleToExporter (ProjectExporter& exporter, P
         parseAndAddLibs (exporter.linuxLibs, moduleInfo.moduleInfo ["linuxLibs"].toString());
         parseAndAddLibs (exporter.linuxPackages, moduleInfo.moduleInfo ["linuxPackages"].toString());
     }
-    else if (exporter.isCodeBlocks() && exporter.isWindows())
+    else if (exporter.isWindows())
     {
-        parseAndAddLibs (exporter.mingwLibs, moduleInfo.moduleInfo ["mingwLibs"].toString());
+        if (exporter.isCodeBlocks())
+            parseAndAddLibs (exporter.mingwLibs, moduleInfo.moduleInfo ["mingwLibs"].toString());
+        else
+            parseAndAddLibs (exporter.windowsLibs, moduleInfo.moduleInfo ["windowsLibs"].toString());
+    }
+    else if (exporter.isAndroid())
+    {
+        parseAndAddLibs (exporter.androidLibs, moduleInfo.moduleInfo ["androidLibs"].toString());
     }
 }
 
 void LibraryModule::getConfigFlags (Project& project, OwnedArray<Project::ConfigFlag>& flags) const
 {
-    const File header (moduleInfo.getHeader());
+    auto header = moduleInfo.getHeader();
     jassert (header.exists());
 
     StringArray lines;
@@ -373,7 +466,7 @@ void LibraryModule::getConfigFlags (Project& project, OwnedArray<Project::Config
 
     for (int i = 0; i < lines.size(); ++i)
     {
-        String line (lines[i].trim());
+        auto line = lines[i].trim();
 
         if (line.startsWith ("/**") && line.containsIgnoreCase ("Config:"))
         {
@@ -394,7 +487,20 @@ void LibraryModule::getConfigFlags (Project& project, OwnedArray<Project::Config
                 }
 
                 config->description = config->description.upToFirstOccurrenceOf ("*/", false, false);
-                config->value.referTo (project.getConfigFlag (config->symbol));
+                config->value = project.getConfigFlag (config->symbol);
+
+                i += 2;
+                if (lines[i].contains ("#define " + config->symbol))
+                {
+                    auto value = lines[i].fromFirstOccurrenceOf ("#define " + config->symbol, false, true).trim();
+                    config->value.setDefault (value == "0" ? false : true);
+                }
+
+                auto currentValue = config->value.get().toString();
+
+                if      (currentValue == "enabled")     config->value = true;
+                else if (currentValue == "disabled")    config->value = false;
+
                 flags.add (config.release());
             }
         }
@@ -412,7 +518,7 @@ struct FileSorter
 
 bool LibraryModule::CompileUnit::hasSuffix (const File& f, const char* suffix)
 {
-    String fileWithoutSuffix = f.getFileNameWithoutExtension() + String (".");
+    auto fileWithoutSuffix = f.getFileNameWithoutExtension() + ".";
 
     return fileWithoutSuffix.containsIgnoreCase (suffix + String ("."))
              || fileWithoutSuffix.containsIgnoreCase (suffix + String ("_"));
@@ -424,52 +530,53 @@ void LibraryModule::CompileUnit::writeInclude (MemoryOutputStream&) const
 
 bool LibraryModule::CompileUnit::isNeededForExporter (ProjectExporter& exporter) const
 {
-    Project& project = exporter.getProject();
-
     if ((hasSuffix (file, "_OSX")        && ! exporter.isOSX())
      || (hasSuffix (file, "_iOS")        && ! exporter.isiOS())
      || (hasSuffix (file, "_Windows")    && ! exporter.isWindows())
      || (hasSuffix (file, "_Linux")      && ! exporter.isLinux())
-     || (hasSuffix (file, "_Android")    && ! exporter.isAndroid())
-     || (hasSuffix (file, "_AU")         && ! (project.shouldBuildAU()  .getValue()       && exporter.supportsAU()))
-     || (hasSuffix (file, "_AUv3")       && ! (project.shouldBuildAUv3().getValue()       && exporter.supportsAUv3()))
-     || (hasSuffix (file, "_AAX")        && ! (project.shouldBuildAAX() .getValue()       && exporter.supportsAAX()))
-     || (hasSuffix (file, "_RTAS")       && ! (project.shouldBuildRTAS().getValue()       && exporter.supportsRTAS()))
-     || (hasSuffix (file, "_VST2")       && ! (project.shouldBuildVST() .getValue()       && exporter.supportsVST()))
-     || (hasSuffix (file, "_VST3")       && ! (project.shouldBuildVST3().getValue()       && exporter.supportsVST3()))
-     || (hasSuffix (file, "_Standalone") && ! (project.shouldBuildStandalone().getValue() && exporter.supportsStandalone())))
+     || (hasSuffix (file, "_Android")    && ! exporter.isAndroid()))
+        return false;
+
+    auto targetType = Project::getTargetTypeFromFilePath (file, false);
+
+    if (targetType != ProjectType::Target::unspecified && ! exporter.shouldBuildTargetType (targetType))
         return false;
 
     return exporter.usesMMFiles() ? isCompiledForObjC
                                   : isCompiledForNonObjC;
 }
 
-Array<LibraryModule::CompileUnit> LibraryModule::getAllCompileUnits() const
+String LibraryModule::CompileUnit::getFilenameForProxyFile() const
 {
-    Array<File> files;
-    getFolder().findChildFiles (files, File::findFiles, false);
+    return "include_" + file.getFileName();
+}
+
+Array<LibraryModule::CompileUnit> LibraryModule::getAllCompileUnits (ProjectType::Target::Type forTarget) const
+{
+    auto files = getFolder().findChildFiles (File::findFiles, false);
 
     FileSorter sorter;
     files.sort (sorter);
 
     Array<LibraryModule::CompileUnit> units;
 
-    for (int i = 0; i < files.size(); ++i)
+    for (auto& file : files)
     {
-        CompileUnit cu;
-        cu.file = files.getReference(i);
-
-        if (cu.file.getFileName().startsWithIgnoreCase (getID())
-              && cu.file.hasFileExtension (sourceFileExtensions))
+        if (file.getFileName().startsWithIgnoreCase (getID())
+              && file.hasFileExtension (sourceFileExtensions))
         {
-            units.add (cu);
+            if (forTarget == ProjectType::Target::unspecified
+             || forTarget == Project::getTargetTypeFromFilePath (file, true))
+            {
+                CompileUnit cu;
+                cu.file = file;
+                units.add (cu);
+            }
         }
     }
 
-    for (int i = 0; i < units.size(); ++i)
+    for (auto& cu : units)
     {
-        CompileUnit& cu = units.getReference(i);
-
         cu.isCompiledForObjC = true;
         cu.isCompiledForNonObjC = ! cu.file.hasFileExtension ("mm;m");
 
@@ -485,17 +592,15 @@ Array<LibraryModule::CompileUnit> LibraryModule::getAllCompileUnits() const
 
 void LibraryModule::findAndAddCompiledUnits (ProjectExporter& exporter,
                                              ProjectSaver* projectSaver,
-                                             Array<File>& result) const
+                                             Array<File>& result,
+                                             ProjectType::Target::Type forTarget) const
 {
-    Array<CompileUnit> units = getAllCompileUnits();
-
-    for (int i = 0; i < units.size(); ++i)
+    for (auto& cu : getAllCompileUnits (forTarget))
     {
-        const CompileUnit& cu = units.getReference(i);
-
         if (cu.isNeededForExporter (exporter))
         {
-            File localFile = exporter.getProject().getGeneratedCodeFolder().getChildFile (cu.file.getFileName());
+            auto localFile = exporter.getProject().getGeneratedCodeFolder()
+                                                  .getChildFile (cu.getFilenameForProxyFile());
             result.add (localFile);
 
             if (projectSaver != nullptr)
@@ -506,14 +611,14 @@ void LibraryModule::findAndAddCompiledUnits (ProjectExporter& exporter,
 
 static void addFileWithGroups (Project::Item& group, const RelativePath& file, const String& path)
 {
-    const int slash = path.indexOfChar (File::separator);
+    auto slash = path.indexOfChar (File::getSeparatorChar());
 
     if (slash >= 0)
     {
-        const String topLevelGroup (path.substring (0, slash));
-        const String remainingPath (path.substring (slash + 1));
+        auto topLevelGroup = path.substring (0, slash);
+        auto remainingPath = path.substring (slash + 1);
 
-        Project::Item newGroup (group.getOrCreateSubGroup (topLevelGroup));
+        auto newGroup = group.getOrCreateSubGroup (topLevelGroup);
         addFileWithGroups (newGroup, file, remainingPath);
     }
     else
@@ -540,29 +645,31 @@ void LibraryModule::findBrowseableFiles (const File& folder, Array<File>& filesF
 
 void LibraryModule::addBrowseableCode (ProjectExporter& exporter, const Array<File>& compiled, const File& localModuleFolder) const
 {
-    if (sourceFiles.size() == 0)
+    if (sourceFiles.isEmpty())
         findBrowseableFiles (localModuleFolder, sourceFiles);
 
-    Project::Item sourceGroup (Project::Item::createGroup (exporter.getProject(), getID(), "__mainsourcegroup" + getID(), false));
+    auto sourceGroup = Project::Item::createGroup (exporter.getProject(), getID(), "__mainsourcegroup" + getID(), false);
 
-    const RelativePath moduleFromProject (exporter.getModuleFolderRelativeToProject (getID()));
+    auto moduleFromProject = exporter.getModuleFolderRelativeToProject (getID());
 
-    for (int i = 0; i < sourceFiles.size(); ++i)
+    auto moduleHeader = moduleInfo.getHeader();
+
+    for (auto& sourceFile : sourceFiles)
     {
-        const String pathWithinModule (FileHelpers::getRelativePathFrom (sourceFiles.getReference(i), localModuleFolder));
+        auto pathWithinModule = FileHelpers::getRelativePathFrom (sourceFile, localModuleFolder);
 
         // (Note: in exporters like MSVC we have to avoid adding the same file twice, even if one of those instances
         // is flagged as being excluded from the build, because this overrides the other and it fails to compile)
-        if (exporter.canCopeWithDuplicateFiles() || ! compiled.contains (sourceFiles.getReference(i)))
+        if ((exporter.canCopeWithDuplicateFiles() || ! compiled.contains (sourceFile)) && sourceFile != moduleHeader)
             addFileWithGroups (sourceGroup,
                                moduleFromProject.getChildFile (pathWithinModule),
                                pathWithinModule);
     }
 
     sourceGroup.sortAlphabetically (true, true);
-    sourceGroup.addFileAtIndex (moduleInfo.getHeader(), -1, false);
+    sourceGroup.addFileAtIndex (moduleHeader, -1, false);
 
-    exporter.getModulesGroup().state.addChild (sourceGroup.state.createCopy(), -1, nullptr);
+    exporter.getModulesGroup().state.appendChild (sourceGroup.state.createCopy(), nullptr);
 }
 
 
@@ -579,11 +686,7 @@ ModuleDescription EnabledModuleList::getModuleInfo (const String& moduleID)
 
 bool EnabledModuleList::isModuleEnabled (const String& moduleID) const
 {
-    for (int i = 0; i < state.getNumChildren(); ++i)
-        if (state.getChild(i) [Ids::ID] == moduleID)
-            return true;
-
-    return false;
+    return state.getChildWithProperty (Ids::ID, moduleID).isValid();
 }
 
 bool EnabledModuleList::isAudioPluginModuleMissing() const
@@ -592,49 +695,68 @@ bool EnabledModuleList::isAudioPluginModuleMissing() const
             && ! isModuleEnabled ("juce_audio_plugin_client");
 }
 
+bool EnabledModuleList::shouldUseGlobalPath (const String& moduleID) const
+{
+    return static_cast<bool> (state.getChildWithProperty (Ids::ID, moduleID)
+                                   .getProperty (Ids::useGlobalPath));
+}
+
+Value EnabledModuleList::getShouldUseGlobalPathValue (const String& moduleID) const
+{
+    return state.getChildWithProperty (Ids::ID, moduleID)
+                .getPropertyAsValue (Ids::useGlobalPath, getUndoManager());
+}
+
 Value EnabledModuleList::shouldShowAllModuleFilesInProject (const String& moduleID)
 {
     return state.getChildWithProperty (Ids::ID, moduleID)
                 .getPropertyAsValue (Ids::showAllCode, getUndoManager());
 }
 
-File EnabledModuleList::findLocalModuleFolder (const String& moduleID, bool useExportersForOtherOSes)
+File EnabledModuleList::findUserModuleFolder (const String& possiblePaths, const String& moduleID)
 {
-    for (Project::ExporterIterator exporter (project); exporter.next();)
+    auto paths = StringArray::fromTokens (possiblePaths, ";", {});
+
+    for (auto p : paths)
     {
-        if (useExportersForOtherOSes || exporter->mayCompileOnCurrentOS())
+        auto f = File::createFileWithoutCheckingPath (p.trim());
+        if (f.exists())
         {
-            const String path (exporter->getPathForModuleString (moduleID));
-
-            if (path.isNotEmpty())
-            {
-                const File moduleFolder (project.resolveFilename (path));
-
-                if (moduleFolder.exists())
-                {
-                    if (ModuleDescription (moduleFolder).isValid())
-                        return moduleFolder;
-
-                    File f = moduleFolder.getChildFile (moduleID);
-
-                    if (ModuleDescription (f).isValid())
-                        return f;
-                }
-            }
+            auto moduleFolder = getModuleFolderFromPathIfItExists (f.getFullPathName(), moduleID, project);
+            if (moduleFolder != File())
+                return moduleFolder;
         }
     }
 
-    return File();
+    return {};
 }
 
 File EnabledModuleList::getModuleFolder (const String& moduleID)
 {
-    File f = findLocalModuleFolder (moduleID, false);
+    if (shouldUseGlobalPath (moduleID))
+    {
+        if (isJuceModule (moduleID))
+            return getModuleFolderFromPathIfItExists (getAppSettings().getStoredPath (Ids::defaultJuceModulePath).toString(), moduleID, project);
 
-    if (f == File())
-        f = findLocalModuleFolder (moduleID, true);
+        return findUserModuleFolder (getAppSettings().getStoredPath (Ids::defaultUserModulePath).toString(), moduleID);
+    }
 
-    return f;
+    {
+        auto path = getPathToSpecifiedModule (project, moduleID);
+
+        if (path != File())
+            return path;
+    }
+
+    auto paths = getAllPossibleModulePathsFromExporters (project);
+    for (auto p : paths)
+    {
+        auto f = getModuleFolderFromPathIfItExists (p.getFullPathName(), moduleID, project);
+        if (f != File())
+            return f;
+    }
+
+    return {};
 }
 
 struct ModuleTreeSorter
@@ -657,24 +779,25 @@ Value EnabledModuleList::shouldCopyModuleFilesLocally (const String& moduleID) c
                 .getPropertyAsValue (Ids::useLocalCopy, getUndoManager());
 }
 
-void EnabledModuleList::addModule (const File& moduleFolder, bool copyLocally)
+void EnabledModuleList::addModule (const File& moduleFolder, bool copyLocally, bool useGlobalPath)
 {
     ModuleDescription info (moduleFolder);
 
     if (info.isValid())
     {
-        const String moduleID (info.getID());
+        auto moduleID = info.getID();
 
         if (! isModuleEnabled (moduleID))
         {
             ValueTree module (Ids::MODULE);
             module.setProperty (Ids::ID, moduleID, nullptr);
 
-            state.addChild (module, -1, getUndoManager());
+            state.appendChild (module, getUndoManager());
             sortAlphabetically();
 
             shouldShowAllModuleFilesInProject (moduleID) = true;
             shouldCopyModuleFilesLocally (moduleID) = copyLocally;
+            getShouldUseGlobalPathValue (moduleID) = useGlobalPath;
 
             RelativePath path (moduleFolder.getParentDirectory(),
                                project.getProjectFolder(), RelativePath::projectFolder);
@@ -687,7 +810,7 @@ void EnabledModuleList::addModule (const File& moduleFolder, bool copyLocally)
 
 void EnabledModuleList::removeModule (String moduleID) // must be pass-by-value, and not a const ref!
 {
-    for (int i = state.getNumChildren(); --i >= 0;)
+    for (auto i = state.getNumChildren(); --i >= 0;)
         if (state.getChild(i) [Ids::ID] == moduleID)
             state.removeChild (i, getUndoManager());
 
@@ -706,14 +829,14 @@ StringArray EnabledModuleList::getAllModules() const
     StringArray moduleIDs;
 
     for (int i = 0; i < getNumModules(); ++i)
-        moduleIDs.add (getModuleID(i));
+        moduleIDs.add (getModuleID (i));
 
     return moduleIDs;
 }
 
 static void getDependencies (Project& project, const String& moduleID, StringArray& dependencies)
 {
-    ModuleDescription info (project.getModules().getModuleInfo (moduleID));
+    auto info = project.getModules().getModuleInfo (moduleID);
 
     for (auto uid : info.getDependencies())
     {
@@ -737,11 +860,38 @@ StringArray EnabledModuleList::getExtraDependenciesNeeded (const String& moduleI
     return extraDepsNeeded;
 }
 
+bool EnabledModuleList::doesModuleHaveHigherCppStandardThanProject (const String& moduleID)
+{
+    auto projectCppStandard = project.getCppStandardString();
+
+    if (projectCppStandard == "latest")
+        return false;
+
+    auto moduleCppStandard = getModuleInfo (moduleID).getMinimumCppStandard();
+
+    return (moduleCppStandard.getIntValue() > projectCppStandard.getIntValue());
+}
+
+bool EnabledModuleList::areMostModulesUsingGlobalPath() const
+{
+    int numYes = 0, numNo = 0;
+
+    for (auto i = getNumModules(); --i >= 0;)
+    {
+        if (shouldUseGlobalPath (getModuleID (i)))
+            ++numYes;
+        else
+            ++numNo;
+    }
+
+    return numYes > numNo;
+}
+
 bool EnabledModuleList::areMostModulesCopiedLocally() const
 {
     int numYes = 0, numNo = 0;
 
-    for (int i = getNumModules(); --i >= 0;)
+    for (auto i = getNumModules(); --i >= 0;)
     {
         if (shouldCopyModuleFilesLocally (getModuleID (i)).getValue())
             ++numYes;
@@ -754,18 +904,34 @@ bool EnabledModuleList::areMostModulesCopiedLocally() const
 
 void EnabledModuleList::setLocalCopyModeForAllModules (bool copyLocally)
 {
-    for (int i = getNumModules(); --i >= 0;)
+    for (auto i = getNumModules(); --i >= 0;)
         shouldCopyModuleFilesLocally (project.getModules().getModuleID (i)) = copyLocally;
 }
 
+File EnabledModuleList::findGlobalModulesFolder()
+{
+    auto& settings = getAppSettings();
+    auto path = settings.getStoredPath (Ids::defaultJuceModulePath).toString();
+
+    if (settings.isGlobalPathValid ({}, Ids::defaultJuceModulePath, path))
+        return { path };
+
+    return {};
+}
+
 File EnabledModuleList::findDefaultModulesFolder (Project& project)
 {
-    ModuleList available;
-    available.scanAllKnownFolders (project);
+    auto globalPath = findGlobalModulesFolder();
 
-    for (int i = available.modules.size(); --i >= 0;)
+    if (globalPath != File())
+        return globalPath;
+
+    ModuleList available;
+    available.scanProjectExporterModulePaths (project);
+
+    for (auto i = available.modules.size(); --i >= 0;)
     {
-        File f (available.modules.getUnchecked(i)->getFolder());
+        auto f = available.modules.getUnchecked(i)->getFolder();
 
         if (f.isDirectory())
             return f.getParentDirectory();
@@ -774,31 +940,75 @@ File EnabledModuleList::findDefaultModulesFolder (Project& project)
     return File::getCurrentWorkingDirectory();
 }
 
+bool EnabledModuleList::isJuceModule (const String& moduleID)
+{
+    static StringArray juceModuleIds =
+    {
+        "juce_analytics",
+        "juce_audio_basics",
+        "juce_audio_devices",
+        "juce_audio_formats",
+        "juce_audio_plugin_client",
+        "juce_audio_processors",
+        "juce_audio_utils",
+        "juce_blocks_basics",
+        "juce_box2d",
+        "juce_core",
+        "juce_cryptography",
+        "juce_data_structures",
+        "juce_dsp",
+        "juce_events",
+        "juce_graphics",
+        "juce_gui_basics",
+        "juce_gui_extra",
+        "juce_opengl",
+        "juce_osc",
+        "juce_product_unlocking",
+        "juce_video"
+    };
+
+    return juceModuleIds.contains (moduleID);
+}
+
 void EnabledModuleList::addModuleFromUserSelectedFile()
 {
-    static File lastLocation (findDefaultModulesFolder (project));
+    static auto lastLocation = findDefaultModulesFolder (project);
 
-    FileChooser fc ("Select a module to add...", lastLocation, String(), false);
+    FileChooser fc ("Select a module to add...", lastLocation, {});
 
     if (fc.browseForDirectory())
     {
         lastLocation = fc.getResult();
-        addModuleOfferingToCopy (lastLocation);
+        addModuleOfferingToCopy (lastLocation, true);
     }
 }
 
 void EnabledModuleList::addModuleInteractive (const String& moduleID)
 {
     ModuleList list;
-    list.scanAllKnownFolders (project);
 
-    if (const ModuleDescription* info = list.getModuleWithID (moduleID))
-        addModule (info->moduleFolder, areMostModulesCopiedLocally());
+    list.scanGlobalJuceModulePath();
+    if (auto* info = list.getModuleWithID (moduleID))
+    {
+        addModule (info->moduleFolder, areMostModulesCopiedLocally(), areMostModulesUsingGlobalPath());
+        return;
+    }
+
+    list.scanGlobalUserModulePath();
+    if (auto* info = list.getModuleWithID (moduleID))
+    {
+        addModule (info->moduleFolder, areMostModulesCopiedLocally(), areMostModulesUsingGlobalPath());
+        return;
+    }
+
+    list.scanProjectExporterModulePaths (project);
+    if (auto* info = list.getModuleWithID (moduleID))
+        addModule (info->moduleFolder, areMostModulesCopiedLocally(), false);
     else
         addModuleFromUserSelectedFile();
 }
 
-void EnabledModuleList::addModuleOfferingToCopy (const File& f)
+void EnabledModuleList::addModuleOfferingToCopy (const File& f, bool isFromUserSpecifiedFolder)
 {
     ModuleDescription m (f);
 
@@ -816,7 +1026,8 @@ void EnabledModuleList::addModuleOfferingToCopy (const File& f)
         return;
     }
 
-    addModule (m.moduleFolder, areMostModulesCopiedLocally());
+    addModule (m.moduleFolder, areMostModulesCopiedLocally(), isFromUserSpecifiedFolder ? false
+                                                                                        : areMostModulesUsingGlobalPath());
 }
 
 bool isJuceFolder (const File& f)
