@@ -2,31 +2,33 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCER_PROJECT_H_INCLUDED
-#define JUCER_PROJECT_H_INCLUDED
+#pragma once
+
+#include "jucer_ProjectType.h"
 
 class ProjectExporter;
-class ProjectType;
 class LibraryModule;
 class EnabledModuleList;
 
@@ -52,12 +54,19 @@ public:
     void setTitle (const String& newTitle);
 
     //==============================================================================
-    File getProjectFolder() const                       { return getFile().getParentDirectory(); }
-    ValueTree getProjectRoot() const                    { return projectRoot; }
-    String getTitle() const;
-    Value getProjectNameValue()                         { return getMainGroup().getNameValue(); }
-    String getProjectFilenameRoot()                     { return File::createLegalFileName (getDocumentTitle()); }
-    String getProjectUID() const                        { return projectRoot [Ids::ID]; }
+    File getProjectFolder() const                               { return getFile().getParentDirectory(); }
+    File getGeneratedCodeFolder() const                         { return getFile().getSiblingFile ("JuceLibraryCode"); }
+    File getSourceFilesFolder() const                           { return getProjectFolder().getChildFile ("Source"); }
+    File getLocalModulesFolder() const                          { return getGeneratedCodeFolder().getChildFile ("modules"); }
+    File getLocalModuleFolder (const String& moduleID) const    { return getLocalModulesFolder().getChildFile (moduleID); }
+    File getAppIncludeFile() const                              { return getGeneratedCodeFolder().getChildFile (getJuceSourceHFilename()); }
+
+    File getBinaryDataCppFile (int index) const;
+    File getBinaryDataHeaderFile() const                        { return getBinaryDataCppFile (0).withFileExtension (".h"); }
+
+    String getAppConfigFilename() const                         { return "AppConfig.h"; }
+    String getJuceSourceFilenameRoot() const                    { return "JuceLibraryCode"; }
+    String getJuceSourceHFilename() const                       { return "JuceHeader.h"; }
 
     //==============================================================================
     template <class FileType>
@@ -74,89 +83,95 @@ public:
     void createPropertyEditors (PropertyListBuilder&);
 
     //==============================================================================
-    // project types
-    const ProjectType& getProjectType() const;
-    Value getProjectTypeValue()                         { return getProjectValue (Ids::projectType); }
-    String getProjectTypeString() const                 { return projectRoot [Ids::projectType]; }
+    ValueTree getProjectRoot() const                     { return projectRoot; }
+    Value getProjectValue (const Identifier& name)       { return projectRoot.getPropertyAsValue (name, getUndoManagerFor (projectRoot)); }
+    var   getProjectVar   (const Identifier& name) const { return projectRoot.getProperty        (name); }
 
-    Value getVersionValue()                             { return getProjectValue (Ids::version); }
-    String getVersionString() const                     { return projectRoot [Ids::version]; }
+    const ProjectType& getProjectType() const;
+    String getProjectTypeString() const                  { return projectTypeValue.get(); }
+    void setProjectType (const String& newProjectType)   { projectTypeValue = newProjectType; }
+
+    String getProjectNameString() const                  { return projectNameValue.get(); }
+    String getProjectFilenameRootString()                { return File::createLegalFileName (getDocumentTitle()); }
+    String getProjectUIDString() const                   { return projectUIDValue.get(); }
+
+    String getVersionString() const                      { return versionValue.get(); }
     String getVersionAsHex() const;
     int getVersionAsHexInteger() const;
+    void setProjectVersion (const String& newVersion)    { versionValue = newVersion; }
 
-    Value getBundleIdentifier()                         { return getProjectValue (Ids::bundleIdentifier); }
-    String getDefaultBundleIdentifier()                 { return "com.yourcompany." + CodeHelpers::makeValidIdentifier (getTitle(), false, true, false); }
+    String getBundleIdentifierString() const             { return bundleIdentifierValue.get(); }
+    String getDefaultBundleIdentifierString()            { return "com.yourcompany." + CodeHelpers::makeValidIdentifier (getProjectNameString(), false, true, false); }
+    String getDefaultAAXIdentifierString()               { return getDefaultBundleIdentifierString(); }
 
-    Value getAAXIdentifier()                            { return getProjectValue (Ids::aaxIdentifier); }
-    String getDefaultAAXIdentifier()                    { return getDefaultBundleIdentifier(); }
+    String getCompanyNameString() const                  { return companyNameValue.get(); }
+    String getCompanyCopyrightString() const             { return companyCopyrightValue.get(); }
+    String getCompanyWebsiteString() const               { return companyWebsiteValue.get(); }
+    String getCompanyEmailString() const                 { return companyEmailValue.get(); }
 
-    Value getCompanyName()                              { return getProjectValue (Ids::companyName); }
-    Value getCompanyWebsite()                           { return getProjectValue (Ids::companyWebsite); }
-    Value getCompanyEmail()                             { return getProjectValue (Ids::companyEmail); }
+    String getHeaderSearchPathsString() const            { return headerSearchPathsValue.get(); }
+
+    StringPairArray getPreprocessorDefs() const          { return parsedPreprocessorDefs; }
+
+    int getMaxBinaryFileSize() const                     { return maxBinaryFileSizeValue.get(); }
+    bool shouldIncludeBinaryInAppConfig() const          { return includeBinaryDataInAppConfigValue.get(); }
+    String getBinaryDataNamespaceString() const          { return binaryDataNamespaceValue.get(); }
+
+    bool shouldDisplaySplashScreen() const               { return displaySplashScreenValue.get(); }
+    bool shouldReportAppUsage() const                    { return reportAppUsageValue.get(); }
+    String getSplashScreenColourString() const           { return splashScreenColourValue.get(); }
+
+    String getCppStandardString() const                  { return cppStandardValue.get(); }
 
     //==============================================================================
-    Value getProjectValue (const Identifier& name)      { return projectRoot.getPropertyAsValue (name, getUndoManagerFor (projectRoot)); }
-
-    Value getProjectPreprocessorDefs()                  { return getProjectValue (Ids::defines); }
-    StringPairArray getPreprocessorDefs() const;
-
-    Value getProjectUserNotes()                         { return getProjectValue (Ids::userNotes); }
+    bool shouldBuildVST() const                 { return buildVSTValue.get(); }
+    bool shouldBuildVST3() const                { return buildVST3Value.get(); }
+    bool shouldBuildAU() const                  { return buildAUValue.get(); }
+    bool shouldBuildAUv3() const                { return buildAUv3Value.get(); }
+    bool shouldBuildRTAS() const                { return buildRTASValue.get(); }
+    bool shouldBuildAAX() const                 { return buildAAXValue.get(); }
+    bool shouldBuildStandalonePlugin() const    { return buildStandaloneValue.get(); }
+    bool shouldEnableIAA() const                { return enableIAAValue.get(); }
 
     //==============================================================================
-    File getGeneratedCodeFolder() const                         { return getFile().getSiblingFile ("JuceLibraryCode"); }
-    File getSourceFilesFolder() const                           { return getProjectFolder().getChildFile ("Source"); }
-    File getLocalModulesFolder() const                          { return getGeneratedCodeFolder().getChildFile ("modules"); }
-    File getLocalModuleFolder (const String& moduleID) const    { return getLocalModulesFolder().getChildFile (moduleID); }
-    File getAppIncludeFile() const                              { return getGeneratedCodeFolder().getChildFile (getJuceSourceHFilename()); }
+    String getPluginNameString() const                { return pluginNameValue.get(); }
+    String getPluginDescriptionString() const         { return pluginDescriptionValue.get();}
+    String getPluginManufacturerString() const        { return pluginManufacturerValue.get(); }
+    String getPluginManufacturerCodeString() const    { return pluginManufacturerCodeValue.get(); }
+    String getPluginCodeString() const                { return pluginCodeValue.get(); }
+    String getPluginChannelConfigsString() const      { return pluginChannelConfigsValue.get(); }
+    String getPluginAUExportPrefixString() const      { return pluginAUExportPrefixValue.get(); }
+    String getPluginAUMainTypeString() const          { return pluginAUMainTypeValue.get(); }
+    String getPluginRTASCategoryString() const        { return pluginRTASCategoryValue.get(); }
+    String getAAXIdentifierString() const             { return pluginAAXIdentifierValue.get(); }
+    String getPluginAAXCategoryString() const         { return pluginAAXCategoryValue.get(); }
 
-    File getBinaryDataCppFile (int index) const;
-    File getBinaryDataHeaderFile() const                { return getBinaryDataCppFile (0).withFileExtension (".h"); }
-    Value getMaxBinaryFileSize()                        { return getProjectValue (Ids::maxBinaryFileSize); }
-    Value shouldIncludeBinaryInAppConfig()              { return getProjectValue (Ids::includeBinaryInAppConfig); }
+    bool isPluginSynth() const                        { return pluginIsSynthValue.get(); }
+    bool pluginWantsMidiInput() const                 { return pluginWantsMidiInputValue.get(); }
+    bool pluginProducesMidiOutput() const             { return pluginProducesMidiOutValue.get(); }
+    bool isPluginMidiEffect() const                   { return pluginIsMidiEffectPluginValue.get(); }
+    bool pluginEditorNeedsKeyFocus() const            { return pluginEditorNeedsKeyFocusValue.get(); }
+    bool isPluginRTASBypassDisabled() const           { return pluginRTASBypassDisabledValue.get(); }
+    bool isPluginRTASMultiMonoDisabled() const        { return pluginRTASMultiMonoDisabledValue.get(); }
+    bool isPluginAAXBypassDisabled() const            { return pluginAAXBypassDisabledValue.get(); }
+    bool isPluginAAXMultiMonoDisabled() const         { return pluginAAXMultiMonoDisabledValue.get(); }
 
-    //==============================================================================
-    String getAppConfigFilename() const                 { return "AppConfig.h"; }
-    String getJuceSourceFilenameRoot() const            { return "JuceLibraryCode"; }
-    String getJuceSourceHFilename() const               { return "JuceHeader.h"; }
-
-    //==============================================================================
-    // Some helper methods for audio plugin/host projects.
-    Value shouldBuildVST()                      { return getProjectValue ("buildVST"); }
-    Value shouldBuildVST3()                     { return getProjectValue ("buildVST3"); }
-    Value shouldBuildAU()                       { return getProjectValue ("buildAU"); }
-    Value shouldBuildAUv3()                     { return getProjectValue ("buildAUv3"); }
-    Value shouldBuildRTAS()                     { return getProjectValue ("buildRTAS"); }
-    Value shouldBuildAAX()                      { return getProjectValue ("buildAAX"); }
-    Value shouldBuildStandalone()               { return shouldBuildAUv3(); /* TODO: enable this when standalone becomes independent from AUv3: getProjectValue ("buildStandalone"); */}
-    Value getPluginName()                       { return getProjectValue ("pluginName"); }
-    Value getPluginDesc()                       { return getProjectValue ("pluginDesc"); }
-    Value getPluginManufacturer()               { return getProjectValue ("pluginManufacturer"); }
-    Value getPluginManufacturerCode()           { return getProjectValue ("pluginManufacturerCode"); }
-    Value getPluginCode()                       { return getProjectValue ("pluginCode"); }
-    Value getPluginChannelConfigs()             { return getProjectValue ("pluginChannelConfigs"); }
-    Value getPluginIsSynth()                    { return getProjectValue ("pluginIsSynth"); }
-    Value getPluginWantsMidiInput()             { return getProjectValue ("pluginWantsMidiIn"); }
-    Value getPluginProducesMidiOut()            { return getProjectValue ("pluginProducesMidiOut"); }
-    Value getPluginIsMidiEffectPlugin()         { return getProjectValue ("pluginIsMidiEffectPlugin"); }
-    Value getPluginEditorNeedsKeyFocus()        { return getProjectValue ("pluginEditorRequiresKeys"); }
-    Value getPluginVSTCategory()                { return getProjectValue ("pluginVSTCategory"); }
-    Value getPluginAUExportPrefix()             { return getProjectValue ("pluginAUExportPrefix"); }
-    Value getPluginAUMainType()                 { return getProjectValue ("pluginAUMainType"); }
-    Value getPluginRTASCategory()               { return getProjectValue ("pluginRTASCategory"); }
-    Value getPluginRTASBypassDisabled()         { return getProjectValue ("pluginRTASDisableBypass"); }
-    Value getPluginRTASMultiMonoDisabled()      { return getProjectValue ("pluginRTASDisableMultiMono"); }
-    Value getPluginAAXCategory()                { return getProjectValue ("pluginAAXCategory"); }
-    Value getPluginAAXBypassDisabled()          { return getProjectValue ("pluginAAXDisableBypass"); }
-    Value getPluginAAXMultiMonoDisabled()       { return getProjectValue ("pluginAAXDisableMultiMono"); }
     String getPluginRTASCategoryCode();
     String getAUMainTypeString();
     String getAUMainTypeCode();
+    String getIAATypeCode();
+    String getIAAPluginName();
     String getPluginVSTCategoryString();
 
     bool isAUPluginHost();
     bool isVSTPluginHost();
     bool isVST3PluginHost();
 
+    //==============================================================================
+    bool shouldBuildTargetType (ProjectType::Target::Type targetType) const noexcept;
+    static ProjectType::Target::Type getTargetTypeFromFilePath (const File& file, bool returnSharedTargetIfNoValidSuffix);
+
+    //==============================================================================
     void updateDeprecatedProjectSettingsInteractively();
 
     //==============================================================================
@@ -235,7 +250,7 @@ public:
 
         UndoManager* getUndoManager() const              { return project.getUndoManagerFor (state); }
 
-        Icon getIcon() const;
+        Icon getIcon (bool isOpen = false) const;
         bool isIconCrossedOut() const;
 
         Project& project;
@@ -265,7 +280,7 @@ public:
         bool next();
 
         ProjectExporter& operator*() const       { return *exporter; }
-        ProjectExporter* operator->() const      { return exporter; }
+        ProjectExporter* operator->() const      { return exporter.get(); }
 
         ScopedPointer<ProjectExporter> exporter;
         int index;
@@ -278,16 +293,12 @@ public:
     //==============================================================================
     struct ConfigFlag
     {
-        String symbol, description, sourceModuleID, defaultValue;
-        Value value;   // 1 = true, 2 = false, anything else = use default
+        String symbol, description, sourceModuleID;
+        ValueWithDefault value;
     };
 
-    static const char* const configFlagDefault;
-    static const char* const configFlagEnabled;
-    static const char* const configFlagDisabled;
-
-    Value getConfigFlag (const String& name);
-    bool isConfigFlagEnabled (const String& name) const;
+    ValueWithDefault getConfigFlag (const String& name);
+    bool isConfigFlagEnabled (const String& name, bool defaultIsEnabled = false) const;
 
     //==============================================================================
     EnabledModuleList& getModules();
@@ -307,6 +318,7 @@ public:
 
     //==============================================================================
     UndoManager* getUndoManagerFor (const ValueTree&) const             { return nullptr; }
+    UndoManager* getUndoManager() const                                 { return nullptr; }
 
     //==============================================================================
     static const char* projectFileExtension;
@@ -315,20 +327,46 @@ public:
     bool hasProjectBeenModified();
     void updateModificationTime() { modificationTime = getFile().getLastModificationTime(); }
 
-private:
     //==============================================================================
-    void setMissingAudioPluginDefaultValues();
-    void createAudioPluginPropertyEditors (PropertyListBuilder& props);
+    String getUniqueTargetFolderSuffixForExporter (const String& exporterName, const String& baseTargetFolder);
+
+    //==============================================================================
+    bool isCurrentlySaving() const noexcept     { return isSaving; }
+    bool shouldWaitAfterSaving = false;
+    String specifiedExporterToSave = {};
+
+private:
+    ValueTree projectRoot  { Ids::JUCERPROJECT };
+
+    ValueWithDefault projectNameValue, projectUIDValue, projectTypeValue, versionValue, bundleIdentifierValue, companyNameValue, companyCopyrightValue,
+                     companyWebsiteValue, companyEmailValue, displaySplashScreenValue, reportAppUsageValue, splashScreenColourValue, cppStandardValue,
+                     headerSearchPathsValue, preprocessorDefsValue, userNotesValue, maxBinaryFileSizeValue, includeBinaryDataInAppConfigValue, binaryDataNamespaceValue;
+
+    ValueWithDefault buildVSTValue, buildVST3Value, buildAUValue, buildAUv3Value, buildRTASValue, buildAAXValue, buildStandaloneValue,
+                     enableIAAValue, pluginNameValue, pluginDescriptionValue, pluginManufacturerValue, pluginManufacturerCodeValue,
+                     pluginCodeValue, pluginChannelConfigsValue, pluginIsSynthValue, pluginWantsMidiInputValue, pluginProducesMidiOutValue,
+                     pluginIsMidiEffectPluginValue, pluginEditorNeedsKeyFocusValue, pluginVSTCategoryValue, pluginAUExportPrefixValue,
+                     pluginAUMainTypeValue, pluginRTASCategoryValue, pluginRTASBypassDisabledValue, pluginRTASMultiMonoDisabledValue,
+                     pluginAAXIdentifierValue, pluginAAXCategoryValue, pluginAAXBypassDisabledValue, pluginAAXMultiMonoDisabledValue;
 
     //==============================================================================
     friend class Item;
-    ValueTree projectRoot;
     ScopedPointer<EnabledModuleList> enabledModulesList;
-    bool isSaving;
+    bool isSaving = false;
+    Time modificationTime;
+    StringPairArray parsedPreprocessorDefs;
 
+    //==============================================================================
+    void initialiseProjectValues();
+    void initialiseMainGroup();
+    void initialiseAudioPluginValues();
+
+    bool setCppVersionFromOldExporterSettings();
+
+    void createAudioPluginPropertyEditors (PropertyListBuilder& props);
+
+    //==============================================================================
     void updateProjectSettings();
-    void sanitiseConfigFlags();
-    void setMissingDefaultValues();
     ValueTree getConfigurations() const;
     ValueTree getConfigNode();
 
@@ -338,10 +376,5 @@ private:
     void updateOldModulePaths();
     void warnAboutOldProjucerVersion();
 
-    Time modificationTime;
-
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Project)
 };
-
-
-#endif   // JUCER_PROJECT_H_INCLUDED
