@@ -2,32 +2,33 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 MidiBuffer MPEMessages::addZone (MPEZone zone)
 {
-    MidiBuffer buffer (MidiRPNGenerator::generate (zone.getFirstNoteChannel(),
-                                                   zoneLayoutMessagesRpnNumber,
-                                                   zone.getNumNoteChannels(),
-                                                   false, false));
+    auto buffer = MidiRPNGenerator::generate (zone.getFirstNoteChannel(),
+                                              zoneLayoutMessagesRpnNumber,
+                                              zone.getNumNoteChannels(),
+                                              false, false);
 
     buffer.addEvents (perNotePitchbendRange (zone), 0, -1, 0);
     buffer.addEvents (masterPitchbendRange (zone), 0, -1, 0);
@@ -73,7 +74,7 @@ MidiBuffer MPEMessages::setZoneLayout (const MPEZoneLayout& layout)
 class MPEMessagesTests  : public UnitTest
 {
 public:
-    MPEMessagesTests() : UnitTest ("MPEMessages class") {}
+    MPEMessagesTests() : UnitTest ("MPEMessages class", "MIDI/MPE") {}
 
     void runTest() override
     {
@@ -195,3 +196,5 @@ private:
 static MPEMessagesTests MPEMessagesUnitTests;
 
 #endif // JUCE_UNIT_TESTS
+
+} // namespace juce
