@@ -2,26 +2,27 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 namespace
 {
     uint16 generateNoteID (int midiChannel, int midiNoteNumber) noexcept
@@ -48,25 +49,13 @@ MPENote::MPENote (int midiChannel_,
       pitchbend (pitchbend_),
       pressure (pressure_),
       timbre (timbre_),
-      noteOffVelocity (MPEValue::minValue()),
       keyState (keyState_)
 {
     jassert (keyState != MPENote::off);
     jassert (isValid());
 }
 
-MPENote::MPENote() noexcept
-    : noteID (0),
-      midiChannel (0),
-      initialNote (0),
-      noteOnVelocity (MPEValue::minValue()),
-      pitchbend (MPEValue::centreValue()),
-      pressure (MPEValue::centreValue()),
-      timbre (MPEValue::centreValue()),
-      noteOffVelocity (MPEValue::minValue()),
-      keyState (MPENote::off)
-{
-}
+MPENote::MPENote() noexcept {}
 
 //==============================================================================
 bool MPENote::isValid() const noexcept
@@ -77,7 +66,7 @@ bool MPENote::isValid() const noexcept
 //==============================================================================
 double MPENote::getFrequencyInHertz (double frequencyOfA) const noexcept
 {
-    double pitchInSemitones = double (initialNote) + totalPitchbendInSemitones;
+    auto pitchInSemitones = double (initialNote) + totalPitchbendInSemitones;
     return frequencyOfA * std::pow (2.0, (pitchInSemitones - 69.0) / 12.0);
 }
 
@@ -101,7 +90,7 @@ bool MPENote::operator!= (const MPENote& other) const noexcept
 class MPENoteTests : public UnitTest
 {
 public:
-    MPENoteTests() : UnitTest ("MPENote class") {}
+    MPENoteTests() : UnitTest ("MPENote class", "MIDI/MPE") {}
 
     //==============================================================================
     void runTest() override
@@ -130,3 +119,5 @@ private:
 static MPENoteTests MPENoteUnitTests;
 
 #endif // JUCE_UNIT_TESTS
+
+} // namespace juce
