@@ -2,29 +2,26 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_MPENOTE_H_INCLUDED
-#define JUCE_MPENOTE_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -42,10 +39,10 @@ struct JUCE_API  MPENote
     //==============================================================================
     enum KeyState
     {
-        off                  = 0,
-        keyDown              = 1,
-        sustained            = 2,
-        keyDownAndSustained  = 3
+        off                  = 0, /**< The key is up (off). */
+        keyDown              = 1, /**< The note key is currently down (pressed). */
+        sustained            = 2, /**< The note is sustained (by a sustain or sostenuto pedal). */
+        keyDownAndSustained  = 3  /**< The note key is down and sustained (by a sustain or sostenuto pedal). */
     };
 
     //==============================================================================
@@ -67,7 +64,7 @@ struct JUCE_API  MPENote
         @param keyState       The key state of the note (whether the key is down
                               and/or the note is sustained). This value must not
                               be MPENote::off, since you are triggering a new note.
-                              (If not specified, the default value will be MPENOte::keyDown.)
+                              (If not specified, the default value will be MPENote::keyDown.)
     */
     MPENote (int midiChannel,
              int initialNote,
@@ -95,17 +92,17 @@ struct JUCE_API  MPENote
         sounding notes that may use the same note number or MIDI channel.
         This should never change during the lifetime of a note object.
     */
-    uint16 noteID;
+    uint16 noteID = 0;
 
     /** The MIDI channel which this note uses.
         This should never change during the lifetime of an MPENote object.
     */
-    uint8 midiChannel;
+    uint8 midiChannel = 0;
 
     /** The MIDI note number that was sent when the note was triggered.
         This should never change during the lifetime of an MPENote object.
     */
-    uint8 initialNote;
+    uint8 initialNote = 0;
 
     //==============================================================================
     // The five dimensions of continuous expressive control
@@ -113,7 +110,7 @@ struct JUCE_API  MPENote
     /** The velocity ("strike") of the note-on.
         This dimension will stay constant after the note has been turned on.
     */
-    MPEValue noteOnVelocity;
+    MPEValue noteOnVelocity  { MPEValue::minValue() };
 
     /** Current per-note pitchbend of the note  (in units of MIDI pitchwheel
         position). This dimension can be modulated while the note sounds.
@@ -125,18 +122,18 @@ struct JUCE_API  MPENote
 
         @see totalPitchbendInSemitones, getFrequencyInHertz
     */
-    MPEValue pitchbend;
+    MPEValue pitchbend       { MPEValue::centreValue() };
 
     /** Current pressure with which the note is held down.
         This dimension can be modulated while the note sounds.
     */
-    MPEValue pressure;
+    MPEValue pressure        { MPEValue::centreValue() };
 
     /** Current value of the note's third expressive dimension, tyically
          encoding some kind of timbre parameter.
         This dimension can be modulated while the note sounds.
     */
-    MPEValue timbre;
+    MPEValue timbre          { MPEValue::centreValue() };
 
     /** The release velocity ("lift") of the note after a note-off has been
         received.
@@ -144,7 +141,7 @@ struct JUCE_API  MPENote
         been received for the note (and keyState is set to MPENote::off or
         MPENOte::sustained). Initially, the value is undefined.
     */
-    MPEValue noteOffVelocity;
+    MPEValue noteOffVelocity { MPEValue::minValue() };
 
     //==============================================================================
     /** Current effective pitchbend of the note in units of semitones, relative
@@ -161,7 +158,7 @@ struct JUCE_API  MPENote
     /** Current key state. Indicates whether the note key is currently down (pressed)
         and/or the note is sustained (by a sustain or sostenuto pedal).
     */
-    KeyState keyState;
+    KeyState keyState        { MPENote::off };
 
     //==============================================================================
     /** Returns the current frequency of the note in Hertz. This is the a sum of
@@ -176,5 +173,4 @@ struct JUCE_API  MPENote
     bool operator!= (const MPENote& other) const noexcept;
 };
 
-
-#endif // JUCE_MPENOTE_H_INCLUDED
+} // namespace juce
