@@ -2,29 +2,26 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_MPESynthesiser_H_INCLUDED
-#define JUCE_MPESynthesiser_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -50,7 +47,7 @@
     what the target playback rate is. This value is passed on to the voices so that
     they can pitch their output correctly.
 
-    @see MPESynthesiserBase, MPESythesiserVoice, MPENote, MPEInstrument
+    @see MPESynthesiserBase, MPESynthesiserVoice, MPENote, MPEInstrument
 */
 class JUCE_API  MPESynthesiser   : public MPESynthesiserBase
 {
@@ -300,14 +297,13 @@ protected:
 
     //==============================================================================
     OwnedArray<MPESynthesiserVoice> voices;
+    CriticalSection voicesLock;
 
 private:
     //==============================================================================
-    bool shouldStealVoices;
-    CriticalSection voicesLock;
+    bool shouldStealVoices = false;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MPESynthesiser)
 };
 
-
-#endif // JUCE_MPESynthesiser_H_INCLUDED
+} // namespace juce
