@@ -2,26 +2,27 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 namespace
 {
     void checkAndLimitZoneParameters (int minValue,
@@ -34,7 +35,7 @@ namespace
             // was not within the allowed range!
             // we fit this back into the allowed range here to maintain a valid
             // state for the zone, but probably the resulting zone is not what you
-            //wanted it to be!
+            // wanted it to be!
             jassertfalse;
 
             valueToCheckAndLimit = jlimit (minValue, maxValue, valueToCheckAndLimit);
@@ -133,7 +134,7 @@ bool MPEZone::overlapsWith (MPEZone other) const noexcept
 //==============================================================================
 bool MPEZone::truncateToFit (MPEZone other) noexcept
 {
-    const int masterChannelDiff = other.masterChannel - masterChannel;
+    auto masterChannelDiff = other.masterChannel - masterChannel;
 
     // we need at least 2 channels to be left after truncation:
     // 1 master channel and 1 note channel. otherwise we can't truncate.
@@ -165,7 +166,7 @@ bool MPEZone::operator!= (const MPEZone& other) const noexcept
 class MPEZoneTests   : public UnitTest
 {
 public:
-    MPEZoneTests()  : UnitTest ("MPEZone class") {}
+    MPEZoneTests()  : UnitTest ("MPEZone class", "MIDI/MPE") {}
 
     void runTest() override
     {
@@ -314,3 +315,5 @@ private:
 static MPEZoneTests MPEZoneUnitTests;
 
 #endif // JUCE_UNIT_TESTS
+
+} // namespace juce
