@@ -2,44 +2,33 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 SynthesiserSound::SynthesiserSound() {}
 SynthesiserSound::~SynthesiserSound() {}
 
 //==============================================================================
-SynthesiserVoice::SynthesiserVoice()
-    : currentSampleRate (44100.0),
-      currentlyPlayingNote (-1),
-      currentPlayingMidiChannel (0),
-      noteOnTime (0),
-      keyIsDown (false),
-      sustainPedalDown (false),
-      sostenutoPedalDown (false)
-{
-}
-
-SynthesiserVoice::~SynthesiserVoice()
-{
-}
+SynthesiserVoice::SynthesiserVoice() {}
+SynthesiserVoice::~SynthesiserVoice() {}
 
 bool SynthesiserVoice::isPlayingChannel (const int midiChannel) const
 {
@@ -78,18 +67,13 @@ void SynthesiserVoice::renderNextBlock (AudioBuffer<double>& outputBuffer,
                                    outputBuffer.getNumChannels(),
                                    startSample, numSamples);
 
-    tempBuffer.makeCopyOf (subBuffer);
+    tempBuffer.makeCopyOf (subBuffer, true);
     renderNextBlock (tempBuffer, 0, numSamples);
-    subBuffer.makeCopyOf (tempBuffer);
+    subBuffer.makeCopyOf (tempBuffer, true);
 }
 
 //==============================================================================
 Synthesiser::Synthesiser()
-    : sampleRate (0),
-      lastNoteOnCounter (0),
-      minimumSubBlockSize (32),
-      subBlockSubdivisionIsStrict (false),
-      shouldStealNotes (true)
 {
     for (int i = 0; i < numElementsInArray (lastPitchWheelValues); ++i)
         lastPitchWheelValues[i] = 0x2000;
@@ -161,13 +145,11 @@ void Synthesiser::setCurrentPlaybackSampleRate (const double newRate)
     if (sampleRate != newRate)
     {
         const ScopedLock sl (lock);
-
         allNotesOff (0, false);
-
         sampleRate = newRate;
 
-        for (int i = voices.size(); --i >= 0;)
-            voices.getUnchecked (i)->setCurrentPlaybackSampleRate (newRate);
+        for (auto* voice : voices)
+            voice->setCurrentPlaybackSampleRate (newRate);
     }
 }
 
@@ -232,25 +214,19 @@ void Synthesiser::processNextBlock (AudioBuffer<floatType>& outputAudio,
 }
 
 // explicit template instantiation
-template void Synthesiser::processNextBlock<float> (AudioBuffer<float>& outputAudio,
-                                                    const MidiBuffer& midiData,
-                                                    int startSample,
-                                                    int numSamples);
-template void Synthesiser::processNextBlock<double> (AudioBuffer<double>& outputAudio,
-                                                     const MidiBuffer& midiData,
-                                                     int startSample,
-                                                     int numSamples);
+template void Synthesiser::processNextBlock<float>  (AudioBuffer<float>&,  const MidiBuffer&, int, int);
+template void Synthesiser::processNextBlock<double> (AudioBuffer<double>&, const MidiBuffer&, int, int);
 
 void Synthesiser::renderVoices (AudioBuffer<float>& buffer, int startSample, int numSamples)
 {
-    for (int i = voices.size(); --i >= 0;)
-        voices.getUnchecked (i)->renderNextBlock (buffer, startSample, numSamples);
+    for (auto* voice : voices)
+        voice->renderNextBlock (buffer, startSample, numSamples);
 }
 
 void Synthesiser::renderVoices (AudioBuffer<double>& buffer, int startSample, int numSamples)
 {
-    for (int i = voices.size(); --i >= 0;)
-        voices.getUnchecked (i)->renderNextBlock (buffer, startSample, numSamples);
+    for (auto* voice : voices)
+        voice->renderNextBlock (buffer, startSample, numSamples);
 }
 
 void Synthesiser::handleMidiEvent (const MidiMessage& m)
@@ -300,23 +276,15 @@ void Synthesiser::noteOn (const int midiChannel,
 {
     const ScopedLock sl (lock);
 
-    for (int i = sounds.size(); --i >= 0;)
+    for (auto* sound : sounds)
     {
-        SynthesiserSound* const sound = sounds.getUnchecked(i);
-
-        if (sound->appliesToNote (midiNoteNumber)
-             && sound->appliesToChannel (midiChannel))
+        if (sound->appliesToNote (midiNoteNumber) && sound->appliesToChannel (midiChannel))
         {
             // If hitting a note that's still ringing, stop it first (it could be
             // still playing because of the sustain or sostenuto pedal).
-            for (int j = voices.size(); --j >= 0;)
-            {
-                SynthesiserVoice* const voice = voices.getUnchecked (j);
-
-                if (voice->getCurrentlyPlayingNote() == midiNoteNumber
-                     && voice->isPlayingChannel (midiChannel))
+            for (auto* voice : voices)
+                if (voice->getCurrentlyPlayingNote() == midiNoteNumber && voice->isPlayingChannel (midiChannel))
                     stopVoice (voice, 1.0f, true);
-            }
 
             startVoice (findFreeVoice (sound, midiChannel, midiNoteNumber, shouldStealNotes),
                         sound, midiChannel, midiNoteNumber, velocity);
@@ -339,9 +307,9 @@ void Synthesiser::startVoice (SynthesiserVoice* const voice,
         voice->currentPlayingMidiChannel = midiChannel;
         voice->noteOnTime = ++lastNoteOnCounter;
         voice->currentlyPlayingSound = sound;
-        voice->keyIsDown = true;
-        voice->sostenutoPedalDown = false;
-        voice->sustainPedalDown = sustainPedalsDown[midiChannel];
+        voice->setKeyDown (true);
+        voice->setSostenutoPedalDown (false);
+        voice->setSustainPedalDown (sustainPedalsDown[midiChannel]);
 
         voice->startNote (midiNoteNumber, velocity, sound,
                           lastPitchWheelValues [midiChannel - 1]);
@@ -365,10 +333,8 @@ void Synthesiser::noteOff (const int midiChannel,
 {
     const ScopedLock sl (lock);
 
-    for (int i = voices.size(); --i >= 0;)
+    for (auto* voice : voices)
     {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
         if (voice->getCurrentlyPlayingNote() == midiNoteNumber
               && voice->isPlayingChannel (midiChannel))
         {
@@ -377,11 +343,11 @@ void Synthesiser::noteOff (const int midiChannel,
                 if (sound->appliesToNote (midiNoteNumber)
                      && sound->appliesToChannel (midiChannel))
                 {
-                    jassert (! voice->keyIsDown || voice->sustainPedalDown == sustainPedalsDown [midiChannel]);
+                    jassert (! voice->keyIsDown || voice->isSustainPedalDown() == sustainPedalsDown [midiChannel]);
 
-                    voice->keyIsDown = false;
+                    voice->setKeyDown (false);
 
-                    if (! (voice->sustainPedalDown || voice->sostenutoPedalDown))
+                    if (! (voice->isSustainPedalDown() || voice->isSostenutoPedalDown()))
                         stopVoice (voice, velocity, allowTailOff);
                 }
             }
@@ -393,13 +359,9 @@ void Synthesiser::allNotesOff (const int midiChannel, const bool allowTailOff)
 {
     const ScopedLock sl (lock);
 
-    for (int i = voices.size(); --i >= 0;)
-    {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
+    for (auto* voice : voices)
         if (midiChannel <= 0 || voice->isPlayingChannel (midiChannel))
             voice->stopNote (1.0f, allowTailOff);
-    }
 
     sustainPedalsDown.clear();
 }
@@ -408,13 +370,9 @@ void Synthesiser::handlePitchWheel (const int midiChannel, const int wheelValue)
 {
     const ScopedLock sl (lock);
 
-    for (int i = voices.size(); --i >= 0;)
-    {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
+    for (auto* voice : voices)
         if (midiChannel <= 0 || voice->isPlayingChannel (midiChannel))
             voice->pitchWheelMoved (wheelValue);
-    }
 }
 
 void Synthesiser::handleController (const int midiChannel,
@@ -431,40 +389,28 @@ void Synthesiser::handleController (const int midiChannel,
 
     const ScopedLock sl (lock);
 
-    for (int i = voices.size(); --i >= 0;)
-    {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
+    for (auto* voice : voices)
         if (midiChannel <= 0 || voice->isPlayingChannel (midiChannel))
             voice->controllerMoved (controllerNumber, controllerValue);
-    }
 }
 
 void Synthesiser::handleAftertouch (int midiChannel, int midiNoteNumber, int aftertouchValue)
 {
     const ScopedLock sl (lock);
 
-    for (int i = voices.size(); --i >= 0;)
-    {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
+    for (auto* voice : voices)
         if (voice->getCurrentlyPlayingNote() == midiNoteNumber
               && (midiChannel <= 0 || voice->isPlayingChannel (midiChannel)))
             voice->aftertouchChanged (aftertouchValue);
-    }
 }
 
 void Synthesiser::handleChannelPressure (int midiChannel, int channelPressureValue)
 {
     const ScopedLock sl (lock);
 
-    for (int i = voices.size(); --i >= 0;)
-    {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
+    for (auto* voice : voices)
         if (midiChannel <= 0 || voice->isPlayingChannel (midiChannel))
             voice->channelPressureChanged (channelPressureValue);
-    }
 }
 
 void Synthesiser::handleSustainPedal (int midiChannel, bool isDown)
@@ -476,25 +422,19 @@ void Synthesiser::handleSustainPedal (int midiChannel, bool isDown)
     {
         sustainPedalsDown.setBit (midiChannel);
 
-        for (int i = voices.size(); --i >= 0;)
-        {
-            SynthesiserVoice* const voice = voices.getUnchecked (i);
-
+        for (auto* voice : voices)
             if (voice->isPlayingChannel (midiChannel) && voice->isKeyDown())
-                voice->sustainPedalDown = true;
-        }
+                voice->setSustainPedalDown (true);
     }
     else
     {
-        for (int i = voices.size(); --i >= 0;)
+        for (auto* voice : voices)
         {
-            SynthesiserVoice* const voice = voices.getUnchecked (i);
-
             if (voice->isPlayingChannel (midiChannel))
             {
-                voice->sustainPedalDown = false;
+                voice->setSustainPedalDown (false);
 
-                if (! voice->isKeyDown())
+                if (! (voice->isKeyDown() || voice->isSostenutoPedalDown()))
                     stopVoice (voice, 1.0f, true);
             }
         }
@@ -508,15 +448,13 @@ void Synthesiser::handleSostenutoPedal (int midiChannel, bool isDown)
     jassert (midiChannel > 0 && midiChannel <= 16);
     const ScopedLock sl (lock);
 
-    for (int i = voices.size(); --i >= 0;)
+    for (auto* voice : voices)
     {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
         if (voice->isPlayingChannel (midiChannel))
         {
             if (isDown)
-                voice->sostenutoPedalDown = true;
-            else if (voice->sostenutoPedalDown)
+                voice->setSostenutoPedalDown (true);
+            else if (voice->isSostenutoPedalDown())
                 stopVoice (voice, 1.0f, true);
         }
     }
@@ -541,13 +479,9 @@ SynthesiserVoice* Synthesiser::findFreeVoice (SynthesiserSound* soundToPlay,
 {
     const ScopedLock sl (lock);
 
-    for (int i = 0; i < voices.size(); ++i)
-    {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
+    for (auto* voice : voices)
         if ((! voice->isVoiceActive()) && voice->canPlaySound (soundToPlay))
             return voice;
-    }
 
     if (stealIfNoneAvailable)
         return findVoiceToSteal (soundToPlay, midiChannel, midiNoteNumber);
@@ -555,14 +489,6 @@ SynthesiserVoice* Synthesiser::findFreeVoice (SynthesiserSound* soundToPlay,
     return nullptr;
 }
 
-struct VoiceAgeSorter
-{
-    static int compareElements (SynthesiserVoice* v1, SynthesiserVoice* v2) noexcept
-    {
-        return v1->wasStartedBefore (*v2) ? -1 : (v2->wasStartedBefore (*v1) ? 1 : 0);
-    }
-};
-
 SynthesiserVoice* Synthesiser::findVoiceToSteal (SynthesiserSound* soundToPlay,
                                                  int /*midiChannel*/, int midiNoteNumber) const
 {
@@ -571,7 +497,7 @@ SynthesiserVoice* Synthesiser::findVoiceToSteal (SynthesiserSound* soundToPlay,
     // - Protect the lowest & topmost notes, even if sustained, but not if they've been released.
 
     // apparently you are trying to render audio without having any voices...
-    jassert (voices.size() > 0);
+    jassert (! voices.isEmpty());
 
     // These are the voices we want to protect (ie: only steal if unavoidable)
     SynthesiserVoice* low = nullptr; // Lowest sounding note, might be sustained, but NOT in release phase
@@ -581,20 +507,26 @@ SynthesiserVoice* Synthesiser::findVoiceToSteal (SynthesiserSound* soundToPlay,
     Array<SynthesiserVoice*> usableVoices;
     usableVoices.ensureStorageAllocated (voices.size());
 
-    for (int i = 0; i < voices.size(); ++i)
+    for (auto* voice : voices)
     {
-        SynthesiserVoice* const voice = voices.getUnchecked (i);
-
         if (voice->canPlaySound (soundToPlay))
         {
             jassert (voice->isVoiceActive()); // We wouldn't be here otherwise
 
-            VoiceAgeSorter sorter;
-            usableVoices.addSorted (sorter, voice);
+            usableVoices.add (voice);
+
+            // NB: Using a functor rather than a lambda here due to scare-stories about
+            // compilers generating code containing heap allocations..
+            struct Sorter
+            {
+                bool operator() (const SynthesiserVoice* a, const SynthesiserVoice* b) const noexcept { return a->wasStartedBefore (*b); }
+            };
+
+            std::sort (usableVoices.begin(), usableVoices.end(), Sorter());
 
             if (! voice->isPlayingButReleased()) // Don't protect released notes
             {
-                const int note = voice->getCurrentlyPlayingNote();
+                auto note = voice->getCurrentlyPlayingNote();
 
                 if (low == nullptr || note < low->getCurrentlyPlayingNote())
                     low = voice;
@@ -609,43 +541,25 @@ SynthesiserVoice* Synthesiser::findVoiceToSteal (SynthesiserSound* soundToPlay,
     if (top == low)
         top = nullptr;
 
-    const int numUsableVoices = usableVoices.size();
-
     // The oldest note that's playing with the target pitch is ideal..
-    for (int i = 0; i < numUsableVoices; ++i)
-    {
-        SynthesiserVoice* const voice = usableVoices.getUnchecked (i);
-
+    for (auto* voice : usableVoices)
         if (voice->getCurrentlyPlayingNote() == midiNoteNumber)
             return voice;
-    }
 
     // Oldest voice that has been released (no finger on it and not held by sustain pedal)
-    for (int i = 0; i < numUsableVoices; ++i)
-    {
-        SynthesiserVoice* const voice = usableVoices.getUnchecked (i);
-
+    for (auto* voice : usableVoices)
         if (voice != low && voice != top && voice->isPlayingButReleased())
             return voice;
-    }
 
     // Oldest voice that doesn't have a finger on it:
-    for (int i = 0; i < numUsableVoices; ++i)
-    {
-        SynthesiserVoice* const voice = usableVoices.getUnchecked (i);
-
+    for (auto* voice : usableVoices)
         if (voice != low && voice != top && ! voice->isKeyDown())
             return voice;
-    }
 
     // Oldest voice that isn't protected
-    for (int i = 0; i < numUsableVoices; ++i)
-    {
-        SynthesiserVoice* const voice = usableVoices.getUnchecked (i);
-
+    for (auto* voice : usableVoices)
         if (voice != low && voice != top)
             return voice;
-    }
 
     // We've only got "protected" voices now: lowest note takes priority
     jassert (low != nullptr);
@@ -656,3 +570,5 @@ SynthesiserVoice* Synthesiser::findVoiceToSteal (SynthesiserSound* soundToPlay,
 
     return low;
 }
+
+} // namespace juce
