@@ -2,29 +2,26 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_SYNTHESISER_H_INCLUDED
-#define JUCE_SYNTHESISER_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -185,6 +182,8 @@ public:
     virtual void renderNextBlock (AudioBuffer<float>& outputBuffer,
                                   int startSample,
                                   int numSamples) = 0;
+
+    /** A double-precision version of renderNextBlock() */
     virtual void renderNextBlock (AudioBuffer<double>& outputBuffer,
                                   int startSample,
                                   int numSamples);
@@ -217,12 +216,23 @@ public:
     */
     bool isKeyDown() const noexcept                             { return keyIsDown; }
 
+    /** Allows you to modify the flag indicating that the key that triggered this voice is still held down.
+        @see isKeyDown
+    */
+    void setKeyDown (bool isNowDown) noexcept                   { keyIsDown = isNowDown; }
+
     /** Returns true if the sustain pedal is currently active for this voice. */
     bool isSustainPedalDown() const noexcept                    { return sustainPedalDown; }
 
+    /** Modifies the sustain pedal flag. */
+    void setSustainPedalDown (bool isNowDown) noexcept          { sustainPedalDown = isNowDown; }
+
     /** Returns true if the sostenuto pedal is currently active for this voice. */
     bool isSostenutoPedalDown() const noexcept                  { return sostenutoPedalDown; }
 
+    /** Modifies the sostenuto pedal flag. */
+    void setSostenutoPedalDown (bool isNowDown) noexcept        { sostenutoPedalDown = isNowDown; }
+
     /** Returns true if a voice is sounding in its release phase **/
     bool isPlayingButReleased() const noexcept
     {
@@ -252,11 +262,11 @@ private:
     //==============================================================================
     friend class Synthesiser;
 
-    double currentSampleRate;
-    int currentlyPlayingNote, currentPlayingMidiChannel;
-    uint32 noteOnTime;
+    double currentSampleRate = 44100.0;
+    int currentlyPlayingNote = -1, currentPlayingMidiChannel = 0;
+    uint32 noteOnTime = 0;
     SynthesiserSound::Ptr currentlyPlayingSound;
-    bool keyIsDown, sustainPedalDown, sostenutoPedalDown;
+    bool keyIsDown = false, sustainPedalDown = false, sostenutoPedalDown = false;
 
     AudioBuffer<float> tempBuffer;
 
@@ -510,16 +520,20 @@ public:
         with timestamps outside the specified region will be ignored.
     */
     inline void renderNextBlock (AudioBuffer<float>& outputAudio,
-                          const MidiBuffer& inputMidi,
-                          int startSample,
-                          int numSamples)
-        { processNextBlock (outputAudio, inputMidi, startSample, numSamples); }
+                                 const MidiBuffer& inputMidi,
+                                 int startSample,
+                                 int numSamples)
+    {
+        processNextBlock (outputAudio, inputMidi, startSample, numSamples);
+    }
 
     inline void renderNextBlock (AudioBuffer<double>& outputAudio,
-                          const MidiBuffer& inputMidi,
-                          int startSample,
-                          int numSamples)
-        { processNextBlock (outputAudio, inputMidi, startSample, numSamples); }
+                                 const MidiBuffer& inputMidi,
+                                 int startSample,
+                                 int numSamples)
+    {
+        processNextBlock (outputAudio, inputMidi, startSample, numSamples);
+    }
 
     /** Returns the current target sample rate at which rendering is being done.
         Subclasses may need to know this so that they can pitch things correctly.
@@ -539,8 +553,14 @@ public:
         The default setting is 32, which means that midi messages are accurate to about < 1ms
         accuracy, which is probably fine for most purposes, but you may want to increase or
         decrease this value for your synth.
+
+        If shouldBeStrict is true, the audio sub-blocks will strictly never be smaller than numSamples.
+
+        If shouldBeStrict is false (default), the first audio sub-block in the buffer is allowed
+        to be smaller, to make sure that the first MIDI event in a buffer will always be sample-accurate
+        (this can sometimes help to avoid quantisation or phasing issues).
     */
-    void setMinimumRenderingSubdivisionSize (int numSamples) noexcept;
+    void setMinimumRenderingSubdivisionSize (int numSamples, bool shouldBeStrict = false) noexcept;
 
 protected:
     //==============================================================================
@@ -612,10 +632,11 @@ private:
                            int startSample,
                            int numSamples);
     //==============================================================================
-    double sampleRate;
-    uint32 lastNoteOnCounter;
-    int minimumSubBlockSize;
-    bool shouldStealNotes;
+    double sampleRate = 0;
+    uint32 lastNoteOnCounter = 0;
+    int minimumSubBlockSize = 32;
+    bool subBlockSubdivisionIsStrict = false;
+    bool shouldStealNotes = true;
     BigInteger sustainPedalsDown;
 
    #if JUCE_CATCH_DEPRECATED_CODE_MISUSE
@@ -629,5 +650,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Synthesiser)
 };
 
-
-#endif   // JUCE_SYNTHESISER_H_INCLUDED
+} // namespace juce
