@@ -2,28 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-} // (juce namespace)
-
 extern "C"
 {
     // Declare just the minimum number of interfaces for the DSound objects that we need..
@@ -270,6 +266,10 @@ public:
         pDirectSound = nullptr;
         pOutputBuffer = nullptr;
         writeOffset = 0;
+        xruns = 0;
+
+        firstPlayTime = true;
+        lastPlayTime = 0;
 
         String error;
         HRESULT hr = E_NOINTERFACE;
@@ -280,6 +280,7 @@ public:
         if (SUCCEEDED (hr))
         {
             bytesPerBuffer = (bufferSizeSamples * (bitDepth >> 2)) & ~15;
+            ticksPerBuffer = bytesPerBuffer * Time::getHighResolutionTicksPerSecond() / (sampleRate * (bitDepth >> 2));
             totalBytesPerBuffer = (blocksPerOverallBuffer * bytesPerBuffer) & ~15;
             const int numChannels = 2;
 
@@ -352,7 +353,7 @@ public:
                                         hr = pOutputBuffer->Play (0, 0, 1 /* DSBPLAY_LOOPING */);
 
                                         if (SUCCEEDED (hr))
-                                            return String();
+                                            return {};
                                     }
                                 }
                             }
@@ -401,6 +402,18 @@ public:
             return true;
         }
 
+        auto currentPlayTime = Time::getHighResolutionTicks();
+        if (! firstPlayTime)
+        {
+            auto expectedBuffers = (currentPlayTime - lastPlayTime) / ticksPerBuffer;
+
+            playCursor += static_cast<DWORD> (expectedBuffers * bytesPerBuffer);
+        }
+        else
+            firstPlayTime = false;
+
+        lastPlayTime = currentPlayTime;
+
         int playWriteGap = (int) (writeCursor - playCursor);
         if (playWriteGap < 0)
             playWriteGap += totalBytesPerBuffer;
@@ -413,6 +426,9 @@ public:
         {
             writeOffset = writeCursor;
             bytesEmpty = totalBytesPerBuffer - playWriteGap;
+
+            // buffer underflow
+            xruns++;
         }
 
         if (bytesEmpty >= bytesPerBuffer)
@@ -484,7 +500,7 @@ public:
         }
     }
 
-    int bitDepth;
+    int bitDepth, xruns;
     bool doneFlag;
 
 private:
@@ -500,6 +516,9 @@ private:
     int totalBytesPerBuffer, bytesPerBuffer;
     unsigned int lastPlayCursor;
 
+    bool firstPlayTime;
+    int64 lastPlayTime, ticksPerBuffer;
+
     static inline int convertInputValues (const float l, const float r) noexcept
     {
         return jlimit (-32768, 32767, roundToInt (32767.0f * r)) << 16
@@ -515,9 +534,8 @@ struct DSoundInternalInChannel
 public:
     DSoundInternalInChannel (const String& name_, const GUID& guid_, int rate,
                              int bufferSize, float* left, float* right)
-        : bitDepth (16), name (name_), guid (guid_), sampleRate (rate),
-          bufferSizeSamples (bufferSize), leftBuffer (left), rightBuffer (right),
-          pDirectSound (nullptr), pDirectSoundCapture (nullptr), pInputBuffer (nullptr)
+        : name (name_), guid (guid_), sampleRate (rate),
+          bufferSizeSamples (bufferSize), leftBuffer (left), rightBuffer (right)
     {
     }
 
@@ -595,7 +613,7 @@ public:
                 hr = pInputBuffer->Start (1 /* DSCBSTART_LOOPING */);
 
                 if (SUCCEEDED (hr))
-                    return String();
+                    return {};
             }
         }
 
@@ -628,6 +646,7 @@ public:
             return true;
 
         int bytesFilled = (int) (readPos - readOffset);
+
         if (bytesFilled < 0)
             bytesFilled += totalBytesPerBuffer;
 
@@ -696,7 +715,7 @@ public:
 
     unsigned int readOffset;
     int bytesPerBuffer, totalBytesPerBuffer;
-    int bitDepth;
+    int bitDepth = 16;
     bool doneFlag;
 
 private:
@@ -706,9 +725,9 @@ private:
     float* leftBuffer;
     float* rightBuffer;
 
-    IDirectSound* pDirectSound;
-    IDirectSoundCapture* pDirectSoundCapture;
-    IDirectSoundCaptureBuffer* pInputBuffer;
+    IDirectSound* pDirectSound = nullptr;
+    IDirectSoundCapture* pDirectSoundCapture = nullptr;
+    IDirectSoundCaptureBuffer* pInputBuffer = nullptr;
 
     JUCE_DECLARE_NON_COPYABLE (DSoundInternalInChannel)
 };
@@ -722,14 +741,9 @@ public:
                          const int outputDeviceIndex_,
                          const int inputDeviceIndex_)
         : AudioIODevice (deviceName, "DirectSound"),
-          Thread ("Juce DSound"),
+          Thread ("JUCE DSound"),
           outputDeviceIndex (outputDeviceIndex_),
-          inputDeviceIndex (inputDeviceIndex_),
-          isOpen_ (false),
-          isStarted (false),
-          bufferSizeSamples (0),
-          sampleRate (0.0),
-          callback (nullptr)
+          inputDeviceIndex (inputDeviceIndex_)
     {
         if (outputDeviceIndex_ >= 0)
         {
@@ -782,8 +796,7 @@ public:
 
     Array<double> getAvailableSampleRates() override
     {
-        static const double rates[] = { 44100.0, 48000.0, 88200.0, 96000.0 };
-        return Array<double> (rates, numElementsInArray (rates));
+        return { 44100.0, 48000.0, 88200.0, 96000.0 };
     }
 
     Array<int> getAvailableBufferSizes() override
@@ -843,7 +856,7 @@ public:
     {
         if (isStarted)
         {
-            AudioIODeviceCallback* const callbackLocal = callback;
+            auto* callbackLocal = callback;
 
             {
                 const ScopedLock sl (startStopLock);
@@ -858,25 +871,30 @@ public:
     bool isPlaying() override            { return isStarted && isOpen_ && isThreadRunning(); }
     String getLastError() override       { return lastError; }
 
+    int getXRunCount() const noexcept override
+    {
+        return outChans[0] != nullptr ? outChans[0]->xruns : -1;
+    }
+
     //==============================================================================
     StringArray inChannels, outChannels;
     int outputDeviceIndex, inputDeviceIndex;
 
 private:
-    bool isOpen_;
-    bool isStarted;
+    bool isOpen_ = false;
+    bool isStarted = false;
     String lastError;
 
     OwnedArray<DSoundInternalInChannel> inChans;
     OwnedArray<DSoundInternalOutChannel> outChans;
     WaitableEvent startEvent;
 
-    int bufferSizeSamples;
-    double sampleRate;
+    int bufferSizeSamples = 0;
+    double sampleRate = 0;
     BigInteger enabledInputs, enabledOutputs;
-    AudioSampleBuffer inputBuffers, outputBuffers;
+    AudioBuffer<float> inputBuffers, outputBuffers;
 
-    AudioIODeviceCallback* callback;
+    AudioIODeviceCallback* callback = nullptr;
     CriticalSection startStopLock;
 
     String openDevice (const BigInteger& inputChannels,
@@ -890,8 +908,6 @@ private:
 
         inChans.clear();
         outChans.clear();
-        inputBuffers.setSize (1, 1);
-        outputBuffers.setSize (1, 1);
     }
 
     void resync()
@@ -1093,7 +1109,7 @@ String DSoundAudioIODevice::openDevice (const BigInteger& inputChannels,
                             enabledInputs.getHighestBit() + 1 - inChannels.size(),
                             false);
 
-    inputBuffers.setSize (jmax (1, enabledInputs.countNumberOfSetBits()), bufferSizeSamples);
+    inputBuffers.setSize (enabledInputs.countNumberOfSetBits(), bufferSizeSamples);
     inputBuffers.clear();
     int numIns = 0;
 
@@ -1114,7 +1130,7 @@ String DSoundAudioIODevice::openDevice (const BigInteger& inputChannels,
                              enabledOutputs.getHighestBit() + 1 - outChannels.size(),
                              false);
 
-    outputBuffers.setSize (jmax (1, enabledOutputs.countNumberOfSetBits()), bufferSizeSamples);
+    outputBuffers.setSize (enabledOutputs.countNumberOfSetBits(), bufferSizeSamples);
     outputBuffers.clear();
     int numOuts = 0;
 
@@ -1194,8 +1210,7 @@ class DSoundAudioIODeviceType  : public AudioIODeviceType,
 public:
     DSoundAudioIODeviceType()
         : AudioIODeviceType ("DirectSound"),
-          DeviceChangeDetector (L"DirectSound"),
-          hasScanned (false)
+          DeviceChangeDetector (L"DirectSound")
     {
         initialiseDSoundFunctions();
     }
@@ -1251,7 +1266,7 @@ public:
 
 private:
     DSoundDeviceList deviceList;
-    bool hasScanned;
+    bool hasScanned = false;
 
     void systemDeviceChanged() override
     {
@@ -1273,3 +1288,5 @@ AudioIODeviceType* AudioIODeviceType::createAudioIODeviceType_DirectSound()
 {
     return new DSoundAudioIODeviceType();
 }
+
+} // namespace juce
