@@ -2,29 +2,26 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_AUDIOTRANSPORTSOURCE_H_INCLUDED
-#define JUCE_AUDIOTRANSPORTSOURCE_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -88,7 +85,9 @@ public:
         The next time the getNextAudioBlock() method is called, this
         is the time from which it'll read data.
 
-        @see getPosition
+        @param newPosition    the new playback position in seconds
+
+        @see getCurrentPosition
     */
     void setPosition (double newPosition);
 
@@ -158,23 +157,22 @@ public:
 
 private:
     //==============================================================================
-    PositionableAudioSource* source;
-    ResamplingAudioSource* resamplerSource;
-    BufferingAudioSource* bufferingSource;
-    PositionableAudioSource* positionableSource;
-    AudioSource* masterSource;
+    PositionableAudioSource* source = nullptr;
+    ResamplingAudioSource* resamplerSource = nullptr;
+    BufferingAudioSource* bufferingSource = nullptr;
+    PositionableAudioSource* positionableSource = nullptr;
+    AudioSource* masterSource = nullptr;
 
     CriticalSection callbackLock;
-    float volatile gain, lastGain;
-    bool volatile playing, stopped;
-    double sampleRate, sourceSampleRate;
-    int blockSize, readAheadBufferSize;
-    bool volatile isPrepared, inputStreamEOF;
+    float volatile gain = 1.0f, lastGain = 1.0f;
+    bool volatile playing = false, stopped = true;
+    double sampleRate = 44100.0, sourceSampleRate = 0;
+    int blockSize = 128, readAheadBufferSize = 0;
+    bool volatile isPrepared = false, inputStreamEOF = false;
 
     void releaseMasterResources();
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioTransportSource)
 };
 
-
-#endif   // JUCE_AUDIOTRANSPORTSOURCE_H_INCLUDED
+} // namespace juce
