@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 static const char* const aiffFormatName = "AIFF file";
 
 //==============================================================================
@@ -99,7 +104,7 @@ namespace AiffFileHelpers
             if (values.getAllKeys().contains ("MidiUnityNote", true))
             {
                 block.setSize ((sizeof (InstChunk) + 3) & ~(size_t) 3, true);
-                InstChunk& inst = *static_cast<InstChunk*> (block.getData());
+                auto& inst = *static_cast<InstChunk*> (block.getData());
 
                 inst.baseNote      = getValue8 (values, "MidiUnityNote", "60");
                 inst.detune        = getValue8 (values, "Detune", "0");
@@ -163,7 +168,7 @@ namespace AiffFileHelpers
 
             switch (key)
             {
-                case minor:     keyString = "major";        break;
+                case minor:     keyString = "minor";        break;
                 case major:     keyString = "major";        break;
                 case neither:   keyString = "neither";      break;
                 case both:      keyString = "both";         break;
@@ -197,9 +202,9 @@ namespace AiffFileHelpers
     {
         static bool isValidTag (const char* d) noexcept
         {
-            return CharacterFunctions::isLetterOrDigit (d[0]) && CharacterFunctions::isUpperCase (d[0])
-                && CharacterFunctions::isLetterOrDigit (d[1]) && CharacterFunctions::isLowerCase (d[1])
-                && CharacterFunctions::isLetterOrDigit (d[2]) && CharacterFunctions::isLowerCase (d[2]);
+            return CharacterFunctions::isLetterOrDigit (d[0]) && CharacterFunctions::isUpperCase (static_cast<juce_wchar> (d[0]))
+                && CharacterFunctions::isLetterOrDigit (d[1]) && CharacterFunctions::isLowerCase (static_cast<juce_wchar> (d[1]))
+                && CharacterFunctions::isLetterOrDigit (d[2]) && CharacterFunctions::isLowerCase (static_cast<juce_wchar> (d[2]));
         }
 
         static bool isAppleGenre (const String& tag) noexcept
@@ -233,8 +238,8 @@ namespace AiffFileHelpers
 
             StringArray tagsArray;
 
-            const char* data = static_cast<const char*> (mb.getData());
-            const char* dataEnd = data + mb.getSize();
+            auto* data = static_cast<const char*> (mb.getData());
+            auto* dataEnd = data + mb.getSize();
 
             while (data < dataEnd)
             {
@@ -242,7 +247,7 @@ namespace AiffFileHelpers
 
                 if (isValidTag (data))
                 {
-                    const String tag = String (CharPointer_UTF8 (data), CharPointer_UTF8 (dataEnd));
+                    auto tag = String (CharPointer_UTF8 (data), CharPointer_UTF8 (dataEnd));
                     isGenre = isAppleGenre (tag);
                     tagsArray.add (tag);
                 }
@@ -271,22 +276,14 @@ namespace AiffFileHelpers
             const String noteString ("CueNote");
             const String identifierString ("Identifier");
 
-            const StringArray& keys = values.getAllKeys();
-
-            for (int i = 0; i < keys.size(); ++i)
+            for (auto& key : values.getAllKeys())
             {
-                const String key (keys[i]);
-
                 if (key.startsWith (noteString))
                     continue; // zero identifier IS valid in a COMT chunk
 
                 if (key.startsWith (cueString) && key.contains (identifierString))
-                {
-                    const int value = values.getValue (key, "-1").getIntValue();
-
-                    if (value == 0)
+                    if (values.getValue (key, "-1").getIntValue() == 0)
                         return true;
-                }
             }
 
             return false;
@@ -294,16 +291,15 @@ namespace AiffFileHelpers
 
         static void create (MemoryBlock& block, const StringPairArray& values)
         {
-            const int numCues = values.getValue ("NumCuePoints", "0").getIntValue();
+            auto numCues = values.getValue ("NumCuePoints", "0").getIntValue();
 
             if (numCues > 0)
             {
                 MemoryOutputStream out (block, false);
-
                 out.writeShortBigEndian ((short) numCues);
 
-                const int numCueLabels = values.getValue ("NumCueLabels", "0").getIntValue();
-                const int idOffset = metaDataContainsZeroIdentifiers (values) ? 1 : 0; // can't have zero IDs in AIFF
+                auto numCueLabels = values.getValue ("NumCueLabels", "0").getIntValue();
+                auto idOffset = metaDataContainsZeroIdentifiers (values) ? 1 : 0; // can't have zero IDs in AIFF
 
                #if JUCE_DEBUG
                 Array<int> identifiers;
@@ -311,21 +307,21 @@ namespace AiffFileHelpers
 
                 for (int i = 0; i < numCues; ++i)
                 {
-                    const String prefixCue ("Cue" + String (i));
-                    const int identifier = idOffset + values.getValue (prefixCue + "Identifier", "1").getIntValue();
+                    auto prefixCue = "Cue" + String (i);
+                    auto identifier = idOffset + values.getValue (prefixCue + "Identifier", "1").getIntValue();
 
                    #if JUCE_DEBUG
                     jassert (! identifiers.contains (identifier));
                     identifiers.add (identifier);
                    #endif
 
-                    const int offset = values.getValue (prefixCue + "Offset", "0").getIntValue();
-                    String label ("CueLabel" + String (i));
+                    auto offset = values.getValue (prefixCue + "Offset", "0").getIntValue();
+                    auto label = "CueLabel" + String (i);
 
                     for (int labelIndex = 0; labelIndex < numCueLabels; ++labelIndex)
                     {
-                        const String prefixLabel ("CueLabel" + String (labelIndex));
-                        const int labelIdentifier = idOffset + values.getValue (prefixLabel + "Identifier", "1").getIntValue();
+                        auto prefixLabel = "CueLabel" + String (labelIndex);
+                        auto labelIdentifier = idOffset + values.getValue (prefixLabel + "Identifier", "1").getIntValue();
 
                         if (labelIdentifier == identifier)
                         {
@@ -337,7 +333,7 @@ namespace AiffFileHelpers
                     out.writeShortBigEndian ((short) identifier);
                     out.writeIntBigEndian (offset);
 
-                    const size_t labelLength = jmin ((size_t) 254, label.getNumBytesAsUTF8()); // seems to need null terminator even though it's a pstring
+                    auto labelLength = jmin ((size_t) 254, label.getNumBytesAsUTF8()); // seems to need null terminator even though it's a pstring
                     out.writeByte ((char) labelLength + 1);
                     out.write (label.toUTF8(), labelLength);
                     out.writeByte (0);
@@ -354,7 +350,7 @@ namespace AiffFileHelpers
     {
         static void create (MemoryBlock& block, const StringPairArray& values)
         {
-            const int numNotes = values.getValue ("NumCueNotes", "0").getIntValue();
+            auto numNotes = values.getValue ("NumCueNotes", "0").getIntValue();
 
             if (numNotes > 0)
             {
@@ -363,14 +359,14 @@ namespace AiffFileHelpers
 
                 for (int i = 0; i < numNotes; ++i)
                 {
-                    const String prefix ("CueNote" + String (i));
+                    auto prefix = "CueNote" + String (i);
 
                     out.writeIntBigEndian (values.getValue (prefix + "TimeStamp", "0").getIntValue());
                     out.writeShortBigEndian ((short) values.getValue (prefix + "Identifier", "0").getIntValue());
 
-                    const String comment (values.getValue (prefix + "Text", String()));
+                    auto comment = values.getValue (prefix + "Text", String());
+                    auto commentLength = jmin (comment.getNumBytesAsUTF8(), (size_t) 65534);
 
-                    const size_t commentLength = jmin (comment.getNumBytesAsUTF8(), (size_t) 65534);
                     out.writeShortBigEndian ((short) commentLength + 1);
                     out.write (comment.toUTF8(), commentLength);
                     out.writeByte (0);
@@ -394,10 +390,10 @@ public:
 
         if (input->readInt() == chunkName ("FORM"))
         {
-            const int len = input->readIntBigEndian();
-            const int64 end = input->getPosition() + len;
+            auto len = input->readIntBigEndian();
+            auto end = input->getPosition() + len;
+            auto nextType = input->readInt();
 
-            const int nextType = input->readInt();
             if (nextType == chunkName ("AIFF") || nextType == chunkName ("AIFC"))
             {
                 bool hasGotVer = false;
@@ -406,15 +402,15 @@ public:
 
                 while (input->getPosition() < end)
                 {
-                    const int type = input->readInt();
-                    const uint32 length = (uint32) input->readIntBigEndian();
-                    const int64 chunkEnd = input->getPosition() + length;
+                    auto type = input->readInt();
+                    auto length = (uint32) input->readIntBigEndian();
+                    auto chunkEnd = input->getPosition() + length;
 
                     if (type == chunkName ("FVER"))
                     {
                         hasGotVer = true;
+                        auto ver = input->readIntBigEndian();
 
-                        const int ver = input->readIntBigEndian();
                         if (ver != 0 && ver != (int) 0xa2805140)
                             break;
                     }
@@ -436,7 +432,7 @@ public:
                              || (byte0 == 0x40 && sampleRateBytes[1] > 0x1C))
                             break;
 
-                        unsigned int sampRate = ByteOrder::bigEndianInt (sampleRateBytes + 2);
+                        auto sampRate = ByteOrder::bigEndianInt (sampleRateBytes + 2);
                         sampRate >>= (16414 - ByteOrder::bigEndianShort (sampleRateBytes));
                         sampleRate = (int) sampRate;
 
@@ -448,7 +444,7 @@ public:
                         }
                         else
                         {
-                            const int compType = input->readInt();
+                            auto compType = input->readInt();
 
                             if (compType == chunkName ("NONE") || compType == chunkName ("twos"))
                             {
@@ -474,13 +470,13 @@ public:
                     {
                         hasGotData = true;
 
-                        const int offset = input->readIntBigEndian();
+                        auto offset = input->readIntBigEndian();
                         dataChunkStart = input->getPosition() + 4 + offset;
                         lengthInSamples = (bytesPerFrame > 0) ? jmin (lengthInSamples, ((int64) length) / (int64) bytesPerFrame) : 0;
                     }
                     else if (type == chunkName ("MARK"))
                     {
-                        const uint16 numCues = (uint16) input->readShortBigEndian();
+                        auto numCues = (uint16) input->readShortBigEndian();
 
                         // these two are always the same for AIFF-read files
                         metadataValues.set ("NumCuePoints", String (numCues));
@@ -488,9 +484,9 @@ public:
 
                         for (uint16 i = 0; i < numCues; ++i)
                         {
-                            uint16 identifier = (uint16) input->readShortBigEndian();
-                            uint32 offset = (uint32) input->readIntBigEndian();
-                            uint8 stringLength = (uint8) input->readByte();
+                            auto identifier = (uint16) input->readShortBigEndian();
+                            auto offset = (uint32) input->readIntBigEndian();
+                            auto stringLength = (uint8) input->readByte();
                             MemoryBlock textBlock;
                             input->readIntoMemoryBlock (textBlock, stringLength);
 
@@ -500,30 +496,30 @@ public:
                             if ((stringLength & 1) == 0)
                                 input->readByte();
 
-                            const String prefixCue ("Cue" + String (i));
+                            auto prefixCue = "Cue" + String (i);
                             metadataValues.set (prefixCue + "Identifier", String (identifier));
                             metadataValues.set (prefixCue + "Offset", String (offset));
 
-                            const String prefixLabel ("CueLabel" + String (i));
+                            auto prefixLabel = "CueLabel" + String (i);
                             metadataValues.set (prefixLabel + "Identifier", String (identifier));
                             metadataValues.set (prefixLabel + "Text", textBlock.toString());
                         }
                     }
                     else if (type == chunkName ("COMT"))
                     {
-                        const uint16 numNotes = (uint16) input->readShortBigEndian();
+                        auto numNotes = (uint16) input->readShortBigEndian();
                         metadataValues.set ("NumCueNotes", String (numNotes));
 
                         for (uint16 i = 0; i < numNotes; ++i)
                         {
-                            uint32 timestamp = (uint32) input->readIntBigEndian();
-                            uint16 identifier = (uint16) input->readShortBigEndian(); // may be zero in this case
-                            uint16 stringLength = (uint16) input->readShortBigEndian();
+                            auto timestamp = (uint32) input->readIntBigEndian();
+                            auto identifier = (uint16) input->readShortBigEndian(); // may be zero in this case
+                            auto stringLength = (uint16) input->readShortBigEndian();
 
                             MemoryBlock textBlock;
                             input->readIntoMemoryBlock (textBlock, stringLength + (stringLength & 1));
 
-                            const String prefix ("CueNote" + String (i));
+                            auto prefix = "CueNote" + String (i);
                             metadataValues.set (prefix + "TimeStamp", String (timestamp));
                             metadataValues.set (prefix + "Identifier", String (identifier));
                             metadataValues.set (prefix + "Text", textBlock.toString());
@@ -604,7 +600,7 @@ public:
     }
 
     template <typename Endianness>
-    static void copySampleData (unsigned int bitsPerSample, const bool usesFloatingPointData,
+    static void copySampleData (unsigned int bitsPerSample, bool usesFloatingPointData,
                                 int* const* destSamples, int startOffsetInDestBuffer, int numDestChannels,
                                 const void* sourceData, int numChannels, int numSamples) noexcept
     {
@@ -614,7 +610,8 @@ public:
             case 16:    ReadHelper<AudioData::Int32, AudioData::Int16, Endianness>::read (destSamples, startOffsetInDestBuffer, numDestChannels, sourceData, numChannels, numSamples); break;
             case 24:    ReadHelper<AudioData::Int32, AudioData::Int24, Endianness>::read (destSamples, startOffsetInDestBuffer, numDestChannels, sourceData, numChannels, numSamples); break;
             case 32:    if (usesFloatingPointData) ReadHelper<AudioData::Float32, AudioData::Float32, Endianness>::read (destSamples, startOffsetInDestBuffer, numDestChannels, sourceData, numChannels, numSamples);
-                        else                       ReadHelper<AudioData::Int32,   AudioData::Int32,   Endianness>::read (destSamples, startOffsetInDestBuffer, numDestChannels, sourceData, numChannels, numSamples); break;
+                        else                       ReadHelper<AudioData::Int32,   AudioData::Int32,   Endianness>::read (destSamples, startOffsetInDestBuffer, numDestChannels, sourceData, numChannels, numSamples);
+                        break;
             default:    jassertfalse; break;
         }
     }
@@ -634,10 +631,7 @@ public:
     AiffAudioFormatWriter (OutputStream* out, double rate,
                            unsigned int numChans, unsigned int bits,
                            const StringPairArray& metadataValues)
-        : AudioFormatWriter (out, aiffFormatName, rate, numChans, bits),
-          lengthInSamples (0),
-          bytesWritten (0),
-          writeFailed (false)
+        : AudioFormatWriter (out, aiffFormatName, rate, numChans, bits)
     {
         using namespace AiffFileHelpers;
 
@@ -674,7 +668,7 @@ public:
         if (writeFailed)
             return false;
 
-        const size_t bytes = numChannels * (size_t) numSamples * bitsPerSample / 8;
+        auto bytes = numChannels * (size_t) numSamples * bitsPerSample / 8;
         tempBlock.ensureSize (bytes, false);
 
         switch (bitsPerSample)
@@ -704,9 +698,9 @@ public:
 
 private:
     MemoryBlock tempBlock, markChunk, comtChunk, instChunk;
-    uint64 lengthInSamples, bytesWritten;
-    int64 headerPosition;
-    bool writeFailed;
+    uint64 lengthInSamples = 0, bytesWritten = 0;
+    int64 headerPosition = 0;
+    bool writeFailed = false;
 
     void writeHeader()
     {
@@ -719,10 +713,10 @@ private:
         // to be able to seek back to write the header
         jassert (couldSeekOk);
 
-        const int headerLen = (int) (54 + (markChunk.getSize() > 0 ? markChunk.getSize() + 8 : 0)
-                                        + (comtChunk.getSize() > 0 ? comtChunk.getSize() + 8 : 0)
-                                        + (instChunk.getSize() > 0 ? instChunk.getSize() + 8 : 0));
-        int audioBytes = (int) (lengthInSamples * ((bitsPerSample * numChannels) / 8));
+        auto headerLen = (int) (54 + (markChunk.getSize() > 0 ? markChunk.getSize() + 8 : 0)
+                                   + (comtChunk.getSize() > 0 ? comtChunk.getSize() + 8 : 0)
+                                   + (instChunk.getSize() > 0 ? instChunk.getSize() + 8 : 0));
+        auto audioBytes = (int) (lengthInSamples * ((bitsPerSample * numChannels) / 8));
         audioBytes += (audioBytes & 1);
 
         output->writeInt (chunkName ("FORM"));
@@ -734,7 +728,7 @@ private:
         output->writeIntBigEndian ((int) lengthInSamples);
         output->writeShortBigEndian ((short) bitsPerSample);
 
-        uint8 sampleRateBytes[10] = { 0 };
+        uint8 sampleRateBytes[10] = {};
 
         if (sampleRate <= 1)
         {
@@ -755,8 +749,8 @@ private:
             else
             {
                 int n = (int) sampleRate;
-
                 int i;
+
                 for (i = 0; i <= 32 ; ++i)
                 {
                     if ((n & mask) != 0)
@@ -846,7 +840,7 @@ public:
 
     void getSample (int64 sample, float* result) const noexcept override
     {
-        const int num = (int) numChannels;
+        auto num = (int) numChannels;
 
         if (map == nullptr || ! mappedSection.contains (sample))
         {
@@ -867,7 +861,8 @@ public:
                 case 16:    ReadHelper<AudioData::Float32, AudioData::Int16, AudioData::LittleEndian>::read (dest, 0, 1, source, 1, num); break;
                 case 24:    ReadHelper<AudioData::Float32, AudioData::Int24, AudioData::LittleEndian>::read (dest, 0, 1, source, 1, num); break;
                 case 32:    if (usesFloatingPointData) ReadHelper<AudioData::Float32, AudioData::Float32, AudioData::LittleEndian>::read (dest, 0, 1, source, 1, num);
-                            else                       ReadHelper<AudioData::Float32, AudioData::Int32,   AudioData::LittleEndian>::read (dest, 0, 1, source, 1, num); break;
+                            else                       ReadHelper<AudioData::Float32, AudioData::Int32,   AudioData::LittleEndian>::read (dest, 0, 1, source, 1, num);
+                            break;
                 default:    jassertfalse; break;
             }
         }
@@ -879,7 +874,8 @@ public:
                 case 16:    ReadHelper<AudioData::Float32, AudioData::Int16, AudioData::BigEndian>::read (dest, 0, 1, source, 1, num); break;
                 case 24:    ReadHelper<AudioData::Float32, AudioData::Int24, AudioData::BigEndian>::read (dest, 0, 1, source, 1, num); break;
                 case 32:    if (usesFloatingPointData) ReadHelper<AudioData::Float32, AudioData::Float32, AudioData::BigEndian>::read (dest, 0, 1, source, 1, num);
-                            else                       ReadHelper<AudioData::Float32, AudioData::Int32,   AudioData::BigEndian>::read (dest, 0, 1, source, 1, num); break;
+                            else                       ReadHelper<AudioData::Float32, AudioData::Int32,   AudioData::BigEndian>::read (dest, 0, 1, source, 1, num);
+                            break;
                 default:    jassertfalse; break;
             }
         }
@@ -905,7 +901,8 @@ public:
             case 16:    scanMinAndMax<AudioData::Int16> (startSampleInFile, numSamples, results, numChannelsToRead); break;
             case 24:    scanMinAndMax<AudioData::Int24> (startSampleInFile, numSamples, results, numChannelsToRead); break;
             case 32:    if (usesFloatingPointData) scanMinAndMax<AudioData::Float32> (startSampleInFile, numSamples, results, numChannelsToRead);
-                        else                       scanMinAndMax<AudioData::Int32>   (startSampleInFile, numSamples, results, numChannelsToRead); break;
+                        else                       scanMinAndMax<AudioData::Int32>   (startSampleInFile, numSamples, results, numChannelsToRead);
+                        break;
             default:    jassertfalse; break;
         }
     }
@@ -931,24 +928,17 @@ private:
 };
 
 //==============================================================================
-AiffAudioFormat::AiffAudioFormat()   : AudioFormat (aiffFormatName, ".aiff .aif")
-{
-}
-
-AiffAudioFormat::~AiffAudioFormat()
-{
-}
+AiffAudioFormat::AiffAudioFormat()  : AudioFormat (aiffFormatName, ".aiff .aif") {}
+AiffAudioFormat::~AiffAudioFormat() {}
 
 Array<int> AiffAudioFormat::getPossibleSampleRates()
 {
-    const int rates[] = { 22050, 32000, 44100, 48000, 88200, 96000, 176400, 192000, 0 };
-    return Array<int> (rates);
+    return { 22050, 32000, 44100, 48000, 88200, 96000, 176400, 192000 };
 }
 
 Array<int> AiffAudioFormat::getPossibleBitDepths()
 {
-    const int depths[] = { 8, 16, 24, 0 };
-    return Array<int> (depths);
+     return { 8, 16, 24 };
 }
 
 bool AiffAudioFormat::canDoStereo() { return true; }
@@ -960,7 +950,7 @@ bool AiffAudioFormat::canHandleFile (const File& f)
     if (AudioFormat::canHandleFile (f))
         return true;
 
-    const OSType type = f.getMacOSType();
+    auto type = f.getMacOSType();
 
     // (NB: written as hex to avoid four-char-constant warnings)
     return type == 0x41494646 /* AIFF */ || type == 0x41494643 /* AIFC */
@@ -968,7 +958,7 @@ bool AiffAudioFormat::canHandleFile (const File& f)
 }
 #endif
 
-AudioFormatReader* AiffAudioFormat::createReaderFor (InputStream* sourceStream, const bool deleteStreamIfOpeningFails)
+AudioFormatReader* AiffAudioFormat::createReaderFor (InputStream* sourceStream, bool deleteStreamIfOpeningFails)
 {
     ScopedPointer<AiffAudioFormatReader> w (new AiffAudioFormatReader (sourceStream));
 
@@ -983,12 +973,17 @@ AudioFormatReader* AiffAudioFormat::createReaderFor (InputStream* sourceStream,
 
 MemoryMappedAudioFormatReader* AiffAudioFormat::createMemoryMappedReader (const File& file)
 {
-    if (FileInputStream* fin = file.createInputStream())
+    return createMemoryMappedReader (file.createInputStream());
+}
+
+MemoryMappedAudioFormatReader* AiffAudioFormat::createMemoryMappedReader (FileInputStream* fin)
+{
+    if (fin != nullptr)
     {
         AiffAudioFormatReader reader (fin);
 
         if (reader.lengthInSamples > 0)
-            return new MemoryMappedAiffReader (file, reader);
+            return new MemoryMappedAiffReader (fin->getFile(), reader);
     }
 
     return nullptr;
@@ -1002,7 +997,10 @@ AudioFormatWriter* AiffAudioFormat::createWriterFor (OutputStream* out,
                                                      int /*qualityOptionIndex*/)
 {
     if (out != nullptr && getPossibleBitDepths().contains (bitsPerSample))
-        return new AiffAudioFormatWriter (out, sampleRate, numberOfChannels, (unsigned int) bitsPerSample, metadataValues);
+        return new AiffAudioFormatWriter (out, sampleRate, numberOfChannels,
+                                          (unsigned int) bitsPerSample, metadataValues);
 
     return nullptr;
 }
+
+} // namespace juce
