@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 #if JUCE_USE_OGGVORBIS
 
 #if JUCE_MAC && ! defined (__MACOSX__)
@@ -34,13 +39,14 @@ namespace OggVorbisNamespace
  #if JUCE_MSVC
   #pragma warning (push)
   #pragma warning (disable: 4267 4127 4244 4996 4100 4701 4702 4013 4133 4206 4305 4189 4706 4995 4365 4456 4457 4459)
- #endif
-
- #if JUCE_CLANG
+ #elif JUCE_CLANG
   #pragma clang diagnostic push
   #pragma clang diagnostic ignored "-Wconversion"
   #pragma clang diagnostic ignored "-Wshadow"
   #pragma clang diagnostic ignored "-Wdeprecated-register"
+ #elif JUCE_GCC
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wshadow"
  #endif
 
  #include "oggvorbis/vorbisenc.h"
@@ -73,10 +79,10 @@ namespace OggVorbisNamespace
 
  #if JUCE_MSVC
   #pragma warning (pop)
- #endif
-
- #if JUCE_CLANG
+ #elif JUCE_CLANG
   #pragma clang diagnostic pop
+ #elif JUCE_GCC
+  #pragma GCC diagnostic pop
  #endif
 #else
  #include <vorbis/vorbisenc.h>
@@ -105,12 +111,8 @@ const char* const OggVorbisAudioFormat::id3trackNumber = "id3trackNumber";
 class OggReader : public AudioFormatReader
 {
 public:
-    OggReader (InputStream* const inp)
-        : AudioFormatReader (inp, oggFormatName),
-          reservoirStart (0),
-          samplesInReservoir (0)
+    OggReader (InputStream* inp)  : AudioFormatReader (inp, oggFormatName)
     {
-        using namespace OggVorbisNamespace;
         sampleRate = 0;
         usesFloatingPointData = true;
 
@@ -119,13 +121,13 @@ public:
         callbacks.close_func = &oggCloseCallback;
         callbacks.tell_func  = &oggTellCallback;
 
-        const int err = ov_open_callbacks (input, &ovFile, 0, 0, callbacks);
+        auto err = ov_open_callbacks (input, &ovFile, 0, 0, callbacks);
 
         if (err == 0)
         {
-            vorbis_info* info = ov_info (&ovFile, -1);
+            auto* info = ov_info (&ovFile, -1);
 
-            vorbis_comment* const comment = ov_comment (&ovFile, -1);
+            auto* comment = ov_comment (&ovFile, -1);
             addMetadataItem (comment, "ENCODER",     OggVorbisAudioFormat::encoderName);
             addMetadataItem (comment, "TITLE",       OggVorbisAudioFormat::id3title);
             addMetadataItem (comment, "ARTIST",      OggVorbisAudioFormat::id3artist);
@@ -146,12 +148,12 @@ public:
 
     ~OggReader()
     {
-        OggVorbisNamespace::ov_clear (&ovFile);
+        ov_clear (&ovFile);
     }
 
     void addMetadataItem (OggVorbisNamespace::vorbis_comment* comment, const char* name, const char* metadataName)
     {
-        if (const char* value = vorbis_comment_query (comment, name, 0))
+        if (auto* value = vorbis_comment_query (comment, name, 0))
             metadataValues.set (metadataName, value);
     }
 
@@ -161,13 +163,13 @@ public:
     {
         while (numSamples > 0)
         {
-            const int numAvailable = (int) (reservoirStart + samplesInReservoir - startSampleInFile);
+            auto numAvailable = (int) (reservoirStart + samplesInReservoir - startSampleInFile);
 
             if (startSampleInFile >= reservoirStart && numAvailable > 0)
             {
                 // got a few samples overlapping, so use them before seeking..
 
-                const int numToUse = jmin (numSamples, numAvailable);
+                auto numToUse = jmin (numSamples, numAvailable);
 
                 for (int i = jmin (numDestChannels, reservoir.getNumChannels()); --i >= 0;)
                     if (destSamples[i] != nullptr)
@@ -187,22 +189,21 @@ public:
                 || startSampleInFile + numSamples > reservoirStart + samplesInReservoir)
             {
                 // buffer miss, so refill the reservoir
-                int bitStream = 0;
-
                 reservoirStart = jmax (0, (int) startSampleInFile);
                 samplesInReservoir = reservoir.getNumSamples();
 
-                if (reservoirStart != (int) OggVorbisNamespace::ov_pcm_tell (&ovFile))
-                    OggVorbisNamespace::ov_pcm_seek (&ovFile, reservoirStart);
+                if (reservoirStart != (int) ov_pcm_tell (&ovFile))
+                    ov_pcm_seek (&ovFile, reservoirStart);
 
+                int bitStream = 0;
                 int offset = 0;
                 int numToRead = samplesInReservoir;
 
                 while (numToRead > 0)
                 {
                     float** dataIn = nullptr;
+                    auto samps = ov_read_float (&ovFile, &dataIn, numToRead, &bitStream);
 
-                    const long samps = OggVorbisNamespace::ov_read_float (&ovFile, &dataIn, numToRead, &bitStream);
                     if (samps <= 0)
                         break;
 
@@ -238,7 +239,7 @@ public:
 
     static int oggSeekCallback (void* datasource, OggVorbisNamespace::ogg_int64_t offset, int whence)
     {
-        InputStream* const in = static_cast<InputStream*> (datasource);
+        auto* in = static_cast<InputStream*> (datasource);
 
         if (whence == SEEK_CUR)
             offset += in->getPosition();
@@ -262,8 +263,8 @@ public:
 private:
     OggVorbisNamespace::OggVorbis_File ovFile;
     OggVorbisNamespace::ov_callbacks callbacks;
-    AudioSampleBuffer reservoir;
-    int reservoirStart, samplesInReservoir;
+    AudioBuffer<float> reservoir;
+    int reservoirStart = 0, samplesInReservoir = 0;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (OggReader)
 };
@@ -272,20 +273,14 @@ private:
 class OggWriter  : public AudioFormatWriter
 {
 public:
-    OggWriter (OutputStream* const out,
-               const double sampleRate_,
-               const unsigned int numChannels_,
-               const unsigned int bitsPerSample_,
-               const int qualityIndex,
-               const StringPairArray& metadata)
-        : AudioFormatWriter (out, oggFormatName, sampleRate_, numChannels_, bitsPerSample_),
-          ok (false)
+    OggWriter (OutputStream* out, double rate,
+               unsigned int numChans, unsigned int bitsPerSamp,
+               int qualityIndex, const StringPairArray& metadata)
+        : AudioFormatWriter (out, oggFormatName, rate, numChans, bitsPerSamp)
     {
-        using namespace OggVorbisNamespace;
-
         vorbis_info_init (&vi);
 
-        if (vorbis_encode_init_vbr (&vi, (int) numChannels_, (int) sampleRate_,
+        if (vorbis_encode_init_vbr (&vi, (int) numChans, (int) rate,
                                     jlimit (0.0f, 1.0f, qualityIndex * 0.1f)) == 0)
         {
             vorbis_comment_init (&vc);
@@ -304,10 +299,7 @@ public:
 
             ogg_stream_init (&os, Random::getSystemRandom().nextInt());
 
-            ogg_packet header;
-            ogg_packet header_comm;
-            ogg_packet header_code;
-
+            OggVorbisNamespace::ogg_packet header, header_comm, header_code;
             vorbis_analysis_headerout (&vd, &vc, &header, &header_comm, &header_code);
 
             ogg_stream_packetin (&os, &header);
@@ -329,7 +321,6 @@ public:
 
     ~OggWriter()
     {
-        using namespace OggVorbisNamespace;
         if (ok)
         {
             // write a zero-length packet to show ogg that we're finished..
@@ -356,8 +347,6 @@ public:
     {
         if (ok)
         {
-            using namespace OggVorbisNamespace;
-
             if (numSamples > 0)
             {
                 const double gain = 1.0 / 0x80000000u;
@@ -365,13 +354,13 @@ public:
 
                 for (int i = (int) numChannels; --i >= 0;)
                 {
-                    float* const dst = vorbisBuffer[i];
-                    const int* const src = samplesToWrite [i];
-
-                    if (src != nullptr && dst != nullptr)
+                    if (auto* dst = vorbisBuffer[i])
                     {
-                        for (int j = 0; j < numSamples; ++j)
-                            dst[j] = (float) (src[j] * gain);
+                        if (const int* src = samplesToWrite [i])
+                        {
+                            for (int j = 0; j < numSamples; ++j)
+                                dst[j] = (float) (src[j] * gain);
+                        }
                     }
                 }
             }
@@ -384,8 +373,6 @@ public:
 
     void writeSamples (int numSamples)
     {
-        using namespace OggVorbisNamespace;
-
         vorbis_analysis_wrote (&vd, numSamples);
 
         while (vorbis_analysis_blockout (&vd, &vb) == 1)
@@ -412,7 +399,7 @@ public:
         }
     }
 
-    bool ok;
+    bool ok = false;
 
 private:
     OggVorbisNamespace::ogg_stream_state os;
@@ -425,7 +412,7 @@ private:
 
     void addMetadata (const StringPairArray& metadata, const char* name, const char* vorbisName)
     {
-        const String s (metadata [name]);
+        auto s = metadata [name];
 
         if (s.isNotEmpty())
             vorbis_comment_add_tag (&vc, vorbisName, const_cast<char*> (s.toRawUTF8()));
@@ -446,24 +433,20 @@ OggVorbisAudioFormat::~OggVorbisAudioFormat()
 
 Array<int> OggVorbisAudioFormat::getPossibleSampleRates()
 {
-    const int rates[] = { 8000, 11025, 12000, 16000, 22050, 32000,
-                          44100, 48000, 88200, 96000, 176400, 192000 };
-
-    return Array<int> (rates, numElementsInArray (rates));
+    return { 8000, 11025, 12000, 16000, 22050, 32000,
+             44100, 48000, 88200, 96000, 176400, 192000 };
 }
 
 Array<int> OggVorbisAudioFormat::getPossibleBitDepths()
 {
-    const int depths[] = { 32 };
-
-    return Array<int> (depths, numElementsInArray (depths));
+    return { 32 };
 }
 
 bool OggVorbisAudioFormat::canDoStereo()    { return true; }
 bool OggVorbisAudioFormat::canDoMono()      { return true; }
 bool OggVorbisAudioFormat::isCompressed()   { return true; }
 
-AudioFormatReader* OggVorbisAudioFormat::createReaderFor (InputStream* in, const bool deleteStreamIfOpeningFails)
+AudioFormatReader* OggVorbisAudioFormat::createReaderFor (InputStream* in, bool deleteStreamIfOpeningFails)
 {
     ScopedPointer<OggReader> r (new OggReader (in));
 
@@ -495,29 +478,28 @@ AudioFormatWriter* OggVorbisAudioFormat::createWriterFor (OutputStream* out,
 
 StringArray OggVorbisAudioFormat::getQualityOptions()
 {
-    static const char* options[] = { "64 kbps", "80 kbps", "96 kbps", "112 kbps", "128 kbps", "160 kbps",
-                                     "192 kbps", "224 kbps", "256 kbps", "320 kbps", "500 kbps", 0 };
-    return StringArray (options);
+    return { "64 kbps", "80 kbps", "96 kbps", "112 kbps", "128 kbps", "160 kbps",
+             "192 kbps", "224 kbps", "256 kbps", "320 kbps", "500 kbps" };
 }
 
 int OggVorbisAudioFormat::estimateOggFileQuality (const File& source)
 {
-    if (FileInputStream* const in = source.createInputStream())
+    if (auto* in = source.createInputStream())
     {
         ScopedPointer<AudioFormatReader> r (createReaderFor (in, true));
 
         if (r != nullptr)
         {
-            const double lengthSecs = r->lengthInSamples / r->sampleRate;
-            const int approxBitsPerSecond = (int) (source.getSize() * 8 / lengthSecs);
+            auto lengthSecs = r->lengthInSamples / r->sampleRate;
+            auto approxBitsPerSecond = (int) (source.getSize() * 8 / lengthSecs);
 
-            const StringArray qualities (getQualityOptions());
+            auto qualities = getQualityOptions();
             int bestIndex = 0;
             int bestDiff = 10000;
 
             for (int i = qualities.size(); --i >= 0;)
             {
-                const int diff = std::abs (qualities[i].getIntValue() - approxBitsPerSecond);
+                auto diff = std::abs (qualities[i].getIntValue() - approxBitsPerSecond);
 
                 if (diff < bestDiff)
                 {
@@ -534,3 +516,5 @@ int OggVorbisAudioFormat::estimateOggFileQuality (const File& source)
 }
 
 #endif
+
+} // namespace juce
