@@ -2,34 +2,36 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+#if JucePlugin_Build_RTAS
+
 #include "../../juce_core/system/juce_TargetPlatform.h"
 #include "../utility/juce_CheckSettingMacros.h"
 
 // (these functions are in their own file because of problems including windows.h
 // at the same time as the Digi headers)
 
-#if JucePlugin_Build_RTAS
-
 #define _DO_NOT_DECLARE_INTERLOCKED_INTRINSICS_IN_MEMORY // (workaround for a VC build problem)
 
 #undef _WIN32_WINNT
@@ -43,6 +45,9 @@
 #include "../utility/juce_IncludeModuleHeaders.h"
 #pragma pack (pop)
 
+namespace juce
+{
+
 //==============================================================================
 void JUCE_CALLTYPE attachSubWindow (void* hostWindow,
                                     int& titleW, int& titleH,
@@ -84,6 +89,20 @@ void JUCE_CALLTYPE resizeHostWindow (void* hostWindow,
                   SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER | SWP_NOOWNERZORDER);
 }
 
+extern "C" BOOL WINAPI DllMainRTAS (HINSTANCE, DWORD, LPVOID);
+
+extern "C" BOOL WINAPI DllMain (HINSTANCE instance, DWORD reason, LPVOID reserved)
+{
+    if (reason == DLL_PROCESS_ATTACH)
+        Process::setCurrentModuleInstanceHandle (instance);
+
+    if (GetModuleHandleA ("DAE.DLL") != 0)
+        return DllMainRTAS (instance, reason, reserved);
+
+    ignoreUnused (reserved);
+    return TRUE;
+}
+
 #if ! JucePlugin_EditorRequiresKeyboardFocus
 
 namespace
@@ -134,4 +153,7 @@ void JUCE_CALLTYPE passFocusToHostWindow (void* hostWindow)
 }
 
 #endif
+
+} // namespace juce
+
 #endif
