@@ -2,33 +2,38 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 namespace PluginFormatManagerHelpers
 {
     struct ErrorCallbackOnMessageThread : public CallbackMessage
     {
         ErrorCallbackOnMessageThread (const String& inError,
-                                      AudioPluginFormat::InstantiationCompletionCallback* inCallback)
-            : error (inError), callback (inCallback)
+                                      AudioPluginFormat::InstantiationCompletionCallback* c)
+            : error (inError), callback (c)
         {
         }
 
@@ -38,7 +43,6 @@ namespace PluginFormatManagerHelpers
         ScopedPointer<AudioPluginFormat::InstantiationCompletionCallback> callback;
     };
 
-   #if JUCE_COMPILER_SUPPORTS_LAMBDAS
     struct ErrorLambdaOnMessageThread : public CallbackMessage
     {
         ErrorLambdaOnMessageThread (const String& inError,
@@ -52,7 +56,6 @@ namespace PluginFormatManagerHelpers
         String error;
         std::function<void (AudioPluginInstance*, const String&)> lambda;
     };
-   #endif
 }
 
 AudioPluginFormatManager::AudioPluginFormatManager() {}
@@ -63,22 +66,24 @@ void AudioPluginFormatManager::addDefaultFormats()
 {
    #if JUCE_DEBUG
     // you should only call this method once!
-    for (int i = formats.size(); --i >= 0;)
+    for (auto* format : formats)
     {
+        ignoreUnused (format);
+
        #if JUCE_PLUGINHOST_VST && (JUCE_MAC || JUCE_WINDOWS || JUCE_LINUX || JUCE_IOS)
-        jassert (dynamic_cast<VSTPluginFormat*> (formats[i]) == nullptr);
+        jassert (dynamic_cast<VSTPluginFormat*> (format) == nullptr);
        #endif
 
        #if JUCE_PLUGINHOST_VST3 && (JUCE_MAC || JUCE_WINDOWS)
-        jassert (dynamic_cast<VST3PluginFormat*> (formats[i]) == nullptr);
+        jassert (dynamic_cast<VST3PluginFormat*> (format) == nullptr);
        #endif
 
        #if JUCE_PLUGINHOST_AU && (JUCE_MAC || JUCE_IOS)
-        jassert (dynamic_cast<AudioUnitPluginFormat*> (formats[i]) == nullptr);
+        jassert (dynamic_cast<AudioUnitPluginFormat*> (format) == nullptr);
        #endif
 
        #if JUCE_PLUGINHOST_LADSPA && JUCE_LINUX
-        jassert (dynamic_cast<LADSPAPluginFormat*> (formats[i]) == nullptr);
+        jassert (dynamic_cast<LADSPAPluginFormat*> (format) == nullptr);
        #endif
     }
    #endif
@@ -105,12 +110,12 @@ int AudioPluginFormatManager::getNumFormats()
     return formats.size();
 }
 
-AudioPluginFormat* AudioPluginFormatManager::getFormat (const int index)
+AudioPluginFormat* AudioPluginFormatManager::getFormat (int index)
 {
-    return formats [index];
+    return formats[index];
 }
 
-void AudioPluginFormatManager::addFormat (AudioPluginFormat* const format)
+void AudioPluginFormatManager::addFormat (AudioPluginFormat* format)
 {
     formats.add (format);
 }
@@ -118,7 +123,7 @@ void AudioPluginFormatManager::addFormat (AudioPluginFormat* const format)
 AudioPluginInstance* AudioPluginFormatManager::createPluginInstance (const PluginDescription& description, double rate,
                                                                      int blockSize, String& errorMessage) const
 {
-    if (AudioPluginFormat* format = findFormatForDescription (description, errorMessage))
+    if (auto* format = findFormatForDescription (description, errorMessage))
         return format->createInstanceFromDescription (description, rate, blockSize, errorMessage);
 
     return nullptr;
@@ -131,13 +136,12 @@ void AudioPluginFormatManager::createPluginInstanceAsync (const PluginDescriptio
 {
     String error;
 
-    if (AudioPluginFormat* format = findFormatForDescription (description, error))
+    if (auto* format = findFormatForDescription (description, error))
         return format->createPluginInstanceAsync (description, initialSampleRate, initialBufferSize, callback);
 
     (new PluginFormatManagerHelpers::ErrorCallbackOnMessageThread (error, callback))->post();
 }
 
-#if JUCE_COMPILER_SUPPORTS_LAMBDAS
 void AudioPluginFormatManager::createPluginInstanceAsync (const PluginDescription& description,
                                                           double initialSampleRate,
                                                           int initialBufferSize,
@@ -145,25 +149,21 @@ void AudioPluginFormatManager::createPluginInstanceAsync (const PluginDescriptio
 {
     String error;
 
-    if (AudioPluginFormat* format = findFormatForDescription (description, error))
+    if (auto* format = findFormatForDescription (description, error))
         return format->createPluginInstanceAsync (description, initialSampleRate, initialBufferSize, f);
 
     (new PluginFormatManagerHelpers::ErrorLambdaOnMessageThread (error, f))->post();
 }
-#endif
 
-AudioPluginFormat* AudioPluginFormatManager::findFormatForDescription (const PluginDescription& description, String& errorMessage) const
+AudioPluginFormat* AudioPluginFormatManager::findFormatForDescription (const PluginDescription& description,
+                                                                       String& errorMessage) const
 {
-    errorMessage = String();
+    errorMessage = {};
 
-    for (int i = 0; i < formats.size(); ++i)
-    {
-        AudioPluginFormat* format;
-
-        if ((format = formats.getUnchecked (i))->getName() == description.pluginFormatName
-               && format->fileMightContainThisPluginType (description.fileOrIdentifier))
+    for (auto* format : formats)
+        if (format->getName() == description.pluginFormatName
+              && format->fileMightContainThisPluginType (description.fileOrIdentifier))
             return format;
-    }
 
     errorMessage = NEEDS_TRANS ("No compatible plug-in format exists for this plug-in");
 
@@ -172,9 +172,11 @@ AudioPluginFormat* AudioPluginFormatManager::findFormatForDescription (const Plu
 
 bool AudioPluginFormatManager::doesPluginStillExist (const PluginDescription& description) const
 {
-    for (int i = 0; i < formats.size(); ++i)
-        if (formats.getUnchecked(i)->getName() == description.pluginFormatName)
-            return formats.getUnchecked(i)->doesPluginStillExist (description);
+    for (auto* format : formats)
+        if (format->getName() == description.pluginFormatName)
+            return format->doesPluginStillExist (description);
 
     return false;
 }
+
+} // namespace juce
