@@ -2,31 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
 #if JUCE_PLUGINHOST_AU && (JUCE_MAC || JUCE_IOS)
 
-} // (juce namespace)
-
-#include <AudioUnit/AudioUnit.h>
 #if JUCE_MAC
 #include <AudioUnit/AUCocoaUIView.h>
 #include <CoreAudioKit/AUGenericView.h>
@@ -40,7 +39,7 @@
 #endif
 
 #ifndef JUCE_SUPPORTS_AUv3
- #if JUCE_COMPILER_SUPPORTS_VARIADIC_TEMPLATES && __OBJC2__  \
+ #if __OBJC2__ \
       &&  ((defined (MAC_OS_X_VERSION_MIN_REQUIRED)    && defined (MAC_OS_X_VERSION_10_11) && (MAC_OS_X_VERSION_MIN_REQUIRED    >= MAC_OS_X_VERSION_10_11)) \
        ||  (defined (__IPHONE_OS_VERSION_MIN_REQUIRED) && defined (__IPHONE_9_0)           && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_9_0)))
   #define JUCE_SUPPORTS_AUv3 1
@@ -53,19 +52,13 @@
  #include <CoreAudioKit/AUViewController.h>
 #endif
 
+#include "../../juce_audio_basics/native/juce_mac_CoreAudioLayouts.h"
+#include "../../juce_audio_devices/native/juce_MidiDataConcatenator.h"
+#include "juce_AU_Shared.h"
+
 namespace juce
 {
 
-#include "../../juce_audio_devices/native/juce_MidiDataConcatenator.h"
-
-#if JUCE_SUPPORT_CARBON
- #include "../../juce_gui_extra/native/juce_mac_CarbonViewWrapperComponent.h"
-#endif
-
-#include "../../juce_core/native/juce_osx_ObjCHelpers.h"
-
-#include "juce_AU_Shared.h"
-
 // Change this to disable logging of various activities
 #ifndef AU_LOGGING
  #define AU_LOGGING 1
@@ -122,6 +115,8 @@ namespace AudioUnitFormatHelpers
             s << "Panners/";
         else if (desc.componentType == kAudioUnitType_Mixer)
             s << "Mixers/";
+        else if (desc.componentType == kAudioUnitType_MIDIProcessor)
+            s << "MidiEffects/";
 
         s << osTypeToString (desc.componentType) << ","
           << osTypeToString (desc.componentSubType) << ","
@@ -254,7 +249,8 @@ namespace AudioUnitFormatHelpers
                              || types[0] == kAudioUnitType_Effect
                              || types[0] == kAudioUnitType_Generator
                              || types[0] == kAudioUnitType_Panner
-                             || types[0] == kAudioUnitType_Mixer)
+                             || types[0] == kAudioUnitType_Mixer
+                             || types[0] == kAudioUnitType_MIDIProcessor)
                         {
                             desc.componentType = types[0];
                             desc.componentSubType = types[1];
@@ -290,6 +286,7 @@ namespace AudioUnitFormatHelpers
             case kAudioUnitType_Generator:      return "Generator";
             case kAudioUnitType_Panner:         return "Panner";
             case kAudioUnitType_Mixer:          return "Mixer";
+            case kAudioUnitType_MIDIProcessor:  return "MidiEffects";
             default: break;
         }
 
@@ -329,7 +326,10 @@ public:
       #endif
 
         wantsMidiMessages = componentDesc.componentType == kAudioUnitType_MusicDevice
-                         || componentDesc.componentType == kAudioUnitType_MusicEffect;
+                         || componentDesc.componentType == kAudioUnitType_MusicEffect
+                         || componentDesc.componentType == kAudioUnitType_MIDIProcessor;
+
+        isMidiEffectPlugin = (componentDesc.componentType == kAudioUnitType_MIDIProcessor);
 
         AudioComponentDescription ignore;
         getComponentDescFromIdentifier (createPluginIdentifier (componentDesc), ignore, pluginName, version, manufacturer);
@@ -437,18 +437,18 @@ public:
         for (int dir = 0; dir < 2; ++dir)
         {
             const bool isInput = (dir == 0);
-            const Array<AudioChannelSet>& requestedLayouts         = (isInput ? layouts.inputBuses  : layouts.outputBuses);
-            const Array<AudioChannelSet>& oppositeRequestedLayouts = (isInput ? layouts.outputBuses : layouts.inputBuses);
-            const Array<Array<AudioChannelSet> >& supported = (isInput ? supportedInLayouts : supportedOutLayouts);
+            auto& requestedLayouts         = (isInput ? layouts.inputBuses  : layouts.outputBuses);
+            auto& oppositeRequestedLayouts = (isInput ? layouts.outputBuses : layouts.inputBuses);
+            auto& supported                = (isInput ? supportedInLayouts : supportedOutLayouts);
             const int n = getBusCount (isInput);
 
             for (int busIdx = 0; busIdx < n; ++busIdx)
             {
-                const AudioChannelSet& requested  = requestedLayouts.getReference (busIdx);
+                auto& requested = requestedLayouts.getReference (busIdx);
                 const int oppositeBusIdx = jmin (getBusCount (! isInput) - 1, busIdx);
                 const bool hasOppositeBus = (oppositeBusIdx >= 0);
-                const AudioChannelSet oppositeRequested = (hasOppositeBus ? oppositeRequestedLayouts.getReference (oppositeBusIdx) : AudioChannelSet());
-                const Array<AudioChannelSet>& possible = supported.getReference (busIdx);
+                auto oppositeRequested = (hasOppositeBus ? oppositeRequestedLayouts.getReference (oppositeBusIdx) : AudioChannelSet());
+                auto& possible = supported.getReference (busIdx);
 
                 if (requested.isDisabled())
                     return false;
@@ -459,9 +459,9 @@ public:
                 int i;
                 for (i = 0; i < numChannelInfos; ++i)
                 {
-                    const AUChannelInfo& info = channelInfos[i];
-                    const SInt16& thisChannels = (isInput ? info.inChannels  : info.outChannels);
-                    const SInt16& opChannels   = (isInput ? info.outChannels : info.inChannels);
+                    auto& info = channelInfos[i];
+                    auto& thisChannels = (isInput ? info.inChannels  : info.outChannels);
+                    auto& opChannels   = (isInput ? info.outChannels : info.inChannels);
 
                     // this bus
                     if      (thisChannels == 0) continue;
@@ -479,11 +479,13 @@ public:
                     {
                         int numOppositeBuses = getBusCount (! isInput);
                         int j;
+
                         for (j = 0; j < numOppositeBuses; ++j)
                             if (requested.size() != oppositeRequestedLayouts.getReference (j).size())
                                 break;
 
-                        if (j < numOppositeBuses) continue;
+                        if (j < numOppositeBuses)
+                            continue;
                     }
 
                     break;
@@ -556,7 +558,7 @@ public:
 
                 if (! set.isDiscreteLayout())
                 {
-                    const AudioChannelLayoutTag requestedTag = AudioUnitHelpers::ChannelSetToCALayoutTag (set);
+                    const AudioChannelLayoutTag requestedTag = CoreAudioLayouts::toCoreAudio (set);
 
                     AudioChannelLayout layout;
                     const UInt32 minDataSize = sizeof (layout) - sizeof (AudioChannelDescription);
@@ -576,12 +578,13 @@ public:
                         dataSize = expectedSize;
 
                         err = AudioUnitGetProperty (audioUnit, kAudioUnitProperty_AudioChannelLayout, scope,
-                                                    static_cast<UInt32> (i), layoutBuffer.getData(), &dataSize);
+                                                    static_cast<UInt32> (i), layoutBuffer.get(), &dataSize);
 
                         if (err != noErr || dataSize < expectedSize)
                             return false;
 
-                        actualTag = AudioUnitHelpers::ChannelSetToCALayoutTag (AudioUnitHelpers::CoreAudioChannelLayoutToJuceType (layout));
+                        // try to convert the layout into a tag
+                        actualTag = CoreAudioLayouts::toCoreAudio (CoreAudioLayouts::fromCoreAudio (layout));
                     }
 
                     if (actualTag != requestedTag)
@@ -684,37 +687,44 @@ public:
         {
             releaseResources();
 
-            for (int dir = 0; dir < 2; ++dir)
+            if (isMidiEffectPlugin)
             {
-                const bool isInput = (dir == 0);
-                const AudioUnitScope scope = isInput ? kAudioUnitScope_Input : kAudioUnitScope_Output;
-                const int n = getBusCount (isInput);
-
-                for (int i = 0; i < n; ++i)
+                outputBufferList.add (new AUBuffer (1));
+            }
+            else
+            {
+                for (int dir = 0; dir < 2; ++dir)
                 {
-                    Float64 sampleRate;
-                    UInt32 sampleRateSize = sizeof (sampleRate);
-                    const Float64 sr = newSampleRate;
+                    const bool isInput = (dir == 0);
+                    const AudioUnitScope scope = isInput ? kAudioUnitScope_Input : kAudioUnitScope_Output;
+                    const int n = getBusCount (isInput);
 
-                    AudioUnitGetProperty (audioUnit, kAudioUnitProperty_SampleRate, scope, static_cast<UInt32> (i), &sampleRate, &sampleRateSize);
-
-                    if (sampleRate != sr)
-                        AudioUnitSetProperty (audioUnit, kAudioUnitProperty_SampleRate, scope, static_cast<UInt32> (i), &sr, sizeof (sr));
-
-                    if (isInput)
+                    for (int i = 0; i < n; ++i)
                     {
-                        AURenderCallbackStruct info;
-                        zerostruct (info); // (can't use "= { 0 }" on this object because it's typedef'ed as a C struct)
+                        Float64 sampleRate;
+                        UInt32 sampleRateSize = sizeof (sampleRate);
+                        const Float64 sr = newSampleRate;
 
-                        info.inputProcRefCon = this;
-                        info.inputProc = renderGetInputCallback;
+                        AudioUnitGetProperty (audioUnit, kAudioUnitProperty_SampleRate, scope, static_cast<UInt32> (i), &sampleRate, &sampleRateSize);
 
-                        AudioUnitSetProperty (audioUnit, kAudioUnitProperty_SetRenderCallback, kAudioUnitScope_Input,
-                                              static_cast<UInt32> (i), &info, sizeof (info));
-                    }
-                    else
-                    {
-                        outputBufferList.add (new AUBuffer (static_cast<size_t> (getChannelCountOfBus (false, i))));
+                        if (sampleRate != sr)
+                            AudioUnitSetProperty (audioUnit, kAudioUnitProperty_SampleRate, scope, static_cast<UInt32> (i), &sr, sizeof (sr));
+
+                        if (isInput)
+                        {
+                            AURenderCallbackStruct info;
+                            zerostruct (info); // (can't use "= { 0 }" on this object because it's typedef'ed as a C struct)
+
+                            info.inputProcRefCon = this;
+                            info.inputProc = renderGetInputCallback;
+
+                            AudioUnitSetProperty (audioUnit, kAudioUnitProperty_SetRenderCallback, kAudioUnitScope_Input,
+                                                  static_cast<UInt32> (i), &info, sizeof (info));
+                        }
+                        else
+                        {
+                            outputBufferList.add (new AUBuffer (static_cast<size_t> (getChannelCountOfBus (false, i))));
+                        }
                     }
                 }
             }
@@ -777,27 +787,48 @@ public:
         for (int i = 0; i < getBusCount (false); ++i)  AudioUnitReset (audioUnit, kAudioUnitScope_Output, static_cast<UInt32> (i));
     }
 
-    void processBlock (AudioSampleBuffer& buffer, MidiBuffer& midiMessages) override
+    void processBlock (AudioBuffer<float>& buffer, MidiBuffer& midiMessages) override
     {
-        const int numSamples = buffer.getNumSamples();
+        auto numSamples = buffer.getNumSamples();
 
         if (prepared)
         {
             timeStamp.mHostTime = GetCurrentHostTime (numSamples, getSampleRate(), isAUv3);
+            int numOutputBuses;
 
-            int chIdx = 0;
-            const int numOutputBuses = getBusCount (false);
-            for (int i = 0; i < numOutputBuses; ++i)
+            if (isMidiEffectPlugin)
             {
-                if (AUBuffer* buf = outputBufferList[i])
+                numOutputBuses = 1;
+
+                if (AUBuffer* buf = outputBufferList[0])
                 {
                     AudioBufferList& abl = *buf;
 
                     for (AudioUnitElement j = 0; j < abl.mNumberBuffers; ++j)
                     {
-                        abl.mBuffers[j].mNumberChannels = 1;
-                        abl.mBuffers[j].mDataByteSize = (UInt32) (sizeof (float) * (size_t) numSamples);
-                        abl.mBuffers[j].mData = buffer.getWritePointer (chIdx++);
+                        abl.mBuffers[j].mNumberChannels = 0;
+                        abl.mBuffers[j].mDataByteSize = 0;
+                        abl.mBuffers[j].mData = nullptr;
+                    }
+                }
+            }
+            else
+            {
+                int chIdx = 0;
+                numOutputBuses = getBusCount (false);
+
+                for (int i = 0; i < numOutputBuses; ++i)
+                {
+                    if (AUBuffer* buf = outputBufferList[i])
+                    {
+                        AudioBufferList& abl = *buf;
+
+                        for (AudioUnitElement j = 0; j < abl.mNumberBuffers; ++j)
+                        {
+                            abl.mBuffers[j].mNumberChannels = 1;
+                            abl.mBuffers[j].mDataByteSize = (UInt32) (sizeof (float) * (size_t) numSamples);
+                            abl.mBuffers[j].mData = buffer.getWritePointer (chIdx++);
+                        }
                     }
                 }
             }
@@ -822,13 +853,22 @@ public:
                 midiMessages.clear();
             }
 
-
-            for (int i = 0; i < numOutputBuses; ++i)
+            if (isMidiEffectPlugin)
             {
                 AudioUnitRenderActionFlags flags = 0;
 
-                if (AUBuffer* buf = outputBufferList[i])
-                    AudioUnitRender (audioUnit, &flags, &timeStamp, static_cast<UInt32> (i), (UInt32) numSamples, buf->bufferList.getData());
+                if (AUBuffer* buf = outputBufferList[0])
+                    AudioUnitRender (audioUnit, &flags, &timeStamp, 0, (UInt32) numSamples, buf->bufferList.get());
+            }
+            else
+            {
+                for (int i = 0; i < numOutputBuses; ++i)
+                {
+                    AudioUnitRenderActionFlags flags = 0;
+
+                    if (AUBuffer* buf = outputBufferList[i])
+                        AudioUnitRender (audioUnit, &flags, &timeStamp, static_cast<UInt32> (i), (UInt32) numSamples, buf->bufferList.get());
+                }
             }
 
             timeStamp.mSampleTime += numSamples;
@@ -882,7 +922,7 @@ public:
         if (isPositiveAndBelow (index, getTotalNumInputChannels()))
             return "Input " + String (index + 1);
 
-        return String();
+        return {};
     }
 
     const String getOutputChannelName (int index) const override
@@ -890,7 +930,7 @@ public:
         if (isPositiveAndBelow (index, getTotalNumOutputChannels()))
             return "Output " + String (index + 1);
 
-        return String();
+        return {};
     }
 
     bool isInputChannelStereoPair (int index) const override    { return isPositiveAndBelow (index, getTotalNumInputChannels()); }
@@ -951,7 +991,7 @@ public:
         ev.mArgument.mParameter.mScope       = kAudioUnitScope_Global;
         ev.mArgument.mParameter.mElement     = 0;
 
-        AUEventListenerNotify (nullptr, nullptr, &ev);
+        AUEventListenerNotify (eventListenerRef, nullptr, &ev);
        #else
         ignoreUnused (index);
        #endif
@@ -972,17 +1012,33 @@ public:
 
     const String getParameterName (int index) override
     {
-        if (const ParamInfo* p = parameters[index])
+        if (auto* p = parameters[index])
             return p->name;
 
-        return String();
+        return {};
     }
 
     const String getParameterText (int index) override   { return String (getParameter (index)); }
 
+    int getParameterNumSteps (int index) override
+    {
+        if (auto* p = parameters[index])
+            return p->numSteps;
+
+        return AudioProcessor::getDefaultNumParameterSteps();
+    }
+
+    bool isParameterDiscrete (int index) const override
+    {
+        if (auto* p = parameters[index])
+            return p->discrete;
+
+        return false;
+    }
+
     bool isParameterAutomatable (int index) const override
     {
-        if (const ParamInfo* p = parameters[index])
+        if (auto* p = parameters[index])
             return p->automatable;
 
         return false;
@@ -1061,6 +1117,19 @@ public:
         jassertfalse; // xxx not implemented!
     }
 
+    //==============================================================================
+    void updateTrackProperties (const TrackProperties& properties) override
+    {
+        if (properties.name.isNotEmpty())
+        {
+            CFStringRef contextName = properties.name.toCFString();
+            AudioUnitSetProperty (audioUnit, kAudioUnitProperty_ContextName, kAudioUnitScope_Global,
+                                  0, &contextName, sizeof (CFStringRef));
+
+            CFRelease (contextName);
+        }
+    }
+
     //==============================================================================
     void getStateInformation (MemoryBlock& destData) override
     {
@@ -1124,6 +1193,7 @@ public:
     void refreshParameterList() override
     {
         parameters.clear();
+        paramIDToIndex.clear();
 
         if (audioUnit != nullptr)
         {
@@ -1154,9 +1224,12 @@ public:
                         ParamInfo* const param = new ParamInfo();
                         parameters.add (param);
                         param->paramID = ids[i];
+                        paramIDToIndex.getReference (ids[i]) = i;
                         param->minValue = info.minValue;
                         param->maxValue = info.maxValue;
                         param->automatable = (info.flags & kAudioUnitParameterFlag_NonRealTime) == 0;
+                        param->discrete = (info.unit == kAudioUnitParameterUnit_Indexed);
+                        param->numSteps = param->discrete ? (int) (info.maxValue + 1.0f) : AudioProcessor::getDefaultNumParameterSteps();
 
                         if ((info.flags & kAudioUnitParameterFlag_HasCFNameString) != 0)
                         {
@@ -1204,21 +1277,21 @@ private:
     String pluginName, manufacturer, version;
     String fileOrIdentifier;
     CriticalSection lock;
-    bool wantsMidiMessages, producesMidiMessages, wasPlaying, prepared, isAUv3;
+    bool wantsMidiMessages, producesMidiMessages, wasPlaying, prepared, isAUv3, isMidiEffectPlugin;
 
     struct AUBuffer
     {
         AUBuffer (size_t numBuffers)
         {
             bufferList.calloc (1, (sizeof (AudioBufferList) - sizeof (::AudioBuffer)) + (sizeof (::AudioBuffer) * numBuffers));
-            AudioBufferList& buffer = *bufferList.getData();
+            AudioBufferList& buffer = *bufferList.get();
 
             buffer.mNumberBuffers = static_cast<UInt32> (numBuffers);
         }
 
         operator AudioBufferList&()
         {
-            return *bufferList.getData();
+            return *bufferList.get();
         }
 
         HeapBlock<AudioBufferList> bufferList;
@@ -1226,8 +1299,8 @@ private:
 
     OwnedArray<AUBuffer> outputBufferList;
     AudioTimeStamp timeStamp;
-    AudioSampleBuffer* currentBuffer;
-    Array<Array<AudioChannelSet> > supportedInLayouts, supportedOutLayouts;
+    AudioBuffer<float>* currentBuffer;
+    Array<Array<AudioChannelSet>> supportedInLayouts, supportedOutLayouts;
 
     int numChannelInfos;
     HeapBlock<AUChannelInfo> channelInfos;
@@ -1242,10 +1315,12 @@ private:
         UInt32 paramID;
         String name;
         AudioUnitParameterValue minValue, maxValue;
-        bool automatable;
+        bool automatable, discrete;
+        int numSteps;
     };
 
     OwnedArray<ParamInfo> parameters;
+    HashMap<uint32, size_t> paramIDToIndex;
 
     MidiDataConcatenator midiConcatenator;
     CriticalSection midiInLock;
@@ -1331,13 +1406,12 @@ private:
          || event.mEventType == kAudioUnitEvent_BeginParameterChangeGesture
          || event.mEventType == kAudioUnitEvent_EndParameterChangeGesture)
         {
-            for (paramIndex = 0; paramIndex < parameters.size(); ++paramIndex)
-            {
-                const ParamInfo& p = *parameters.getUnchecked(paramIndex);
+            auto paramID = event.mArgument.mParameter.mParameterID;
 
-                if (p.paramID == event.mArgument.mParameter.mParameterID)
-                    break;
-            }
+            if (! paramIDToIndex.contains (paramID))
+                return;
+
+            paramIndex = static_cast<int> (paramIDToIndex [paramID]);
 
             if (! isPositiveAndBelow (paramIndex, parameters.size()))
                 return;
@@ -1347,7 +1421,7 @@ private:
         {
             case kAudioUnitEvent_ParameterValueChange:
                 {
-                    const ParamInfo& p = *parameters.getUnchecked(paramIndex);
+                    auto& p = *parameters.getUnchecked (paramIndex);
                     sendParamChangeMessageToListeners (paramIndex, (newValue - p.minValue) / (p.maxValue - p.minValue));
                 }
                 break;
@@ -1391,10 +1465,9 @@ private:
         {
             // if this ever happens, might need to add extra handling
             jassert (inNumberFrames == (UInt32) currentBuffer->getNumSamples());
-            AudioSampleBuffer buffer =
-                (static_cast<int> (inBusNumber) < getBusCount (true)
-                    ? getBusBuffer (*currentBuffer, true, static_cast<int> (inBusNumber))
-                    : AudioSampleBuffer());
+            auto buffer = static_cast<int> (inBusNumber) < getBusCount (true)
+                             ? getBusBuffer (*currentBuffer, true, static_cast<int> (inBusNumber))
+                             : AudioBuffer<float>();
 
             for (int i = 0; i < static_cast<int> (ioData->mNumberBuffers); ++i)
             {
@@ -1634,7 +1707,7 @@ private:
                 propertySize = sizeof (auLayout);
 
                 if (AudioUnitGetProperty (comp, kAudioUnitProperty_AudioChannelLayout, scope, static_cast<UInt32> (busIdx), &auLayout, &propertySize) == noErr)
-                    currentLayout = AudioUnitHelpers::CoreAudioChannelLayoutToJuceType (auLayout);
+                    currentLayout = CoreAudioLayouts::fromCoreAudio (auLayout);
             }
 
             if (currentLayout.isDisabled())
@@ -1677,7 +1750,7 @@ private:
                     UInt32 propertySize = sizeof (auLayout);
 
                     if (AudioUnitGetProperty (audioUnit, kAudioUnitProperty_AudioChannelLayout, scope, static_cast<UInt32> (busIdx), &auLayout, &propertySize) == noErr)
-                        currentLayout = AudioUnitHelpers::CoreAudioChannelLayoutToJuceType (auLayout);
+                        currentLayout = CoreAudioLayouts::fromCoreAudio (auLayout);
                 }
 
                 if (currentLayout.isDisabled())
@@ -1702,14 +1775,19 @@ private:
                         propertySize = static_cast<UInt32> (sizeof (AudioChannelLayoutTag) * numElements);
 
                         if (AudioUnitGetProperty (audioUnit, kAudioUnitProperty_SupportedChannelLayoutTags, scope,
-                                                  static_cast<UInt32> (busIdx), layoutTags.getData(), &propertySize) == noErr)
+                                                  static_cast<UInt32> (busIdx), layoutTags.get(), &propertySize) == noErr)
                         {
                             for (int j = 0; j < static_cast<int> (numElements); ++j)
                             {
                                 const AudioChannelLayoutTag tag = layoutTags[j];
 
                                 if (tag != kAudioChannelLayoutTag_UseChannelDescriptions)
-                                    supported.addIfNotAlreadyThere (AudioUnitHelpers::CALayoutTagToChannelSet (tag));
+                                {
+                                    AudioChannelLayout caLayout;
+
+                                    caLayout.mChannelLayoutTag = tag;
+                                    supported.addIfNotAlreadyThere (CoreAudioLayouts::fromCoreAudio (caLayout));
+                                }
                             }
 
                             if (supported.size() > 0)
@@ -1733,15 +1811,15 @@ private:
                 channelInfos.malloc (static_cast<size_t> (numChannelInfos));
                 propertySize = static_cast<UInt32> (sizeof (AUChannelInfo) * static_cast<size_t> (numChannelInfos));
 
-                if (AudioUnitGetProperty (audioUnit, kAudioUnitProperty_SupportedNumChannels, kAudioUnitScope_Global, 0, channelInfos.getData(), &propertySize) != noErr)
+                if (AudioUnitGetProperty (audioUnit, kAudioUnitProperty_SupportedNumChannels, kAudioUnitScope_Global, 0, channelInfos.get(), &propertySize) != noErr)
                     numChannelInfos = 0;
             }
             else
             {
                 numChannelInfos = 1;
                 channelInfos.malloc (static_cast<size_t> (numChannelInfos));
-                channelInfos.getData()->inChannels  = -1;
-                channelInfos.getData()->outChannels = -1;
+                channelInfos.get()->inChannels  = -1;
+                channelInfos.get()->outChannels = -1;
             }
         }
     }
@@ -2145,7 +2223,7 @@ AudioProcessorEditor* AudioUnitPluginInstance::createEditor()
     ScopedPointer<AudioProcessorEditor> w (new AudioUnitPluginWindowCocoa (*this, false));
 
     if (! static_cast<AudioUnitPluginWindowCocoa*> (w.get())->isValid())
-        w = nullptr;
+        w.reset();
 
    #if JUCE_SUPPORT_CARBON
     if (w == nullptr)
@@ -2158,7 +2236,7 @@ AudioProcessorEditor* AudioUnitPluginInstance::createEditor()
    #endif
 
     if (w == nullptr)
-        w = new AudioUnitPluginWindowCocoa (*this, true); // use AUGenericView as a fallback
+        w.reset (new AudioUnitPluginWindowCocoa (*this, true)); // use AUGenericView as a fallback
 
     return w.release();
 }
@@ -2358,7 +2436,8 @@ StringArray AudioUnitPluginFormat::searchPathsForPlugins (const FileSearchPath&,
              || desc.componentType == kAudioUnitType_Effect
              || desc.componentType == kAudioUnitType_Generator
              || desc.componentType == kAudioUnitType_Panner
-             || desc.componentType == kAudioUnitType_Mixer)
+             || desc.componentType == kAudioUnitType_Mixer
+             || desc.componentType == kAudioUnitType_MIDIProcessor)
         {
             ignoreUnused (allowPluginsWhichRequireAsynchronousInstantiation);
 
@@ -2420,9 +2499,11 @@ bool AudioUnitPluginFormat::doesPluginStillExist (const PluginDescription& desc)
 
 FileSearchPath AudioUnitPluginFormat::getDefaultLocationsToSearch()
 {
-    return FileSearchPath();
+    return {};
 }
 
 #undef JUCE_AU_LOG
 
+} // namespace juce
+
 #endif
