@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -32,6 +34,7 @@
 #endif
 
 #define JUCE_CORE_INCLUDE_NATIVE_HEADERS 1
+#define JUCE_CORE_INCLUDE_OBJC_HELPERS 1
 
 #include "juce_audio_processors.h"
 #include <juce_gui_extra/juce_gui_extra.h>
@@ -42,6 +45,7 @@
       && ((JUCE_PLUGINHOST_VST || JUCE_PLUGINHOST_AU) \
            || ! (defined (MAC_OS_X_VERSION_10_6) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_6))
   #include <Carbon/Carbon.h>
+  #include "../juce_gui_extra/native/juce_mac_CarbonViewWrapperComponent.h"
  #endif
 #endif
 
@@ -56,6 +60,10 @@
  #define JUCE_PLUGINHOST_VST3 0
 #endif
 
+#if JUCE_PLUGINHOST_AU && (JUCE_MAC || JUCE_IOS)
+ #include <AudioUnit/AudioUnit.h>
+#endif
+
 //==============================================================================
 namespace juce
 {
@@ -63,8 +71,8 @@ namespace juce
 static inline bool arrayContainsPlugin (const OwnedArray<PluginDescription>& list,
                                         const PluginDescription& desc)
 {
-    for (int i = list.size(); --i >= 0;)
-        if (list.getUnchecked(i)->isDuplicateOf (desc))
+    for (auto* p : list)
+        if (p->isDuplicateOf (desc))
             return true;
 
     return false;
@@ -138,6 +146,8 @@ struct AutoResizingNSViewComponentWithParent  : public AutoResizingNSViewCompone
 };
 #endif
 
+} // namespace juce
+
 #if JUCE_CLANG
  #pragma clang diagnostic ignored "-Wdeprecated-declarations"
 #endif
@@ -156,7 +166,5 @@ struct AutoResizingNSViewComponentWithParent  : public AutoResizingNSViewCompone
 #include "scanning/juce_KnownPluginList.cpp"
 #include "scanning/juce_PluginDirectoryScanner.cpp"
 #include "scanning/juce_PluginListComponent.cpp"
-#include "utilities/juce_AudioProcessorValueTreeState.cpp"
 #include "utilities/juce_AudioProcessorParameters.cpp"
-
-}
+#include "utilities/juce_AudioProcessorValueTreeState.cpp"
