@@ -2,30 +2,34 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_AUDIOPROCESSOREDITOR_H_INCLUDED
-#define JUCE_AUDIOPROCESSOREDITOR_H_INCLUDED
+namespace juce
+{
 
+class AudioProcessor;
 class AudioProcessorEditorListener;
+
 //==============================================================================
 /**
     Base class for the component that acts as the GUI for an AudioProcessor.
@@ -84,6 +88,33 @@ public:
     */
     virtual int getControlParameterIndex (Component&);
 
+    /** Override this method to indicate if your editor supports the presence or
+        absence of a host-provided MIDI controller.
+
+        Currently only AUv3 plug-ins compiled for MacOS 10.13 or iOS 11.0 (or later)
+        support this functionality, and even then the host may choose to ignore this
+        information.
+
+        The default behaviour is to report support for both cases.
+    */
+    virtual bool supportsHostMIDIControllerPresence (bool hostMIDIControllerIsAvailable);
+
+    /** Called to indicate if a host is providing a MIDI controller when the host
+        reconfigures its layout.
+
+        Use this as an opportunity to hide or display your own onscreen keyboard or
+        other input component.
+
+        Currently only AUv3 plug-ins compiled for MacOS 10.13 or iOS 11.0 (or later)
+        support this functionality.
+    */
+    virtual void hostMIDIControllerIsAvailable (bool controllerIsAvailable);
+
+    /** Can be called by a host to tell the editor that it should use a non-unity
+        GUI scale.
+    */
+    virtual void setScaleFactor (float newScale);
+
     //==============================================================================
     /** Marks the host's editor window as resizable
 
@@ -150,27 +181,30 @@ private:
     //==============================================================================
     struct AudioProcessorEditorListener : ComponentListener
     {
-        AudioProcessorEditorListener (AudioProcessorEditor* audioEditor) : e (audioEditor) {}
+        AudioProcessorEditorListener (AudioProcessorEditor& e) : ed (e) {}
 
-        void componentMovedOrResized (Component&, bool, bool wasResized) override   { e->editorResized (wasResized); }
-        void componentParentHierarchyChanged (Component&) override                  { e->updatePeer(); }
-        AudioProcessorEditor* e;
+        void componentMovedOrResized (Component&, bool, bool wasResized) override   { ed.editorResized (wasResized); }
+        void componentParentHierarchyChanged (Component&) override                  { ed.updatePeer(); }
+
+        AudioProcessorEditor& ed;
+
+        JUCE_DECLARE_NON_COPYABLE (AudioProcessorEditorListener)
     };
 
     //==============================================================================
     void initialise();
     void editorResized (bool wasResized);
     void updatePeer();
-    void attachConstrainer (ComponentBoundsConstrainer* newConstrainer);
+    void attachConstrainer (ComponentBoundsConstrainer*);
 
     //==============================================================================
     ScopedPointer<AudioProcessorEditorListener> resizeListener;
     bool resizable;
     ComponentBoundsConstrainer defaultConstrainer;
-    ComponentBoundsConstrainer* constrainer;
+    ComponentBoundsConstrainer* constrainer = {};
+    Component::SafePointer<Component> splashScreen;
 
     JUCE_DECLARE_NON_COPYABLE (AudioProcessorEditor)
 };
 
-
-#endif   // JUCE_AUDIOPROCESSOREDITOR_H_INCLUDED
+} // namespace juce
