@@ -2,28 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_AUDIOPROCESSORGRAPH_H_INCLUDED
-#define JUCE_AUDIOPROCESSORGRAPH_H_INCLUDED
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -40,6 +42,7 @@
     AudioProcessorPlayer object.
 */
 class JUCE_API  AudioProcessorGraph   : public AudioProcessor,
+                                        public ChangeBroadcaster,
                                         private AsyncUpdater
 {
 public:
@@ -52,6 +55,32 @@ public:
     */
     ~AudioProcessorGraph();
 
+    /** Each node in the graph has a UID of this type. */
+    typedef uint32 NodeID;
+
+    //==============================================================================
+    /** A special index that represents the midi channel of a node.
+
+        This is used as a channel index value if you want to refer to the midi input
+        or output instead of an audio channel.
+    */
+    enum { midiChannelIndex = 0x1000 };
+
+    //==============================================================================
+    /**
+        Represents an input or output channel of a node in an AudioProcessorGraph.
+    */
+    struct NodeAndChannel
+    {
+        NodeID nodeID;
+        int channelIndex;
+
+        bool isMIDI() const noexcept                                    { return channelIndex == midiChannelIndex; }
+
+        bool operator== (const NodeAndChannel& other) const noexcept    { return nodeID == other.nodeID && channelIndex == other.channelIndex; }
+        bool operator!= (const NodeAndChannel& other) const noexcept    { return ! operator== (other); }
+    };
+
     //==============================================================================
     /** Represents one of the nodes, or processors, in an AudioProcessorGraph.
 
@@ -64,10 +93,10 @@ public:
         /** The ID number assigned to this node.
             This is assigned by the graph that owns it, and can't be changed.
         */
-        const uint32 nodeId;
+        const NodeID nodeID;
 
         /** The actual processor object that this node represents. */
-        AudioProcessor* getProcessor() const noexcept           { return processor; }
+        AudioProcessor* getProcessor() const noexcept           { return processor.get(); }
 
         /** A set of user-definable properties that are associated with this node.
 
@@ -85,10 +114,19 @@ public:
         //==============================================================================
         friend class AudioProcessorGraph;
 
-        const ScopedPointer<AudioProcessor> processor;
-        bool isPrepared;
+        struct Connection
+        {
+            Node* otherNode;
+            int otherChannel, thisChannel;
 
-        Node (uint32 nodeId, AudioProcessor*) noexcept;
+            bool operator== (const Connection&) const noexcept;
+        };
+
+        const ScopedPointer<AudioProcessor> processor;
+        Array<Connection> inputs, outputs;
+        bool isPrepared = false;
+
+        Node (NodeID, AudioProcessor*) noexcept;
 
         void setParentGraph (AudioProcessorGraph*) const;
         void prepare (double newSampleRate, int newBlockSize, AudioProcessorGraph*, ProcessingPrecision);
@@ -104,43 +142,22 @@ public:
     */
     struct JUCE_API  Connection
     {
-    public:
         //==============================================================================
-        Connection (uint32 sourceNodeId, int sourceChannelIndex,
-                    uint32 destNodeId, int destChannelIndex) noexcept;
+        Connection (NodeAndChannel source, NodeAndChannel destination) noexcept;
+
+        Connection (const Connection&) = default;
+        Connection& operator= (const Connection&) = default;
+
+        bool operator== (const Connection&) const noexcept;
+        bool operator!= (const Connection&) const noexcept;
+        bool operator<  (const Connection&) const noexcept;
 
         //==============================================================================
-        /** The ID number of the node which is the input source for this connection.
-            @see AudioProcessorGraph::getNodeForId
-        */
-        uint32 sourceNodeId;
+        /** The channel and node which is the input source for this connection. */
+        NodeAndChannel source;
 
-        /** The index of the output channel of the source node from which this
-            connection takes its data.
-
-            If this value is the special number AudioProcessorGraph::midiChannelIndex, then
-            it is referring to the source node's midi output. Otherwise, it is the zero-based
-            index of an audio output channel in the source node.
-        */
-        int sourceChannelIndex;
-
-        /** The ID number of the node which is the destination for this connection.
-            @see AudioProcessorGraph::getNodeForId
-        */
-        uint32 destNodeId;
-
-        /** The index of the input channel of the destination node to which this
-            connection delivers its data.
-
-            If this value is the special number AudioProcessorGraph::midiChannelIndex, then
-            it is referring to the destination node's midi input. Otherwise, it is the zero-based
-            index of an audio input channel in the destination node.
-        */
-        int destChannelIndex;
-
-    private:
-        //==============================================================================
-        JUCE_LEAK_DETECTOR (Connection)
+        /** The channel and node which is the input source for this connection. */
+        NodeAndChannel destination;
     };
 
     //==============================================================================
@@ -149,6 +166,9 @@ public:
     */
     void clear();
 
+    /** Returns the array of nodes in the graph. */
+    const ReferenceCountedArray<Node>& getNodes() const noexcept    { return nodes; }
+
     /** Returns the number of nodes in the graph. */
     int getNumNodes() const noexcept                                { return nodes.size(); }
 
@@ -156,13 +176,13 @@ public:
         This will return nullptr if the index is out of range.
         @see getNodeForId
     */
-    Node* getNode (const int index) const noexcept                  { return nodes [index]; }
+    Node* getNode (int index) const noexcept                        { return nodes [index]; }
 
     /** Searches the graph for a node with the given ID number and returns it.
         If no such node was found, this returns nullptr.
         @see getNode
     */
-    Node* getNodeForId (const uint32 nodeId) const;
+    Node* getNodeForId (NodeID) const;
 
     /** Adds a node to the graph.
 
@@ -175,71 +195,56 @@ public:
 
         If this succeeds, it returns a pointer to the newly-created node.
     */
-    Node* addNode (AudioProcessor* newProcessor, uint32 nodeId = 0);
+    Node::Ptr addNode (AudioProcessor* newProcessor, NodeID nodeId = {});
 
     /** Deletes a node within the graph which has the specified ID.
-
         This will also delete any connections that are attached to this node.
     */
-    bool removeNode (uint32 nodeId);
-
-    /** Deletes a node within the graph which has the specified ID.
+    bool removeNode (NodeID);
 
+    /** Deletes a node within the graph.
         This will also delete any connections that are attached to this node.
-     */
-    bool removeNode (Node* node);
-
-    //==============================================================================
-    /** Returns the number of connections in the graph. */
-    int getNumConnections() const                                       { return connections.size(); }
-
-    /** Returns a pointer to one of the connections in the graph. */
-    const Connection* getConnection (int index) const                   { return connections [index]; }
-
-    /** Searches for a connection between some specified channels.
-        If no such connection is found, this returns nullptr.
     */
-    const Connection* getConnectionBetween (uint32 sourceNodeId,
-                                            int sourceChannelIndex,
-                                            uint32 destNodeId,
-                                            int destChannelIndex) const;
+    bool removeNode (Node*);
 
-    /** Returns true if there is a connection between any of the channels of
+    /** Returns the list of connections in the graph. */
+    std::vector<Connection> getConnections() const;
+
+    /** Returns true if the given connection exists. */
+    bool isConnected (const Connection&) const noexcept;
+
+    /** Returns true if there is a direct connection between any of the channels of
         two specified nodes.
     */
-    bool isConnected (uint32 possibleSourceNodeId,
-                      uint32 possibleDestNodeId) const;
+    bool isConnected (NodeID possibleSourceNodeID, NodeID possibleDestNodeID) const noexcept;
+
+    /** Does a recursive check to see if there's a direct or indirect series of connections
+        between these two nodes.
+    */
+    bool isAnInputTo (Node& source, Node& destination) const noexcept;
 
     /** Returns true if it would be legal to connect the specified points. */
-    bool canConnect (uint32 sourceNodeId, int sourceChannelIndex,
-                     uint32 destNodeId, int destChannelIndex) const;
+    bool canConnect (const Connection&) const;
 
     /** Attempts to connect two specified channels of two nodes.
 
         If this isn't allowed (e.g. because you're trying to connect a midi channel
         to an audio one or other such nonsense), then it'll return false.
     */
-    bool addConnection (uint32 sourceNodeId, int sourceChannelIndex,
-                        uint32 destNodeId, int destChannelIndex);
+    bool addConnection (const Connection&);
 
-    /** Deletes the connection with the specified index. */
-    void removeConnection (int index);
-
-    /** Deletes any connection between two specified points.
-        Returns true if a connection was actually deleted.
-    */
-    bool removeConnection (uint32 sourceNodeId, int sourceChannelIndex,
-                           uint32 destNodeId, int destChannelIndex);
+    /** Deletes the given connection. */
+    bool removeConnection (const Connection&);
 
     /** Removes all connections from the specified node. */
-    bool disconnectNode (uint32 nodeId);
+    bool disconnectNode (NodeID);
 
     /** Returns true if the given connection's channel numbers map on to valid
         channels at each end.
         Even if a connection is valid when created, its status could change if
         a node changes its channel config.
     */
-    bool isConnectionLegal (const Connection* connection) const;
+    bool isConnectionLegal (const Connection&) const;
 
     /** Performs a sanity checks of all the connections.
 
@@ -248,15 +253,6 @@ public:
     */
     bool removeIllegalConnections();
 
-    //==============================================================================
-    /** A special number that represents the midi channel of a node.
-
-        This is used as a channel index value if you want to refer to the midi input
-        or output instead of an audio channel.
-    */
-    static const int midiChannelIndex;
-
-
     //==============================================================================
     /** A special type of AudioProcessor that can live inside an AudioProcessorGraph
         in order to use the audio that comes into and out of the graph itself.
@@ -306,7 +302,7 @@ public:
         bool isOutput() const noexcept;
 
         //==============================================================================
-        AudioGraphIOProcessor (const IODeviceType type);
+        AudioGraphIOProcessor (IODeviceType);
         ~AudioGraphIOProcessor();
 
         const String getName() const override;
@@ -338,11 +334,7 @@ public:
 
     private:
         const IODeviceType type;
-        AudioProcessorGraph* graph;
-
-        //==============================================================================
-        template <typename floatType>
-        void processAudio (AudioBuffer<floatType>& buffer, MidiBuffer& midiMessages);
+        AudioProcessorGraph* graph = nullptr;
 
         JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioGraphIOProcessor)
     };
@@ -357,7 +349,6 @@ public:
 
     void reset() override;
     void setNonRealtime (bool) noexcept override;
-    void setPlayHead (AudioPlayHead*) override;
 
     double getTailLengthSeconds() const override;
     bool acceptsMidi() const override;
@@ -368,39 +359,37 @@ public:
     int getNumPrograms() override                           { return 0; }
     int getCurrentProgram() override                        { return 0; }
     void setCurrentProgram (int) override                   { }
-    const String getProgramName (int) override              { return String(); }
+    const String getProgramName (int) override              { return {}; }
     void changeProgramName (int, const String&) override    { }
     void getStateInformation (juce::MemoryBlock&) override;
     void setStateInformation (const void* data, int sizeInBytes) override;
 
 private:
-    //==============================================================================
-    template <typename floatType>
-    void processAudio (AudioBuffer<floatType>& buffer, MidiBuffer& midiMessages);
-
     //==============================================================================
     ReferenceCountedArray<Node> nodes;
-    OwnedArray<Connection> connections;
-    uint32 lastNodeId;
-    OwnedArray<MidiBuffer> midiBuffers;
-    Array<void*> renderingOps;
+    NodeID lastNodeID = {};
+
+    struct RenderSequenceFloat;
+    struct RenderSequenceDouble;
+    ScopedPointer<RenderSequenceFloat> renderSequenceFloat;
+    ScopedPointer<RenderSequenceDouble> renderSequenceDouble;
 
     friend class AudioGraphIOProcessor;
-    struct AudioProcessorGraphBufferHelpers;
-    ScopedPointer<AudioProcessorGraphBufferHelpers> audioBuffers;
 
-    MidiBuffer* currentMidiInputBuffer;
-    MidiBuffer currentMidiOutputBuffer;
-
-    bool isPrepared;
+    bool isPrepared = false;
 
+    void topologyChanged();
     void handleAsyncUpdate() override;
     void clearRenderingSequence();
     void buildRenderingSequence();
-    bool isAnInputTo (uint32 possibleInputId, uint32 possibleDestinationId, int recursionCheck) const;
+    bool anyNodesNeedPreparing() const noexcept;
+    bool isConnected (Node* src, int sourceChannel, Node* dest, int destChannel) const noexcept;
+    bool isAnInputTo (Node& src, Node& dst, int recursionCheck) const noexcept;
+    bool canConnect (Node* src, int sourceChannel, Node* dest, int destChannel) const noexcept;
+    bool isLegal (Node* src, int sourceChannel, Node* dest, int destChannel) const noexcept;
+    static void getNodeConnections (Node&, std::vector<Connection>&);
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioProcessorGraph)
 };
 
-
-#endif   // JUCE_AUDIOPROCESSORGRAPH_H_INCLUDED
+} // namespace juce
