@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 class ProcessorParameterPropertyComp   : public PropertyComponent,
                                          private AudioProcessorListener,
                                          private Timer
@@ -31,7 +36,6 @@ public:
         : PropertyComponent (name),
           owner (p),
           index (paramIndex),
-          paramHasChanged (false),
           slider (p, paramIndex)
     {
         startTimer (100);
@@ -82,13 +86,16 @@ private:
     public:
         ParamSlider (AudioProcessor& p, int paramIndex)  : owner (p), index (paramIndex)
         {
-            const int steps = owner.getParameterNumSteps (index);
+            auto steps = owner.getParameterNumSteps (index);
+            auto category = p.getParameterCategory (index);
+            bool isLevelMeter = (((category & 0xffff0000) >> 16) == 2);
 
             if (steps > 1 && steps < 0x7fffffff)
                 setRange (0.0, 1.0, 1.0 / (steps - 1.0));
             else
                 setRange (0.0, 1.0);
 
+            setEnabled (! isLevelMeter);
             setSliderStyle (Slider::LinearBar);
             setTextBoxIsEditable (false);
             setScrollWheelEnabled (true);
@@ -96,7 +103,7 @@ private:
 
         void valueChanged() override
         {
-            const float newVal = (float) getValue();
+            auto newVal = static_cast<float> (getValue());
 
             if (owner.getParameter (index) != newVal)
             {
@@ -130,7 +137,7 @@ private:
 
     AudioProcessor& owner;
     const int index;
-    bool volatile paramHasChanged;
+    bool volatile paramHasChanged = false;
     ParamSlider slider;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ProcessorParameterPropertyComp)
@@ -146,18 +153,19 @@ GenericAudioProcessorEditor::GenericAudioProcessorEditor (AudioProcessor* const
 
     addAndMakeVisible (panel);
 
-    Array <PropertyComponent*> params;
+    Array<PropertyComponent*> params;
 
-    const int numParams = p->getNumParameters();
+    auto numParams = p->getNumParameters();
     int totalHeight = 0;
 
     for (int i = 0; i < numParams; ++i)
     {
-        String name (p->getParameterName (i));
+        auto name = p->getParameterName (i);
+
         if (name.trim().isEmpty())
             name = "Unnamed";
 
-        ProcessorParameterPropertyComp* const pc = new ProcessorParameterPropertyComp (name, *p, i);
+        auto* pc = new ProcessorParameterPropertyComp (name, *p, i);
         params.add (pc);
         totalHeight += pc->getPreferredHeight();
     }
@@ -180,3 +188,5 @@ void GenericAudioProcessorEditor::resized()
 {
     panel.setBounds (getLocalBounds());
 }
+
+} // namespace juce
