@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_KNOWNPLUGINLIST_H_INCLUDED
-#define JUCE_KNOWNPLUGINLIST_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -61,6 +62,7 @@ public:
 
     /** Type iteration. */
     PluginDescription** begin() const noexcept                      { return types.begin(); }
+
     /** Type iteration. */
     PluginDescription** end() const noexcept                        { return types.end(); }
 
@@ -94,7 +96,7 @@ public:
     */
     bool scanAndAddFile (const String& possiblePluginFileOrIdentifier,
                          bool dontRescanIfAlreadyInList,
-                         OwnedArray <PluginDescription>& typesFound,
+                         OwnedArray<PluginDescription>& typesFound,
                          AudioPluginFormat& formatToUse);
 
     /** Tells a custom scanner that a scan has finished, and it can release any resources. */
@@ -111,7 +113,7 @@ public:
     */
     void scanAndAddDragAndDroppedFiles (AudioPluginFormatManager& formatManager,
                                         const StringArray& filenames,
-                                        OwnedArray <PluginDescription>& typesFound);
+                                        OwnedArray<PluginDescription>& typesFound);
 
     //==============================================================================
     /** Returns the list of blacklisted files. */
@@ -193,7 +195,7 @@ public:
             @returns true if the plugin loaded, false if it crashed
         */
         virtual bool findPluginTypesFor (AudioPluginFormat& format,
-                                         OwnedArray <PluginDescription>& result,
+                                         OwnedArray<PluginDescription>& result,
                                          const String& fileOrIdentifier) = 0;
 
         /** Called when a scan has finished, to allow clean-up of resources. */
@@ -221,5 +223,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (KnownPluginList)
 };
 
-
-#endif   // JUCE_KNOWNPLUGINLIST_H_INCLUDED
+} // namespace juce
