@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 /**
     Provides a class of AudioProcessorParameter that can be used as a boolean value.
 
@@ -33,7 +38,8 @@ public:
     /** Creates a AudioParameterBool with an ID and name.
         On creation, its value is set to the default value.
     */
-    AudioParameterBool (String parameterID, String name, bool defaultValue);
+    AudioParameterBool (const String& parameterID, const String& name, bool defaultValue,
+                        const String& label = String());
 
     /** Destructor. */
     ~AudioParameterBool();
@@ -46,17 +52,26 @@ public:
     /** Changes the parameter's current value to a new boolean. */
     AudioParameterBool& operator= (bool newValue);
 
+protected:
+    /** Override this method if you are interested in receiving callbacks
+        when the parameter value changes.
+    */
+    virtual void valueChanged (bool newValue);
 
 private:
     //==============================================================================
-    float value, defaultValue;
+    float value;
+    const float defaultValue;
 
     float getValue() const override;
     void setValue (float newValue) override;
     float getDefaultValue() const override;
     int getNumSteps() const override;
+    bool isDiscrete() const override;
     String getText (float, int) const override;
     float getValueForText (const String&) const override;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (AudioParameterBool)
 };
+
+} // namespace juce
