@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -31,14 +33,13 @@
  #error "Incorrect use of JUCE cpp file"
 #endif
 
-#define JUCE_CORE_INCLUDE_JNI_HELPERS 1
 #define JUCE_CORE_INCLUDE_NATIVE_HEADERS 1
+#define JUCE_CORE_INCLUDE_JNI_HELPERS 1
+#define JUCE_CORE_INCLUDE_OBJC_HELPERS 1
+#define JUCE_CORE_INCLUDE_COM_SMART_PTR 1
 
 #include "juce_audio_utils.h"
-
-#if JUCE_MODULE_AVAILABLE_juce_gui_extra
- #include <juce_gui_extra/juce_gui_extra.h>
-#endif
+#include <juce_gui_extra/juce_gui_extra.h>
 
 #if JUCE_MAC
   #import <DiscRecording/DiscRecording.h>
@@ -53,9 +54,6 @@
  #endif
 #endif
 
-namespace juce
-{
-
 #include "gui/juce_AudioDeviceSelectorComponent.cpp"
 #include "gui/juce_AudioThumbnail.cpp"
 #include "gui/juce_AudioThumbnailCache.cpp"
@@ -67,9 +65,7 @@ namespace juce
 #include "audio_cd/juce_AudioCDReader.cpp"
 
 #if JUCE_MAC
-
  #include "native/juce_mac_BluetoothMidiDevicePairingDialogue.mm"
- #include "../juce_core/native/juce_osx_ObjCHelpers.h"
 
  #if JUCE_USE_CDREADER
   #include "native/juce_mac_AudioCDReader.mm"
@@ -80,15 +76,12 @@ namespace juce
  #endif
 
 #elif JUCE_IOS
-
  #include "native/juce_ios_BluetoothMidiDevicePairingDialogue.mm"
 
 #elif JUCE_ANDROID
-
  #include "native/juce_android_BluetoothMidiDevicePairingDialogue.cpp"
 
 #elif JUCE_LINUX
-
  #if JUCE_USE_CDREADER
   #include "native/juce_linux_AudioCDReader.cpp"
  #endif
@@ -96,9 +89,7 @@ namespace juce
  #include "native/juce_linux_BluetoothMidiDevicePairingDialogue.cpp"
 
 #elif JUCE_WINDOWS
-
  #include "native/juce_win_BluetoothMidiDevicePairingDialogue.cpp"
- #include "../juce_core/native/juce_win32_ComSmartPtr.h"
 
  #if JUCE_USE_CDREADER
   #include "native/juce_win32_AudioCDReader.cpp"
@@ -109,5 +100,3 @@ namespace juce
  #endif
 
 #endif
-
-}
