@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -33,13 +35,13 @@
 
   ID:               juce_audio_utils
   vendor:           juce
-  version:          4.3.0
+  version:          5.2.1
   name:             JUCE extra audio utility classes
   description:      Classes for audio-related GUI and miscellaneous tasks.
   website:          http://www.juce.com/juce
   license:          GPL/Commercial
 
-  dependencies:     juce_gui_extra, juce_audio_basics, juce_audio_processors, juce_audio_formats
+  dependencies:     juce_gui_extra, juce_audio_processors, juce_audio_formats, juce_audio_devices
   OSXFrameworks:    DiscRecording
   iOSFrameworks:    CoreAudioKit
 
@@ -48,7 +50,7 @@
 *******************************************************************************/
 
 
-#ifndef JUCE_AUDIO_UTILS_H_INCLUDED
+#pragma once
 #define JUCE_AUDIO_UTILS_H_INCLUDED
 
 #include <juce_gui_basics/juce_gui_basics.h>
@@ -58,23 +60,20 @@
 
 //==============================================================================
 /** Config: JUCE_USE_CDREADER
- Enables the AudioCDReader class (on supported platforms).
- */
+    Enables the AudioCDReader class (on supported platforms).
+*/
 #ifndef JUCE_USE_CDREADER
 #define JUCE_USE_CDREADER 0
 #endif
 
 /** Config: JUCE_USE_CDBURNER
- Enables the AudioCDBurner class (on supported platforms).
- */
+    Enables the AudioCDBurner class (on supported platforms).
+*/
 #ifndef JUCE_USE_CDBURNER
 #define JUCE_USE_CDBURNER 0
 #endif
 
 //==============================================================================
-namespace juce
-{
-
 #include "gui/juce_AudioDeviceSelectorComponent.h"
 #include "gui/juce_AudioThumbnailBase.h"
 #include "gui/juce_AudioThumbnail.h"
@@ -87,7 +86,3 @@ namespace juce
 #include "players/juce_AudioProcessorPlayer.h"
 #include "audio_cd/juce_AudioCDBurner.h"
 #include "audio_cd/juce_AudioCDReader.h"
-
-}
-
-#endif   // JUCE_AUDIO_UTILS_H_INCLUDED
