@@ -2,22 +2,20 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2016 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -38,11 +36,11 @@ namespace littlefoot
  #define LITTLEFOOT_DUMP_PROGRAM 0
 #endif
 
-using int8        = char;
+using int8        = signed char;
 using uint8       = unsigned char;
-using int16       = short;
+using int16       = signed short;
 using uint16      = unsigned short;
-using int32       = int;
+using int32       = signed int;
 using uint32      = unsigned int;
 using FunctionID  = int16;
 
@@ -174,8 +172,8 @@ struct NativeFunction
         jassert (nameAndArgTypes[slash + 1] != 0);  // The slash must be followed by a return type character
         jassert (juce::String (nameAndArgTypes).substring (0, slash).containsOnly ("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_"));
         jassert (! juce::String ("0123456789").containsChar (nameAndArgTypes[0]));
-        jassert (juce::String (nameAndArgTypes).substring (slash + 1).containsOnly ("vif"));
-        jassert (juce::String (nameAndArgTypes).substring (slash + 2).containsOnly ("if")); // arguments must only be of these types
+        jassert (juce::String (nameAndArgTypes).substring (slash + 1).containsOnly ("vifb"));
+        jassert (juce::String (nameAndArgTypes).substring (slash + 2).containsOnly ("ifb")); // arguments must only be of these types
 
         for (; nameAndArgTypes[i] != 0; ++i)
             if (i != slash + 1)
@@ -241,13 +239,6 @@ struct Program
         return calculateChecksum() == getStoredChecksum();
     }
 
-    uint32 getProgramSize() const noexcept
-    {
-        auto size = (uint16) readInt16 (programStart + 2);
-        return size < programHeaderSize ? programHeaderSize
-                                        : (size > maxProgramSize ? maxProgramSize : size);
-    }
-
     uint32 getNumFunctions() const noexcept
     {
         return (uint16) readInt16 (programStart + 4);
@@ -280,10 +271,11 @@ struct Program
                                                     : getFunctionStartAddress (functionIndex);
     }
 
-    /** Returns the number of global variables the program uses */
-    uint16 getNumGlobals() const noexcept
+    uint32 getProgramSize() const noexcept
     {
-        return (uint16) readInt16 (programStart + 6);
+        auto size = (uint16) readInt16 (programStart + 2);
+        return size < programHeaderSize ? programHeaderSize
+                                        : (size > maxProgramSize ? maxProgramSize : size);
     }
 
     /** Returns the number of bytes of heap space the program needs */
@@ -292,6 +284,17 @@ struct Program
         return (uint16) readInt16 (programStart + 8);
     }
 
+    /** Returns the number of global variables the program uses */
+    uint16 getNumGlobals() const noexcept
+    {
+        return (uint16) readInt16 (programStart + 6);
+    }
+
+    uint32 getTotalSpaceNeeded() const noexcept
+    {
+        return getProgramSize() + getHeapSizeBytes();
+    }
+
    #if JUCE_DEBUG
     //==============================================================================
     /** Prints the assembly code for a given function. */
@@ -301,7 +304,7 @@ struct Program
             << "  (" << juce::String::toHexString (getFunctionID (functionIndex)) << ")" << juce::newLine;
 
         if (auto codeStart = getFunctionStartAddress (functionIndex))
-            if (auto codeEnd   = getFunctionEndAddress (functionIndex))
+            if (auto codeEnd = getFunctionEndAddress (functionIndex))
                 for (auto prog = codeStart; prog < codeEnd;)
                     out << getOpDisassembly (prog) << juce::newLine;
     }
@@ -333,22 +336,44 @@ struct Program
     /** Calls dumpFunctionDisassembly() for all functions. */
     void dumpAllFunctions (juce::OutputStream& out) const
     {
-        DBG ("Program size: " << (int) getProgramSize() << " bytes");
+        if (programStart != nullptr)
+        {
+            DBG ("Program size: " << (int) getProgramSize() << " bytes");
 
-        for (uint32 i = 0; i < getNumFunctions(); ++i)
-            dumpFunctionDisassembly (out, i);
+            for (uint32 i = 0; i < getNumFunctions(); ++i)
+                dumpFunctionDisassembly (out, i);
+        }
     }
    #endif
 
+    /** For a given op code, this returns the number of program bytes that follow it. */
+    static uint8 getNumExtraBytesForOpcode (OpCode op) noexcept
+    {
+        switch (op)
+        {
+           #define LITTLEFOOT_OP(name)         case OpCode::name:  return 0;
+           #define LITTLEFOOT_OP_INT8(name)    case OpCode::name:  return 1;
+           #define LITTLEFOOT_OP_INT16(name)   case OpCode::name:  return 2;
+           #define LITTLEFOOT_OP_INT32(name)   case OpCode::name:  return 4;
+            LITTLEFOOT_OPCODES (LITTLEFOOT_OP, LITTLEFOOT_OP_INT8, LITTLEFOOT_OP_INT16, LITTLEFOOT_OP_INT32)
+           #undef LITTLEFOOT_OP
+           #undef LITTLEFOOT_OP_INT8
+           #undef LITTLEFOOT_OP_INT16
+           #undef LITTLEFOOT_OP_INT32
+
+            default:  jassertfalse; return 0;
+        }
+    }
+
     //==============================================================================
     static float intToFloat (int32 value) noexcept          { float v; copyFloatMem (&v, &value); return v; }
     static int32 floatToInt (float value) noexcept          { int32 v; copyFloatMem (&v, &value); return v; }
 
-    static int16 readInt16 (const uint8* d) noexcept        { return (int16) (d[0] + (((uint16) d[1]) << 8)); }
-    static int32 readInt32 (const uint8* d) noexcept        { return (int32) ((uint32) readInt16 (d) + (((uint32) readInt16 (d + 2)) << 16)); }
+    static int16 readInt16 (const uint8* d) noexcept        { return (int16) (d[0] | (((uint16) d[1]) << 8)); }
+    static int32 readInt32 (const uint8* d) noexcept        { return (int32) (d[0] | (((uint32) d[1]) << 8) | (((uint32) d[2]) << 16) | (((uint32) d[3]) << 24)); }
 
-    static void writeInt16 (uint8* d, int16 v) noexcept     { d[0] = v & 0xff; d[1] = (uint8) (v >> 8); }
-    static void writeInt32 (uint8* d, int32 v) noexcept     { writeInt16 (d, (int16) (v & 0xffff)); writeInt16 (d + 2, (int16) (v >> 16)); }
+    static void writeInt16 (uint8* d, int16 v) noexcept     { d[0] = (uint8) v; d[1] = (uint8) (v >> 8); }
+    static void writeInt32 (uint8* d, int32 v) noexcept     { d[0] = (uint8) v; d[1] = (uint8) (v >> 8); d[2] = (uint8) (v >> 16); d[3] = (uint8) (v >> 24); }
 
     //==============================================================================
     static constexpr uint32 programHeaderSize = 10;
@@ -410,6 +435,16 @@ struct Runner
             allMemory[i] = 0;
     }
 
+    /** Clears all the non-program data. */
+    void clearHeapAndGlobals() noexcept
+    {
+        auto* start = getProgramAndDataStart() + program.getProgramSize();
+        auto* end = allMemory + sizeof (allMemory);
+
+        for (auto m = start; m < end; ++m)
+            *m = 0;
+    }
+
     /** Return codes from a function call */
     enum class ErrorCode  : uint8
     {
@@ -423,6 +458,23 @@ struct Runner
         unknownFunction
     };
 
+    /** Returns a text description for an error code */
+    static const char* getErrorDescription (ErrorCode e) noexcept
+    {
+        switch (e)
+        {
+            case ErrorCode::ok:                    return "OK";
+            case ErrorCode::executionTimedOut:     return "Timed-out";
+            case ErrorCode::unknownInstruction:    return "Illegal instruction";
+            case ErrorCode::stackOverflow:         return "Stack overflow";
+            case ErrorCode::stackUnderflow:        return "Stack underflow";
+            case ErrorCode::illegalAddress:        return "Illegal access";
+            case ErrorCode::divisionByZero:        return "Division by zero";
+            case ErrorCode::unknownFunction:       return "Unknown function";
+            default:                               return "Unknown error";
+        }
+    }
+
     /** Calls one of the functions in the program, by its textual signature. */
     ErrorCode callFunction (const char* functionSignature) noexcept
     {
@@ -459,14 +511,19 @@ struct Runner
     uint16 getProgramHeapSize() const noexcept          { return heapSize; }
 
     /** */
+    bool isProgramValid() const noexcept                { return heapStart != nullptr; }
+
+    /** Sets a byte of data. */
     void setDataByte (uint32 index, uint8 value) noexcept
     {
         if (index < programAndHeapSpace)
         {
-            if (index < program.getProgramSize())
+            auto& dest = getProgramAndDataStart()[index];
+
+            if (index < program.getProgramSize() && dest != value)
                 heapStart = nullptr; // force a re-initialise of the memory layout when the program changes
 
-            getProgramAndDataStart()[index] = value;
+            dest = value;
         }
     }
 
@@ -501,7 +558,7 @@ struct Runner
     /** */
     int32 setHeapInt (uint32 byteOffset, uint32 value) noexcept
     {
-        if (byteOffset < (uint32) (getProgramHeapSize() - 3))
+        if (byteOffset + 3 < getProgramHeapSize())
             Program::writeInt32 (getProgramHeapStart() + byteOffset, (int32) value);
 
         return 0;
@@ -510,7 +567,7 @@ struct Runner
     /** */
     int32 getHeapInt (uint32 byteOffset) const noexcept
     {
-        return byteOffset < getProgramHeapSize() - 3 ? Program::readInt32 (getProgramHeapStart() + byteOffset) : 0;
+        return byteOffset + 3 < getProgramHeapSize() ? Program::readInt32 (getProgramHeapStart() + byteOffset) : 0;
     }
 
     //==============================================================================
@@ -552,7 +609,7 @@ struct Runner
                     if (prog.getFunctionID (i) == function)
                     {
                         programCounter  = prog.getFunctionStartAddress (i);
-                        functionEnd     = prog.getFunctionEndAddress (i);
+                        programEnd      = r.getProgramHeapStart();
                         tos             = *--stack = 0;
                         return;
                     }
@@ -590,7 +647,7 @@ struct Runner
 
             for (;;)
             {
-                if (programCounter >= functionEnd)
+                if (programCounter >= programEnd)
                     return error;
 
                 if ((++opsPerformed & 63) == 0 && hasTimedOut())
@@ -619,7 +676,7 @@ struct Runner
         //==============================================================================
         Runner* runner;
         const uint8* programCounter;
-        const uint8* functionEnd;
+        const uint8* programEnd;
         const uint8* programBase;
         uint8* heapStart;
         int32* stack;
@@ -630,14 +687,14 @@ struct Runner
         int32 tos; // top of stack
         ErrorCode error;
 
-        template <typename Type1, typename... Args> void pushArguments (Type1 arg1, Args... args) noexcept   { pushArguments (args...); push32 (arg1); }
+        template <typename Type1, typename... Args> void pushArguments (Type1 arg1, Args... args) noexcept   { pushArguments (args...); pushArguments (arg1); }
         void pushArguments (int32 arg1) noexcept    { push32 (arg1); }
         void pushArguments (float arg1) noexcept    { push32 (Program::floatToInt (arg1)); }
 
         int16 readProgram16() noexcept              { auto v = Program::readInt16 (programCounter); programCounter += sizeof (int16); return v; }
         int32 readProgram32() noexcept              { auto v = Program::readInt32 (programCounter); programCounter += sizeof (int32); return v; }
 
-        void setError (ErrorCode e) noexcept        { error = e; programCounter = functionEnd; jassert (error == ErrorCode::ok); }
+        void setError (ErrorCode e) noexcept        { error = e; programCounter = programEnd; jassert (error == ErrorCode::ok); }
 
         bool checkStackUnderflow() noexcept         { if (stack <= stackEnd) return true; setError (ErrorCode::stackUnderflow); return false; }
         bool flushTopToStack() noexcept             { if (--stack < stackStart) { setError (ErrorCode::stackOverflow); return false; } *stack = tos; return true; }
