@@ -1,34 +1,27 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_HASHMAP_H_INCLUDED
-#define JUCE_HASHMAP_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -38,16 +31,20 @@
 */
 struct DefaultHashFunctions
 {
+    /** Generates a simple hash from an unsigned int. */
+    static int generateHash (uint32 key, int upperLimit) noexcept           { return (int) (key % (uint32) upperLimit); }
     /** Generates a simple hash from an integer. */
-    int generateHash (const int key, const int upperLimit) const noexcept        { return std::abs (key) % upperLimit; }
+    static int generateHash (int32 key, int upperLimit) noexcept            { return generateHash ((uint32) key, upperLimit); }
+    /** Generates a simple hash from a uint64. */
+    static int generateHash (uint64 key, int upperLimit) noexcept           { return (int) (key % (uint64) upperLimit); }
     /** Generates a simple hash from an int64. */
-    int generateHash (const int64 key, const int upperLimit) const noexcept      { return std::abs ((int) key) % upperLimit; }
+    static int generateHash (int64 key, int upperLimit) noexcept            { return generateHash ((uint64) key, upperLimit); }
     /** Generates a simple hash from a string. */
-    int generateHash (const String& key, const int upperLimit) const noexcept    { return (int) (((uint32) key.hashCode()) % (uint32) upperLimit); }
+    static int generateHash (const String& key, int upperLimit) noexcept    { return generateHash ((uint32) key.hashCode(), upperLimit); }
     /** Generates a simple hash from a variant. */
-    int generateHash (const var& key, const int upperLimit) const noexcept       { return generateHash (key.toString(), upperLimit); }
+    static int generateHash (const var& key, int upperLimit) noexcept       { return generateHash (key.toString(), upperLimit); }
     /** Generates a simple hash from a void ptr. */
-    int generateHash (const void* key, const int upperLimit) const noexcept      { return (int)(((pointer_sized_uint) key) % ((pointer_sized_uint) upperLimit)); }
+    static int generateHash (const void* key, int upperLimit) noexcept      { return generateHash ((pointer_sized_uint) key, upperLimit); }
 };
 
 
@@ -100,8 +97,8 @@ template <typename KeyType,
 class HashMap
 {
 private:
-    typedef PARAMETER_TYPE (KeyType)   KeyTypeParameter;
-    typedef PARAMETER_TYPE (ValueType) ValueTypeParameter;
+    typedef typename TypeHelpers::ParameterType<KeyType>::type   KeyTypeParameter;
+    typedef typename TypeHelpers::ParameterType<ValueType>::type ValueTypeParameter;
 
 public:
     //==============================================================================
@@ -137,9 +134,9 @@ public:
     {
         const ScopedLockType sl (getLock());
 
-        for (int i = hashSlots.size(); --i >= 0;)
+        for (auto i = hashSlots.size(); --i >= 0;)
         {
-            HashEntry* h = hashSlots.getUnchecked(i);
+            auto* h = hashSlots.getUnchecked(i);
 
             while (h != nullptr)
             {
@@ -168,24 +165,44 @@ public:
     {
         const ScopedLockType sl (getLock());
 
-        for (const HashEntry* entry = hashSlots.getUnchecked (generateHashFor (keyToLookFor)); entry != nullptr; entry = entry->nextEntry)
-            if (entry->key == keyToLookFor)
-                return entry->value;
+        if (auto* entry = getEntry (getSlot (keyToLookFor), keyToLookFor))
+            return entry->value;
 
         return ValueType();
     }
 
+    /** Returns a reference to the value corresponding to a given key.
+        If the map doesn't contain the key, a default instance of the value type is
+        added to the map and a reference to this is returned.
+        @param keyToLookFor    the key of the item being requested
+    */
+    inline ValueType& getReference (KeyTypeParameter keyToLookFor)
+    {
+        const ScopedLockType sl (getLock());
+        auto hashIndex = generateHashFor (keyToLookFor, getNumSlots());
+
+        auto* firstEntry = hashSlots.getUnchecked (hashIndex);
+
+        if (auto* entry = getEntry (firstEntry, keyToLookFor))
+            return entry->value;
+
+        auto* entry = new HashEntry (keyToLookFor, ValueType(), firstEntry);
+        hashSlots.set (hashIndex, entry);
+        ++totalNumItems;
+
+        if (totalNumItems > (getNumSlots() * 3) / 2)
+            remapTable (getNumSlots() * 2);
+
+        return entry->value;
+    }
+
     //==============================================================================
     /** Returns true if the map contains an item with the specied key. */
     bool contains (KeyTypeParameter keyToLookFor) const
     {
         const ScopedLockType sl (getLock());
 
-        for (const HashEntry* entry = hashSlots.getUnchecked (generateHashFor (keyToLookFor)); entry != nullptr; entry = entry->nextEntry)
-            if (entry->key == keyToLookFor)
-                return true;
-
-        return false;
+        return (getEntry (getSlot (keyToLookFor), keyToLookFor) != nullptr);
     }
 
     /** Returns true if the hash contains at least one occurrence of a given value. */
@@ -193,8 +210,8 @@ public:
     {
         const ScopedLockType sl (getLock());
 
-        for (int i = getNumSlots(); --i >= 0;)
-            for (const HashEntry* entry = hashSlots.getUnchecked(i); entry != nullptr; entry = entry->nextEntry)
+        for (auto i = getNumSlots(); --i >= 0;)
+            for (auto* entry = hashSlots.getUnchecked(i); entry != nullptr; entry = entry->nextEntry)
                 if (entry->value == valueToLookFor)
                     return true;
 
@@ -206,35 +223,14 @@ public:
         If there's already an item with the given key, this will replace its value. Otherwise, a new item
         will be added to the map.
     */
-    void set (KeyTypeParameter newKey, ValueTypeParameter newValue)
-    {
-        const ScopedLockType sl (getLock());
-        const int hashIndex = generateHashFor (newKey);
-
-        HashEntry* const firstEntry = hashSlots.getUnchecked (hashIndex);
-
-        for (HashEntry* entry = firstEntry; entry != nullptr; entry = entry->nextEntry)
-        {
-            if (entry->key == newKey)
-            {
-                entry->value = newValue;
-                return;
-            }
-        }
-
-        hashSlots.set (hashIndex, new HashEntry (newKey, newValue, firstEntry));
-        ++totalNumItems;
-
-        if (totalNumItems > (getNumSlots() * 3) / 2)
-            remapTable (getNumSlots() * 2);
-    }
+    void set (KeyTypeParameter newKey, ValueTypeParameter newValue)        { getReference (newKey) = newValue; }
 
     /** Removes an item with the given key. */
     void remove (KeyTypeParameter keyToRemove)
     {
         const ScopedLockType sl (getLock());
-        const int hashIndex = generateHashFor (keyToRemove);
-        HashEntry* entry = hashSlots.getUnchecked (hashIndex);
+        auto hashIndex = generateHashFor (keyToRemove, getNumSlots());
+        auto* entry = hashSlots.getUnchecked (hashIndex);
         HashEntry* previous = nullptr;
 
         while (entry != nullptr)
@@ -265,9 +261,9 @@ public:
     {
         const ScopedLockType sl (getLock());
 
-        for (int i = getNumSlots(); --i >= 0;)
+        for (auto i = getNumSlots(); --i >= 0;)
         {
-            HashEntry* entry = hashSlots.getUnchecked(i);
+            auto* entry = hashSlots.getUnchecked(i);
             HashEntry* previous = nullptr;
 
             while (entry != nullptr)
@@ -300,13 +296,27 @@ public:
     */
     void remapTable (int newNumberOfSlots)
     {
-        HashMap newTable (newNumberOfSlots);
+        const ScopedLockType sl (getLock());
 
-        for (int i = getNumSlots(); --i >= 0;)
-            for (const HashEntry* entry = hashSlots.getUnchecked(i); entry != nullptr; entry = entry->nextEntry)
-                newTable.set (entry->key, entry->value);
+        Array<HashEntry*> newSlots;
+        newSlots.insertMultiple (0, nullptr, newNumberOfSlots);
 
-        swapWith (newTable);
+        for (auto i = getNumSlots(); --i >= 0;)
+        {
+            HashEntry* nextEntry = nullptr;
+
+            for (auto* entry = hashSlots.getUnchecked(i); entry != nullptr; entry = nextEntry)
+            {
+                auto hashIndex = generateHashFor (entry->key, newNumberOfSlots);
+
+                nextEntry = entry->nextEntry;
+                entry->nextEntry = newSlots.getUnchecked (hashIndex);
+
+                newSlots.set (hashIndex, entry);
+            }
+        }
+
+        hashSlots.swapWith (newSlots);
     }
 
     /** Returns the number of slots which are available for hashing.
@@ -380,19 +390,21 @@ public:
 
         @see HashMap
     */
-    class Iterator
+    struct Iterator
     {
-    public:
-        //==============================================================================
-        Iterator (const HashMap& hashMapToIterate)
+        Iterator (const HashMap& hashMapToIterate) noexcept
             : hashMap (hashMapToIterate), entry (nullptr), index (0)
         {}
 
+        Iterator (const Iterator& other) noexcept
+            : hashMap (other.hashMap), entry (other.entry), index (other.index)
+        {}
+
         /** Moves to the next item, if one is available.
             When this returns true, you can get the item's key and value using getKey() and
             getValue(). If it returns false, the iteration has finished and you should stop.
         */
-        bool next()
+        bool next() noexcept
         {
             if (entry != nullptr)
                 entry = entry->nextEntry;
@@ -431,34 +443,58 @@ public:
             index = 0;
         }
 
+        Iterator& operator++() noexcept                         { next(); return *this; }
+        ValueType operator*() const                             { return getValue(); }
+        bool operator!= (const Iterator& other) const noexcept  { return entry != other.entry || index != other.index; }
+        void resetToEnd() noexcept                              { index = hashMap.getNumSlots(); }
+
     private:
         //==============================================================================
         const HashMap& hashMap;
         HashEntry* entry;
         int index;
 
-        JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Iterator)
+        // using the copy constructor is ok, but you cannot assign iterators
+        Iterator& operator= (const Iterator&) = delete;
+
+        JUCE_LEAK_DETECTOR (Iterator)
     };
 
+    /** Returns a start iterator for the values in this tree. */
+    Iterator begin() const noexcept             { Iterator i (*this); i.next(); return i; }
+
+    /** Returns an end iterator for the values in this tree. */
+    Iterator end() const noexcept               { Iterator i (*this); i.resetToEnd(); return i; }
+
 private:
     //==============================================================================
     enum { defaultHashTableSize = 101 };
-    friend class Iterator;
+    friend struct Iterator;
 
     HashFunctionType hashFunctionToUse;
     Array<HashEntry*> hashSlots;
     int totalNumItems;
     TypeOfCriticalSectionToUse lock;
 
-    int generateHashFor (KeyTypeParameter key) const
+    int generateHashFor (KeyTypeParameter key, int numSlots) const
     {
-        const int hash = hashFunctionToUse.generateHash (key, getNumSlots());
-        jassert (isPositiveAndBelow (hash, getNumSlots())); // your hash function is generating out-of-range numbers!
+        const int hash = hashFunctionToUse.generateHash (key, numSlots);
+        jassert (isPositiveAndBelow (hash, numSlots)); // your hash function is generating out-of-range numbers!
         return hash;
     }
 
+    static inline HashEntry* getEntry (HashEntry* firstEntry, KeyType keyToLookFor) noexcept
+    {
+        for (auto* entry = firstEntry; entry != nullptr; entry = entry->nextEntry)
+            if (entry->key == keyToLookFor)
+                return entry;
+
+        return nullptr;
+    }
+
+    inline HashEntry* getSlot (KeyType key) const noexcept     { return hashSlots.getUnchecked (generateHashFor (key, getNumSlots())); }
+
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (HashMap)
 };
 
-
-#endif   // JUCE_HASHMAP_H_INCLUDED
+} // namespace juce
