@@ -1,34 +1,27 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_FILE_H_INCLUDED
-#define JUCE_FILE_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -42,7 +35,7 @@
 
     @see FileInputStream, FileOutputStream
 */
-class JUCE_API  File
+class JUCE_API  File final
 {
 public:
     //==============================================================================
@@ -88,19 +81,11 @@ public:
     /** Copies from another file object. */
     File& operator= (const File& otherFile);
 
-   #if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
+    /** Move constructor */
     File (File&&) noexcept;
-    File& operator= (File&&) noexcept;
-   #endif
 
-    //==============================================================================
-   #if JUCE_ALLOW_STATIC_NULL_VARIABLES
-    /** This static constant is used for referring to an 'invalid' file.
-        Bear in mind that you should avoid this kind of static variable, and always prefer
-        to use File() or {} if you need a default-constructed File object.
-    */
-    static const File nonexistent;
-   #endif
+    /** Move assignment operator */
+    File& operator= (File&&) noexcept;
 
     //==============================================================================
     /** Checks whether the file actually exists.
@@ -126,6 +111,14 @@ public:
     */
     bool isDirectory() const;
 
+    /** Checks whether the path of this file represents the root of a file system,
+        irrespective of its existance.
+
+        This will return true for "C:", "D:", etc on Windows and "/" on other
+        platforms.
+    */
+    bool isRoot() const;
+
     /** Returns the size of the file in bytes.
 
         @returns    the number of bytes in the file, or 0 if it doesn't exist.
@@ -273,6 +266,9 @@ public:
     /** Returns the directory that contains this file or directory.
 
         e.g. for "/moose/fish/foo.txt" this will return "/moose/fish".
+
+        If you are already at the root directory ("/" or "C:") then this method will
+        return the root directory.
     */
     File getParentDirectory() const;
 
@@ -434,6 +430,9 @@ public:
 
         If it already exists or is a directory, this method will do nothing.
 
+        If the parent directories of the File do not exist then this method will
+        recursively create the parent directories.
+
         @returns    a result to indicate whether the file was created successfully,
                     or an error message if it failed.
         @see createDirectory
@@ -545,27 +544,33 @@ public:
         ignoreHiddenFiles           = 4     /**< Add this flag to avoid returning any hidden files in the results. */
     };
 
-    /** Searches inside a directory for files matching a wildcard pattern.
+    /** Searches this directory for files matching a wildcard pattern.
 
         Assuming that this file is a directory, this method will search it
         for either files or subdirectories whose names match a filename pattern.
+        Note that the order in which files are returned is completely undefined!
 
-        @param results                  an array to which File objects will be added for the
-                                        files that the search comes up with
         @param whatToLookFor            a value from the TypesOfFileToFind enum, specifying whether to
                                         return files, directories, or both. If the ignoreHiddenFiles flag
                                         is also added to this value, hidden files won't be returned
         @param searchRecursively        if true, all subdirectories will be recursed into to do
                                         an exhaustive search
         @param wildCardPattern          the filename pattern to search for, e.g. "*.txt"
-        @returns                        the number of results that have been found
+        @returns                        the set of files that were found
 
         @see getNumberOfChildFiles, DirectoryIterator
     */
-    int findChildFiles (Array<File>& results,
-                        int whatToLookFor,
-                        bool searchRecursively,
-                        const String& wildCardPattern = "*") const;
+    Array<File> findChildFiles (int whatToLookFor,
+                                bool searchRecursively,
+                                const String& wildCardPattern = "*") const;
+
+    /** Searches inside a directory for files matching a wildcard pattern.
+        Note that there's a newer, better version of this method which returns the results
+        array, and in almost all cases, you should use that one instead! This one is kept around
+        mainly for legacy code to use.
+    */
+    int findChildFiles (Array<File>& results, int whatToLookFor,
+                        bool searchRecursively, const String& wildCardPattern = "*") const;
 
     /** Searches inside a directory and counts how many files match a wildcard pattern.
 
@@ -805,6 +810,7 @@ public:
             On Windows, this might be "\Documents and Settings\username\Application Data".
             On the Mac, it might be "~/Library". If you're going to store your settings in here,
             always create your own sub-folder to put them in, to avoid making a mess.
+            On GNU/Linux it is "~/.config".
         */
         userApplicationDataDirectory,
 
@@ -814,6 +820,8 @@ public:
             On the Mac it'll be "/Library", on Windows, it could be something like
             "\Documents and Settings\All Users\Application Data".
 
+            On GNU/Linux it is "/opt".
+
             Depending on the setup, this folder may be read-only.
         */
         commonApplicationDataDirectory,
@@ -863,16 +871,27 @@ public:
         /** In a plugin, this will return the path of the host executable. */
         hostApplicationPath,
 
-       #if JUCE_WINDOWS
+       #if JUCE_WINDOWS || DOXYGEN
         /** On a Windows machine, returns the location of the Windows/System32 folder. */
         windowsSystemDirectory,
        #endif
 
         /** The directory in which applications normally get installed.
-            So on windows, this would be something like "c:\program files", on the
+            So on windows, this would be something like "C:\Program Files", on the
             Mac "/Applications", or "/usr" on linux.
         */
-        globalApplicationsDirectory
+        globalApplicationsDirectory,
+
+       #if JUCE_WINDOWS || DOXYGEN
+        /** On a Windows machine, returns the directory in which 32 bit applications
+            normally get installed. On a 64 bit machine this would be something like
+            "C:\Program Files (x86)", whereas for 32 bit machines this would match
+            globalApplicationsDirectory and be something like "C:\Program Files".
+
+            @see globalApplicationsDirectory
+        */
+        globalApplicationsDirectoryX86
+       #endif
     };
 
     /** Finds the location of a special type of file or directory, such as a home folder or
@@ -908,12 +927,12 @@ public:
     /** The system-specific file separator character.
         On Windows, this will be '\', on Mac/Linux, it'll be '/'
     */
-    static const juce_wchar separator;
+    static juce_wchar getSeparatorChar();
 
     /** The system-specific file separator character, as a string.
         On Windows, this will be '\', on Mac/Linux, it'll be '/'
     */
-    static const String separatorString;
+    static StringRef getSeparatorString();
 
     //==============================================================================
     /** Returns a version of a filename with any illegal characters removed.
@@ -966,7 +985,7 @@ public:
     */
     File getLinkedTarget() const;
 
-   #if JUCE_WINDOWS
+   #if JUCE_WINDOWS || DOXYGEN
     /** Windows ONLY - Creates a win32 .LNK shortcut file that links to this file. */
     bool createShortcut (const String& description, const File& linkFileToCreate) const;
 
@@ -988,6 +1007,42 @@ public:
     void addToDock() const;
    #endif
 
+    //==============================================================================
+    struct NaturalFileComparator
+    {
+        NaturalFileComparator (bool shouldPutFoldersFirst) noexcept : foldersFirst (shouldPutFoldersFirst) {}
+
+        int compareElements (const File& firstFile, const File& secondFile) const
+        {
+            if (foldersFirst && (firstFile.isDirectory() != secondFile.isDirectory()))
+                return firstFile.isDirectory() ? -1 : 1;
+
+           #if NAMES_ARE_CASE_SENSITIVE
+            return firstFile.getFullPathName().compareNatural (secondFile.getFullPathName(), true);
+           #else
+            return firstFile.getFullPathName().compareNatural (secondFile.getFullPathName(), false);
+           #endif
+        }
+
+        bool foldersFirst;
+    };
+
+   #if (! defined(DOXYGEN)) && (! defined (JUCE_GCC))
+    // Deprecated: use File::getSeparatorChar() and File::getSeparatorString() instead!
+    JUCE_DEPRECATED (static const juce_wchar separator);
+    JUCE_DEPRECATED (static const StringRef separatorString);
+   #endif
+
+    //==============================================================================
+   #if JUCE_ALLOW_STATIC_NULL_VARIABLES
+    /** This was a static empty File object, but is now deprecated as it's too easy to accidentally
+        use it indirectly during a static constructor, leading to hard-to-find order-of-initialisation
+        problems.
+        @deprecated If you need a default-constructed File object, just use File() or {}.
+    */
+    static const File nonexistent;
+   #endif
+
 private:
     //==============================================================================
     String fullPath;
@@ -1005,4 +1060,4 @@ private:
     bool setFileExecutableInternal (bool) const;
 };
 
-#endif   // JUCE_FILE_H_INCLUDED
+} // namespace juce
