@@ -1,31 +1,28 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 #if JUCE_MSVC
  #pragma warning (push)
  #pragma warning (disable : 4127 4389 4018)
@@ -37,10 +34,19 @@
 
 #if JUCE_WINDOWS
  typedef int       juce_socklen_t;
+ typedef int       juce_recvsend_size_t;
  typedef SOCKET    SocketHandle;
+ static const SocketHandle invalidSocket = INVALID_SOCKET;
+#elif JUCE_ANDROID
+ typedef socklen_t juce_socklen_t;
+ typedef size_t    juce_recvsend_size_t;
+ typedef int       SocketHandle;
+ static const SocketHandle invalidSocket = -1;
 #else
  typedef socklen_t juce_socklen_t;
+ typedef socklen_t juce_recvsend_size_t;
  typedef int       SocketHandle;
+ static const SocketHandle invalidSocket = -1;
 #endif
 
 //==============================================================================
@@ -68,18 +74,18 @@ namespace SocketHelpers
     }
 
     template <typename Type>
-    static bool setOption (const SocketHandle handle, int mode, int property, Type value) noexcept
+    static bool setOption (SocketHandle handle, int mode, int property, Type value) noexcept
     {
         return setsockopt (handle, mode, property, reinterpret_cast<const char*> (&value), sizeof (value)) == 0;
     }
 
     template <typename Type>
-    static bool setOption (const SocketHandle handle, int property, Type value) noexcept
+    static bool setOption (SocketHandle handle, int property, Type value) noexcept
     {
         return setOption (handle, SOL_SOCKET, property, value);
     }
 
-    static bool resetSocketOptions (const SocketHandle handle, const bool isDatagram, const bool allowBroadcast) noexcept
+    static bool resetSocketOptions (SocketHandle handle, bool isDatagram, bool allowBroadcast) noexcept
     {
         return handle > 0
                 && setOption (handle, SO_RCVBUF, (int) 65536)
@@ -89,7 +95,7 @@ namespace SocketHelpers
     }
 
     static void closeSocket (volatile int& handle, CriticalSection& readLock,
-                             const bool isListener, int portNumber, bool& connected) noexcept
+                             bool isListener, int portNumber, bool& connected) noexcept
     {
         const SocketHandle h = handle;
         handle = -1;
@@ -97,11 +103,11 @@ namespace SocketHelpers
        #if JUCE_WINDOWS
         ignoreUnused (portNumber, isListener, readLock);
 
-        if (h != SOCKET_ERROR || connected)
+        if (h != (unsigned) SOCKET_ERROR || connected)
             closesocket (h);
 
         // make sure any read process finishes before we delete the socket
-        CriticalSection::ScopedLockType lock(readLock);
+        CriticalSection::ScopedLockType lock (readLock);
         connected = false;
        #else
         if (connected)
@@ -127,7 +133,7 @@ namespace SocketHelpers
                 // a chance to process before close is called. On Mac OS X shutdown
                 // does not unblock a select call, so using a lock here will dead-lock
                 // both threads.
-               #if JUCE_LINUX
+               #if JUCE_LINUX || JUCE_ANDROID
                 CriticalSection::ScopedLockType lock (readLock);
                 ::close (h);
                #else
@@ -139,7 +145,7 @@ namespace SocketHelpers
        #endif
     }
 
-    static bool bindSocket (const SocketHandle handle, int port, const String& address) noexcept
+    static bool bindSocket (SocketHandle handle, int port, const String& address) noexcept
     {
         if (handle <= 0 || ! isValidPortNumber (port))
             return false;
@@ -155,7 +161,7 @@ namespace SocketHelpers
         return ::bind (handle, (struct sockaddr*) &addr, sizeof (addr)) >= 0;
     }
 
-    static int getBoundPort (const SocketHandle handle) noexcept
+    static int getBoundPort (SocketHandle handle) noexcept
     {
         if (handle > 0)
         {
@@ -169,10 +175,21 @@ namespace SocketHelpers
         return -1;
     }
 
-    static int readSocket (const SocketHandle handle,
+    static String getConnectedAddress (SocketHandle handle) noexcept
+    {
+        struct sockaddr_in addr;
+        socklen_t len = sizeof (addr);
+
+        if (getpeername (handle, (struct sockaddr*) &addr, &len) >= 0)
+            return inet_ntoa (addr.sin_addr);
+
+        return String ("0.0.0.0");
+    }
+
+    static int readSocket (SocketHandle handle,
                            void* const destBuffer, const int maxBytesToRead,
                            bool volatile& connected,
-                           const bool blockUntilSpecifiedAmountHasArrived,
+                           bool blockUntilSpecifiedAmountHasArrived,
                            CriticalSection& readLock,
                            String* senderIP = nullptr,
                            int* senderPort = nullptr) noexcept
@@ -182,8 +199,8 @@ namespace SocketHelpers
         while (bytesRead < maxBytesToRead)
         {
             long bytesThisTime = -1;
-            char* const buffer = static_cast<char*> (destBuffer) + bytesRead;
-            const juce_socklen_t numToRead = (juce_socklen_t) (maxBytesToRead - bytesRead);
+            auto buffer = static_cast<char*> (destBuffer) + bytesRead;
+            auto numToRead = (juce_recvsend_size_t) (maxBytesToRead - bytesRead);
 
             {
                 // avoid race-condition
@@ -291,7 +308,7 @@ namespace SocketHelpers
         return FD_ISSET (h, forReading ? &rset : &wset) ? 1 : 0;
     }
 
-    static bool setSocketBlockingState (const SocketHandle handle, const bool shouldBlock) noexcept
+    static bool setSocketBlockingState (SocketHandle handle, const bool shouldBlock) noexcept
     {
        #if JUCE_WINDOWS
         u_long nonBlocking = shouldBlock ? 0 : (u_long) 1;
@@ -321,7 +338,8 @@ namespace SocketHelpers
         hints.ai_flags = AI_NUMERICSERV;
 
         struct addrinfo* info = nullptr;
-        if (getaddrinfo (hostName.toUTF8(), String (portNumber).toUTF8(), &hints, &info) == 0)
+
+        if (getaddrinfo (hostName.toRawUTF8(), String (portNumber).toRawUTF8(), &hints, &info) == 0)
             return info;
 
         return nullptr;
@@ -335,16 +353,16 @@ namespace SocketHelpers
     {
         bool success = false;
 
-        if (struct addrinfo* info = getAddressInfo (false, hostName, portNumber))
+        if (auto* info = getAddressInfo (false, hostName, portNumber))
         {
-            for (struct addrinfo* i = info; i != nullptr; i = i->ai_next)
+            for (auto* i = info; i != nullptr; i = i->ai_next)
             {
-                const SocketHandle newHandle = socket (i->ai_family, i->ai_socktype, 0);
+                auto newHandle = socket (i->ai_family, i->ai_socktype, 0);
 
-                if (newHandle >= 0)
+                if (newHandle != invalidSocket)
                 {
                     setSocketBlockingState (newHandle, false);
-                    const int result = ::connect (newHandle, i->ai_addr, (socklen_t) i->ai_addrlen);
+                    auto result = ::connect (newHandle, i->ai_addr, (socklen_t) i->ai_addrlen);
                     success = (result >= 0);
 
                     if (! success)
@@ -356,6 +374,7 @@ namespace SocketHelpers
                        #endif
                         {
                             const volatile int cvHandle = (int) newHandle;
+
                             if (waitForReadiness (cvHandle, readLock, false, timeOutMillisecs) == 1)
                                 success = true;
                         }
@@ -413,10 +432,6 @@ namespace SocketHelpers
 
 //==============================================================================
 StreamingSocket::StreamingSocket()
-    : portNumber (0),
-      handle (-1),
-      connected (false),
-      isListener (false)
 {
     SocketHelpers::initSockets();
 }
@@ -425,8 +440,7 @@ StreamingSocket::StreamingSocket (const String& host, int portNum, int h)
     : hostName (host),
       portNumber (portNum),
       handle (h),
-      connected (true),
-      isListener (false)
+      connected (true)
 {
     jassert (SocketHelpers::isValidPortNumber (portNum));
 
@@ -452,7 +466,7 @@ int StreamingSocket::write (const void* sourceBuffer, const int numBytesToWrite)
     if (isListener || ! connected)
         return -1;
 
-    return (int) ::send (handle, (const char*) sourceBuffer, (juce_socklen_t) numBytesToWrite, 0);
+    return (int) ::send (handle, (const char*) sourceBuffer, (juce_recvsend_size_t) numBytesToWrite, 0);
 }
 
 //==============================================================================
@@ -564,7 +578,7 @@ StreamingSocket* StreamingSocket::waitForNextConnection() const
     {
         struct sockaddr_storage address;
         juce_socklen_t len = sizeof (address);
-        const int newSocket = (int) accept (handle, (struct sockaddr*) &address, &len);
+        auto newSocket = (int) accept (handle, (struct sockaddr*) &address, &len);
 
         if (newSocket >= 0 && connected)
             return new StreamingSocket (inet_ntoa (((struct sockaddr_in*) &address)->sin_addr),
@@ -576,17 +590,24 @@ StreamingSocket* StreamingSocket::waitForNextConnection() const
 
 bool StreamingSocket::isLocal() const noexcept
 {
+    if (! isConnected())
+        return false;
+
+    Array<IPAddress> localAddresses;
+    IPAddress::findAllAddresses (localAddresses);
+    IPAddress currentIP (SocketHelpers::getConnectedAddress (handle));
+
+    for (auto& a : localAddresses)
+        if (a == currentIP)
+            return true;
+
     return hostName == "127.0.0.1";
 }
 
 
 //==============================================================================
 //==============================================================================
-DatagramSocket::DatagramSocket (const bool canBroadcast)
-    : handle (-1),
-      isBound (false),
-      lastServerPort (-1),
-      lastServerAddress (nullptr)
+DatagramSocket::DatagramSocket (bool canBroadcast)
 {
     SocketHelpers::initSockets();
 
@@ -612,7 +633,7 @@ void DatagramSocket::shutdown()
     if (handle < 0)
         return;
 
-    int copyOfHandle = handle;
+    auto copyOfHandle = handle;
     handle = -1;
     bool connected = false;
     SocketHelpers::closeSocket (copyOfHandle, readLock, false, 0, connected);
@@ -639,10 +660,7 @@ bool DatagramSocket::bindToPort (const int port, const String& addr)
 
 int DatagramSocket::getBoundPort() const noexcept
 {
-    if (handle < 0)
-        return -1;
-
-    return isBound ? SocketHelpers::getBoundPort (handle) : -1;
+    return (handle >= 0 && isBound) ? SocketHelpers::getBoundPort (handle) : -1;
 }
 
 //==============================================================================
@@ -703,7 +721,7 @@ int DatagramSocket::write (const String& remoteHostname, int remotePortNumber,
     }
 
     return (int) ::sendto (handle, (const char*) sourceBuffer,
-                           (juce_socklen_t) numBytesToWrite, 0,
+                           (juce_recvsend_size_t) numBytesToWrite, 0,
                            info->ai_addr, (socklen_t) info->ai_addrlen);
 }
 
@@ -723,9 +741,19 @@ bool DatagramSocket::leaveMulticast (const String& multicastIPAddress)
     return SocketHelpers::multicast (handle, multicastIPAddress, lastBindAddress, false);
 }
 
+bool DatagramSocket::setMulticastLoopbackEnabled (bool enable)
+{
+    if (! isBound || handle < 0)
+        return false;
+
+    return SocketHelpers::setOption<bool> (handle, IPPROTO_IP, IP_MULTICAST_LOOP, enable);
+}
+
 bool DatagramSocket::setEnablePortReuse (bool enabled)
 {
-   #if ! JUCE_ANDROID
+   #if JUCE_ANDROID
+    ignoreUnused (enabled);
+   #else
     if (handle >= 0)
         return SocketHelpers::setOption (handle,
                                         #if JUCE_WINDOWS || JUCE_LINUX
@@ -742,3 +770,5 @@ bool DatagramSocket::setEnablePortReuse (bool enabled)
 #if JUCE_MSVC
  #pragma warning (pop)
 #endif
+
+} // namespace juce
