@@ -1,34 +1,27 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_SOCKET_H_INCLUDED
-#define JUCE_SOCKET_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -39,7 +32,7 @@
 
     @see DatagramSocket, InterprocessConnection, InterprocessConnectionServer
 */
-class JUCE_API  StreamingSocket
+class JUCE_API  StreamingSocket  final
 {
 public:
     //==============================================================================
@@ -81,7 +74,8 @@ public:
 
         This is useful if you need to know to which port the OS has actually bound your
         socket when calling the constructor or bindToPort with zero as the
-        localPortNumber argument. Returns -1 if the function fails. */
+        localPortNumber argument. Returns -1 if the function fails.
+    */
     int getBoundPort() const noexcept;
 
     /** Tries to connect the socket to hostname:port.
@@ -181,8 +175,8 @@ public:
 private:
     //==============================================================================
     String hostName;
-    int volatile portNumber, handle;
-    bool connected, isListener;
+    int volatile portNumber = 0, handle = -1;
+    bool connected = false, isListener = false;
     mutable CriticalSection readLock;
 
     StreamingSocket (const String& hostname, int portNumber, int handle);
@@ -200,7 +194,7 @@ private:
 
     @see StreamingSocket, InterprocessConnection, InterprocessConnectionServer
 */
-class JUCE_API  DatagramSocket
+class JUCE_API  DatagramSocket  final
 {
 public:
     //==============================================================================
@@ -321,18 +315,24 @@ public:
     void shutdown();
 
     //==============================================================================
-    /** Join a multicast group
+    /** Join a multicast group.
 
         @returns true if it succeeds.
     */
     bool joinMulticast (const String& multicastIPAddress);
 
-    /** Leave a multicast group
+    /** Leave a multicast group.
 
         @returns true if it succeeds.
     */
     bool leaveMulticast (const String& multicastIPAddress);
 
+    /** Enables or disables multicast loopback.
+
+        @returns true if it succeeds.
+    */
+    bool setMulticastLoopbackEnabled (bool enableLoopback);
+
     //==============================================================================
     /** Allow other applications to re-use the port.
 
@@ -346,15 +346,14 @@ public:
 
 private:
     //==============================================================================
-    int handle;
-    bool isBound;
+    int handle = -1;
+    bool isBound = false;
     String lastBindAddress, lastServerHost;
-    int lastServerPort;
-    void* lastServerAddress;
+    int lastServerPort = -1;
+    void* lastServerAddress = nullptr;
     mutable CriticalSection readLock;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (DatagramSocket)
 };
 
-
-#endif   // JUCE_SOCKET_H_INCLUDED
+} // namespace juce
