@@ -1,34 +1,29 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_URL_H_INCLUDED
-#define JUCE_URL_H_INCLUDED
+namespace juce
+{
 
+class WebInputStream;
 
 //==============================================================================
 /**
@@ -42,7 +37,7 @@ class JUCE_API  URL
 public:
     //==============================================================================
     /** Creates an empty URL. */
-    URL();
+    URL() noexcept;
 
     /** Creates a URL from a string.
         This will parse any embedded parameters after a '?' character and store them
@@ -51,15 +46,19 @@ public:
     */
     URL (const String& url);
 
-    /** Creates a copy of another URL. */
-    URL (const URL& other);
+    URL (const URL&) = default;
+    URL& operator= (const URL&) = default;
+
+    // VS2013 can't default move constructors and assignments
+    URL (URL&&);
+    URL& operator= (URL&&);
+
+    /** Creates URL referring to a local file on your disk using the file:// scheme. */
+    explicit URL (File);
 
     /** Destructor. */
     ~URL();
 
-    /** Copies this URL from another one. */
-    URL& operator= (const URL& other);
-
     /** Compares two URLs.
         All aspects of the URLs must be identical for them to match, including any parameters,
         upload files, etc.
@@ -83,31 +82,50 @@ public:
     bool isWellFormed() const;
 
     /** Returns just the domain part of the URL.
-
         E.g. for "http://www.xyz.com/foobar", this will return "www.xyz.com".
     */
     String getDomain() const;
 
     /** Returns the path part of the URL.
-
         E.g. for "http://www.xyz.com/foo/bar?x=1", this will return "foo/bar".
     */
     String getSubPath() const;
 
     /** Returns the scheme of the URL.
-
         E.g. for "http://www.xyz.com/foobar", this will return "http". (It won't
         include the colon).
     */
     String getScheme() const;
 
+    /** Returns true if this URL refers to a local file. */
+    bool isLocalFile() const;
+
+    /** Returns the file path of the local file to which this URL refers to.
+        If the URL does not represent a local file URL (i.e. the URL's scheme is not 'file')
+        then this method will assert.
+
+        This method also supports converting Android's content:// URLs to
+        local file paths.
+
+        @see isLocalFile
+    */
+    File getLocalFile() const;
+
+    /** Returns the file name. For all but Android's content:// scheme, it will
+        simply return the last segment of the URL.
+        E.g. for "http://www.xyz.com/foo/bar.txt", this will return "bar.txt".
+
+        For Android's content:// scheme, it will attempt to resolve the filename
+        located under the URL.
+    */
+    String getFileName() const;
+
     /** Attempts to read a port number from the URL.
         @returns the port number, or 0 if none is explicitly specified.
     */
     int getPort() const;
 
     /** Returns a new version of this URL with a different domain and path.
-
         E.g. if the URL is "http://www.xyz.com/foo?x=1" and you call this with
         "abc.com/zzz", it'll return "http://abc.com/zzz?x=1".
         @see withNewSubPath
@@ -115,7 +133,6 @@ public:
     URL withNewDomainAndPath (const String& newFullPath) const;
 
     /** Returns a new version of this URL with a different sub-path.
-
         E.g. if the URL is "http://www.xyz.com/foo?x=1" and you call this with
         "bar", it'll return "http://www.xyz.com/bar?x=1".
         @see withNewDomainAndPath
@@ -123,7 +140,6 @@ public:
     URL withNewSubPath (const String& newPath) const;
 
     /** Returns a new URL that refers to a sub-path relative to this one.
-
         E.g. if the URL is "http://www.xyz.com/foo" and you call this with
         "bar", it'll return "http://www.xyz.com/foo/bar". Note that there's no way for
         this method to know whether the original URL is a file or directory, so it's
@@ -139,7 +155,6 @@ public:
     /** Returns a copy of this URL, with a GET or POST parameter added to the end.
 
         Any control characters in the value will be encoded.
-
         e.g. calling "withParameter ("amount", "some fish") for the url "www.fish.com"
         would produce a new url whose toString(true) method would return
         "www.fish.com?amount=some+fish".
@@ -239,27 +254,24 @@ public:
     URL withPOSTData (const MemoryBlock& postData) const;
 
     /** Returns the data that was set using withPOSTData(). */
-    String getPostData() const noexcept                  { return postData.toString(); }
+    String getPostData() const noexcept                             { return postData.toString(); }
 
     /** Returns the data that was set using withPOSTData() as MemoryBlock. */
-    const MemoryBlock& getPostDataAsMemoryBlock() const noexcept { return postData; }
+    const MemoryBlock& getPostDataAsMemoryBlock() const noexcept    { return postData; }
 
     //==============================================================================
     /** Tries to launch the system's default browser to open the URL.
-
         Returns true if this seems to have worked.
     */
     bool launchInDefaultBrowser() const;
 
     //==============================================================================
     /** Takes a guess as to whether a string might be a valid website address.
-
         This isn't foolproof!
     */
     static bool isProbablyAWebsiteURL (const String& possibleURL);
 
     /** Takes a guess as to whether a string might be a valid email address.
-
         This isn't foolproof!
     */
     static bool isProbablyAnEmailAddress (const String& possibleEmailAddress);
@@ -274,9 +286,18 @@ public:
 
     /** Attempts to open a stream that can read from this URL.
 
+        Note that this method will block until the first byte of data has been received or an
+        error has occurred.
+
         Note that on some platforms (Android, for example) it's not permitted to do any network
         action from the message thread, so you must only call it from a background thread.
 
+        Unless the URL represents a local file, this method returns an instance of a
+        WebInputStream. You can use dynamic_cast to cast the return value to a WebInputStream
+        which allows you more fine-grained control of the transfer process.
+
+        If the URL represents a local file, then this method simply returns a FileInputStream.
+
         @param doPostLikeRequest if true, the parameters added to this class will be transferred
                                  via the HTTP headers which is typical for POST requests. Otherwise
                                  the parameters will be added to the URL address. Additionally,
@@ -315,6 +336,92 @@ public:
                                     int numRedirectsToFollow = 5,
                                     String httpRequestCmd = String()) const;
 
+    /** Attempts to open an output stream to a URL for writing
+
+        This method can only be used for certain scheme types such as local files
+        and content:// URIs on Android.
+    */
+    OutputStream* createOutputStream() const;
+
+    //==============================================================================
+    /** Represents a download task.
+        Returned by downloadToFile to allow querying and controling the download task.
+    */
+    class DownloadTask
+    {
+    public:
+        struct Listener
+        {
+            virtual ~Listener();
+
+            /** Called when the download has finished. Be aware that this callback may
+                come on an arbitrary thread. */
+            virtual void finished (DownloadTask* task, bool success) = 0;
+
+            /** Called periodically by the OS to indicate download progress.
+                Beware that this callback may come on an arbitrary thread.
+            */
+            virtual void progress (DownloadTask* task, int64 bytesDownloaded, int64 totalLength);
+        };
+
+
+        /** Releases the resources of the download task, unregisters the listener
+            and cancels the download if necessary. */
+        virtual ~DownloadTask();
+
+        /** Returns the total length of the download task. This may return -1 if the length
+            was not returned by the server. */
+        int64 getTotalLength() const                      { return contentLength; }
+
+        /** Returns the number of bytes that have been downloaded so far. */
+        int64 getLengthDownloaded() const                 { return downloaded; }
+
+        /** Returns true if the download finished or there was an error. */
+        bool isFinished() const                           { return finished; }
+
+        /** Returns the status code of the server's response.
+            This will only be valid after the download has finished.
+            @see isFinished
+        */
+        int statusCode() const                            { return httpCode; }
+
+        /** Returns true if there was an error. */
+        inline bool hadError() const                      { return error; }
+
+    protected:
+        int64 contentLength = -1, downloaded = 0;
+        bool finished = false, error = false;
+        int httpCode = -1;
+
+        DownloadTask();
+
+    private:
+        friend class URL;
+        static DownloadTask* createFallbackDownloader (const URL&, const File&, const String&, Listener*, bool);
+
+    public:
+       #if JUCE_IOS
+        /** internal **/
+        static void juce_iosURLSessionNotify (const String&);
+       #endif
+
+    private:
+        JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (DownloadTask)
+    };
+
+    /** Download the URL to a file.
+
+        This method attempts to download the URL to a given file location.
+
+        Using this method to download files on mobile is less flexible but more reliable
+        than using createInputStream or WebInputStreams as it will attempt to download the file
+        using a native OS background network task. Such tasks automatically deal with
+        network re-connections and continuing your download while your app is suspended.
+    */
+    DownloadTask* downloadToFile (const File& targetLocation,
+                                  String extraHeaders = String(),
+                                  DownloadTask::Listener* listener = nullptr,
+                                  bool usePostCommand = false);
 
     //==============================================================================
     /** Tries to download the entire contents of this URL into a binary data block.
@@ -352,7 +459,7 @@ public:
     /** Tries to download the entire contents of this URL and parse it as XML.
 
         If it fails, or if the text that it reads can't be parsed as XML, this will
-        return 0.
+        return nullptr.
 
         When it returns a valid XmlElement object, the caller is responsibile for deleting
         this object when no longer needed.
@@ -409,10 +516,14 @@ public:
 
 private:
     //==============================================================================
+    friend class WebInputStream;
+
     String url;
     MemoryBlock postData;
     StringArray parameterNames, parameterValues;
 
+    static File fileFromFileSchemeURL (const URL&);
+
     struct Upload  : public ReferenceCountedObject
     {
         Upload (const String&, const String&, const String&, const File&, MemoryBlock*);
@@ -426,7 +537,25 @@ private:
     friend struct ContainerDeletePolicy<Upload>;
     ReferenceCountedArray<Upload> filesToUpload;
 
+  #if JUCE_IOS
+    struct Bookmark : public ReferenceCountedObject
+    {
+        using Ptr = ReferenceCountedObjectPtr<Bookmark>;
+
+        Bookmark (void*);
+        ~Bookmark();
+
+        void* data;
+    };
+
+    Bookmark::Ptr bookmark;
+
+    friend void setURLBookmark (URL&, void*);
+    friend void* getURLBookmark (URL&);
+  #endif
+
     URL (const String&, int);
+    void init();
     void addParameter (const String&, const String&);
     void createHeadersAndPostData (String&, MemoryBlock&) const;
     URL withUpload (Upload*) const;
@@ -434,5 +563,4 @@ private:
     JUCE_LEAK_DETECTOR (URL)
 };
 
-
-#endif   // JUCE_URL_H_INCLUDED
+} // namespace juce
