@@ -1,33 +1,27 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_PLATFORMDEFS_H_INCLUDED
-#define JUCE_PLATFORMDEFS_H_INCLUDED
+namespace juce
+{
 
 //==============================================================================
 /*  This file defines miscellaneous macros for debugging, assertions, etc.
@@ -42,7 +36,7 @@
  #endif
 #endif
 
-/** This macro defines the C calling convention used as the standard for Juce calls. */
+/** This macro defines the C calling convention used as the standard for JUCE calls. */
 #if JUCE_MSVC
  #define JUCE_CALLTYPE   __stdcall
  #define JUCE_CDECL      __cdecl
@@ -61,7 +55,7 @@
 #endif
 
 //==============================================================================
-#if JUCE_IOS || JUCE_LINUX || JUCE_ANDROID
+#if JUCE_IOS || JUCE_LINUX
   /** This will try to break into the debugger if the app is currently being debugged.
       If called by an app that's not being debugged, the behaviour isn't defined - it may
       crash or not, depending on the platform.
@@ -79,6 +73,8 @@
   #else
    #define JUCE_BREAK_IN_DEBUGGER       { asm ("int $3"); }
   #endif
+#elif JUCE_ANDROID
+  #define JUCE_BREAK_IN_DEBUGGER        { __builtin_trap(); }
 #else
   #define JUCE_BREAK_IN_DEBUGGER        { __asm int 3 }
 #endif
@@ -86,7 +82,7 @@
 #if JUCE_CLANG && defined (__has_feature) && ! defined (JUCE_ANALYZER_NORETURN)
  #if __has_feature (attribute_analyzer_noreturn)
   inline void __attribute__((analyzer_noreturn)) juce_assert_noreturn() {}
-  #define JUCE_ANALYZER_NORETURN juce_assert_noreturn();
+  #define JUCE_ANALYZER_NORETURN juce::juce_assert_noreturn();
  #endif
 #endif
 
@@ -170,30 +166,6 @@
 #define JUCE_STRINGIFY(item)  JUCE_STRINGIFY_MACRO_HELPER (item)
 
 //==============================================================================
-#if JUCE_COMPILER_SUPPORTS_STATIC_ASSERT
-  /** A compile-time assertion macro.
-     If the expression parameter is false, the macro will cause a compile error. (The actual error
-     message that the compiler generates may be completely bizarre and seem to have no relation to
-     the place where you put the static_assert though!)
-  */
-  #define static_jassert(expression) static_assert(expression, #expression);
-#else
- #ifndef DOXYGEN
-  namespace juce
-  {
-     template <bool b> struct JuceStaticAssert;
-     template <>       struct JuceStaticAssert<true> { static void dummy() {} };
-  }
- #endif
-
-  /** A compile-time assertion macro.
-      If the expression parameter is false, the macro will cause a compile error. (The actual error
-      message that the compiler generates may be completely bizarre and seem to have no relation to
-      the place where you put the static_assert though!)
-  */
-  #define static_jassert(expression)      juce::JuceStaticAssert<expression>::dummy();
-#endif
-
 /** This is a shorthand macro for declaring stubs for a class's copy constructor and operator=.
 
     For example, instead of
@@ -219,8 +191,8 @@
     };@endcode
 */
 #define JUCE_DECLARE_NON_COPYABLE(className) \
-    className (const className&) JUCE_DELETED_FUNCTION;\
-    className& operator= (const className&) JUCE_DELETED_FUNCTION;
+    className (const className&) = delete;\
+    className& operator= (const className&) = delete;
 
 /** This is a shorthand way of writing both a JUCE_DECLARE_NON_COPYABLE and
     JUCE_LEAK_DETECTOR macro for a class.
@@ -234,8 +206,8 @@
 */
 #define JUCE_PREVENT_HEAP_ALLOCATION \
    private: \
-    static void* operator new (size_t) JUCE_DELETED_FUNCTION; \
-    static void operator delete (void*) JUCE_DELETED_FUNCTION;
+    static void* operator new (size_t) = delete; \
+    static void operator delete (void*) = delete;
 
 //==============================================================================
 #if JUCE_MSVC && ! defined (DOXYGEN)
@@ -288,11 +260,13 @@
  #define JUCE_DEPRECATED(functionDef)
  #define JUCE_DEPRECATED_WITH_BODY(functionDef, body)
 #elif JUCE_MSVC && ! JUCE_NO_DEPRECATION_WARNINGS
- #define JUCE_DEPRECATED(functionDef)                   __declspec(deprecated) functionDef
- #define JUCE_DEPRECATED_WITH_BODY(functionDef, body)   __declspec(deprecated) functionDef body
+ #define JUCE_DEPRECATED_ATTRIBUTE                      __declspec(deprecated)
+ #define JUCE_DEPRECATED(functionDef)                   JUCE_DEPRECATED_ATTRIBUTE functionDef
+ #define JUCE_DEPRECATED_WITH_BODY(functionDef, body)   JUCE_DEPRECATED_ATTRIBUTE functionDef body
 #elif (JUCE_GCC || JUCE_CLANG) && ! JUCE_NO_DEPRECATION_WARNINGS
- #define JUCE_DEPRECATED(functionDef)                   functionDef __attribute__ ((deprecated))
- #define JUCE_DEPRECATED_WITH_BODY(functionDef, body)   functionDef __attribute__ ((deprecated)) body
+ #define JUCE_DEPRECATED_ATTRIBUTE                      __attribute__ ((deprecated))
+ #define JUCE_DEPRECATED(functionDef)                   functionDef JUCE_DEPRECATED_ATTRIBUTE
+ #define JUCE_DEPRECATED_WITH_BODY(functionDef, body)   functionDef JUCE_DEPRECATED_ATTRIBUTE body
 #else
  #define JUCE_DEPRECATED(functionDef)                   functionDef
  #define JUCE_DEPRECATED_WITH_BODY(functionDef, body)   functionDef body
@@ -323,4 +297,4 @@
  #define JUCE_NO_ASSOCIATIVE_MATH_OPTIMISATIONS
 #endif
 
-#endif   // JUCE_PLATFORMDEFS_H_INCLUDED
+} // namespace juce
