@@ -1,44 +1,37 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_STANDARDHEADER_H_INCLUDED
-#define JUCE_STANDARDHEADER_H_INCLUDED
+#pragma once
 
 //==============================================================================
 /** Current JUCE version number.
 
     See also SystemStats::getJUCEVersion() for a string version.
 */
-#define JUCE_MAJOR_VERSION      4
-#define JUCE_MINOR_VERSION      3
-#define JUCE_BUILDNUMBER        0
+#define JUCE_MAJOR_VERSION      5
+#define JUCE_MINOR_VERSION      2
+#define JUCE_BUILDNUMBER        1
 
-/** Current Juce version number.
+/** Current JUCE version number.
 
     Bits 16 to 32 = major version.
     Bits 8 to 16 = minor version.
@@ -56,6 +49,7 @@
 #include <iostream>
 #include <functional>
 #include <algorithm>
+#include <limits>
 
 //==============================================================================
 #include "juce_CompilerSupport.h"
@@ -71,11 +65,11 @@
 
 #if JUCE_MAC || JUCE_IOS
  #include <libkern/OSAtomic.h>
+ #include <xlocale.h>
 #endif
 
 #if JUCE_LINUX
  #include <cstring>
- #include <limits>
  #include <signal.h>
 
  #if __INTEL_COMPILER
@@ -114,6 +108,17 @@
 #undef minor
 #undef KeyPress
 
+// Include a replacement for std::function on older platforms and the live
+// build
+#if JUCE_PROJUCER_LIVE_BUILD || ! defined (JUCE_STDLIB_HAS_STD_FUNCTION_SUPPORT)
+ #include "../misc/juce_StdFunctionCompat.h"
+#endif
+
+// Include std::atomic if it's supported by the compiler
+#if JUCE_ATOMIC_AVAILABLE
+ #include <atomic>
+#endif
+
 //==============================================================================
 // DLL building settings on Windows
 #if JUCE_MSVC
@@ -133,7 +138,7 @@
 
 //==============================================================================
 #ifndef JUCE_API
- #define JUCE_API   /**< This macro is added to all juce public class declarations. */
+ #define JUCE_API   /**< This macro is added to all JUCE public class declarations. */
 #endif
 
 #if JUCE_MSVC && JUCE_DLL_BUILD
@@ -142,7 +147,7 @@
  #define JUCE_PUBLIC_IN_DLL_BUILD(declaration)  declaration;
 #endif
 
-/** This macro is added to all juce public function declarations. */
+/** This macro is added to all JUCE public function declarations. */
 #define JUCE_PUBLIC_FUNCTION        JUCE_API JUCE_CALLTYPE
 
 #if (! defined (JUCE_CATCH_DEPRECATED_CODE_MISUSE)) && JUCE_DEBUG && ! DOXYGEN
@@ -155,5 +160,3 @@
 #ifndef DOXYGEN
  #define JUCE_NAMESPACE juce  // This old macro is deprecated: you should just use the juce namespace directly.
 #endif
-
-#endif   // JUCE_STANDARDHEADER_H_INCLUDED
