@@ -1,33 +1,27 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_CHARPOINTER_UTF8_H_INCLUDED
-#define JUCE_CHARPOINTER_UTF8_H_INCLUDED
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -35,12 +29,12 @@
     various methods to operate on the data.
     @see CharPointer_UTF16, CharPointer_UTF32
 */
-class CharPointer_UTF8
+class CharPointer_UTF8  final
 {
 public:
     typedef char CharType;
 
-    inline explicit CharPointer_UTF8 (const CharType* const rawPointer) noexcept
+    inline explicit CharPointer_UTF8 (const CharType* rawPointer) noexcept
         : data (const_cast<CharType*> (rawPointer))
     {
     }
@@ -79,10 +73,13 @@ public:
     /** Returns true if this pointer is pointing to a null character. */
     inline bool isEmpty() const noexcept                { return *data == 0; }
 
+    /** Returns true if this pointer is not pointing to a null character. */
+    inline bool isNotEmpty() const noexcept             { return *data != 0; }
+
     /** Returns the unicode character that this pointer is pointing to. */
     juce_wchar operator*() const noexcept
     {
-        const signed char byte = (signed char) *data;
+        auto byte = (signed char) *data;
 
         if (byte >= 0)
             return (juce_wchar) (uint8) byte;
@@ -103,7 +100,7 @@ public:
 
         for (int i = 1; i <= numExtraValues; ++i)
         {
-            const uint32 nextByte = (uint32) (uint8) data[i];
+            auto nextByte = (uint32) (uint8) data[i];
 
             if ((nextByte & 0xc0) != 0x80)
                 break;
@@ -119,13 +116,13 @@ public:
     CharPointer_UTF8& operator++() noexcept
     {
         jassert (*data != 0); // trying to advance past the end of the string?
-        const signed char n = (signed char) *data++;
+        auto n = (signed char) *data++;
 
         if (n < 0)
         {
             juce_wchar bit = 0x40;
 
-            while ((n & bit) != 0 && bit > 0x8)
+            while ((static_cast<juce_wchar> (n) & bit) != 0 && bit > 0x8)
             {
                 ++data;
                 bit >>= 1;
@@ -150,7 +147,7 @@ public:
         advances the pointer to point to the next character. */
     juce_wchar getAndAdvance() noexcept
     {
-        const signed char byte = (signed char) *data++;
+        auto byte = (signed char) *data++;
 
         if (byte >= 0)
             return (juce_wchar) (uint8) byte;
@@ -171,7 +168,7 @@ public:
 
         while (--numExtraValues >= 0)
         {
-            const uint32 nextByte = (uint32) (uint8) *data;
+            auto nextByte = (uint32) (uint8) *data;
 
             if ((nextByte & 0xc0) != 0x80)
                 break;
@@ -216,7 +213,7 @@ public:
     /** Returns the character at a given character index from the start of the string. */
     juce_wchar operator[] (int characterIndex) const noexcept
     {
-        CharPointer_UTF8 p (*this);
+        auto p (*this);
         p += characterIndex;
         return *p;
     }
@@ -224,7 +221,7 @@ public:
     /** Returns a pointer which is moved forwards from this one by the specified number of characters. */
     CharPointer_UTF8 operator+ (int numToSkip) const noexcept
     {
-        CharPointer_UTF8 p (*this);
+        auto p (*this);
         p += numToSkip;
         return p;
     }
@@ -232,7 +229,7 @@ public:
     /** Returns a pointer which is moved backwards from this one by the specified number of characters. */
     CharPointer_UTF8 operator- (int numToSkip) const noexcept
     {
-        CharPointer_UTF8 p (*this);
+        auto p (*this);
         p += -numToSkip;
         return p;
     }
@@ -240,12 +237,12 @@ public:
     /** Returns the number of characters in this string. */
     size_t length() const noexcept
     {
-        const CharType* d = data;
+        auto* d = data;
         size_t count = 0;
 
         for (;;)
         {
-            const uint32 n = (uint32) (uint8) *d++;
+            auto n = (uint32) (uint8) *d++;
 
             if ((n & 0x80) != 0)
             {
@@ -288,7 +285,7 @@ public:
     static size_t getBytesRequiredFor (const juce_wchar charToWrite) noexcept
     {
         size_t num = 1;
-        const uint32 c = (uint32) charToWrite;
+        auto c = (uint32) charToWrite;
 
         if (c >= 0x80)
         {
@@ -313,7 +310,7 @@ public:
     {
         size_t count = 0;
 
-        while (juce_wchar n = text.getAndAdvance())
+        while (auto n = text.getAndAdvance())
             count += getBytesRequiredFor (n);
 
         return count;
@@ -328,7 +325,7 @@ public:
     /** Writes a unicode character to this string, and advances this pointer to point to the next position. */
     void write (const juce_wchar charToWrite) noexcept
     {
-        const uint32 c = (uint32) charToWrite;
+        auto c = (uint32) charToWrite;
 
         if (c >= 0x80)
         {
@@ -367,7 +364,7 @@ public:
     /** Copies a source string to this pointer, advancing this pointer as it goes. */
     void writeAll (const CharPointer_UTF8 src) noexcept
     {
-        const CharType* s = src.data;
+        auto* s = src.data;
 
         while ((*data = *s) != 0)
         {
@@ -451,48 +448,46 @@ public:
     }
 
     /** Returns true if the first character of this string is whitespace. */
-    bool isWhitespace() const noexcept      { const CharType c = *data; return c == ' ' || (c <= 13 && c >= 9); }
+    bool isWhitespace() const noexcept          { const CharType c = *data; return c == ' ' || (c <= 13 && c >= 9); }
     /** Returns true if the first character of this string is a digit. */
-    bool isDigit() const noexcept           { const CharType c = *data; return c >= '0' && c <= '9'; }
+    bool isDigit() const noexcept               { const CharType c = *data; return c >= '0' && c <= '9'; }
     /** Returns true if the first character of this string is a letter. */
-    bool isLetter() const noexcept          { return CharacterFunctions::isLetter (operator*()) != 0; }
+    bool isLetter() const noexcept              { return CharacterFunctions::isLetter (operator*()) != 0; }
     /** Returns true if the first character of this string is a letter or digit. */
-    bool isLetterOrDigit() const noexcept   { return CharacterFunctions::isLetterOrDigit (operator*()) != 0; }
+    bool isLetterOrDigit() const noexcept       { return CharacterFunctions::isLetterOrDigit (operator*()) != 0; }
     /** Returns true if the first character of this string is upper-case. */
-    bool isUpperCase() const noexcept       { return CharacterFunctions::isUpperCase (operator*()) != 0; }
+    bool isUpperCase() const noexcept           { return CharacterFunctions::isUpperCase (operator*()) != 0; }
     /** Returns true if the first character of this string is lower-case. */
-    bool isLowerCase() const noexcept       { return CharacterFunctions::isLowerCase (operator*()) != 0; }
+    bool isLowerCase() const noexcept           { return CharacterFunctions::isLowerCase (operator*()) != 0; }
 
     /** Returns an upper-case version of the first character of this string. */
-    juce_wchar toUpperCase() const noexcept { return CharacterFunctions::toUpperCase (operator*()); }
+    juce_wchar toUpperCase() const noexcept     { return CharacterFunctions::toUpperCase (operator*()); }
     /** Returns a lower-case version of the first character of this string. */
-    juce_wchar toLowerCase() const noexcept { return CharacterFunctions::toLowerCase (operator*()); }
+    juce_wchar toLowerCase() const noexcept     { return CharacterFunctions::toLowerCase (operator*()); }
 
     /** Parses this string as a 32-bit integer. */
-    int getIntValue32() const noexcept      { return atoi (data); }
+    int getIntValue32() const noexcept          { return atoi (data); }
 
     /** Parses this string as a 64-bit integer. */
     int64 getIntValue64() const noexcept
     {
-       #if JUCE_LINUX || JUCE_ANDROID || JUCE_MINGW
-        return atoll (data);
-       #elif JUCE_WINDOWS
+       #if JUCE_WINDOWS && ! JUCE_MINGW
         return _atoi64 (data);
        #else
-        return CharacterFunctions::getIntValue <int64, CharPointer_UTF8> (*this);
+        return atoll (data);
        #endif
     }
 
     /** Parses this string as a floating point double. */
-    double getDoubleValue() const noexcept  { return CharacterFunctions::getDoubleValue (*this); }
+    double getDoubleValue() const noexcept                      { return CharacterFunctions::getDoubleValue (*this); }
 
     /** Returns the first non-whitespace character in the string. */
-    CharPointer_UTF8 findEndOfWhitespace() const noexcept   { return CharacterFunctions::findEndOfWhitespace (*this); }
+    CharPointer_UTF8 findEndOfWhitespace() const noexcept       { return CharacterFunctions::findEndOfWhitespace (*this); }
 
     /** Returns true if the given unicode character can be represented in this encoding. */
     static bool canRepresent (juce_wchar character) noexcept
     {
-        return ((unsigned int) character) < (unsigned int) 0x10ffff;
+        return ((uint32) character) < (uint32) 0x10ffff;
     }
 
     /** Returns true if this data contains a valid string in this encoding. */
@@ -500,7 +495,7 @@ public:
     {
         while (--maxBytesToRead >= 0 && *dataToTest != 0)
         {
-            const signed char byte = (signed char) *dataToTest++;
+            auto byte = (signed char) *dataToTest++;
 
             if (byte < 0)
             {
@@ -556,7 +551,7 @@ public:
     static bool isByteOrderMark (const void* possibleByteOrder) noexcept
     {
         jassert (possibleByteOrder != nullptr);
-        const uint8* const c = static_cast<const uint8*> (possibleByteOrder);
+        auto c = static_cast<const uint8*> (possibleByteOrder);
 
         return c[0] == (uint8) byteOrderMark1
             && c[1] == (uint8) byteOrderMark2
@@ -567,4 +562,4 @@ private:
     CharType* data;
 };
 
-#endif   // JUCE_CHARPOINTER_UTF8_H_INCLUDED
+} // namespace juce
