@@ -1,31 +1,28 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 StringArray::StringArray() noexcept
 {
 }
@@ -35,12 +32,10 @@ StringArray::StringArray (const StringArray& other)
 {
 }
 
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
 StringArray::StringArray (StringArray&& other) noexcept
-    : strings (static_cast<Array <String>&&> (other.strings))
+    : strings (static_cast<Array<String>&&> (other.strings))
 {
 }
-#endif
 
 StringArray::StringArray (const String& firstValue)
 {
@@ -72,20 +67,6 @@ StringArray::StringArray (const wchar_t* const* initialStrings, int numberOfStri
     strings.addArray (initialStrings, numberOfStrings);
 }
 
-StringArray& StringArray::operator= (const StringArray& other)
-{
-    strings = other.strings;
-    return *this;
-}
-
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
-StringArray& StringArray::operator= (StringArray&& other) noexcept
-{
-    strings = static_cast<Array<String>&&> (other.strings);
-    return *this;
-}
-#endif
-
 #if JUCE_COMPILER_SUPPORTS_INITIALIZER_LISTS
 StringArray::StringArray (const std::initializer_list<const char*>& stringList)
 {
@@ -93,6 +74,18 @@ StringArray::StringArray (const std::initializer_list<const char*>& stringList)
 }
 #endif
 
+StringArray& StringArray::operator= (const StringArray& other)
+{
+    strings = other.strings;
+    return *this;
+}
+
+StringArray& StringArray::operator= (StringArray&& other) noexcept
+{
+    strings = static_cast<Array<String>&&> (other.strings);
+    return *this;
+}
+
 StringArray::~StringArray()
 {
 }
@@ -122,20 +115,16 @@ void StringArray::clearQuick()
     strings.clearQuick();
 }
 
-const String& StringArray::operator[] (const int index) const noexcept
+const String& StringArray::operator[] (int index) const noexcept
 {
     if (isPositiveAndBelow (index, strings.size()))
         return strings.getReference (index);
 
-   #if JUCE_ALLOW_STATIC_NULL_VARIABLES
-    return String::empty;
-   #else
     static String empty;
     return empty;
-   #endif
 }
 
-String& StringArray::getReference (const int index) noexcept
+String& StringArray::getReference (int index) noexcept
 {
     return strings.getReference (index);
 }
@@ -145,19 +134,17 @@ void StringArray::add (const String& newString)
     strings.add (newString);
 }
 
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
 void StringArray::add (String&& stringToAdd)
 {
     strings.add (static_cast<String&&> (stringToAdd));
 }
-#endif
 
-void StringArray::insert (const int index, const String& newString)
+void StringArray::insert (int index, const String& newString)
 {
     strings.insert (index, newString);
 }
 
-bool StringArray::addIfNotAlreadyThere (const String& newString, const bool ignoreCase)
+bool StringArray::addIfNotAlreadyThere (const String& newString, bool ignoreCase)
 {
     if (contains (newString, ignoreCase))
         return false;
@@ -181,28 +168,28 @@ void StringArray::addArray (const StringArray& otherArray, int startIndex, int n
         strings.add (otherArray.strings.getReference (startIndex++));
 }
 
-void StringArray::mergeArray (const StringArray& otherArray, const bool ignoreCase)
+void StringArray::mergeArray (const StringArray& otherArray, bool ignoreCase)
 {
-    for (int i = 0; i < otherArray.size(); ++i)
-        addIfNotAlreadyThere (otherArray[i], ignoreCase);
+    for (auto& s : otherArray)
+        addIfNotAlreadyThere (s, ignoreCase);
 }
 
-void StringArray::set (const int index, const String& newString)
+void StringArray::set (int index, const String& newString)
 {
     strings.set (index, newString);
 }
 
-bool StringArray::contains (StringRef stringToLookFor, const bool ignoreCase) const
+bool StringArray::contains (StringRef stringToLookFor, bool ignoreCase) const
 {
     return indexOf (stringToLookFor, ignoreCase) >= 0;
 }
 
-int StringArray::indexOf (StringRef stringToLookFor, const bool ignoreCase, int i) const
+int StringArray::indexOf (StringRef stringToLookFor, bool ignoreCase, int i) const
 {
     if (i < 0)
         i = 0;
 
-    const int numElements = size();
+    auto numElements = size();
 
     if (ignoreCase)
     {
@@ -220,18 +207,18 @@ int StringArray::indexOf (StringRef stringToLookFor, const bool ignoreCase, int
     return -1;
 }
 
-void StringArray::move (const int currentIndex, const int newIndex) noexcept
+void StringArray::move (int currentIndex, int newIndex) noexcept
 {
     strings.move (currentIndex, newIndex);
 }
 
 //==============================================================================
-void StringArray::remove (const int index)
+void StringArray::remove (int index)
 {
     strings.remove (index);
 }
 
-void StringArray::removeString (StringRef stringToRemove, const bool ignoreCase)
+void StringArray::removeString (StringRef stringToRemove, bool ignoreCase)
 {
     if (ignoreCase)
     {
@@ -253,7 +240,7 @@ void StringArray::removeRange (int startIndex, int numberToRemove)
 }
 
 //==============================================================================
-void StringArray::removeEmptyStrings (const bool removeWhitespaceStrings)
+void StringArray::removeEmptyStrings (bool removeWhitespaceStrings)
 {
     if (removeWhitespaceStrings)
     {
@@ -271,66 +258,43 @@ void StringArray::removeEmptyStrings (const bool removeWhitespaceStrings)
 
 void StringArray::trim()
 {
-    for (int i = size(); --i >= 0;)
-    {
-        String& s = strings.getReference(i);
+    for (auto& s : strings)
         s = s.trim();
-    }
 }
 
 //==============================================================================
-struct InternalStringArrayComparator_CaseSensitive
-{
-    static int compareElements (String& s1, String& s2) noexcept    { return s1.compare (s2); }
-};
-
-struct InternalStringArrayComparator_CaseInsensitive
-{
-    static int compareElements (String& s1, String& s2) noexcept    { return s1.compareIgnoreCase (s2); }
-};
-
-struct InternalStringArrayComparator_Natural
-{
-    static int compareElements (String& s1, String& s2) noexcept    { return s1.compareNatural (s2); }
-};
-
-void StringArray::sort (const bool ignoreCase)
+void StringArray::sort (bool ignoreCase)
 {
     if (ignoreCase)
-    {
-        InternalStringArrayComparator_CaseInsensitive comp;
-        strings.sort (comp);
-    }
+        std::sort (strings.begin(), strings.end(),
+                   [] (const String& a, const String& b) { return a.compareIgnoreCase (b) < 0; });
     else
-    {
-        InternalStringArrayComparator_CaseSensitive comp;
-        strings.sort (comp);
-    }
+        std::sort (strings.begin(), strings.end());
 }
 
 void StringArray::sortNatural()
 {
-    InternalStringArrayComparator_Natural comp;
-    strings.sort (comp);
+    std::sort (strings.begin(), strings.end(),
+               [] (const String& a, const String& b) { return a.compareNatural (b) < 0; });
 }
 
 //==============================================================================
 String StringArray::joinIntoString (StringRef separator, int start, int numberToJoin) const
 {
-    const int last = (numberToJoin < 0) ? size()
-                                        : jmin (size(), start + numberToJoin);
+    auto last = (numberToJoin < 0) ? size()
+                                   : jmin (size(), start + numberToJoin);
 
     if (start < 0)
         start = 0;
 
     if (start >= last)
-        return String();
+        return {};
 
     if (start == last - 1)
         return strings.getReference (start);
 
-    const size_t separatorBytes = separator.text.sizeInBytes() - sizeof (String::CharPointerType::CharType);
-    size_t bytesNeeded = separatorBytes * (size_t) (last - start - 1);
+    auto separatorBytes = separator.text.sizeInBytes() - sizeof (String::CharPointerType::CharType);
+    auto bytesNeeded = separatorBytes * (size_t) (last - start - 1);
 
     for (int i = start; i < last; ++i)
         bytesNeeded += strings.getReference(i).getCharPointer().sizeInBytes() - sizeof (String::CharPointerType::CharType);
@@ -338,11 +302,11 @@ String StringArray::joinIntoString (StringRef separator, int start, int numberTo
     String result;
     result.preallocateBytes (bytesNeeded);
 
-    String::CharPointerType dest (result.getCharPointer());
+    auto dest = result.getCharPointer();
 
     while (start < last)
     {
-        const String& s = strings.getReference (start);
+        auto& s = strings.getReference (start);
 
         if (! s.isEmpty())
             dest.writeAll (s.getCharPointer());
@@ -352,7 +316,6 @@ String StringArray::joinIntoString (StringRef separator, int start, int numberTo
     }
 
     dest.writeNull();
-
     return result;
 }
 
@@ -367,11 +330,11 @@ int StringArray::addTokens (StringRef text, StringRef breakCharacters, StringRef
 
     if (text.isNotEmpty())
     {
-        for (String::CharPointerType t (text.text);;)
+        for (auto t = text.text;;)
         {
-            String::CharPointerType tokenEnd (CharacterFunctions::findEndOfToken (t,
-                                                                                  breakCharacters.text,
-                                                                                  quoteCharacters.text));
+            auto tokenEnd = CharacterFunctions::findEndOfToken (t,
+                                                                breakCharacters.text,
+                                                                quoteCharacters.text);
             strings.add (String (t, tokenEnd));
             ++num;
 
@@ -388,14 +351,14 @@ int StringArray::addTokens (StringRef text, StringRef breakCharacters, StringRef
 int StringArray::addLines (StringRef sourceText)
 {
     int numLines = 0;
-    String::CharPointerType text (sourceText.text);
+    auto text = sourceText.text;
     bool finished = text.isEmpty();
 
     while (! finished)
     {
-        for (String::CharPointerType startOfLine (text);;)
+        for (auto startOfLine = text;;)
         {
-            const String::CharPointerType endOfLine (text);
+            auto endOfLine = text;
 
             switch (text.getAndAdvance())
             {
@@ -438,11 +401,11 @@ StringArray StringArray::fromLines (StringRef stringToBreakUp)
 }
 
 //==============================================================================
-void StringArray::removeDuplicates (const bool ignoreCase)
+void StringArray::removeDuplicates (bool ignoreCase)
 {
     for (int i = 0; i < size() - 1; ++i)
     {
-        const String s (strings.getReference(i));
+        auto s = strings.getReference(i);
 
         for (int nextIndex = i + 1;;)
         {
@@ -456,29 +419,25 @@ void StringArray::removeDuplicates (const bool ignoreCase)
     }
 }
 
-void StringArray::appendNumbersToDuplicates (const bool ignoreCase,
-                                             const bool appendNumberToFirstInstance,
+void StringArray::appendNumbersToDuplicates (bool ignoreCase,
+                                             bool appendNumberToFirstInstance,
                                              CharPointer_UTF8 preNumberString,
                                              CharPointer_UTF8 postNumberString)
 {
-    CharPointer_UTF8 defaultPre (" ("), defaultPost (")");
-
     if (preNumberString.getAddress() == nullptr)
-        preNumberString = defaultPre;
+        preNumberString = CharPointer_UTF8 (" (");
 
     if (postNumberString.getAddress() == nullptr)
-        postNumberString = defaultPost;
+        postNumberString = CharPointer_UTF8 (")");
 
     for (int i = 0; i < size() - 1; ++i)
     {
-        String& s = strings.getReference(i);
-
-        int nextIndex = indexOf (s, ignoreCase, i + 1);
+        auto& s = strings.getReference(i);
+        auto nextIndex = indexOf (s, ignoreCase, i + 1);
 
         if (nextIndex >= 0)
         {
-            const String original (s);
-
+            auto original = s;
             int number = 0;
 
             if (appendNumberToFirstInstance)
@@ -504,3 +463,5 @@ void StringArray::minimiseStorageOverheads()
 {
     strings.minimiseStorageOverheads();
 }
+
+} // namespace juce
