@@ -1,36 +1,33 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 class ThreadPool::ThreadPoolThread  : public Thread
 {
 public:
-    ThreadPoolThread (ThreadPool& p, size_t stackSize = 0)
-       : Thread ("Pool", stackSize), currentJob (nullptr), pool (p)
+    ThreadPoolThread (ThreadPool& p, size_t stackSize)
+       : Thread ("Pool", stackSize), pool (p)
     {
     }
 
@@ -41,16 +38,14 @@ public:
                 wait (500);
     }
 
-    ThreadPoolJob* volatile currentJob;
+    ThreadPoolJob* volatile currentJob = nullptr;
     ThreadPool& pool;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ThreadPoolThread)
 };
 
 //==============================================================================
-ThreadPoolJob::ThreadPoolJob (const String& name)
-    : jobName (name), pool (nullptr),
-      shouldStop (false), isActive (false), shouldBeDeleted (false)
+ThreadPoolJob::ThreadPoolJob (const String& name)  : jobName (name)
 {
 }
 
@@ -74,11 +69,22 @@ void ThreadPoolJob::setJobName (const String& newName)
 void ThreadPoolJob::signalJobShouldExit()
 {
     shouldStop = true;
+    listeners.call ([] (Thread::Listener& l) { l.exitSignalSent(); });
+}
+
+void ThreadPoolJob::addListener (Thread::Listener* listener)
+{
+    listeners.add (listener);
+}
+
+void ThreadPoolJob::removeListener (Thread::Listener* listener)
+{
+    listeners.remove (listener);
 }
 
 ThreadPoolJob* ThreadPoolJob::getCurrentThreadPoolJob()
 {
-    if (ThreadPool::ThreadPoolThread* t = dynamic_cast<ThreadPool::ThreadPoolThread*> (Thread::getCurrentThread()))
+    if (auto* t = dynamic_cast<ThreadPool::ThreadPoolThread*> (Thread::getCurrentThread()))
         return t->currentJob;
 
     return nullptr;
@@ -108,17 +114,17 @@ void ThreadPool::createThreads (int numThreads, size_t threadStackSize)
     for (int i = jmax (1, numThreads); --i >= 0;)
         threads.add (new ThreadPoolThread (*this, threadStackSize));
 
-    for (int i = threads.size(); --i >= 0;)
-        threads.getUnchecked(i)->startThread();
+    for (auto* t : threads)
+        t->startThread();
 }
 
 void ThreadPool::stopThreads()
 {
-    for (int i = threads.size(); --i >= 0;)
-        threads.getUnchecked(i)->signalThreadShouldExit();
+    for (auto* t : threads)
+        t->signalThreadShouldExit();
 
-    for (int i = threads.size(); --i >= 0;)
-        threads.getUnchecked(i)->stopThread (500);
+    for (auto* t : threads)
+        t->stopThread (500);
 }
 
 void ThreadPool::addJob (ThreadPoolJob* const job, const bool deleteJobWhenFinished)
@@ -138,44 +144,83 @@ void ThreadPool::addJob (ThreadPoolJob* const job, const bool deleteJobWhenFinis
             jobs.add (job);
         }
 
-        for (int i = threads.size(); --i >= 0;)
-            threads.getUnchecked(i)->notify();
+        for (auto* t : threads)
+            t->notify();
     }
 }
 
-int ThreadPool::getNumJobs() const
+void ThreadPool::addJob (std::function<ThreadPoolJob::JobStatus()> jobToRun)
+{
+    struct LambdaJobWrapper  : public ThreadPoolJob
+    {
+        LambdaJobWrapper (std::function<ThreadPoolJob::JobStatus()> j) : ThreadPoolJob ("lambda"), job (j) {}
+        JobStatus runJob() override      { return job(); }
+
+        std::function<ThreadPoolJob::JobStatus()> job;
+    };
+
+    addJob (new LambdaJobWrapper (jobToRun), true);
+}
+
+void ThreadPool::addJob (std::function<void()> jobToRun)
+{
+    struct LambdaJobWrapper  : public ThreadPoolJob
+    {
+        LambdaJobWrapper (std::function<void()> j) : ThreadPoolJob ("lambda"), job (j) {}
+        JobStatus runJob() override      { job(); return ThreadPoolJob::jobHasFinished; }
+
+        std::function<void()> job;
+    };
+
+    addJob (new LambdaJobWrapper (jobToRun), true);
+}
+
+int ThreadPool::getNumJobs() const noexcept
 {
     return jobs.size();
 }
 
-int ThreadPool::getNumThreads() const
+int ThreadPool::getNumThreads() const noexcept
 {
     return threads.size();
 }
 
-ThreadPoolJob* ThreadPool::getJob (const int index) const
+ThreadPoolJob* ThreadPool::getJob (int index) const noexcept
 {
     const ScopedLock sl (lock);
     return jobs [index];
 }
 
-bool ThreadPool::contains (const ThreadPoolJob* const job) const
+bool ThreadPool::contains (const ThreadPoolJob* const job) const noexcept
 {
     const ScopedLock sl (lock);
     return jobs.contains (const_cast<ThreadPoolJob*> (job));
 }
 
-bool ThreadPool::isJobRunning (const ThreadPoolJob* const job) const
+bool ThreadPool::isJobRunning (const ThreadPoolJob* const job) const noexcept
 {
     const ScopedLock sl (lock);
     return jobs.contains (const_cast<ThreadPoolJob*> (job)) && job->isActive;
 }
 
+void ThreadPool::moveJobToFront (const ThreadPoolJob* job) noexcept
+{
+    const ScopedLock sl (lock);
+
+    if (! job->isActive)
+    {
+        auto index = jobs.indexOf (const_cast<ThreadPoolJob*> (job));
+
+        if (index > 0)
+            jobs.move (index, 0);
+    }
+}
+
 bool ThreadPool::waitForJobToFinish (const ThreadPoolJob* const job, const int timeOutMs) const
 {
     if (job != nullptr)
     {
-        const uint32 start = Time::getMillisecondCounter();
+        auto start = Time::getMillisecondCounter();
 
         while (contains (job))
         {
@@ -223,7 +268,7 @@ bool ThreadPool::removeJob (ThreadPoolJob* const job,
 bool ThreadPool::removeAllJobs (const bool interruptRunningJobs, const int timeOutMs,
                                 ThreadPool::JobSelector* const selectedJobsToRemove)
 {
-    Array <ThreadPoolJob*> jobsToWaitFor;
+    Array<ThreadPoolJob*> jobsToWaitFor;
 
     {
         OwnedArray<ThreadPoolJob> deletionList;
@@ -233,7 +278,7 @@ bool ThreadPool::removeAllJobs (const bool interruptRunningJobs, const int timeO
 
             for (int i = jobs.size(); --i >= 0;)
             {
-                ThreadPoolJob* const job = jobs.getUnchecked(i);
+                auto* job = jobs.getUnchecked(i);
 
                 if (selectedJobsToRemove == nullptr || selectedJobsToRemove->isJobSuitable (job))
                 {
@@ -254,13 +299,13 @@ bool ThreadPool::removeAllJobs (const bool interruptRunningJobs, const int timeO
         }
     }
 
-    const uint32 start = Time::getMillisecondCounter();
+    auto start = Time::getMillisecondCounter();
 
     for (;;)
     {
         for (int i = jobsToWaitFor.size(); --i >= 0;)
         {
-            ThreadPoolJob* const job = jobsToWaitFor.getUnchecked (i);
+            auto* job = jobsToWaitFor.getUnchecked (i);
 
             if (! isJobRunning (job))
                 jobsToWaitFor.remove (i);
@@ -283,12 +328,9 @@ StringArray ThreadPool::getNamesOfAllJobs (const bool onlyReturnActiveJobs) cons
     StringArray s;
     const ScopedLock sl (lock);
 
-    for (int i = 0; i < jobs.size(); ++i)
-    {
-        const ThreadPoolJob* const job = jobs.getUnchecked(i);
+    for (auto* job : jobs)
         if (job->isActive || ! onlyReturnActiveJobs)
             s.add (job->getJobName());
-    }
 
     return s;
 }
@@ -297,8 +339,8 @@ bool ThreadPool::setThreadPriorities (const int newPriority)
 {
     bool ok = true;
 
-    for (int i = threads.size(); --i >= 0;)
-        if (! threads.getUnchecked(i)->setPriority (newPriority))
+    for (auto* t : threads)
+        if (! t->setPriority (newPriority))
             ok = false;
 
     return ok;
@@ -313,20 +355,21 @@ ThreadPoolJob* ThreadPool::pickNextJobToRun()
 
         for (int i = 0; i < jobs.size(); ++i)
         {
-            ThreadPoolJob* job = jobs[i];
-
-            if (job != nullptr && ! job->isActive)
+            if (auto* job = jobs[i])
             {
-                if (job->shouldStop)
+                if (! job->isActive)
                 {
-                    jobs.remove (i);
-                    addToDeleteList (deletionList, job);
-                    --i;
-                    continue;
-                }
+                    if (job->shouldStop)
+                    {
+                        jobs.remove (i);
+                        addToDeleteList (deletionList, job);
+                        --i;
+                        continue;
+                    }
 
-                job->isActive = true;
-                return job;
+                    job->isActive = true;
+                    return job;
+                }
             }
         }
     }
@@ -336,9 +379,9 @@ ThreadPoolJob* ThreadPool::pickNextJobToRun()
 
 bool ThreadPool::runNextJob (ThreadPoolThread& thread)
 {
-    if (ThreadPoolJob* const job = pickNextJobToRun())
+    if (auto* job = pickNextJobToRun())
     {
-        ThreadPoolJob::JobStatus result = ThreadPoolJob::jobHasFinished;
+        auto result = ThreadPoolJob::jobHasFinished;
         thread.currentJob = job;
 
         try
@@ -390,3 +433,5 @@ void ThreadPool::addToDeleteList (OwnedArray<ThreadPoolJob>& deletionList, Threa
     if (job->shouldBeDeleted)
         deletionList.add (job);
 }
+
+} // namespace juce
