@@ -1,37 +1,34 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 namespace TimeHelpers
 {
     static std::tm millisToLocal (int64 millis) noexcept
     {
        #if JUCE_WINDOWS && JUCE_MINGW
-        time_t now = (time_t) (millis / 1000);
+        auto now = (time_t) (millis / 1000);
         return *localtime (&now);
 
        #elif JUCE_WINDOWS
@@ -45,7 +42,7 @@ namespace TimeHelpers
 
        #else
         std::tm result;
-        time_t now = (time_t) (millis / 1000);
+        auto now = (time_t) (millis / 1000);
 
         if (localtime_r (&now, &result) == nullptr)
             zerostruct (result);
@@ -57,7 +54,7 @@ namespace TimeHelpers
     static std::tm millisToUTC (int64 millis) noexcept
     {
        #if JUCE_WINDOWS && JUCE_MINGW
-        time_t now = (time_t) (millis / 1000);
+        auto now = (time_t) (millis / 1000);
         return *gmtime (&now);
 
        #elif JUCE_WINDOWS
@@ -71,7 +68,7 @@ namespace TimeHelpers
 
        #else
         std::tm result;
-        time_t now = (time_t) (millis / 1000);
+        auto now = (time_t) (millis / 1000);
 
         if (gmtime_r (&now, &result) == nullptr)
             zerostruct (result);
@@ -82,7 +79,7 @@ namespace TimeHelpers
 
     static int getUTCOffsetSeconds (const int64 millis) noexcept
     {
-        std::tm utc = millisToUTC (millis);
+        auto utc = millisToUTC (millis);
         utc.tm_isdst = -1;  // Treat this UTC time as local to find the offset
 
         return (int) ((millis / 1000) - (int64) mktime (&utc));
@@ -106,21 +103,21 @@ namespace TimeHelpers
 
        #ifdef JUCE_MSVC
         if (tm->tm_year < -1900 || tm->tm_year > 8099)
-            return String();   // Visual Studio's library can only handle 0 -> 9999 AD
+            return {};   // Visual Studio's library can only handle 0 -> 9999 AD
         #endif
 
         for (size_t bufferSize = 256; ; bufferSize += 256)
         {
             HeapBlock<StringType::CharType> buffer (bufferSize);
 
-            const size_t numChars =
-                           #if JUCE_ANDROID
-                            strftime (buffer, bufferSize - 1, format.toUTF8(), tm);
-                           #elif JUCE_WINDOWS
-                            wcsftime (buffer, bufferSize - 1, format.toWideCharPointer(), tm);
-                           #else
-                            wcsftime (buffer, bufferSize - 1, format.toUTF32(), tm);
-                           #endif
+            auto numChars =
+                       #if JUCE_ANDROID
+                        strftime (buffer, bufferSize - 1, format.toUTF8(), tm);
+                       #elif JUCE_WINDOWS
+                        wcsftime (buffer, bufferSize - 1, format.toWideCharPointer(), tm);
+                       #else
+                        wcsftime (buffer, bufferSize - 1, format.toUTF32(), tm);
+                       #endif
 
             if (numChars > 0 || format.isEmpty())
                 return String (StringType (buffer),
@@ -180,7 +177,7 @@ namespace TimeHelpers
                 + t.tm_sec;
     }
 
-    static uint32 lastMSCounterValue = 0;
+    static Atomic<uint32> lastMSCounterValue { (uint32) 0 };
 }
 
 //==============================================================================
@@ -192,18 +189,13 @@ Time::Time (const Time& other) noexcept  : millisSinceEpoch (other.millisSinceEp
 {
 }
 
-Time::Time (const int64 ms) noexcept  : millisSinceEpoch (ms)
+Time::Time (int64 ms) noexcept  : millisSinceEpoch (ms)
 {
 }
 
-Time::Time (const int year,
-            const int month,
-            const int day,
-            const int hours,
-            const int minutes,
-            const int seconds,
-            const int milliseconds,
-            const bool useLocalTime) noexcept
+Time::Time (int year, int month, int day,
+            int hours, int minutes, int seconds, int milliseconds,
+            bool useLocalTime) noexcept
 {
     std::tm t;
     t.tm_year   = year - 1900;
@@ -253,14 +245,14 @@ uint32 juce_millisecondsSinceStartup() noexcept;
 
 uint32 Time::getMillisecondCounter() noexcept
 {
-    const uint32 now = juce_millisecondsSinceStartup();
+    auto now = juce_millisecondsSinceStartup();
 
-    if (now < TimeHelpers::lastMSCounterValue)
+    if (now < TimeHelpers::lastMSCounterValue.get())
     {
         // in multi-threaded apps this might be called concurrently, so
         // make sure that our last counter value only increases and doesn't
         // go backwards..
-        if (now < TimeHelpers::lastMSCounterValue - 1000)
+        if (now < TimeHelpers::lastMSCounterValue.get() - (uint32) 1000)
             TimeHelpers::lastMSCounterValue = now;
     }
     else
@@ -273,22 +265,20 @@ uint32 Time::getMillisecondCounter() noexcept
 
 uint32 Time::getApproximateMillisecondCounter() noexcept
 {
-    if (TimeHelpers::lastMSCounterValue == 0)
-        getMillisecondCounter();
-
-    return TimeHelpers::lastMSCounterValue;
+    auto t = TimeHelpers::lastMSCounterValue.get();
+    return t == 0 ? getMillisecondCounter() : t;
 }
 
-void Time::waitForMillisecondCounter (const uint32 targetTime) noexcept
+void Time::waitForMillisecondCounter (uint32 targetTime) noexcept
 {
     for (;;)
     {
-        const uint32 now = getMillisecondCounter();
+        auto now = getMillisecondCounter();
 
         if (now >= targetTime)
             break;
 
-        const int toWait = (int) (targetTime - now);
+        auto toWait = (int) (targetTime - now);
 
         if (toWait > 2)
         {
@@ -335,14 +325,14 @@ String Time::toString (const bool includeDate,
 
     if (includeTime)
     {
-        const int mins = getMinutes();
+        auto mins = getMinutes();
 
         result << (use24HourClock ? getHours() : getHoursInAmPmFormat())
                << (mins < 10 ? ":0" : ":") << mins;
 
         if (includeSeconds)
         {
-            const int secs = getSeconds();
+            auto secs = getSeconds();
             result << (secs < 10 ? ":0" : ":") << secs;
         }
 
@@ -372,7 +362,7 @@ int Time::getMilliseconds() const noexcept  { return TimeHelpers::extendedModulo
 
 int Time::getHoursInAmPmFormat() const noexcept
 {
-    const int hours = getHours();
+    auto hours = getHours();
 
     if (hours == 0)  return 12;
     if (hours <= 12) return hours;
@@ -411,7 +401,7 @@ String Time::getTimeZone() const noexcept
   #else
     tzset();
 
-    const char** const zonePtr = (const char**) tzname;
+    auto zonePtr = (const char**) tzname;
     zone[0] = zonePtr[0];
     zone[1] = zonePtr[1];
   #endif
@@ -485,19 +475,21 @@ static int parseFixedSizeIntAndSkip (String::CharPointerType& t, int numChars, c
 
 Time Time::fromISO8601 (StringRef iso) noexcept
 {
-    String::CharPointerType t = iso.text;
+    auto t = iso.text;
+    auto year = parseFixedSizeIntAndSkip (t, 4, '-');
 
-    const int year = parseFixedSizeIntAndSkip (t, 4, '-');
     if (year < 0)
-        return Time();
+        return {};
+
+    auto month = parseFixedSizeIntAndSkip (t, 2, '-');
 
-    const int month = parseFixedSizeIntAndSkip (t, 2, '-');
     if (month < 0)
-        return Time();
+        return {};
+
+    auto day = parseFixedSizeIntAndSkip (t, 2, 0);
 
-    const int day = parseFixedSizeIntAndSkip (t, 2, 0);
     if (day < 0)
-        return Time();
+        return {};
 
     int hours = 0, minutes = 0, milliseconds = 0;
 
@@ -505,34 +497,52 @@ Time Time::fromISO8601 (StringRef iso) noexcept
     {
         ++t;
         hours = parseFixedSizeIntAndSkip (t, 2, ':');
+
         if (hours < 0)
-            return Time();
+            return {};
 
         minutes = parseFixedSizeIntAndSkip (t, 2, ':');
-        if (minutes < 0)
-            return Time();
 
-        milliseconds = (int) (1000.0 * CharacterFunctions::readDoubleValue (t));
+        if (minutes < 0)
+            return {};
+
+        auto seconds = parseFixedSizeIntAndSkip (t, 2, 0);
+
+        if (seconds < 0)
+             return {};
+
+        if (*t == '.')
+        {
+            ++t;
+            milliseconds = parseFixedSizeIntAndSkip (t, 3, 0);
+
+            if (milliseconds < 0)
+                return {};
+        }
+
+        milliseconds += 1000 * seconds;
     }
 
-    const juce_wchar nextChar = t.getAndAdvance();
+    auto nextChar = t.getAndAdvance();
 
     if (nextChar == '-' || nextChar == '+')
     {
-        const int offsetHours = parseFixedSizeIntAndSkip (t, 2, ':');
+        auto offsetHours = parseFixedSizeIntAndSkip (t, 2, ':');
+
         if (offsetHours < 0)
-            return Time();
+            return {};
+
+        auto offsetMinutes = parseFixedSizeIntAndSkip (t, 2, 0);
 
-        const int offsetMinutes = parseFixedSizeIntAndSkip (t, 2, 0);
         if (offsetMinutes < 0)
-            return Time();
+            return {};
 
-        const int offsetMs = (offsetHours * 60 + offsetMinutes) * 60 * 1000;
+        auto offsetMs = (offsetHours * 60 + offsetMinutes) * 60 * 1000;
         milliseconds += nextChar == '-' ? offsetMs : -offsetMs; // NB: this seems backwards but is correct!
     }
     else if (nextChar != 0 && nextChar != 'Z')
     {
-        return Time();
+        return {};
     }
 
     return Time (year, month - 1, day, hours, minutes, 0, milliseconds, false);
@@ -622,7 +632,7 @@ Time Time::getCompilationDate()
 class TimeTests  : public UnitTest
 {
 public:
-    TimeTests() : UnitTest ("Time") {}
+    TimeTests() : UnitTest ("Time", "Time") {}
 
     void runTest() override
     {
@@ -673,3 +683,5 @@ public:
 static TimeTests timeTests;
 
 #endif
+
+} // namespace juce
