@@ -1,63 +1,57 @@
 /*
   ==============================================================================
 
-   This file is part of the juce_core module of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   This file is part of the JUCE library.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission to use, copy, modify, and/or distribute this software for any purpose with
-   or without fee is hereby granted, provided that the above copyright notice and this
-   permission notice appear in all copies.
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
-   TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN
-   NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
-   DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
-   IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
-   CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   ------------------------------------------------------------------------------
-
-   NOTE! This permissive ISC license applies ONLY to files within the juce_core module!
-   All other JUCE modules are covered by a dual GPL/commercial license, so if you are
-   using any other modules, be sure to check that you also comply with their license.
-
-   For more details, visit www.juce.com
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-namespace
+namespace juce
 {
-    inline bool isValidXmlNameStartCharacter (const juce_wchar character) noexcept
-    {
-        return character == ':'
-            || character == '_'
-            || (character >= 'a'     && character <= 'z')
-            || (character >= 'A'     && character <= 'Z')
-            || (character >= 0xc0    && character <= 0xd6)
-            || (character >= 0xd8    && character <= 0xf6)
-            || (character >= 0xf8    && character <= 0x2ff)
-            || (character >= 0x370   && character <= 0x37d)
-            || (character >= 0x37f   && character <= 0x1fff)
-            || (character >= 0x200c  && character <= 0x200d)
-            || (character >= 0x2070  && character <= 0x218f)
-            || (character >= 0x2c00  && character <= 0x2fef)
-            || (character >= 0x3001  && character <= 0xd7ff)
-            || (character >= 0xf900  && character <= 0xfdcf)
-            || (character >= 0xfdf0  && character <= 0xfffd)
-            || (character >= 0x10000 && character <= 0xeffff);
-    }
 
-    inline bool isValidXmlNameBodyCharacter (const juce_wchar character) noexcept
-    {
-        return isValidXmlNameStartCharacter (character)
-            || character == '-'
-            || character == '.'
-            || character == 0xb7
-            || (character >= '0'    && character <= '9')
-            || (character >= 0x300  && character <= 0x036f)
-            || (character >= 0x203f && character <= 0x2040);
-    }
+inline static bool isValidXmlNameStartCharacter (juce_wchar character) noexcept
+{
+    return character == ':'
+        || character == '_'
+        || (character >= 'a'     && character <= 'z')
+        || (character >= 'A'     && character <= 'Z')
+        || (character >= 0xc0    && character <= 0xd6)
+        || (character >= 0xd8    && character <= 0xf6)
+        || (character >= 0xf8    && character <= 0x2ff)
+        || (character >= 0x370   && character <= 0x37d)
+        || (character >= 0x37f   && character <= 0x1fff)
+        || (character >= 0x200c  && character <= 0x200d)
+        || (character >= 0x2070  && character <= 0x218f)
+        || (character >= 0x2c00  && character <= 0x2fef)
+        || (character >= 0x3001  && character <= 0xd7ff)
+        || (character >= 0xf900  && character <= 0xfdcf)
+        || (character >= 0xfdf0  && character <= 0xfffd)
+        || (character >= 0x10000 && character <= 0xeffff);
+}
+
+inline static bool isValidXmlNameBodyCharacter (juce_wchar character) noexcept
+{
+    return isValidXmlNameStartCharacter (character)
+        || character == '-'
+        || character == '.'
+        || character == 0xb7
+        || (character >= '0'    && character <= '9')
+        || (character >= 0x300  && character <= 0x036f)
+        || (character >= 0x203f && character <= 0x2040);
 }
 
 XmlElement::XmlAttributeNode::XmlAttributeNode (const XmlAttributeNode& other) noexcept
@@ -132,7 +126,6 @@ XmlElement& XmlElement::operator= (const XmlElement& other)
     return *this;
 }
 
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
 XmlElement::XmlElement (XmlElement&& other) noexcept
     : nextListItem      (static_cast<LinkedListPointer<XmlElement>&&> (other.nextListItem)),
       firstChildElement (static_cast<LinkedListPointer<XmlElement>&&> (other.firstChildElement)),
@@ -155,7 +148,6 @@ XmlElement& XmlElement::operator= (XmlElement&& other) noexcept
 
     return *this;
 }
-#endif
 
 void XmlElement::copyChildrenAndAttributesFrom (const XmlElement& other)
 {
@@ -220,11 +212,11 @@ namespace XmlOutputFunctions
 
     static void escapeIllegalXmlChars (OutputStream& outputStream, const String& text, const bool changeNewLines)
     {
-        String::CharPointerType t (text.getCharPointer());
+        auto t = text.getCharPointer();
 
         for (;;)
         {
-            const uint32 character = (uint32) t.getAndAdvance();
+            auto character = (uint32) t.getAndAdvance();
 
             if (character == 0)
                 break;
@@ -237,22 +229,22 @@ namespace XmlOutputFunctions
             {
                 switch (character)
                 {
-                case '&':   outputStream << "&amp;"; break;
-                case '"':   outputStream << "&quot;"; break;
-                case '>':   outputStream << "&gt;"; break;
-                case '<':   outputStream << "&lt;"; break;
+                    case '&':   outputStream << "&amp;"; break;
+                    case '"':   outputStream << "&quot;"; break;
+                    case '>':   outputStream << "&gt;"; break;
+                    case '<':   outputStream << "&lt;"; break;
 
-                case '\n':
-                case '\r':
-                    if (! changeNewLines)
-                    {
-                        outputStream << (char) character;
+                    case '\n':
+                    case '\r':
+                        if (! changeNewLines)
+                        {
+                            outputStream << (char) character;
+                            break;
+                        }
+                        // Note: deliberate fall-through here!
+                    default:
+                        outputStream << "&#" << ((int) character) << ';';
                         break;
-                    }
-                    // Note: deliberate fall-through here!
-                default:
-                    outputStream << "&#" << ((int) character) << ';';
-                    break;
                 }
             }
         }
@@ -279,10 +271,10 @@ void XmlElement::writeElementAsText (OutputStream& outputStream,
         outputStream << tagName;
 
         {
-            const size_t attIndent = (size_t) (indentationLevel + tagName.length() + 1);
+            auto attIndent = (size_t) (indentationLevel + tagName.length() + 1);
             int lineLen = 0;
 
-            for (const XmlAttributeNode* att = attributes; att != nullptr; att = att->nextListItem)
+            for (auto* att = attributes.get(); att != nullptr; att = att->nextListItem)
             {
                 if (lineLen > lineWrapLength && indentationLevel >= 0)
                 {
@@ -291,7 +283,7 @@ void XmlElement::writeElementAsText (OutputStream& outputStream,
                     lineLen = 0;
                 }
 
-                const int64 startPos = outputStream.getPosition();
+                auto startPos = outputStream.getPosition();
                 outputStream.writeByte (' ');
                 outputStream << att->name;
                 outputStream.write ("=\"", 2);
@@ -301,13 +293,12 @@ void XmlElement::writeElementAsText (OutputStream& outputStream,
             }
         }
 
-        if (firstChildElement != nullptr)
+        if (auto* child = firstChildElement.get())
         {
             outputStream.writeByte ('>');
-
             bool lastWasTextNode = false;
 
-            for (XmlElement* child = firstChildElement; child != nullptr; child = child->nextListItem)
+            for (; child != nullptr; child = child->nextListItem)
             {
                 if (child->isTextElement())
                 {
@@ -358,12 +349,9 @@ String XmlElement::createDocument (StringRef dtdToUse,
     return mem.toUTF8();
 }
 
-void XmlElement::writeToStream (OutputStream& output,
-                                StringRef dtdToUse,
-                                const bool allOnOneLine,
-                                const bool includeXmlHeader,
-                                StringRef encodingType,
-                                const int lineWrapLength) const
+void XmlElement::writeToStream (OutputStream& output, StringRef dtdToUse,
+                                bool allOnOneLine, bool includeXmlHeader,
+                                StringRef encodingType, int lineWrapLength) const
 {
     using namespace XmlOutputFunctions;
 
@@ -393,10 +381,8 @@ void XmlElement::writeToStream (OutputStream& output,
         output << newLine;
 }
 
-bool XmlElement::writeToFile (const File& file,
-                              StringRef dtdToUse,
-                              StringRef encodingType,
-                              const int lineWrapLength) const
+bool XmlElement::writeToFile (const File& file, StringRef dtdToUse,
+                              StringRef encodingType, int lineWrapLength) const
 {
     TemporaryFile tempFile (file);
 
@@ -407,7 +393,6 @@ bool XmlElement::writeToFile (const File& file,
             return false;
 
         writeToStream (out, dtdToUse, false, true, encodingType, lineWrapLength);
-
         out.flush(); // (called explicitly to force an fsync on posix)
 
         if (out.getStatus().failed())
@@ -446,7 +431,7 @@ bool XmlElement::hasTagNameIgnoringNamespace (StringRef possibleTagName) const
 
 XmlElement* XmlElement::getNextElementWithTagName (StringRef requiredTagName) const
 {
-    XmlElement* e = nextListItem;
+    auto* e = nextListItem.get();
 
     while (e != nullptr && ! e->hasTagName (requiredTagName))
         e = e->nextListItem;
@@ -462,17 +447,13 @@ int XmlElement::getNumAttributes() const noexcept
 
 static const String& getEmptyStringRef() noexcept
 {
-   #if JUCE_ALLOW_STATIC_NULL_VARIABLES
-    return String::empty;
-   #else
     static String empty;
     return empty;
-   #endif
 }
 
 const String& XmlElement::getAttributeName (const int index) const noexcept
 {
-    if (const XmlAttributeNode* const att = attributes [index])
+    if (auto* att = attributes[index].get())
         return att->name.toString();
 
     return getEmptyStringRef();
@@ -480,7 +461,7 @@ const String& XmlElement::getAttributeName (const int index) const noexcept
 
 const String& XmlElement::getAttributeValue (const int index) const noexcept
 {
-    if (const XmlAttributeNode* const att = attributes [index])
+    if (auto* att = attributes[index].get())
         return att->value;
 
     return getEmptyStringRef();
@@ -488,7 +469,7 @@ const String& XmlElement::getAttributeValue (const int index) const noexcept
 
 XmlElement::XmlAttributeNode* XmlElement::getAttribute (StringRef attributeName) const noexcept
 {
-    for (XmlAttributeNode* att = attributes; att != nullptr; att = att->nextListItem)
+    for (auto* att = attributes.get(); att != nullptr; att = att->nextListItem)
         if (att->name == attributeName)
             return att;
 
@@ -503,7 +484,7 @@ bool XmlElement::hasAttribute (StringRef attributeName) const noexcept
 //==============================================================================
 const String& XmlElement::getStringAttribute (StringRef attributeName) const noexcept
 {
-    if (const XmlAttributeNode* att = getAttribute (attributeName))
+    if (auto* att = getAttribute (attributeName))
         return att->value;
 
     return getEmptyStringRef();
@@ -511,7 +492,7 @@ const String& XmlElement::getStringAttribute (StringRef attributeName) const noe
 
 String XmlElement::getStringAttribute (StringRef attributeName, const String& defaultReturnValue) const
 {
-    if (const XmlAttributeNode* att = getAttribute (attributeName))
+    if (auto* att = getAttribute (attributeName))
         return att->value;
 
     return defaultReturnValue;
@@ -519,7 +500,7 @@ String XmlElement::getStringAttribute (StringRef attributeName, const String& de
 
 int XmlElement::getIntAttribute (StringRef attributeName, const int defaultReturnValue) const
 {
-    if (const XmlAttributeNode* att = getAttribute (attributeName))
+    if (auto* att = getAttribute (attributeName))
         return att->value.getIntValue();
 
     return defaultReturnValue;
@@ -527,7 +508,7 @@ int XmlElement::getIntAttribute (StringRef attributeName, const int defaultRetur
 
 double XmlElement::getDoubleAttribute (StringRef attributeName, const double defaultReturnValue) const
 {
-    if (const XmlAttributeNode* att = getAttribute (attributeName))
+    if (auto* att = getAttribute (attributeName))
         return att->value.getDoubleValue();
 
     return defaultReturnValue;
@@ -535,9 +516,9 @@ double XmlElement::getDoubleAttribute (StringRef attributeName, const double def
 
 bool XmlElement::getBoolAttribute (StringRef attributeName, const bool defaultReturnValue) const
 {
-    if (const XmlAttributeNode* att = getAttribute (attributeName))
+    if (auto* att = getAttribute (attributeName))
     {
-        const juce_wchar firstChar = *(att->value.getCharPointer().findEndOfWhitespace());
+        auto firstChar = *(att->value.getCharPointer().findEndOfWhitespace());
 
         return firstChar == '1'
             || firstChar == 't'
@@ -553,7 +534,7 @@ bool XmlElement::compareAttribute (StringRef attributeName,
                                    StringRef stringToCompareAgainst,
                                    const bool ignoreCase) const noexcept
 {
-    if (const XmlAttributeNode* att = getAttribute (attributeName))
+    if (auto* att = getAttribute (attributeName))
         return ignoreCase ? att->value.equalsIgnoreCase (stringToCompareAgainst)
                           : att->value == stringToCompareAgainst;
 
@@ -569,7 +550,7 @@ void XmlElement::setAttribute (const Identifier& attributeName, const String& va
     }
     else
     {
-        for (XmlAttributeNode* att = attributes; ; att = att->nextListItem)
+        for (auto* att = attributes.get(); ; att = att->nextListItem)
         {
             if (att->name == attributeName)
             {
@@ -598,9 +579,7 @@ void XmlElement::setAttribute (const Identifier& attributeName, const double num
 
 void XmlElement::removeAttribute (const Identifier& attributeName) noexcept
 {
-    for (LinkedListPointer<XmlAttributeNode>* att = &attributes;
-         att->get() != nullptr;
-         att = &(att->get()->nextListItem))
+    for (auto* att = &attributes; att->get() != nullptr; att = &(att->get()->nextListItem))
     {
         if (att->get()->name == attributeName)
         {
@@ -630,7 +609,7 @@ XmlElement* XmlElement::getChildByName (StringRef childName) const noexcept
 {
     jassert (! childName.isEmpty());
 
-    for (XmlElement* child = firstChildElement; child != nullptr; child = child->nextListItem)
+    for (auto* child = firstChildElement.get(); child != nullptr; child = child->nextListItem)
         if (child->hasTagName (childName))
             return child;
 
@@ -641,7 +620,7 @@ XmlElement* XmlElement::getChildByAttribute (StringRef attributeName, StringRef
 {
     jassert (! attributeName.isEmpty());
 
-    for (XmlElement* child = firstChildElement; child != nullptr; child = child->nextListItem)
+    for (auto* child = firstChildElement.get(); child != nullptr; child = child->nextListItem)
         if (child->compareAttribute (attributeName, attributeValue))
             return child;
 
@@ -683,7 +662,7 @@ void XmlElement::prependChildElement (XmlElement* newNode) noexcept
 
 XmlElement* XmlElement::createNewChildElement (StringRef childTagName)
 {
-    XmlElement* const newElement = new XmlElement (childTagName);
+    auto newElement = new XmlElement (childTagName);
     addChildElement (newElement);
     return newElement;
 }
@@ -693,7 +672,7 @@ bool XmlElement::replaceChildElement (XmlElement* const currentChildElement,
 {
     if (newNode != nullptr)
     {
-        if (LinkedListPointer<XmlElement>* const p = firstChildElement.findPointerTo (currentChildElement))
+        if (auto* p = firstChildElement.findPointerTo (currentChildElement))
         {
             if (currentChildElement != newNode)
                 delete p->replaceNext (newNode);
@@ -729,7 +708,7 @@ bool XmlElement::isEquivalentTo (const XmlElement* const other,
         {
             int totalAtts = 0;
 
-            for (const XmlAttributeNode* att = attributes; att != nullptr; att = att->nextListItem)
+            for (auto* att = attributes.get(); att != nullptr; att = att->nextListItem)
             {
                 if (! other->compareAttribute (att->name, att->value))
                     return false;
@@ -742,8 +721,8 @@ bool XmlElement::isEquivalentTo (const XmlElement* const other,
         }
         else
         {
-            const XmlAttributeNode* thisAtt = attributes;
-            const XmlAttributeNode* otherAtt = other->attributes;
+            auto* thisAtt = attributes.get();
+            auto* otherAtt = other->attributes.get();
 
             for (;;)
             {
@@ -766,8 +745,8 @@ bool XmlElement::isEquivalentTo (const XmlElement* const other,
             }
         }
 
-        const XmlElement* thisChild = firstChildElement;
-        const XmlElement* otherChild = other->firstChildElement;
+        auto* thisChild = firstChildElement.get();
+        auto* otherChild = other->firstChildElement.get();
 
         for (;;)
         {
@@ -797,9 +776,9 @@ void XmlElement::deleteAllChildElements() noexcept
 
 void XmlElement::deleteAllChildElementsWithTagName (StringRef name) noexcept
 {
-    for (XmlElement* child = firstChildElement; child != nullptr;)
+    for (auto* child = firstChildElement.get(); child != nullptr;)
     {
-        XmlElement* const nextChild = child->nextListItem;
+        auto* nextChild = child->nextListItem.get();
 
         if (child->hasTagName (name))
             removeChildElement (child, true);
@@ -818,12 +797,12 @@ XmlElement* XmlElement::findParentElementOf (const XmlElement* const elementToLo
     if (this == elementToLookFor || elementToLookFor == nullptr)
         return nullptr;
 
-    for (XmlElement* child = firstChildElement; child != nullptr; child = child->nextListItem)
+    for (auto* child = firstChildElement.get(); child != nullptr; child = child->nextListItem)
     {
         if (elementToLookFor == child)
             return this;
 
-        if (XmlElement* const found = child->findParentElementOf (elementToLookFor))
+        if (auto* found = child->findParentElementOf (elementToLookFor))
             return found;
     }
 
@@ -835,9 +814,10 @@ void XmlElement::getChildElementsAsArray (XmlElement** elems) const noexcept
     firstChildElement.copyToArray (elems);
 }
 
-void XmlElement::reorderChildElements (XmlElement** const elems, const int num) noexcept
+void XmlElement::reorderChildElements (XmlElement** elems, int num) noexcept
 {
-    XmlElement* e = firstChildElement = elems[0];
+    auto* e = elems[0];
+    firstChildElement = e;
 
     for (int i = 1; i < num; ++i)
     {
@@ -883,7 +863,7 @@ String XmlElement::getAllSubText() const
 
     MemoryOutputStream mem (1024);
 
-    for (const XmlElement* child = firstChildElement; child != nullptr; child = child->nextListItem)
+    for (auto* child = firstChildElement.get(); child != nullptr; child = child->nextListItem)
         mem << child->getAllSubText();
 
     return mem.toUTF8();
@@ -891,7 +871,7 @@ String XmlElement::getAllSubText() const
 
 String XmlElement::getChildElementAllSubText (StringRef childTagName, const String& defaultReturnValue) const
 {
-    if (const XmlElement* const child = getChildByName (childTagName))
+    if (auto* child = getChildByName (childTagName))
         return child->getAllSubText();
 
     return defaultReturnValue;
@@ -899,7 +879,7 @@ String XmlElement::getChildElementAllSubText (StringRef childTagName, const Stri
 
 XmlElement* XmlElement::createTextElement (const String& text)
 {
-    XmlElement* const e = new XmlElement ((int) 0);
+    auto e = new XmlElement ((int) 0);
     e->setAttribute (juce_xmltextContentAttributeName, text);
     return e;
 }
@@ -926,9 +906,9 @@ void XmlElement::addTextElement (const String& text)
 
 void XmlElement::deleteAllTextElements() noexcept
 {
-    for (XmlElement* child = firstChildElement; child != nullptr;)
+    for (auto* child = firstChildElement.get(); child != nullptr;)
     {
-        XmlElement* const next = child->nextListItem;
+        auto* next = child->nextListItem.get();
 
         if (child->isTextElement())
             removeChildElement (child, true);
@@ -936,3 +916,5 @@ void XmlElement::deleteAllTextElements() noexcept
         child = next;
     }
 }
+
+} // namespace juce
