@@ -2,39 +2,44 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 BlowFish::BlowFish (const void* const keyData, const int keyBytes)
 {
     jassert (keyData != nullptr);
     jassert (keyBytes > 0);
 
-    static const uint32 initialPValues [18] =
+    static const uint32 initialPValues[18] =
     {
         0x243f6a88, 0x85a308d3, 0x13198a2e, 0x03707344, 0xa4093822, 0x299f31d0, 0x082efa98, 0xec4e6c89,
         0x452821e6, 0x38d01377, 0xbe5466cf, 0x34e90c6c, 0xc0ac29b7, 0xc97c50dd, 0x3f84d5b5, 0xb5470917,
         0x9216d5d9, 0x8979fb1b
     };
 
-    static const uint32 initialSValues [4 * 256] =
+    static const uint32 initialSValues[4 * 256] =
     {
         0xd1310ba6, 0x98dfb5ac, 0x2ffd72db, 0xd01adfb7, 0xb8e1afed, 0x6a267e96, 0xba7c9045, 0xf12c7f99,
         0x24a19947, 0xb3916cf7, 0x0801f2e2, 0x858efc16, 0x636920d8, 0x71574e69, 0xa458fea3, 0xf4933d7e,
@@ -168,23 +173,24 @@ BlowFish::BlowFish (const void* const keyData, const int keyBytes)
 
     memcpy (p, initialPValues, sizeof (p));
 
-    int i, j = 0;
-    for (i = 4; --i >= 0;)
+    int keyIndex = 0;
+
+    for (int i = 0; i < 4; ++i)
     {
         s[i].malloc (256);
         memcpy (s[i], initialSValues + i * 256, 256 * sizeof (uint32));
     }
 
-    for (i = 0; i < 18; ++i)
+    for (int i = 0; i < 18; ++i)
     {
         uint32 d = 0;
 
         for (int k = 0; k < 4; ++k)
         {
-            d = (d << 8) | static_cast<const uint8*> (keyData)[j];
+            d = (d << 8) | static_cast<const uint8*> (keyData)[keyIndex];
 
-            if (++j >= keyBytes)
-                j = 0;
+            if (++keyIndex >= keyBytes)
+                keyIndex = 0;
         }
 
         p[i] = initialPValues[i] ^ d;
@@ -192,7 +198,7 @@ BlowFish::BlowFish (const void* const keyData, const int keyBytes)
 
     uint32 l = 0, r = 0;
 
-    for (i = 0; i < 18; i += 2)
+    for (int i = 0; i < 18; i += 2)
     {
         encrypt (l, r);
 
@@ -200,9 +206,9 @@ BlowFish::BlowFish (const void* const keyData, const int keyBytes)
         p[i + 1] = r;
     }
 
-    for (i = 0; i < 4; ++i)
+    for (int i = 0; i < 4; ++i)
     {
-        for (j = 0; j < 256; j += 2)
+        for (int j = 0; j < 256; j += 2)
         {
             encrypt (l, r);
 
@@ -214,8 +220,8 @@ BlowFish::BlowFish (const void* const keyData, const int keyBytes)
 
 BlowFish::BlowFish (const BlowFish& other)
 {
-    for (int i = 4; --i >= 0;)
-        s[i].malloc (256);
+    for (auto& block : s)
+        block.malloc (256);
 
     operator= (other);
 }
@@ -240,8 +246,8 @@ uint32 BlowFish::F (const uint32 x) const noexcept
 
 void BlowFish::encrypt (uint32& data1, uint32& data2) const noexcept
 {
-    uint32 l = data1;
-    uint32 r = data2;
+    auto l = data1;
+    auto r = data2;
 
     for (int i = 0; i < 16; ++i)
     {
@@ -256,8 +262,8 @@ void BlowFish::encrypt (uint32& data1, uint32& data2) const noexcept
 
 void BlowFish::decrypt (uint32& data1, uint32& data2) const noexcept
 {
-    uint32 l = data1;
-    uint32 r = data2;
+    auto l = data1;
+    auto r = data2;
 
     for (int i = 17; i > 1; --i)
     {
@@ -269,3 +275,200 @@ void BlowFish::decrypt (uint32& data1, uint32& data2) const noexcept
     data1 = r ^ p[0];
     data2 = l ^ p[1];
 }
+
+void BlowFish::encrypt (MemoryBlock& data) const
+{
+    auto size = data.getSize();
+    data.setSize (size + (8u - (size % 8u)));
+
+    auto success = encrypt (data.getData(), size, data.getSize());
+
+    ignoreUnused (success);
+    jassert (success >= 0);
+}
+
+void BlowFish::decrypt (MemoryBlock& data) const
+{
+    auto newSize = decrypt (data.getData(), data.getSize());
+
+    if (newSize >= 0)
+        data.setSize (static_cast<size_t> (newSize));
+    else
+        jassertfalse;
+}
+
+int BlowFish::encrypt (void* data, size_t size, size_t bufferSize) const noexcept
+{
+    auto encryptedSize = pad (data, size, bufferSize);
+
+    if (encryptedSize >= 0 && apply (data, static_cast<size_t> (encryptedSize), &BlowFish::encrypt))
+        return encryptedSize;
+
+    return -1;
+}
+
+int BlowFish::decrypt (void* data, size_t size) const noexcept
+{
+    if (apply (data, size, &BlowFish::decrypt))
+        return unpad (data, size);
+
+    return -1;
+}
+
+bool BlowFish::apply (void* data, size_t size, void (BlowFish::*op) (uint32&, uint32&) const) const
+{
+    union AlignedAccessHelper
+    {
+        int8 byte[sizeof(uint32) * 2];
+        uint32 data[2];
+    };
+
+    if ((size % 8u) != 0)
+        return false;
+
+    auto n = size / 8u;
+    auto* ptr = reinterpret_cast<AlignedAccessHelper*> (data);
+
+    for (size_t i = 0; i < n; ++i)
+        (this->*op) (ptr[i].data[0], ptr[i].data[1]);
+
+    return true;
+}
+
+int BlowFish::pad (void* data, size_t size, size_t bufferSize) noexcept
+{
+    // add padding according to https://tools.ietf.org/html/rfc2898#section-6.1.1
+    const uint8 paddingSize = 8u - (size % 8u);
+    auto n = size + paddingSize;
+
+    if (n > bufferSize)
+        return -1;
+
+    auto* dst = reinterpret_cast<uint8*> (data);
+
+    for (size_t i = size; i < n; ++i)
+        dst[i] = paddingSize;
+
+    return static_cast<int> (n);
+}
+
+int BlowFish::unpad (const void* data, size_t size) noexcept
+{
+    if (size == 0)
+        return -1;
+
+    // remove padding according to https://tools.ietf.org/html/rfc2898#section-6.1.1
+    auto paddingSize = reinterpret_cast<const uint8*>(data)[size - 1u];
+
+    if (paddingSize == 0 || paddingSize > 8 || paddingSize > size)
+        return -1;
+
+    return static_cast<int> (size - static_cast<size_t> (paddingSize));
+}
+
+//==============================================================================
+#if JUCE_UNIT_TESTS
+
+class BlowFishTests  : public UnitTest
+{
+public:
+    BlowFishTests() : UnitTest ("BlowFish", "Cryptography") {}
+
+    static void fillMemoryBlockWithRandomData (MemoryBlock& block, Random& random)
+    {
+        const size_t n = block.getSize() / sizeof (int32);
+        auto* dst = reinterpret_cast<uint8*> (block.getData());
+
+        for (size_t i = 0; i < n; ++i)
+            dst[i] = static_cast<uint8> (random.nextInt(255));
+    }
+
+    void expectEqualData (const void* dataA, const void* dataB, size_t size, const String& failureMessage)
+    {
+        auto* a = reinterpret_cast<const uint8*> (dataA);
+        auto* b = reinterpret_cast<const uint8*> (dataB);
+
+        for (size_t i = 0; i < size; ++i)
+            expectEquals ((int) a[i], (int) b[i], failureMessage);
+    }
+
+    void expectEqualMemoryBlocks (const MemoryBlock& a, const MemoryBlock& b, const String& failureMessage)
+    {
+        expectEquals ((int) a.getSize(), (int) b.getSize(), failureMessage);
+        expectEqualData (a.getData(), b.getData(), a.getSize(), failureMessage);
+    }
+
+    void encryptDecryptTest (const BlowFish& blowFish, void* data, size_t size, size_t bufferSize)
+    {
+        MemoryBlock copy (data, size);
+
+        int encryptedSize = blowFish.encrypt (data, size, bufferSize);
+        expectGreaterThan (encryptedSize, (int) size);
+        expectLessOrEqual (encryptedSize, (int) bufferSize);
+
+        int decryptedSize = blowFish.decrypt (data, static_cast<size_t> (encryptedSize));
+        expectEquals ((int) size, decryptedSize);
+
+        expectEqualData (data, copy.getData(), size, "Length/Content changed during encryption");
+    }
+
+    void encryptDecryptTest (const BlowFish& blowFish, MemoryBlock& data)
+    {
+        MemoryBlock copy (data);
+
+        blowFish.encrypt (data);
+        blowFish.decrypt (data);
+
+        expectEqualMemoryBlocks (data, copy, "Length/Content changed during encryption");
+    }
+
+    void encryptDecryptTest (const BlowFish& blowFish, const String& data)
+    {
+        MemoryBlock block (data.toRawUTF8(), static_cast<size_t> (data.length()));
+        encryptDecryptTest (blowFish, block);
+    }
+
+    void runTest() override
+    {
+        beginTest ("BlowFish");
+        auto random = getRandom();
+
+        for (int i = 0; i < 100; ++i)
+        {
+            const int keySize = (random.nextInt(17) + 1) * static_cast<int> (sizeof (uint32));
+            MemoryBlock key (static_cast<size_t> (keySize));
+            fillMemoryBlockWithRandomData (key, random);
+
+            BlowFish bf (key.getData(), keySize);
+
+            encryptDecryptTest (bf, "");
+            encryptDecryptTest (bf, "a");
+            encryptDecryptTest (bf, "Hello World!");
+
+            const int minSize = 8 + sizeof (void*);
+            const int dataSize = random.nextInt (2048 - minSize) + minSize;
+            MemoryBlock data (static_cast<size_t> (dataSize));
+            fillMemoryBlockWithRandomData (data, random);
+
+            encryptDecryptTest (bf, data);
+            encryptDecryptTest (bf, data.getData(), data.getSize() - 8, data.getSize());
+            encryptDecryptTest (bf, data.getData(), 0, 8);
+
+            {
+                // Test unaligned data encryption/decryption. This will be flagged up by a check for
+                // undefined behaviour!
+                auto nudge = static_cast<uintptr_t> (random.nextInt (sizeof(void*) - 1));
+                auto unalignedData = (void*) (reinterpret_cast<uintptr_t> (data.getData()) + nudge);
+                size_t newSize = data.getSize() - nudge;
+
+                encryptDecryptTest (bf, unalignedData, newSize - 8, newSize);
+            }
+        }
+    }
+};
+
+static BlowFishTests blowFishUnitTests;
+
+#endif
+
+} // namespace juce
