@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 struct UndoManager::ActionSet
 {
     ActionSet (const String& transactionName)
@@ -31,8 +36,8 @@ struct UndoManager::ActionSet
 
     bool perform() const
     {
-        for (int i = 0; i < actions.size(); ++i)
-            if (! actions.getUnchecked(i)->perform())
+        for (auto* a : actions)
+            if (! a->perform())
                 return false;
 
         return true;
@@ -51,8 +56,8 @@ struct UndoManager::ActionSet
     {
         int total = 0;
 
-        for (int i = actions.size(); --i >= 0;)
-            total += actions.getUnchecked(i)->getSizeInUnits();
+        for (auto* a : actions)
+            total += a->getSizeInUnits();
 
         return total;
     }
@@ -63,15 +68,9 @@ struct UndoManager::ActionSet
 };
 
 //==============================================================================
-UndoManager::UndoManager (const int maxNumberOfUnitsToKeep,
-                          const int minimumTransactions)
-   : totalUnitsStored (0),
-     nextIndex (0),
-     newTransaction (true),
-     reentrancyCheck (false)
+UndoManager::UndoManager (int maxNumberOfUnitsToKeep, int minimumTransactions)
 {
-    setMaxNumberOfStoredUnits (maxNumberOfUnitsToKeep,
-                               minimumTransactions);
+    setMaxNumberOfStoredUnits (maxNumberOfUnitsToKeep, minimumTransactions);
 }
 
 UndoManager::~UndoManager()
@@ -92,11 +91,10 @@ int UndoManager::getNumberOfUnitsTakenUpByStoredCommands() const
     return totalUnitsStored;
 }
 
-void UndoManager::setMaxNumberOfStoredUnits (const int maxNumberOfUnitsToKeep,
-                                             const int minimumTransactions)
+void UndoManager::setMaxNumberOfStoredUnits (int maxUnits, int minTransactions)
 {
-    maxNumUnitsToKeep          = jmax (1, maxNumberOfUnitsToKeep);
-    minimumTransactionsToKeep  = jmax (1, minimumTransactions);
+    maxNumUnitsToKeep          = jmax (1, maxUnits);
+    minimumTransactionsToKeep  = jmax (1, minTransactions);
 }
 
 //==============================================================================
@@ -113,7 +111,7 @@ bool UndoManager::perform (UndoableAction* const newAction, const String& action
     return false;
 }
 
-bool UndoManager::perform (UndoableAction* const newAction)
+bool UndoManager::perform (UndoableAction* newAction)
 {
     if (newAction != nullptr)
     {
@@ -128,15 +126,15 @@ bool UndoManager::perform (UndoableAction* const newAction)
 
         if (action->perform())
         {
-            ActionSet* actionSet = getCurrentSet();
+            auto* actionSet = getCurrentSet();
 
             if (actionSet != nullptr && ! newTransaction)
             {
-                if (UndoableAction* const lastAction = actionSet->actions.getLast())
+                if (auto* lastAction = actionSet->actions.getLast())
                 {
-                    if (UndoableAction* const coalescedAction = lastAction->createCoalescedAction (action))
+                    if (auto coalescedAction = lastAction->createCoalescedAction (action.get()))
                     {
-                        action = coalescedAction;
+                        action.reset (coalescedAction);
                         totalUnitsStored -= lastAction->getSizeInUnits();
                         actionSet->actions.removeLast();
                     }
@@ -171,7 +169,7 @@ void UndoManager::moveFutureTransactionsToStash()
 
         while (nextIndex < transactions.size())
         {
-            ActionSet* removed = transactions.removeAndReturn (nextIndex);
+            auto* removed = transactions.removeAndReturn (nextIndex);
             stashedFutureTransactions.add (removed);
             totalUnitsStored -= removed->getTotalSize();
         }
@@ -186,11 +184,10 @@ void UndoManager::restoreStashedFutureTransactions()
         transactions.remove (nextIndex);
     }
 
-    for (int i = 0; i < stashedFutureTransactions.size(); ++i)
+    for (auto* stashed : stashedFutureTransactions)
     {
-        ActionSet* action = stashedFutureTransactions.removeAndReturn (i);
-        totalUnitsStored += action->getTotalSize();
-        transactions.add (action);
+        transactions.add (stashed);
+        totalUnitsStored += stashed->getTotalSize();
     }
 
     stashedFutureTransactions.clearQuick (false);
@@ -214,7 +211,7 @@ void UndoManager::dropOldTransactionsIfTooLarge()
 
 void UndoManager::beginNewTransaction() noexcept
 {
-    beginNewTransaction (String());
+    beginNewTransaction ({});
 }
 
 void UndoManager::beginNewTransaction (const String& actionName) noexcept
@@ -227,13 +224,13 @@ void UndoManager::setCurrentTransactionName (const String& newName) noexcept
 {
     if (newTransaction)
         newTransactionName = newName;
-    else if (ActionSet* action = getCurrentSet())
+    else if (auto* action = getCurrentSet())
         action->name = newName;
 }
 
 String UndoManager::getCurrentTransactionName() const noexcept
 {
-    if (ActionSet* action = getCurrentSet())
+    if (auto* action = getCurrentSet())
         return action->name;
 
     return newTransactionName;
@@ -248,7 +245,7 @@ bool UndoManager::canRedo() const noexcept   { return getNextSet()    != nullptr
 
 bool UndoManager::undo()
 {
-    if (const ActionSet* const s = getCurrentSet())
+    if (auto* s = getCurrentSet())
     {
         const ScopedValueSetter<bool> setter (reentrancyCheck, true);
 
@@ -267,7 +264,7 @@ bool UndoManager::undo()
 
 bool UndoManager::redo()
 {
-    if (const ActionSet* const s = getNextSet())
+    if (auto* s = getNextSet())
     {
         const ScopedValueSetter<bool> setter (reentrancyCheck, true);
 
@@ -286,31 +283,31 @@ bool UndoManager::redo()
 
 String UndoManager::getUndoDescription() const
 {
-    if (const ActionSet* const s = getCurrentSet())
+    if (auto* s = getCurrentSet())
         return s->name;
 
-    return String();
+    return {};
 }
 
 String UndoManager::getRedoDescription() const
 {
-    if (const ActionSet* const s = getNextSet())
+    if (auto* s = getNextSet())
         return s->name;
 
-    return String();
+    return {};
 }
 
 Time UndoManager::getTimeOfUndoTransaction() const
 {
-    if (const ActionSet* const s = getCurrentSet())
+    if (auto* s = getCurrentSet())
         return s->time;
 
-    return Time();
+    return {};
 }
 
 Time UndoManager::getTimeOfRedoTransaction() const
 {
-    if (const ActionSet* const s = getNextSet())
+    if (auto* s = getNextSet())
         return s->time;
 
     return Time::getCurrentTime();
@@ -330,16 +327,18 @@ bool UndoManager::undoCurrentTransactionOnly()
 void UndoManager::getActionsInCurrentTransaction (Array<const UndoableAction*>& actionsFound) const
 {
     if (! newTransaction)
-        if (const ActionSet* const s = getCurrentSet())
-            for (int i = 0; i < s->actions.size(); ++i)
-                actionsFound.add (s->actions.getUnchecked(i));
+        if (auto* s = getCurrentSet())
+            for (auto* a : s->actions)
+                actionsFound.add (a);
 }
 
 int UndoManager::getNumActionsInCurrentTransaction() const
 {
     if (! newTransaction)
-        if (const ActionSet* const s = getCurrentSet())
+        if (auto* s = getCurrentSet())
             return s->actions.size();
 
     return 0;
 }
+
+} // namespace juce
