@@ -2,48 +2,51 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 class ValueTree::SharedObject  : public ReferenceCountedObject
 {
 public:
     typedef ReferenceCountedObjectPtr<SharedObject> Ptr;
 
-    explicit SharedObject (const Identifier& t) noexcept
-        : type (t), parent (nullptr)
-    {
-    }
+    explicit SharedObject (const Identifier& t) noexcept  : type (t) {}
 
     SharedObject (const SharedObject& other)
-        : ReferenceCountedObject(),
-          type (other.type), properties (other.properties), parent (nullptr)
+        : ReferenceCountedObject(), type (other.type), properties (other.properties)
     {
-        for (int i = 0; i < other.children.size(); ++i)
+        for (auto* c : other.children)
         {
-            SharedObject* const child = new SharedObject (*other.children.getObjectPointerUnchecked(i));
+            auto child = new SharedObject (*c);
             child->parent = this;
             children.add (child);
         }
     }
 
+    SharedObject& operator= (const SharedObject&) = delete;
+
     ~SharedObject()
     {
         jassert (parent == nullptr); // this should never happen unless something isn't obeying the ref-counting!
@@ -57,105 +60,63 @@ public:
         }
     }
 
-    template <typename Method>
-    void callListeners (Method method, ValueTree& tree) const
+    SharedObject* getRoot() noexcept
     {
-        const int numListeners = valueTreesWithListeners.size();
+        return parent == nullptr ? this : parent->getRoot();
+    }
+
+    template <typename Function>
+    void callListeners (ValueTree::Listener* listenerToExclude, Function fn) const
+    {
+        auto numListeners = valueTreesWithListeners.size();
 
         if (numListeners == 1)
         {
-            valueTreesWithListeners.getUnchecked(0)->listeners.call (method, tree);
+            valueTreesWithListeners.getUnchecked(0)->listeners.callExcluding (listenerToExclude, fn);
         }
         else if (numListeners > 0)
         {
-            const SortedSet<ValueTree*> listenersCopy (valueTreesWithListeners);
+            auto listenersCopy = valueTreesWithListeners;
 
             for (int i = 0; i < numListeners; ++i)
             {
-                ValueTree* const v = listenersCopy.getUnchecked(i);
+                auto* v = listenersCopy.getUnchecked(i);
 
                 if (i == 0 || valueTreesWithListeners.contains (v))
-                    v->listeners.call (method, tree);
+                    v->listeners.callExcluding (listenerToExclude, fn);
             }
         }
     }
 
-    template <typename Method, typename ParamType>
-    void callListeners (Method method, ValueTree& tree, ParamType& param2) const
+    template <typename Function>
+    void callListenersForAllParents (ValueTree::Listener* listenerToExclude, Function fn) const
     {
-        const int numListeners = valueTreesWithListeners.size();
-
-        if (numListeners == 1)
-        {
-            valueTreesWithListeners.getUnchecked(0)->listeners.call (method, tree, param2);
-        }
-        else if (numListeners > 0)
-        {
-            const SortedSet<ValueTree*> listenersCopy (valueTreesWithListeners);
-
-            for (int i = 0; i < numListeners; ++i)
-            {
-                ValueTree* const v = listenersCopy.getUnchecked(i);
-
-                if (i == 0 || valueTreesWithListeners.contains (v))
-                    v->listeners.call (method, tree, param2);
-            }
-        }
+        for (auto* t = this; t != nullptr; t = t->parent)
+            t->callListeners (listenerToExclude, fn);
     }
 
-    template <typename Method, typename ParamType1, typename ParamType2>
-    void callListeners (Method method, ValueTree& tree, ParamType1& param2, ParamType2& param3) const
-    {
-        const int numListeners = valueTreesWithListeners.size();
-
-        if (numListeners == 1)
-        {
-            valueTreesWithListeners.getUnchecked(0)->listeners.call (method, tree, param2, param3);
-        }
-        else if (numListeners > 0)
-        {
-            const SortedSet<ValueTree*> listenersCopy (valueTreesWithListeners);
-
-            for (int i = 0; i < numListeners; ++i)
-            {
-                ValueTree* const v = listenersCopy.getUnchecked(i);
-
-                if (i == 0 || valueTreesWithListeners.contains (v))
-                    v->listeners.call (method, tree, param2, param3);
-            }
-        }
-    }
-
-    void sendPropertyChangeMessage (const Identifier& property)
+    void sendPropertyChangeMessage (const Identifier& property, ValueTree::Listener* listenerToExclude = nullptr)
     {
         ValueTree tree (this);
-
-        for (ValueTree::SharedObject* t = this; t != nullptr; t = t->parent)
-            t->callListeners (&ValueTree::Listener::valueTreePropertyChanged, tree, property);
+        callListenersForAllParents (listenerToExclude, [&] (Listener& l) { l.valueTreePropertyChanged (tree, property); });
     }
 
     void sendChildAddedMessage (ValueTree child)
     {
         ValueTree tree (this);
-
-        for (ValueTree::SharedObject* t = this; t != nullptr; t = t->parent)
-            t->callListeners (&ValueTree::Listener::valueTreeChildAdded, tree, child);
+        callListenersForAllParents (nullptr, [&] (Listener& l) { l.valueTreeChildAdded (tree, child); });
     }
 
     void sendChildRemovedMessage (ValueTree child, int index)
     {
         ValueTree tree (this);
-
-        for (ValueTree::SharedObject* t = this; t != nullptr; t = t->parent)
-            t->callListeners (&ValueTree::Listener::valueTreeChildRemoved, tree, child, index);
+        callListenersForAllParents (nullptr, [=, &tree, &child] (Listener& l) { l.valueTreeChildRemoved (tree, child, index); });
     }
 
     void sendChildOrderChangedMessage (int oldIndex, int newIndex)
     {
         ValueTree tree (this);
-
-        for (ValueTree::SharedObject* t = this; t != nullptr; t = t->parent)
-            t->callListeners (&ValueTree::Listener::valueTreeChildOrderChanged, tree, oldIndex, newIndex);
+        callListenersForAllParents (nullptr, [=, &tree] (Listener& l) { l.valueTreeChildOrderChanged (tree, oldIndex, newIndex); });
     }
 
     void sendParentChangeMessage()
@@ -163,29 +124,32 @@ public:
         ValueTree tree (this);
 
         for (int j = children.size(); --j >= 0;)
-            if (SharedObject* const child = children.getObjectPointer (j))
+            if (auto* child = children.getObjectPointer (j))
                 child->sendParentChangeMessage();
 
-        callListeners (&ValueTree::Listener::valueTreeParentChanged, tree);
+        callListeners (nullptr, [&] (Listener& l) { l.valueTreeParentChanged (tree); });
     }
 
-    void setProperty (const Identifier& name, const var& newValue, UndoManager* const undoManager)
+    void setProperty (const Identifier& name, const var& newValue, UndoManager* undoManager,
+                      ValueTree::Listener* listenerToExclude = nullptr)
     {
         if (undoManager == nullptr)
         {
             if (properties.set (name, newValue))
-                sendPropertyChangeMessage (name);
+                sendPropertyChangeMessage (name, listenerToExclude);
         }
         else
         {
-            if (const var* const existingValue = properties.getVarPointer (name))
+            if (auto* existingValue = properties.getVarPointer (name))
             {
                 if (*existingValue != newValue)
-                    undoManager->perform (new SetPropertyAction (this, name, newValue, *existingValue, false, false));
+                    undoManager->perform (new SetPropertyAction (this, name, newValue, *existingValue,
+                                                                 false, false, listenerToExclude));
             }
             else
             {
-                undoManager->perform (new SetPropertyAction (this, name, newValue, var(), true, false));
+                undoManager->perform (new SetPropertyAction (this, name, newValue, {},
+                                                             true, false, listenerToExclude));
             }
         }
     }
@@ -195,7 +159,7 @@ public:
         return properties.contains (name);
     }
 
-    void removeProperty (const Identifier& name, UndoManager* const undoManager)
+    void removeProperty (const Identifier& name, UndoManager* undoManager)
     {
         if (undoManager == nullptr)
         {
@@ -205,17 +169,17 @@ public:
         else
         {
             if (properties.contains (name))
-                undoManager->perform (new SetPropertyAction (this, name, var(), properties [name], false, true));
+                undoManager->perform (new SetPropertyAction (this, name, {}, properties [name], false, true));
         }
     }
 
-    void removeAllProperties (UndoManager* const undoManager)
+    void removeAllProperties (UndoManager* undoManager)
     {
         if (undoManager == nullptr)
         {
             while (properties.size() > 0)
             {
-                const Identifier name (properties.getName (properties.size() - 1));
+                auto name = properties.getName (properties.size() - 1);
                 properties.remove (name);
                 sendPropertyChangeMessage (name);
             }
@@ -223,12 +187,12 @@ public:
         else
         {
             for (int i = properties.size(); --i >= 0;)
-                undoManager->perform (new SetPropertyAction (this, properties.getName(i), var(),
+                undoManager->perform (new SetPropertyAction (this, properties.getName(i), {},
                                                              properties.getValueAt(i), false, true));
         }
     }
 
-    void copyPropertiesFrom (const SharedObject& source, UndoManager* const undoManager)
+    void copyPropertiesFrom (const SharedObject& source, UndoManager* undoManager)
     {
         for (int i = properties.size(); --i >= 0;)
             if (! source.properties.contains (properties.getName (i)))
@@ -240,26 +204,20 @@ public:
 
     ValueTree getChildWithName (const Identifier& typeToMatch) const
     {
-        for (int i = 0; i < children.size(); ++i)
-        {
-            SharedObject* const s = children.getObjectPointerUnchecked (i);
+        for (auto* s : children)
             if (s->type == typeToMatch)
                 return ValueTree (s);
-        }
 
-        return ValueTree();
+        return {};
     }
 
     ValueTree getOrCreateChildWithName (const Identifier& typeToMatch, UndoManager* undoManager)
     {
-        for (int i = 0; i < children.size(); ++i)
-        {
-            SharedObject* const s = children.getObjectPointerUnchecked (i);
+        for (auto* s : children)
             if (s->type == typeToMatch)
                 return ValueTree (s);
-        }
 
-        SharedObject* const newObject = new SharedObject (typeToMatch);
+        auto newObject = new SharedObject (typeToMatch);
         addChild (newObject, -1, undoManager);
         return ValueTree (newObject);
 
@@ -267,19 +225,16 @@ public:
 
     ValueTree getChildWithProperty (const Identifier& propertyName, const var& propertyValue) const
     {
-        for (int i = 0; i < children.size(); ++i)
-        {
-            SharedObject* const s = children.getObjectPointerUnchecked (i);
+        for (auto* s : children)
             if (s->properties[propertyName] == propertyValue)
                 return ValueTree (s);
-        }
 
-        return ValueTree();
+        return {};
     }
 
-    bool isAChildOf (const SharedObject* const possibleParent) const noexcept
+    bool isAChildOf (const SharedObject* possibleParent) const noexcept
     {
-        for (const SharedObject* p = parent; p != nullptr; p = p->parent)
+        for (auto* p = parent; p != nullptr; p = p->parent)
             if (p == possibleParent)
                 return true;
 
@@ -291,7 +246,7 @@ public:
         return children.indexOf (child.object);
     }
 
-    void addChild (SharedObject* child, int index, UndoManager* const undoManager)
+    void addChild (SharedObject* child, int index, UndoManager* undoManager)
     {
         if (child != nullptr && child->parent != this)
         {
@@ -332,9 +287,9 @@ public:
         }
     }
 
-    void removeChild (const int childIndex, UndoManager* const undoManager)
+    void removeChild (int childIndex, UndoManager* undoManager)
     {
-        if (const Ptr child = children.getObjectPointer (childIndex))
+        if (Ptr child = children.getObjectPointer (childIndex))
         {
             if (undoManager == nullptr)
             {
@@ -350,7 +305,7 @@ public:
         }
     }
 
-    void removeAllChildren (UndoManager* const undoManager)
+    void removeAllChildren (UndoManager* undoManager)
     {
         while (children.size() > 0)
             removeChild (children.size() - 1, undoManager);
@@ -385,11 +340,11 @@ public:
 
         for (int i = 0; i < children.size(); ++i)
         {
-            SharedObject* const child = newOrder.getUnchecked(i)->object;
+            auto* child = newOrder.getUnchecked(i)->object.get();
 
             if (children.getObjectPointerUnchecked (i) != child)
             {
-                const int oldIndex = children.indexOf (child);
+                auto oldIndex = children.indexOf (child);
                 jassert (oldIndex >= 0);
                 moveChild (oldIndex, i, undoManager);
             }
@@ -413,7 +368,7 @@ public:
 
     XmlElement* createXml() const
     {
-        XmlElement* const xml = new XmlElement (type);
+        auto xml = new XmlElement (type);
         properties.copyToXmlAttributes (*xml);
 
         // (NB: it's faster to add nodes to XML elements in reverse order)
@@ -440,7 +395,7 @@ public:
             writeObjectToStream (output, children.getObjectPointerUnchecked(i));
     }
 
-    static void writeObjectToStream (OutputStream& output, const SharedObject* const object)
+    static void writeObjectToStream (OutputStream& output, const SharedObject* object)
     {
         if (object != nullptr)
         {
@@ -448,20 +403,21 @@ public:
         }
         else
         {
-            output.writeString (String());
+            output.writeString ({});
             output.writeCompressedInt (0);
             output.writeCompressedInt (0);
         }
     }
 
     //==============================================================================
-    class SetPropertyAction  : public UndoableAction
+    struct SetPropertyAction  : public UndoableAction
     {
-    public:
-        SetPropertyAction (SharedObject* const so, const Identifier& propertyName,
-                           const var& newVal, const var& oldVal, bool isAdding, bool isDeleting)
+        SetPropertyAction (SharedObject* so, const Identifier& propertyName,
+                           const var& newVal, const var& oldVal, bool isAdding, bool isDeleting,
+                           ValueTree::Listener* listenerToExclude = nullptr)
             : target (so), name (propertyName), newValue (newVal), oldValue (oldVal),
-              isAddingNewProperty (isAdding), isDeletingProperty (isDeleting)
+              isAddingNewProperty (isAdding), isDeletingProperty (isDeleting),
+              excludeListener (listenerToExclude)
         {
         }
 
@@ -472,7 +428,7 @@ public:
             if (isDeletingProperty)
                 target->removeProperty (name, nullptr);
             else
-                target->setProperty (name, newValue, nullptr);
+                target->setProperty (name, newValue, nullptr, excludeListener);
 
             return true;
         }
@@ -496,7 +452,7 @@ public:
         {
             if (! (isAddingNewProperty || isDeletingProperty))
             {
-                if (SetPropertyAction* const next = dynamic_cast<SetPropertyAction*> (nextAction))
+                if (auto* next = dynamic_cast<SetPropertyAction*> (nextAction))
                     if (next->target == target && next->name == name
                           && ! (next->isAddingNewProperty || next->isDeletingProperty))
                         return new SetPropertyAction (target, name, next->newValue, oldValue, false, false);
@@ -511,14 +467,14 @@ public:
         const var newValue;
         var oldValue;
         const bool isAddingNewProperty : 1, isDeletingProperty : 1;
+        ValueTree::Listener* excludeListener;
 
         JUCE_DECLARE_NON_COPYABLE (SetPropertyAction)
     };
 
     //==============================================================================
-    class AddOrRemoveChildAction  : public UndoableAction
+    struct AddOrRemoveChildAction  : public UndoableAction
     {
-    public:
         AddOrRemoveChildAction (SharedObject* parentObject, int index, SharedObject* newChild)
             : target (parentObject),
               child (newChild != nullptr ? newChild : parentObject->children.getObjectPointer (index)),
@@ -569,9 +525,8 @@ public:
     };
 
     //==============================================================================
-    class MoveChildAction  : public UndoableAction
+    struct MoveChildAction  : public UndoableAction
     {
-    public:
         MoveChildAction (SharedObject* parentObject, int fromIndex, int toIndex) noexcept
             : parent (parentObject), startIndex (fromIndex), endIndex (toIndex)
         {
@@ -596,7 +551,7 @@ public:
 
         UndoableAction* createCoalescedAction (UndoableAction* nextAction) override
         {
-            if (MoveChildAction* next = dynamic_cast<MoveChildAction*> (nextAction))
+            if (auto* next = dynamic_cast<MoveChildAction*> (nextAction))
                 if (next->parent == parent && next->startIndex == endIndex)
                     return new MoveChildAction (parent, startIndex, next->endIndex);
 
@@ -615,10 +570,8 @@ public:
     NamedValueSet properties;
     ReferenceCountedArray<SharedObject> children;
     SortedSet<ValueTree*> valueTreesWithListeners;
-    SharedObject* parent;
+    SharedObject* parent = nullptr;
 
-private:
-    SharedObject& operator= (const SharedObject&);
     JUCE_LEAK_DETECTOR (SharedObject)
 };
 
@@ -636,6 +589,20 @@ ValueTree::ValueTree (const Identifier& type)  : object (new ValueTree::SharedOb
     jassert (type.toString().isNotEmpty()); // All objects must be given a sensible type name!
 }
 
+#if JUCE_COMPILER_SUPPORTS_INITIALIZER_LISTS
+ValueTree::ValueTree (const Identifier& type,
+                      std::initializer_list<std::pair<Identifier, var>> properties,
+                      std::initializer_list<ValueTree> subTrees)
+    : ValueTree (type)
+{
+    for (auto& prop : properties)
+        setProperty (prop.first, prop.second, nullptr);
+
+    for (auto& tree : subTrees)
+        addChild (tree, -1, nullptr);
+}
+#endif
+
 ValueTree::ValueTree (SharedObject* so) noexcept  : object (so)
 {
 }
@@ -662,19 +629,17 @@ ValueTree& ValueTree::operator= (const ValueTree& other)
 
             object = other.object;
 
-            listeners.call (&ValueTree::Listener::valueTreeRedirected, *this);
+            listeners.call ([this] (Listener& l) { l.valueTreeRedirected (*this); });
         }
     }
 
     return *this;
 }
 
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
 ValueTree::ValueTree (ValueTree&& other) noexcept
     : object (static_cast<SharedObject::Ptr&&> (other.object))
 {
 }
-#endif
 
 ValueTree::~ValueTree()
 {
@@ -720,23 +685,25 @@ ValueTree ValueTree::getParent() const noexcept
                                         : static_cast<SharedObject*> (nullptr));
 }
 
-ValueTree ValueTree::getSibling (const int delta) const noexcept
+ValueTree ValueTree::getRoot() const noexcept
+{
+    return ValueTree (object != nullptr ? object->getRoot()
+                                        : static_cast<SharedObject*> (nullptr));
+}
+
+ValueTree ValueTree::getSibling (int delta) const noexcept
 {
     if (object == nullptr || object->parent == nullptr)
-        return ValueTree();
+        return {};
 
-    const int index = object->parent->indexOf (*this) + delta;
+    auto index = object->parent->indexOf (*this) + delta;
     return ValueTree (object->parent->children.getObjectPointer (index));
 }
 
 static const var& getNullVarRef() noexcept
 {
-   #if JUCE_ALLOW_STATIC_NULL_VARIABLES
-    return var::null;
-   #else
     static var nullVar;
     return nullVar;
-   #endif
 }
 
 const var& ValueTree::operator[] (const Identifier& name) const noexcept
@@ -762,12 +729,18 @@ const var* ValueTree::getPropertyPointer (const Identifier& name) const noexcept
 }
 
 ValueTree& ValueTree::setProperty (const Identifier& name, const var& newValue, UndoManager* undoManager)
+{
+    return setPropertyExcludingListener (nullptr, name, newValue, undoManager);
+}
+
+ValueTree& ValueTree::setPropertyExcludingListener (Listener* listenerToExclude, const Identifier& name,
+                                                    const var& newValue, UndoManager* undoManager)
 {
     jassert (name.toString().isNotEmpty()); // Must have a valid property name!
     jassert (object != nullptr); // Trying to add a property to a null ValueTree will fail!
 
     if (object != nullptr)
-        object->setProperty (name, newValue, undoManager);
+        object->setProperty (name, newValue, undoManager, listenerToExclude);
 
     return *this;
 }
@@ -777,13 +750,13 @@ bool ValueTree::hasProperty (const Identifier& name) const noexcept
     return object != nullptr && object->hasProperty (name);
 }
 
-void ValueTree::removeProperty (const Identifier& name, UndoManager* const undoManager)
+void ValueTree::removeProperty (const Identifier& name, UndoManager* undoManager)
 {
     if (object != nullptr)
         object->removeProperty (name, undoManager);
 }
 
-void ValueTree::removeAllProperties (UndoManager* const undoManager)
+void ValueTree::removeAllProperties (UndoManager* undoManager)
 {
     if (object != nullptr)
         object->removeAllProperties (undoManager);
@@ -794,13 +767,13 @@ int ValueTree::getNumProperties() const noexcept
     return object == nullptr ? 0 : object->properties.size();
 }
 
-Identifier ValueTree::getPropertyName (const int index) const noexcept
+Identifier ValueTree::getPropertyName (int index) const noexcept
 {
     return object == nullptr ? Identifier()
                              : object->properties.getName (index);
 }
 
-void ValueTree::copyPropertiesFrom (const ValueTree& source, UndoManager* const undoManager)
+void ValueTree::copyPropertiesFrom (const ValueTree& source, UndoManager* undoManager)
 {
     jassert (object != nullptr || source.object == nullptr); // Trying to add properties to a null ValueTree will fail!
 
@@ -816,12 +789,11 @@ int ValueTree::getReferenceCount() const noexcept
 }
 
 //==============================================================================
-class ValueTreePropertyValueSource  : public Value::ValueSource,
-                                      private ValueTree::Listener
+struct ValueTreePropertyValueSource  : public Value::ValueSource,
+                                       private ValueTree::Listener
 {
-public:
-    ValueTreePropertyValueSource (const ValueTree& vt, const Identifier& prop, UndoManager* um)
-        : tree (vt), property (prop), undoManager (um)
+    ValueTreePropertyValueSource (const ValueTree& vt, const Identifier& prop, UndoManager* um, bool sync)
+        : tree (vt), property (prop), undoManager (um), updateSynchronously (sync)
     {
         tree.addListener (this);
     }
@@ -838,11 +810,12 @@ private:
     ValueTree tree;
     const Identifier property;
     UndoManager* const undoManager;
+    const bool updateSynchronously;
 
     void valueTreePropertyChanged (ValueTree& changedTree, const Identifier& changedProperty) override
     {
         if (tree == changedTree && property == changedProperty)
-            sendChangeMessage (false);
+            sendChangeMessage (updateSynchronously);
     }
 
     void valueTreeChildAdded (ValueTree&, ValueTree&) override {}
@@ -853,9 +826,9 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ValueTreePropertyValueSource)
 };
 
-Value ValueTree::getPropertyAsValue (const Identifier& name, UndoManager* const undoManager)
+Value ValueTree::getPropertyAsValue (const Identifier& name, UndoManager* undoManager, bool updateSynchronously)
 {
-    return Value (new ValueTreePropertyValueSource (*this, name, undoManager));
+    return Value (new ValueTreePropertyValueSource (*this, name, undoManager, updateSynchronously));
 }
 
 //==============================================================================
@@ -872,7 +845,8 @@ ValueTree ValueTree::getChild (int index) const
 
 ValueTree::Iterator::Iterator (const ValueTree& v, bool isEnd) noexcept
    : internal (v.object != nullptr ? (isEnd ? v.object->children.end() : v.object->children.begin()) : nullptr)
-{}
+{
+}
 
 ValueTree::Iterator& ValueTree::Iterator::operator++() noexcept
 {
@@ -918,7 +892,7 @@ int ValueTree::indexOf (const ValueTree& child) const noexcept
     return object != nullptr ? object->indexOf (child) : -1;
 }
 
-void ValueTree::addChild (const ValueTree& child, int index, UndoManager* const undoManager)
+void ValueTree::addChild (const ValueTree& child, int index, UndoManager* undoManager)
 {
     jassert (object != nullptr); // Trying to add a child to a null ValueTree!
 
@@ -926,19 +900,24 @@ void ValueTree::addChild (const ValueTree& child, int index, UndoManager* const
         object->addChild (child.object, index, undoManager);
 }
 
-void ValueTree::removeChild (const int childIndex, UndoManager* const undoManager)
+void ValueTree::appendChild (const ValueTree& child, UndoManager* undoManager)
+{
+    addChild (child, -1, undoManager);
+}
+
+void ValueTree::removeChild (int childIndex, UndoManager* undoManager)
 {
     if (object != nullptr)
         object->removeChild (childIndex, undoManager);
 }
 
-void ValueTree::removeChild (const ValueTree& child, UndoManager* const undoManager)
+void ValueTree::removeChild (const ValueTree& child, UndoManager* undoManager)
 {
     if (object != nullptr)
         object->removeChild (object->children.indexOf (child.object), undoManager);
 }
 
-void ValueTree::removeAllChildren (UndoManager* const undoManager)
+void ValueTree::removeAllChildren (UndoManager* undoManager)
 {
     if (object != nullptr)
         object->removeAllChildren (undoManager);
@@ -955,8 +934,8 @@ void ValueTree::createListOfChildren (OwnedArray<ValueTree>& list) const
 {
     jassert (object != nullptr);
 
-    for (int i = 0; i < object->children.size(); ++i)
-        list.add (new ValueTree (object->children.getObjectPointerUnchecked(i)));
+    for (auto* o : object->children)
+        list.add (new ValueTree (o));
 }
 
 void ValueTree::reorderChildren (const OwnedArray<ValueTree>& newOrder, UndoManager* undoManager)
@@ -1005,20 +984,24 @@ ValueTree ValueTree::fromXml (const XmlElement& xml)
         v.object->properties.setFromXmlAttributes (xml);
 
         forEachXmlChildElement (xml, e)
-            v.addChild (fromXml (*e), -1, nullptr);
+            v.appendChild (fromXml (*e), nullptr);
 
         return v;
     }
 
     // ValueTrees don't have any equivalent to XML text elements!
     jassertfalse;
-    return ValueTree();
+    return {};
 }
 
 String ValueTree::toXmlString() const
 {
-    const ScopedPointer<XmlElement> xml (createXml());
-    return xml != nullptr ? xml->createDocument (StringRef()) : String();
+    ScopedPointer<XmlElement> xml (createXml());
+
+    if (xml != nullptr)
+        return xml->createDocument ({});
+
+    return {};
 }
 
 //==============================================================================
@@ -1029,14 +1012,14 @@ void ValueTree::writeToStream (OutputStream& output) const
 
 ValueTree ValueTree::readFromStream (InputStream& input)
 {
-    const String type (input.readString());
+    auto type = input.readString();
 
     if (type.isEmpty())
-        return ValueTree();
+        return {};
 
     ValueTree v (type);
 
-    const int numProps = input.readCompressedInt();
+    auto numProps = input.readCompressedInt();
 
     if (numProps < 0)
     {
@@ -1046,25 +1029,20 @@ ValueTree ValueTree::readFromStream (InputStream& input)
 
     for (int i = 0; i < numProps; ++i)
     {
-        const String name (input.readString());
+        auto name = input.readString();
 
         if (name.isNotEmpty())
-        {
-            const var value (var::readFromStream (input));
-            v.object->properties.set (name, value);
-        }
+            v.object->properties.set (name, var::readFromStream (input));
         else
-        {
             jassertfalse;  // trying to read corrupted data!
-        }
     }
 
-    const int numChildren = input.readCompressedInt();
+    auto numChildren = input.readCompressedInt();
     v.object->children.ensureStorageAllocated (numChildren);
 
     for (int i = 0; i < numChildren; ++i)
     {
-        ValueTree child (readFromStream (input));
+        auto child = readFromStream (input);
 
         if (! child.isValid())
             return v;
@@ -1076,13 +1054,13 @@ ValueTree ValueTree::readFromStream (InputStream& input)
     return v;
 }
 
-ValueTree ValueTree::readFromData (const void* const data, const size_t numBytes)
+ValueTree ValueTree::readFromData (const void* data, size_t numBytes)
 {
     MemoryInputStream in (data, numBytes, false);
     return readFromStream (in);
 }
 
-ValueTree ValueTree::readFromGZIPData (const void* const data, const size_t numBytes)
+ValueTree ValueTree::readFromGZIPData (const void* data, size_t numBytes)
 {
     MemoryInputStream in (data, numBytes, false);
     GZIPDecompressorInputStream gzipStream (in);
@@ -1097,7 +1075,7 @@ void ValueTree::Listener::valueTreeRedirected (ValueTree&) {}
 class ValueTreeTests  : public UnitTest
 {
 public:
-    ValueTreeTests() : UnitTest ("ValueTrees") {}
+    ValueTreeTests() : UnitTest ("ValueTrees", "Values") {}
 
     static String createRandomIdentifier (Random& r)
     {
@@ -1159,23 +1137,30 @@ public:
     void runTest() override
     {
         beginTest ("ValueTree");
-        Random r = getRandom();
+        auto r = getRandom();
 
         for (int i = 10; --i >= 0;)
         {
             MemoryOutputStream mo;
-            ValueTree v1 (createRandomTree (nullptr, 0, r));
+            auto v1 = createRandomTree (nullptr, 0, r);
             v1.writeToStream (mo);
 
             MemoryInputStream mi (mo.getData(), mo.getDataSize(), false);
-            ValueTree v2 = ValueTree::readFromStream (mi);
+            auto v2 = ValueTree::readFromStream (mi);
             expect (v1.isEquivalentTo (v2));
 
+            MemoryOutputStream zipped;
+            {
+                GZIPCompressorOutputStream zippedOut (zipped);
+                v1.writeToStream (zippedOut);
+            }
+            expect (v1.isEquivalentTo (ValueTree::readFromGZIPData (zipped.getData(), zipped.getDataSize())));
+
             ScopedPointer<XmlElement> xml1 (v1.createXml());
             ScopedPointer<XmlElement> xml2 (v2.createCopy().createXml());
-            expect (xml1->isEquivalentTo (xml2, false));
+            expect (xml1->isEquivalentTo (xml2.get(), false));
 
-            ValueTree v4 = v2.createCopy();
+            auto v4 = v2.createCopy();
             expect (v1.isEquivalentTo (v4));
         }
     }
@@ -1184,3 +1169,5 @@ public:
 static ValueTreeTests valueTreeTests;
 
 #endif
+
+} // namespace juce
