@@ -2,29 +2,26 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_INTERPROCESSCONNECTIONSERVER_H_INCLUDED
-#define JUCE_INTERPROCESSCONNECTIONSERVER_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -51,7 +48,7 @@ public:
     //==============================================================================
     /** Starts an internal thread which listens on the given port number.
 
-        While this is running, in another process tries to connect with the
+        While this is running, if another process tries to connect with the
         InterprocessConnection::connectToSocket() method, this object will call
         createConnectionObject() to create a connection to that client.
 
@@ -74,6 +71,15 @@ public:
     */
     void stop();
 
+    /** Returns the local port number to which this server is currently bound.
+
+        This is useful if you need to know to which port the OS has actually bound your
+        socket when calling beginWaitingForSocket with a port number of zero.
+
+        Returns -1 if the function fails.
+    */
+    int getBoundPort() const noexcept;
+
 protected:
     /** Creates a suitable connection object for a client process that wants to
         connect to this one.
@@ -86,7 +92,6 @@ protected:
     */
     virtual InterprocessConnection* createConnectionObject() = 0;
 
-
 private:
     //==============================================================================
     ScopedPointer<StreamingSocket> socket;
@@ -96,5 +101,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (InterprocessConnectionServer)
 };
 
-
-#endif   // JUCE_INTERPROCESSCONNECTIONSERVER_H_INCLUDED
+} // namespace juce
