@@ -2,26 +2,27 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 typedef void (*AppFocusChangeCallback)();
 AppFocusChangeCallback appFocusChangeCallback = nullptr;
 
@@ -54,7 +55,8 @@ public:
             [[NSDistributedNotificationCenter defaultCenter] addObserver: delegate
                                                                 selector: @selector (broadcastMessageCallback:)
                                                                     name: getBroadcastEventName()
-                                                                  object: nil];
+                                                                  object: nil
+                                                      suspensionBehavior: NSNotificationSuspensionBehaviorDeliverImmediately];
         }
         else
         {
@@ -100,7 +102,7 @@ private:
     {
         AppDelegateClass()  : ObjCClass<NSObject> ("JUCEAppDelegate_")
         {
-            addMethod (@selector (applicationWillFinishLaunching:), applicationWillFinishLaunching, "v@:@@");
+            addMethod (@selector (applicationWillFinishLaunching:), applicationWillFinishLaunching, "v@:@");
             addMethod (@selector (getUrl:withReplyEvent:),          getUrl_withReplyEvent,          "v@:@@");
             addMethod (@selector (applicationShouldTerminate:),     applicationShouldTerminate,     "I@:@");
             addMethod (@selector (applicationWillTerminate:),       applicationWillTerminate,       "v@:@");
@@ -114,11 +116,22 @@ private:
             addMethod (@selector (mainMenuTrackingEnded:),          mainMenuTrackingEnded,          "v@:@");
             addMethod (@selector (dummyMethod),                     dummyMethod,                    "v@:");
 
+           #if JUCE_PUSH_NOTIFICATIONS
+            //==============================================================================
+            addIvar<NSObject<NSApplicationDelegate, NSUserNotificationCenterDelegate>*> ("pushNotificationsDelegate");
+
+            addMethod (@selector (applicationDidFinishLaunching:),                                applicationDidFinishLaunching,          "v@:@");
+            addMethod (@selector (setPushNotificationsDelegate:),                                 setPushNotificationsDelegate,           "v@:@");
+            addMethod (@selector (application:didRegisterForRemoteNotificationsWithDeviceToken:), registeredForRemoteNotifications,       "v@:@@");
+            addMethod (@selector (application:didFailToRegisterForRemoteNotificationsWithError:), failedToRegisterForRemoteNotifications, "v@:@@");
+            addMethod (@selector (application:didReceiveRemoteNotification:),                     didReceiveRemoteNotification,           "v@:@@");
+           #endif
+
             registerClass();
         }
 
     private:
-        static void applicationWillFinishLaunching (id self, SEL, NSApplication*, NSNotification*)
+        static void applicationWillFinishLaunching (id self, SEL, NSNotification*)
         {
             [[NSAppleEventManager sharedAppleEventManager] setEventHandler: self
                                                                andSelector: @selector (getUrl:withReplyEvent:)
@@ -126,9 +139,22 @@ private:
                                                                 andEventID: kAEGetURL];
         }
 
+       #if JUCE_PUSH_NOTIFICATIONS
+        static void applicationDidFinishLaunching (id self, SEL, NSNotification* notification)
+        {
+            if (notification.userInfo != nil)
+            {
+                NSUserNotification* userNotification = [notification.userInfo objectForKey: nsStringLiteral ("NSApplicationLaunchUserNotificationKey")];
+
+                if (userNotification != nil && userNotification.userInfo != nil)
+                    didReceiveRemoteNotification (self, nil, [NSApplication sharedApplication], userNotification.userInfo);
+            }
+        }
+       #endif
+
         static NSApplicationTerminateReply applicationShouldTerminate (id /*self*/, SEL, NSApplication*)
         {
-            if (JUCEApplicationBase* const app = JUCEApplicationBase::getInstance())
+            if (auto* app = JUCEApplicationBase::getInstance())
             {
                 app->systemRequestedQuit();
 
@@ -146,7 +172,7 @@ private:
 
         static BOOL application_openFile (id /*self*/, SEL, NSApplication*, NSString* filename)
         {
-            if (JUCEApplicationBase* const app = JUCEApplicationBase::getInstance())
+            if (auto* app = JUCEApplicationBase::getInstance())
             {
                 app->anotherInstanceStarted (quotedIfContainsSpaces (filename));
                 return YES;
@@ -157,7 +183,7 @@ private:
 
         static void application_openFiles (id /*self*/, SEL, NSApplication*, NSArray* filenames)
         {
-            if (JUCEApplicationBase* const app = JUCEApplicationBase::getInstance())
+            if (auto* app = JUCEApplicationBase::getInstance())
             {
                 StringArray files;
 
@@ -176,7 +202,7 @@ private:
         static void broadcastMessageCallback (id /*self*/, SEL, NSNotification* n)
         {
             NSDictionary* dict = (NSDictionary*) [n userInfo];
-            const String messageString (nsStringToJuce ((NSString*) [dict valueForKey: nsStringLiteral ("message")]));
+            auto messageString = nsStringToJuce ((NSString*) [dict valueForKey: nsStringLiteral ("message")]);
             MessageManager::getInstance()->deliverBroadcastMessage (messageString);
         }
 
@@ -202,7 +228,7 @@ private:
 
         static void getUrl_withReplyEvent (id /*self*/, SEL, NSAppleEventDescriptor* event, NSAppleEventDescriptor*)
         {
-            if (JUCEApplicationBase* const app = JUCEApplicationBase::getInstance())
+            if (auto* app = JUCEApplicationBase::getInstance())
                 app->anotherInstanceStarted (quotedIfContainsSpaces ([[event paramDescriptorForKeyword: keyDirectObject] stringValue]));
         }
 
@@ -214,13 +240,80 @@ private:
 
             return s;
         }
+
+       #if JUCE_PUSH_NOTIFICATIONS
+        //==============================================================================
+        static void setPushNotificationsDelegate (id self, SEL, NSObject<NSApplicationDelegate, NSUserNotificationCenterDelegate>* delegate)
+        {
+            object_setInstanceVariable (self, "pushNotificationsDelegate", delegate);
+        }
+
+        static NSObject<NSApplicationDelegate, NSUserNotificationCenterDelegate>* getPushNotificationsDelegate (id self)
+        {
+            return getIvar<NSObject<NSApplicationDelegate, NSUserNotificationCenterDelegate>*> (self, "pushNotificationsDelegate");
+        }
+
+        static void registeredForRemoteNotifications (id self, SEL, NSApplication* application, NSData* deviceToken)
+        {
+            auto* delegate = getPushNotificationsDelegate (self);
+
+            SEL selector = NSSelectorFromString (@"application:didRegisterForRemoteNotificationsWithDeviceToken:");
+
+            if (delegate != nil && [delegate respondsToSelector: selector])
+            {
+                NSInvocation* invocation = [NSInvocation invocationWithMethodSignature: [delegate methodSignatureForSelector: selector]];
+                [invocation setSelector: selector];
+                [invocation setTarget: delegate];
+                [invocation setArgument: &application atIndex:2];
+                [invocation setArgument: &deviceToken atIndex:3];
+
+                [invocation invoke];
+            }
+        }
+
+        static void failedToRegisterForRemoteNotifications (id self, SEL, NSApplication* application, NSError* error)
+        {
+            auto* delegate = getPushNotificationsDelegate (self);
+
+            SEL selector = NSSelectorFromString (@"application:didFailToRegisterForRemoteNotificationsWithError:");
+
+            if (delegate != nil && [delegate respondsToSelector: selector])
+            {
+                NSInvocation* invocation = [NSInvocation invocationWithMethodSignature: [delegate methodSignatureForSelector: selector]];
+                [invocation setSelector: selector];
+                [invocation setTarget: delegate];
+                [invocation setArgument: &application atIndex:2];
+                [invocation setArgument: &error       atIndex:3];
+
+                [invocation invoke];
+            }
+        }
+
+        static void didReceiveRemoteNotification (id self, SEL, NSApplication* application, NSDictionary* userInfo)
+        {
+            auto* delegate = getPushNotificationsDelegate (self);
+
+            SEL selector = NSSelectorFromString (@"application:didReceiveRemoteNotification:");
+
+            if (delegate != nil && [delegate respondsToSelector: selector])
+            {
+                NSInvocation* invocation = [NSInvocation invocationWithMethodSignature: [delegate methodSignatureForSelector: selector]];
+                [invocation setSelector: selector];
+                [invocation setTarget: delegate];
+                [invocation setArgument: &application atIndex:2];
+                [invocation setArgument: &userInfo    atIndex:3];
+
+                [invocation invoke];
+            }
+        }
+       #endif
     };
 };
 
 //==============================================================================
 void MessageManager::runDispatchLoop()
 {
-    if (! quitMessagePosted) // check that the quit message wasn't already posted..
+    if (quitMessagePosted.get() == 0) // check that the quit message wasn't already posted..
     {
         JUCE_AUTORELEASEPOOL
         {
@@ -255,7 +348,6 @@ void MessageManager::runDispatchLoop()
 static void shutdownNSApp()
 {
     [NSApp stop: nil];
-    [NSApp activateIgnoringOtherApps: YES]; // (if the app is inactive, it sits there and ignores the quit request until the next time it gets activated)
     [NSEvent startPeriodicEventsAfterDelay: 0  withPeriod: 0.1];
 }
 
@@ -291,13 +383,13 @@ bool MessageManager::runDispatchLoopUntil (int millisecondsToRunFor)
 
     uint32 endTime = Time::getMillisecondCounter() + (uint32) millisecondsToRunFor;
 
-    while (! quitMessagePosted)
+    while (quitMessagePosted.get() == 0)
     {
         JUCE_AUTORELEASEPOOL
         {
             CFRunLoopRunInMode (kCFRunLoopDefaultMode, 0.001, true);
 
-            NSEvent* e = [NSApp nextEventMatchingMask: NSAnyEventMask
+            NSEvent* e = [NSApp nextEventMatchingMask: NSEventMaskAny
                                             untilDate: [NSDate dateWithTimeIntervalSinceNow: 0.001]
                                                inMode: NSDefaultRunLoopMode
                                               dequeue: YES];
@@ -310,7 +402,7 @@ bool MessageManager::runDispatchLoopUntil (int millisecondsToRunFor)
         }
     }
 
-    return ! quitMessagePosted;
+    return quitMessagePosted.get() == 0;
 }
 #endif
 
@@ -422,6 +514,8 @@ private:
     };
 };
 
-MountedVolumeListChangeDetector::MountedVolumeListChangeDetector()  { pimpl = new Pimpl (*this); }
+MountedVolumeListChangeDetector::MountedVolumeListChangeDetector()  { pimpl.reset (new Pimpl (*this)); }
 MountedVolumeListChangeDetector::~MountedVolumeListChangeDetector() {}
 #endif
+
+} // namespace juce
