@@ -2,29 +2,26 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   The code included in this file is provided under the terms of the ISC license
+   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
+   To use, copy, modify, and/or distribute this software for any purpose with or
+   without fee is hereby granted provided that the above copyright notice and
+   this permission notice appear in all copies.
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
-   ------------------------------------------------------------------------------
-
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_TIMER_H_INCLUDED
-#define JUCE_TIMER_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -116,6 +113,9 @@ public:
     */
     int getTimerInterval() const noexcept                   { return timerPeriodMs; }
 
+    //==============================================================================
+    /** Invokes a lambda after a given number of milliseconds. */
+    static void JUCE_CALLTYPE callAfterDelay (int milliseconds, std::function<void()> functionToCall);
 
     //==============================================================================
     /** For internal use only: invokes any timers that need callbacks.
@@ -126,10 +126,10 @@ public:
 private:
     class TimerThread;
     friend class TimerThread;
-    int timerCountdownMs, timerPeriodMs; // NB: these member variable names are a little verbose
-    Timer* previousTimer, *nextTimer;    // to reduce risk of name-clashes with user subclasses
+    size_t positionInQueue = (size_t) -1;
+    int timerPeriodMs = 0;
 
-    Timer& operator= (const Timer&) JUCE_DELETED_FUNCTION;
+    Timer& operator= (const Timer&) = delete;
 };
 
-#endif   // JUCE_TIMER_H_INCLUDED
+} // namespace juce
