@@ -2,27 +2,32 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-ColourGradient::ColourGradient() noexcept
+namespace juce
+{
+
+ColourGradient::ColourGradient() noexcept  : isRadial (false)
 {
    #if JUCE_DEBUG
     point1.setX (987654.0f);
@@ -32,19 +37,60 @@ ColourGradient::ColourGradient() noexcept
    #endif
 }
 
-ColourGradient::ColourGradient (Colour colour1, const float x1, const float y1,
-                                Colour colour2, const float x2, const float y2,
-                                const bool radial)
-    : point1 (x1, y1),
-      point2 (x2, y2),
-      isRadial (radial)
+ColourGradient::ColourGradient (const ColourGradient& other)
+    : point1 (other.point1), point2 (other.point2), isRadial (other.isRadial), colours (other.colours)
+{}
+
+ColourGradient::ColourGradient (ColourGradient&& other) noexcept
+    : point1 (other.point1), point2 (other.point2), isRadial (other.isRadial),
+      colours (static_cast<Array<ColourPoint>&&> (other.colours))
+{}
+
+ColourGradient& ColourGradient::operator= (const ColourGradient& other)
 {
-    colours.add (ColourPoint (0.0, colour1));
-    colours.add (ColourPoint (1.0, colour2));
+    point1 = other.point1;
+    point2 = other.point2;
+    isRadial = other.isRadial;
+    colours = other.colours;
+    return *this;
 }
 
-ColourGradient::~ColourGradient()
+ColourGradient& ColourGradient::operator= (ColourGradient&& other) noexcept
 {
+    point1 = other.point1;
+    point2 = other.point2;
+    isRadial = other.isRadial;
+    colours = static_cast<Array<ColourPoint>&&> (other.colours);
+    return *this;
+}
+
+ColourGradient::ColourGradient (Colour colour1, float x1, float y1,
+                                Colour colour2, float x2, float y2, bool radial)
+    : ColourGradient (colour1, Point<float> (x1, y1),
+                      colour2, Point<float> (x2, y2), radial)
+{
+}
+
+ColourGradient::ColourGradient (Colour colour1, Point<float> p1,
+                                Colour colour2, Point<float> p2, bool radial)
+    : point1 (p1),
+      point2 (p2),
+      isRadial (radial)
+{
+    colours.add (ColourPoint { 0.0, colour1 },
+                 ColourPoint { 1.0, colour2 });
+}
+
+ColourGradient::~ColourGradient() {}
+
+ColourGradient ColourGradient::vertical (Colour c1, float y1, Colour c2, float y2)
+{
+    return { c1, 0, y1, c2, 0, y2, false };
+}
+
+ColourGradient ColourGradient::horizontal (Colour c1, float x1, Colour c2, float x2)
+{
+    return { c1, x1, 0, c2, x2, 0, false };
 }
 
 bool ColourGradient::operator== (const ColourGradient& other) const noexcept
@@ -72,18 +118,18 @@ int ColourGradient::addColour (const double proportionAlongGradient, Colour colo
 
     if (proportionAlongGradient <= 0)
     {
-        colours.set (0, ColourPoint (0.0, colour));
+        colours.set (0, { 0.0, colour });
         return 0;
     }
 
-    const double pos = jmin (1.0, proportionAlongGradient);
+    auto pos = jmin (1.0, proportionAlongGradient);
 
     int i;
     for (i = 0; i < colours.size(); ++i)
         if (colours.getReference(i).position > pos)
             break;
 
-    colours.insert (i, ColourPoint (pos, colour));
+    colours.insert (i, { pos, colour });
     return i;
 }
 
@@ -95,11 +141,8 @@ void ColourGradient::removeColour (int index)
 
 void ColourGradient::multiplyOpacity (const float multiplier) noexcept
 {
-    for (int i = 0; i < colours.size(); ++i)
-    {
-        Colour& c = colours.getReference(i).colour;
-        c = c.withMultipliedAlpha (multiplier);
-    }
+    for (auto& c : colours)
+        c.colour = c.colour.withMultipliedAlpha (multiplier);
 }
 
 //==============================================================================
@@ -108,7 +151,7 @@ int ColourGradient::getNumColours() const noexcept
     return colours.size();
 }
 
-double ColourGradient::getColourPosition (const int index) const noexcept
+double ColourGradient::getColourPosition (int index) const noexcept
 {
     if (isPositiveAndBelow (index, colours.size()))
         return colours.getReference (index).position;
@@ -116,12 +159,12 @@ double ColourGradient::getColourPosition (const int index) const noexcept
     return 0;
  }
 
-Colour ColourGradient::getColour (const int index) const noexcept
+Colour ColourGradient::getColour (int index) const noexcept
 {
     if (isPositiveAndBelow (index, colours.size()))
         return colours.getReference (index).colour;
 
-    return Colour();
+    return {};
 }
 
 void ColourGradient::setColour (int index, Colour newColour) noexcept
@@ -130,9 +173,9 @@ void ColourGradient::setColour (int index, Colour newColour) noexcept
         colours.getReference (index).colour = newColour;
 }
 
-Colour ColourGradient::getColourAtPosition (const double position) const noexcept
+Colour ColourGradient::getColourAtPosition (double position) const noexcept
 {
-    jassert (colours.getReference(0).position == 0); // the first colour specified has to go at position 0
+    jassert (colours.getReference(0).position == 0.0); // the first colour specified has to go at position 0
 
     if (position <= 0 || colours.size() <= 1)
         return colours.getReference(0).colour;
@@ -141,12 +184,12 @@ Colour ColourGradient::getColourAtPosition (const double position) const noexcep
     while (position < colours.getReference(i).position)
         --i;
 
-    const ColourPoint& p1 = colours.getReference (i);
+    auto& p1 = colours.getReference (i);
 
     if (i >= colours.size() - 1)
         return p1.colour;
 
-    const ColourPoint& p2 = colours.getReference (i + 1);
+    auto& p2 = colours.getReference (i + 1);
 
     return p1.colour.interpolatedWith (p2.colour, (float) ((position - p1.position) / (p2.position - p1.position)));
 }
@@ -157,16 +200,16 @@ void ColourGradient::createLookupTable (PixelARGB* const lookupTable, const int
     JUCE_COLOURGRADIENT_CHECK_COORDS_INITIALISED // Trying to use this object without setting its coordinates?
     jassert (colours.size() >= 2);
     jassert (numEntries > 0);
-    jassert (colours.getReference(0).position == 0); // The first colour specified has to go at position 0
+    jassert (colours.getReference(0).position == 0.0); // The first colour specified has to go at position 0
 
-    PixelARGB pix1 (colours.getReference (0).colour.getPixelARGB());
+    auto pix1 = colours.getReference (0).colour.getPixelARGB();
     int index = 0;
 
     for (int j = 1; j < colours.size(); ++j)
     {
-        const ColourPoint& p = colours.getReference (j);
-        const int numToDo = roundToInt (p.position * (numEntries - 1)) - index;
-        const PixelARGB pix2 (p.colour.getPixelARGB());
+        auto& p = colours.getReference (j);
+        auto numToDo = roundToInt (p.position * (numEntries - 1)) - index;
+        auto pix2 = p.colour.getPixelARGB();
 
         for (int i = 0; i < numToDo; ++i)
         {
@@ -189,18 +232,18 @@ int ColourGradient::createLookupTable (const AffineTransform& transform, HeapBlo
     JUCE_COLOURGRADIENT_CHECK_COORDS_INITIALISED // Trying to use this object without setting its coordinates?
     jassert (colours.size() >= 2);
 
-    const int numEntries = jlimit (1, jmax (1, (colours.size() - 1) << 8),
-                                   3 * (int) point1.transformedBy (transform)
-                                                .getDistanceFrom (point2.transformedBy (transform)));
-    lookupTable.malloc ((size_t) numEntries);
+    auto numEntries = jlimit (1, jmax (1, (colours.size() - 1) << 8),
+                              3 * (int) point1.transformedBy (transform)
+                                              .getDistanceFrom (point2.transformedBy (transform)));
+    lookupTable.malloc (numEntries);
     createLookupTable (lookupTable, numEntries);
     return numEntries;
 }
 
 bool ColourGradient::isOpaque() const noexcept
 {
-    for (int i = 0; i < colours.size(); ++i)
-        if (! colours.getReference(i).colour.isOpaque())
+    for (auto& c : colours)
+        if (! c.colour.isOpaque())
             return false;
 
     return true;
@@ -208,19 +251,21 @@ bool ColourGradient::isOpaque() const noexcept
 
 bool ColourGradient::isInvisible() const noexcept
 {
-    for (int i = 0; i < colours.size(); ++i)
-        if (! colours.getReference(i).colour.isTransparent())
+    for (auto& c : colours)
+        if (! c.colour.isTransparent())
             return false;
 
     return true;
 }
 
-bool ColourGradient::ColourPoint::operator== (const ColourPoint& other) const noexcept
+bool ColourGradient::ColourPoint::operator== (ColourPoint other) const noexcept
 {
     return position == other.position && colour == other.colour;
 }
 
-bool ColourGradient::ColourPoint::operator!= (const ColourPoint& other) const noexcept
+bool ColourGradient::ColourPoint::operator!= (ColourPoint other) const noexcept
 {
     return position != other.position || colour != other.colour;
 }
+
+} // namespace juce
