@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 FillType::FillType() noexcept
     : colour (0xff000000)
 {
@@ -32,19 +37,24 @@ FillType::FillType (Colour c) noexcept
 {
 }
 
-FillType::FillType (const ColourGradient& gradient_)
-    : colour (0xff000000), gradient (new ColourGradient (gradient_))
+FillType::FillType (const ColourGradient& g)
+    : colour (0xff000000), gradient (new ColourGradient (g))
 {
 }
 
-FillType::FillType (const Image& image_, const AffineTransform& transform_) noexcept
-    : colour (0xff000000), image (image_), transform (transform_)
+FillType::FillType (ColourGradient&& g)
+    : colour (0xff000000), gradient (new ColourGradient (static_cast<ColourGradient&&> (g)))
+{
+}
+
+FillType::FillType (const Image& im, const AffineTransform& t) noexcept
+    : colour (0xff000000), image (im), transform (t)
 {
 }
 
 FillType::FillType (const FillType& other)
     : colour (other.colour),
-      gradient (other.gradient.createCopy()),
+      gradient (createCopyIfNotNull (other.gradient.get())),
       image (other.image),
       transform (other.transform)
 {
@@ -55,7 +65,7 @@ FillType& FillType::operator= (const FillType& other)
     if (this != &other)
     {
         colour = other.colour;
-        gradient = other.gradient.createCopy();
+        gradient.reset (createCopyIfNotNull (other.gradient.get()));
         image = other.image;
         transform = other.transform;
     }
@@ -63,10 +73,9 @@ FillType& FillType::operator= (const FillType& other)
     return *this;
 }
 
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
 FillType::FillType (FillType&& other) noexcept
     : colour (other.colour),
-      gradient (other.gradient.release()),
+      gradient (static_cast<ScopedPointer<ColourGradient>&&> (other.gradient)),
       image (static_cast<Image&&> (other.image)),
       transform (other.transform)
 {
@@ -77,12 +86,11 @@ FillType& FillType::operator= (FillType&& other) noexcept
     jassert (this != &other); // hopefully the compiler should make this situation impossible!
 
     colour = other.colour;
-    gradient = other.gradient.release();
+    gradient = static_cast<ScopedPointer<ColourGradient>&&> (other.gradient);
     image = static_cast<Image&&> (other.image);
     transform = other.transform;
     return *this;
 }
-#endif
 
 FillType::~FillType() noexcept
 {
@@ -103,8 +111,8 @@ bool FillType::operator!= (const FillType& other) const
 
 void FillType::setColour (Colour newColour) noexcept
 {
-    gradient = nullptr;
-    image = Image();
+    gradient.reset();
+    image = {};
     colour = newColour;
 }
 
@@ -116,17 +124,17 @@ void FillType::setGradient (const ColourGradient& newGradient)
     }
     else
     {
-        image = Image();
-        gradient = new ColourGradient (newGradient);
+        image = {};
+        gradient.reset (new ColourGradient (newGradient));
         colour = Colours::black;
     }
 }
 
-void FillType::setTiledImage (const Image& image_, const AffineTransform& transform_) noexcept
+void FillType::setTiledImage (const Image& newImage, const AffineTransform& newTransform) noexcept
 {
-    gradient = nullptr;
-    image = image_;
-    transform = transform_;
+    gradient.reset();
+    image = newImage;
+    transform = newTransform;
     colour = Colours::black;
 }
 
@@ -146,3 +154,5 @@ FillType FillType::transformed (const AffineTransform& t) const
     f.transform = f.transform.followedBy (t);
     return f;
 }
+
+} // namespace juce
