@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 // this will throw an assertion if you try to draw something that's not
 // possible in postscript
 #define WARN_ABOUT_NON_POSTSCRIPT_OPERATIONS 0
@@ -200,7 +205,7 @@ void LowLevelGraphicsPostScriptRenderer::writeClip()
 
         int itemsOnLine = 0;
 
-        for (const Rectangle<int>* i = stateStack.getLast()->clip.begin(), * const e = stateStack.getLast()->clip.end(); i != e; ++i)
+        for (auto& i : stateStack.getLast()->clip)
         {
             if (++itemsOnLine == 6)
             {
@@ -208,8 +213,8 @@ void LowLevelGraphicsPostScriptRenderer::writeClip()
                 out << '\n';
             }
 
-            out << i->getX() << ' ' << -i->getY() << ' '
-                << i->getWidth() << ' ' << -i->getHeight() << " pr ";
+            out << i.getX() << ' ' << -i.getY() << ' '
+                << i.getWidth() << ' ' << -i.getHeight() << " pr ";
         }
 
         out << "endclip\n";
@@ -482,7 +487,7 @@ void LowLevelGraphicsPostScriptRenderer::drawImage (const Image& sourceImage, co
     out << "newpath ";
     int itemsOnLine = 0;
 
-    for (const Rectangle<int>* i = imageClip.begin(), * const e = imageClip.end(); i != e; ++i)
+    for (auto& i : imageClip)
     {
         if (++itemsOnLine == 6)
         {
@@ -490,7 +495,7 @@ void LowLevelGraphicsPostScriptRenderer::drawImage (const Image& sourceImage, co
             itemsOnLine = 0;
         }
 
-        out << i->getX() << ' ' << i->getY() << ' ' << i->getWidth() << ' ' << i->getHeight() << " pr ";
+        out << i.getX() << ' ' << i.getY() << ' ' << i.getWidth() << ' ' << i.getHeight() << " pr ";
     }
 
     out << " clip newpath\n";
@@ -531,3 +536,5 @@ void LowLevelGraphicsPostScriptRenderer::drawGlyph (int glyphNumber, const Affin
     font.getTypeface()->getOutlineForGlyph (glyphNumber, p);
     fillPath (p, AffineTransform::scale (font.getHeight() * font.getHorizontalScale(), font.getHeight()).followedBy (transform));
 }
+
+} // namespace juce
