@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 namespace FontValues
 {
     static float limitFontHeight (const float height) noexcept
@@ -38,14 +43,14 @@ namespace FontValues
 typedef Typeface::Ptr (*GetTypefaceForFont) (const Font&);
 GetTypefaceForFont juce_getTypefaceForFont = nullptr;
 
-float Font::getDefaultMinimumHorizontalScaleFactor() noexcept   { return FontValues::minimumHorizontalScale; }
+float Font::getDefaultMinimumHorizontalScaleFactor() noexcept                { return FontValues::minimumHorizontalScale; }
 void Font::setDefaultMinimumHorizontalScaleFactor (float newValue) noexcept  { FontValues::minimumHorizontalScale = newValue; }
 
 //==============================================================================
 class TypefaceCache  : private DeletedAtShutdown
 {
 public:
-    TypefaceCache()  : counter (0)
+    TypefaceCache()
     {
         setSize (10);
     }
@@ -55,7 +60,7 @@ public:
         clearSingletonInstance();
     }
 
-    juce_DeclareSingleton (TypefaceCache, false)
+    JUCE_DECLARE_SINGLETON (TypefaceCache, false)
 
     void setSize (const int numToCache)
     {
@@ -77,8 +82,8 @@ public:
     {
         const ScopedReadLock slr (lock);
 
-        const String faceName (font.getTypefaceName());
-        const String faceStyle (font.getTypefaceStyle());
+        auto faceName = font.getTypefaceName();
+        auto faceStyle = font.getTypefaceStyle();
 
         jassert (faceName.isNotEmpty());
 
@@ -98,11 +103,11 @@ public:
 
         const ScopedWriteLock slw (lock);
         int replaceIndex = 0;
-        size_t bestLastUsageCount = std::numeric_limits<size_t>::max();
+        auto bestLastUsageCount = std::numeric_limits<size_t>::max();
 
         for (int i = faces.size(); --i >= 0;)
         {
-            const size_t lu = faces.getReference(i).lastUsageCount;
+            auto lu = faces.getReference(i).lastUsageCount;
 
             if (bestLastUsageCount > lu)
             {
@@ -111,7 +116,7 @@ public:
             }
         }
 
-        CachedFace& face = faces.getReference (replaceIndex);
+        auto& face = faces.getReference (replaceIndex);
         face.typefaceName = faceName;
         face.typefaceStyle = faceStyle;
         face.lastUsageCount = ++counter;
@@ -134,25 +139,25 @@ public:
 private:
     struct CachedFace
     {
-        CachedFace() noexcept  : lastUsageCount (0) {}
+        CachedFace() noexcept {}
 
         // Although it seems a bit wacky to store the name here, it's because it may be a
         // placeholder rather than a real one, e.g. "<Sans-Serif>" vs the actual typeface name.
         // Since the typeface itself doesn't know that it may have this alias, the name under
         // which it was fetched needs to be stored separately.
         String typefaceName, typefaceStyle;
-        size_t lastUsageCount;
+        size_t lastUsageCount = 0;
         Typeface::Ptr typeface;
     };
 
     ReadWriteLock lock;
     Array<CachedFace> faces;
-    size_t counter;
+    size_t counter = 0;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (TypefaceCache)
 };
 
-juce_ImplementSingleton (TypefaceCache)
+JUCE_IMPLEMENT_SINGLETON (TypefaceCache)
 
 void Typeface::setTypefaceCacheSize (int numFontsToCache)
 {
@@ -179,8 +184,7 @@ public:
         : typeface (TypefaceCache::getInstance()->defaultFace),
           typefaceName (Font::getDefaultSansSerifFontName()),
           typefaceStyle (Font::getDefaultStyle()),
-          height (FontValues::defaultFontHeight),
-          horizontalScale (1.0f), kerning (0), ascent (0), underline (false)
+          height (FontValues::defaultFontHeight)
     {
     }
 
@@ -188,7 +192,7 @@ public:
         : typefaceName (Font::getDefaultSansSerifFontName()),
           typefaceStyle (FontStyleHelpers::getStyleName (styleFlags)),
           height (fontHeight),
-          horizontalScale (1.0f), kerning (0), ascent (0), underline ((styleFlags & underlined) != 0)
+          underline ((styleFlags & underlined) != 0)
     {
         if (styleFlags == plain)
             typeface = TypefaceCache::getInstance()->defaultFace;
@@ -198,15 +202,14 @@ public:
         : typefaceName (name),
           typefaceStyle (FontStyleHelpers::getStyleName (styleFlags)),
           height (fontHeight),
-          horizontalScale (1.0f), kerning (0), ascent (0), underline ((styleFlags & underlined) != 0)
+          underline ((styleFlags & underlined) != 0)
     {
         if (styleFlags == plain && typefaceName.isEmpty())
             typeface = TypefaceCache::getInstance()->defaultFace;
     }
 
     SharedFontInternal (const String& name, const String& style, float fontHeight) noexcept
-        : typefaceName (name), typefaceStyle (style), height (fontHeight),
-          horizontalScale (1.0f), kerning (0), ascent (0), underline (false)
+        : typefaceName (name), typefaceStyle (style), height (fontHeight)
     {
         if (typefaceName.isEmpty())
             typefaceName = Font::getDefaultSansSerifFontName();
@@ -216,8 +219,7 @@ public:
         : typeface (face),
           typefaceName (face->getName()),
           typefaceStyle (face->getStyle()),
-          height (FontValues::defaultFontHeight),
-          horizontalScale (1.0f), kerning (0), ascent (0), underline (false)
+          height (FontValues::defaultFontHeight)
     {
         jassert (typefaceName.isNotEmpty());
     }
@@ -247,8 +249,8 @@ public:
 
     Typeface::Ptr typeface;
     String typefaceName, typefaceStyle;
-    float height, horizontalScale, kerning, ascent;
-    bool underline;
+    float height, horizontalScale = 1.0f, kerning = 0, ascent = 0;
+    bool underline = false;
 };
 
 //==============================================================================
@@ -277,7 +279,6 @@ Font& Font::operator= (const Font& other) noexcept
     return *this;
 }
 
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
 Font::Font (Font&& other) noexcept
     : font (static_cast<ReferenceCountedObjectPtr<SharedFontInternal>&&> (other.font))
 {
@@ -288,7 +289,6 @@ Font& Font::operator= (Font&& other) noexcept
     font = static_cast<ReferenceCountedObjectPtr<SharedFontInternal>&&> (other.font);
     return *this;
 }
-#endif
 
 Font::~Font() noexcept
 {
@@ -320,18 +320,13 @@ void Font::checkTypefaceSuitability()
 //==============================================================================
 struct FontPlaceholderNames
 {
-    FontPlaceholderNames()
-       : sans    ("<Sans-Serif>"),
-         serif   ("<Serif>"),
-         mono    ("<Monospaced>"),
-         regular ("<Regular>")
-    {
-    }
-
-    String sans, serif, mono, regular;
+    String sans    { "<Sans-Serif>" },
+           serif   { "<Serif>" },
+           mono    { "<Monospaced>" },
+           regular { "<Regular>" };
 };
 
-const FontPlaceholderNames& getFontPlaceholderNames()
+static const FontPlaceholderNames& getFontPlaceholderNames()
 {
     static FontPlaceholderNames names;
     return names;
@@ -592,14 +587,14 @@ bool Font::isUnderlined() const noexcept    { return font->underline; }
 
 void Font::setBold (const bool shouldBeBold)
 {
-    const int flags = getStyleFlags();
+    auto flags = getStyleFlags();
     setStyleFlags (shouldBeBold ? (flags | bold)
                                 : (flags & ~bold));
 }
 
 void Font::setItalic (const bool shouldBeItalic)
 {
-    const int flags = getStyleFlags();
+    auto flags = getStyleFlags();
     setStyleFlags (shouldBeItalic ? (flags | italic)
                                   : (flags & ~italic));
 }
@@ -613,7 +608,7 @@ void Font::setUnderline (const bool shouldBeUnderlined)
 
 float Font::getAscent() const
 {
-    if (font->ascent == 0)
+    if (font->ascent == 0.0f)
         font->ascent = getTypeface()->getAscent();
 
     return font->height * font->ascent;
@@ -633,9 +628,13 @@ int Font::getStringWidth (const String& text) const
 
 float Font::getStringWidthFloat (const String& text) const
 {
-    float w = getTypeface()->getStringWidth (text);
+    // This call isn't thread-safe when there's a message thread running
+    jassert (MessageManager::getInstanceWithoutCreating() == nullptr
+               || MessageManager::getInstanceWithoutCreating()->currentThreadHasLockedMessageManager());
 
-    if (font->kerning != 0)
+    auto w = getTypeface()->getStringWidth (text);
+
+    if (font->kerning != 0.0f)
         w += font->kerning * text.length();
 
     return w * font->height * font->horizontalScale;
@@ -643,16 +642,18 @@ float Font::getStringWidthFloat (const String& text) const
 
 void Font::getGlyphPositions (const String& text, Array<int>& glyphs, Array<float>& xOffsets) const
 {
+    // This call isn't thread-safe when there's a message thread running
+    jassert (MessageManager::getInstanceWithoutCreating() == nullptr
+               || MessageManager::getInstanceWithoutCreating()->currentThreadHasLockedMessageManager());
+
     getTypeface()->getGlyphPositions (text, glyphs, xOffsets);
 
-    const int num = xOffsets.size();
-
-    if (num > 0)
+    if (auto num = xOffsets.size())
     {
-        const float scale = font->height * font->horizontalScale;
-        float* const x = xOffsets.getRawDataPointer();
+        auto scale = font->height * font->horizontalScale;
+        auto* x = xOffsets.getRawDataPointer();
 
-        if (font->kerning != 0)
+        if (font->kerning != 0.0f)
         {
             for (int i = 0; i < num; ++i)
                 x[i] = (x[i] + i * font->kerning) * scale;
@@ -667,18 +668,16 @@ void Font::getGlyphPositions (const String& text, Array<int>& glyphs, Array<floa
 
 void Font::findFonts (Array<Font>& destArray)
 {
-    const StringArray names (findAllTypefaceNames());
-
-    for (int i = 0; i < names.size(); ++i)
+    for (auto& name : findAllTypefaceNames())
     {
-        const StringArray styles (findAllTypefaceStyles (names[i]));
+        auto styles = findAllTypefaceStyles (name);
 
         String style ("Regular");
 
         if (! styles.contains (style, true))
             style = styles[0];
 
-        destArray.add (Font (names[i],  style, FontValues::defaultFontHeight));
+        destArray.add (Font (name, style, FontValues::defaultFontHeight));
     }
 }
 
@@ -719,3 +718,5 @@ Font Font::fromString (const String& fontDescription)
 
     return Font (name, style, height);
 }
+
+} // namespace juce
