@@ -2,27 +2,32 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-TextLayout::Glyph::Glyph (const int glyph, Point<float> anch, float w) noexcept
+namespace juce
+{
+
+TextLayout::Glyph::Glyph (int glyph, Point<float> anch, float w) noexcept
     : glyphCode (glyph), anchor (anch), width (w)
 {
 }
@@ -48,7 +53,7 @@ TextLayout::Run::Run() noexcept
 {
 }
 
-TextLayout::Run::Run (Range<int> range, const int numGlyphsToPreallocate)
+TextLayout::Run::Run (Range<int> range, int numGlyphsToPreallocate)
     : colour (0xff000000), stringRange (range)
 {
     glyphs.ensureStorageAllocated (numGlyphsToPreallocate);
@@ -64,6 +69,29 @@ TextLayout::Run::Run (const Run& other)
 
 TextLayout::Run::~Run() noexcept {}
 
+Range<float> TextLayout::Run::getRunBoundsX() const noexcept
+{
+    Range<float> range;
+    bool isFirst = true;
+
+    for (auto& glyph : glyphs)
+    {
+        Range<float> r (glyph.anchor.x, glyph.anchor.x + glyph.width);
+
+        if (isFirst)
+        {
+            isFirst = false;
+            range = r;
+        }
+        else
+        {
+            range = range.getUnionWith (r);
+        }
+    }
+
+    return range;
+}
+
 //==============================================================================
 TextLayout::Line::Line() noexcept
     : ascent (0.0f), descent (0.0f), leading (0.0f)
@@ -94,32 +122,18 @@ Range<float> TextLayout::Line::getLineBoundsX() const noexcept
     Range<float> range;
     bool isFirst = true;
 
-    for (int i = runs.size(); --i >= 0;)
+    for (auto* run : runs)
     {
-        const Run& run = *runs.getUnchecked(i);
+        auto runRange = run->getRunBoundsX();
 
-        if (run.glyphs.size() > 0)
+        if (isFirst)
         {
-            float minX = run.glyphs.getReference(0).anchor.x;
-            float maxX = minX;
-
-            for (int j = run.glyphs.size(); --j >= 0;)
-            {
-                const Glyph& glyph = run.glyphs.getReference (j);
-                const float x = glyph.anchor.x;
-                minX = jmin (minX, x);
-                maxX = jmax (maxX, x + glyph.width);
-            }
-
-            if (isFirst)
-            {
-                isFirst = false;
-                range = Range<float> (minX, maxX);
-            }
-            else
-            {
-                range = range.getUnionWith (Range<float> (minX, maxX));
-            }
+            isFirst = false;
+            range = runRange;
+        }
+        else
+        {
+            range = range.getUnionWith (runRange);
         }
     }
 
@@ -128,16 +142,16 @@ Range<float> TextLayout::Line::getLineBoundsX() const noexcept
 
 Range<float> TextLayout::Line::getLineBoundsY() const noexcept
 {
-    return Range<float> (lineOrigin.y - ascent,
-                         lineOrigin.y + descent);
+    return { lineOrigin.y - ascent,
+             lineOrigin.y + descent };
 }
 
 Rectangle<float> TextLayout::Line::getLineBounds() const noexcept
 {
-    const Range<float> x (getLineBoundsX()),
-                       y (getLineBoundsY());
+    auto x = getLineBoundsX();
+    auto y = getLineBoundsY();
 
-    return Rectangle<float> (x.getStart(), y.getStart(), x.getLength(), y.getLength());
+    return { x.getStart(), y.getStart(), x.getLength(), y.getLength() };
 }
 
 //==============================================================================
@@ -153,7 +167,6 @@ TextLayout::TextLayout (const TextLayout& other)
     lines.addCopiesOf (other.lines);
 }
 
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
 TextLayout::TextLayout (TextLayout&& other) noexcept
     : lines (static_cast<OwnedArray<Line>&&> (other.lines)),
       width (other.width), height (other.height),
@@ -169,7 +182,6 @@ TextLayout& TextLayout::operator= (TextLayout&& other) noexcept
     justification = other.justification;
     return *this;
 }
-#endif
 
 TextLayout& TextLayout::operator= (const TextLayout& other)
 {
@@ -185,7 +197,7 @@ TextLayout::~TextLayout()
 {
 }
 
-TextLayout::Line& TextLayout::getLine (const int index) const
+TextLayout::Line& TextLayout::getLine (int index) const noexcept
 {
     return *lines.getUnchecked (index);
 }
@@ -200,28 +212,43 @@ void TextLayout::addLine (Line* line)
     lines.add (line);
 }
 
-void TextLayout::draw (Graphics& g, const Rectangle<float>& area) const
+void TextLayout::draw (Graphics& g, Rectangle<float> area) const
 {
-    const Point<float> origin (justification.appliedToRectangle (Rectangle<float> (width, getHeight()), area).getPosition());
+    auto origin = justification.appliedToRectangle (Rectangle<float> (width, getHeight()), area).getPosition();
 
-    LowLevelGraphicsContext& context = g.getInternalContext();
+    auto& context   = g.getInternalContext();
+    auto clip       = context.getClipBounds();
+    auto clipTop    = clip.getY()      - origin.y;
+    auto clipBottom = clip.getBottom() - origin.y;
 
-    for (int i = 0; i < lines.size(); ++i)
+    for (auto* line : lines)
     {
-        const Line& line = getLine (i);
-        const Point<float> lineOrigin (origin + line.lineOrigin);
+        auto lineRangeY = line->getLineBoundsY();
 
-        for (int j = 0; j < line.runs.size(); ++j)
+        if (lineRangeY.getEnd() < clipTop)
+            continue;
+
+        if (lineRangeY.getStart() > clipBottom)
+            break;
+
+        auto lineOrigin = origin + line->lineOrigin;
+
+        for (auto* run : line->runs)
         {
-            const Run& run = *line.runs.getUnchecked (j);
-            context.setFont (run.font);
-            context.setFill (run.colour);
+            context.setFont (run->font);
+            context.setFill (run->colour);
 
-            for (int k = 0; k < run.glyphs.size(); ++k)
-            {
-                const Glyph& glyph = run.glyphs.getReference (k);
+            for (auto& glyph : run->glyphs)
                 context.drawGlyph (glyph.glyphCode, AffineTransform::translation (lineOrigin.x + glyph.anchor.x,
                                                                                   lineOrigin.y + glyph.anchor.y));
+
+            if (run->font.isUnderlined())
+            {
+                auto runExtent = run->getRunBoundsX();
+                auto lineThickness = run->font.getDescent() * 0.3f;
+
+                context.fillRect ({ runExtent.getStart() + lineOrigin.x, lineOrigin.y + lineThickness * 2.0f,
+                                    runExtent.getLength(), lineThickness });
             }
         }
     }
@@ -252,8 +279,8 @@ void TextLayout::createLayoutWithBalancedLineLengths (const AttributedString& te
 
 void TextLayout::createLayoutWithBalancedLineLengths (const AttributedString& text, float maxWidth, float maxHeight)
 {
-    const float minimumWidth = maxWidth / 2.0f;
-    float bestWidth = maxWidth;
+    auto minimumWidth = maxWidth / 2.0f;
+    auto bestWidth = maxWidth;
     float bestLineProportion = 0.0f;
 
     while (maxWidth > minimumWidth)
@@ -263,12 +290,13 @@ void TextLayout::createLayoutWithBalancedLineLengths (const AttributedString& te
         if (getNumLines() < 2)
             return;
 
-        const float line1 = lines.getUnchecked (lines.size() - 1)->getLineBoundsX().getLength();
-        const float line2 = lines.getUnchecked (lines.size() - 2)->getLineBoundsX().getLength();
-        const float shortestLine = jmin (line1, line2);
-        const float prop = (shortestLine > 0) ? jmax (line1, line2) / shortestLine : 1.0f;
+        auto line1 = lines.getUnchecked (lines.size() - 1)->getLineBoundsX().getLength();
+        auto line2 = lines.getUnchecked (lines.size() - 2)->getLineBoundsX().getLength();
+        auto shortest = jmin (line1, line2);
+        auto longest  = jmax (line1, line2);
+        auto prop = shortest > 0 ? longest / shortest : 1.0f;
 
-        if (prop > 0.9f)
+        if (prop > 0.9f && prop < 1.1f)
             return;
 
         if (prop > bestLineProportion)
@@ -289,7 +317,7 @@ namespace TextLayoutHelpers
 {
     struct Token
     {
-        Token (const String& t, const Font& f, Colour c, const bool whitespace)
+        Token (const String& t, const Font& f, Colour c, bool whitespace)
             : text (t), font (f), colour (c),
               area (font.getStringWidthFloat (t), f.getHeight()),
               isWhitespace (whitespace),
@@ -304,20 +332,19 @@ namespace TextLayoutHelpers
         float lineHeight;
         const bool isWhitespace, isNewLine;
 
-    private:
-        Token& operator= (const Token&);
+        Token& operator= (const Token&) = delete;
     };
 
     struct TokenList
     {
-        TokenList() noexcept  : totalLines (0) {}
+        TokenList() noexcept {}
 
         void createLayout (const AttributedString& text, TextLayout& layout)
         {
             layout.ensureStorageAllocated (totalLines);
 
             addTextRuns (text);
-            layoutRuns (layout.getWidth(), text.getLineSpacing());
+            layoutRuns (layout.getWidth(), text.getLineSpacing(), text.getWordWrap());
 
             int charPosition = 0;
             int lineStartPosition = 0;
@@ -330,19 +357,19 @@ namespace TextLayoutHelpers
 
             for (int i = 0; i < tokens.size(); ++i)
             {
-                const Token& t = *tokens.getUnchecked (i);
+                auto& t = *tokens.getUnchecked (i);
 
                 Array<int> newGlyphs;
                 Array<float> xOffsets;
                 t.font.getGlyphPositions (getTrimmedEndIfNotAllWhitespace (t.text), newGlyphs, xOffsets);
 
-                if (currentRun == nullptr)  currentRun  = new TextLayout::Run();
-                if (currentLine == nullptr) currentLine = new TextLayout::Line();
+                if (currentRun == nullptr)  currentRun .reset (new TextLayout::Run());
+                if (currentLine == nullptr) currentLine.reset (new TextLayout::Line());
 
                 if (newGlyphs.size() > 0)
                 {
                     currentRun->glyphs.ensureStorageAllocated (currentRun->glyphs.size() + newGlyphs.size());
-                    const Point<float> tokenOrigin (t.area.getPosition().translated (0, t.font.getAscent()));
+                    auto tokenOrigin = t.area.getPosition().translated (0, t.font.getAscent());
 
                     if (needToSetLineOrigin)
                     {
@@ -350,11 +377,11 @@ namespace TextLayoutHelpers
                         currentLine->lineOrigin = tokenOrigin;
                     }
 
-                    const Point<float> glyphOffset (tokenOrigin - currentLine->lineOrigin);
+                    auto glyphOffset = tokenOrigin - currentLine->lineOrigin;
 
                     for (int j = 0; j < newGlyphs.size(); ++j)
                     {
-                        const float x = xOffsets.getUnchecked (j);
+                        auto x = xOffsets.getUnchecked (j);
                         currentRun->glyphs.add (TextLayout::Glyph (newGlyphs.getUnchecked(j),
                                                                    glyphOffset.translated (x, 0),
                                                                    xOffsets.getUnchecked (j + 1) - x));
@@ -366,19 +393,7 @@ namespace TextLayoutHelpers
                 if (t.isWhitespace || t.isNewLine)
                     ++charPosition;
 
-                const Token* const nextToken = tokens [i + 1];
-
-                if (nextToken == nullptr) // this is the last token
-                {
-                    addRun (*currentLine, currentRun.release(), t, runStartPosition, charPosition);
-                    currentLine->stringRange = Range<int> (lineStartPosition, charPosition);
-
-                    if (! needToSetLineOrigin)
-                        layout.addLine (currentLine.release());
-
-                    needToSetLineOrigin = true;
-                }
-                else
+                if (auto* nextToken = tokens[i + 1])
                 {
                     if (t.font != nextToken->font || t.colour != nextToken->colour)
                     {
@@ -389,10 +404,10 @@ namespace TextLayoutHelpers
                     if (t.line != nextToken->line)
                     {
                         if (currentRun == nullptr)
-                            currentRun = new TextLayout::Run();
+                            currentRun.reset (new TextLayout::Run());
 
                         addRun (*currentLine, currentRun.release(), t, runStartPosition, charPosition);
-                        currentLine->stringRange = Range<int> (lineStartPosition, charPosition);
+                        currentLine->stringRange = { lineStartPosition, charPosition };
 
                         if (! needToSetLineOrigin)
                             layout.addLine (currentLine.release());
@@ -402,16 +417,26 @@ namespace TextLayoutHelpers
                         needToSetLineOrigin = true;
                     }
                 }
+                else
+                {
+                    addRun (*currentLine, currentRun.release(), t, runStartPosition, charPosition);
+                    currentLine->stringRange = { lineStartPosition, charPosition };
+
+                    if (! needToSetLineOrigin)
+                        layout.addLine (currentLine.release());
+
+                    needToSetLineOrigin = true;
+                }
             }
 
             if ((text.getJustification().getFlags() & (Justification::right | Justification::horizontallyCentred)) != 0)
             {
-                const float totalW = layout.getWidth();
-                const bool isCentred = (text.getJustification().getFlags() & Justification::horizontallyCentred) != 0;
+                auto totalW = layout.getWidth();
+                bool isCentred = (text.getJustification().getFlags() & Justification::horizontallyCentred) != 0;
 
                 for (int i = 0; i < layout.getNumLines(); ++i)
                 {
-                    float dx = totalW - layout.getLine(i).getLineBoundsX().getLength();
+                    auto dx = totalW - layout.getLine(i).getLineBoundsX().getLength();
 
                     if (isCentred)
                         dx /= 2.0f;
@@ -423,9 +448,9 @@ namespace TextLayoutHelpers
 
     private:
         static void addRun (TextLayout::Line& glyphLine, TextLayout::Run* glyphRun,
-                            const Token& t, const int start, const int end)
+                            const Token& t, int start, int end)
         {
-            glyphRun->stringRange = Range<int> (start, end);
+            glyphRun->stringRange = { start, end };
             glyphRun->font = t.font;
             glyphRun->colour = t.colour;
             glyphLine.ascent  = jmax (glyphLine.ascent,  t.font.getAscent());
@@ -433,7 +458,7 @@ namespace TextLayoutHelpers
             glyphLine.runs.add (glyphRun);
         }
 
-        static int getCharacterType (const juce_wchar c) noexcept
+        static int getCharacterType (juce_wchar c) noexcept
         {
             if (c == '\r' || c == '\n')
                 return 0;
@@ -443,17 +468,18 @@ namespace TextLayoutHelpers
 
         void appendText (const String& stringText, const Font& font, Colour colour)
         {
-            String::CharPointerType t (stringText.getCharPointer());
+            auto t = stringText.getCharPointer();
             String currentString;
             int lastCharType = 0;
 
             for (;;)
             {
-                const juce_wchar c = t.getAndAdvance();
+                auto c = t.getAndAdvance();
+
                 if (c == 0)
                     break;
 
-                const int charType = getCharacterType (c);
+                auto charType = getCharacterType (c);
 
                 if (charType == 0 || charType != lastCharType)
                 {
@@ -478,25 +504,27 @@ namespace TextLayoutHelpers
                 tokens.add (new Token (currentString, font, colour, lastCharType == 2));
         }
 
-        void layoutRuns (const float maxWidth, const float extraLineSpacing)
+        void layoutRuns (float maxWidth, float extraLineSpacing, AttributedString::WordWrap wordWrap)
         {
             float x = 0, y = 0, h = 0;
             int i;
 
             for (i = 0; i < tokens.size(); ++i)
             {
-                Token& t = *tokens.getUnchecked(i);
+                auto& t = *tokens.getUnchecked(i);
                 t.area.setPosition (x, y);
                 t.line = totalLines;
                 x += t.area.getWidth();
                 h = jmax (h, t.area.getHeight() + extraLineSpacing);
 
-                const Token* const nextTok = tokens[i + 1];
+                auto* nextTok = tokens[i + 1];
 
                 if (nextTok == nullptr)
                     break;
 
-                if (t.isNewLine || ((! nextTok->isWhitespace) && x + nextTok->area.getWidth() > maxWidth))
+                bool tokenTooLarge = (x + nextTok->area.getWidth() > maxWidth);
+
+                if (t.isNewLine || ((! nextTok->isWhitespace) && (tokenTooLarge && wordWrap != AttributedString::none)))
                 {
                     setLastLineHeight (i + 1, h);
                     x = 0;
@@ -510,11 +538,11 @@ namespace TextLayoutHelpers
             ++totalLines;
         }
 
-        void setLastLineHeight (int i, const float height) noexcept
+        void setLastLineHeight (int i, float height) noexcept
         {
             while (--i >= 0)
             {
-                Token& tok = *tokens.getUnchecked (i);
+                auto& tok = *tokens.getUnchecked (i);
 
                 if (tok.line == totalLines)
                     tok.lineHeight = height;
@@ -525,12 +553,12 @@ namespace TextLayoutHelpers
 
         void addTextRuns (const AttributedString& text)
         {
-            const int numAttributes = text.getNumAttributes();
+            auto numAttributes = text.getNumAttributes();
             tokens.ensureStorageAllocated (jmax (64, numAttributes));
 
             for (int i = 0; i < numAttributes; ++i)
             {
-                const AttributedString::Attribute& attr = text.getAttribute (i);
+                auto& attr = text.getAttribute (i);
 
                 appendText (text.getText().substring (attr.range.getStart(), attr.range.getEnd()),
                             attr.font, attr.colour);
@@ -539,7 +567,8 @@ namespace TextLayoutHelpers
 
         static String getTrimmedEndIfNotAllWhitespace (const String& s)
         {
-            String trimmed (s.trimEnd());
+            auto trimmed = s.trimEnd();
+
             if (trimmed.isEmpty() && s.isNotEmpty())
                 trimmed = s.replaceCharacters ("\r\n\t", "   ");
 
@@ -547,7 +576,7 @@ namespace TextLayoutHelpers
         }
 
         OwnedArray<Token> tokens;
-        int totalLines;
+        int totalLines = 0;
 
         JUCE_DECLARE_NON_COPYABLE (TokenList)
     };
@@ -562,15 +591,15 @@ void TextLayout::createStandardLayout (const AttributedString& text)
 
 void TextLayout::recalculateSize()
 {
-    if (lines.size() > 0)
+    if (! lines.isEmpty())
     {
-        Rectangle<float> bounds (lines.getFirst()->getLineBounds());
+        auto bounds = lines.getFirst()->getLineBounds();
 
-        for (int i = lines.size(); --i > 0;)
-            bounds = bounds.getUnion (lines.getUnchecked(i)->getLineBounds());
+        for (auto* line : lines)
+            bounds = bounds.getUnion (line->getLineBounds());
 
-        for (int i = lines.size(); --i >= 0;)
-            lines.getUnchecked(i)->lineOrigin.x -= bounds.getX();
+        for (auto* line : lines)
+            line->lineOrigin.x -= bounds.getX();
 
         width  = bounds.getWidth();
         height = bounds.getHeight();
@@ -581,3 +610,5 @@ void TextLayout::recalculateSize()
         height = 0;
     }
 }
+
+} // namespace juce
