@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_TEXTLAYOUT_H_INCLUDED
-#define JUCE_TEXTLAYOUT_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -35,7 +36,7 @@
 
     @see AttributedString
 */
-class JUCE_API  TextLayout
+class JUCE_API  TextLayout  final
 {
 public:
     /** Creates an empty layout.
@@ -45,10 +46,8 @@ public:
     TextLayout();
     TextLayout (const TextLayout&);
     TextLayout& operator= (const TextLayout&);
-   #if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
     TextLayout (TextLayout&&) noexcept;
     TextLayout& operator= (TextLayout&&) noexcept;
-   #endif
 
     /** Destructor. */
     ~TextLayout();
@@ -84,7 +83,7 @@ public:
         The position of the text within the rectangle is controlled by the justification
         flags set in the original AttributedString that was used to create this layout.
     */
-    void draw (Graphics&, const Rectangle<float>& area) const;
+    void draw (Graphics&, Rectangle<float> area) const;
 
     //==============================================================================
     /** A positioned glyph. */
@@ -120,6 +119,9 @@ public:
         Run (Range<int> stringRange, int numGlyphsToPreallocate);
         ~Run() noexcept;
 
+        /** Returns the X position range which contains all the glyphs in this run. */
+        Range<float> getRunBoundsX() const noexcept;
+
         Font font;              /**< The run's font. */
         Colour colour;          /**< The run's colour. */
         Array<Glyph> glyphs;    /**< The glyphs in this run. */
@@ -172,7 +174,7 @@ public:
     int getNumLines() const noexcept    { return lines.size(); }
 
     /** Returns one of the lines. */
-    Line& getLine (int index) const;
+    Line& getLine (int index) const noexcept;
 
     /** Adds a line to the layout. The layout will take ownership of this line object
         and will delete it when it is no longer needed. */
@@ -193,4 +195,4 @@ private:
     JUCE_LEAK_DETECTOR (TextLayout)
 };
 
-#endif   // JUCE_TEXTLAYOUT_H_INCLUDED
+} // namespace juce
