@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_PATH_H_INCLUDED
-#define JUCE_PATH_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -59,7 +60,7 @@
 
     @see PathFlatteningIterator, PathStrokeType, Graphics
 */
-class JUCE_API  Path
+class JUCE_API  Path  final
 {
 public:
     //==============================================================================
@@ -75,10 +76,11 @@ public:
     /** Copies this path from another one. */
     Path& operator= (const Path&);
 
-   #if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
+    /** Move constructor */
     Path (Path&&) noexcept;
+
+    /** Move assignment operator */
     Path& operator= (Path&&) noexcept;
-   #endif
 
     bool operator== (const Path&) const noexcept;
     bool operator!= (const Path&) const noexcept;
@@ -127,7 +129,7 @@ public:
 
         @see closeSubPath, setUsingNonZeroWinding
     */
-    bool contains (const Point<float> point,
+    bool contains (Point<float> point,
                    float tolerance = defaultToleranceForTesting) const;
 
     /** Checks whether a line crosses the path.
@@ -209,7 +211,7 @@ public:
 
         @see lineTo, quadraticTo, cubicTo, closeSubPath
     */
-    void startNewSubPath (const Point<float> start);
+    void startNewSubPath (Point<float> start);
 
     /** Closes a the current sub-path with a line back to its start-point.
 
@@ -245,7 +247,7 @@ public:
 
         @see startNewSubPath, quadraticTo, cubicTo, closeSubPath
     */
-    void lineTo (const Point<float> end);
+    void lineTo (Point<float> end);
 
     /** Adds a quadratic bezier curve from the shape's last position to a new position.
 
@@ -270,8 +272,8 @@ public:
 
         @see startNewSubPath, lineTo, cubicTo, closeSubPath
     */
-    void quadraticTo (const Point<float> controlPoint,
-                      const Point<float> endPoint);
+    void quadraticTo (Point<float> controlPoint,
+                      Point<float> endPoint);
 
     /** Adds a cubic bezier curve from the shape's last position to a new position.
 
@@ -298,9 +300,9 @@ public:
 
         @see startNewSubPath, lineTo, quadraticTo, closeSubPath
     */
-    void cubicTo (const Point<float> controlPoint1,
-                  const Point<float> controlPoint2,
-                  const Point<float> endPoint);
+    void cubicTo (Point<float> controlPoint1,
+                  Point<float> controlPoint2,
+                  Point<float> endPoint);
 
     /** Returns the last point that was added to the path by one of the drawing methods.
     */
@@ -318,7 +320,7 @@ public:
         @see addRoundedRectangle, addTriangle
     */
     template <typename ValueType>
-    void addRectangle (const Rectangle<ValueType>& rectangle)
+    void addRectangle (Rectangle<ValueType> rectangle)
     {
         addRectangle (static_cast<float> (rectangle.getX()), static_cast<float> (rectangle.getY()),
                       static_cast<float> (rectangle.getWidth()), static_cast<float> (rectangle.getHeight()));
@@ -353,7 +355,7 @@ public:
         @see addRectangle, addTriangle
     */
     template <typename ValueType>
-    void addRoundedRectangle (const Rectangle<ValueType>& rectangle, float cornerSizeX, float cornerSizeY)
+    void addRoundedRectangle (Rectangle<ValueType> rectangle, float cornerSizeX, float cornerSizeY)
     {
         addRoundedRectangle (static_cast<float> (rectangle.getX()), static_cast<float> (rectangle.getY()),
                              static_cast<float> (rectangle.getWidth()), static_cast<float> (rectangle.getHeight()),
@@ -365,7 +367,7 @@ public:
         @see addRectangle, addTriangle
     */
     template <typename ValueType>
-    void addRoundedRectangle (const Rectangle<ValueType>& rectangle, float cornerSize)
+    void addRoundedRectangle (Rectangle<ValueType> rectangle, float cornerSize)
     {
         addRoundedRectangle (rectangle, cornerSize, cornerSize);
     }
@@ -532,13 +534,13 @@ public:
 
         @see addArrow
     */
-    void addLineSegment (const Line<float>& line, float lineThickness);
+    void addLineSegment (Line<float> line, float lineThickness);
 
     /** Adds a line with an arrowhead on the end.
         The arrow is added as a new closed sub-path. (Any currently open paths will be left open).
         @see PathStrokeType::createStrokeWithArrowheads
     */
-    void addArrow (const Line<float>& line,
+    void addArrow (Line<float> line,
                    float lineThickness,
                    float arrowheadWidth,
                    float arrowheadLength);
@@ -546,7 +548,7 @@ public:
     /** Adds a polygon shape to the path.
         @see addStar
     */
-    void addPolygon (const Point<float> centre,
+    void addPolygon (Point<float> centre,
                      int numberOfSides,
                      float radius,
                      float startAngle = 0.0f);
@@ -554,7 +556,7 @@ public:
     /** Adds a star shape to the path.
         @see addPolygon
     */
-    void addStar (const Point<float> centre,
+    void addStar (Point<float> centre,
                   int numberOfPoints,
                   float innerRadius,
                   float outerRadius,
@@ -570,8 +572,8 @@ public:
         @param cornerSize       the size of the rounded corners
         @param arrowBaseWidth   the width of the base of the arrow where it joins the main rectangle
     */
-    void addBubble (const Rectangle<float>& bodyArea,
-                    const Rectangle<float>& maximumArea,
+    void addBubble (Rectangle<float> bodyArea,
+                    Rectangle<float> maximumArea,
                     const Point<float> arrowTipPosition,
                     const float cornerSize,
                     const float arrowBaseWidth);
@@ -675,7 +677,7 @@ public:
         @see applyTransform, scaleToFit
 
     */
-    AffineTransform getTransformToScaleToFit (const Rectangle<float>& area,
+    AffineTransform getTransformToScaleToFit (Rectangle<float> area,
                                               bool preserveProportions,
                                               Justification justificationType = Justification::centred) const;
 
@@ -746,12 +748,12 @@ public:
 
         PathElementType elementType;
 
-        float x1, y1, x2, y2, x3, y3;
+        float x1 = 0, y1 = 0, x2 = 0, y2 = 0, x3 = 0, y3 = 0;
 
         //==============================================================================
     private:
         const Path& path;
-        size_t index;
+        const float* index;
 
         JUCE_DECLARE_NON_COPYABLE (Iterator)
     };
@@ -779,9 +781,7 @@ public:
     void loadPathFromData (const void* data, size_t numberOfBytes);
 
     /** Stores the path by writing it out to a stream.
-
         After writing out a path, you can reload it using loadPathFromStream().
-
         @see loadPathFromStream, loadPathFromData
     */
     void writePathToStream (OutputStream& destination) const;
@@ -801,8 +801,9 @@ private:
     //==============================================================================
     friend class PathFlatteningIterator;
     friend class Path::Iterator;
-    ArrayAllocationBase<float, DummyCriticalSection> data;
-    size_t numElements;
+    friend class EdgeTable;
+
+    Array<float> data;
 
     struct PathBounds
     {
@@ -811,13 +812,19 @@ private:
         void reset() noexcept;
         void reset (float, float) noexcept;
         void extend (float, float) noexcept;
-        void extend (float, float, float, float) noexcept;
 
-        float pathXMin, pathXMax, pathYMin, pathYMax;
+        template <typename... Coords>
+        void extend (float x, float y, Coords... coords) noexcept
+        {
+            extend (x, y);
+            extend (coords...);
+        }
+
+        float pathXMin = 0, pathXMax = 0, pathYMin = 0, pathYMax = 0;
     };
 
     PathBounds bounds;
-    bool useNonZeroWinding;
+    bool useNonZeroWinding = true;
 
     static const float lineMarker;
     static const float moveMarker;
@@ -828,4 +835,4 @@ private:
     JUCE_LEAK_DETECTOR (Path)
 };
 
-#endif   // JUCE_PATH_H_INCLUDED
+} // namespace juce
