@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_RECTANGLELIST_H_INCLUDED
-#define JUCE_RECTANGLELIST_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -37,7 +38,7 @@
     @see Rectangle
 */
 template <typename ValueType>
-class RectangleList
+class RectangleList  final
 {
 public:
     typedef Rectangle<ValueType> RectangleType;
@@ -52,7 +53,7 @@ public:
     }
 
     /** Creates a list containing just one rectangle. */
-    RectangleList (const RectangleType& rect)
+    RectangleList (RectangleType rect)
     {
         addWithoutMerging (rect);
     }
@@ -64,22 +65,22 @@ public:
         return *this;
     }
 
-   #if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
+    /** Move constructor */
     RectangleList (RectangleList&& other) noexcept
         : rects (static_cast<Array<RectangleType>&&> (other.rects))
     {
     }
 
+    /** Move assignment operator */
     RectangleList& operator= (RectangleList&& other) noexcept
     {
         rects = static_cast<Array<RectangleType>&&> (other.rects);
         return *this;
     }
-   #endif
 
     //==============================================================================
     /** Returns true if the region is empty. */
-    bool isEmpty() const noexcept                               { return rects.size() == 0; }
+    bool isEmpty() const noexcept                               { return rects.isEmpty(); }
 
     /** Returns the number of rectangles in the list. */
     int getNumRectangles() const noexcept                       { return rects.size(); }
@@ -105,13 +106,13 @@ public:
         The rectangle can have any size and may be empty, but if it's floating point
         then it's expected to not contain any INF values.
     */
-    void add (const RectangleType& rect)
+    void add (RectangleType rect)
     {
         jassert (rect.isFinite()); // You must provide a valid rectangle to this method!
 
         if (! rect.isEmpty())
         {
-            if (rects.size() == 0)
+            if (isEmpty())
             {
                 rects.add (rect);
             }
@@ -121,7 +122,7 @@ public:
 
                 for (int j = rects.size(); --j >= 0;)
                 {
-                    RectangleType& ourRect = rects.getReference (j);
+                    auto& ourRect = rects.getReference (j);
 
                     if (rect.intersects (ourRect))
                     {
@@ -132,19 +133,17 @@ public:
                     }
                 }
 
-                if (anyOverlaps && rects.size() > 0)
+                if (anyOverlaps && ! isEmpty())
                 {
                     RectangleList r (rect);
 
-                    for (int i = rects.size(); --i >= 0;)
+                    for (auto& ourRect : rects)
                     {
-                        const RectangleType& ourRect = rects.getReference (i);
-
                         if (rect.intersects (ourRect))
                         {
                             r.subtract (ourRect);
 
-                            if (r.rects.size() == 0)
+                            if (r.isEmpty())
                                 return;
                         }
                     }
@@ -177,7 +176,7 @@ public:
         The rectangle can have any size and may be empty, but if it's floating point
         then it's expected to not contain any INF values.
     */
-    void addWithoutMerging (const RectangleType& rect)
+    void addWithoutMerging (RectangleType rect)
     {
         jassert (rect.isFinite()); // You must provide a valid rectangle to this method!
 
@@ -192,7 +191,7 @@ public:
     */
     void add (const RectangleList& other)
     {
-        for (const RectangleType* r = other.begin(), * const e = other.end(); r != e; ++r)
+        for (auto& r : other)
             add (*r);
     }
 
@@ -201,25 +200,23 @@ public:
         Any rectangles in the list which overlap this will be clipped and subdivided
         if necessary.
     */
-    void subtract (const RectangleType& rect)
+    void subtract (RectangleType rect)
     {
-        const int originalNumRects = rects.size();
-
-        if (originalNumRects > 0)
+        if (auto numRects = rects.size())
         {
-            const ValueType x1 = rect.getX();
-            const ValueType y1 = rect.getY();
-            const ValueType x2 = x1 + rect.getWidth();
-            const ValueType y2 = y1 + rect.getHeight();
+            auto x1 = rect.getX();
+            auto y1 = rect.getY();
+            auto x2 = x1 + rect.getWidth();
+            auto y2 = y1 + rect.getHeight();
 
-            for (int i = getNumRectangles(); --i >= 0;)
+            for (int i = numRects; --i >= 0;)
             {
-                RectangleType& r = rects.getReference (i);
+                auto& r = rects.getReference (i);
 
-                const ValueType rx1 = r.getX();
-                const ValueType ry1 = r.getY();
-                const ValueType rx2 = rx1 + r.getWidth();
-                const ValueType ry2 = ry1 + r.getHeight();
+                auto rx1 = r.getX();
+                auto ry1 = r.getY();
+                auto rx2 = rx1 + r.getWidth();
+                auto ry2 = ry1 + r.getHeight();
 
                 if (! (x2 <= rx1 || x1 >= rx2 || y2 <= ry1 || y1 >= ry2))
                 {
@@ -293,10 +290,15 @@ public:
     */
     bool subtract (const RectangleList& otherList)
     {
-        for (int i = otherList.rects.size(); --i >= 0 && rects.size() > 0;)
-            subtract (otherList.rects.getReference (i));
+        for (auto& r : otherList)
+        {
+            if (isEmpty())
+                return false;
 
-        return rects.size() > 0;
+            subtract (r);
+        }
+
+        return ! isEmpty();
     }
 
     /** Removes any areas of the region that lie outside a given rectangle.
@@ -308,7 +310,7 @@ public:
 
         @see getIntersectionWith
     */
-    bool clipTo (const RectangleType& rect)
+    bool clipTo (RectangleType rect)
     {
         jassert (rect.isFinite()); // You must provide a valid rectangle to this method!
 
@@ -322,7 +324,7 @@ public:
         {
             for (int i = rects.size(); --i >= 0;)
             {
-                RectangleType& r = rects.getReference (i);
+                auto& r = rects.getReference (i);
 
                 if (! rect.intersectRectangle (r))
                     rects.remove (i);
@@ -346,18 +348,16 @@ public:
     template <typename OtherValueType>
     bool clipTo (const RectangleList<OtherValueType>& other)
     {
-        if (rects.size() == 0)
+        if (isEmpty())
             return false;
 
         RectangleList result;
 
-        for (int j = 0; j < rects.size(); ++j)
+        for (auto& rect : rects)
         {
-            const RectangleType& rect = rects.getReference (j);
-
-            for (const Rectangle<OtherValueType>* r = other.begin(), * const e = other.end(); r != e; ++r)
+            for (auto& r : other)
             {
-                RectangleType clipped (r->template toType<ValueType>());
+                auto clipped = r.template toType<ValueType>();
 
                 if (rect.intersectRectangle (clipped))
                     result.rects.add (clipped);
@@ -377,24 +377,18 @@ public:
 
         @see clipTo
     */
-    bool getIntersectionWith (const RectangleType& rect, RectangleList& destRegion) const
+    bool getIntersectionWith (RectangleType rect, RectangleList& destRegion) const
     {
         jassert (rect.isFinite()); // You must provide a valid rectangle to this method!
 
         destRegion.clear();
 
         if (! rect.isEmpty())
-        {
-            for (int i = rects.size(); --i >= 0;)
-            {
-                RectangleType r (rects.getReference (i));
-
+            for (auto& r : rects)
                 if (rect.intersectRectangle (r))
                     destRegion.rects.add (r);
-            }
-        }
 
-        return destRegion.rects.size() > 0;
+        return ! destRegion.isEmpty();
     }
 
     /** Swaps the contents of this and another list.
@@ -413,8 +407,8 @@ public:
     */
     bool containsPoint (Point<ValueType> point) const noexcept
     {
-        for (const RectangleType* r = rects.begin(), * const e = rects.end(); r != e; ++r)
-            if (r->contains (point))
+        for (auto& r : rects)
+            if (r.contains (point))
                 return true;
 
         return false;
@@ -434,21 +428,21 @@ public:
                     defined by this object
         @see intersectsRectangle, containsPoint
     */
-    bool containsRectangle (const RectangleType& rectangleToCheck) const
+    bool containsRectangle (RectangleType rectangleToCheck) const
     {
         if (rects.size() > 1)
         {
             RectangleList r (rectangleToCheck);
 
-            for (int i = rects.size(); --i >= 0;)
+            for (auto& rect : rects)
             {
-                r.subtract (rects.getReference (i));
+                r.subtract (rect);
 
-                if (r.rects.size() == 0)
+                if (r.isEmpty())
                     return true;
             }
         }
-        else if (rects.size() > 0)
+        else if (! isEmpty())
         {
             return rects.getReference (0).contains (rectangleToCheck);
         }
@@ -462,23 +456,22 @@ public:
                     defined by this object
         @see containsRectangle
     */
-    bool intersectsRectangle (const RectangleType& rectangleToCheck) const noexcept
+    bool intersectsRectangle (RectangleType rectangleToCheck) const noexcept
     {
-        for (const RectangleType* r = rects.begin(), * const e = rects.end(); r != e; ++r)
-            if (r->intersects (rectangleToCheck))
+        for (auto& r : rects)
+            if (r.intersects (rectangleToCheck))
                 return true;
 
         return false;
     }
 
     /** Checks whether this region intersects any part of another one.
-
         @see intersectsRectangle
     */
     bool intersects (const RectangleList& other) const noexcept
     {
-        for (const RectangleType* r = rects.begin(), * const e = rects.end(); r != e; ++r)
-            if (other.intersectsRectangle (*r))
+        for (auto& r : rects)
+            if (other.intersectsRectangle (r))
                 return true;
 
         return false;
@@ -488,24 +481,22 @@ public:
     /** Returns the smallest rectangle that can enclose the whole of this region. */
     RectangleType getBounds() const noexcept
     {
-        if (rects.size() <= 1)
-        {
-            if (rects.size() == 0)
-                return RectangleType();
+        if (isEmpty())
+            return {};
 
-            return rects.getReference (0);
-        }
+        auto& r = rects.getReference (0);
 
-        const RectangleType& r = rects.getReference (0);
+        if (rects.size() == 1)
+            return r;
 
-        ValueType minX = r.getX();
-        ValueType minY = r.getY();
-        ValueType maxX = minX + r.getWidth();
-        ValueType maxY = minY + r.getHeight();
+        auto minX = r.getX();
+        auto minY = r.getY();
+        auto maxX = minX + r.getWidth();
+        auto maxY = minY + r.getHeight();
 
         for (int i = rects.size(); --i > 0;)
         {
-            const RectangleType& r2 = rects.getReference (i);
+            auto& r2 = rects.getReference (i);
 
             minX = jmin (minX, r2.getX());
             minY = jmin (minY, r2.getY());
@@ -513,7 +504,7 @@ public:
             maxY = jmax (maxY, r2.getBottom());
         }
 
-        return RectangleType (minX, minY, maxX - minX, maxY - minY);
+        return { minX, minY, maxX - minX, maxY - minY };
     }
 
     /** Optimises the list into a minimum number of constituent rectangles.
@@ -526,19 +517,19 @@ public:
     {
         for (int i = 0; i < rects.size() - 1; ++i)
         {
-            RectangleType& r = rects.getReference (i);
-            const ValueType rx1 = r.getX();
-            const ValueType ry1 = r.getY();
-            const ValueType rx2 = rx1 + r.getWidth();
-            const ValueType ry2 = ry1 + r.getHeight();
+            auto& r = rects.getReference (i);
+            auto rx1 = r.getX();
+            auto ry1 = r.getY();
+            auto rx2 = rx1 + r.getWidth();
+            auto ry2 = ry1 + r.getHeight();
 
             for (int j = rects.size(); --j > i;)
             {
-                RectangleType& r2 = rects.getReference (j);
-                const ValueType jrx1 = r2.getX();
-                const ValueType jry1 = r2.getY();
-                const ValueType jrx2 = jrx1 + r2.getWidth();
-                const ValueType jry2 = jry1 + r2.getHeight();
+                auto& r2 = rects.getReference (j);
+                auto jrx1 = r2.getX();
+                auto jry1 = r2.getY();
+                auto jrx2 = jrx1 + r2.getWidth();
+                auto jry2 = jry1 + r2.getHeight();
 
                 // if the vertical edges of any blocks are touching and their horizontals don't
                 // line up, split them horizontally..
@@ -579,7 +570,7 @@ public:
 
         for (int i = 0; i < rects.size() - 1; ++i)
         {
-            RectangleType& r = rects.getReference (i);
+            auto& r = rects.getReference (i);
 
             for (int j = rects.size(); --j > i;)
             {
@@ -596,8 +587,8 @@ public:
     /** Adds an x and y value to all the coordinates. */
     void offsetAll (Point<ValueType> offset) noexcept
     {
-        for (RectangleType* r = rects.begin(), * const e = rects.end(); r != e; ++r)
-            *r += offset;
+        for (auto& r : rects)
+            r += offset;
     }
 
     /** Adds an x and y value to all the coordinates. */
@@ -610,8 +601,8 @@ public:
     template <typename ScaleType>
     void scaleAll (ScaleType scaleFactor) noexcept
     {
-        for (RectangleType* r = rects.begin(), * const e = rects.end(); r != e; ++r)
-            *r *= scaleFactor;
+        for (auto& r : rects)
+            r *= scaleFactor;
     }
 
     /** Applies a transform to all the rectangles.
@@ -620,8 +611,8 @@ public:
     */
     void transformAll (const AffineTransform& transform) noexcept
     {
-        for (RectangleType* r = rects.begin(), * const e = rects.end(); r != e; ++r)
-            *r = r->transformedBy (transform);
+        for (auto& r : rects)
+            r = r.transformedBy (transform);
     }
 
     //==============================================================================
@@ -630,8 +621,8 @@ public:
     {
         Path p;
 
-        for (int i = 0; i < rects.size(); ++i)
-            p.addRectangle (rects.getReference (i));
+        for (auto& r : rects)
+            p.addRectangle (r);
 
         return p;
     }
@@ -658,5 +649,4 @@ private:
     Array<RectangleType> rects;
 };
 
-
-#endif   // JUCE_RECTANGLELIST_H_INCLUDED
+} // namespace juce
