@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -33,7 +35,7 @@
 
   ID:               juce_graphics
   vendor:           juce
-  version:          4.3.0
+  version:          5.2.1
   name:             JUCE graphics classes
   description:      Classes for 2D vector graphics, image loading/saving, font handling, etc.
   website:          http://www.juce.com/juce
@@ -41,7 +43,7 @@
 
   dependencies:     juce_events
   OSXFrameworks:    Cocoa QuartzCore
-  iOSFrameworks:    CoreGraphics CoreText QuartzCore
+  iOSFrameworks:    CoreGraphics CoreImage CoreText QuartzCore
   linuxPackages:    x11 xinerama xext freetype2
 
  END_JUCE_MODULE_DECLARATION
@@ -49,7 +51,7 @@
 *******************************************************************************/
 
 
-#ifndef JUCE_GRAPHICS_H_INCLUDED // %%
+#pragma once
 #define JUCE_GRAPHICS_H_INCLUDED
 
 #include <juce_core/juce_core.h>
@@ -90,19 +92,20 @@
 //==============================================================================
 namespace juce
 {
-
-class Image;
-class AffineTransform;
-class Path;
-class Font;
-class Graphics;
-class FillType;
-class LowLevelGraphicsContext;
+    class Image;
+    class AffineTransform;
+    class Path;
+    class Font;
+    class Graphics;
+    class FillType;
+    class LowLevelGraphicsContext;
+}
 
 #include "geometry/juce_AffineTransform.h"
 #include "geometry/juce_Point.h"
 #include "geometry/juce_Line.h"
 #include "geometry/juce_Rectangle.h"
+#include "geometry/juce_Parallelogram.h"
 #include "placement/juce_Justification.h"
 #include "geometry/juce_Path.h"
 #include "geometry/juce_RectangleList.h"
@@ -140,6 +143,6 @@ class LowLevelGraphicsContext;
  #include "native/juce_mac_CoreGraphicsContext.h"
 #endif
 
-}
-
-#endif   // JUCE_GRAPHICS_H_INCLUDED
+#if JUCE_DIRECT2D && JUCE_WINDOWS
+#include "native/juce_win32_Direct2DGraphicsContext.h"
+#endif
