@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 /*  This is some quick-and-dirty code to extract the typeface name from a lump of TTF file data.
     It's needed because although win32 will happily load a TTF file from in-memory data, it won't
     tell you the name of the damned font that it just loaded.. and in order to actually use the font,
@@ -73,7 +78,7 @@ namespace TTFNameExtractor
             for (int i = 0; i < numChars; ++i)
                 buffer[i] = ByteOrder::swapIfLittleEndian (buffer[i]);
 
-            static_jassert (sizeof (CharPointer_UTF16::CharType) == sizeof (uint16));
+            static_assert (sizeof (CharPointer_UTF16::CharType) == sizeof (uint16), "Sanity check UTF-16 type");
             result = CharPointer_UTF16 ((CharPointer_UTF16::CharType*) buffer.getData());
         }
         else
@@ -110,7 +115,7 @@ namespace TTFNameExtractor
             }
         }
 
-        return String();
+        return {};
     }
 
     static String getTypefaceNameFromFile (MemoryInputStream& input)
@@ -128,7 +133,7 @@ namespace TTFNameExtractor
                 return parseNameTable (input, ByteOrder::swapIfLittleEndian (tableDirectory.offset));
         }
 
-        return String();
+        return {};
     }
 }
 
@@ -642,3 +647,5 @@ void Typeface::scanFolderForFonts (const File&)
 {
     jassertfalse; // not implemented on this platform
 }
+
+} // namespace juce
