@@ -2,32 +2,36 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-class Button::CallbackHelper  : public Timer,
-                                public ApplicationCommandManagerListener,
-                                public ValueListener,
-                                public KeyListener
+namespace juce
+{
+
+struct Button::CallbackHelper  : public Timer,
+                                 public ApplicationCommandManagerListener,
+                                 public Value::Listener,
+                                 public KeyListener
 {
-public:
     CallbackHelper (Button& b) : button (b)   {}
 
     void timerCallback() override
@@ -43,7 +47,7 @@ public:
     void valueChanged (Value& value) override
     {
         if (value.refersToSameSourceAs (button.isOn))
-            button.setToggleState (button.isOn.getValue(), sendNotification);
+            button.setToggleState (button.isOn.getValue(), dontSendNotification);
     }
 
     bool keyPressed (const KeyPress&, Component*) override
@@ -64,39 +68,18 @@ public:
         button.applicationCommandListChangeCallback();
     }
 
-private:
     Button& button;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (CallbackHelper)
 };
 
 //==============================================================================
-Button::Button (const String& name)
-  : Component (name),
-    text (name),
-    buttonPressTime (0),
-    lastRepeatTime (0),
-    commandManagerToUse (nullptr),
-    autoRepeatDelay (-1),
-    autoRepeatSpeed (0),
-    autoRepeatMinimumDelay (-1),
-    radioGroupId (0),
-    connectedEdgeFlags (0),
-    commandID(),
-    buttonState (buttonNormal),
-    lastStatePainted (buttonNormal),
-    lastToggleState (false),
-    clickTogglesState (false),
-    needsToRelease (false),
-    needsRepainting (false),
-    isKeyDown (false),
-    triggerOnMouseDown (false),
-    generateTooltip (false)
+Button::Button (const String& name)  : Component (name), text (name)
 {
-    callbackHelper = new CallbackHelper (*this);
+    callbackHelper.reset (new CallbackHelper (*this));
 
     setWantsKeyboardFocus (true);
-    isOn.addListener (callbackHelper);
+    isOn.addListener (callbackHelper.get());
 }
 
 Button::~Button()
@@ -104,10 +87,10 @@ Button::~Button()
     clearShortcuts();
 
     if (commandManagerToUse != nullptr)
-        commandManagerToUse->removeListener (callbackHelper);
+        commandManagerToUse->removeListener (callbackHelper.get());
 
-    isOn.removeListener (callbackHelper);
-    callbackHelper = nullptr;
+    isOn.removeListener (callbackHelper.get());
+    callbackHelper.reset();
 }
 
 //==============================================================================
@@ -130,14 +113,12 @@ void Button::updateAutomaticTooltip (const ApplicationCommandInfo& info)
 {
     if (generateTooltip && commandManagerToUse != nullptr)
     {
-        String tt (info.description.isNotEmpty() ? info.description
-                                                 : info.shortName);
+        auto tt = info.description.isNotEmpty() ? info.description
+                                                : info.shortName;
 
-        Array<KeyPress> keyPresses (commandManagerToUse->getKeyMappings()->getKeyPressesAssignedToCommand (commandID));
-
-        for (int i = 0; i < keyPresses.size(); ++i)
+        for (auto& kp : commandManagerToUse->getKeyMappings()->getKeyPressesAssignedToCommand (commandID))
         {
-            const String key (keyPresses.getReference(i).getTextDescription());
+            auto key = kp.getTextDescription();
 
             tt << " [";
 
@@ -240,19 +221,17 @@ void Button::setRadioGroupId (const int newGroupId, NotificationType notificatio
 
 void Button::turnOffOtherButtonsInGroup (const NotificationType notification)
 {
-    if (Component* const p = getParentComponent())
+    if (auto* p = getParentComponent())
     {
         if (radioGroupId != 0)
         {
             WeakReference<Component> deletionWatcher (this);
 
-            for (int i = p->getNumChildComponents(); --i >= 0;)
+            for (auto* c : p->getChildren())
             {
-                Component* const c = p->getChildComponent (i);
-
                 if (c != this)
                 {
-                    if (Button* const b = dynamic_cast<Button*> (c))
+                    if (auto b = dynamic_cast<Button*> (c))
                     {
                         if (b->getRadioGroupId() == radioGroupId)
                         {
@@ -320,7 +299,7 @@ void Button::buttonStateChanged() {}
 
 uint32 Button::getMillisecondsSinceButtonDown() const noexcept
 {
-    const uint32 now = Time::getApproximateMillisecondCounter();
+    auto now = Time::getApproximateMillisecondCounter();
     return now > buttonPressTime ? now - buttonPressTime : 0;
 }
 
@@ -329,6 +308,11 @@ void Button::setTriggeredOnMouseDown (const bool isTriggeredOnMouseDown) noexcep
     triggerOnMouseDown = isTriggeredOnMouseDown;
 }
 
+bool Button::getTriggeredOnMouseDown() const noexcept
+{
+    return triggerOnMouseDown;
+}
+
 //==============================================================================
 void Button::clicked()
 {
@@ -389,15 +373,8 @@ void Button::handleCommandMessage (int commandId)
 }
 
 //==============================================================================
-void Button::addListener (ButtonListener* const newListener)
-{
-    buttonListeners.add (newListener);
-}
-
-void Button::removeListener (ButtonListener* const listener)
-{
-    buttonListeners.remove (listener);
-}
+void Button::addListener (Listener* l)      { buttonListeners.add (l); }
+void Button::removeListener (Listener* l)   { buttonListeners.remove (l); }
 
 void Button::sendClickMessage (const ModifierKeys& modifiers)
 {
@@ -414,8 +391,16 @@ void Button::sendClickMessage (const ModifierKeys& modifiers)
 
     clicked (modifiers);
 
-    if (! checker.shouldBailOut())
-        buttonListeners.callChecked (checker, &ButtonListener::buttonClicked, this);  // (can't use Button::Listener due to idiotic VC2005 bug)
+    if (checker.shouldBailOut())
+        return;
+
+    buttonListeners.callChecked (checker, [this] (Listener& l) { l.buttonClicked (this); });
+
+    if (checker.shouldBailOut())
+        return;
+
+    if (onClick != nullptr)
+        onClick();
 }
 
 void Button::sendStateMessage()
@@ -424,8 +409,16 @@ void Button::sendStateMessage()
 
     buttonStateChanged();
 
-    if (! checker.shouldBailOut())
-        buttonListeners.callChecked (checker, &ButtonListener::buttonStateChanged, this);
+    if (checker.shouldBailOut())
+        return;
+
+    buttonListeners.callChecked (checker, [this] (Listener& l) { l.buttonStateChanged (this); });
+
+    if (checker.shouldBailOut())
+        return;
+
+    if (onStateChange != nullptr)
+        onStateChange();
 }
 
 //==============================================================================
@@ -476,7 +469,7 @@ void Button::mouseUp (const MouseEvent& e)
 
 void Button::mouseDrag (const MouseEvent& e)
 {
-    const ButtonState oldState = buttonState;
+    auto oldState = buttonState;
     updateState (isMouseOrTouchOver (e), true);
 
     if (autoRepeatDelay >= 0 && buttonState != oldState && isDown())
@@ -511,17 +504,17 @@ void Button::visibilityChanged()
 
 void Button::parentHierarchyChanged()
 {
-    Component* const newKeySource = (shortcuts.size() == 0) ? nullptr : getTopLevelComponent();
+    auto* newKeySource = shortcuts.isEmpty() ? nullptr : getTopLevelComponent();
 
     if (newKeySource != keySource.get())
     {
         if (keySource != nullptr)
-            keySource->removeKeyListener (callbackHelper);
+            keySource->removeKeyListener (callbackHelper.get());
 
         keySource = newKeySource;
 
         if (keySource != nullptr)
-            keySource->addKeyListener (callbackHelper);
+            keySource->addKeyListener (callbackHelper.get());
     }
 }
 
@@ -535,12 +528,12 @@ void Button::setCommandToTrigger (ApplicationCommandManager* const newCommandMan
     if (commandManagerToUse != newCommandManager)
     {
         if (commandManagerToUse != nullptr)
-            commandManagerToUse->removeListener (callbackHelper);
+            commandManagerToUse->removeListener (callbackHelper.get());
 
         commandManagerToUse = newCommandManager;
 
         if (commandManagerToUse != nullptr)
-            commandManagerToUse->addListener (callbackHelper);
+            commandManagerToUse->addListener (callbackHelper.get());
 
         // if you've got clickTogglesState turned on, you shouldn't also connect the button
         // up to be a command invoker. Instead, your command handler must flip the state of whatever
@@ -595,8 +588,8 @@ void Button::clearShortcuts()
 bool Button::isShortcutPressed() const
 {
     if (isShowing() && ! isCurrentlyBlockedByAnotherModalComponent())
-        for (int i = shortcuts.size(); --i >= 0;)
-            if (shortcuts.getReference(i).isCurrentlyDown())
+        for (auto& s : shortcuts)
+            if (s.isCurrentlyDown())
                 return true;
 
     return false;
@@ -604,8 +597,8 @@ bool Button::isShortcutPressed() const
 
 bool Button::isRegisteredForShortcut (const KeyPress& key) const
 {
-    for (int i = shortcuts.size(); --i >= 0;)
-        if (key == shortcuts.getReference(i))
+    for (auto& s : shortcuts)
+        if (key == s)
             return true;
 
     return false;
@@ -666,11 +659,11 @@ void Button::repeatTimerCallback()
     }
     else if (autoRepeatSpeed > 0 && (isKeyDown || (updateState() == buttonDown)))
     {
-        int repeatSpeed = autoRepeatSpeed;
+        auto repeatSpeed = autoRepeatSpeed;
 
         if (autoRepeatMinimumDelay >= 0)
         {
-            double timeHeldDown = jmin (1.0, getMillisecondsSinceButtonDown() / 4000.0);
+            auto timeHeldDown = jmin (1.0, getMillisecondsSinceButtonDown() / 4000.0);
             timeHeldDown *= timeHeldDown;
 
             repeatSpeed = repeatSpeed + (int) (timeHeldDown * (autoRepeatMinimumDelay - repeatSpeed));
@@ -678,7 +671,7 @@ void Button::repeatTimerCallback()
 
         repeatSpeed = jmax (1, repeatSpeed);
 
-        const uint32 now = Time::getMillisecondCounter();
+        auto now = Time::getMillisecondCounter();
 
         // if we've been blocked from repeating often enough, speed up the repeat timer to compensate..
         if (lastRepeatTime != 0 && (int) (now - lastRepeatTime) > repeatSpeed * 2)
@@ -694,3 +687,5 @@ void Button::repeatTimerCallback()
         callbackHelper->stopTimer();
     }
 }
+
+} // namespace juce
