@@ -2,35 +2,36 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 Desktop::Desktop()
     : mouseSources (new MouseInputSource::SourceList()),
-      mouseClickCounter (0), mouseWheelCounter (0),
-      kioskModeComponent (nullptr),
-      kioskModeReentrant (false),
-      allowedOrientations (allOrientations),
       masterScaleFactor ((float) getDefaultMasterScale())
 {
-    displays = new Displays (*this);
+    displays.reset (new Displays (*this));
 }
 
 Desktop::~Desktop()
@@ -62,7 +63,7 @@ int Desktop::getNumComponents() const noexcept
     return desktopComponents.size();
 }
 
-Component* Desktop::getComponent (const int index) const noexcept
+Component* Desktop::getComponent (int index) const noexcept
 {
     return desktopComponents [index];
 }
@@ -73,11 +74,11 @@ Component* Desktop::findComponentAt (Point<int> screenPosition) const
 
     for (int i = desktopComponents.size(); --i >= 0;)
     {
-        Component* const c = desktopComponents.getUnchecked(i);
+        auto* c = desktopComponents.getUnchecked(i);
 
         if (c->isVisible())
         {
-            const Point<int> relative (c->getLocalPoint (nullptr, screenPosition));
+            auto relative = c->getLocalPoint (nullptr, screenPosition);
 
             if (c->contains (relative))
                 return c->getComponentAt (relative);
@@ -93,9 +94,9 @@ LookAndFeel& Desktop::getDefaultLookAndFeel() noexcept
     if (currentLookAndFeel == nullptr)
     {
         if (defaultLookAndFeel == nullptr)
-            defaultLookAndFeel = new LookAndFeel_V3();
+            defaultLookAndFeel.reset (new LookAndFeel_V4());
 
-        currentLookAndFeel = defaultLookAndFeel;
+        currentLookAndFeel = defaultLookAndFeel.get();
     }
 
     return *currentLookAndFeel;
@@ -107,26 +108,26 @@ void Desktop::setDefaultLookAndFeel (LookAndFeel* newDefaultLookAndFeel)
     currentLookAndFeel = newDefaultLookAndFeel;
 
     for (int i = getNumComponents(); --i >= 0;)
-        if (Component* const c = getComponent (i))
+        if (auto* c = getComponent (i))
             c->sendLookAndFeelChange();
 }
 
 //==============================================================================
-void Desktop::addDesktopComponent (Component* const c)
+void Desktop::addDesktopComponent (Component* c)
 {
     jassert (c != nullptr);
     jassert (! desktopComponents.contains (c));
     desktopComponents.addIfNotAlreadyThere (c);
 }
 
-void Desktop::removeDesktopComponent (Component* const c)
+void Desktop::removeDesktopComponent (Component* c)
 {
     desktopComponents.removeFirstMatchingValue (c);
 }
 
-void Desktop::componentBroughtToFront (Component* const c)
+void Desktop::componentBroughtToFront (Component* c)
 {
-    const int index = desktopComponents.indexOf (c);
+    auto index = desktopComponents.indexOf (c);
     jassert (index >= 0);
 
     if (index >= 0)
@@ -183,16 +184,16 @@ MouseInputSource Desktop::getMainMouseSource() const noexcept
 void Desktop::beginDragAutoRepeat (int interval)                                { mouseSources->beginDragAutoRepeat (interval); }
 
 //==============================================================================
-void Desktop::addFocusChangeListener    (FocusChangeListener* const listener)   { focusListeners.add (listener); }
-void Desktop::removeFocusChangeListener (FocusChangeListener* const listener)   { focusListeners.remove (listener); }
-void Desktop::triggerFocusCallback()                                            { triggerAsyncUpdate(); }
+void Desktop::addFocusChangeListener    (FocusChangeListener* l)   { focusListeners.add (l); }
+void Desktop::removeFocusChangeListener (FocusChangeListener* l)   { focusListeners.remove (l); }
+void Desktop::triggerFocusCallback()                               { triggerAsyncUpdate(); }
 
 void Desktop::handleAsyncUpdate()
 {
     // The component may be deleted during this operation, but we'll use a SafePointer rather than a
     // BailOutChecker so that any remaining listeners will still get a callback (with a null pointer).
     WeakReference<Component> currentFocus (Component::getCurrentlyFocusedComponent());
-    focusListeners.call (&FocusChangeListener::globalFocusChanged, currentFocus);
+    focusListeners.call ([&] (FocusChangeListener& l) { l.globalFocusChanged (currentFocus); });
 }
 
 //==============================================================================
@@ -212,14 +213,14 @@ ListenerList<MouseListener>& Desktop::getMouseListeners()
     return mouseListeners;
 }
 
-void Desktop::addGlobalMouseListener (MouseListener* const listener)
+void Desktop::addGlobalMouseListener (MouseListener* listener)
 {
     ASSERT_MESSAGE_MANAGER_IS_LOCKED
     mouseListeners.add (listener);
     resetTimer();
 }
 
-void Desktop::removeGlobalMouseListener (MouseListener* const listener)
+void Desktop::removeGlobalMouseListener (MouseListener* listener)
 {
     ASSERT_MESSAGE_MANAGER_IS_LOCKED
     mouseListeners.remove (listener);
@@ -240,19 +241,21 @@ void Desktop::sendMouseMove()
 
         lastFakeMouseMove = getMousePositionFloat();
 
-        if (Component* const target = findComponentAt (lastFakeMouseMove.roundToInt()))
+        if (auto* target = findComponentAt (lastFakeMouseMove.roundToInt()))
         {
             Component::BailOutChecker checker (target);
-            const Point<float> pos (target->getLocalPoint (nullptr, lastFakeMouseMove));
-            const Time now (Time::getCurrentTime());
+            auto pos = target->getLocalPoint (nullptr, lastFakeMouseMove);
+            auto now = Time::getCurrentTime();
 
-            const MouseEvent me (getMainMouseSource(), pos, ModifierKeys::getCurrentModifiers(),
-                                 MouseInputSource::invalidPressure, target, target, now, pos, now, 0, false);
+            const MouseEvent me (getMainMouseSource(), pos, ModifierKeys::getCurrentModifiers(), MouseInputSource::invalidPressure,
+                                 MouseInputSource::invalidOrientation, MouseInputSource::invalidRotation,
+                                 MouseInputSource::invalidTiltX, MouseInputSource::invalidTiltY,
+                                 target, target, now, pos, now, 0, false);
 
             if (me.mods.isAnyMouseButtonDown())
-                mouseListeners.callChecked (checker, &MouseListener::mouseDrag, me);
+                mouseListeners.callChecked (checker, [&] (MouseListener& l) { l.mouseDrag (me); });
             else
-                mouseListeners.callChecked (checker, &MouseListener::mouseMove, me);
+                mouseListeners.callChecked (checker, [&] (MouseListener& l) { l.mouseMove (me); });
         }
     }
 }
@@ -272,20 +275,18 @@ const Desktop::Displays::Display& Desktop::Displays::getMainDisplay() const noex
 const Desktop::Displays::Display& Desktop::Displays::getDisplayContaining (Point<int> position) const noexcept
 {
     ASSERT_MESSAGE_MANAGER_IS_LOCKED
-    const Display* best = &displays.getReference(0);
+    auto* best = &displays.getReference(0);
     double bestDistance = 1.0e10;
 
-    for (int i = displays.size(); --i >= 0;)
+    for (auto& d : displays)
     {
-        const Display& d = displays.getReference(i);
-
         if (d.totalArea.contains (position))
         {
             best = &d;
             break;
         }
 
-        const double distance = d.totalArea.getCentre().getDistanceFrom (position);
+        auto distance = d.totalArea.getCentre().getDistanceFrom (position);
 
         if (distance < bestDistance)
         {
@@ -302,11 +303,8 @@ RectangleList<int> Desktop::Displays::getRectangleList (bool userAreasOnly) cons
     ASSERT_MESSAGE_MANAGER_IS_LOCKED
     RectangleList<int> rl;
 
-    for (int i = 0; i < displays.size(); ++i)
-    {
-        const Display& d = displays.getReference(i);
+    for (auto& d : displays)
         rl.addWithoutMerging (userAreasOnly ? d.userArea : d.totalArea);
-    }
 
     return rl;
 }
@@ -346,13 +344,13 @@ void Desktop::Displays::refresh()
     if (oldDisplays != displays)
     {
         for (int i = ComponentPeer::getNumPeers(); --i >= 0;)
-            if (ComponentPeer* const peer = ComponentPeer::getPeer (i))
+            if (auto* peer = ComponentPeer::getPeer (i))
                 peer->handleScreenSizeChange();
     }
 }
 
 //==============================================================================
-void Desktop::setKioskModeComponent (Component* componentToUse, const bool allowMenusAndBars)
+void Desktop::setKioskModeComponent (Component* componentToUse, bool allowMenusAndBars)
 {
     if (kioskModeReentrant)
         return;
@@ -364,7 +362,7 @@ void Desktop::setKioskModeComponent (Component* componentToUse, const bool allow
         // agh! Don't delete or remove a component from the desktop while it's still the kiosk component!
         jassert (kioskModeComponent == nullptr || ComponentPeer::getPeerFor (kioskModeComponent) != nullptr);
 
-        if (Component* const oldKioskComp = kioskModeComponent)
+        if (auto* oldKioskComp = kioskModeComponent)
         {
             kioskModeComponent = nullptr; // (to make sure that isKioskMode() returns false when resizing the old one)
             setKioskComponent (oldKioskComp, false, allowMenusAndBars);
@@ -385,7 +383,7 @@ void Desktop::setKioskModeComponent (Component* componentToUse, const bool allow
 }
 
 //==============================================================================
-void Desktop::setOrientationsEnabled (const int newOrientations)
+void Desktop::setOrientationsEnabled (int newOrientations)
 {
     if (allowedOrientations != newOrientations)
     {
@@ -397,7 +395,12 @@ void Desktop::setOrientationsEnabled (const int newOrientations)
     }
 }
 
-bool Desktop::isOrientationEnabled (const DisplayOrientation orientation) const noexcept
+int Desktop::getOrientationsEnabled() const noexcept
+{
+    return allowedOrientations;
+}
+
+bool Desktop::isOrientationEnabled (DisplayOrientation orientation) const noexcept
 {
     // Make sure you only pass one valid flag in here...
     jassert (orientation == upright || orientation == upsideDown
@@ -416,3 +419,5 @@ void Desktop::setGlobalScaleFactor (float newScaleFactor) noexcept
         displays->refresh();
     }
 }
+
+} // namespace juce
