@@ -2,33 +2,36 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-class ModalComponentManager::ModalItem  : public ComponentMovementWatcher
+namespace juce
 {
-public:
-    ModalItem (Component* const comp, const bool autoDelete_)
+
+struct ModalComponentManager::ModalItem  : public ComponentMovementWatcher
+{
+    ModalItem (Component* comp, bool shouldAutoDelete)
         : ComponentMovementWatcher (comp),
-          component (comp), returnValue (0),
-          isActive (true), autoDelete (autoDelete_)
+          component (comp), autoDelete (shouldAutoDelete)
     {
         jassert (comp != nullptr);
     }
@@ -37,8 +40,7 @@ public:
 
     void componentPeerChanged() override
     {
-        if (! component->isShowing())
-            cancel();
+        componentVisibilityChanged();
     }
 
     void componentVisibilityChanged() override
@@ -64,17 +66,16 @@ public:
         {
             isActive = false;
 
-            if (ModalComponentManager* mcm = ModalComponentManager::getInstanceWithoutCreating())
+            if (auto* mcm = ModalComponentManager::getInstanceWithoutCreating())
                 mcm->triggerAsyncUpdate();
         }
     }
 
     Component* component;
     OwnedArray<Callback> callbacks;
-    int returnValue;
-    bool isActive, autoDelete;
+    int returnValue = 0;
+    bool isActive = true, autoDelete;
 
-private:
     JUCE_DECLARE_NON_COPYABLE (ModalItem)
 };
 
@@ -89,7 +90,7 @@ ModalComponentManager::~ModalComponentManager()
     clearSingletonInstance();
 }
 
-juce_ImplementSingleton_SingleThreaded (ModalComponentManager)
+JUCE_IMPLEMENT_SINGLETON (ModalComponentManager)
 
 
 //==============================================================================
@@ -107,7 +108,7 @@ void ModalComponentManager::attachCallback (Component* component, Callback* call
 
         for (int i = stack.size(); --i >= 0;)
         {
-            ModalItem* const item = stack.getUnchecked(i);
+            auto* item = stack.getUnchecked(i);
 
             if (item->component == component)
             {
@@ -123,7 +124,7 @@ void ModalComponentManager::endModal (Component* component)
 {
     for (int i = stack.size(); --i >= 0;)
     {
-        ModalItem* const item = stack.getUnchecked(i);
+        auto* item = stack.getUnchecked(i);
 
         if (item->component == component)
             item->cancel();
@@ -134,7 +135,7 @@ void ModalComponentManager::endModal (Component* component, int returnValue)
 {
     for (int i = stack.size(); --i >= 0;)
     {
-        ModalItem* const item = stack.getUnchecked(i);
+        auto* item = stack.getUnchecked(i);
 
         if (item->component == component)
         {
@@ -147,19 +148,22 @@ void ModalComponentManager::endModal (Component* component, int returnValue)
 int ModalComponentManager::getNumModalComponents() const
 {
     int n = 0;
-    for (int i = 0; i < stack.size(); ++i)
-        if (stack.getUnchecked(i)->isActive)
+
+    for (auto* item : stack)
+        if (item->isActive)
             ++n;
 
     return n;
 }
 
-Component* ModalComponentManager::getModalComponent (const int index) const
+Component* ModalComponentManager::getModalComponent (int index) const
 {
     int n = 0;
+
     for (int i = stack.size(); --i >= 0;)
     {
-        const ModalItem* const item = stack.getUnchecked(i);
+        auto* item = stack.getUnchecked(i);
+
         if (item->isActive)
             if (n++ == index)
                 return item->component;
@@ -168,19 +172,16 @@ Component* ModalComponentManager::getModalComponent (const int index) const
     return nullptr;
 }
 
-bool ModalComponentManager::isModal (Component* const comp) const
+bool ModalComponentManager::isModal (const Component* comp) const
 {
-    for (int i = stack.size(); --i >= 0;)
-    {
-        const ModalItem* const item = stack.getUnchecked(i);
+    for (auto* item : stack)
         if (item->isActive && item->component == comp)
             return true;
-    }
 
     return false;
 }
 
-bool ModalComponentManager::isFrontModalComponent (Component* const comp) const
+bool ModalComponentManager::isFrontModalComponent (const Component* comp) const
 {
     return comp == getModalComponent (0);
 }
@@ -189,7 +190,7 @@ void ModalComponentManager::handleAsyncUpdate()
 {
     for (int i = stack.size(); --i >= 0;)
     {
-        const ModalItem* const item = stack.getUnchecked(i);
+        auto* item = stack.getUnchecked(i);
 
         if (! item->isActive)
         {
@@ -210,36 +211,39 @@ void ModalComponentManager::bringModalComponentsToFront (bool topOneShouldGrabFo
 
     for (int i = 0; i < getNumModalComponents(); ++i)
     {
-        Component* const c = getModalComponent (i);
+        auto* c = getModalComponent (i);
 
         if (c == nullptr)
             break;
 
-        ComponentPeer* peer = c->getPeer();
-
-        if (peer != nullptr && peer != lastOne)
+        if (auto* peer = c->getPeer())
         {
-            if (lastOne == nullptr)
+            if (peer != lastOne)
             {
-                peer->toFront (topOneShouldGrabFocus);
+                if (lastOne == nullptr)
+                {
+                    peer->toFront (topOneShouldGrabFocus);
 
-                if (topOneShouldGrabFocus)
-                    peer->grabFocus();
+                    if (topOneShouldGrabFocus)
+                        peer->grabFocus();
+                }
+                else
+                {
+                    peer->toBehind (lastOne);
+                }
+
+                lastOne = peer;
             }
-            else
-                peer->toBehind (lastOne);
-
-            lastOne = peer;
         }
     }
 }
 
 bool ModalComponentManager::cancelAllModalComponents()
 {
-    const int numModal = getNumModalComponents();
+    auto numModal = getNumModalComponents();
 
     for (int i = numModal; --i >= 0;)
-        if (Component* const c = getModalComponent(i))
+        if (auto* c = getModalComponent(i))
             c->exitModalState (0);
 
     return numModal > 0;
@@ -247,24 +251,6 @@ bool ModalComponentManager::cancelAllModalComponents()
 
 //==============================================================================
 #if JUCE_MODAL_LOOPS_PERMITTED
-class ModalComponentManager::ReturnValueRetriever     : public ModalComponentManager::Callback
-{
-public:
-    ReturnValueRetriever (int& v, bool& done) : value (v), finished (done) {}
-
-    void modalStateFinished (int returnValue) override
-    {
-        finished = true;
-        value = returnValue;
-    }
-
-private:
-    int& value;
-    bool& finished;
-
-    JUCE_DECLARE_NON_COPYABLE (ReturnValueRetriever)
-};
-
 int ModalComponentManager::runEventLoopForCurrentComponent()
 {
     // This can only be run from the message thread!
@@ -272,12 +258,12 @@ int ModalComponentManager::runEventLoopForCurrentComponent()
 
     int returnValue = 0;
 
-    if (Component* currentlyModal = getModalComponent (0))
+    if (auto* currentlyModal = getModalComponent (0))
     {
         FocusRestorer focusRestorer;
-
         bool finished = false;
-        attachCallback (currentlyModal, new ReturnValueRetriever (returnValue, finished));
+
+        attachCallback (currentlyModal, ModalCallbackFunction::create ([&] (int r) { returnValue = r; finished = true; }));
 
         JUCE_TRY
         {
@@ -295,7 +281,6 @@ int ModalComponentManager::runEventLoopForCurrentComponent()
 #endif
 
 //==============================================================================
-#if JUCE_COMPILER_SUPPORTS_LAMBDAS
 struct LambdaCallback  : public ModalComponentManager::Callback
 {
     LambdaCallback (std::function<void(int)> fn) noexcept : function (fn) {}
@@ -310,4 +295,5 @@ ModalComponentManager::Callback* ModalCallbackFunction::create (std::function<vo
 {
     return new LambdaCallback (f);
 }
-#endif
+
+} // namespace juce
