@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_DRAWABLE_H_INCLUDED
-#define JUCE_DRAWABLE_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -53,6 +54,9 @@ public:
     */
     virtual Drawable* createCopy() const = 0;
 
+    /** Creates a path that describes the outline of this drawable. */
+    virtual Path getOutlineAsPath() const = 0;
+
     //==============================================================================
     /** Renders this Drawable object.
 
@@ -97,7 +101,7 @@ public:
         @param opacity                  the opacity to use, in the range 0 to 1.0
     */
     void drawWithin (Graphics& g,
-                     const Rectangle<float>& destArea,
+                     Rectangle<float> destArea,
                      RectanglePlacement placement,
                      float opacity) const;
 
@@ -116,6 +120,11 @@ public:
     /** Returns the DrawableComposite that contains this object, if there is one. */
     DrawableComposite* getParent() const;
 
+    /** Sets a the clipping region of this drawable using another drawable.
+        The drawbale passed in ill be deleted when no longer needed.
+    */
+    void setClipPath (Drawable* drawableClipPath);
+
     //==============================================================================
     /** Tries to turn some kind of image file into a drawable.
 
@@ -149,26 +158,24 @@ public:
     */
     static Drawable* createFromSVG (const XmlElement& svgDocument);
 
+    /** Attempts to parse an SVG (Scalable Vector Graphics) document from a file,
+        and to turn this into a Drawable tree.
+
+        The object returned must be deleted by the caller. If something goes wrong
+        while parsing, it may return nullptr.
+
+        SVG is a pretty large and complex spec, and this doesn't aim to be a full
+        implementation, but it can return the basic vector objects.
+
+        Any references to references to external image files will be relative to
+        the parent directory of the file passed.
+    */
+    static Drawable* createFromSVGFile (const File& svgFile);
+
     /** Parses an SVG path string and returns it. */
     static Path parseSVGPath (const String& svgPath);
 
     //==============================================================================
-    /** Tries to create a Drawable from a previously-saved ValueTree.
-        The ValueTree must have been created by the createValueTree() method.
-        If there are any images used within the drawable, you'll need to provide a valid
-        ImageProvider object that can be used to retrieve these images from whatever type
-        of identifier is used to represent them.
-        Internally, this uses a ComponentBuilder, and registerDrawableTypeHandlers().
-    */
-    static Drawable* createFromValueTree (const ValueTree& tree, ComponentBuilder::ImageProvider* imageProvider);
-
-    /** Creates a ValueTree to represent this Drawable.
-        The ValueTree that is returned can be turned back into a Drawable with createFromValueTree().
-        If there are any images used in this drawable, you'll need to provide a valid ImageProvider
-        object that can be used to create storable representations of them.
-    */
-    virtual ValueTree createValueTree (ComponentBuilder::ImageProvider* imageProvider) const = 0;
-
     /** Returns the area that this drawble covers.
         The result is expressed in this drawable's own coordinate space, and does not take
         into account any transforms that may be applied to the component.
@@ -180,28 +187,6 @@ public:
     */
     virtual bool replaceColour (Colour originalColour, Colour replacementColour);
 
-    //==============================================================================
-    /** Internal class used to manage ValueTrees that represent Drawables. */
-    class ValueTreeWrapperBase
-    {
-    public:
-        ValueTreeWrapperBase (const ValueTree& state);
-
-        ValueTree& getState() noexcept          { return state; }
-
-        String getID() const;
-        void setID (const String& newID);
-
-        ValueTree state;
-    };
-
-    //==============================================================================
-    /** Registers a set of ComponentBuilder::TypeHandler objects that can be used to
-        load all the different Drawable types from a saved state.
-        @see ComponentBuilder::registerTypeHandler()
-    */
-    static void registerDrawableTypeHandlers (ComponentBuilder& componentBuilder);
-
 protected:
     //==============================================================================
     friend class DrawableComposite;
@@ -212,49 +197,18 @@ protected:
     /** @internal */
     void parentHierarchyChanged() override;
     /** @internal */
-    void setBoundsToEnclose (const Rectangle<float>&);
+    void setBoundsToEnclose (Rectangle<float>);
+    /** @internal */
+    void applyDrawableClipPath (Graphics&);
 
     Point<int> originRelativeToComponent;
+    ScopedPointer<Drawable> drawableClipPath;
 
-  #ifndef DOXYGEN
-    /** Internal utility class used by Drawables. */
-    template <class DrawableType>
-    class Positioner  : public RelativeCoordinatePositionerBase
-    {
-    public:
-        Positioner (DrawableType& c)
-            : RelativeCoordinatePositionerBase (c),
-              owner (c)
-        {}
-
-        bool registerCoordinates() override      { return owner.registerCoordinates (*this); }
-
-        void applyToComponentBounds() override
-        {
-            ComponentScope scope (getComponent());
-            owner.recalculateCoordinates (&scope);
-        }
-
-        void applyNewBounds (const Rectangle<int>&) override
-        {
-            jassertfalse; // drawables can't be resized directly!
-        }
-
-    private:
-        DrawableType& owner;
-
-        JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Positioner)
-    };
-
-    Drawable (const Drawable&);
-  #endif
-
-private:
     void nonConstDraw (Graphics&, float opacity, const AffineTransform&);
 
+    Drawable (const Drawable&);
     Drawable& operator= (const Drawable&);
     JUCE_LEAK_DETECTOR (Drawable)
 };
 
-
-#endif   // JUCE_DRAWABLE_H_INCLUDED
+} // namespace juce
