@@ -2,37 +2,42 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 FileBrowserComponent::FileBrowserComponent (int flags_,
                                             const File& initialFileOrDirectory,
                                             const FileFilter* fileFilter_,
                                             FilePreviewComponent* previewComp_)
-   : FileFilter (String()),
+   : FileFilter ({}),
      fileFilter (fileFilter_),
      flags (flags_),
      previewComp (previewComp_),
      currentPathBox ("path"),
      fileLabel ("f", TRANS ("file:")),
-     thread ("Juce FileBrowser"),
+     thread ("JUCE FileBrowser"),
      wasProcessActive (true)
 {
     // You need to specify one or other of the open/save flags..
@@ -59,12 +64,12 @@ FileBrowserComponent::FileBrowserComponent (int flags_,
         filename = initialFileOrDirectory.getFileName();
     }
 
-    fileList = new DirectoryContentsList (this, thread);
+    fileList.reset (new DirectoryContentsList (this, thread));
 
     if ((flags & useTreeView) != 0)
     {
-        FileTreeComponent* const tree = new FileTreeComponent (*fileList);
-        fileListComponent = tree;
+        auto tree = new FileTreeComponent (*fileList);
+        fileListComponent.reset (tree);
 
         if ((flags & canSelectMultipleItems) != 0)
             tree->setMultiSelectEnabled (true);
@@ -73,8 +78,8 @@ FileBrowserComponent::FileBrowserComponent (int flags_,
     }
     else
     {
-        FileListComponent* const list = new FileListComponent (*fileList);
-        fileListComponent = list;
+        auto list = new FileListComponent (*fileList);
+        fileListComponent.reset (list);
         list->setOutlineThickness (1);
 
         if ((flags & canSelectMultipleItems) != 0)
@@ -88,25 +93,35 @@ FileBrowserComponent::FileBrowserComponent (int flags_,
     addAndMakeVisible (currentPathBox);
     currentPathBox.setEditableText (true);
     resetRecentPaths();
-    currentPathBox.addListener (this);
+    currentPathBox.onChange = [this] { updateSelectedPath(); };
 
     addAndMakeVisible (filenameBox);
     filenameBox.setMultiLine (false);
     filenameBox.setSelectAllWhenFocused (true);
     filenameBox.setText (filename, false);
-    filenameBox.addListener (this);
+    filenameBox.onTextChange = [this] { sendListenerChangeMessage(); };
+    filenameBox.onReturnKey  = [this] { changeFilename(); };
+    filenameBox.onFocusLost  = [this]
+    {
+        if (! isSaveMode())
+            selectionChanged();
+    };
+
     filenameBox.setReadOnly ((flags & (filenameBoxIsReadOnly | canSelectMultipleItems)) != 0);
 
     addAndMakeVisible (fileLabel);
     fileLabel.attachToComponent (&filenameBox, true);
 
-    addAndMakeVisible (goUpButton = getLookAndFeel().createFileBrowserGoUpButton());
-    goUpButton->addListener (this);
+    goUpButton.reset (getLookAndFeel().createFileBrowserGoUpButton());
+    addAndMakeVisible (goUpButton.get());
+    goUpButton->onClick = [this] { goUp(); };
     goUpButton->setTooltip (TRANS ("Go up to parent directory"));
 
     if (previewComp != nullptr)
         addAndMakeVisible (previewComp);
 
+    lookAndFeelChanged();
+
     setRoot (currentRoot);
 
     thread.startThread (4);
@@ -116,8 +131,8 @@ FileBrowserComponent::FileBrowserComponent (int flags_,
 
 FileBrowserComponent::~FileBrowserComponent()
 {
-    fileListComponent = nullptr;
-    fileList = nullptr;
+    fileListComponent.reset();
+    fileList.reset();
     thread.stopThread (10000);
 }
 
@@ -140,7 +155,7 @@ bool FileBrowserComponent::isSaveMode() const noexcept
 
 int FileBrowserComponent::getNumSelectedFiles() const noexcept
 {
-    if (chosenFiles.size() == 0 && currentFileIsValid())
+    if (chosenFiles.isEmpty() && currentFileIsValid())
         return 1;
 
     return chosenFiles.size();
@@ -159,7 +174,7 @@ File FileBrowserComponent::getSelectedFile (int index) const noexcept
 
 bool FileBrowserComponent::currentFileIsValid() const
 {
-    const File f (getSelectedFile (0));
+    auto f = getSelectedFile (0);
 
     if (isSaveMode())
         return (flags & canSelectDirectories) != 0 || ! f.isDirectory();
@@ -217,7 +232,7 @@ void FileBrowserComponent::setRoot (const File& newRootDirectory)
         String path (newRootDirectory.getFullPathName());
 
         if (path.isEmpty())
-            path = File::separatorString;
+            path = File::getSeparatorString();
 
         StringArray rootNames, rootPaths;
         getRoots (rootNames, rootPaths);
@@ -243,12 +258,13 @@ void FileBrowserComponent::setRoot (const File& newRootDirectory)
     currentRoot = newRootDirectory;
     fileList->setDirectory (currentRoot, true, true);
 
-    if (FileTreeComponent* tree = dynamic_cast<FileTreeComponent*> (fileListComponent.get()))
+    if (auto* tree = dynamic_cast<FileTreeComponent*> (fileListComponent.get()))
         tree->refresh();
 
-    String currentRootName (currentRoot.getFullPathName());
+    auto currentRootName = currentRoot.getFullPathName();
+
     if (currentRootName.isEmpty())
-        currentRootName = File::separatorString;
+        currentRootName = File::getSeparatorString();
 
     currentPathBox.setText (currentRootName, dontSendNotification);
 
@@ -258,7 +274,7 @@ void FileBrowserComponent::setRoot (const File& newRootDirectory)
     if (callListeners)
     {
         Component::BailOutChecker checker (this);
-        listeners.callChecked (checker, &FileBrowserListener::browserRootChanged, currentRoot);
+        listeners.callChecked (checker, [&] (FileBrowserListener& l) { l.browserRootChanged (currentRoot); });
     }
 }
 
@@ -325,15 +341,26 @@ FilePreviewComponent* FileBrowserComponent::getPreviewComponent() const noexcept
 
 DirectoryContentsDisplayComponent* FileBrowserComponent::getDisplayComponent() const noexcept
 {
-    return fileListComponent;
+    return fileListComponent.get();
 }
 
 //==============================================================================
 void FileBrowserComponent::resized()
 {
     getLookAndFeel()
-        .layoutFileBrowserComponent (*this, fileListComponent, previewComp,
-                                     &currentPathBox, &filenameBox, goUpButton);
+        .layoutFileBrowserComponent (*this, fileListComponent.get(), previewComp,
+                                     &currentPathBox, &filenameBox, goUpButton.get());
+}
+
+//==============================================================================
+void FileBrowserComponent::lookAndFeelChanged()
+{
+    currentPathBox.setColour (ComboBox::backgroundColourId,    findColour (currentPathBoxBackgroundColourId));
+    currentPathBox.setColour (ComboBox::textColourId,          findColour (currentPathBoxTextColourId));
+    currentPathBox.setColour (ComboBox::arrowColourId,         findColour (currentPathBoxArrowColourId));
+
+    filenameBox.setColour (TextEditor::backgroundColourId,     findColour (filenameBoxBackgroundColourId));
+    filenameBox.setColour (TextEditor::textColourId,           findColour (filenameBoxTextColourId));
 }
 
 //==============================================================================
@@ -347,7 +374,7 @@ void FileBrowserComponent::sendListenerChangeMessage()
     // You shouldn't delete the browser when the file gets changed!
     jassert (! checker.shouldBailOut());
 
-    listeners.callChecked (checker, &FileBrowserListener::selectionChanged);
+    listeners.callChecked (checker, [] (FileBrowserListener& l) { l.selectionChanged(); });
 }
 
 void FileBrowserComponent::selectionChanged()
@@ -381,7 +408,7 @@ void FileBrowserComponent::selectionChanged()
 void FileBrowserComponent::fileClicked (const File& f, const MouseEvent& e)
 {
     Component::BailOutChecker checker (this);
-    listeners.callChecked (checker, &FileBrowserListener::fileClicked, f, e);
+    listeners.callChecked (checker, [&] (FileBrowserListener& l) { l.fileClicked (f, e); });
 }
 
 void FileBrowserComponent::fileDoubleClicked (const File& f)
@@ -391,12 +418,12 @@ void FileBrowserComponent::fileDoubleClicked (const File& f)
         setRoot (f);
 
         if ((flags & canSelectDirectories) != 0 && (flags & doNotClearFileNameOnRootChange) == 0)
-            filenameBox.setText (String());
+            filenameBox.setText ({});
     }
     else
     {
         Component::BailOutChecker checker (this);
-        listeners.callChecked (checker, &FileBrowserListener::fileDoubleClicked, f);
+        listeners.callChecked (checker, [&] (FileBrowserListener& l) { l.fileDoubleClicked (f); });
     }
 }
 
@@ -419,16 +446,11 @@ bool FileBrowserComponent::keyPressed (const KeyPress& key)
 }
 
 //==============================================================================
-void FileBrowserComponent::textEditorTextChanged (TextEditor&)
+void FileBrowserComponent::changeFilename()
 {
-    sendListenerChangeMessage();
-}
-
-void FileBrowserComponent::textEditorReturnKeyPressed (TextEditor&)
-{
-    if (filenameBox.getText().containsChar (File::separator))
+    if (filenameBox.getText().containsChar (File::getSeparatorChar()))
     {
-        const File f (currentRoot.getChildFile (filenameBox.getText()));
+        auto f = currentRoot.getChildFile (filenameBox.getText());
 
         if (f.isDirectory())
         {
@@ -436,7 +458,7 @@ void FileBrowserComponent::textEditorReturnKeyPressed (TextEditor&)
             chosenFiles.clear();
 
             if ((flags & doNotClearFileNameOnRootChange) == 0)
-                filenameBox.setText (String());
+                filenameBox.setText ({});
         }
         else
         {
@@ -452,36 +474,21 @@ void FileBrowserComponent::textEditorReturnKeyPressed (TextEditor&)
     }
 }
 
-void FileBrowserComponent::textEditorEscapeKeyPressed (TextEditor&)
-{
-}
-
-void FileBrowserComponent::textEditorFocusLost (TextEditor&)
-{
-    if (! isSaveMode())
-        selectionChanged();
-}
-
 //==============================================================================
-void FileBrowserComponent::buttonClicked (Button*)
+void FileBrowserComponent::updateSelectedPath()
 {
-    goUp();
-}
-
-void FileBrowserComponent::comboBoxChanged (ComboBox*)
-{
-    const String newText (currentPathBox.getText().trim().unquoted());
+    auto newText = currentPathBox.getText().trim().unquoted();
 
     if (newText.isNotEmpty())
     {
-        const int index = currentPathBox.getSelectedId() - 1;
+        auto index = currentPathBox.getSelectedId() - 1;
 
         StringArray rootNames, rootPaths;
         getRoots (rootNames, rootPaths);
 
-        if (rootPaths [index].isNotEmpty())
+        if (rootPaths[index].isNotEmpty())
         {
-            setRoot (File (rootPaths [index]));
+            setRoot (File (rootPaths[index]));
         }
         else
         {
@@ -535,8 +542,8 @@ void FileBrowserComponent::getDefaultRoots (StringArray& rootNames, StringArray&
         rootNames.add (name);
     }
 
-    rootPaths.add (String());
-    rootNames.add (String());
+    rootPaths.add ({});
+    rootNames.add ({});
 
     rootPaths.add (File::getSpecialLocation (File::userDocumentsDirectory).getFullPathName());
     rootNames.add (TRANS("Documents"));
@@ -559,17 +566,11 @@ void FileBrowserComponent::getDefaultRoots (StringArray& rootNames, StringArray&
     rootPaths.add (File::getSpecialLocation (File::userDesktopDirectory).getFullPathName());
     rootNames.add (TRANS("Desktop"));
 
-    rootPaths.add (String());
-    rootNames.add (String());
+    rootPaths.add ({});
+    rootNames.add ({});
 
-    Array<File> volumes;
-    File vol ("/Volumes");
-    vol.findChildFiles (volumes, File::findDirectories, false);
-
-    for (int i = 0; i < volumes.size(); ++i)
+    for (auto& volume : File ("/Volumes").findChildFiles (File::findDirectories, false))
     {
-        const File& volume = volumes.getReference(i);
-
         if (volume.isDirectory() && ! volume.getFileName().startsWithChar ('.'))
         {
             rootPaths.add (volume.getFullPathName());
@@ -604,3 +605,5 @@ void FileBrowserComponent::timerCallback()
             refresh();
     }
 }
+
+} // namespace juce
