@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_FILEBROWSERCOMPONENT_H_INCLUDED
-#define JUCE_FILEBROWSERCOMPONENT_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -38,9 +39,6 @@
 */
 class JUCE_API  FileBrowserComponent  : public Component,
                                         private FileBrowserListener,
-                                        private TextEditorListener,
-                                        private ButtonListener,
-                                        private ComboBoxListener,  // (can't use ComboBox::Listener due to idiotic VC2005 bug)
                                         private FileFilter,
                                         private Timer
 {
@@ -194,6 +192,7 @@ public:
                                                               const String& instructions) = 0;
 
         virtual void drawFileBrowserRow (Graphics&, int width, int height,
+                                         const File& file,
                                          const String& filename,
                                          Image* optionalIcon,
                                          const String& fileSizeDescription,
@@ -213,21 +212,27 @@ public:
                                                  Button* goUpButton) = 0;
     };
 
+    /** A set of colour IDs to use to change the colour of various aspects of the FileBrowserComponent.
+
+        These constants can be used either via the Component::setColour(), or LookAndFeel::setColour()
+        methods.
+
+        @see Component::setColour, Component::findColour, LookAndFeel::setColour, LookAndFeel::findColour
+    */
+    enum ColourIds
+    {
+        currentPathBoxBackgroundColourId    = 0x1000640, /**< The colour to use to fill the background of the current path ComboBox. */
+        currentPathBoxTextColourId          = 0x1000641, /**< The colour to use for the text of the current path ComboBox. */
+        currentPathBoxArrowColourId         = 0x1000642, /**< The colour to use to draw the arrow of the current path ComboBox. */
+        filenameBoxBackgroundColourId       = 0x1000643, /**< The colour to use to fill the background of the filename TextEditor. */
+        filenameBoxTextColourId             = 0x1000644  /**< The colour to use for the text of the filename TextEditor. */
+    };
+
     //==============================================================================
     /** @internal */
     void resized() override;
     /** @internal */
-    void buttonClicked (Button*) override;
-    /** @internal */
-    void comboBoxChanged (ComboBox*) override;
-    /** @internal */
-    void textEditorTextChanged (TextEditor&) override;
-    /** @internal */
-    void textEditorReturnKeyPressed (TextEditor&) override;
-    /** @internal */
-    void textEditorEscapeKeyPressed (TextEditor&) override;
-    /** @internal */
-    void textEditorFocusLost (TextEditor&) override;
+    void lookAndFeelChanged() override;
     /** @internal */
     bool keyPressed (const KeyPress&) override;
     /** @internal */
@@ -280,10 +285,10 @@ private:
     void timerCallback() override;
     void sendListenerChangeMessage();
     bool isFileOrDirSuitable (const File&) const;
+    void updateSelectedPath();
+    void changeFilename();
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (FileBrowserComponent)
 };
 
-
-
-#endif   // JUCE_FILEBROWSERCOMPONENT_H_INCLUDED
+} // namespace juce
