@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_FILECHOOSER_H_INCLUDED
-#define JUCE_FILECHOOSER_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -66,11 +67,32 @@ public:
                                               when the dialog box opens. If this parameter is
                                               set to File(), a sensible default directory will
                                               be used instead.
-        @param filePatternsAllowed            a set of file patterns to specify which files
-                                              can be selected - each pattern should be
-                                              separated by a comma or semi-colon, e.g. "*" or
-                                              "*.jpg;*.gif". An empty string means that all
-                                              files are allowed
+
+                                              Note: on iOS when saving a file, a user will not
+                                              be able to change a file name, so it may be a good
+                                              idea to include at least a valid file name in
+                                              initialFileOrDirectory. When no filename is found,
+                                              "Untitled" will be used.
+
+                                              Also, if you pass an already existing file on iOS,
+                                              that file will be automatically copied to the
+                                              destination chosen by user and if it can be previewed,
+                                              its preview will be presented in the dialog too. You
+                                              will still be able to write into this file copy, since
+                                              its URL will be returned by getURLResult(). This can be
+                                              useful when you want to save e.g. an image, so that
+                                              you can pass a (temporary) file with low quality
+                                              preview and after the user picks the destination,
+                                              you can write a high quality image into the copied
+                                              file. If you create such a temporary file, you need
+                                              to delete it yourself, once it is not needed anymore.
+
+        @param filePatternsAllowed            a set of file patterns to specify which files can be
+                                              selected - each pattern should be separated by a comma or
+                                              semi-colon, e.g. "*" or "*.jpg;*.gif". The native MacOS
+                                              file browser only supports wildcard that specify
+                                              extensions, so "*.jpg" is OK but "myfilename*" will not
+                                              work. An empty string means that all files are allowed
         @param useOSNativeDialogBox           if true, then a native dialog box will be used
                                               if possible; if false, then a Juce-based
                                               browser dialog box will always be used
@@ -158,6 +180,25 @@ public:
     */
     bool showDialog (int flags, FilePreviewComponent* previewComponent);
 
+    /** Use this method to launch the file browser window asynchronously.
+
+        This will create a file browser dialog based on the settings in this
+        structure and will launch it modally, returning immediately.
+
+        You must specify a callback which is called when the file browser is
+        canceled or a file is selected. To abort the file selection, simply
+        delete the FileChooser object.
+
+        You can use the ModalCallbackFunction::create method to wrap a lambda
+        into a modal Callback object.
+
+        You must ensure that the lifetime of the callback object is longer than
+        the lifetime of the file-chooser.
+    */
+    void launchAsync (int flags,
+                      std::function<void (const FileChooser&)>,
+                      FilePreviewComponent* previewComponent = nullptr);
+
     //==============================================================================
     /** Returns the last file that was chosen by one of the browseFor methods.
 
@@ -167,39 +208,120 @@ public:
         Note that the file returned is only valid if the browse method returned true (i.e.
         if the user pressed 'ok' rather than cancelling).
 
+        On mobile platforms, the file browser may return a URL instead of a local file.
+        Therefore, om mobile platforms, you should call getURLResult() instead.
+
         If you're using a multiple-file select, then use the getResults() method instead,
         to obtain the list of all files chosen.
 
-        @see getResults
+        @see getURLResult, getResults
     */
     File getResult() const;
 
     /** Returns a list of all the files that were chosen during the last call to a
         browse method.
 
+        On mobile platforms, the file browser may return a URL instead of a local file.
+        Therefore, om mobile platforms, you should call getURLResults() instead.
+
         This array may be empty if no files were chosen, or can contain multiple entries
         if multiple files were chosen.
 
-        @see getResult
+        @see getURLResults, getResult
     */
-    const Array<File>& getResults() const noexcept      { return results; }
+    Array<File> getResults() const noexcept;
+
+    //==============================================================================
+    /** Returns the last document that was chosen by one of the browseFor methods.
+
+        Use this method if you are using the FileChooser on a mobile platform which
+        may return a URL to a remote document. If a local file is chosen then you can
+        convert this file to a JUCE File class via the URL::getLocalFile method.
+
+        Note: on iOS you must use the returned URL object directly (you are also
+        allowed to copy- or move-construct another URL from the returned URL), rather
+        than just storing the path as a String and then creating a new URL from that
+        String. This is because the returned URL contains internally a security
+        bookmark that is required to access the files pointed by it. Then, once you stop
+        dealing with the file pointed by the URL, you should dispose that URL object,
+        so that the security bookmark can be released by the system (only a limited
+        number of such URLs is allowed).
+
+        @see getResult, URL::getLocalFile
+    */
+    URL getURLResult() const;
+
+    /** Returns a list of all the files that were chosen during the last call to a
+        browse method.
+
+        Use this method if you are using the FileChooser on a mobile platform which
+        may return a URL to a remote document. If a local file is chosen then you can
+        convert this file to a JUCE File class via the URL::getLocalFile method.
+
+        This array may be empty if no files were chosen, or can contain multiple entries
+        if multiple files were chosen.
+
+        Note: on iOS you must use the returned URL object directly (you are also
+        allowed to copy- or move-construct another URL from the returned URL), rather
+        than just storing the path as a String and then creating a new URL from that
+        String. This is because the returned URL contains internally a security
+        bookmark that is required to access the files pointed by it. Then, once you stop
+        dealing with the file pointed by the URL, you should dispose that URL object,
+        so that the security bookmark can be released by the system (only a limited
+        number of such URLs is allowed).
+
+        @see getResults, URL::getLocalFile
+    */
+    const Array<URL>& getURLResults() const noexcept      { return results; }
+
+    //==============================================================================
+    /** Returns if a native filechooser is currently available on this platform.
+
+        Note: On iOS this will only return true if you have iCloud permissions
+        and code-signing enabled in the Projucer and have added iCloud containers
+        to your app in Apple's online developer portal. Additionally, the user must
+        have installed the iCloud app on their device and used the app at leat once.
+    */
+    static bool isPlatformDialogAvailable();
+
+    //==============================================================================
+   #ifndef DOXYGEN
+    class Native;
+   #endif
 
 private:
     //==============================================================================
     String title, filters;
-    const File startingFile;
-    Array<File> results;
+    File startingFile;
+    Array<URL> results;
     const bool useNativeDialogBox;
     const bool treatFilePackagesAsDirs;
+    std::function<void (const FileChooser&)> asyncCallback;
 
-    static void showPlatformDialog (Array<File>& results, const String& title, const File& file,
-                                    const String& filters, bool selectsDirectories, bool selectsFiles,
-                                    bool isSave, bool warnAboutOverwritingExistingFiles, bool selectMultipleFiles,
-                                    bool treatFilePackagesAsDirs, FilePreviewComponent* previewComponent);
-    static bool isPlatformDialogAvailable();
+    //==============================================================================
+    void finished (const Array<URL>&);
+
+    //==============================================================================
+    struct Pimpl
+    {
+        virtual ~Pimpl() {}
+
+        virtual void launch()     = 0;
+        virtual void runModally() = 0;
+    };
+
+    ScopedPointer<Pimpl> pimpl;
+
+    //==============================================================================
+    Pimpl* createPimpl (int, FilePreviewComponent*);
+    static Pimpl* showPlatformDialog (FileChooser&, int,
+                                      FilePreviewComponent*);
+
+    class NonNative;
+    friend class NonNative;
+    friend class Native;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (FileChooser)
 };
 
-
-#endif   // JUCE_FILECHOOSER_H_INCLUDED
+} // namespace juce
