@@ -2,41 +2,46 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 Image juce_createIconForFile (const File& file);
 
 
 //==============================================================================
 FileListComponent::FileListComponent (DirectoryContentsList& listToShow)
-    : ListBox (String(), nullptr),
+    : ListBox ({}, nullptr),
       DirectoryContentsDisplayComponent (listToShow)
 {
     setModel (this);
-    fileList.addChangeListener (this);
+    directoryContentsList.addChangeListener (this);
 }
 
 FileListComponent::~FileListComponent()
 {
-    fileList.removeChangeListener (this);
+    directoryContentsList.removeChangeListener (this);
 }
 
 int FileListComponent::getNumSelectedFiles() const
@@ -46,7 +51,7 @@ int FileListComponent::getNumSelectedFiles() const
 
 File FileListComponent::getSelectedFile (int index) const
 {
-    return fileList.getFile (getSelectedRow (index));
+    return directoryContentsList.getFile (getSelectedRow (index));
 }
 
 void FileListComponent::deselectAllFiles()
@@ -56,14 +61,14 @@ void FileListComponent::deselectAllFiles()
 
 void FileListComponent::scrollToTop()
 {
-    getVerticalScrollBar()->setCurrentRangeStart (0);
+    getVerticalScrollBar().setCurrentRangeStart (0);
 }
 
 void FileListComponent::setSelectedFile (const File& f)
 {
-    for (int i = fileList.getNumFiles(); --i >= 0;)
+    for (int i = directoryContentsList.getNumFiles(); --i >= 0;)
     {
-        if (fileList.getFile(i) == f)
+        if (directoryContentsList.getFile(i) == f)
         {
             selectRow (i);
             return;
@@ -78,9 +83,9 @@ void FileListComponent::changeListenerCallback (ChangeBroadcaster*)
 {
     updateContent();
 
-    if (lastDirectory != fileList.getDirectory())
+    if (lastDirectory != directoryContentsList.getDirectory())
     {
-        lastDirectory = fileList.getDirectory();
+        lastDirectory = directoryContentsList.getDirectory();
         deselectAllRows();
     }
 }
@@ -92,7 +97,7 @@ class FileListComponent::ItemComponent  : public Component,
 {
 public:
     ItemComponent (FileListComponent& fc, TimeSliceThread& t)
-        : owner (fc), thread (t), index (0), highlighted (false)
+        : owner (fc), thread (t)
     {
     }
 
@@ -105,7 +110,7 @@ public:
     void paint (Graphics& g) override
     {
         getLookAndFeel().drawFileBrowserRow (g, getWidth(), getHeight(),
-                                             file.getFileName(),
+                                             file, file.getFileName(),
                                              &icon, fileSize, modTime,
                                              isDirectory, highlighted,
                                              index, owner);
@@ -122,17 +127,15 @@ public:
         owner.sendDoubleClickMessage (file);
     }
 
-    void update (const File& root,
-                 const DirectoryContentsList::FileInfo* const fileInfo,
-                 const int index_,
-                 const bool highlighted_)
+    void update (const File& root, const DirectoryContentsList::FileInfo* fileInfo,
+                 int newIndex, bool nowHighlighted)
     {
         thread.removeTimeSliceClient (this);
 
-        if (highlighted_ != highlighted || index_ != index)
+        if (nowHighlighted != highlighted || newIndex != index)
         {
-            index = index_;
-            highlighted = highlighted_;
+            index = newIndex;
+            highlighted = nowHighlighted;
             repaint();
         }
 
@@ -186,15 +189,15 @@ private:
     File file;
     String fileSize, modTime;
     Image icon;
-    int index;
-    bool highlighted, isDirectory;
+    int index = 0;
+    bool highlighted = false, isDirectory = false;
 
     void updateIcon (const bool onlyUpdateIfCached)
     {
         if (icon.isNull())
         {
-            const int hashCode = (file.getFullPathName() + "_iconCacheSalt").hashCode();
-            Image im (ImageCache::getFromHashCode (hashCode));
+            auto hashCode = (file.getFullPathName() + "_iconCacheSalt").hashCode();
+            auto im = ImageCache::getFromHashCode (hashCode);
 
             if (im.isNull() && ! onlyUpdateIfCached)
             {
@@ -218,7 +221,7 @@ private:
 //==============================================================================
 int FileListComponent::getNumRows()
 {
-    return fileList.getNumFiles();
+    return directoryContentsList.getNumFiles();
 }
 
 void FileListComponent::paintListBoxItem (int, Graphics&, int, int, bool)
@@ -229,14 +232,14 @@ Component* FileListComponent::refreshComponentForRow (int row, bool isSelected,
 {
     jassert (existingComponentToUpdate == nullptr || dynamic_cast<ItemComponent*> (existingComponentToUpdate) != nullptr);
 
-    ItemComponent* comp = static_cast<ItemComponent*> (existingComponentToUpdate);
+    auto comp = static_cast<ItemComponent*> (existingComponentToUpdate);
 
     if (comp == nullptr)
-        comp = new ItemComponent (*this, fileList.getTimeSliceThread());
+        comp = new ItemComponent (*this, directoryContentsList.getTimeSliceThread());
 
     DirectoryContentsList::FileInfo fileInfo;
-    comp->update (fileList.getDirectory(),
-                  fileList.getFileInfo (row, fileInfo) ? &fileInfo : nullptr,
+    comp->update (directoryContentsList.getDirectory(),
+                  directoryContentsList.getFileInfo (row, fileInfo) ? &fileInfo : nullptr,
                   row, isSelected);
 
     return comp;
@@ -253,5 +256,7 @@ void FileListComponent::deleteKeyPressed (int /*currentSelectedRow*/)
 
 void FileListComponent::returnKeyPressed (int currentSelectedRow)
 {
-    sendDoubleClickMessage (fileList.getFile (currentSelectedRow));
+    sendDoubleClickMessage (directoryContentsList.getFile (currentSelectedRow));
 }
+
+} // namespace juce
