@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -51,6 +53,13 @@
   #import <Carbon/Carbon.h> // still needed for SetSystemUIMode()
  #endif
 
+#elif JUCE_IOS
+ #if JUCE_PUSH_NOTIFICATIONS && defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0
+  #import <UserNotifications/UserNotifications.h>
+ #endif
+
+ #import <UIKit/UIActivityViewController.h>
+
 //==============================================================================
 #elif JUCE_WINDOWS
  #include <windowsx.h>
@@ -74,10 +83,6 @@
   #endif
  #endif
 
- #if JUCE_QUICKTIME && JUCE_MSVC && ! JUCE_DONT_AUTOLINK_TO_WIN32_LIBRARIES
-  #pragma comment (lib, "QTMLClient.lib")
- #endif
-
  #if JUCE_DIRECT2D && JUCE_MSVC && ! JUCE_DONT_AUTOLINK_TO_WIN32_LIBRARIES
   #pragma comment (lib, "Dwrite.lib")
   #pragma comment (lib, "D2d1.lib")
@@ -130,17 +135,20 @@
  #undef KeyPress
 #endif
 
-//==============================================================================
-namespace juce
-{
+#include <map>
+#include <set>
 
+//==============================================================================
 #define ASSERT_MESSAGE_MANAGER_IS_LOCKED \
     jassert (MessageManager::getInstance()->currentThreadHasLockedMessageManager());
 
 #define ASSERT_MESSAGE_MANAGER_IS_LOCKED_OR_OFFSCREEN \
     jassert (MessageManager::getInstance()->currentThreadHasLockedMessageManager() || getPeer() == nullptr);
 
-extern bool juce_areThereAnyAlwaysOnTopWindows();
+namespace juce
+{
+    extern bool juce_areThereAnyAlwaysOnTopWindows();
+}
 
 #include "components/juce_Component.cpp"
 #include "components/juce_ComponentListener.cpp"
@@ -185,6 +193,7 @@ extern bool juce_areThereAnyAlwaysOnTopWindows();
 #include "filebrowser/juce_FileSearchPathListComponent.cpp"
 #include "filebrowser/juce_FileTreeComponent.cpp"
 #include "filebrowser/juce_ImagePreviewComponent.cpp"
+#include "filebrowser/juce_ContentSharer.cpp"
 #include "layout/juce_ComponentAnimator.cpp"
 #include "layout/juce_ComponentBoundsConstrainer.cpp"
 #include "layout/juce_ComponentBuilder.cpp"
@@ -196,6 +205,7 @@ extern bool juce_areThereAnyAlwaysOnTopWindows();
 #include "layout/juce_ResizableCornerComponent.cpp"
 #include "layout/juce_ResizableEdgeComponent.cpp"
 #include "layout/juce_ScrollBar.cpp"
+#include "layout/juce_SidePanel.cpp"
 #include "layout/juce_StretchableLayoutManager.cpp"
 #include "layout/juce_StretchableLayoutResizerBar.cpp"
 #include "layout/juce_StretchableObjectResizer.cpp"
@@ -206,7 +216,9 @@ extern bool juce_areThereAnyAlwaysOnTopWindows();
 #include "lookandfeel/juce_LookAndFeel_V2.cpp"
 #include "lookandfeel/juce_LookAndFeel_V1.cpp"
 #include "lookandfeel/juce_LookAndFeel_V3.cpp"
+#include "lookandfeel/juce_LookAndFeel_V4.cpp"
 #include "menus/juce_MenuBarComponent.cpp"
+#include "menus/juce_BurgerMenuComponent.cpp"
 #include "menus/juce_MenuBarModel.cpp"
 #include "menus/juce_PopupMenu.cpp"
 #include "positioning/juce_MarkerList.cpp"
@@ -252,10 +264,18 @@ extern bool juce_areThereAnyAlwaysOnTopWindows();
 #include "application/juce_Application.cpp"
 #include "misc/juce_BubbleComponent.cpp"
 #include "misc/juce_DropShadower.cpp"
+#include "misc/juce_JUCESplashScreen.cpp"
 
 // these classes are C++11-only
-#if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS && JUCE_COMPILER_SUPPORTS_INITIALIZER_LISTS && JUCE_COMPILER_SUPPORTS_LAMBDAS
+#if JUCE_COMPILER_SUPPORTS_INITIALIZER_LISTS
  #include "layout/juce_FlexBox.cpp"
+ #if JUCE_HAS_CONSTEXPR
+  #include "layout/juce_GridItem.cpp"
+  #include "layout/juce_Grid.cpp"
+  #if JUCE_UNIT_TESTS
+   #include "layout/juce_GridUnitTests.cpp"
+  #endif
+ #endif
 #endif
 
 #if JUCE_IOS || JUCE_WINDOWS
@@ -264,17 +284,28 @@ extern bool juce_areThereAnyAlwaysOnTopWindows();
 
 #if JUCE_MAC || JUCE_IOS
 
+ #if JUCE_CLANG
+  #pragma clang diagnostic push
+  #pragma clang diagnostic ignored "-Wundeclared-selector"
+ #endif
+
  #if JUCE_IOS
   #include "native/juce_ios_UIViewComponentPeer.mm"
   #include "native/juce_ios_Windowing.mm"
+  #include "native/juce_ios_FileChooser.mm"
+  #include "native/juce_ios_ContentSharer.cpp"
  #else
   #include "native/juce_mac_NSViewComponentPeer.mm"
   #include "native/juce_mac_Windowing.mm"
   #include "native/juce_mac_MainMenu.mm"
+  #include "native/juce_mac_FileChooser.mm"
+ #endif
+
+ #if JUCE_CLANG
+  #pragma clang diagnostic pop
  #endif
 
  #include "native/juce_mac_MouseCursor.mm"
- #include "native/juce_mac_FileChooser.mm"
 
 #elif JUCE_WINDOWS
  #include "native/juce_win32_Windowing.cpp"
@@ -282,14 +313,15 @@ extern bool juce_areThereAnyAlwaysOnTopWindows();
  #include "native/juce_win32_FileChooser.cpp"
 
 #elif JUCE_LINUX
- #include "native/juce_linux_Clipboard.cpp"
- #include "native/juce_linux_Windowing.cpp"
+ #include "native/juce_linux_X11.cpp"
+ #include "native/juce_linux_X11_Clipboard.cpp"
+ #include "native/juce_linux_X11_Windowing.cpp"
  #include "native/juce_linux_FileChooser.cpp"
 
 #elif JUCE_ANDROID
  #include "native/juce_android_Windowing.cpp"
+ #include "native/juce_common_MimeTypes.cpp"
  #include "native/juce_android_FileChooser.cpp"
+ #include "native/juce_android_ContentSharer.cpp"
 
 #endif
-
-}
