@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 MultiDocumentPanelWindow::MultiDocumentPanelWindow (Colour backgroundColour)
     : DocumentWindow (String(), backgroundColour,
                       DocumentWindow::maximiseButton | DocumentWindow::closeButton, false)
@@ -35,7 +40,7 @@ MultiDocumentPanelWindow::~MultiDocumentPanelWindow()
 //==============================================================================
 void MultiDocumentPanelWindow::maximiseButtonPressed()
 {
-    if (MultiDocumentPanel* const owner = getOwner())
+    if (auto* owner = getOwner())
         owner->setLayoutMode (MultiDocumentPanel::MaximisedWindowsWithTabs);
     else
         jassertfalse; // these windows are only designed to be used inside a MultiDocumentPanel!
@@ -43,7 +48,7 @@ void MultiDocumentPanelWindow::maximiseButtonPressed()
 
 void MultiDocumentPanelWindow::closeButtonPressed()
 {
-    if (MultiDocumentPanel* const owner = getOwner())
+    if (auto* owner = getOwner())
         owner->closeDocument (getContentComponent(), true);
     else
         jassertfalse; // these windows are only designed to be used inside a MultiDocumentPanel!
@@ -63,7 +68,7 @@ void MultiDocumentPanelWindow::broughtToFront()
 
 void MultiDocumentPanelWindow::updateOrder()
 {
-    if (MultiDocumentPanel* const owner = getOwner())
+    if (auto* owner = getOwner())
         owner->updateOrder();
 }
 
@@ -72,19 +77,14 @@ MultiDocumentPanel* MultiDocumentPanelWindow::getOwner() const noexcept
     return findParentComponentOfClass<MultiDocumentPanel>();
 }
 
-
 //==============================================================================
-class MultiDocumentPanel::TabbedComponentInternal   : public TabbedComponent
+struct MultiDocumentPanel::TabbedComponentInternal   : public TabbedComponent
 {
-public:
-    TabbedComponentInternal()
-        : TabbedComponent (TabbedButtonBar::TabsAtTop)
-    {
-    }
+    TabbedComponentInternal() : TabbedComponent (TabbedButtonBar::TabsAtTop) {}
 
-    void currentTabChanged (int, const String&)
+    void currentTabChanged (int, const String&) override
     {
-        if (MultiDocumentPanel* const owner = findParentComponentOfClass<MultiDocumentPanel>())
+        if (auto* owner = findParentComponentOfClass<MultiDocumentPanel>())
             owner->updateOrder();
     }
 };
@@ -92,10 +92,6 @@ public:
 
 //==============================================================================
 MultiDocumentPanel::MultiDocumentPanel()
-    : mode (MaximisedWindowsWithTabs),
-      backgroundColour (Colours::lightblue),
-      maximumNumDocuments (0),
-      numDocsBeforeTabsUsed (0)
 {
     setOpaque (true);
 }
@@ -116,7 +112,7 @@ namespace MultiDocHelpers
 
 bool MultiDocumentPanel::closeAllDocuments (const bool checkItsOkToCloseFirst)
 {
-    while (components.size() > 0)
+    while (! components.isEmpty())
         if (! closeDocument (components.getLast(), checkItsOkToCloseFirst))
             return false;
 
@@ -130,24 +126,24 @@ MultiDocumentPanelWindow* MultiDocumentPanel::createNewDocumentWindow()
 
 void MultiDocumentPanel::addWindow (Component* component)
 {
-    MultiDocumentPanelWindow* const dw = createNewDocumentWindow();
+    auto* dw = createNewDocumentWindow();
 
     dw->setResizable (true, false);
     dw->setContentNonOwned (component, true);
     dw->setName (component->getName());
 
-    const var bkg (component->getProperties() ["mdiDocumentBkg_"]);
+    auto bkg = component->getProperties() ["mdiDocumentBkg_"];
     dw->setBackgroundColour (bkg.isVoid() ? backgroundColour : Colour ((uint32) static_cast<int> (bkg)));
 
     int x = 4;
 
-    if (Component* const topComp = getChildComponent (getNumChildComponents() - 1))
+    if (auto* topComp = getChildren().getLast())
         if (topComp->getX() == x && topComp->getY() == x)
             x += 16;
 
     dw->setTopLeftPosition (x, x);
 
-    const var pos (component->getProperties() ["mdiDocumentPos_"]);
+    auto pos = component->getProperties() ["mdiDocumentPos_"];
     if (pos.toString().isNotEmpty())
         dw->restoreWindowStateFromString (pos.toString());
 
@@ -196,12 +192,13 @@ bool MultiDocumentPanel::addDocument (Component* const component,
     {
         if (tabComponent == nullptr && components.size() > numDocsBeforeTabsUsed)
         {
-            addAndMakeVisible (tabComponent = new TabbedComponentInternal());
+            tabComponent.reset (new TabbedComponentInternal());
+            addAndMakeVisible (tabComponent.get());
 
-            Array <Component*> temp (components);
+            auto temp = components;
 
-            for (int i = 0; i < temp.size(); ++i)
-                tabComponent->addTab (temp[i]->getName(), docColour, temp[i], false);
+            for (auto& c : temp)
+                tabComponent->addTab (c->getName(), docColour, c, false);
 
             resized();
         }
@@ -237,9 +234,9 @@ bool MultiDocumentPanel::closeDocument (Component* component,
 
         if (mode == FloatingWindows)
         {
-            for (int i = getNumChildComponents(); --i >= 0;)
+            for (auto* child : getChildren())
             {
-                if (MultiDocumentPanelWindow* const dw = dynamic_cast<MultiDocumentPanelWindow*> (getChildComponent (i)))
+                if (auto* dw = dynamic_cast<MultiDocumentPanelWindow*> (child))
                 {
                     if (dw->getContentComponent() == component)
                     {
@@ -286,7 +283,7 @@ bool MultiDocumentPanel::closeDocument (Component* component,
                 delete component;
 
             if (tabComponent != nullptr && tabComponent->getNumTabs() <= numDocsBeforeTabsUsed)
-                tabComponent = nullptr;
+                tabComponent.reset();
 
             components.removeFirstMatchingValue (component);
 
@@ -297,7 +294,7 @@ bool MultiDocumentPanel::closeDocument (Component* component,
         resized();
 
         // This ensures that the active tab is painted properly when a tab is closed!
-        if (Component* activeComponent = getActiveDocument())
+        if (auto* activeComponent = getActiveDocument())
             setActiveDocument (activeComponent);
 
         activeDocumentChanged();
@@ -324,8 +321,8 @@ Component* MultiDocumentPanel::getActiveDocument() const noexcept
 {
     if (mode == FloatingWindows)
     {
-        for (int i = getNumChildComponents(); --i >= 0;)
-            if (MultiDocumentPanelWindow* const dw = dynamic_cast<MultiDocumentPanelWindow*> (getChildComponent (i)))
+        for (auto* child : getChildren())
+            if (auto* dw = dynamic_cast<MultiDocumentPanelWindow*> (child))
                 if (dw->isActiveWindow())
                     return dw->getContentComponent();
     }
@@ -391,7 +388,7 @@ void MultiDocumentPanel::setLayoutMode (const LayoutMode newLayoutMode)
 
         if (mode == FloatingWindows)
         {
-            tabComponent = nullptr;
+            tabComponent.reset();
         }
         else
         {
@@ -409,17 +406,13 @@ void MultiDocumentPanel::setLayoutMode (const LayoutMode newLayoutMode)
 
         resized();
 
-        const Array <Component*> tempComps (components);
+        auto tempComps = components;
         components.clear();
 
-        for (int i = 0; i < tempComps.size(); ++i)
-        {
-            Component* const c = tempComps.getUnchecked(i);
-
+        for (auto* c : tempComps)
             addDocument (c,
                          Colour ((uint32) static_cast<int> (c->getProperties().getWithDefault ("mdiDocumentBkg_", (int) Colours::white.getARGB()))),
                          MultiDocHelpers::shouldDeleteComp (c));
-        }
     }
 }
 
@@ -443,8 +436,8 @@ void MultiDocumentPanel::resized()
 {
     if (mode == MaximisedWindowsWithTabs || components.size() == numDocsBeforeTabsUsed)
     {
-        for (int i = getNumChildComponents(); --i >= 0;)
-            getChildComponent (i)->setBounds (getLocalBounds());
+        for (auto* child : getChildren())
+            child->setBounds (getLocalBounds());
     }
 
     setWantsKeyboardFocus (components.size() == 0);
@@ -454,8 +447,8 @@ Component* MultiDocumentPanel::getContainerComp (Component* c) const
 {
     if (mode == FloatingWindows)
     {
-        for (int i = 0; i < getNumChildComponents(); ++i)
-            if (MultiDocumentPanelWindow* const dw = dynamic_cast<MultiDocumentPanelWindow*> (getChildComponent (i)))
+        for (auto* child : getChildren())
+            if (auto* dw = dynamic_cast<MultiDocumentPanelWindow*> (child))
                 if (dw->getContentComponent() == c)
                     return dw;
     }
@@ -467,8 +460,8 @@ void MultiDocumentPanel::componentNameChanged (Component&)
 {
     if (mode == FloatingWindows)
     {
-        for (int i = 0; i < getNumChildComponents(); ++i)
-            if (MultiDocumentPanelWindow* const dw = dynamic_cast<MultiDocumentPanelWindow*> (getChildComponent (i)))
+        for (auto* child : getChildren())
+            if (auto* dw = dynamic_cast<MultiDocumentPanelWindow*> (child))
                 dw->setName (dw->getContentComponent()->getName());
     }
     else if (tabComponent != nullptr)
@@ -480,21 +473,21 @@ void MultiDocumentPanel::componentNameChanged (Component&)
 
 void MultiDocumentPanel::updateOrder()
 {
-    const Array <Component*> oldList (components);
+    auto oldList = components;
 
     if (mode == FloatingWindows)
     {
         components.clear();
 
-        for (int i = 0; i < getNumChildComponents(); ++i)
-            if (MultiDocumentPanelWindow* const dw = dynamic_cast<MultiDocumentPanelWindow*> (getChildComponent (i)))
+        for (auto* child : getChildren())
+            if (auto* dw = dynamic_cast<MultiDocumentPanelWindow*> (child))
                 components.add (dw->getContentComponent());
     }
     else
     {
         if (tabComponent != nullptr)
         {
-            if (Component* const current = tabComponent->getCurrentContentComponent())
+            if (auto* current = tabComponent->getCurrentContentComponent())
             {
                 components.removeFirstMatchingValue (current);
                 components.add (current);
@@ -505,3 +498,5 @@ void MultiDocumentPanel::updateOrder()
     if (components != oldList)
         activeDocumentChanged();
 }
+
+} // namespace juce
