@@ -2,39 +2,32 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-Viewport::Viewport (const String& name)
-  : Component (name),
-    scrollBarThickness (0),
-    singleStepX (16),
-    singleStepY (16),
-    showHScrollbar (true),
-    showVScrollbar (true),
-    deleteContent (true),
-    customScrollBarThickness (false),
-    allowScrollingWithoutScrollbarV (false),
-    allowScrollingWithoutScrollbarH (false),
-    verticalScrollBar (true),
-    horizontalScrollBar (false)
+namespace juce
+{
+
+Viewport::Viewport (const String& name)  : Component (name)
 {
     // content holder is used to clip the contents so they don't overlap the scrollbars
     addAndMakeVisible (contentHolder);
@@ -42,14 +35,14 @@ Viewport::Viewport (const String& name)
 
     scrollBarThickness = getLookAndFeel().getDefaultScrollbarWidth();
 
-    addChildComponent (verticalScrollBar);
-    addChildComponent (horizontalScrollBar);
-
-    verticalScrollBar.addListener (this);
-    horizontalScrollBar.addListener (this);
-
     setInterceptsMouseClicks (false, true);
     setWantsKeyboardFocus (true);
+
+  #if JUCE_ANDROID || JUCE_IOS
+    setScrollOnDragEnabled (true);
+  #endif
+
+    recreateScrollbars();
 }
 
 Viewport::~Viewport()
@@ -104,24 +97,44 @@ void Viewport::setViewedComponent (Component* const newViewedComponent, const bo
     }
 }
 
-int Viewport::getMaximumVisibleWidth() const    { return contentHolder.getWidth(); }
-int Viewport::getMaximumVisibleHeight() const   { return contentHolder.getHeight(); }
+void Viewport::recreateScrollbars()
+{
+    verticalScrollBar.reset();
+    horizontalScrollBar.reset();
+
+    verticalScrollBar  .reset (createScrollBarComponent (true));
+    horizontalScrollBar.reset (createScrollBarComponent (false));
+
+    addChildComponent (verticalScrollBar.get());
+    addChildComponent (horizontalScrollBar.get());
+
+    getVerticalScrollBar().addListener (this);
+    getHorizontalScrollBar().addListener (this);
+
+    resized();
+}
+
+int Viewport::getMaximumVisibleWidth() const            { return contentHolder.getWidth(); }
+int Viewport::getMaximumVisibleHeight() const           { return contentHolder.getHeight(); }
+
+bool Viewport::canScrollVertically() const noexcept     { return contentComp->getY() < 0 || contentComp->getBottom() > getHeight(); }
+bool Viewport::canScrollHorizontally() const noexcept   { return contentComp->getX() < 0 || contentComp->getRight()  > getWidth(); }
 
 Point<int> Viewport::viewportPosToCompPos (Point<int> pos) const
 {
     jassert (contentComp != nullptr);
 
-    Rectangle<int> contentBounds = contentHolder.getLocalArea (contentComp, contentComp->getLocalBounds());
+    auto contentBounds = contentHolder.getLocalArea (contentComp, contentComp->getLocalBounds());
+
     Point<int> p (jmax (jmin (0, contentHolder.getWidth()  - contentBounds.getWidth()),  jmin (0, -(pos.x))),
                   jmax (jmin (0, contentHolder.getHeight() - contentBounds.getHeight()), jmin (0, -(pos.y))));
 
-
     return p.transformedBy (contentComp->getTransform().inverted());
 }
 
 void Viewport::setViewPosition (const int xPixelsOffset, const int yPixelsOffset)
 {
-    setViewPosition (Point<int> (xPixelsOffset, yPixelsOffset));
+    setViewPosition ({ xPixelsOffset, yPixelsOffset });
 }
 
 void Viewport::setViewPosition (Point<int> newPosition)
@@ -143,7 +156,7 @@ bool Viewport::autoScroll (const int mouseX, const int mouseY, const int activeB
     {
         int dx = 0, dy = 0;
 
-        if (horizontalScrollBar.isVisible() || contentComp->getX() < 0 || contentComp->getRight() > getWidth())
+        if (getHorizontalScrollBar().isVisible() || canScrollHorizontally())
         {
             if (mouseX < activeBorderThickness)
                 dx = activeBorderThickness - mouseX;
@@ -156,7 +169,7 @@ bool Viewport::autoScroll (const int mouseX, const int mouseY, const int activeB
                 dx = jmin (dx, maximumSpeed, -contentComp->getX());
         }
 
-        if (verticalScrollBar.isVisible() || contentComp->getY() < 0 || contentComp->getBottom() > getHeight())
+        if (getVerticalScrollBar().isVisible() || canScrollVertically())
         {
             if (mouseY < activeBorderThickness)
                 dy = activeBorderThickness - mouseY;
@@ -192,12 +205,13 @@ typedef AnimatedPosition<AnimatedPositionBehaviours::ContinuousWithMomentum> Vie
 struct Viewport::DragToScrollListener   : private MouseListener,
                                           private ViewportDragPosition::Listener
 {
-    DragToScrollListener (Viewport& v)
-        : viewport (v), numTouches (0), isDragging (false)
+    DragToScrollListener (Viewport& v)  : viewport (v)
     {
         viewport.contentHolder.addMouseListener (this, true);
         offsetX.addListener (this);
         offsetY.addListener (this);
+        offsetX.behaviour.setMinimumVelocity (60);
+        offsetY.behaviour.setMinimumVelocity (60);
     }
 
     ~DragToScrollListener()
@@ -213,14 +227,16 @@ struct Viewport::DragToScrollListener   : private MouseListener,
 
     void mouseDown (const MouseEvent&) override
     {
+        offsetX.setPosition (offsetX.getPosition());
+        offsetY.setPosition (offsetY.getPosition());
         ++numTouches;
     }
 
     void mouseDrag (const MouseEvent& e) override
     {
-        if (numTouches == 1)
+        if (numTouches == 1 && ! doesMouseEventComponentBlockViewportDrag (e.eventComponent))
         {
-            Point<float> totalOffset = e.getOffsetFromDragStart().toFloat();
+            auto totalOffset = e.getOffsetFromDragStart().toFloat();
 
             if (! isDragging && totalOffset.getDistanceFromOrigin() > 8.0f)
             {
@@ -243,21 +259,29 @@ struct Viewport::DragToScrollListener   : private MouseListener,
 
     void mouseUp (const MouseEvent&) override
     {
-        if (--numTouches == 0)
+        if (--numTouches <= 0)
         {
             offsetX.endDrag();
             offsetY.endDrag();
             isDragging = false;
+            numTouches = 0;
         }
+    }
 
-        jassert (numTouches >= 0);
+    bool doesMouseEventComponentBlockViewportDrag (const Component* eventComp)
+    {
+        for (auto c = eventComp; c != nullptr && c != &viewport; c = c->getParentComponent())
+            if (c->getViewportIgnoreDragFlag())
+                return true;
+
+        return false;
     }
 
     Viewport& viewport;
     ViewportDragPosition offsetX, offsetY;
     Point<int> originalViewPos;
-    int numTouches;
-    bool isDragging;
+    int numTouches = 0;
+    bool isDragging = false;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (DragToScrollListener)
 };
@@ -267,9 +291,9 @@ void Viewport::setScrollOnDragEnabled (bool shouldScrollOnDrag)
     if (isScrollOnDragEnabled() != shouldScrollOnDrag)
     {
         if (shouldScrollOnDrag)
-            dragToScrollListener = new DragToScrollListener (*this);
+            dragToScrollListener.reset (new DragToScrollListener (*this));
         else
-            dragToScrollListener = nullptr;
+            dragToScrollListener.reset();
     }
 }
 
@@ -301,7 +325,7 @@ void Viewport::resized()
 //==============================================================================
 void Viewport::updateVisibleArea()
 {
-    const int scrollbarWidth = getScrollBarThickness();
+    auto scrollbarWidth = getScrollBarThickness();
     const bool canShowAnyBars = getWidth() > scrollbarWidth && getHeight() > scrollbarWidth;
     const bool canShowHBar = showHScrollbar && canShowAnyBars;
     const bool canShowVBar = showVScrollbar && canShowAnyBars;
@@ -311,8 +335,8 @@ void Viewport::updateVisibleArea()
 
     for (int i = 3; --i >= 0;)
     {
-        hBarVisible = canShowHBar && ! horizontalScrollBar.autoHides();
-        vBarVisible = canShowVBar && ! verticalScrollBar.autoHides();
+        hBarVisible = canShowHBar && ! getHorizontalScrollBar().autoHides();
+        vBarVisible = canShowVBar && ! getVerticalScrollBar().autoHides();
         contentArea = getLocalBounds();
 
         if (contentComp != nullptr && ! contentArea.contains (contentComp->getBounds()))
@@ -342,7 +366,7 @@ void Viewport::updateVisibleArea()
             break;
         }
 
-        const Rectangle<int> oldContentBounds (contentComp->getBounds());
+        auto oldContentBounds = contentComp->getBounds();
         contentHolder.setBounds (contentArea);
 
         // If the content has changed its size, that might affect our scrollbars, so go round again and re-caclulate..
@@ -354,33 +378,36 @@ void Viewport::updateVisibleArea()
     if (contentComp != nullptr)
         contentBounds = contentHolder.getLocalArea (contentComp, contentComp->getLocalBounds());
 
-    Point<int> visibleOrigin (-contentBounds.getPosition());
+    auto visibleOrigin = -contentBounds.getPosition();
 
-    horizontalScrollBar.setBounds (0, contentArea.getHeight(), contentArea.getWidth(), scrollbarWidth);
-    horizontalScrollBar.setRangeLimits (0.0, contentBounds.getWidth());
-    horizontalScrollBar.setCurrentRange (visibleOrigin.x, contentArea.getWidth());
-    horizontalScrollBar.setSingleStepSize (singleStepX);
-    horizontalScrollBar.cancelPendingUpdate();
+    auto& hbar = getHorizontalScrollBar();
+    auto& vbar = getVerticalScrollBar();
+
+    hbar.setBounds (0, contentArea.getHeight(), contentArea.getWidth(), scrollbarWidth);
+    hbar.setRangeLimits (0.0, contentBounds.getWidth());
+    hbar.setCurrentRange (visibleOrigin.x, contentArea.getWidth());
+    hbar.setSingleStepSize (singleStepX);
+    hbar.cancelPendingUpdate();
 
     if (canShowHBar && ! hBarVisible)
         visibleOrigin.setX (0);
 
-    verticalScrollBar.setBounds (contentArea.getWidth(), 0, scrollbarWidth, contentArea.getHeight());
-    verticalScrollBar.setRangeLimits (0.0, contentBounds.getHeight());
-    verticalScrollBar.setCurrentRange (visibleOrigin.y, contentArea.getHeight());
-    verticalScrollBar.setSingleStepSize (singleStepY);
-    verticalScrollBar.cancelPendingUpdate();
+    vbar.setBounds (contentArea.getWidth(), 0, scrollbarWidth, contentArea.getHeight());
+    vbar.setRangeLimits (0.0, contentBounds.getHeight());
+    vbar.setCurrentRange (visibleOrigin.y, contentArea.getHeight());
+    vbar.setSingleStepSize (singleStepY);
+    vbar.cancelPendingUpdate();
 
     if (canShowVBar && ! vBarVisible)
         visibleOrigin.setY (0);
 
     // Force the visibility *after* setting the ranges to avoid flicker caused by edge conditions in the numbers.
-    horizontalScrollBar.setVisible (hBarVisible);
-    verticalScrollBar.setVisible (vBarVisible);
+    hbar.setVisible (hBarVisible);
+    vbar.setVisible (vBarVisible);
 
     if (contentComp != nullptr)
     {
-        const Point<int> newContentCompPos (viewportPosToCompPos (visibleOrigin));
+        auto newContentCompPos = viewportPosToCompPos (visibleOrigin);
 
         if (contentComp->getBounds().getPosition() != newContentCompPos)
         {
@@ -399,8 +426,8 @@ void Viewport::updateVisibleArea()
         visibleAreaChanged (visibleArea);
     }
 
-    horizontalScrollBar.handleUpdateNowIfNeeded();
-    verticalScrollBar.handleUpdateNowIfNeeded();
+    hbar.handleUpdateNowIfNeeded();
+    vbar.handleUpdateNowIfNeeded();
 }
 
 //==============================================================================
@@ -463,13 +490,13 @@ int Viewport::getScrollBarThickness() const
 
 void Viewport::scrollBarMoved (ScrollBar* scrollBarThatHasMoved, double newRangeStart)
 {
-    const int newRangeStartInt = roundToInt (newRangeStart);
+    auto newRangeStartInt = roundToInt (newRangeStart);
 
-    if (scrollBarThatHasMoved == &horizontalScrollBar)
+    if (scrollBarThatHasMoved == horizontalScrollBar.get())
     {
         setViewPosition (newRangeStartInt, getViewPositionY());
     }
-    else if (scrollBarThatHasMoved == &verticalScrollBar)
+    else if (scrollBarThatHasMoved == verticalScrollBar.get())
     {
         setViewPosition (getViewPositionX(), newRangeStartInt);
     }
@@ -483,7 +510,7 @@ void Viewport::mouseWheelMove (const MouseEvent& e, const MouseWheelDetails& whe
 
 static int rescaleMouseWheelDistance (float distance, int singleStepSize) noexcept
 {
-    if (distance == 0)
+    if (distance == 0.0f)
         return 0;
 
     distance *= 14.0f * singleStepSize;
@@ -496,15 +523,15 @@ bool Viewport::useMouseWheelMoveIfNeeded (const MouseEvent& e, const MouseWheelD
 {
     if (! (e.mods.isAltDown() || e.mods.isCtrlDown() || e.mods.isCommandDown()))
     {
-        const bool canScrollVert = (allowScrollingWithoutScrollbarV || verticalScrollBar.isVisible());
-        const bool canScrollHorz = (allowScrollingWithoutScrollbarH || horizontalScrollBar.isVisible());
+        const bool canScrollVert = (allowScrollingWithoutScrollbarV || getVerticalScrollBar().isVisible());
+        const bool canScrollHorz = (allowScrollingWithoutScrollbarH || getHorizontalScrollBar().isVisible());
 
         if (canScrollHorz || canScrollVert)
         {
-            const int deltaX = rescaleMouseWheelDistance (wheel.deltaX, singleStepX);
-            const int deltaY = rescaleMouseWheelDistance (wheel.deltaY, singleStepY);
+            auto deltaX = rescaleMouseWheelDistance (wheel.deltaX, singleStepX);
+            auto deltaY = rescaleMouseWheelDistance (wheel.deltaY, singleStepY);
 
-            Point<int> pos (getViewPosition());
+            auto pos = getViewPosition();
 
             if (deltaX != 0 && deltaY != 0 && canScrollHorz && canScrollVert)
             {
@@ -551,13 +578,13 @@ bool Viewport::keyPressed (const KeyPress& key)
 {
     const bool isUpDownKey = isUpDownKeyPress (key);
 
-    if (verticalScrollBar.isVisible() && isUpDownKey)
-        return verticalScrollBar.keyPressed (key);
+    if (getVerticalScrollBar().isVisible() && isUpDownKey)
+        return getVerticalScrollBar().keyPressed (key);
 
     const bool isLeftRightKey = isLeftRightKeyPress (key);
 
-    if (horizontalScrollBar.isVisible() && (isUpDownKey || isLeftRightKey))
-        return horizontalScrollBar.keyPressed (key);
+    if (getHorizontalScrollBar().isVisible() && (isUpDownKey || isLeftRightKey))
+        return getHorizontalScrollBar().keyPressed (key);
 
     return false;
 }
@@ -566,3 +593,10 @@ bool Viewport::respondsToKey (const KeyPress& key)
 {
     return isUpDownKeyPress (key) || isLeftRightKeyPress (key);
 }
+
+ScrollBar* Viewport::createScrollBarComponent (bool isVertical)
+{
+    return new ScrollBar (isVertical);
+}
+
+} // namespace juce
