@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_VIEWPORT_H_INCLUDED
-#define JUCE_VIEWPORT_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -212,15 +213,12 @@ public:
     bool isHorizontalScrollBarShown() const noexcept            { return showHScrollbar; }
 
     /** Changes the width of the scrollbars.
-
         If this isn't specified, the default width from the LookAndFeel class will be used.
-
         @see LookAndFeel::getDefaultScrollbarWidth
     */
     void setScrollBarThickness (int thickness);
 
     /** Returns the thickness of the scrollbars.
-
         @see setScrollBarThickness
     */
     int getScrollBarThickness() const;
@@ -233,13 +231,25 @@ public:
     /** Returns a pointer to the scrollbar component being used.
         Handy if you need to customise the bar somehow.
     */
-    ScrollBar* getVerticalScrollBar() noexcept                  { return &verticalScrollBar; }
+    ScrollBar& getVerticalScrollBar() noexcept                  { return *verticalScrollBar; }
 
     /** Returns a pointer to the scrollbar component being used.
         Handy if you need to customise the bar somehow.
     */
-    ScrollBar* getHorizontalScrollBar() noexcept                { return &horizontalScrollBar; }
+    ScrollBar& getHorizontalScrollBar() noexcept                { return *horizontalScrollBar; }
 
+    /** Re-instantiates the scrollbars, which is only really useful if you've overridden createScrollBarComponent(). */
+    void recreateScrollbars();
+
+    /** True if there's any off-screen content that could be scrolled vertically,
+        or false if everything is currently visible.
+    */
+    bool canScrollVertically() const noexcept;
+
+    /** True if there's any off-screen content that could be scrolled horizontally,
+        or false if everything is currently visible.
+    */
+    bool canScrollHorizontally() const noexcept;
 
     /** Enables or disables drag-to-scroll functionality in the viewport. */
     void setScrollOnDragEnabled (bool shouldScrollOnDrag);
@@ -270,17 +280,24 @@ public:
     /** @internal */
     static bool respondsToKey (const KeyPress&);
 
+protected:
+    //==============================================================================
+    /** Creates the Scrollbar components that will be added to the Viewport.
+        Subclasses can override this if they need to customise the scrollbars in some way.
+    */
+    virtual ScrollBar* createScrollBarComponent (bool isVertical);
+
 private:
     //==============================================================================
+    ScopedPointer<ScrollBar> verticalScrollBar, horizontalScrollBar;
+    Component contentHolder;
     WeakReference<Component> contentComp;
     Rectangle<int> lastVisibleArea;
-    int scrollBarThickness;
-    int singleStepX, singleStepY;
-    bool showHScrollbar, showVScrollbar, deleteContent;
-    bool customScrollBarThickness;
-    bool allowScrollingWithoutScrollbarV, allowScrollingWithoutScrollbarH;
-    Component contentHolder;
-    ScrollBar verticalScrollBar, horizontalScrollBar;
+    int scrollBarThickness = 0;
+    int singleStepX = 16, singleStepY = 16;
+    bool showHScrollbar = true, showVScrollbar = true, deleteContent = true;
+    bool customScrollBarThickness = false;
+    bool allowScrollingWithoutScrollbarV = false, allowScrollingWithoutScrollbarH = false;
 
     struct DragToScrollListener;
     friend struct DragToScrollListener;
@@ -300,5 +317,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Viewport)
 };
 
-
-#endif   // JUCE_VIEWPORT_H_INCLUDED
+} // namespace juce
