@@ -2,34 +2,40 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 LookAndFeel_V3::LookAndFeel_V3()
 {
     setColour (TreeView::selectedItemBackgroundColourId, Colour (0x301111ee));
 
     const Colour textButtonColour (0xffeeeeff);
     setColour (TextButton::buttonColourId, textButtonColour);
-    setColour (TextButton::buttonOnColourId, Colour (0xff000000));
+    setColour (TextButton::buttonOnColourId, Colour (0xff888888));
     setColour (ComboBox::buttonColourId, textButtonColour);
+    setColour (ComboBox::focusedOutlineColourId, textButtonColour);
     setColour (TextEditor::outlineColourId, Colours::transparentBlack);
     setColour (TabbedButtonBar::tabOutlineColourId, Colour (0x66000000));
     setColour (TabbedComponent::outlineColourId, Colour (0x66000000));
@@ -37,6 +43,8 @@ LookAndFeel_V3::LookAndFeel_V3()
     setColour (Slider::thumbColourId, Colour (0xffddddff));
     setColour (BubbleComponent::backgroundColourId, Colour (0xeeeeeedd));
     setColour (ScrollBar::thumbColourId, Colour::greyLevel (0.8f).contrasting().withAlpha (0.13f));
+    setColour (TableHeaderComponent::backgroundColourId, Colours::white.withAlpha (0.6f));
+    setColour (TableHeaderComponent::outlineColourId,    Colours::black.withAlpha (0.5f));
 }
 
 LookAndFeel_V3::~LookAndFeel_V3() {}
@@ -86,8 +94,8 @@ void LookAndFeel_V3::drawConcertinaPanelHeader (Graphics& g, const Rectangle<int
 {
     const Colour bkg (Colours::grey);
 
-    g.setGradientFill (ColourGradient (Colours::white.withAlpha (isMouseOver ? 0.4f : 0.2f), 0, (float) area.getY(),
-                                       Colours::darkgrey.withAlpha (0.1f), 0, (float) area.getBottom(), false));
+    g.setGradientFill (ColourGradient::vertical (Colours::white.withAlpha (isMouseOver ? 0.4f : 0.2f), (float) area.getY(),
+                                                 Colours::darkgrey.withAlpha (0.1f), (float) area.getBottom()));
     g.fillAll();
 
     g.setColour (bkg.contrasting().withAlpha (0.1f));
@@ -104,8 +112,8 @@ static void drawButtonShape (Graphics& g, const Path& outline, Colour baseColour
     const float mainBrightness = baseColour.getBrightness();
     const float mainAlpha = baseColour.getFloatAlpha();
 
-    g.setGradientFill (ColourGradient (baseColour.brighter (0.2f), 0.0f, 0.0f,
-                                       baseColour.darker (0.25f), 0.0f, height, false));
+    g.setGradientFill (ColourGradient::vertical (baseColour.brighter (0.2f), 0.0f,
+                                                 baseColour.darker (0.25f), height));
     g.fillPath (outline);
 
     g.setColour (Colours::white.withAlpha (0.4f * mainAlpha * mainBrightness * mainBrightness));
@@ -151,14 +159,15 @@ void LookAndFeel_V3::drawButtonBackground (Graphics& g, Button& button, const Co
 void LookAndFeel_V3::drawTableHeaderBackground (Graphics& g, TableHeaderComponent& header)
 {
     Rectangle<int> r (header.getLocalBounds());
+    auto outlineColour = header.findColour (TableHeaderComponent::outlineColourId);
 
-    g.setColour (Colours::black.withAlpha (0.5f));
+    g.setColour (outlineColour);
     g.fillRect (r.removeFromBottom (1));
 
-    g.setColour (Colours::white.withAlpha (0.6f));
+    g.setColour (header.findColour (TableHeaderComponent::backgroundColourId));
     g.fillRect (r);
 
-    g.setColour (Colours::black.withAlpha (0.5f));
+    g.setColour (outlineColour);
 
     for (int i = header.getNumColumns (true); --i >= 0;)
         g.fillRect (header.getColumnPosition (i).removeFromRight (1));
@@ -205,8 +214,8 @@ void LookAndFeel_V3::drawTabButton (TabBarButton& button, Graphics& g, bool isMo
             default:                              jassertfalse; break;
         }
 
-        g.setGradientFill (ColourGradient (bkg.brighter (0.2f), (float) p1.x, (float) p1.y,
-                                           bkg.darker (0.1f),   (float) p2.x, (float) p2.y, false));
+        g.setGradientFill (ColourGradient (bkg.brighter (0.2f), p1.toFloat(),
+                                           bkg.darker (0.1f),   p2.toFloat(), false));
     }
 
     g.fillRect (activeArea);
@@ -250,8 +259,8 @@ void LookAndFeel_V3::drawTabButton (TabBarButton& button, Graphics& g, bool isMo
 
     switch (o)
     {
-        case TabbedButtonBar::TabsAtLeft:   t = t.rotated (float_Pi * -0.5f).translated (area.getX(), area.getBottom()); break;
-        case TabbedButtonBar::TabsAtRight:  t = t.rotated (float_Pi *  0.5f).translated (area.getRight(), area.getY()); break;
+        case TabbedButtonBar::TabsAtLeft:   t = t.rotated (MathConstants<float>::pi * -0.5f).translated (area.getX(), area.getBottom()); break;
+        case TabbedButtonBar::TabsAtRight:  t = t.rotated (MathConstants<float>::pi *  0.5f).translated (area.getRight(), area.getY()); break;
         case TabbedButtonBar::TabsAtTop:
         case TabbedButtonBar::TabsAtBottom: t = t.translated (area.getX(), area.getY()); break;
         default:                            jassertfalse; break;
@@ -349,11 +358,9 @@ void LookAndFeel_V3::drawComboBox (Graphics& g, int width, int height, const boo
 {
     g.fillAll (box.findColour (ComboBox::backgroundColourId));
 
-    const Colour buttonColour (box.findColour (ComboBox::buttonColourId));
-
     if (box.isEnabled() && box.hasKeyboardFocus (false))
     {
-        g.setColour (buttonColour);
+        g.setColour (box.findColour (ComboBox::focusedOutlineColourId));
         g.drawRect (0, 0, width, height, 2);
     }
     else
@@ -395,12 +402,12 @@ void LookAndFeel_V3::drawLinearSlider (Graphics& g, int x, int y, int width, int
         else
             p.addRectangle (fx, fy, sliderPos - fx, fh);
 
-        Colour baseColour (slider.findColour (Slider::thumbColourId)
+        auto baseColour = slider.findColour (Slider::thumbColourId)
                                 .withMultipliedSaturation (slider.isEnabled() ? 1.0f : 0.5f)
-                                .withMultipliedAlpha (0.8f));
+                                .withMultipliedAlpha (0.8f);
 
-        g.setGradientFill (ColourGradient (baseColour.brighter (0.08f), 0.0f, 0.0f,
-                                           baseColour.darker (0.08f), 0.0f, (float) height, false));
+        g.setGradientFill (ColourGradient::vertical (baseColour.brighter (0.08f), 0.0f,
+                                                     baseColour.darker (0.08f), (float) height));
         g.fillPath (p);
 
         g.setColour (baseColour.darker (0.2f));
@@ -432,19 +439,17 @@ void LookAndFeel_V3::drawLinearSliderBackground (Graphics& g, int x, int y, int
 
     if (slider.isHorizontal())
     {
-        const float iy = y + height * 0.5f - sliderRadius * 0.5f;
+        auto iy = y + height * 0.5f - sliderRadius * 0.5f;
 
-        g.setGradientFill (ColourGradient (gradCol1, 0.0f, iy,
-                                           gradCol2, 0.0f, iy + sliderRadius, false));
+        g.setGradientFill (ColourGradient::vertical (gradCol1, iy, gradCol2, iy + sliderRadius));
 
         indent.addRoundedRectangle (x - sliderRadius * 0.5f, iy, width + sliderRadius, sliderRadius, 5.0f);
     }
     else
     {
-        const float ix = x + width * 0.5f - sliderRadius * 0.5f;
+        auto ix = x + width * 0.5f - sliderRadius * 0.5f;
 
-        g.setGradientFill (ColourGradient (gradCol1, ix, 0.0f,
-                                           gradCol2, ix + sliderRadius, 0.0f, false));
+        g.setGradientFill (ColourGradient::horizontal (gradCol1, ix, gradCol2, ix + sliderRadius));
 
         indent.addRoundedRectangle (ix, y - sliderRadius * 0.5f, sliderRadius, height + sliderRadius, 5.0f);
     }
@@ -469,7 +474,7 @@ void LookAndFeel_V3::drawPopupMenuBackground (Graphics& g, int width, int height
 void LookAndFeel_V3::drawMenuBarBackground (Graphics& g, int width, int height,
                                             bool, MenuBarComponent& menuBar)
 {
-    const Colour colour (menuBar.findColour (PopupMenu::backgroundColourId));
+    auto colour = menuBar.findColour (PopupMenu::backgroundColourId);
 
     Rectangle<int> r (width, height);
 
@@ -477,7 +482,7 @@ void LookAndFeel_V3::drawMenuBarBackground (Graphics& g, int width, int height,
     g.fillRect (r.removeFromTop (1));
     g.fillRect (r.removeFromBottom (1));
 
-    g.setGradientFill (ColourGradient (colour, 0, 0, colour.darker (0.08f), 0, (float) height, false));
+    g.setGradientFill (ColourGradient::vertical (colour, 0, colour.darker (0.08f), (float) height));
     g.fillRect (r);
 }
 
@@ -632,3 +637,5 @@ Path LookAndFeel_V3::getCrossShape (const float height)
     p.scaleToFit (0, 0, height * 2.0f, height, true);
     return p;
 }
+
+} // namespace juce
