@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 MenuBarModel::MenuBarModel() noexcept
     : manager (nullptr)
 {
@@ -70,12 +75,12 @@ void MenuBarModel::removeListener (Listener* const listenerToRemove) noexcept
 //==============================================================================
 void MenuBarModel::handleAsyncUpdate()
 {
-    listeners.call (&MenuBarModel::Listener::menuBarItemsChanged, this);
+    listeners.call ([this] (Listener& l) { l.menuBarItemsChanged (this); });
 }
 
 void MenuBarModel::applicationCommandInvoked (const ApplicationCommandTarget::InvocationInfo& info)
 {
-    listeners.call (&MenuBarModel::Listener::menuCommandInvoked, this, info);
+    listeners.call ([this, &info] (Listener& l) { l.menuCommandInvoked (this, info); });
 }
 
 void MenuBarModel::applicationCommandListChanged()
@@ -86,8 +91,10 @@ void MenuBarModel::applicationCommandListChanged()
 void MenuBarModel::handleMenuBarActivate (bool isActive)
 {
     menuBarActivated (isActive);
-    listeners.call (&MenuBarModel::Listener::menuBarActivated, this, isActive);
+    listeners.call ([this, isActive] (Listener& l) { l.menuBarActivated (this, isActive); });
 }
 
 void MenuBarModel::menuBarActivated (bool) {}
 void MenuBarModel::Listener::menuBarActivated (MenuBarModel*, bool) {}
+
+} // namespace juce
