@@ -2,35 +2,35 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 class MouseInputSourceInternal   : private AsyncUpdater
 {
 public:
-    //==============================================================================
-    MouseInputSourceInternal (const int i, const bool isMouse)
-        : index (i), isMouseDevice (isMouse), pressure (0.0f),
-          isUnboundedMouseModeOn (false), isCursorVisibleUntilOffscreen (false),
-          lastPeer (nullptr), currentCursorHandle (nullptr),
-          mouseEventCounter (0), mouseMovedSignificantlySincePressed (false)
+    MouseInputSourceInternal (int i, MouseInputSource::InputSourceType type)  : index (i), inputType (type)
     {
     }
 
@@ -60,10 +60,10 @@ public:
 
     static Point<float> screenPosToLocalPos (Component& comp, Point<float> pos)
     {
-        if (ComponentPeer* const peer = comp.getPeer())
+        if (auto* peer = comp.getPeer())
         {
             pos = peer->globalToLocal (pos);
-            Component& peerComp = peer->getComponent();
+            auto& peerComp = peer->getComponent();
             return comp.getLocalPoint (&peerComp, ScalingHelpers::unscaledScreenPosToScaled (peerComp, pos));
         }
 
@@ -72,13 +72,12 @@ public:
 
     Component* findComponentAt (Point<float> screenPos)
     {
-        if (ComponentPeer* const peer = getPeer())
+        if (auto* peer = getPeer())
         {
-            Point<float> relativePos (ScalingHelpers::unscaledScreenPosToScaled (peer->getComponent(),
-                                                                                 peer->globalToLocal (screenPos)));
-            Component& comp = peer->getComponent();
-
-            const Point<int> pos (relativePos.roundToInt());
+            auto relativePos = ScalingHelpers::unscaledScreenPosToScaled (peer->getComponent(),
+                                                                          peer->globalToLocal (screenPos));
+            auto& comp = peer->getComponent();
+            auto pos = relativePos.roundToInt();
 
             // (the contains() call is needed to test for overlapping desktop windows)
             if (comp.contains (pos))
@@ -88,13 +87,17 @@ public:
         return nullptr;
     }
 
-    Point<float> getScreenPosition() const
+    Point<float> getScreenPosition() const noexcept
     {
         // This needs to return the live position if possible, but it mustn't update the lastScreenPos
         // value, because that can cause continuity problems.
-        return ScalingHelpers::unscaledScreenPosToScaled
-                    (unboundedMouseOffset + (isMouseDevice ? MouseInputSource::getCurrentRawMousePosition()
-                                                           : lastScreenPos));
+        return ScalingHelpers::unscaledScreenPosToScaled (getRawScreenPosition());
+    }
+
+    Point<float> getRawScreenPosition() const noexcept
+    {
+        return unboundedMouseOffset + (inputType != MouseInputSource::InputSourceType::touch ? MouseInputSource::getCurrentRawMousePosition()
+                                                                                             : lastScreenPos);
     }
 
     void setScreenPosition (Point<float> p)
@@ -102,7 +105,10 @@ public:
         MouseInputSource::setRawMousePosition (ScalingHelpers::scaledScreenPosToUnscaled (p));
     }
 
-    bool isPressureValid() const noexcept       { return pressure > 0.0f && pressure < 1.0f; }
+    bool isPressureValid()    const noexcept     { return pressure >= 0.0f && pressure <= 1.0f; }
+    bool isOrientationValid() const noexcept     { return orientation >= 0.0f && orientation <= MathConstants<float>::twoPi; }
+    bool isRotationValid() const noexcept        { return rotation >= 0.0f && rotation <= MathConstants<float>::twoPi; }
+    bool isTiltValid (bool isX) const noexcept   { return isX ? (tiltX >= -1.0f && tiltX <= 1.0f) : (tiltY >= -1.0f && tiltY <= 1.0f); }
 
     //==============================================================================
    #if JUCE_DUMP_MOUSE_EVENTS
@@ -134,19 +140,19 @@ public:
     void sendMouseDown (Component& comp, Point<float> screenPos, Time time)
     {
         JUCE_MOUSE_EVENT_DBG ("down")
-        comp.internalMouseDown (MouseInputSource (this), screenPosToLocalPos (comp, screenPos), time, pressure);
+        comp.internalMouseDown (MouseInputSource (this), screenPosToLocalPos (comp, screenPos), time, pressure, orientation, rotation, tiltX, tiltY);
     }
 
     void sendMouseDrag (Component& comp, Point<float> screenPos, Time time)
     {
         JUCE_MOUSE_EVENT_DBG ("drag")
-        comp.internalMouseDrag (MouseInputSource (this), screenPosToLocalPos (comp, screenPos), time, pressure);
+        comp.internalMouseDrag (MouseInputSource (this), screenPosToLocalPos (comp, screenPos), time, pressure, orientation, rotation, tiltX, tiltY);
     }
 
     void sendMouseUp (Component& comp, Point<float> screenPos, Time time, const ModifierKeys oldMods)
     {
         JUCE_MOUSE_EVENT_DBG ("up")
-        comp.internalMouseUp (MouseInputSource (this), screenPosToLocalPos (comp, screenPos), time, oldMods, pressure);
+            comp.internalMouseUp (MouseInputSource (this), screenPosToLocalPos (comp, screenPos), time, oldMods, pressure, orientation, rotation, tiltX, tiltY);
     }
 
     void sendMouseWheel (Component& comp, Point<float> screenPos, Time time, const MouseWheelDetails& wheel)
@@ -179,13 +185,13 @@ public:
             return false;
         }
 
-        const int lastCounter = mouseEventCounter;
+        auto lastCounter = mouseEventCounter;
 
         if (buttonState.isAnyMouseButtonDown())
         {
-            if (Component* const current = getComponentUnderMouse())
+            if (auto* current = getComponentUnderMouse())
             {
-                const ModifierKeys oldMods (getCurrentModifiers());
+                auto oldMods = getCurrentModifiers();
                 buttonState = newButtonState; // must change this before calling sendMouseUp, in case it runs a modal loop
 
                 sendMouseUp (*current, screenPos + unboundedMouseOffset, time, oldMods);
@@ -203,9 +209,10 @@ public:
         {
             Desktop::getInstance().incrementMouseClickCounter();
 
-            if (Component* const current = getComponentUnderMouse())
+            if (auto* current = getComponentUnderMouse())
             {
-                registerMouseDown (screenPos, time, *current, buttonState);
+                registerMouseDown (screenPos, time, *current, buttonState,
+                                   inputType == MouseInputSource::InputSourceType::touch);
                 sendMouseDown (*current, screenPos, time);
             }
         }
@@ -215,12 +222,12 @@ public:
 
     void setComponentUnderMouse (Component* const newComponent, Point<float> screenPos, Time time)
     {
-        Component* current = getComponentUnderMouse();
+        auto* current = getComponentUnderMouse();
 
         if (newComponent != current)
         {
             WeakReference<Component> safeNewComp (newComponent);
-            const ModifierKeys originalButtonState (buttonState);
+            auto originalButtonState = buttonState;
 
             if (current != nullptr)
             {
@@ -268,7 +275,7 @@ public:
             cancelPendingUpdate();
             lastScreenPos = newScreenPos;
 
-            if (Component* const current = getComponentUnderMouse())
+            if (auto* current = getComponentUnderMouse())
             {
                 if (isDragging())
                 {
@@ -290,23 +297,38 @@ public:
 
     //==============================================================================
     void handleEvent (ComponentPeer& newPeer, Point<float> positionWithinPeer, Time time,
-                      const ModifierKeys newMods, float newPressure)
+                      const ModifierKeys newMods, float newPressure, float newOrientation, PenDetails pen)
     {
         lastTime = time;
+
         const bool pressureChanged = (pressure != newPressure);
         pressure = newPressure;
+
+        const bool orientationChanged = (orientation != newOrientation);
+        orientation = newOrientation;
+
+        const bool rotationChanged = (rotation != pen.rotation);
+        rotation = pen.rotation;
+
+        const bool tiltChanged = (tiltX != pen.tiltX || tiltY != pen.tiltY);
+        tiltX = pen.tiltX;
+        tiltY = pen.tiltY;
+
+        const bool shouldUpdate = (pressureChanged || orientationChanged || rotationChanged || tiltChanged);
+
         ++mouseEventCounter;
-        const Point<float> screenPos (newPeer.localToGlobal (positionWithinPeer));
+
+        auto screenPos = newPeer.localToGlobal (positionWithinPeer);
 
         if (isDragging() && newMods.isAnyMouseButtonDown())
         {
-            setScreenPos (screenPos, time, pressureChanged);
+            setScreenPos (screenPos, time, shouldUpdate);
         }
         else
         {
             setPeer (newPeer, screenPos, time);
 
-            if (ComponentPeer* peer = getPeer())
+            if (auto* peer = getPeer())
             {
                 if (setButtons (screenPos, time, newMods))
                     return; // some modal events have been dispatched, so the current event is now out-of-date
@@ -314,7 +336,7 @@ public:
                 peer = getPeer();
 
                 if (peer != nullptr)
-                    setScreenPos (screenPos, time, pressureChanged);
+                    setScreenPos (screenPos, time, shouldUpdate);
             }
         }
     }
@@ -345,6 +367,8 @@ public:
         // scrollable components.
         if (lastNonInertialWheelTarget == nullptr || ! wheel.isInertial)
             lastNonInertialWheelTarget = getTargetForGesture (peer, positionWithinPeer, time, screenPos);
+        else
+            screenPos = peer.localToGlobal (positionWithinPeer);
 
         if (Component* target = lastNonInertialWheelTarget)
             sendMouseWheel (*target, screenPos, time, wheel);
@@ -354,7 +378,8 @@ public:
                                Time time, const float scaleFactor)
     {
         Point<float> screenPos;
-        if (Component* current = getTargetForGesture (peer, positionWithinPeer, time, screenPos))
+
+        if (auto* current = getTargetForGesture (peer, positionWithinPeer, time, screenPos))
             sendMagnifyGesture (*current, screenPos, time, scaleFactor);
     }
 
@@ -408,13 +433,13 @@ public:
             if ((! enable) && ((! isCursorVisibleUntilOffscreen) || ! unboundedMouseOffset.isOrigin()))
             {
                 // when released, return the mouse to within the component's bounds
-                if (Component* current = getComponentUnderMouse())
+                if (auto* current = getComponentUnderMouse())
                     setScreenPosition (current->getScreenBounds().toFloat()
                                           .getConstrainedPoint (ScalingHelpers::unscaledScreenPosToScaled (lastScreenPos)));
             }
 
             isUnboundedMouseModeOn = enable;
-            unboundedMouseOffset = Point<float>();
+            unboundedMouseOffset = {};
 
             revealCursor (true);
         }
@@ -422,12 +447,11 @@ public:
 
     void handleUnboundedDrag (Component& current)
     {
-        const Rectangle<float> componentScreenBounds
-                = ScalingHelpers::scaledScreenPosToUnscaled (current.getParentMonitorArea().reduced (2, 2).toFloat());
+        auto componentScreenBounds = ScalingHelpers::scaledScreenPosToUnscaled (current.getParentMonitorArea().reduced (2, 2).toFloat());
 
         if (! componentScreenBounds.contains (lastScreenPos))
         {
-            const Point<float> componentCentre (current.getScreenBounds().toFloat().getCentre());
+            auto componentCentre = current.getScreenBounds().toFloat().getCentre();
             unboundedMouseOffset += (lastScreenPos - ScalingHelpers::scaledScreenPosToUnscaled (componentCentre));
             setScreenPosition (componentCentre);
         }
@@ -436,7 +460,7 @@ public:
                   && componentScreenBounds.contains (lastScreenPos + unboundedMouseOffset))
         {
             MouseInputSource::setRawMousePosition (lastScreenPos + unboundedMouseOffset);
-            unboundedMouseOffset = Point<float>();
+            unboundedMouseOffset = {};
         }
     }
 
@@ -465,7 +489,7 @@ public:
     {
         MouseCursor mc (MouseCursor::NormalCursor);
 
-        if (Component* current = getComponentUnderMouse())
+        if (auto* current = getComponentUnderMouse())
             mc = current->getLookAndFeel().getMouseCursorFor (*current);
 
         showMouseCursor (mc, forcedUpdate);
@@ -473,45 +497,52 @@ public:
 
     //==============================================================================
     const int index;
-    const bool isMouseDevice;
+    const MouseInputSource::InputSourceType inputType;
     Point<float> lastScreenPos, unboundedMouseOffset; // NB: these are unscaled coords
     ModifierKeys buttonState;
-    float pressure;
+    float pressure = 0;
+    float orientation = 0;
+    float rotation = 0;
+    float tiltX = 0;
+    float tiltY = 0;
 
-    bool isUnboundedMouseModeOn, isCursorVisibleUntilOffscreen;
+    bool isUnboundedMouseModeOn = false, isCursorVisibleUntilOffscreen = false;
 
 private:
     WeakReference<Component> componentUnderMouse, lastNonInertialWheelTarget;
-    ComponentPeer* lastPeer;
+    ComponentPeer* lastPeer = nullptr;
 
-    void* currentCursorHandle;
-    int mouseEventCounter;
+    void* currentCursorHandle = nullptr;
+    int mouseEventCounter = 0;
 
     struct RecentMouseDown
     {
-        RecentMouseDown() noexcept  : peerID (0) {}
+        RecentMouseDown() = default;
 
         Point<float> position;
         Time time;
         ModifierKeys buttons;
-        uint32 peerID;
+        uint32 peerID = 0;
+        bool isTouch = false;
 
-        bool canBePartOfMultipleClickWith (const RecentMouseDown& other, const int maxTimeBetweenMs) const
+        bool canBePartOfMultipleClickWith (const RecentMouseDown& other, int maxTimeBetweenMs) const noexcept
         {
             return time - other.time < RelativeTime::milliseconds (maxTimeBetweenMs)
-                    && std::abs (position.x - other.position.x) < 8
-                    && std::abs (position.y - other.position.y) < 8
+                    && std::abs (position.x - other.position.x) < getPositionToleranceForInputType()
+                    && std::abs (position.y - other.position.y) < getPositionToleranceForInputType()
                     && buttons == other.buttons
                     && peerID == other.peerID;
         }
+
+        int getPositionToleranceForInputType() const noexcept    { return isTouch ? 25 : 8;  }
     };
 
     RecentMouseDown mouseDowns[4];
     Time lastTime;
-    bool mouseMovedSignificantlySincePressed;
+    bool mouseMovedSignificantlySincePressed = false;
 
-    void registerMouseDown (Point<float> screenPos, Time time,
-                            Component& component, const ModifierKeys modifiers) noexcept
+    void registerMouseDown (Point<float> screenPos, Time time, Component& component,
+                            const ModifierKeys modifiers, bool isTouchSource) noexcept
     {
         for (int i = numElementsInArray (mouseDowns); --i > 0;)
             mouseDowns[i] = mouseDowns[i - 1];
@@ -519,8 +550,9 @@ private:
         mouseDowns[0].position = screenPos;
         mouseDowns[0].time = time;
         mouseDowns[0].buttons = modifiers.withOnlyMouseButtons();
+        mouseDowns[0].isTouch = isTouchSource;
 
-        if (ComponentPeer* const peer = component.getPeer())
+        if (auto* peer = component.getPeer())
             mouseDowns[0].peerID = peer->getUniqueID();
         else
             mouseDowns[0].peerID = 0;
@@ -549,36 +581,46 @@ MouseInputSource& MouseInputSource::operator= (const MouseInputSource& other) no
     return *this;
 }
 
-bool MouseInputSource::isMouse() const noexcept                          { return pimpl->isMouseDevice; }
-bool MouseInputSource::isTouch() const noexcept                          { return ! isMouse(); }
-bool MouseInputSource::canHover() const noexcept                         { return isMouse(); }
-bool MouseInputSource::hasMouseWheel() const noexcept                    { return isMouse(); }
-int MouseInputSource::getIndex() const noexcept                          { return pimpl->index; }
-bool MouseInputSource::isDragging() const noexcept                       { return pimpl->isDragging(); }
-Point<float> MouseInputSource::getScreenPosition() const noexcept        { return pimpl->getScreenPosition(); }
-ModifierKeys MouseInputSource::getCurrentModifiers() const noexcept      { return pimpl->getCurrentModifiers(); }
-float MouseInputSource::getCurrentPressure() const noexcept              { return pimpl->pressure; }
-bool MouseInputSource::isPressureValid() const noexcept                  { return pimpl->isPressureValid(); }
-Component* MouseInputSource::getComponentUnderMouse() const              { return pimpl->getComponentUnderMouse(); }
-void MouseInputSource::triggerFakeMove() const                           { pimpl->triggerFakeMove(); }
-int MouseInputSource::getNumberOfMultipleClicks() const noexcept         { return pimpl->getNumberOfMultipleClicks(); }
-Time MouseInputSource::getLastMouseDownTime() const noexcept             { return pimpl->getLastMouseDownTime(); }
-Point<float> MouseInputSource::getLastMouseDownPosition() const noexcept { return pimpl->getLastMouseDownPosition(); }
+MouseInputSource::InputSourceType MouseInputSource::getType() const noexcept    { return pimpl->inputType; }
+bool MouseInputSource::isMouse() const noexcept                                 { return (getType() == MouseInputSource::InputSourceType::mouse); }
+bool MouseInputSource::isTouch() const noexcept                                 { return (getType() == MouseInputSource::InputSourceType::touch); }
+bool MouseInputSource::isPen() const noexcept                                   { return (getType() == MouseInputSource::InputSourceType::pen); }
+bool MouseInputSource::canHover() const noexcept                                { return ! isTouch(); }
+bool MouseInputSource::hasMouseWheel() const noexcept                           { return ! isTouch(); }
+int MouseInputSource::getIndex() const noexcept                                 { return pimpl->index; }
+bool MouseInputSource::isDragging() const noexcept                              { return pimpl->isDragging(); }
+Point<float> MouseInputSource::getScreenPosition() const noexcept               { return pimpl->getScreenPosition(); }
+Point<float> MouseInputSource::getRawScreenPosition() const noexcept            { return pimpl->getRawScreenPosition();  }
+ModifierKeys MouseInputSource::getCurrentModifiers() const noexcept             { return pimpl->getCurrentModifiers(); }
+float MouseInputSource::getCurrentPressure() const noexcept                     { return pimpl->pressure; }
+bool MouseInputSource::isPressureValid() const noexcept                         { return pimpl->isPressureValid(); }
+float MouseInputSource::getCurrentOrientation() const noexcept                  { return pimpl->orientation; }
+bool MouseInputSource::isOrientationValid() const noexcept                      { return pimpl->isOrientationValid(); }
+float MouseInputSource::getCurrentRotation() const noexcept                     { return pimpl->rotation; }
+bool MouseInputSource::isRotationValid() const noexcept                         { return pimpl->isRotationValid(); }
+float MouseInputSource::getCurrentTilt (bool tiltX) const noexcept              { return tiltX ? pimpl->tiltX : pimpl->tiltY; }
+bool MouseInputSource::isTiltValid (bool isX) const noexcept                    { return pimpl->isTiltValid (isX); }
+Component* MouseInputSource::getComponentUnderMouse() const                     { return pimpl->getComponentUnderMouse(); }
+void MouseInputSource::triggerFakeMove() const                                  { pimpl->triggerFakeMove(); }
+int MouseInputSource::getNumberOfMultipleClicks() const noexcept                { return pimpl->getNumberOfMultipleClicks(); }
+Time MouseInputSource::getLastMouseDownTime() const noexcept                    { return pimpl->getLastMouseDownTime(); }
+Point<float> MouseInputSource::getLastMouseDownPosition() const noexcept        { return pimpl->getLastMouseDownPosition(); }
 bool MouseInputSource::hasMouseMovedSignificantlySincePressed() const noexcept  { return pimpl->hasMouseMovedSignificantlySincePressed(); }
-bool MouseInputSource::canDoUnboundedMovement() const noexcept           { return isMouse(); }
+bool MouseInputSource::canDoUnboundedMovement() const noexcept                  { return ! isTouch(); }
 void MouseInputSource::enableUnboundedMouseMovement (bool isEnabled, bool keepCursorVisibleUntilOffscreen) const
                                                                          { pimpl->enableUnboundedMouseMovement (isEnabled, keepCursorVisibleUntilOffscreen); }
 bool MouseInputSource::isUnboundedMouseMovementEnabled() const           { return pimpl->isUnboundedMouseModeOn; }
-bool MouseInputSource::hasMouseCursor() const noexcept                   { return isMouse(); }
+bool MouseInputSource::hasMouseCursor() const noexcept                   { return ! isTouch(); }
 void MouseInputSource::showMouseCursor (const MouseCursor& cursor)       { pimpl->showMouseCursor (cursor, false); }
 void MouseInputSource::hideCursor()                                      { pimpl->hideCursor(); }
 void MouseInputSource::revealCursor()                                    { pimpl->revealCursor (false); }
 void MouseInputSource::forceMouseCursorUpdate()                          { pimpl->revealCursor (true); }
 void MouseInputSource::setScreenPosition (Point<float> p)                { pimpl->setScreenPosition (p); }
 
-void MouseInputSource::handleEvent (ComponentPeer& peer, Point<float> pos, int64 time, ModifierKeys mods, float pressure)
+void MouseInputSource::handleEvent (ComponentPeer& peer, Point<float> pos, int64 time, ModifierKeys mods,
+                                    float pressure, float orientation, const PenDetails& penDetails)
 {
-    pimpl->handleEvent (peer, pos, Time (time), mods.withOnlyMouseButtons(), pressure);
+    pimpl->handleEvent (peer, pos, Time (time), mods.withOnlyMouseButtons(), pressure, orientation, penDetails);
 }
 
 void MouseInputSource::handleWheel (ComponentPeer& peer, Point<float> pos, int64 time, const MouseWheelDetails& wheel)
@@ -592,22 +634,36 @@ void MouseInputSource::handleMagnifyGesture (ComponentPeer& peer, Point<float> p
 }
 
 const float MouseInputSource::invalidPressure = 0.0f;
+const float MouseInputSource::invalidOrientation = 0.0f;
+const float MouseInputSource::invalidRotation = 0.0f;
+
+const float MouseInputSource::invalidTiltX = 0.0f;
+const float MouseInputSource::invalidTiltY = 0.0f;
 
 //==============================================================================
 struct MouseInputSource::SourceList  : public Timer
 {
     SourceList()
     {
-        addSource();
+       #if JUCE_ANDROID || JUCE_IOS
+        auto mainMouseInputType = MouseInputSource::InputSourceType::touch;
+       #else
+        auto mainMouseInputType = MouseInputSource::InputSourceType::mouse;
+       #endif
+
+        addSource (0, mainMouseInputType);
     }
 
     bool addSource();
+    bool canUseTouch();
 
-    void addSource (int index, bool isMouse)
+    MouseInputSource* addSource (int index, MouseInputSource::InputSourceType type)
     {
-        MouseInputSourceInternal* s = new MouseInputSourceInternal (index, isMouse);
+        auto* s = new MouseInputSourceInternal (index, type);
         sources.add (s);
         sourceArray.add (MouseInputSource (s));
+
+        return &sourceArray.getReference (sourceArray.size() - 1);
     }
 
     MouseInputSource* getMouseSource (int index) const noexcept
@@ -616,29 +672,37 @@ struct MouseInputSource::SourceList  : public Timer
                                                               : nullptr;
     }
 
-    MouseInputSource* getOrCreateMouseInputSource (int touchIndex)
+    MouseInputSource* getOrCreateMouseInputSource (MouseInputSource::InputSourceType type, int touchIndex = 0)
     {
-        jassert (touchIndex >= 0 && touchIndex < 100); // sanity-check on number of fingers
-
-        for (;;)
+        if (type == MouseInputSource::InputSourceType::mouse || type == MouseInputSource::InputSourceType::pen)
         {
-            if (MouseInputSource* mouse = getMouseSource (touchIndex))
-                return mouse;
+            for (auto& m : sourceArray)
+                if (type == m.getType())
+                    return &m;
 
-            if (! addSource())
-            {
-                jassertfalse; // not enough mouse sources!
-                return nullptr;
-            }
+            addSource (0, type);
         }
+        else if (type == MouseInputSource::InputSourceType::touch)
+        {
+            jassert (touchIndex >= 0 && touchIndex < 100); // sanity-check on number of fingers
+
+            for (auto& m : sourceArray)
+                if (type == m.getType() && touchIndex == m.getIndex())
+                    return &m;
+
+            if (canUseTouch())
+                return addSource (touchIndex, type);
+        }
+
+        return nullptr;
     }
 
     int getNumDraggingMouseSources() const noexcept
     {
         int num = 0;
 
-        for (int i = 0; i < sources.size(); ++i)
-            if (sources.getUnchecked(i)->isDragging())
+        for (auto* s : sources)
+            if (s->isDragging())
                 ++num;
 
         return num;
@@ -648,14 +712,12 @@ struct MouseInputSource::SourceList  : public Timer
     {
         int num = 0;
 
-        for (int i = 0; i < sources.size(); ++i)
+        for (auto& s : sourceArray)
         {
-            MouseInputSource* const mi = &(sourceArray.getReference(i));
-
-            if (mi->isDragging())
+            if (s.isDragging())
             {
                 if (index == num)
-                    return mi;
+                    return &s;
 
                 ++num;
             }
@@ -679,23 +741,26 @@ struct MouseInputSource::SourceList  : public Timer
 
     void timerCallback() override
     {
-        int numMiceDown = 0;
+        bool anyDragging = false;
 
-        for (int i = 0; i < sources.size(); ++i)
+        for (auto* s : sources)
         {
-            MouseInputSourceInternal* const mi = sources.getUnchecked(i);
-
-            if (mi->isDragging())
+            // NB: when doing auto-repeat, we need to force an update of the current position and button state,
+            // because on some OSes the queue can get overloaded with messages so that mouse-events don't get through..
+            if (s->isDragging() && ModifierKeys::getCurrentModifiersRealtime().isAnyMouseButtonDown())
             {
-                mi->triggerFakeMove();
-                ++numMiceDown;
+                s->lastScreenPos = s->getRawScreenPosition();
+                s->triggerFakeMove();
+                anyDragging = true;
             }
         }
 
-        if (numMiceDown == 0)
+        if (! anyDragging)
             stopTimer();
     }
 
     OwnedArray<MouseInputSourceInternal> sources;
     Array<MouseInputSource> sourceArray;
 };
+
+} // namespace juce
