@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_SELECTEDITEMSET_H_INCLUDED
-#define JUCE_SELECTEDITEMSET_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /** Manages a list of selectable items.
@@ -46,7 +47,7 @@ public:
     //==============================================================================
     typedef SelectableItemType ItemType;
     typedef Array<SelectableItemType> ItemArray;
-    typedef PARAMETER_TYPE (SelectableItemType) ParameterType;
+    typedef typename TypeHelpers::ParameterType<SelectableItemType>::type ParameterType;
 
     //==============================================================================
     /** Creates an empty set. */
@@ -77,12 +78,12 @@ public:
                 if (! other.isSelected (selectedItems.getReference (i)))
                     itemDeselected (selectedItems.removeAndReturn (i));
 
-            for (SelectableItemType* i = other.selectedItems.begin(), *e = other.selectedItems.end(); i != e; ++i)
+            for (auto& i : other.selectedItems)
             {
-                if (! isSelected (*i))
+                if (! isSelected (i))
                 {
-                    selectedItems.add (*i);
-                    itemSelected (*i);
+                    selectedItems.add (i);
+                    itemSelected (i);
                 }
             }
         }
@@ -319,5 +320,4 @@ private:
     JUCE_LEAK_DETECTOR (SelectedItemSet<SelectableItemType>)
 };
 
-
-#endif   // JUCE_SELECTEDITEMSET_H_INCLUDED
+} // namespace juce
