@@ -2,35 +2,71 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
+//==============================================================================
+struct JuceMainMenuBarHolder : private DeletedAtShutdown
+{
+    JuceMainMenuBarHolder()
+        : mainMenuBar ([[NSMenu alloc] initWithTitle: nsStringLiteral ("MainMenu")])
+    {
+        auto* item = [mainMenuBar addItemWithTitle: nsStringLiteral ("Apple")
+                                            action: nil
+                                     keyEquivalent: nsEmptyString()];
+
+        auto* appMenu = [[NSMenu alloc] initWithTitle: nsStringLiteral ("Apple")];
+
+        [NSApp performSelector: @selector (setAppleMenu:) withObject: appMenu];
+        [mainMenuBar setSubmenu: appMenu forItem: item];
+        [appMenu release];
+
+        [NSApp setMainMenu: mainMenuBar];
+    }
+
+    ~JuceMainMenuBarHolder()
+    {
+        clearSingletonInstance();
+
+        [NSApp setMainMenu: nil];
+        [mainMenuBar release];
+    }
+
+    NSMenu* mainMenuBar = nil;
+
+    JUCE_DECLARE_SINGLETON_SINGLETHREADED (JuceMainMenuBarHolder, true)
+};
+
+JUCE_IMPLEMENT_SINGLETON (JuceMainMenuBarHolder)
+
+//==============================================================================
 class JuceMainMenuHandler   : private MenuBarModel::Listener,
                               private DeletedAtShutdown
 {
 public:
     JuceMainMenuHandler()
-        : currentModel (nullptr),
-          lastUpdateTime (0),
-          isOpen (false),
-          defferedUpdateRequested (false)
     {
         static JuceMenuCallbackClass cls;
         callback = [cls.createInstance() init];
@@ -66,7 +102,7 @@ public:
             menuBarItemsChanged (nullptr);
         }
 
-        extraAppleMenuItems = createCopyIfNotNull (newExtraAppleMenuItems);
+        extraAppleMenuItems.reset (createCopyIfNotNull (newExtraAppleMenuItems));
     }
 
     void addTopLevelMenu (NSMenu* parent, const PopupMenu& child, const String& name, int menuId, int topLevelIndex)
@@ -94,11 +130,30 @@ public:
 
         [menu setAutoenablesItems: false];
         [menu update];
+
+        removeItemRecursive ([parentItem submenu]);
         [parentItem setSubmenu: menu];
+
         [menu release];
     }
 
-    void menuBarItemsChanged (MenuBarModel*)
+    void updateTopLevelMenu (NSMenu* menu)
+    {
+        NSMenu* superMenu = [menu supermenu];
+        auto menuNames = currentModel->getMenuBarNames();
+        auto indexOfMenu = (int) [superMenu indexOfItemWithSubmenu: menu] - 1;
+
+        removeItemRecursive (menu);
+
+        auto updatedPopup = currentModel->getMenuForIndex (indexOfMenu, menuNames[indexOfMenu]);
+
+        for (PopupMenu::MenuItemIterator iter (updatedPopup); iter.next();)
+            addMenuItem (iter, menu, 1, indexOfMenu);
+
+        [menu update];
+    }
+
+    void menuBarItemsChanged (MenuBarModel*) override
     {
         if (isOpen)
         {
@@ -112,61 +167,53 @@ public:
         if (currentModel != nullptr)
             menuNames = currentModel->getMenuBarNames();
 
-        NSMenu* menuBar = [[NSApp mainMenu] retain];
+        auto* menuBar = getMainMenuBar();
 
         while ([menuBar numberOfItems] > 1 + menuNames.size())
-            [menuBar removeItemAtIndex: [menuBar numberOfItems] - 1];
+            removeItemRecursive (menuBar, static_cast<int> ([menuBar numberOfItems] - 1));
 
         int menuId = 1;
 
         for (int i = 0; i < menuNames.size(); ++i)
         {
-            const PopupMenu menu (currentModel->getMenuForIndex (i, menuNames [i]));
+            const PopupMenu menu (currentModel->getMenuForIndex (i, menuNames[i]));
 
             if (i >= [menuBar numberOfItems] - 1)
                 addTopLevelMenu (menuBar, menu, menuNames[i], menuId, i);
             else
                 updateTopLevelMenu ([menuBar itemAtIndex: 1 + i], menu, menuNames[i], menuId, i);
         }
-
-        [menuBar release];
     }
 
-    void menuCommandInvoked (MenuBarModel*, const ApplicationCommandTarget::InvocationInfo& info)
+    void menuCommandInvoked (MenuBarModel*, const ApplicationCommandTarget::InvocationInfo& info) override
     {
         if ((info.commandFlags & ApplicationCommandInfo::dontTriggerVisualFeedback) == 0
               && info.invocationMethod != ApplicationCommandTarget::InvocationInfo::fromKeyPress)
-            if (NSMenuItem* item = findMenuItemWithTag ([NSApp mainMenu], info.commandID))
+            if (auto* item = findMenuItemWithCommandID (getMainMenuBar(), info.commandID))
                 flashMenuBar ([item menu]);
     }
 
-    void updateMenus (NSMenu* menu)
-    {
-        if (PopupMenu::dismissAllActiveMenus())
-        {
-            // If we were running a juce menu, then we should let that modal loop finish before allowing
-            // the OS menus to start their own modal loop - so cancel the menu that was being opened..
-            if ([menu respondsToSelector: @selector (cancelTracking)])
-                [menu performSelector: @selector (cancelTracking)];
-        }
-
-        if (Time::getMillisecondCounter() > lastUpdateTime + 100)
-            (new AsyncMenuUpdater())->post();
-    }
-
-    void invoke (int commandId, ApplicationCommandManager* commandManager, int topLevelIndex) const
+    void invoke (const PopupMenu::Item& item, int topLevelIndex) const
     {
         if (currentModel != nullptr)
         {
-            if (commandManager != nullptr)
+            if (item.customCallback != nullptr)
+                if (! item.customCallback->menuItemTriggered())
+                    return;
+
+            if (item.commandManager != nullptr)
             {
-                ApplicationCommandTarget::InvocationInfo info (commandId);
+                ApplicationCommandTarget::InvocationInfo info (item.itemID);
                 info.invocationMethod = ApplicationCommandTarget::InvocationInfo::fromMenu;
 
-                commandManager->invoke (info, true);
+                item.commandManager->invoke (info, true);
             }
 
-            (new AsyncCommandInvoker (commandId, topLevelIndex))->post();
+            MessageManager::callAsync ([=]
+            {
+                if (instance != nullptr)
+                    instance->invokeDirectly (item.itemID, topLevelIndex);
+            });
         }
     }
 
@@ -202,7 +249,7 @@ public:
             if (i.text == recentItemsMenuName)
             {
                 if (recent == nullptr)
-                    recent = new RecentFilesMenuItem();
+                    recent.reset (new RecentFilesMenuItem());
 
                 if (recent->recentItem != nil)
                 {
@@ -227,40 +274,42 @@ public:
         }
         else
         {
-            NSMenuItem* item = [menuToAddTo addItemWithTitle: text
-                                                      action: @selector (menuItemInvoked:)
-                                               keyEquivalent: nsEmptyString()];
+            auto* item = [[NSMenuItem alloc] initWithTitle: text
+                                                    action: @selector (menuItemInvoked:)
+                                             keyEquivalent: nsEmptyString()];
 
-            [item setTag: i.itemID];
+            [item setTag: topLevelIndex];
             [item setEnabled: i.isEnabled];
             [item setState: i.isTicked ? NSOnState : NSOffState];
             [item setTarget: (id) callback];
 
-            NSMutableArray* info = [NSMutableArray arrayWithObject: [NSNumber numberWithUnsignedLongLong: (pointer_sized_uint) (void*) i.commandManager]];
-            [info addObject: [NSNumber numberWithInt: topLevelIndex]];
-            [item setRepresentedObject: info];
+            auto* juceItem = new PopupMenu::Item (i);
+            juceItem->customComponent = nullptr;
+
+            [item setRepresentedObject: [createNSObjectFromJuceClass (juceItem) autorelease]];
 
             if (i.commandManager != nullptr)
             {
-                const Array<KeyPress> keyPresses (i.commandManager->getKeyMappings()
-                                                     ->getKeyPressesAssignedToCommand (i.itemID));
-
-                if (keyPresses.size() > 0)
+                for (auto& kp : i.commandManager->getKeyMappings()->getKeyPressesAssignedToCommand (i.itemID))
                 {
-                    const KeyPress& kp = keyPresses.getReference(0);
-
                     if (kp != KeyPress::backspaceKey   // (adding these is annoying because it flashes the menu bar
                          && kp != KeyPress::deleteKey) // every time you press the key while editing text)
                     {
                         juce_wchar key = kp.getTextCharacter();
+
                         if (key == 0)
                             key = (juce_wchar) kp.getKeyCode();
 
                         [item setKeyEquivalent: juceStringToNS (String::charToString (key).toLowerCase())];
                         [item setKeyEquivalentModifierMask: juceModsToNSMods (kp.getModifiers())];
                     }
+
+                    break;
                 }
             }
+
+            [menuToAddTo addItem: item];
+            [item release];
         }
     }
 
@@ -292,12 +341,12 @@ public:
 
     static JuceMainMenuHandler* instance;
 
-    MenuBarModel* currentModel;
+    MenuBarModel* currentModel = nullptr;
     ScopedPointer<PopupMenu> extraAppleMenuItems;
-    uint32 lastUpdateTime;
-    NSObject* callback;
+    uint32 lastUpdateTime = 0;
+    NSObject* callback = nil;
     String recentItemsMenuName;
-    bool isOpen, defferedUpdateRequested;
+    bool isOpen = false, defferedUpdateRequested = false;
 
 private:
     struct RecentFilesMenuItem
@@ -354,16 +403,17 @@ private:
     ScopedPointer<RecentFilesMenuItem> recent;
 
     //==============================================================================
-    static NSMenuItem* findMenuItemWithTag (NSMenu* const menu, int tag)
+    static NSMenuItem* findMenuItemWithCommandID (NSMenu* const menu, int commandID)
     {
         for (NSInteger i = [menu numberOfItems]; --i >= 0;)
         {
             NSMenuItem* m = [menu itemAtIndex: i];
-            if ([m tag] == tag)
-                return m;
+            if (auto* menuItem = getJuceClassFromNSObject<PopupMenu::Item> ([m representedObject]))
+                if (menuItem->itemID == commandID)
+                    return m;
 
             if (NSMenu* sub = [m submenu])
-                if (NSMenuItem* found = findMenuItemWithTag (sub, tag))
+                if (NSMenuItem* found = findMenuItemWithCommandID (sub, commandID))
                     return found;
         }
 
@@ -389,9 +439,9 @@ private:
 
         if ([menu indexOfItem: item] >= 0)
         {
-            NSEvent* f35Event = [NSEvent keyEventWithType: NSKeyDown
+            NSEvent* f35Event = [NSEvent keyEventWithType: NSEventTypeKeyDown
                                                  location: NSZeroPoint
-                                            modifierFlags: NSCommandKeyMask
+                                            modifierFlags: NSEventModifierFlagCommand
                                                 timestamp: 0
                                              windowNumber: 0
                                                   context: [NSGraphicsContext currentContext]
@@ -412,46 +462,50 @@ private:
     static unsigned int juceModsToNSMods (const ModifierKeys mods)
     {
         unsigned int m = 0;
-        if (mods.isShiftDown())    m |= NSShiftKeyMask;
-        if (mods.isCtrlDown())     m |= NSControlKeyMask;
-        if (mods.isAltDown())      m |= NSAlternateKeyMask;
-        if (mods.isCommandDown())  m |= NSCommandKeyMask;
+        if (mods.isShiftDown())    m |= NSEventModifierFlagShift;
+        if (mods.isCtrlDown())     m |= NSEventModifierFlagControl;
+        if (mods.isAltDown())      m |= NSEventModifierFlagOption;
+        if (mods.isCommandDown())  m |= NSEventModifierFlagCommand;
         return m;
     }
 
-    struct AsyncMenuUpdater  : public CallbackMessage
+    // Apple Bug: For some reason [NSMenu removeAllItems] seems to leak it's objects
+    // on shutdown, so we need this method to release the items one-by-one manually
+    static void removeItemRecursive (NSMenu* parentMenu, int menuItemIndex)
     {
-        AsyncMenuUpdater() {}
-
-        void messageCallback() override
+        if (isPositiveAndBelow (menuItemIndex, (int) [parentMenu numberOfItems]))
         {
-            if (instance != nullptr)
-                instance->menuBarItemsChanged (nullptr);
+            auto* menuItem = [parentMenu itemAtIndex:menuItemIndex];
+
+            if (auto* submenu = [menuItem submenu])
+                removeItemRecursive (submenu);
+
+            [parentMenu removeItem:menuItem];
         }
+        else
+            jassertfalse;
+    }
 
-        JUCE_DECLARE_NON_COPYABLE (AsyncMenuUpdater)
-    };
-
-    struct AsyncCommandInvoker  : public CallbackMessage
+    static void removeItemRecursive (NSMenu* menu)
     {
-        AsyncCommandInvoker (int commandId_, int topLevelIndex_)
-            : commandId (commandId_), topLevelIndex (topLevelIndex_)
-        {}
-
-        void messageCallback() override
+        if (menu != nullptr)
         {
-            if (instance != nullptr)
-                instance->invokeDirectly (commandId, topLevelIndex);
-        }
+            auto n = static_cast<int> ([menu numberOfItems]);
 
-        const int commandId, topLevelIndex;
-        JUCE_DECLARE_NON_COPYABLE (AsyncCommandInvoker)
-    };
+            for (auto i = n; --i >= 0;)
+                removeItemRecursive (menu, i);
+        }
+    }
+
+    static NSMenu* getMainMenuBar()
+    {
+        return JuceMainMenuBarHolder::getInstance()->mainMenuBar;
+    }
 
     //==============================================================================
-    struct JuceMenuCallbackClass   : public ObjCClass <NSObject>
+    struct JuceMenuCallbackClass   : public ObjCClass<NSObject>
     {
-        JuceMenuCallbackClass()  : ObjCClass <NSObject> ("JUCEMainMenu_")
+        JuceMenuCallbackClass()  : ObjCClass<NSObject> ("JUCEMainMenu_")
         {
             addIvar<JuceMainMenuHandler*> ("owner");
 
@@ -473,21 +527,22 @@ private:
     private:
         static void menuItemInvoked (id self, SEL, NSMenuItem* item)
         {
-            JuceMainMenuHandler* const owner = getIvar<JuceMainMenuHandler*> (self, "owner");
+            auto owner = getIvar<JuceMainMenuHandler*> (self, "owner");
 
-            if ([[item representedObject] isKindOfClass: [NSArray class]])
+            if (auto* juceItem = getJuceClassFromNSObject<PopupMenu::Item> ([item representedObject]))
             {
                 // If the menu is being triggered by a keypress, the OS will have picked it up before we had a chance to offer it to
                 // our own components, which may have wanted to intercept it. So, rather than dispatching directly, we'll feed it back
                 // into the focused component and let it trigger the menu item indirectly.
                 NSEvent* e = [NSApp currentEvent];
-                if ([e type] == NSKeyDown || [e type] == NSKeyUp)
+
+                if ([e type] == NSEventTypeKeyDown || [e type] == NSEventTypeKeyUp)
                 {
-                    if (juce::Component* focused = juce::Component::getCurrentlyFocusedComponent())
+                    if (auto* focused = juce::Component::getCurrentlyFocusedComponent())
                     {
-                        if (juce::NSViewComponentPeer* peer = dynamic_cast<juce::NSViewComponentPeer*> (focused->getPeer()))
+                        if (auto peer = dynamic_cast<juce::NSViewComponentPeer*> (focused->getPeer()))
                         {
-                            if ([e type] == NSKeyDown)
+                            if ([e type] == NSEventTypeKeyDown)
                                 peer->redirectKeyDown (e);
                             else
                                 peer->redirectKeyUp (e);
@@ -497,19 +552,13 @@ private:
                     }
                 }
 
-                NSArray* info = (NSArray*) [item representedObject];
-
-                owner->invoke ((int) [item tag],
-                               (ApplicationCommandManager*) (pointer_sized_int)
-                                    [((NSNumber*) [info objectAtIndex: 0]) unsignedLongLongValue],
-                               (int) [((NSNumber*) [info objectAtIndex: 1]) intValue]);
+                owner->invoke (*juceItem, static_cast<int> ([item tag]));
             }
         }
 
-        static void menuNeedsUpdate (id, SEL, NSMenu* menu)
+        static void menuNeedsUpdate (id self, SEL, NSMenu* menu)
         {
-            if (instance != nullptr)
-                instance->updateMenus (menu);
+            getIvar<JuceMainMenuHandler*> (self, "owner")->updateTopLevelMenu (menu);
         }
     };
 };
@@ -521,52 +570,61 @@ class TemporaryMainMenuWithStandardCommands
 {
 public:
     TemporaryMainMenuWithStandardCommands()
-        : oldMenu (MenuBarModel::getMacMainMenu()), oldAppleMenu (nullptr)
+        : oldMenu (MenuBarModel::getMacMainMenu())
     {
-        if (const PopupMenu* appleMenu = MenuBarModel::getMacExtraAppleItemsMenu())
-            oldAppleMenu = new PopupMenu (*appleMenu);
+        if (auto* appleMenu = MenuBarModel::getMacExtraAppleItemsMenu())
+            oldAppleMenu.reset (new PopupMenu (*appleMenu));
 
-        if (JuceMainMenuHandler::instance != nullptr)
-            oldRecentItems = JuceMainMenuHandler::instance->recentItemsMenuName;
+        if (auto* handler = JuceMainMenuHandler::instance)
+            oldRecentItems = handler->recentItemsMenuName;
 
         MenuBarModel::setMacMainMenu (nullptr);
 
-        NSMenu* menu = [[NSMenu alloc] initWithTitle: nsStringLiteral ("Edit")];
-        NSMenuItem* item;
+        if (auto* mainMenu = JuceMainMenuBarHolder::getInstance()->mainMenuBar)
+        {
+            NSMenu* menu = [[NSMenu alloc] initWithTitle: nsStringLiteral ("Edit")];
+            NSMenuItem* item;
 
-        item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString (nsStringLiteral ("Cut"), nil)
-                                          action: @selector (cut:)  keyEquivalent: nsStringLiteral ("x")];
-        [menu addItem: item];
-        [item release];
+            item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString (nsStringLiteral ("Cut"), nil)
+                                              action: @selector (cut:)  keyEquivalent: nsStringLiteral ("x")];
+            [menu addItem: item];
+            [item release];
 
-        item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString (nsStringLiteral ("Copy"), nil)
-                                          action: @selector (copy:)  keyEquivalent: nsStringLiteral ("c")];
-        [menu addItem: item];
-        [item release];
+            item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString (nsStringLiteral ("Copy"), nil)
+                                              action: @selector (copy:)  keyEquivalent: nsStringLiteral ("c")];
+            [menu addItem: item];
+            [item release];
 
-        item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString (nsStringLiteral ("Paste"), nil)
-                                          action: @selector (paste:)  keyEquivalent: nsStringLiteral ("v")];
-        [menu addItem: item];
-        [item release];
+            item = [[NSMenuItem alloc] initWithTitle: NSLocalizedString (nsStringLiteral ("Paste"), nil)
+                                              action: @selector (paste:)  keyEquivalent: nsStringLiteral ("v")];
+            [menu addItem: item];
+            [item release];
 
-        item = [[NSApp mainMenu] addItemWithTitle: NSLocalizedString (nsStringLiteral ("Edit"), nil)
-                                          action: nil  keyEquivalent: nsEmptyString()];
-        [[NSApp mainMenu] setSubmenu: menu forItem: item];
-        [menu release];
+            editMenuIndex = [mainMenu numberOfItems];
+
+            item = [mainMenu addItemWithTitle: NSLocalizedString (nsStringLiteral ("Edit"), nil)
+                                       action: nil  keyEquivalent: nsEmptyString()];
+            [mainMenu setSubmenu: menu forItem: item];
+            [menu release];
+        }
 
         // use a dummy modal component so that apps can tell that something is currently modal.
-        dummyModalComponent.enterModalState();
+        dummyModalComponent.enterModalState (false);
     }
 
     ~TemporaryMainMenuWithStandardCommands()
     {
-        MenuBarModel::setMacMainMenu (oldMenu, oldAppleMenu, oldRecentItems);
+        if (auto* mainMenu = JuceMainMenuBarHolder::getInstance()->mainMenuBar)
+            [mainMenu removeItemAtIndex:editMenuIndex];
+
+        MenuBarModel::setMacMainMenu (oldMenu, oldAppleMenu.get(), oldRecentItems);
     }
 
 private:
-    MenuBarModel* oldMenu;
+    MenuBarModel* const oldMenu;
     ScopedPointer<PopupMenu> oldAppleMenu;
     String oldRecentItems;
+    NSInteger editMenuIndex;
 
     // The OS view already plays an alert when clicking outside
     // the modal comp, so this override avoids adding extra
@@ -621,16 +679,16 @@ namespace MainMenuHelpers
         [NSApp setServicesMenu: servicesMenu];
         [menu addItem: [NSMenuItem separatorItem]];
 
-        createMenuItem (menu, "Hide " + appName, @selector (hide:), nsStringLiteral ("h"));
+        createMenuItem (menu, TRANS("Hide") + String (" ") + appName, @selector (hide:), nsStringLiteral ("h"));
 
-        [createMenuItem (menu, "Hide Others", @selector (hideOtherApplications:), nsStringLiteral ("h"))
-            setKeyEquivalentModifierMask: NSCommandKeyMask | NSAlternateKeyMask];
+        [createMenuItem (menu, TRANS("Hide Others"), @selector (hideOtherApplications:), nsStringLiteral ("h"))
+            setKeyEquivalentModifierMask: NSEventModifierFlagCommand | NSEventModifierFlagOption];
 
-        createMenuItem (menu, "Show All", @selector (unhideAllApplications:), nsEmptyString());
+        createMenuItem (menu, TRANS("Show All"), @selector (unhideAllApplications:), nsEmptyString());
 
         [menu addItem: [NSMenuItem separatorItem]];
 
-        createMenuItem (menu, "Quit " + appName, @selector (terminate:), nsStringLiteral ("q"));
+        createMenuItem (menu, TRANS("Quit") + String (" ") + appName, @selector (terminate:), nsStringLiteral ("q"));
     }
 
     // Since our app has no NIB, this initialises a standard app menu...
@@ -639,23 +697,18 @@ namespace MainMenuHelpers
         // this can't be used in a plugin!
         jassert (JUCEApplicationBase::isStandaloneApp());
 
-        if (JUCEApplicationBase* app = JUCEApplicationBase::getInstance())
+        if (auto* app = JUCEApplicationBase::getInstance())
         {
-            JUCE_AUTORELEASEPOOL
+            if (auto* mainMenu = JuceMainMenuBarHolder::getInstance()->mainMenuBar)
             {
-                NSMenu* mainMenu = [[NSMenu alloc] initWithTitle: nsStringLiteral ("MainMenu")];
-                NSMenuItem* item = [mainMenu addItemWithTitle: nsStringLiteral ("Apple") action: nil keyEquivalent: nsEmptyString()];
-
-                NSMenu* appMenu = [[NSMenu alloc] initWithTitle: nsStringLiteral ("Apple")];
-
-                [NSApp performSelector: @selector (setAppleMenu:) withObject: appMenu];
-                [mainMenu setSubmenu: appMenu forItem: item];
-
-                [NSApp setMainMenu: mainMenu];
-                MainMenuHelpers::createStandardAppMenu (appMenu, app->getApplicationName(), extraItems);
-
-                [appMenu release];
-                [mainMenu release];
+                if ([mainMenu numberOfItems] > 0)
+                {
+                    if (auto* appMenu = [[mainMenu itemAtIndex:0] submenu])
+                    {
+                        [appMenu removeAllItems];
+                        MainMenuHelpers::createStandardAppMenu (appMenu, app->getApplicationName(), extraItems);
+                    }
+                }
             }
         }
     }
@@ -695,7 +748,7 @@ void MenuBarModel::setMacMainMenu (MenuBarModel* newMenuBarModel,
 
 MenuBarModel* MenuBarModel::getMacMainMenu()
 {
-    if (JuceMainMenuHandler* mm = JuceMainMenuHandler::instance)
+    if (auto* mm = JuceMainMenuHandler::instance)
         return mm->currentModel;
 
     return nullptr;
@@ -703,7 +756,7 @@ MenuBarModel* MenuBarModel::getMacMainMenu()
 
 const PopupMenu* MenuBarModel::getMacExtraAppleItemsMenu()
 {
-    if (JuceMainMenuHandler* mm = JuceMainMenuHandler::instance)
+    if (auto* mm = JuceMainMenuHandler::instance)
         return mm->extraAppleMenuItems.get();
 
     return nullptr;
@@ -716,11 +769,11 @@ static void mainMenuTrackingChanged (bool isTracking)
 {
     PopupMenu::dismissAllActiveMenus();
 
-    if (JuceMainMenuHandler* menuHandler = JuceMainMenuHandler::instance)
+    if (auto* menuHandler = JuceMainMenuHandler::instance)
     {
         menuHandler->isOpen = isTracking;
 
-        if (MenuBarModel* model = menuHandler->currentModel)
+        if (auto* model = menuHandler->currentModel)
             model->handleMenuBarActivate (isTracking);
 
         if (menuHandler->defferedUpdateRequested && ! isTracking)
@@ -740,14 +793,16 @@ void juce_initialiseMacMainMenu()
 }
 
 // (used from other modules that need to create an NSMenu)
-NSMenu* createNSMenu (const PopupMenu& menu, const String& name,
-                      int topLevelMenuId, int topLevelIndex, bool addDelegate)
+NSMenu* createNSMenu (const PopupMenu&, const String&, int, int, bool);
+NSMenu* createNSMenu (const PopupMenu& menu, const String& name, int topLevelMenuId, int topLevelIndex, bool addDelegate)
 {
     juce_initialiseMacMainMenu();
 
-    if (JuceMainMenuHandler* mm = JuceMainMenuHandler::instance)
+    if (auto* mm = JuceMainMenuHandler::instance)
         return mm->createMenu (menu, name, topLevelMenuId, topLevelIndex, addDelegate);
 
     jassertfalse; // calling this before making sure the OSX main menu stuff was initialised?
     return nil;
 }
+
+} // namespace juce
