@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_TEXTPROPERTYCOMPONENT_H_INCLUDED
-#define JUCE_TEXTPROPERTYCOMPONENT_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -41,29 +42,50 @@ protected:
         @param propertyName  The name of the property
         @param maxNumChars   If not zero, then this specifies the maximum allowable length of
                              the string. If zero, then the string will have no length limit.
-        @param isMultiLine   isMultiLine sets whether the text editor allows carriage returns.
+        @param isMultiLine   Sets whether the text editor allows carriage returns.
+        @param isEditable    Sets whether the text editor is editable. The default is true.
 
-        @see TextEditor
+        @see TextEditor, setEditable
     */
     TextPropertyComponent (const String& propertyName,
                            int maxNumChars,
-                           bool isMultiLine);
+                           bool isMultiLine,
+                           bool isEditable = true);
 
 public:
     /** Creates a text property component.
 
-        @param valueToControl The Value that is controlled by the TextPropertyCOmponent
+        @param valueToControl The Value that is controlled by the TextPropertyComponent
         @param propertyName   The name of the property
         @param maxNumChars    If not zero, then this specifies the maximum allowable length of
                               the string. If zero, then the string will have no length limit.
-        @param isMultiLine    isMultiLine sets whether the text editor allows carriage returns.
+        @param isMultiLine    Sets whether the text editor allows carriage returns.
+        @param isEditable     Sets whether the text editor is editable. The default is true.
 
-        @see TextEditor
+        @see TextEditor, setEditable
     */
     TextPropertyComponent (const Value& valueToControl,
                            const String& propertyName,
                            int maxNumChars,
-                           bool isMultiLine);
+                           bool isMultiLine,
+                           bool isEditable = true);
+
+    /** Creates a text property component with a default value.
+
+        @param valueToControl The ValueWithDefault that is controlled by the TextPropertyComponent
+        @param propertyName   The name of the property
+        @param maxNumChars    If not zero, then this specifies the maximum allowable length of
+                              the string. If zero, then the string will have no length limit.
+        @param isMultiLine    Sets whether the text editor allows carriage returns.
+        @param isEditable     Sets whether the text editor is editable. The default is true.
+
+        @see TextEditor, setEditable
+    */
+    TextPropertyComponent (const ValueWithDefault& valueToControl,
+                           const String& propertyName,
+                           int maxNumChars,
+                           bool isMultiLine,
+                           bool isEditable = true);
 
     /** Destructor. */
     ~TextPropertyComponent();
@@ -121,6 +143,17 @@ public:
     */
     void removeListener (Listener* listener);
 
+    //==============================================================================
+    /** Sets whether the text property component can have files dropped onto it by an external application.
+
+        The default setting for this is true but you may want to disable this behaviour if you derive
+        from this class and want your subclass to respond to the file drag.
+    */
+    void setInterestedInFileDrag (bool isInterested);
+
+    /** Sets whether the text editor is editable. The default setting for this is true. */
+    void setEditable (bool isEditable);
+
     //==============================================================================
     /** @internal */
     void refresh() override;
@@ -128,6 +161,8 @@ public:
     virtual void textWasEdited();
 
 private:
+    class RemapperValueSourceWithDefault;
+
     class LabelComp;
     friend class LabelComp;
 
@@ -135,14 +170,10 @@ private:
     ListenerList<Listener> listenerList;
 
     void callListeners();
-    void createEditor (int maxNumChars, bool isMultiLine);
+    void createEditor (int maxNumChars, bool isMultiLine, bool isEditable);
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (TextPropertyComponent)
 };
 
-#ifndef DOXYGEN
- /** This typedef is just for compatibility with old code and VC6 - newer code should use TextPropertyComponent::Listener instead. */
- typedef TextPropertyComponent::Listener TextPropertyComponentListener;
-#endif
 
-#endif   // JUCE_TEXTPROPERTYCOMPONENT_H_INCLUDED
+} // namespace juce
