@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_SLIDER_H_INCLUDED
-#define JUCE_SLIDER_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -61,7 +62,7 @@ public:
         LinearHorizontal,               /**< A traditional horizontal slider. */
         LinearVertical,                 /**< A traditional vertical slider. */
         LinearBar,                      /**< A horizontal bar slider with the text label drawn on top of it. */
-        LinearBarVertical,
+        LinearBarVertical,              /**< A vertical bar slider with the text label drawn on top of it. */
         Rotary,                         /**< A rotary control that you move by dragging the mouse in a circular motion, like a knob.
                                              @see setRotaryParameters */
         RotaryHorizontalDrag,           /**< A rotary control that you move by dragging the mouse left-to-right.
@@ -80,7 +81,7 @@ public:
         ThreeValueHorizontal,           /**< A horizontal slider that has three thumbs instead of one, so it can show a minimum and maximum
                                              value, with the current value being somewhere between them.
                                              @see setMinValue, setMaxValue */
-        ThreeValueVertical,             /**< A vertical slider that has three thumbs instead of one, so it can show a minimum and maximum
+        ThreeValueVertical              /**< A vertical slider that has three thumbs instead of one, so it can show a minimum and maximum
                                              value, with the current value being somewhere between them.
                                              @see setMinValue, setMaxValue */
     };
@@ -128,7 +129,7 @@ public:
     /** Changes the type of slider interface being used.
 
         @param newStyle         the type of interface
-        @see setRotaryParameters, setVelocityBasedMode,
+        @see setRotaryParameters, setVelocityBasedMode
     */
     void setSliderStyle (SliderStyle newStyle);
 
@@ -141,19 +142,22 @@ public:
     struct RotaryParameters
     {
         /** The angle (in radians, clockwise from the top) at which
-            the slider's minimum value is represented. */
+            the slider's minimum value is represented.
+        */
         float startAngleRadians;
 
         /** The angle (in radians, clockwise from the top) at which
             the slider's maximum value is represented. This must be
-            greater than startAngleRadians. */
+            greater than startAngleRadians.
+        */
         float endAngleRadians;
 
         /** Determines what happens when a circular drag action rotates beyond
             the minimum or maximum angle. If true, the value will stop changing
             until the mouse moves back the way it came; if false, the value
             will snap back to the value nearest to the mouse. Note that this has
-            no effect if the drag mode is vertical or horizontal.*/
+            no effect if the drag mode is vertical or horizontal.
+        */
         bool stopAtEnd;
     };
 
@@ -207,11 +211,14 @@ public:
                                 the threshold is reached
         @param userCanPressKeyToSwapMode    if true, then the user can hold down the ctrl or command
                                 key to toggle velocity-sensitive mode
+        @param modifiersToSwapModes  this is a set of modifier flags which will be tested when determining
+                                whether to enable/disable velocity-sensitive mode
     */
     void setVelocityModeParameters (double sensitivity = 1.0,
                                     int threshold = 1,
                                     double offset = 0.0,
-                                    bool userCanPressKeyToSwapMode = true);
+                                    bool userCanPressKeyToSwapMode = true,
+                                    ModifierKeys::Flags modifiersToSwapModes = ModifierKeys::ctrlAltCommandModifiers);
 
     /** Returns the velocity sensitivity setting.
         @see setVelocityModeParameters
@@ -373,7 +380,8 @@ public:
                                 nearest interval if one has been set
         @param notification     can be one of the NotificationType values, to request
                                 a synchronous or asynchronous call to the valueChanged() method
-                                of any Slider::Listeners that are registered.
+                                of any Slider::Listeners that are registered. A notification will
+                                only be sent if the Slider's value has changed.
     */
     void setValue (double newValue, NotificationType notification = sendNotificationAsync);
 
@@ -400,18 +408,29 @@ public:
                    double newMaximum,
                    double newInterval = 0);
 
+    /** Sets the limits that the slider's value can take.
+
+        @param newRange     the range to allow
+        @param newInterval  the steps in which the value is allowed to increase - if this
+                            is not zero, the value will always be (newMinimum + (newInterval * an integer)).
+    */
+    void setRange (Range<double> newRange, double newInterval);
+
+    /** Returns the slider's range. */
+    Range<double> getRange() const noexcept;
+
     /** Returns the current maximum value.
-        @see setRange
+        @see setRange, getRange
     */
     double getMaximum() const noexcept;
 
     /** Returns the current minimum value.
-        @see setRange
+        @see setRange, getRange
     */
     double getMinimum() const noexcept;
 
     /** Returns the current step-size for values.
-        @see setRange
+        @see setRange, getRange
     */
     double getInterval() const noexcept;
 
@@ -445,7 +464,8 @@ public:
                                 interval if one has been set.
         @param notification     can be one of the NotificationType values, to request
                                 a synchronous or asynchronous call to the valueChanged() method
-                                of any Slider::Listeners that are registered.
+                                of any Slider::Listeners that are registered. A notification will
+                                only be sent if this value has changed.
         @param allowNudgingOfOtherValues  if false, this value will be restricted to being below the
                                         max value (in a two-value slider) or the mid value (in a three-value
                                         slider). If true, then if this value goes beyond those values,
@@ -485,7 +505,8 @@ public:
                                 interval if one has been set.
         @param notification     can be one of the NotificationType values, to request
                                 a synchronous or asynchronous call to the valueChanged() method
-                                of any Slider::Listeners that are registered.
+                                of any Slider::Listeners that are registered. A notification will
+                                only be sent if this value has changed.
         @param allowNudgingOfOtherValues  if false, this value will be restricted to being above the
                                         min value (in a two-value slider) or the mid value (in a three-value
                                         slider). If true, then if this value goes beyond those values,
@@ -508,7 +529,8 @@ public:
                                 nearest interval if one has been set.
         @param notification     can be one of the NotificationType values, to request
                                 a synchronous or asynchronous call to the valueChanged() method
-                                of any Slider::Listeners that are registered.
+                                of any Slider::Listeners that are registered. A notification will
+                                only be sent if one or more of the values has changed.
         @see setMaxValue, setMinValue, setValue
     */
     void setMinAndMaxValues (double newMinValue, double newMaxValue,
@@ -564,6 +586,16 @@ public:
     /** Removes a previously-registered listener. */
     void removeListener (Listener* listener);
 
+    //==============================================================================
+    /** You can assign a lambda to this callback object to have it called when the slider value is changed. */
+    std::function<void()> onValueChange;
+
+    /** You can assign a lambda to this callback object to have it called when the slider's drag begins. */
+    std::function<void()> onDragStart;
+
+    /** You can assign a lambda to this callback object to have it called when the slider's drag ends. */
+    std::function<void()> onDragEnd;
+
     //==============================================================================
     /** This lets you choose whether double-clicking moves the slider to a given position.
 
@@ -612,7 +644,7 @@ public:
     bool getSliderSnapsToMousePosition() const noexcept;
 
     /** If enabled, this gives the slider a pop-up bubble which appears while the
-        slider is being dragged.
+        slider is being dragged or hovered-over.
 
         This can be handy if your slider doesn't have a text-box, so that users can
         see the value just when they're changing it.
@@ -622,8 +654,15 @@ public:
         pass nullptr, the pop-up will be placed on the desktop instead (note that it's a
         transparent window, so if you're using an OS that can't do transparent windows
         you'll have to add it to a parent component instead).
+
+        By default the popup display shown when hovering will remain visible for 2 seconds,
+        but it is possible to change this by passing a different hoverTimeout value. A
+        value of -1 will cause the popup to remain until a mouseExit() occurs on the slider.
     */
-    void setPopupDisplayEnabled (bool isEnabled, Component* parentComponentToUse);
+    void setPopupDisplayEnabled (bool shouldShowOnMouseDrag,
+                                 bool shouldShowOnMouseHover,
+                                 Component* parentComponentToUse,
+                                 int hoverTimeout = 2000);
 
     /** If a popup display is enabled and is currently visible, this returns the component
         that is being shown, or nullptr if none is currently in use.
@@ -900,6 +939,12 @@ public:
     void focusOfChildComponentChanged (FocusChangeType) override;
     /** @internal */
     void colourChanged() override;
+    /** @internal */
+    void mouseMove (const MouseEvent&) override;
+    /** @internal */
+    void mouseExit (const MouseEvent&) override;
+    /** @internal */
+    void mouseEnter (const MouseEvent&) override;
 
 private:
     //==============================================================================
@@ -928,7 +973,5 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Slider)
 };
 
-/** This typedef is just for compatibility with old code - newer code should use the Slider::Listener class directly. */
-typedef Slider::Listener SliderListener;
 
-#endif   // JUCE_SLIDER_H_INCLUDED
+} // namespace juce
