@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_TEXTEDITOR_H_INCLUDED
-#define JUCE_TEXTEDITOR_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -206,8 +207,8 @@ public:
 
         textColourId             = 0x1000201, /**< The colour that will be used when text is added to the editor. Note
                                                    that because the editor can contain multiple colours, calling this
-                                                   method won't change the colour of existing text - to do that, call
-                                                   applyFontToAllText() after calling this method.*/
+                                                   method won't change the colour of existing text - to do that, use
+                                                   the applyColourToAllText() method */
 
         highlightColourId        = 0x1000202, /**< The colour with which to fill the background of highlighted sections of
                                                    the text - this can be transparent if you don't want to show any
@@ -237,16 +238,27 @@ public:
     void setFont (const Font& newFont);
 
     /** Applies a font to all the text in the editor.
-        This will also set the current font to use for any new text that's added.
+
+        If the changeCurrentFont argument is true then this will also set the
+        new font as the font to be used for any new text that's added.
+
         @see setFont
     */
-    void applyFontToAllText (const Font& newFont);
+    void applyFontToAllText (const Font& newFont, bool changeCurrentFont = true);
 
     /** Returns the font that's currently being used for new text.
+
         @see setFont
     */
     const Font& getFont() const noexcept            { return currentFont; }
 
+    /** Applies a colour to all the text in the editor.
+
+        If the changeCurrentTextColour argument is true then this will also set the
+        new colour as the colour to be used for any new text that's added.
+    */
+    void applyColourToAllText (const Colour& newColour, bool changeCurrentTextColour = true);
+
     //==============================================================================
     /** If set to true, focusing on the editor will highlight all its text.
 
@@ -306,6 +318,19 @@ public:
     */
     void removeListener (Listener* listenerToRemove);
 
+    //==============================================================================
+    /** You can assign a lambda to this callback object to have it called when the text is changed. */
+    std::function<void()> onTextChange;
+
+    /** You can assign a lambda to this callback object to have it called when the return key is pressed. */
+    std::function<void()> onReturnKey;
+
+    /** You can assign a lambda to this callback object to have it called when the escape key is pressed. */
+    std::function<void()> onEscapeKey;
+
+    /** You can assign a lambda to this callback object to have it called when the editor loses key focus. */
+    std::function<void()> onFocusLost;
+
     //==============================================================================
     /** Returns the entire contents of the editor. */
     String getText() const;
@@ -463,6 +488,18 @@ public:
     */
     void setScrollToShowCursor (bool shouldScrollToShowCaret);
 
+    /** Modifies the horizontal justification of the text within the editor window. */
+    void setJustification (Justification newJustification);
+
+    /** Sets the line spacing of the TextEditor.
+        The default (and minimum) value is 1.0 and values > 1.0 will increase the line spacing as a
+        multiple of the line height e.g. for double-spacing call this method with an argument of 2.0.
+    */
+    void setLineSpacing (float newLineSpacing) noexcept             { lineSpacing = jmax (1.0f, newLineSpacing); }
+
+    /** Returns the current line spacing of the TextEditor. */
+    float getLineSpacing() const noexcept                           { return lineSpacing; }
+
     //==============================================================================
     void moveCaretToEnd();
     bool moveCaretLeft (bool moveInWholeWordSteps, bool selecting);
@@ -629,15 +666,13 @@ public:
     /** @internal */
     void enablementChanged() override;
     /** @internal */
-    void colourChanged() override;
-    /** @internal */
     void lookAndFeelChanged() override;
     /** @internal */
     void parentHierarchyChanged() override;
     /** @internal */
     bool isTextInputActive() const override;
     /** @internal */
-    void setTemporaryUnderlining (const Array<Range<int> >&) override;
+    void setTemporaryUnderlining (const Array<Range<int>>&) override;
     /** @internal */
     VirtualKeyboardType getKeyboardType() override    { return keyboardType; }
 
@@ -660,59 +695,61 @@ protected:
 
 private:
     //==============================================================================
-    class Iterator;
     JUCE_PUBLIC_IN_DLL_BUILD (class UniformTextSection)
-    class TextHolderComponent;
-    class InsertAction;
-    class RemoveAction;
-    friend class InsertAction;
-    friend class RemoveAction;
+    struct Iterator;
+    struct TextHolderComponent;
+    struct TextEditorViewport;
+    struct InsertAction;
+    struct RemoveAction;
 
     ScopedPointer<Viewport> viewport;
     TextHolderComponent* textHolder;
-    BorderSize<int> borderSize;
+    BorderSize<int> borderSize { 1, 1, 1, 3 };
+    Justification justification { Justification::left };
 
-    bool readOnly;
-    bool caretVisible;
-    bool multiline;
-    bool wordWrap;
-    bool returnKeyStartsNewLine;
-    bool popupMenuEnabled;
-    bool selectAllTextWhenFocused;
-    bool scrollbarVisible;
-    bool wasFocused;
-    bool keepCaretOnScreen;
-    bool tabKeyUsed;
-    bool menuActive;
-    bool valueTextNeedsUpdating;
-    bool consumeEscAndReturnKeys;
-    bool styleChanged;
+    bool readOnly = false;
+    bool caretVisible = true;
+    bool multiline = false;
+    bool wordWrap = false;
+    bool returnKeyStartsNewLine = false;
+    bool popupMenuEnabled = true;
+    bool selectAllTextWhenFocused = false;
+    bool scrollbarVisible = true;
+    bool wasFocused = false;
+    bool keepCaretOnScreen = true;
+    bool tabKeyUsed = false;
+    bool menuActive = false;
+    bool valueTextNeedsUpdating = false;
+    bool consumeEscAndReturnKeys = true;
 
     UndoManager undoManager;
     ScopedPointer<CaretComponent> caret;
     Range<int> selection;
-    int leftIndent, topIndent;
-    unsigned int lastTransactionTime;
-    Font currentFont;
-    mutable int totalNumChars;
-    int caretPosition;
+    int leftIndent = 4, topIndent = 4;
+    unsigned int lastTransactionTime = 0;
+    Font currentFont { 14.0f };
+    mutable int totalNumChars = 0;
+    int caretPosition = 0;
     OwnedArray<UniformTextSection> sections;
     String textToShowWhenEmpty;
     Colour colourForTextWhenEmpty;
     juce_wchar passwordCharacter;
     OptionalScopedPointer<InputFilter> inputFilter;
     Value textValue;
-    VirtualKeyboardType keyboardType;
+    VirtualKeyboardType keyboardType = TextInputTarget::textKeyboard;
+    float lineSpacing = 1.0f;
 
-    enum
+    enum DragType
     {
         notDragging,
         draggingSelectionStart,
         draggingSelectionEnd
-    } dragType;
+    };
+
+    DragType dragType = notDragging;
 
     ListenerList<Listener> listeners;
-    Array<Range<int> > underlinedSections;
+    Array<Range<int>> underlinedSections;
 
     void moveCaret (int newCaretPos);
     void moveCaretTo (int newPosition, bool isSelecting);
@@ -721,10 +758,11 @@ private:
     void coalesceSimilarSections();
     void splitSection (int sectionIndex, int charToSplitAt);
     void clearInternal (UndoManager*);
-    void insert (const String&, int insertIndex, const Font&, const Colour, UndoManager*, int newCaretPos);
+    void insert (const String&, int insertIndex, const Font&, Colour, UndoManager*, int newCaretPos);
     void reinsert (int insertIndex, const OwnedArray<UniformTextSection>&);
-    void remove (Range<int> range, UndoManager*, int caretPositionToMoveTo);
-    void getCharPosition (int index, float& x, float& y, float& lineHeight) const;
+    void remove (Range<int>, UndoManager*, int caretPositionToMoveTo);
+    void getCharPosition (int index, Point<float>&, float& lineHeight) const;
+    Rectangle<float> getCaretRectangleFloat() const;
     void updateCaretPosition();
     void updateValueFromText();
     void textWasChangedByValue();
@@ -732,11 +770,10 @@ private:
     int findWordBreakAfter (int position) const;
     int findWordBreakBefore (int position) const;
     bool moveCaretWithTransaction (int newPos, bool selecting);
-    friend class TextHolderComponent;
-    friend class TextEditorViewport;
     void drawContent (Graphics&);
     void updateTextHolderSize();
     float getWordWrapWidth() const;
+    float getJustificationWidth() const;
     void timerCallbackInt();
     void repaintText (Range<int>);
     void scrollByLines (int deltaLines);
@@ -746,8 +783,5 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (TextEditor)
 };
 
-/** This typedef is just for compatibility with old code - newer code should use the TextEditor::Listener class directly. */
-typedef TextEditor::Listener TextEditorListener;
 
-
-#endif   // JUCE_TEXTEDITOR_H_INCLUDED
+} // namespace juce
