@@ -2,36 +2,41 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 const char* const Toolbar::toolbarDragDescriptor = "_toolbarItem_";
 
 //==============================================================================
 class Toolbar::Spacer  : public ToolbarItemComponent
 {
 public:
-    Spacer (const int itemId_, const float fixedSize_, const bool drawBar_)
-        : ToolbarItemComponent (itemId_, String(), false),
-          fixedSize (fixedSize_),
-          drawBar (drawBar_)
+    Spacer (int itemID, float sizeToUse, bool shouldDrawBar)
+        : ToolbarItemComponent (itemID, {}, false),
+          fixedSize (sizeToUse),
+          drawBar (shouldDrawBar)
     {
         setWantsKeyboardFocus (false);
     }
@@ -73,14 +78,14 @@ public:
 
     void paint (Graphics& g) override
     {
-        const int w = getWidth();
-        const int h = getHeight();
+        auto w = getWidth();
+        auto h = getHeight();
 
         if (drawBar)
         {
             g.setColour (findColour (Toolbar::separatorColourId, true));
 
-            const float thickness = 0.2f;
+            auto thickness = 0.2f;
 
             if (isToolbarVertical())
                 g.fillRect (w * 0.1f, h * (0.5f - thickness * 0.5f), w * 0.8f, h * thickness);
@@ -92,8 +97,8 @@ public:
         {
             g.setColour (findColour (Toolbar::separatorColourId, true));
 
-            const int indentX = jmin (2, (w - 3) / 2);
-            const int indentY = jmin (2, (h - 3) / 2);
+            auto indentX = jmin (2, (w - 3) / 2);
+            auto indentY = jmin (2, (h - 3) / 2);
             g.drawRect (indentX, indentY, w - indentX * 2, h - indentY * 2, 1);
 
             if (fixedSize <= 0)
@@ -132,8 +137,8 @@ public:
                 }
 
                 Path p;
-                p.addArrow (Line<float> (x1, y1, x2, y2), 1.5f, hw, hl);
-                p.addArrow (Line<float> (x3, y3, x4, y4), 1.5f, hw, hl);
+                p.addArrow ({ x1, y1, x2, y2 }, 1.5f, hw, hl);
+                p.addArrow ({ x3, y3, x4, y4 }, 1.5f, hw, hl);
                 g.fillPath (p);
             }
         }
@@ -150,14 +155,14 @@ private:
 class Toolbar::MissingItemsComponent  : public PopupMenu::CustomComponent
 {
 public:
-    MissingItemsComponent (Toolbar& bar, const int h)
+    MissingItemsComponent (Toolbar& bar, int h)
         : PopupMenu::CustomComponent (true),
           owner (&bar),
           height (h)
     {
         for (int i = bar.items.size(); --i >= 0;)
         {
-            ToolbarItemComponent* const tc = bar.items.getUnchecked(i);
+            auto* tc = bar.items.getUnchecked(i);
 
             if (dynamic_cast<Spacer*> (tc) == nullptr && ! tc->isVisible())
             {
@@ -175,10 +180,10 @@ public:
         {
             for (int i = 0; i < getNumChildComponents(); ++i)
             {
-                if (ToolbarItemComponent* const tc = dynamic_cast<ToolbarItemComponent*> (getChildComponent (i)))
+                if (auto* tc = dynamic_cast<ToolbarItemComponent*> (getChildComponent (i)))
                 {
                     tc->setVisible (false);
-                    const int index = oldIndexes.removeAndReturn (i);
+                    auto index = oldIndexes.removeAndReturn (i);
                     owner->addChildComponent (tc, index);
                     --i;
                 }
@@ -191,13 +196,13 @@ public:
     void layout (const int preferredWidth)
     {
         const int indent = 8;
-        int x = indent;
-        int y = indent;
+        auto x = indent;
+        auto y = indent;
         int maxX = 0;
 
-        for (int i = 0; i < getNumChildComponents(); ++i)
+        for (auto* c : getChildren())
         {
-            if (ToolbarItemComponent* const tc = dynamic_cast<ToolbarItemComponent*> (getChildComponent (i)))
+            if (auto* tc = dynamic_cast<ToolbarItemComponent*> (c))
             {
                 int preferredSize = 1, minSize = 1, maxSize = 1;
 
@@ -237,14 +242,12 @@ private:
 
 //==============================================================================
 Toolbar::Toolbar()
-    : vertical (false),
-      isEditingActive (false),
-      toolbarStyle (Toolbar::iconsOnly)
 {
-    addChildComponent (missingItemsButton = getLookAndFeel().createToolbarMissingItemsButton (*this));
+    missingItemsButton.reset (getLookAndFeel().createToolbarMissingItemsButton (*this));
+    addChildComponent (missingItemsButton.get());
 
     missingItemsButton->setAlwaysOnTop (true);
-    missingItemsButton->addListener (this);
+    missingItemsButton->onClick = [this] { showMissingItems(); };
 }
 
 Toolbar::~Toolbar()
@@ -283,7 +286,7 @@ void Toolbar::addItemInternal (ToolbarItemFactory& factory,
     // An ID can't be zero - this might indicate a mistake somewhere?
     jassert (itemId != 0);
 
-    if (ToolbarItemComponent* const tc = createItem (factory, itemId))
+    if (auto* tc = createItem (factory, itemId))
     {
        #if JUCE_DEBUG
         Array<int> allowedIds;
@@ -299,9 +302,7 @@ void Toolbar::addItemInternal (ToolbarItemFactory& factory,
     }
 }
 
-void Toolbar::addItem (ToolbarItemFactory& factory,
-                       const int itemId,
-                       const int insertIndex)
+void Toolbar::addItem (ToolbarItemFactory& factory, int itemId, int insertIndex)
 {
     addItemInternal (factory, itemId, insertIndex);
     resized();
@@ -314,8 +315,8 @@ void Toolbar::addDefaultItems (ToolbarItemFactory& factoryToUse)
 
     clear();
 
-    for (int i = 0; i < ids.size(); ++i)
-        addItemInternal (factoryToUse, ids.getUnchecked (i), -1);
+    for (auto i : ids)
+        addItemInternal (factoryToUse, i, -1);
 
     resized();
 }
@@ -328,7 +329,7 @@ void Toolbar::removeToolbarItem (const int itemIndex)
 
 ToolbarItemComponent* Toolbar::removeAndReturnItem (const int itemIndex)
 {
-    if (ToolbarItemComponent* const tc = items.removeAndReturn (itemIndex))
+    if (auto* tc = items.removeAndReturn (itemIndex))
     {
         removeChildComponent (tc);
         resized();
@@ -345,7 +346,7 @@ int Toolbar::getNumItems() const noexcept
 
 int Toolbar::getItemId (const int itemIndex) const noexcept
 {
-    if (ToolbarItemComponent* const tc = getItemComponent (itemIndex))
+    if (auto* tc = getItemComponent (itemIndex))
         return tc->getItemId();
 
     return 0;
@@ -353,7 +354,7 @@ int Toolbar::getItemId (const int itemIndex) const noexcept
 
 ToolbarItemComponent* Toolbar::getItemComponent (const int itemIndex) const noexcept
 {
-    return items [itemIndex];
+    return items[itemIndex];
 }
 
 ToolbarItemComponent* Toolbar::getNextActiveComponent (int index, const int delta) const
@@ -362,7 +363,7 @@ ToolbarItemComponent* Toolbar::getNextActiveComponent (int index, const int delt
     {
         index += delta;
 
-        if (ToolbarItemComponent* const tc = getItemComponent (index))
+        if (auto* tc = getItemComponent (index))
         {
             if (tc->isActive)
                 return tc;
@@ -404,8 +405,8 @@ bool Toolbar::restoreFromString (ToolbarItemFactory& factoryToUse,
 
     clear();
 
-    for (int i = 0; i < tokens.size(); ++i)
-        addItemInternal (factoryToUse, tokens[i].getIntValue(), -1);
+    for (auto& t : tokens)
+        addItemInternal (factoryToUse, t.getIntValue(), -1);
 
     resized();
     return true;
@@ -441,22 +442,20 @@ void Toolbar::resized()
     updateAllItemPositions (false);
 }
 
-void Toolbar::updateAllItemPositions (const bool animate)
+void Toolbar::updateAllItemPositions (bool animate)
 {
     if (getWidth() > 0 && getHeight() > 0)
     {
         StretchableObjectResizer resizer;
 
-        for (int i = 0; i < items.size(); ++i)
+        for (auto* tc : items)
         {
-            ToolbarItemComponent* const tc = items.getUnchecked(i);
-
             tc->setEditingMode (isEditingActive ? ToolbarItemComponent::editableOnToolbar
                                                 : ToolbarItemComponent::normalMode);
 
             tc->setStyle (toolbarStyle);
 
-            Spacer* const spacer = dynamic_cast<Spacer*> (tc);
+            auto* spacer = dynamic_cast<Spacer*> (tc);
 
             int preferredSize = 1, minSize = 1, maxSize = 1;
 
@@ -483,7 +482,7 @@ void Toolbar::updateAllItemPositions (const bool animate)
 
         const bool itemsOffTheEnd = totalLength > getLength();
 
-        const int extrasButtonSize = getThickness() / 2;
+        auto extrasButtonSize = getThickness() / 2;
         missingItemsButton->setSize (extrasButtonSize, extrasButtonSize);
         missingItemsButton->setVisible (itemsOffTheEnd);
         missingItemsButton->setEnabled (! isEditingActive);
@@ -495,26 +494,26 @@ void Toolbar::updateAllItemPositions (const bool animate)
             missingItemsButton->setCentrePosition (getWidth() - 4 - extrasButtonSize / 2,
                                                    getHeight() / 2);
 
-        const int maxLength = itemsOffTheEnd ? (vertical ? missingItemsButton->getY()
-                                                         : missingItemsButton->getX()) - 4
-                                             : getLength();
+        auto maxLength = itemsOffTheEnd ? (vertical ? missingItemsButton->getY()
+                                                    : missingItemsButton->getX()) - 4
+                                        : getLength();
 
         int pos = 0, activeIndex = 0;
-        for (int i = 0; i < items.size(); ++i)
-        {
-            ToolbarItemComponent* const tc = items.getUnchecked(i);
 
+        for (auto* tc : items)
+        {
             if (tc->isActive)
             {
-                const int size = (int) resizer.getItemSize (activeIndex++);
+                auto size = (int) resizer.getItemSize (activeIndex++);
 
                 Rectangle<int> newBounds;
+
                 if (vertical)
                     newBounds.setBounds (0, pos, getWidth(), size);
                 else
                     newBounds.setBounds (pos, 0, size, getHeight());
 
-                ComponentAnimator& animator = Desktop::getInstance().getAnimator();
+                auto& animator = Desktop::getInstance().getAnimator();
 
                 if (animate)
                 {
@@ -536,7 +535,7 @@ void Toolbar::updateAllItemPositions (const bool animate)
 }
 
 //==============================================================================
-void Toolbar::buttonClicked (Button*)
+void Toolbar::showMissingItems()
 {
     jassert (missingItemsButton->isShowing());
 
@@ -544,7 +543,7 @@ void Toolbar::buttonClicked (Button*)
     {
         PopupMenu m;
         m.addCustomItem (1, new MissingItemsComponent (*this, getThickness()));
-        m.showMenuAsync (PopupMenu::Options().withTargetComponent (missingItemsButton), nullptr);
+        m.showMenuAsync (PopupMenu::Options().withTargetComponent (missingItemsButton.get()), nullptr);
     }
 }
 
@@ -556,13 +555,13 @@ bool Toolbar::isInterestedInDragSource (const SourceDetails& dragSourceDetails)
 
 void Toolbar::itemDragMove (const SourceDetails& dragSourceDetails)
 {
-    if (ToolbarItemComponent* const tc = dynamic_cast<ToolbarItemComponent*> (dragSourceDetails.sourceComponent.get()))
+    if (auto* tc = dynamic_cast<ToolbarItemComponent*> (dragSourceDetails.sourceComponent.get()))
     {
         if (! items.contains (tc))
         {
             if (tc->getEditingMode() == ToolbarItemComponent::editableOnPalette)
             {
-                if (ToolbarItemPalette* const palette = tc->findParentComponentOfClass<ToolbarItemPalette>())
+                if (auto* palette = tc->findParentComponentOfClass<ToolbarItemPalette>())
                     palette->replaceComponent (*tc);
             }
             else
@@ -575,22 +574,22 @@ void Toolbar::itemDragMove (const SourceDetails& dragSourceDetails)
             updateAllItemPositions (true);
         }
 
-        ComponentAnimator& animator = Desktop::getInstance().getAnimator();
+        auto& animator = Desktop::getInstance().getAnimator();
 
         for (int i = getNumItems(); --i >= 0;)
         {
-            const int currentIndex = items.indexOf (tc);
-            int newIndex = currentIndex;
+            auto currentIndex = items.indexOf (tc);
+            auto newIndex = currentIndex;
 
-            const int dragObjectLeft = vertical ? (dragSourceDetails.localPosition.getY() - tc->dragOffsetY)
-                                                : (dragSourceDetails.localPosition.getX() - tc->dragOffsetX);
-            const int dragObjectRight = dragObjectLeft + (vertical ? tc->getHeight() : tc->getWidth());
+            auto dragObjectLeft = vertical ? (dragSourceDetails.localPosition.getY() - tc->dragOffsetY)
+                                           : (dragSourceDetails.localPosition.getX() - tc->dragOffsetX);
+            auto dragObjectRight = dragObjectLeft + (vertical ? tc->getHeight() : tc->getWidth());
 
-            const Rectangle<int> current (animator.getComponentDestination (getChildComponent (newIndex)));
+            auto current = animator.getComponentDestination (getChildComponent (newIndex));
 
-            if (ToolbarItemComponent* const prev = getNextActiveComponent (newIndex, -1))
+            if (auto* prev = getNextActiveComponent (newIndex, -1))
             {
-                const Rectangle<int> previousPos (animator.getComponentDestination (prev));
+                auto previousPos = animator.getComponentDestination (prev);
 
                 if (std::abs (dragObjectLeft - (vertical ? previousPos.getY() : previousPos.getX()))
                      < std::abs (dragObjectRight - (vertical ? current.getBottom() : current.getRight())))
@@ -599,9 +598,9 @@ void Toolbar::itemDragMove (const SourceDetails& dragSourceDetails)
                 }
             }
 
-            if (ToolbarItemComponent* const next = getNextActiveComponent (newIndex, 1))
+            if (auto* next = getNextActiveComponent (newIndex, 1))
             {
-                const Rectangle<int> nextPos (animator.getComponentDestination (next));
+                auto nextPos = animator.getComponentDestination (next);
 
                 if (std::abs (dragObjectLeft - (vertical ? current.getY() : current.getX()))
                      > std::abs (dragObjectRight - (vertical ? nextPos.getBottom() : nextPos.getRight())))
@@ -624,7 +623,7 @@ void Toolbar::itemDragMove (const SourceDetails& dragSourceDetails)
 
 void Toolbar::itemDragExit (const SourceDetails& dragSourceDetails)
 {
-    if (ToolbarItemComponent* const tc = dynamic_cast<ToolbarItemComponent*> (dragSourceDetails.sourceComponent.get()))
+    if (auto* tc = dynamic_cast<ToolbarItemComponent*> (dragSourceDetails.sourceComponent.get()))
     {
         if (isParentOf (tc))
         {
@@ -637,7 +636,7 @@ void Toolbar::itemDragExit (const SourceDetails& dragSourceDetails)
 
 void Toolbar::itemDropped (const SourceDetails& dragSourceDetails)
 {
-    if (ToolbarItemComponent* const tc = dynamic_cast<ToolbarItemComponent*> (dragSourceDetails.sourceComponent.get()))
+    if (auto* tc = dynamic_cast<ToolbarItemComponent*> (dragSourceDetails.sourceComponent.get()))
         tc->setState (Button::buttonNormal);
 }
 
@@ -675,8 +674,8 @@ public:
 
     void positionNearBar()
     {
-        const Rectangle<int> screenSize (toolbar.getParentMonitorArea());
-        Point<int> pos (toolbar.getScreenPosition());
+        auto screenSize = toolbar.getParentMonitorArea();
+        auto pos = toolbar.getScreenPosition();
         const int gap = 8;
 
         if (toolbar.isVertical())
@@ -702,17 +701,15 @@ public:
 private:
     Toolbar& toolbar;
 
-    class CustomiserPanel  : public Component,
-                             private ComboBoxListener, // (can't use ComboBox::Listener due to idiotic VC2005 bug)
-                             private ButtonListener
+    class CustomiserPanel  : public Component
     {
     public:
         CustomiserPanel (ToolbarItemFactory& tbf, Toolbar& bar, int optionFlags)
-          : factory (tbf), toolbar (bar), palette (tbf, bar),
-            instructions (String(), TRANS ("You can drag the items above and drop them onto a toolbar to add them.")
-                                      + "\n\n"
-                                      + TRANS ("Items on the toolbar can also be dragged around to change their order, or dragged off the edge to delete them.")),
-            defaultButton (TRANS ("Restore to default set of items"))
+           : factory (tbf), toolbar (bar), palette (tbf, bar),
+             instructions ({}, TRANS ("You can drag the items above and drop them onto a toolbar to add them.")
+                                 + "\n\n"
+                                 + TRANS ("Items on the toolbar can also be dragged around to change their order, or dragged off the edge to delete them.")),
+             defaultButton (TRANS ("Restore to default set of items"))
         {
             addAndMakeVisible (palette);
 
@@ -737,13 +734,13 @@ private:
 
                 styleBox.setSelectedId (selectedStyle);
 
-                styleBox.addListener (this);
+                styleBox.onChange = [this] { updateStyle(); };
             }
 
             if ((optionFlags & Toolbar::showResetToDefaultsButton) != 0)
             {
                 addAndMakeVisible (defaultButton);
-                defaultButton.addListener (this);
+                defaultButton.onClick = [this] { toolbar.addDefaultItems (factory); };
             }
 
             addAndMakeVisible (instructions);
@@ -752,7 +749,7 @@ private:
             setSize (500, 300);
         }
 
-        void comboBoxChanged (ComboBox*) override
+        void updateStyle()
         {
             switch (styleBox.getSelectedId())
             {
@@ -764,16 +761,11 @@ private:
             palette.resized(); // to make it update the styles
         }
 
-        void buttonClicked (Button*) override
-        {
-            toolbar.addDefaultItems (factory);
-        }
-
         void paint (Graphics& g) override
         {
             Colour background;
 
-            if (DialogWindow* const dw = findParentComponentOfClass<DialogWindow>())
+            if (auto* dw = findParentComponentOfClass<DialogWindow>())
                 background = dw->getBackgroundColour();
 
             g.setColour (background.contrasting().withAlpha (0.3f));
@@ -809,3 +801,5 @@ void Toolbar::showCustomisationDialog (ToolbarItemFactory& factory, const int op
     (new CustomisationDialog (factory, *this, optionFlags))
         ->enterModalState (true, nullptr, true);
 }
+
+} // namespace juce
