@@ -2,28 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_TREEVIEW_H_INCLUDED
-#define JUCE_TREEVIEW_H_INCLUDED
+namespace juce
+{
 
 class TreeView;
 
@@ -54,15 +56,12 @@ public:
 
     //==============================================================================
     /** Returns the number of sub-items that have been added to this item.
-
         Note that this doesn't mean much if the node isn't open.
-
         @see getSubItem, mightContainSubItems, addSubItem
     */
     int getNumSubItems() const noexcept;
 
     /** Returns one of the item's sub-items.
-
         Remember that the object returned might get deleted at any time when its parent
         item is closed or refreshed, depending on the nature of the items you're using.
 
@@ -183,7 +182,7 @@ public:
 
     /** Selects or deselects the item.
         If shouldNotify == sendNotification, then a callback will be made
-        to itemSelectionChanged()
+        to itemSelectionChanged() if the item's selection has changed.
     */
     void setSelected (bool shouldBeSelected,
                       bool deselectOtherItemsFirst,
@@ -369,7 +368,7 @@ public:
 
         @see itemDoubleClicked
     */
-    virtual void itemClicked (const MouseEvent& e);
+    virtual void itemClicked (const MouseEvent&);
 
     /** Called when the user double-clicks on this item.
 
@@ -385,7 +384,7 @@ public:
 
         @see itemClicked
     */
-    virtual void itemDoubleClicked (const MouseEvent& e);
+    virtual void itemDoubleClicked (const MouseEvent&);
 
     /** Called when the item is selected or deselected.
 
@@ -568,7 +567,7 @@ public:
     class OpennessRestorer
     {
     public:
-        OpennessRestorer (TreeViewItem& treeViewItem);
+        OpennessRestorer (TreeViewItem&);
         ~OpennessRestorer();
 
     private:
@@ -580,11 +579,11 @@ public:
 
 private:
     //==============================================================================
-    TreeView* ownerView;
-    TreeViewItem* parentItem;
+    TreeView* ownerView = nullptr;
+    TreeViewItem* parentItem = nullptr;
     OwnedArray<TreeViewItem> subItems;
-    int y, itemHeight, totalHeight, itemWidth, totalWidth;
-    int uid;
+    int y = 0, itemHeight = 0, totalHeight = 0, itemWidth = 0, totalWidth = 0;
+    int uid = 0;
     bool selected           : 1;
     bool redrawNeeded       : 1;
     bool drawLinesInside    : 1;
@@ -909,12 +908,12 @@ private:
 
     ScopedPointer<TreeViewport> viewport;
     CriticalSection nodeAlterationLock;
-    TreeViewItem* rootItem;
+    TreeViewItem* rootItem = nullptr;
     ScopedPointer<InsertPointHighlight> dragInsertPointHighlight;
     ScopedPointer<TargetGroupHighlight> dragTargetGroupHighlight;
-    int indentSize;
-    bool defaultOpenness, needsRecalculating, rootItemVisible;
-    bool multiSelectEnabled, openCloseButtonsVisible;
+    int indentSize = -1;
+    bool defaultOpenness = false, needsRecalculating = true, rootItemVisible = true;
+    bool multiSelectEnabled = false, openCloseButtonsVisible = true;
 
     void itemsChanged() noexcept;
     void recalculateIfNeeded();
@@ -937,4 +936,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (TreeView)
 };
 
-#endif   // JUCE_TREEVIEW_H_INCLUDED
+} // namespace juce
