@@ -2,43 +2,45 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 static uint32 lastUniquePeerID = 1;
 
 //==============================================================================
-ComponentPeer::ComponentPeer (Component& comp, const int flags)
+ComponentPeer::ComponentPeer (Component& comp, int flags)
     : component (comp),
       styleFlags (flags),
-      constrainer (nullptr),
-      lastDragAndDropCompUnderMouse (nullptr),
-      uniqueID (lastUniquePeerID += 2), // increment by 2 so that this can never hit 0
-      isWindowMinimised (false)
+      uniqueID (lastUniquePeerID += 2) // increment by 2 so that this can never hit 0
 {
     Desktop::getInstance().peers.add (this);
 }
 
 ComponentPeer::~ComponentPeer()
 {
-    Desktop& desktop = Desktop::getInstance();
+    auto& desktop = Desktop::getInstance();
     desktop.peers.removeFirstMatchingValue (this);
     desktop.triggerFocusCallback();
 }
@@ -56,15 +58,9 @@ ComponentPeer* ComponentPeer::getPeer (const int index) noexcept
 
 ComponentPeer* ComponentPeer::getPeerFor (const Component* const component) noexcept
 {
-    const Array<ComponentPeer*>& peers = Desktop::getInstance().peers;
-
-    for (int i = peers.size(); --i >= 0;)
-    {
-        ComponentPeer* const peer = peers.getUnchecked(i);
-
+    for (auto* peer : Desktop::getInstance().peers)
         if (&(peer->getComponent()) == component)
             return peer;
-    }
 
     return nullptr;
 }
@@ -85,21 +81,22 @@ bool ComponentPeer::isKioskMode() const
 }
 
 //==============================================================================
-void ComponentPeer::handleMouseEvent (int touchIndex, Point<float> pos, ModifierKeys newMods, float newPressure, int64 time)
+void ComponentPeer::handleMouseEvent (MouseInputSource::InputSourceType type, Point<float> pos, ModifierKeys newMods,
+                                      float newPressure, float newOrientation, int64 time, PenDetails pen, int touchIndex)
 {
-    if (MouseInputSource* mouse = Desktop::getInstance().mouseSources->getOrCreateMouseInputSource (touchIndex))
-        MouseInputSource (*mouse).handleEvent (*this, pos, time, newMods, newPressure);
+    if (auto* mouse = Desktop::getInstance().mouseSources->getOrCreateMouseInputSource (type, touchIndex))
+        MouseInputSource (*mouse).handleEvent (*this, pos, time, newMods, newPressure, newOrientation, pen);
 }
 
-void ComponentPeer::handleMouseWheel (int touchIndex, Point<float> pos, int64 time, const MouseWheelDetails& wheel)
+void ComponentPeer::handleMouseWheel (MouseInputSource::InputSourceType type, Point<float> pos, int64 time, const MouseWheelDetails& wheel, int touchIndex)
 {
-    if (MouseInputSource* mouse = Desktop::getInstance().mouseSources->getOrCreateMouseInputSource (touchIndex))
+    if (auto* mouse = Desktop::getInstance().mouseSources->getOrCreateMouseInputSource (type, touchIndex))
         MouseInputSource (*mouse).handleWheel (*this, pos, time, wheel);
 }
 
-void ComponentPeer::handleMagnifyGesture (int touchIndex, Point<float> pos, int64 time, float scaleFactor)
+void ComponentPeer::handleMagnifyGesture (MouseInputSource::InputSourceType type, Point<float> pos, int64 time, float scaleFactor, int touchIndex)
 {
-    if (MouseInputSource* mouse = Desktop::getInstance().mouseSources->getOrCreateMouseInputSource (touchIndex))
+    if (auto* mouse = Desktop::getInstance().mouseSources->getOrCreateMouseInputSource (type, touchIndex))
         MouseInputSource (*mouse).handleMagnifyGesture (*this, pos, time, scaleFactor);
 }
 
@@ -113,7 +110,7 @@ void ComponentPeer::handlePaint (LowLevelGraphicsContext& contextToPaintTo)
     if (component.isTransformed())
         g.addTransform (component.getTransform());
 
-    const Rectangle<int> peerBounds (getBounds());
+    auto peerBounds = getBounds();
 
     if (peerBounds.getWidth() != component.getWidth() || peerBounds.getHeight() != component.getHeight())
         // Tweak the scaling so that the component's integer size exactly aligns with the peer's scaled size
@@ -162,13 +159,13 @@ void ComponentPeer::handlePaint (LowLevelGraphicsContext& contextToPaintTo)
 
 Component* ComponentPeer::getTargetForKeyPress()
 {
-    Component* c = Component::getCurrentlyFocusedComponent();
+    auto* c = Component::getCurrentlyFocusedComponent();
 
     if (c == nullptr)
         c = &component;
 
     if (c->isCurrentlyBlockedByAnotherModalComponent())
-        if (Component* const currentModalComp = Component::getCurrentlyModalComponent())
+        if (auto* currentModalComp = Component::getCurrentlyModalComponent())
             c = currentModalComp;
 
     return c;
@@ -188,11 +185,11 @@ bool ComponentPeer::handleKeyPress (const KeyPress& keyInfo)
 {
     bool keyWasUsed = false;
 
-    for (Component* target = getTargetForKeyPress(); target != nullptr; target = target->getParentComponent())
+    for (auto* target = getTargetForKeyPress(); target != nullptr; target = target->getParentComponent())
     {
         const WeakReference<Component> deletionChecker (target);
 
-        if (const Array<KeyListener*>* const keyListeners = target->keyListeners)
+        if (auto* keyListeners = target->keyListeners.get())
         {
             for (int i = keyListeners->size(); --i >= 0;)
             {
@@ -210,7 +207,7 @@ bool ComponentPeer::handleKeyPress (const KeyPress& keyInfo)
         if (keyWasUsed || deletionChecker == nullptr)
             break;
 
-        if (Component* const currentlyFocused = Component::getCurrentlyFocusedComponent())
+        if (auto* currentlyFocused = Component::getCurrentlyFocusedComponent())
         {
             const bool isTab      = (keyInfo == KeyPress::tabKey);
             const bool isShiftTab = (keyInfo == KeyPress (KeyPress::tabKey, ModifierKeys::shiftModifier, 0));
@@ -234,7 +231,7 @@ bool ComponentPeer::handleKeyUpOrDown (const bool isKeyDown)
     ModifierKeys::updateCurrentModifiers();
     bool keyWasUsed = false;
 
-    for (Component* target = getTargetForKeyPress(); target != nullptr; target = target->getParentComponent())
+    for (auto* target = getTargetForKeyPress(); target != nullptr; target = target->getParentComponent())
     {
         const WeakReference<Component> deletionChecker (target);
 
@@ -243,7 +240,7 @@ bool ComponentPeer::handleKeyUpOrDown (const bool isKeyDown)
         if (keyWasUsed || deletionChecker == nullptr)
             break;
 
-        if (const Array<KeyListener*>* const keyListeners = target->keyListeners)
+        if (auto* keyListeners = target->keyListeners.get())
         {
             for (int i = keyListeners->size(); --i >= 0;)
             {
@@ -264,7 +261,7 @@ void ComponentPeer::handleModifierKeysChange()
 {
     ModifierKeys::updateCurrentModifiers();
 
-    Component* target = Desktop::getInstance().getMainMouseSource().getComponentUnderMouse();
+    auto* target = Desktop::getInstance().getMainMouseSource().getComponentUnderMouse();
 
     if (target == nullptr)
         target = Component::getCurrentlyFocusedComponent();
@@ -272,16 +269,15 @@ void ComponentPeer::handleModifierKeysChange()
     if (target == nullptr)
         target = &component;
 
-    if (target != nullptr)
-        target->internalModifierKeysChanged();
+    target->internalModifierKeysChanged();
 }
 
 TextInputTarget* ComponentPeer::findCurrentTextInputTarget()
 {
-    Component* const c = Component::getCurrentlyFocusedComponent();
+    auto* c = Component::getCurrentlyFocusedComponent();
 
     if (c == &component || component.isParentOf (c))
-        if (TextInputTarget* const ti = dynamic_cast<TextInputTarget*> (c))
+        if (auto* ti = dynamic_cast<TextInputTarget*> (c))
             if (ti->isTextInputActive())
                 return ti;
 
@@ -312,8 +308,8 @@ void ComponentPeer::handleMovedOrResized()
     {
         const WeakReference<Component> deletionChecker (&component);
 
-        Rectangle<int> newBounds (Component::ComponentHelpers::rawPeerPositionToLocal (component, getBounds()));
-        Rectangle<int> oldBounds (component.getBounds());
+        auto newBounds = Component::ComponentHelpers::rawPeerPositionToLocal (component, getBounds());
+        auto oldBounds = component.getBounds();
 
         const bool wasMoved   = (oldBounds.getPosition() != newBounds.getPosition());
         const bool wasResized = (oldBounds.getWidth() != newBounds.getWidth() || oldBounds.getHeight() != newBounds.getHeight());
@@ -347,7 +343,9 @@ void ComponentPeer::handleFocusGain()
 {
     ModifierKeys::updateCurrentModifiers();
 
-    if (component.isParentOf (lastFocusedComponent))
+    if (component.isParentOf (lastFocusedComponent)
+          && lastFocusedComponent->isShowing()
+          && lastFocusedComponent->getWantsKeyboardFocus())
     {
         Component::currentlyFocusedComponent = lastFocusedComponent;
         Desktop::getInstance().triggerFocusCallback();
@@ -428,7 +426,7 @@ namespace DragHelpers
 {
     static bool isFileDrag (const ComponentPeer::DragInfo& info)
     {
-        return info.files.size() > 0;
+        return ! info.files.isEmpty();
     }
 
     static bool isSuitableTarget (const ComponentPeer::DragInfo& info, Component* target)
@@ -443,7 +441,7 @@ namespace DragHelpers
                                  : dynamic_cast<TextDragAndDropTarget*> (target)->isInterestedInTextDrag (info.text);
     }
 
-    static Component* findDragAndDropTarget (Component* c, const ComponentPeer::DragInfo& info, Component* const lastOne)
+    static Component* findDragAndDropTarget (Component* c, const ComponentPeer::DragInfo& info, Component* lastOne)
     {
         for (; c != nullptr; c = c->getParentComponent())
             if (isSuitableTarget (info, c) && (c == lastOne || isInterested (info, c)))
@@ -451,40 +449,14 @@ namespace DragHelpers
 
         return nullptr;
     }
-
-    // We'll use an async message to deliver the drop, because if the target decides
-    // to run a modal loop, it can gum-up the operating system..
-    class AsyncDropMessage  : public CallbackMessage
-    {
-    public:
-        AsyncDropMessage (Component* c, const ComponentPeer::DragInfo& d)  : target (c), info (d) {}
-
-        void messageCallback() override
-        {
-            if (Component* const c = target.get())
-            {
-                if (isFileDrag (info))
-                    dynamic_cast<FileDragAndDropTarget*> (c)->filesDropped (info.files, info.position.x, info.position.y);
-                else
-                    dynamic_cast<TextDragAndDropTarget*> (c)->textDropped (info.text, info.position.x, info.position.y);
-            }
-        }
-
-    private:
-        WeakReference<Component> target;
-        const ComponentPeer::DragInfo info;
-
-        JUCE_DECLARE_NON_COPYABLE (AsyncDropMessage)
-    };
 }
 
 bool ComponentPeer::handleDragMove (const ComponentPeer::DragInfo& info)
 {
     ModifierKeys::updateCurrentModifiers();
 
-    Component* const compUnderMouse = component.getComponentAt (info.position);
-
-    Component* const lastTarget = dragAndDropTargetComponent;
+    auto* compUnderMouse = component.getComponentAt (info.position);
+    auto* lastTarget = dragAndDropTargetComponent.get();
     Component* newTarget = nullptr;
 
     if (compUnderMouse != lastDragAndDropCompUnderMouse)
@@ -507,7 +479,7 @@ bool ComponentPeer::handleDragMove (const ComponentPeer::DragInfo& info)
             if (DragHelpers::isSuitableTarget (info, newTarget))
             {
                 dragAndDropTargetComponent = newTarget;
-                const Point<int> pos (newTarget->getLocalPoint (&component, info.position));
+                auto pos = newTarget->getLocalPoint (&component, info.position);
 
                 if (DragHelpers::isFileDrag (info))
                     dynamic_cast<FileDragAndDropTarget*> (newTarget)->fileDragEnter (info.files, pos.x, pos.y);
@@ -524,7 +496,7 @@ bool ComponentPeer::handleDragMove (const ComponentPeer::DragInfo& info)
     if (! DragHelpers::isSuitableTarget (info, newTarget))
         return false;
 
-    const Point<int> pos (newTarget->getLocalPoint (&component, info.position));
+    auto pos = newTarget->getLocalPoint (&component, info.position);
 
     if (DragHelpers::isFileDrag (info))
         dynamic_cast<FileDragAndDropTarget*> (newTarget)->fileDragMove (info.files, pos.x, pos.y);
@@ -549,7 +521,7 @@ bool ComponentPeer::handleDragDrop (const ComponentPeer::DragInfo& info)
 {
     handleDragMove (info);
 
-    if (Component* const targetComp = dragAndDropTargetComponent)
+    if (WeakReference<Component> targetComp = dragAndDropTargetComponent)
     {
         dragAndDropTargetComponent = nullptr;
         lastDragAndDropCompUnderMouse = nullptr;
@@ -564,10 +536,22 @@ bool ComponentPeer::handleDragDrop (const ComponentPeer::DragInfo& info)
                     return true;
             }
 
-            ComponentPeer::DragInfo info2 (info);
-            info2.position = targetComp->getLocalPoint (&component, info.position);
+            ComponentPeer::DragInfo infoCopy (info);
+            infoCopy.position = targetComp->getLocalPoint (&component, info.position);
+
+            // We'll use an async message to deliver the drop, because if the target decides
+            // to run a modal loop, it can gum-up the operating system..
+            MessageManager::callAsync ([=]
+            {
+                if (auto* c = targetComp.get())
+                {
+                    if (DragHelpers::isFileDrag (info))
+                        dynamic_cast<FileDragAndDropTarget*> (c)->filesDropped (infoCopy.files, infoCopy.position.x, infoCopy.position.y);
+                    else
+                        dynamic_cast<TextDragAndDropTarget*> (c)->textDropped (infoCopy.text, infoCopy.position.x, infoCopy.position.y);
+                }
+            });
 
-            (new DragHelpers::AsyncDropMessage (targetComp, info2))->post();
             return true;
         }
     }
@@ -594,3 +578,5 @@ void ComponentPeer::setRepresentedFile (const File&)
 //==============================================================================
 int ComponentPeer::getCurrentRenderingEngine() const            { return 0; }
 void ComponentPeer::setCurrentRenderingEngine (int index)       { jassert (index == 0); ignoreUnused (index); }
+
+} // namespace juce
