@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_DOCUMENTWINDOW_H_INCLUDED
-#define JUCE_DOCUMENTWINDOW_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -105,8 +106,8 @@ public:
     /** Sets an icon to show in the title bar, next to the title.
 
         A copy is made internally of the image, so the caller can delete the
-        image after calling this. If 0 is passed-in, any existing icon will be
-        removed.
+        image after calling this. If an empty Image is passed-in, any existing icon
+        will be removed.
     */
     void setIcon (const Image& imageToUse);
 
@@ -275,12 +276,12 @@ public:
 
 private:
     //==============================================================================
-    int titleBarHeight, menuBarHeight, requiredButtons;
-    bool positionTitleBarButtonsOnLeft, drawTitleTextCentred;
+    int titleBarHeight = 26, menuBarHeight = 24, requiredButtons;
+    bool positionTitleBarButtonsOnLeft, drawTitleTextCentred = true;
     ScopedPointer<Button> titleBarButtons [3];
     Image titleBarIcon;
     ScopedPointer<Component> menuBar;
-    MenuBarModel* menuBarModel;
+    MenuBarModel* menuBarModel = nullptr;
 
     class ButtonListenerProxy;
     friend struct ContainerDeletePolicy<ButtonListenerProxy>;
@@ -291,5 +292,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (DocumentWindow)
 };
 
-
-#endif   // JUCE_DOCUMENTWINDOW_H_INCLUDED
+} // namespace juce
