@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 class CodeDocumentLine
 {
 public:
@@ -39,20 +44,20 @@ public:
 
     static void createLines (Array<CodeDocumentLine*>& newLines, StringRef text)
     {
-        String::CharPointerType t (text.text);
+        auto t = text.text;
         int charNumInFile = 0;
         bool finished = false;
 
         while (! (finished || t.isEmpty()))
         {
-            String::CharPointerType startOfLine (t);
-            int startOfLineInFile = charNumInFile;
+            auto startOfLine = t;
+            auto startOfLineInFile = charNumInFile;
             int lineLength = 0;
             int numNewLineChars = 0;
 
             for (;;)
             {
-                const juce_wchar c = t.getAndAdvance();
+                auto c = t.getAndAdvance();
 
                 if (c == 0)
                 {
@@ -102,9 +107,9 @@ public:
         lineLength = 0;
         lineLengthWithoutNewLines = 0;
 
-        for (String::CharPointerType t (line.getCharPointer());;)
+        for (auto t = line.getCharPointer();;)
         {
-            const juce_wchar c = t.getAndAdvance();
+            auto c = t.getAndAdvance();
 
             if (c == 0)
                 break;
@@ -121,35 +126,9 @@ public:
 };
 
 //==============================================================================
-CodeDocument::Iterator::Iterator (const CodeDocument& doc) noexcept
-    : document (&doc),
-      charPointer (nullptr),
-      line (0),
-      position (0)
-{
-}
+CodeDocument::Iterator::Iterator (const CodeDocument& doc) noexcept : document (&doc) {}
 
-CodeDocument::Iterator::Iterator (const CodeDocument::Iterator& other) noexcept
-    : document (other.document),
-      charPointer (other.charPointer),
-      line (other.line),
-      position (other.position)
-{
-}
-
-CodeDocument::Iterator& CodeDocument::Iterator::operator= (const CodeDocument::Iterator& other) noexcept
-{
-    document = other.document;
-    charPointer = other.charPointer;
-    line = other.line;
-    position = other.position;
-
-    return *this;
-}
-
-CodeDocument::Iterator::~Iterator() noexcept
-{
-}
+CodeDocument::Iterator::~Iterator() noexcept {}
 
 juce_wchar CodeDocument::Iterator::nextChar() noexcept
 {
@@ -157,13 +136,13 @@ juce_wchar CodeDocument::Iterator::nextChar() noexcept
     {
         if (charPointer.getAddress() == nullptr)
         {
-            if (const CodeDocumentLine* const l = document->lines[line])
+            if (auto* l = document->lines[line])
                 charPointer = l->line.getCharPointer();
             else
                 return 0;
         }
 
-        if (const juce_wchar result = charPointer.getAndAdvance())
+        if (auto result = charPointer.getAndAdvance())
         {
             if (charPointer.isEmpty())
             {
@@ -189,12 +168,10 @@ void CodeDocument::Iterator::skipToEndOfLine() noexcept
 {
     if (charPointer.getAddress() == nullptr)
     {
-        const CodeDocumentLine* const l = document->lines[line];
-
-        if (l == nullptr)
+        if (auto* l = document->lines[line])
+            charPointer = l->line.getCharPointer();
+        else
             return;
-
-        charPointer = l->line.getCharPointer();
     }
 
     position += (int) charPointer.length();
@@ -206,16 +183,16 @@ juce_wchar CodeDocument::Iterator::peekNextChar() const noexcept
 {
     if (charPointer.getAddress() == nullptr)
     {
-        if (const CodeDocumentLine* const l = document->lines[line])
+        if (auto* l = document->lines[line])
             charPointer = l->line.getCharPointer();
         else
             return 0;
     }
 
-    if (const juce_wchar c = *charPointer)
+    if (auto c = *charPointer)
         return c;
 
-    if (const CodeDocumentLine* const l = document->lines [line + 1])
+    if (auto* l = document->lines [line + 1])
         return l->line[0];
 
     return 0;
@@ -234,30 +211,26 @@ bool CodeDocument::Iterator::isEOF() const noexcept
 
 //==============================================================================
 CodeDocument::Position::Position() noexcept
-    : owner (nullptr), characterPos (0), line (0),
-      indexInLine (0), positionMaintained (false)
 {
 }
 
 CodeDocument::Position::Position (const CodeDocument& ownerDocument,
                                   const int lineNum, const int index) noexcept
     : owner (const_cast<CodeDocument*> (&ownerDocument)),
-      characterPos (0), line (lineNum),
-      indexInLine (index), positionMaintained (false)
+      line (lineNum), indexInLine (index)
 {
     setLineAndIndex (lineNum, index);
 }
 
-CodeDocument::Position::Position (const CodeDocument& ownerDocument, const int characterPos_) noexcept
-    : owner (const_cast<CodeDocument*> (&ownerDocument)),
-      positionMaintained (false)
+CodeDocument::Position::Position (const CodeDocument& ownerDocument, int pos) noexcept
+    : owner (const_cast<CodeDocument*> (&ownerDocument))
 {
-    setPosition (characterPos_);
+    setPosition (pos);
 }
 
 CodeDocument::Position::Position (const Position& other) noexcept
     : owner (other.owner), characterPos (other.characterPos), line (other.line),
-      indexInLine (other.indexInLine), positionMaintained (false)
+      indexInLine (other.indexInLine)
 {
     jassert (*this == other);
 }
@@ -319,7 +292,7 @@ void CodeDocument::Position::setLineAndIndex (const int newLineNum, const int ne
         {
             line = owner->lines.size() - 1;
 
-            const CodeDocumentLine& l = *owner->lines.getUnchecked (line);
+            auto& l = *owner->lines.getUnchecked (line);
             indexInLine = l.lineLengthWithoutNewLines;
             characterPos = l.lineStartInFile + indexInLine;
         }
@@ -327,7 +300,7 @@ void CodeDocument::Position::setLineAndIndex (const int newLineNum, const int ne
         {
             line = jmax (0, newLineNum);
 
-            const CodeDocumentLine& l = *owner->lines.getUnchecked (line);
+            auto& l = *owner->lines.getUnchecked (line);
 
             if (l.lineLengthWithoutNewLines > 0)
                 indexInLine = jlimit (0, l.lineLengthWithoutNewLines, newIndexInLine);
@@ -350,7 +323,7 @@ void CodeDocument::Position::setPosition (const int newPosition)
     if (newPosition > 0)
     {
         int lineStart = 0;
-        int lineEnd = owner->lines.size();
+        auto lineEnd = owner->lines.size();
 
         for (;;)
         {
@@ -358,8 +331,8 @@ void CodeDocument::Position::setPosition (const int newPosition)
             {
                 for (int i = lineStart; i < lineEnd; ++i)
                 {
-                    const CodeDocumentLine& l = *owner->lines.getUnchecked (i);
-                    const int index = newPosition - l.lineStartInFile;
+                    auto& l = *owner->lines.getUnchecked (i);
+                    auto index = newPosition - l.lineStartInFile;
 
                     if (index >= 0 && (index < l.lineLength || i == lineEnd - 1))
                     {
@@ -373,7 +346,7 @@ void CodeDocument::Position::setPosition (const int newPosition)
             }
             else
             {
-                const int midIndex = (lineStart + lineEnd + 1) / 2;
+                auto midIndex = (lineStart + lineEnd + 1) / 2;
 
                 if (newPosition >= owner->lines.getUnchecked (midIndex)->lineStartInFile)
                     lineStart = midIndex;
@@ -395,7 +368,7 @@ void CodeDocument::Position::moveBy (int characterDelta)
         // If moving right, make sure we don't get stuck between the \r and \n characters..
         if (line < owner->lines.size())
         {
-            const CodeDocumentLine& l = *owner->lines.getUnchecked (line);
+            auto& l = *owner->lines.getUnchecked (line);
 
             if (indexInLine + characterDelta < l.lineLength
                  && indexInLine + characterDelta >= l.lineLengthWithoutNewLines + 1)
@@ -422,7 +395,7 @@ CodeDocument::Position CodeDocument::Position::movedByLines (const int deltaLine
 
 juce_wchar CodeDocument::Position::getCharacter() const
 {
-    if (const CodeDocumentLine* const l = owner->lines [line])
+    if (auto* l = owner->lines [line])
         return l->line [getIndexInLine()];
 
     return 0;
@@ -430,10 +403,10 @@ juce_wchar CodeDocument::Position::getCharacter() const
 
 String CodeDocument::Position::getLineText() const
 {
-    if (const CodeDocumentLine* const l = owner->lines [line])
+    if (auto* l = owner->lines [line])
         return l->line;
 
-    return String();
+    return {};
 }
 
 void CodeDocument::Position::setPositionMaintained (const bool isMaintained)
@@ -460,12 +433,7 @@ void CodeDocument::Position::setPositionMaintained (const bool isMaintained)
 }
 
 //==============================================================================
-CodeDocument::CodeDocument()
-    : undoManager (std::numeric_limits<int>::max(), 10000),
-      currentActionIndex (0),
-      indexOfSavedState (-1),
-      maximumLineLength (-1),
-      newLineChars ("\r\n")
+CodeDocument::CodeDocument() : undoManager (std::numeric_limits<int>::max(), 10000)
 {
 }
 
@@ -482,32 +450,32 @@ String CodeDocument::getAllContent() const
 String CodeDocument::getTextBetween (const Position& start, const Position& end) const
 {
     if (end.getPosition() <= start.getPosition())
-        return String();
+        return {};
 
-    const int startLine = start.getLineNumber();
-    const int endLine = end.getLineNumber();
+    auto startLine = start.getLineNumber();
+    auto endLine = end.getLineNumber();
 
     if (startLine == endLine)
     {
-        if (CodeDocumentLine* const line = lines [startLine])
+        if (auto* line = lines [startLine])
             return line->line.substring (start.getIndexInLine(), end.getIndexInLine());
 
-        return String();
+        return {};
     }
 
     MemoryOutputStream mo;
     mo.preallocate ((size_t) (end.getPosition() - start.getPosition() + 4));
 
-    const int maxLine = jmin (lines.size() - 1, endLine);
+    auto maxLine = jmin (lines.size() - 1, endLine);
 
     for (int i = jmax (0, startLine); i <= maxLine; ++i)
     {
-        const CodeDocumentLine& line = *lines.getUnchecked(i);
-        int len = line.lineLength;
+        auto& line = *lines.getUnchecked(i);
+        auto len = line.lineLength;
 
         if (i == startLine)
         {
-            const int index = start.getIndexInLine();
+            auto index = start.getIndexInLine();
             mo << line.line.substring (index, len);
         }
         else if (i == endLine)
@@ -526,7 +494,7 @@ String CodeDocument::getTextBetween (const Position& start, const Position& end)
 
 int CodeDocument::getNumCharacters() const noexcept
 {
-    if (const CodeDocumentLine* const lastLine = lines.getLast())
+    if (auto* lastLine = lines.getLast())
         return lastLine->lineStartInFile + lastLine->lineLength;
 
     return 0;
@@ -534,10 +502,10 @@ int CodeDocument::getNumCharacters() const noexcept
 
 String CodeDocument::getLine (const int lineIndex) const noexcept
 {
-    if (const CodeDocumentLine* const line = lines [lineIndex])
+    if (auto* line = lines[lineIndex])
         return line->line;
 
-    return String();
+    return {};
 }
 
 int CodeDocument::getMaximumLineLength() noexcept
@@ -546,8 +514,8 @@ int CodeDocument::getMaximumLineLength() noexcept
     {
         maximumLineLength = 0;
 
-        for (int i = lines.size(); --i >= 0;)
-            maximumLineLength = jmax (maximumLineLength, lines.getUnchecked(i)->lineLength);
+        for (auto* l : lines)
+            maximumLineLength = jmax (maximumLineLength, l->lineLength);
     }
 
     return maximumLineLength;
@@ -586,10 +554,8 @@ void CodeDocument::applyChanges (const String& newContent)
 
     TextDiff diff (getAllContent(), corrected);
 
-    for (int i = 0; i < diff.changes.size(); ++i)
+    for (auto& c : diff.changes)
     {
-        const TextDiff::Change& c = diff.changes.getReference(i);
-
         if (c.isDeletion())
             remove (c.start, c.start + c.length, true);
         else
@@ -614,9 +580,9 @@ bool CodeDocument::loadFromStream (InputStream& stream)
 
 bool CodeDocument::writeToStream (OutputStream& stream)
 {
-    for (int i = 0; i < lines.size(); ++i)
+    for (auto* l : lines)
     {
-        String temp (lines.getUnchecked(i)->line); // use a copy to avoid bloating the memory footprint of the stored string.
+        auto temp = l->line; // use a copy to avoid bloating the memory footprint of the stored string.
         const char* utf8 = temp.toUTF8();
 
         if (! stream.write (utf8, strlen (utf8)))
@@ -664,23 +630,15 @@ bool CodeDocument::hasChangedSinceSavePoint() const noexcept
 }
 
 //==============================================================================
-namespace CodeDocumentHelpers
+static inline int getCharacterType (juce_wchar character) noexcept
 {
-    static int getCharacterType (const juce_wchar character) noexcept
-    {
-        return (CharacterFunctions::isLetterOrDigit (character) || character == '_')
-                    ? 2 : (CharacterFunctions::isWhitespace (character) ? 0 : 1);
-    }
-
-    static bool isTokenCharacter (const juce_wchar c) noexcept
-    {
-        return CharacterFunctions::isLetterOrDigit (c) || c == '.' || c == '_';
-    }
+    return (CharacterFunctions::isLetterOrDigit (character) || character == '_')
+                ? 2 : (CharacterFunctions::isWhitespace (character) ? 0 : 1);
 }
 
 CodeDocument::Position CodeDocument::findWordBreakAfter (const Position& position) const noexcept
 {
-    Position p (position);
+    auto p = position;
     const int maxDistance = 256;
     int i = 0;
 
@@ -695,9 +653,9 @@ CodeDocument::Position CodeDocument::findWordBreakAfter (const Position& positio
 
     if (i == 0)
     {
-        const int type = CodeDocumentHelpers::getCharacterType (p.getCharacter());
+        auto type = getCharacterType (p.getCharacter());
 
-        while (i < maxDistance && type == CodeDocumentHelpers::getCharacterType (p.getCharacter()))
+        while (i < maxDistance && type == getCharacterType (p.getCharacter()))
         {
             ++i;
             p.moveBy (1);
@@ -718,14 +676,14 @@ CodeDocument::Position CodeDocument::findWordBreakAfter (const Position& positio
 
 CodeDocument::Position CodeDocument::findWordBreakBefore (const Position& position) const noexcept
 {
-    Position p (position);
+    auto p = position;
     const int maxDistance = 256;
     int i = 0;
     bool stoppedAtLineStart = false;
 
     while (i < maxDistance)
     {
-        const juce_wchar c = p.movedBy (-1).getCharacter();
+        auto c = p.movedBy (-1).getCharacter();
 
         if (c == '\r' || c == '\n')
         {
@@ -744,9 +702,9 @@ CodeDocument::Position CodeDocument::findWordBreakBefore (const Position& positi
 
     if (i < maxDistance && ! stoppedAtLineStart)
     {
-        const int type = CodeDocumentHelpers::getCharacterType (p.movedBy (-1).getCharacter());
+        auto type = getCharacterType (p.movedBy (-1).getCharacter());
 
-        while (i < maxDistance && type == CodeDocumentHelpers::getCharacterType (p.movedBy (-1).getCharacter()))
+        while (i < maxDistance && type == getCharacterType (p.movedBy (-1).getCharacter()))
         {
             p.moveBy (-1);
             ++i;
@@ -758,13 +716,15 @@ CodeDocument::Position CodeDocument::findWordBreakBefore (const Position& positi
 
 void CodeDocument::findTokenContaining (const Position& pos, Position& start, Position& end) const noexcept
 {
+    auto isTokenCharacter = [] (juce_wchar c)  { return CharacterFunctions::isLetterOrDigit (c) || c == '.' || c == '_'; };
+
     end = pos;
-    while (CodeDocumentHelpers::isTokenCharacter (end.getCharacter()))
+    while (isTokenCharacter (end.getCharacter()))
         end.moveBy (1);
 
     start = end;
     while (start.getIndexInLine() > 0
-            && CodeDocumentHelpers::isTokenCharacter (start.movedBy (-1).getCharacter()))
+            && isTokenCharacter (start.movedBy (-1).getCharacter()))
         start.moveBy (-1);
 }
 
@@ -795,14 +755,13 @@ void CodeDocument::checkLastLineStatus()
 }
 
 //==============================================================================
-void CodeDocument::addListener    (CodeDocument::Listener* const l) noexcept   { listeners.add (l); }
-void CodeDocument::removeListener (CodeDocument::Listener* const l) noexcept   { listeners.remove (l); }
+void CodeDocument::addListener    (CodeDocument::Listener* l) noexcept   { listeners.add (l); }
+void CodeDocument::removeListener (CodeDocument::Listener* l) noexcept   { listeners.remove (l); }
 
 //==============================================================================
-class CodeDocumentInsertAction   : public UndoableAction
+struct CodeDocument::InsertAction   : public UndoableAction
 {
-public:
-    CodeDocumentInsertAction (CodeDocument& doc, const String& t, const int pos) noexcept
+    InsertAction (CodeDocument& doc, const String& t, const int pos) noexcept
         : owner (doc), text (t), insertPos (pos)
     {
     }
@@ -823,12 +782,11 @@ public:
 
     int getSizeInUnits() override        { return text.length() + 32; }
 
-private:
     CodeDocument& owner;
     const String text;
     const int insertPos;
 
-    JUCE_DECLARE_NON_COPYABLE (CodeDocumentInsertAction)
+    JUCE_DECLARE_NON_COPYABLE (InsertAction)
 };
 
 void CodeDocument::insert (const String& text, const int insertPos, const bool undoable)
@@ -837,19 +795,19 @@ void CodeDocument::insert (const String& text, const int insertPos, const bool u
     {
         if (undoable)
         {
-            undoManager.perform (new CodeDocumentInsertAction (*this, text, insertPos));
+            undoManager.perform (new InsertAction (*this, text, insertPos));
         }
         else
         {
             Position pos (*this, insertPos);
-            const int firstAffectedLine = pos.getLineNumber();
+            auto firstAffectedLine = pos.getLineNumber();
 
-            CodeDocumentLine* const firstLine = lines [firstAffectedLine];
-            String textInsideOriginalLine (text);
+            auto* firstLine = lines[firstAffectedLine];
+            auto textInsideOriginalLine = text;
 
             if (firstLine != nullptr)
             {
-                const int index = pos.getIndexInLine();
+                auto index = pos.getIndexInLine();
                 textInsideOriginalLine = firstLine->line.substring (0, index)
                                          + textInsideOriginalLine
                                          + firstLine->line.substring (index);
@@ -860,7 +818,7 @@ void CodeDocument::insert (const String& text, const int insertPos, const bool u
             CodeDocumentLine::createLines (newLines, textInsideOriginalLine);
             jassert (newLines.size() > 0);
 
-            CodeDocumentLine* const newFirstLine = newLines.getUnchecked (0);
+            auto* newFirstLine = newLines.getUnchecked (0);
             newFirstLine->lineStartInFile = firstLine != nullptr ? firstLine->lineStartInFile : 0;
             lines.set (firstAffectedLine, newFirstLine);
 
@@ -868,34 +826,30 @@ void CodeDocument::insert (const String& text, const int insertPos, const bool u
                 lines.insertArray (firstAffectedLine + 1, newLines.getRawDataPointer() + 1, newLines.size() - 1);
 
             int lineStart = newFirstLine->lineStartInFile;
+
             for (int i = firstAffectedLine; i < lines.size(); ++i)
             {
-                CodeDocumentLine& l = *lines.getUnchecked (i);
+                auto& l = *lines.getUnchecked (i);
                 l.lineStartInFile = lineStart;
                 lineStart += l.lineLength;
             }
 
             checkLastLineStatus();
+            auto newTextLength = text.length();
 
-            const int newTextLength = text.length();
-            for (int i = 0; i < positionsToMaintain.size(); ++i)
-            {
-                CodeDocument::Position& p = *positionsToMaintain.getUnchecked(i);
+            for (auto* p : positionsToMaintain)
+                if (p->getPosition() >= insertPos)
+                    p->setPosition (p->getPosition() + newTextLength);
 
-                if (p.getPosition() >= insertPos)
-                    p.setPosition (p.getPosition() + newTextLength);
-            }
-
-            listeners.call (&CodeDocument::Listener::codeDocumentTextInserted, text, insertPos);
+            listeners.call ([&] (Listener& l) { l.codeDocumentTextInserted (text, insertPos); });
         }
     }
 }
 
 //==============================================================================
-class CodeDocumentDeleteAction  : public UndoableAction
+struct CodeDocument::DeleteAction  : public UndoableAction
 {
-public:
-    CodeDocumentDeleteAction (CodeDocument& doc, const int start, const int end) noexcept
+    DeleteAction (CodeDocument& doc, int start, int end) noexcept
         : owner (doc), startPos (start), endPos (end),
           removedText (doc.getTextBetween (CodeDocument::Position (doc, start),
                                            CodeDocument::Position (doc, end)))
@@ -918,12 +872,11 @@ public:
 
     int getSizeInUnits() override    { return (endPos - startPos) + 32; }
 
-private:
     CodeDocument& owner;
     const int startPos, endPos;
     const String removedText;
 
-    JUCE_DECLARE_NON_COPYABLE (CodeDocumentDeleteAction)
+    JUCE_DECLARE_NON_COPYABLE (DeleteAction)
 };
 
 void CodeDocument::remove (const int startPos, const int endPos, const bool undoable)
@@ -933,7 +886,7 @@ void CodeDocument::remove (const int startPos, const int endPos, const bool undo
 
     if (undoable)
     {
-        undoManager.perform (new CodeDocumentDeleteAction (*this, startPos, endPos));
+        undoManager.perform (new DeleteAction (*this, startPos, endPos));
     }
     else
     {
@@ -941,9 +894,9 @@ void CodeDocument::remove (const int startPos, const int endPos, const bool undo
         Position endPosition (*this, endPos);
 
         maximumLineLength = -1;
-        const int firstAffectedLine = startPosition.getLineNumber();
-        const int endLine = endPosition.getLineNumber();
-        CodeDocumentLine& firstLine = *lines.getUnchecked (firstAffectedLine);
+        auto firstAffectedLine = startPosition.getLineNumber();
+        auto endLine = endPosition.getLineNumber();
+        auto& firstLine = *lines.getUnchecked (firstAffectedLine);
 
         if (firstAffectedLine == endLine)
         {
@@ -953,7 +906,7 @@ void CodeDocument::remove (const int startPos, const int endPos, const bool undo
         }
         else
         {
-            CodeDocumentLine& lastLine = *lines.getUnchecked (endLine);
+            auto& lastLine = *lines.getUnchecked (endLine);
 
             firstLine.line = firstLine.line.substring (0, startPosition.getIndexInLine())
                             + lastLine.line.substring (endPosition.getIndexInLine());
@@ -965,26 +918,25 @@ void CodeDocument::remove (const int startPos, const int endPos, const bool undo
 
         for (int i = firstAffectedLine + 1; i < lines.size(); ++i)
         {
-            CodeDocumentLine& l = *lines.getUnchecked (i);
-            const CodeDocumentLine& previousLine = *lines.getUnchecked (i - 1);
+            auto& l = *lines.getUnchecked (i);
+            auto& previousLine = *lines.getUnchecked (i - 1);
             l.lineStartInFile = previousLine.lineStartInFile + previousLine.lineLength;
         }
 
         checkLastLineStatus();
+        auto totalChars = getNumCharacters();
 
-        const int totalChars = getNumCharacters();
-
-        for (int i = 0; i < positionsToMaintain.size(); ++i)
+        for (auto* p : positionsToMaintain)
         {
-            CodeDocument::Position& p = *positionsToMaintain.getUnchecked(i);
+            if (p->getPosition() > startPosition.getPosition())
+                p->setPosition (jmax (startPos, p->getPosition() + startPos - endPos));
 
-            if (p.getPosition() > startPosition.getPosition())
-                p.setPosition (jmax (startPos, p.getPosition() + startPos - endPos));
-
-            if (p.getPosition() > totalChars)
-                p.setPosition (totalChars);
+            if (p->getPosition() > totalChars)
+                p->setPosition (totalChars);
         }
 
-        listeners.call (&CodeDocument::Listener::codeDocumentTextDeleted, startPos, endPos);
+        listeners.call ([=] (Listener& l) { l.codeDocumentTextDeleted (startPos, endPos); });
     }
 }
+
+} // namespace juce
