@@ -2,28 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_CODEDOCUMENT_H_INCLUDED
-#define JUCE_CODEDOCUMENT_H_INCLUDED
+namespace juce
+{
 
 class CodeDocumentLine;
 
@@ -59,7 +61,7 @@ public:
     class JUCE_API  Position
     {
     public:
-        /** Creates an uninitialised postion.
+        /** Creates an uninitialised position.
             Don't attempt to call any methods on this until you've given it an owner document
             to refer to!
         */
@@ -177,9 +179,9 @@ public:
         String getLineText() const;
 
     private:
-        CodeDocument* owner;
-        int characterPos, line, indexInLine;
-        bool positionMaintained;
+        CodeDocument* owner = nullptr;
+        int characterPos = 0, line = 0, indexInLine = 0;
+        bool positionMaintained = false;
     };
 
     //==============================================================================
@@ -355,8 +357,8 @@ public:
     {
     public:
         Iterator (const CodeDocument& document) noexcept;
-        Iterator (const Iterator&) noexcept;
-        Iterator& operator= (const Iterator&) noexcept;
+        Iterator (const Iterator&) = default;
+        Iterator& operator= (const Iterator&) = default;
         ~Iterator() noexcept;
 
         /** Reads the next character and returns it.
@@ -387,24 +389,24 @@ public:
 
     private:
         const CodeDocument* document;
-        mutable String::CharPointerType charPointer;
-        int line, position;
+        mutable String::CharPointerType charPointer { nullptr };
+        int line = 0, position = 0;
     };
 
 private:
     //==============================================================================
-    friend class CodeDocumentInsertAction;
-    friend class CodeDocumentDeleteAction;
+    struct InsertAction;
+    struct DeleteAction;
     friend class Iterator;
     friend class Position;
 
-    OwnedArray <CodeDocumentLine> lines;
-    Array <Position*> positionsToMaintain;
+    OwnedArray<CodeDocumentLine> lines;
+    Array<Position*> positionsToMaintain;
     UndoManager undoManager;
-    int currentActionIndex, indexOfSavedState;
-    int maximumLineLength;
-    ListenerList <Listener> listeners;
-    String newLineChars;
+    int currentActionIndex = 0, indexOfSavedState = -1;
+    int maximumLineLength = -1;
+    ListenerList<Listener> listeners;
+    String newLineChars { "\r\n" };
 
     void insert (const String& text, int insertPos, bool undoable);
     void remove (int startPos, int endPos, bool undoable);
@@ -413,5 +415,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (CodeDocument)
 };
 
-
-#endif   // JUCE_CODEDOCUMENT_H_INCLUDED
+} // namespace juce
