@@ -2,28 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_CODEEDITORCOMPONENT_H_INCLUDED
-#define JUCE_CODEEDITORCOMPONENT_H_INCLUDED
+namespace juce
+{
 
 class CodeTokeniser;
 
@@ -227,7 +229,7 @@ public:
 
         Array<TokenType> types;
 
-        void set (const String& name, const Colour colour);
+        void set (const String& name, Colour colour);
     };
 
     /** Changes the syntax highlighting scheme.
@@ -348,7 +350,7 @@ public:
     /** @internal */
     bool isTextInputActive() const override;
     /** @internal */
-    void setTemporaryUnderlining (const Array<Range<int> >&) override;
+    void setTemporaryUnderlining (const Array<Range<int>>&) override;
     /** @internal */
     ApplicationCommandTarget* getNextCommandTarget() override;
     /** @internal */
@@ -363,18 +365,17 @@ private:
     CodeDocument& document;
 
     Font font;
-    int firstLineOnScreen, spacesPerTab;
-    float charWidth;
-    int lineHeight, linesOnScreen, columnsOnScreen;
-    int scrollbarThickness, columnToTryToMaintain;
-    bool readOnly, useSpacesForTabs, showLineNumbers, shouldFollowDocumentChanges;
-    double xOffset;
-
+    int firstLineOnScreen = 0, spacesPerTab = 4;
+    float charWidth = 0;
+    int lineHeight = 0, linesOnScreen = 0, columnsOnScreen = 0;
+    int scrollbarThickness = 16, columnToTryToMaintain = -1;
+    bool readOnly = false, useSpacesForTabs = true, showLineNumbers = false, shouldFollowDocumentChanges = false;
+    double xOffset = 0;
     CodeDocument::Position caretPos, selectionStart, selectionEnd;
 
     ScopedPointer<CaretComponent> caret;
-    ScrollBar verticalScrollBar, horizontalScrollBar;
-    ApplicationCommandManager* appCommandManager;
+    ScrollBar verticalScrollBar { true }, horizontalScrollBar { false };
+    ApplicationCommandManager* appCommandManager = nullptr;
 
     class Pimpl;
     friend class Pimpl;
@@ -393,7 +394,7 @@ private:
         draggingSelectionEnd
     };
 
-    DragType dragType;
+    DragType dragType = notDragging;
 
     //==============================================================================
     CodeTokeniser* codeTokeniser;
@@ -431,5 +432,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (CodeEditorComponent)
 };
 
-
-#endif   // JUCE_CODEEDITORCOMPONENT_H_INCLUDED
+} // namespace juce
