@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 class ColourSelector::ColourComponentSlider  : public Slider
 {
 public:
@@ -41,7 +46,6 @@ public:
         return (double) text.getHexValue32();
     }
 
-private:
     JUCE_DECLARE_NON_COPYABLE (ColourComponentSlider)
 };
 
@@ -62,7 +66,6 @@ public:
         g.drawEllipse (2.0f, 2.0f, getWidth() - 4.0f, getHeight() - 4.0f, 1.0f);
     }
 
-private:
     JUCE_DECLARE_NON_COPYABLE (ColourSpaceMarker)
 };
 
@@ -70,7 +73,7 @@ private:
 class ColourSelector::ColourSpaceView  : public Component
 {
 public:
-    ColourSpaceView (ColourSelector& cs, float& hue, float& sat, float& val, const int edgeSize)
+    ColourSpaceView (ColourSelector& cs, float& hue, float& sat, float& val, int edgeSize)
         : owner (cs), h (hue), s (sat), v (val), lastHue (0.0f), edge (edgeSize)
     {
         addAndMakeVisible (marker);
@@ -81,19 +84,19 @@ public:
     {
         if (colours.isNull())
         {
-            const int width = getWidth() / 2;
-            const int height = getHeight() / 2;
+            auto width = getWidth() / 2;
+            auto height = getHeight() / 2;
             colours = Image (Image::RGB, width, height, false);
 
             Image::BitmapData pixels (colours, Image::BitmapData::writeOnly);
 
             for (int y = 0; y < height; ++y)
             {
-                const float val = 1.0f - y / (float) height;
+                auto val = 1.0f - y / (float) height;
 
                 for (int x = 0; x < width; ++x)
                 {
-                    const float sat = x / (float) width;
+                    auto sat = x / (float) width;
                     pixels.setPixelColour (x, y, Colour (h, sat, val, 1.0f));
                 }
             }
@@ -114,8 +117,8 @@ public:
 
     void mouseDrag (const MouseEvent& e) override
     {
-        const float sat = (e.x - edge) / (float) (getWidth() - edge * 2);
-        const float val = 1.0f - (e.y - edge) / (float) (getHeight() - edge * 2);
+        auto sat =        (e.x - edge) / (float) (getWidth()  - edge * 2);
+        auto val = 1.0f - (e.y - edge) / (float) (getHeight() - edge * 2);
 
         owner.setSV (sat, val);
     }
@@ -169,8 +172,8 @@ public:
 
     void paint (Graphics& g) override
     {
-        const float cw = (float) getWidth();
-        const float ch = (float) getHeight();
+        auto cw = (float) getWidth();
+        auto ch = (float) getHeight();
 
         Path p;
         p.addTriangle (1.0f, 1.0f,
@@ -196,7 +199,7 @@ private:
 class ColourSelector::HueSelectorComp  : public Component
 {
 public:
-    HueSelectorComp (ColourSelector& cs, float& hue, const int edgeSize)
+    HueSelectorComp (ColourSelector& cs, float& hue, int edgeSize)
         : owner (cs), h (hue), edge (edgeSize)
     {
         addAndMakeVisible (marker);
@@ -207,7 +210,7 @@ public:
         ColourGradient cg;
         cg.isRadial = false;
         cg.point1.setXY (0.0f, (float) edge);
-        cg.point2.setXY (0.0f, (float) (getHeight() - edge));
+        cg.point2.setXY (0.0f, (float) getHeight());
 
         for (float i = 0.0f; i <= 1.0f; i += 0.02f)
             cg.addColour (i, Colour (i, 1.0f, 1.0f, 1.0f));
@@ -256,11 +259,11 @@ public:
 
     void paint (Graphics& g) override
     {
-        const Colour c (owner.getSwatchColour (index));
+        auto col = owner.getSwatchColour (index);
 
-        g.fillCheckerBoard (getLocalBounds(), 6, 6,
-                            Colour (0xffdddddd).overlaidWith (c),
-                            Colour (0xffffffff).overlaidWith (c));
+        g.fillCheckerBoard (getLocalBounds().toFloat(), 6.0f, 6.0f,
+                            Colour (0xffdddddd).overlaidWith (col),
+                            Colour (0xffffffff).overlaidWith (col));
     }
 
     void mouseDown (const MouseEvent&) override
@@ -307,7 +310,7 @@ private:
 };
 
 //==============================================================================
-ColourSelector::ColourSelector (const int sectionsToShow, const int edge, const int gapAroundColourSpaceComponent)
+ColourSelector::ColourSelector (int sectionsToShow, int edge, int gapAroundColourSpaceComponent)
     : colour (Colours::white),
       flags (sectionsToShow),
       edgeGap (edge)
@@ -319,21 +322,29 @@ ColourSelector::ColourSelector (const int sectionsToShow, const int edge, const
 
     if ((flags & showSliders) != 0)
     {
-        addAndMakeVisible (sliders[0] = new ColourComponentSlider (TRANS ("red")));
-        addAndMakeVisible (sliders[1] = new ColourComponentSlider (TRANS ("green")));
-        addAndMakeVisible (sliders[2] = new ColourComponentSlider (TRANS ("blue")));
-        addChildComponent (sliders[3] = new ColourComponentSlider (TRANS ("alpha")));
+        sliders[0].reset (new ColourComponentSlider (TRANS ("red")));
+        sliders[1].reset (new ColourComponentSlider (TRANS ("green")));
+        sliders[2].reset (new ColourComponentSlider (TRANS ("blue")));
+        sliders[3].reset (new ColourComponentSlider (TRANS ("alpha")));
+
+        addAndMakeVisible (sliders[0].get());
+        addAndMakeVisible (sliders[1].get());
+        addAndMakeVisible (sliders[2].get());
+        addChildComponent (sliders[3].get());
 
         sliders[3]->setVisible ((flags & showAlphaChannel) != 0);
 
         for (int i = 4; --i >= 0;)
-            sliders[i]->addListener (this);
+            sliders[i]->onValueChange = [this] { changeColour(); };
     }
 
     if ((flags & showColourspace) != 0)
     {
-        addAndMakeVisible (colourSpace = new ColourSpaceView (*this, h, s, v, gapAroundColourSpaceComponent));
-        addAndMakeVisible (hueSelector = new HueSelectorComp (*this, h,  gapAroundColourSpaceComponent));
+        colourSpace.reset (new ColourSpaceView (*this, h, s, v, gapAroundColourSpaceComponent));
+        hueSelector.reset (new HueSelectorComp (*this, h,  gapAroundColourSpaceComponent));
+
+        addAndMakeVisible (colourSpace.get());
+        addAndMakeVisible (hueSelector.get());
     }
 
     update (dontSendNotification);
@@ -427,9 +438,9 @@ void ColourSelector::paint (Graphics& g)
 
     if ((flags & showColourAtTop) != 0)
     {
-        const Colour currentColour (getCurrentColour());
+        auto currentColour = getCurrentColour();
 
-        g.fillCheckerBoard (previewArea, 10, 10,
+        g.fillCheckerBoard (previewArea.toFloat(), 10.0f, 10.0f,
                             Colour (0xffdddddd).overlaidWith (currentColour),
                             Colour (0xffffffff).overlaidWith (currentColour));
 
@@ -488,7 +499,7 @@ void ColourSelector::resized()
 
     if ((flags & showSliders) != 0)
     {
-        const int sliderHeight = jmax (4, sliderSpace / numSliders);
+        auto sliderHeight = jmax (4, sliderSpace / numSliders);
 
         for (int i = 0; i < numSliders; ++i)
         {
@@ -513,7 +524,7 @@ void ColourSelector::resized()
 
             for (int i = 0; i < numSwatches; ++i)
             {
-                SwatchComponent* const sc = new SwatchComponent (*this, i);
+                auto* sc = new SwatchComponent (*this, i);
                 swatchComponents.add (sc);
                 addAndMakeVisible (sc);
             }
@@ -523,7 +534,7 @@ void ColourSelector::resized()
 
         for (int i = 0; i < swatchComponents.size(); ++i)
         {
-            SwatchComponent* const sc = swatchComponents.getUnchecked(i);
+            auto* sc = swatchComponents.getUnchecked(i);
 
             sc->setBounds (x + xGap / 2,
                            y + yGap / 2,
@@ -543,7 +554,7 @@ void ColourSelector::resized()
     }
 }
 
-void ColourSelector::sliderValueChanged (Slider*)
+void ColourSelector::changeColour()
 {
     if (sliders[0] != nullptr)
         setCurrentColour (Colour ((uint8) sliders[0]->getValue(),
@@ -558,13 +569,15 @@ int ColourSelector::getNumSwatches() const
     return 0;
 }
 
-Colour ColourSelector::getSwatchColour (const int) const
+Colour ColourSelector::getSwatchColour (int) const
 {
     jassertfalse; // if you've overridden getNumSwatches(), you also need to implement this method
     return Colours::black;
 }
 
-void ColourSelector::setSwatchColour (const int, const Colour&) const
+void ColourSelector::setSwatchColour (int, const Colour&)
 {
     jassertfalse; // if you've overridden getNumSwatches(), you also need to implement this method
 }
+
+} // namespace juce
