@@ -2,28 +2,63 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
 #if JUCE_MAC
 
+namespace juce
+{
+
+struct WebViewKeyEquivalentResponder : public ObjCClass<WebView>
+{
+    WebViewKeyEquivalentResponder() : ObjCClass<WebView> ("WebViewKeyEquivalentResponder_")
+    {
+        addMethod (@selector (performKeyEquivalent:), performKeyEquivalent, @encode (BOOL), "@:@");
+        registerClass();
+    }
+
+private:
+    static BOOL performKeyEquivalent (id self, SEL selector, NSEvent* event)
+    {
+        NSResponder* first = [[self window] firstResponder];
+
+       #if (defined (MAC_OS_X_VERSION_10_12) && MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_12)
+        if (([event modifierFlags] & NSEventModifierFlagDeviceIndependentFlagsMask) == NSEventModifierFlagCommand)
+       #else
+        if (([event modifierFlags] & NSDeviceIndependentModifierFlagsMask) == NSCommandKeyMask)
+       #endif
+        {
+            if ([[event charactersIgnoringModifiers] isEqualToString:@"x"]) return [NSApp sendAction:@selector(cut:)       to:first from:self];
+            if ([[event charactersIgnoringModifiers] isEqualToString:@"c"]) return [NSApp sendAction:@selector(copy:)      to:first from:self];
+            if ([[event charactersIgnoringModifiers] isEqualToString:@"v"]) return [NSApp sendAction:@selector(paste:)     to:first from:self];
+            if ([[event charactersIgnoringModifiers] isEqualToString:@"a"]) return [NSApp sendAction:@selector(selectAll:) to:first from:self];
+        }
+
+        objc_super s = { self, [WebView class] };
+        return ObjCMsgSendSuper<BOOL, NSEvent*> (&s, selector, event);
+    }
+};
+
 struct DownloadClickDetectorClass  : public ObjCClass<NSObject>
 {
     DownloadClickDetectorClass()  : ObjCClass<NSObject> ("JUCEWebClickDetector_")
@@ -35,6 +70,8 @@ struct DownloadClickDetectorClass  : public ObjCClass<NSObject>
         addMethod (@selector (webView:decidePolicyForNewWindowAction:request:newFrameName:decisionListener:),
                    decidePolicyForNewWindowAction, "v@:@@@@@");
         addMethod (@selector (webView:didFinishLoadForFrame:), didFinishLoadForFrame, "v@:@@");
+        addMethod (@selector (webView:didFailLoadWithError:forFrame:),  didFailLoadWithError,  "v@:@@@");
+        addMethod (@selector (webView:didFailProvisionalLoadWithError:forFrame:),  didFailLoadWithError,  "v@:@@@");
         addMethod (@selector (webView:willCloseFrame:), willCloseFrame, "v@:@@");
         addMethod (@selector (webView:runOpenPanelForFileButtonWithResultListener:allowMultipleFiles:), runOpenPanel, "v@:@@", @encode (BOOL));
 
@@ -50,7 +87,7 @@ private:
         if (NSURL* url = [actionInformation valueForKey: nsStringLiteral ("WebActionOriginalURLKey")])
             return nsStringToJuce ([url absoluteString]);
 
-        return String();
+        return {};
     }
 
     static void decidePolicyForNavigationAction (id self, SEL, WebView*, NSDictionary* actionInformation,
@@ -78,6 +115,19 @@ private:
         }
     }
 
+    static void didFailLoadWithError (id self, SEL, WebView* sender, NSError* error, WebFrame* frame)
+    {
+        if ([frame isEqual: [sender mainFrame]] && error != nullptr && [error code] != NSURLErrorCancelled)
+        {
+            auto errorString = nsStringToJuce ([error localizedDescription]);
+            bool proceedToErrorPage = getOwner (self)->pageLoadHadNetworkError (errorString);
+
+            // WebKit doesn't have an internal error page, so make a really simple one ourselves
+            if (proceedToErrorPage)
+                getOwner (self)->goToURL ("data:text/plain;charset=UTF-8," + errorString);
+        }
+    }
+
     static void willCloseFrame (id self, SEL, WebView*, WebFrame*)
     {
         getOwner (self)->windowCloseRequest();
@@ -92,10 +142,8 @@ private:
         if (allowMultipleFiles ? chooser.browseForMultipleFilesToOpen()
                                : chooser.browseForFileToOpen())
         {
-            const Array<File>& files = chooser.getResults();
-
-            for (int i = 0; i < files.size(); ++i)
-                [resultListener chooseFilename: juceStringToNS (files.getReference(i).getFullPathName())];
+            for (auto& f : chooser.getResults())
+                [resultListener chooseFilename: juceStringToNS (f.getFullPathName())];
         }
        #else
         ignoreUnused (resultListener, allowMultipleFiles);
@@ -106,8 +154,6 @@ private:
 
 #else
 
-} // (juce namespace)
-
 //==============================================================================
 @interface WebViewTapDetector  : NSObject<UIGestureRecognizerDelegate>
 {
@@ -122,7 +168,7 @@ private:
 - (BOOL) gestureRecognizer: (UIGestureRecognizer*) gestureRecognizer
          shouldRecognizeSimultaneouslyWithGestureRecognizer: (UIGestureRecognizer*) otherGestureRecognizer
 {
-    ignoreUnused (gestureRecognizer, otherGestureRecognizer);
+    juce::ignoreUnused (gestureRecognizer, otherGestureRecognizer);
     return YES;
 }
 
@@ -152,17 +198,18 @@ private:
 - (BOOL) webView: (UIWebView*) webView shouldStartLoadWithRequest: (NSURLRequest*) request
                                                    navigationType: (UIWebViewNavigationType) navigationType
 {
-    ignoreUnused (webView, navigationType);
-    return ownerComponent->pageAboutToLoad (nsStringToJuce (request.URL.absoluteString));
+    juce::ignoreUnused (webView, navigationType);
+    return ownerComponent->pageAboutToLoad (juce::nsStringToJuce (request.URL.absoluteString));
 }
 
 - (void) webViewDidFinishLoad: (UIWebView*) webView
 {
-    ownerComponent->pageFinishedLoading (nsStringToJuce (webView.request.URL.absoluteString));
+    ownerComponent->pageFinishedLoading (juce::nsStringToJuce (webView.request.URL.absoluteString));
 }
 @end
 
-namespace juce {
+namespace juce
+{
 
 #endif
 
@@ -178,9 +225,12 @@ public:
     Pimpl (WebBrowserComponent* owner)
     {
        #if JUCE_MAC
-        webView = [[WebView alloc] initWithFrame: NSMakeRect (0, 0, 100.0f, 100.0f)
-                                       frameName: nsEmptyString()
-                                       groupName: nsEmptyString()];
+        static WebViewKeyEquivalentResponder webviewClass;
+        webView = (WebView*) webviewClass.createInstance();
+
+        webView = [webView initWithFrame: NSMakeRect (0, 0, 100.0f, 100.0f)
+                               frameName: nsEmptyString()
+                               groupName: nsEmptyString()];
         setView (webView);
 
         static DownloadClickDetectorClass cls;
@@ -231,8 +281,15 @@ public:
         }
         else
         {
+            NSString* urlString = juceStringToNS (url);
+
+           #if (JUCE_MAC && (defined (__MAC_OS_X_VERSION_MIN_REQUIRED) && defined (__MAC_10_9) && __MAC_OS_X_VERSION_MIN_REQUIRED >= __MAC_10_9)) || (JUCE_IOS && (defined (__IPHONE_OS_VERSION_MIN_REQUIRED) && defined (__IPHONE_7_0) && __IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_7_0))
+            urlString = [urlString stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
+           #else
+            urlString = [urlString stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
+           #endif
             NSMutableURLRequest* r
-                = [NSMutableURLRequest requestWithURL: [NSURL URLWithString: juceStringToNS (url)]
+                = [NSMutableURLRequest requestWithURL: [NSURL URLWithString: urlString]
                                           cachePolicy: NSURLRequestUseProtocolCachePolicy
                                       timeoutInterval: 30.0];
 
@@ -299,19 +356,16 @@ private:
 };
 
 //==============================================================================
-WebBrowserComponent::WebBrowserComponent (const bool unloadWhenHidden)
-    : browser (nullptr),
-      blankPageShown (false),
-      unloadPageWhenBrowserIsHidden (unloadWhenHidden)
+WebBrowserComponent::WebBrowserComponent (bool unloadWhenHidden)
+    : unloadPageWhenBrowserIsHidden (unloadWhenHidden)
 {
     setOpaque (true);
-
-    addAndMakeVisible (browser = new Pimpl (this));
+    browser.reset (new Pimpl (this));
+    addAndMakeVisible (browser.get());
 }
 
 WebBrowserComponent::~WebBrowserComponent()
 {
-    deleteAndZero (browser);
 }
 
 //==============================================================================
@@ -414,3 +468,20 @@ void WebBrowserComponent::visibilityChanged()
 void WebBrowserComponent::focusGained (FocusChangeType)
 {
 }
+
+void WebBrowserComponent::clearCookies()
+{
+    NSHTTPCookieStorage* storage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
+
+    if (NSArray* cookies = [storage cookies])
+    {
+        const NSUInteger n = [cookies count];
+
+        for (NSUInteger i = 0; i < n; ++i)
+            [storage deleteCookie: [cookies objectAtIndex: i]];
+    }
+
+    [[NSUserDefaults standardUserDefaults] synchronize];
+}
+
+} // namespace juce
