@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 extern void* getUser32Function (const char*);
 
 namespace IconConverters
@@ -91,9 +96,9 @@ public:
         if (owner.isCurrentlyBlockedByAnotherModalComponent())
         {
             if (lParam == WM_LBUTTONDOWN || lParam == WM_RBUTTONDOWN
-                 || lParam == WM_LBUTTONDBLCLK || lParam == WM_LBUTTONDBLCLK)
+                 || lParam == WM_LBUTTONDBLCLK || lParam == WM_RBUTTONDBLCLK)
             {
-                if (Component* const current = Component::getCurrentlyModalComponent())
+                if (auto* current = Component::getCurrentlyModalComponent())
                     current->inputAttemptWhenModal();
             }
         }
@@ -110,9 +115,10 @@ public:
 
             const Time eventTime (getMouseEventTime());
 
-            const MouseEvent e (Desktop::getInstance().getMainMouseSource(),
-                                Point<float>(), eventMods, MouseInputSource::invalidPressure,
-                                &owner, &owner, eventTime, Point<float>(), eventTime, 1, false);
+            const MouseEvent e (Desktop::getInstance().getMainMouseSource(), {}, eventMods,
+                                MouseInputSource::invalidPressure, MouseInputSource::invalidOrientation,
+                                MouseInputSource::invalidRotation, MouseInputSource::invalidTiltX, MouseInputSource::invalidTiltY,
+                                &owner, &owner, eventTime, {}, eventTime, 1, false);
 
             if (lParam == WM_LBUTTONDOWN || lParam == WM_RBUTTONDOWN)
             {
@@ -197,13 +203,13 @@ void SystemTrayIconComponent::setIconImage (const Image& newImage)
         HICON hicon = IconConverters::createHICONFromImage (newImage, TRUE, 0, 0);
 
         if (pimpl == nullptr)
-            pimpl = new Pimpl (*this, hicon, (HWND) getWindowHandle());
+            pimpl.reset (new Pimpl (*this, hicon, (HWND) getWindowHandle()));
         else
             pimpl->updateIcon (hicon);
     }
     else
     {
-        pimpl = nullptr;
+        pimpl.reset();
     }
 }
 
@@ -233,3 +239,5 @@ void* SystemTrayIconComponent::getNativeHandle() const
 {
     return pimpl != nullptr ? &(pimpl->iconData) : nullptr;
 }
+
+} // namespace juce
