@@ -2,22 +2,24 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
@@ -33,7 +35,7 @@
 
   ID:               juce_opengl
   vendor:           juce
-  version:          4.3.0
+  version:          5.2.1
   name:             JUCE OpenGL classes
   description:      Classes for rendering OpenGL in a JUCE window.
   website:          http://www.juce.com/juce
@@ -50,7 +52,7 @@
 *******************************************************************************/
 
 
-#ifndef JUCE_OPENGL_H_INCLUDED
+#pragma once
 #define JUCE_OPENGL_H_INCLUDED
 
 #include <juce_gui_extra/juce_gui_extra.h>
@@ -107,7 +109,12 @@
 #elif JUCE_ANDROID
  #include <android/native_window.h>
  #include <android/native_window_jni.h>
- #include <GLES2/gl2.h>
+ #if JUCE_ANDROID_GL_ES_VERSION_3_0
+  #define JUCE_OPENGL3 1
+  #include <GLES3/gl3.h>
+ #else
+  #include <GLES2/gl2.h>
+ #endif
  #include <EGL/egl.h>
 #endif
 
@@ -157,14 +164,14 @@
 //==============================================================================
 namespace juce
 {
+    class OpenGLTexture;
+    class OpenGLFrameBuffer;
+    class OpenGLShaderProgram;
+}
 
-class OpenGLTexture;
-class OpenGLFrameBuffer;
-class OpenGLShaderProgram;
-
-#include "geometry/juce_Quaternion.h"
-#include "geometry/juce_Matrix3D.h"
 #include "geometry/juce_Vector3D.h"
+#include "geometry/juce_Matrix3D.h"
+#include "geometry/juce_Quaternion.h"
 #include "geometry/juce_Draggable3DOrientation.h"
 #include "native/juce_MissingGLDefinitions.h"
 #include "opengl/juce_OpenGLHelpers.h"
@@ -174,13 +181,7 @@ class OpenGLShaderProgram;
 #include "opengl/juce_OpenGLContext.h"
 #include "opengl/juce_OpenGLFrameBuffer.h"
 #include "opengl/juce_OpenGLGraphicsContext.h"
-#include "opengl/juce_OpenGLHelpers.h"
 #include "opengl/juce_OpenGLImage.h"
-#include "opengl/juce_OpenGLRenderer.h"
 #include "opengl/juce_OpenGLShaderProgram.h"
 #include "opengl/juce_OpenGLTexture.h"
 #include "utils/juce_OpenGLAppComponent.h"
-
-}
-
-#endif   // JUCE_OPENGL_H_INCLUDED
