@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 class OpenGLContext::NativeContext
 {
 public:
@@ -106,12 +111,12 @@ public:
         }
     }
 
-    void initialiseOnRenderThread (OpenGLContext&) {}
-    void shutdownOnRenderThread()               { deactivateCurrentContext(); }
+    bool initialiseOnRenderThread (OpenGLContext&)    { return true; }
+    void shutdownOnRenderThread()                     { deactivateCurrentContext(); }
 
-    bool createdOk() const noexcept             { return getRawContext() != nullptr; }
-    void* getRawContext() const noexcept        { return static_cast<void*> (renderContext); }
-    GLuint getFrameBufferID() const noexcept    { return 0; }
+    bool createdOk() const noexcept                   { return getRawContext() != nullptr; }
+    void* getRawContext() const noexcept              { return static_cast<void*> (renderContext); }
+    GLuint getFrameBufferID() const noexcept          { return 0; }
 
     bool makeActive() const noexcept
     {
@@ -190,7 +195,7 @@ public:
         lastSwapTime = now;
     }
 
-    void updateWindowPosition (const Rectangle<int>&) {}
+    void updateWindowPosition (Rectangle<int>) {}
 
     bool setSwapInterval (int numFramesPerSwap)
     {
@@ -245,21 +250,13 @@ bool OpenGLHelpers::isContextActive()
 }
 
 //==============================================================================
-void componentPeerAboutToBeRemovedFromScreen (ComponentPeer& peer)
+void componentPeerAboutToChange (Component& comp, bool shouldSuspend)
 {
-    Array<Component*> stack;
-    stack.add (&peer.getComponent());
+    if (auto* context = OpenGLContext::getContextAttachedTo (comp))
+        context->overrideCanBeAttached (shouldSuspend);
 
-    while (stack.size() != 0)
-    {
-        Component& comp = *stack.removeAndReturn (0);
-
-        const int n = comp.getNumChildComponents();
-        for (int i = 0; i < n; ++i)
-            if (Component* child = comp.getChildComponent (i))
-                stack.add (child);
-
-        if (OpenGLContext* context = OpenGLContext::getContextAttachedTo (comp))
-            context->detach();
-    }
+    for (auto* child : comp.getChildren())
+        componentPeerAboutToChange (*child, shouldSuspend);
 }
+
+} // namespace juce
