@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 #if JUCE_IOS
 struct AppInactivityCallback // NB: this is a duplicate of an internal declaration in juce_core
 {
@@ -69,26 +74,15 @@ public:
     CachedImage (OpenGLContext& c, Component& comp,
                  const OpenGLPixelFormat& pixFormat, void* contextToShare)
         : ThreadPoolJob ("OpenGL Rendering"),
-          context (c), component (comp),
-          scale (1.0),
-         #if JUCE_OPENGL3
-          vertexArrayObject (0),
-         #endif
-         #if JUCE_OPENGL_ES
-          shadersAvailable (true),
-         #else
-          shadersAvailable (false),
-         #endif
-          hasInitialised (false),
-          needsUpdate (1), lastMMLockReleaseTime (0)
+          context (c), component (comp)
     {
-        nativeContext = new NativeContext (component, pixFormat, contextToShare,
-                                           c.useMultisampling, c.versionRequired);
+        nativeContext.reset (new NativeContext (component, pixFormat, contextToShare,
+                                                c.useMultisampling, c.versionRequired));
 
         if (nativeContext->createdOk())
-            context.nativeContext = nativeContext;
+            context.nativeContext = nativeContext.get();
         else
-            nativeContext = nullptr;
+            nativeContext.reset();
     }
 
     ~CachedImage()
@@ -101,7 +95,7 @@ public:
     {
         if (nativeContext != nullptr)
         {
-            renderThread = new ThreadPool (1);
+            renderThread.reset (new ThreadPool (1));
             resume();
         }
     }
@@ -110,8 +104,20 @@ public:
     {
         if (renderThread != nullptr)
         {
+            // make sure everything has finished executing
+            destroying.set (1);
+
+            if (workQueue.size() > 0)
+            {
+                if (! renderThread->contains (this))
+                    resume();
+
+                while (workQueue.size() != 0)
+                    Thread::sleep (20);
+            }
+
             pause();
-            renderThread = nullptr;
+            renderThread.reset();
         }
 
         hasInitialised = false;
@@ -120,6 +126,9 @@ public:
     //==============================================================================
     void pause()
     {
+        signalJobShouldExit();
+        messageManagerLock.abort();
+
         if (renderThread != nullptr)
         {
             repaintEvent.signal();
@@ -134,7 +143,10 @@ public:
     }
 
     //==============================================================================
-    void paint (Graphics&) override {}
+    void paint (Graphics&) override
+    {
+        updateViewportSize (false);
+    }
 
     bool invalidateAll() override
     {
@@ -145,12 +157,15 @@ public:
 
     bool invalidate (const Rectangle<int>& area) override
     {
-        validArea.subtract (area * scale);
+        validArea.subtract (area.toFloat().transformedBy (transform).getSmallestIntegerContainer());
         triggerRepaint();
         return false;
     }
 
-    void releaseResources() override {}
+    void releaseResources() override
+    {
+        stop();
+    }
 
     void triggerRepaint()
     {
@@ -161,8 +176,8 @@ public:
     //==============================================================================
     bool ensureFrameBufferSize()
     {
-        const int fbW = cachedImageFrameBuffer.getWidth();
-        const int fbH = cachedImageFrameBuffer.getHeight();
+        auto fbW = cachedImageFrameBuffer.getWidth();
+        auto fbH = cachedImageFrameBuffer.getHeight();
 
         if (fbW != viewportArea.getWidth() || fbH != viewportArea.getHeight() || ! cachedImageFrameBuffer.isValid())
         {
@@ -181,13 +196,13 @@ public:
         glClearColor (0, 0, 0, 0);
         glEnable (GL_SCISSOR_TEST);
 
-        const GLuint previousFrameBufferTarget = OpenGLFrameBuffer::getCurrentFrameBufferTarget();
+        auto previousFrameBufferTarget = OpenGLFrameBuffer::getCurrentFrameBufferTarget();
         cachedImageFrameBuffer.makeCurrentRenderingTarget();
-        const int imageH = cachedImageFrameBuffer.getHeight();
+        auto imageH = cachedImageFrameBuffer.getHeight();
 
-        for (const Rectangle<int>* i = list.begin(), * const e = list.end(); i != e; ++i)
+        for (auto& r : list)
         {
-            glScissor (i->getX(), imageH - i->getBottom(), i->getWidth(), i->getHeight());
+            glScissor (r.getX(), imageH - r.getBottom(), r.getWidth(), r.getHeight());
             glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
         }
 
@@ -198,8 +213,7 @@ public:
 
     bool renderFrame()
     {
-        ScopedPointer<MessageManagerLock> mmLock;
-
+        MessageManager::Lock::ScopedTryLockType mmLock (messageManagerLock, false);
         const bool isUpdating = needsUpdate.compareAndSetBool (0, 1);
 
         if (context.renderComponents && isUpdating)
@@ -208,11 +222,16 @@ public:
             if (lastMMLockReleaseTime + 1 >= Time::getMillisecondCounter())
                 Thread::sleep (2);
 
-            mmLock = new MessageManagerLock (this);  // need to acquire this before locking the context.
-            if (! mmLock->lockWasGained())
-                return false;
+            while (! shouldExit())
+            {
+                doWorkWhileWaitingForLock (false);
 
-            updateViewportSize (false);
+                if (mmLock.retryLock())
+                    break;
+            }
+
+            if (shouldExit())
+                return false;
         }
 
         if (! context.makeActive())
@@ -222,6 +241,8 @@ public:
 
         JUCE_CHECK_OPENGL_ERROR
 
+        doWorkWhileWaitingForLock (true);
+
         if (context.renderer != nullptr)
         {
             glViewport (0, 0, viewportArea.getWidth(), viewportArea.getHeight());
@@ -237,7 +258,11 @@ public:
             if (isUpdating)
             {
                 paintComponent();
-                mmLock = nullptr;
+
+                if (! hasInitialised)
+                    return false;
+
+                messageManagerLock.exit();
                 lastMMLockReleaseTime = Time::getMillisecondCounter();
             }
 
@@ -253,21 +278,25 @@ public:
 
     void updateViewportSize (bool canTriggerUpdate)
     {
-        if (ComponentPeer* peer = component.getPeer())
+        if (auto* peer = component.getPeer())
         {
             lastScreenBounds = component.getTopLevelComponent()->getScreenBounds();
 
-            const double newScale = Desktop::getInstance().getDisplays()
-                                        .getDisplayContaining (lastScreenBounds.getCentre()).scale;
+            auto newScale = Desktop::getInstance().getDisplays()
+                              .getDisplayContaining (lastScreenBounds.getCentre()).scale;
 
-            Rectangle<int> newArea (peer->getComponent().getLocalArea (&component, component.getLocalBounds())
-                                                        .withZeroOrigin()
-                                     * newScale);
+            auto localBounds = component.getLocalBounds();
+
+            auto newArea = peer->getComponent().getLocalArea (&component, localBounds)
+                                               .withZeroOrigin()
+                             * newScale;
 
             if (scale != newScale || viewportArea != newArea)
             {
                 scale = newScale;
                 viewportArea = newArea;
+                transform = AffineTransform::scale ((float) newArea.getRight()  / (float) localBounds.getRight(),
+                                                    (float) newArea.getBottom() / (float) localBounds.getBottom());
 
                 if (canTriggerUpdate)
                     invalidateAll();
@@ -279,13 +308,13 @@ public:
     {
        #if JUCE_OPENGL3
         if (vertexArrayObject != 0)
-            glBindVertexArray (vertexArrayObject);
+            context.extensions.glBindVertexArray (vertexArrayObject);
        #endif
     }
 
     void checkViewportBounds()
     {
-        const Rectangle<int> screenBounds (component.getTopLevelComponent()->getScreenBounds());
+        auto screenBounds = component.getTopLevelComponent()->getScreenBounds();
 
         if (lastScreenBounds != screenBounds)
             updateViewportSize (true);
@@ -310,7 +339,7 @@ public:
             {
                 ScopedPointer<LowLevelGraphicsContext> g (createOpenGLGraphicsContext (context, cachedImageFrameBuffer));
                 g->clipToRectangleList (invalid);
-                g->addTransform (AffineTransform::scale ((float) scale));
+                g->addTransform (transform);
 
                 paintOwner (*g);
                 JUCE_CHECK_OPENGL_ERROR
@@ -403,12 +432,23 @@ public:
     {
         {
             // Allow the message thread to finish setting-up the context before using it..
-            MessageManagerLock mml (this);
-            if (! mml.lockWasGained())
-                return ThreadPoolJob::jobHasFinished;
+            MessageManager::Lock::ScopedTryLockType mmLock (messageManagerLock, false);
+
+            do
+            {
+                if (shouldExit())
+                    return ThreadPoolJob::jobHasFinished;
+
+            } while (! mmLock.retryLock());
+        }
+
+        if (! initialiseOnThread())
+        {
+            hasInitialised = false;
+
+            return ThreadPoolJob::jobHasFinished;
         }
 
-        initialiseOnThread();
         hasInitialised = true;
 
         while (! shouldExit())
@@ -438,7 +478,7 @@ public:
         return ThreadPoolJob::jobHasFinished;
     }
 
-    void initialiseOnThread()
+    bool initialiseOnThread()
     {
         // On android, this can get called twice, so drop any previous state..
         associatedObjectNames.clear();
@@ -446,7 +486,9 @@ public:
         cachedImageFrameBuffer.release();
 
         context.makeActive();
-        nativeContext->initialiseOnRenderThread (context);
+
+        if (! nativeContext->initialiseOnRenderThread (context))
+            return false;
 
        #if JUCE_ANDROID
         // On android the context may be created in initialiseOnRenderThread
@@ -454,25 +496,30 @@ public:
         context.makeActive();
        #endif
 
+        context.extensions.initialise();
+
        #if JUCE_OPENGL3
         if (OpenGLShaderProgram::getLanguageVersion() > 1.2)
         {
-            glGenVertexArrays (1, &vertexArrayObject);
+            context.extensions.glGenVertexArrays (1, &vertexArrayObject);
             bindVertexArray();
         }
        #endif
 
         glViewport (0, 0, component.getWidth(), component.getHeight());
 
-        context.extensions.initialise();
         nativeContext->setSwapInterval (1);
 
        #if ! JUCE_OPENGL_ES
+        JUCE_CHECK_OPENGL_ERROR
         shadersAvailable = OpenGLShaderProgram::getLanguageVersion() > 0;
+        clearGLError();
        #endif
 
         if (context.renderer != nullptr)
             context.renderer->newOpenGLContextCreated();
+
+        return true;
     }
 
     void shutdownOnThread()
@@ -482,7 +529,7 @@ public:
 
        #if JUCE_OPENGL3
         if (vertexArrayObject != 0)
-            glDeleteVertexArrays (1, &vertexArrayObject);
+            context.extensions.glDeleteVertexArrays (1, &vertexArrayObject);
        #endif
 
         associatedObjectNames.clear();
@@ -491,6 +538,74 @@ public:
         nativeContext->shutdownOnRenderThread();
     }
 
+    //==============================================================================
+    struct BlockingWorker  : public OpenGLContext::AsyncWorker
+    {
+        BlockingWorker (OpenGLContext::AsyncWorker::Ptr && workerToUse)
+            : originalWorker (static_cast<OpenGLContext::AsyncWorker::Ptr&&> (workerToUse))
+        {}
+
+        void operator() (OpenGLContext& calleeContext)
+        {
+            if (originalWorker != nullptr)
+                (*originalWorker) (calleeContext);
+
+            finishedSignal.signal();
+        }
+
+        void block() noexcept  { finishedSignal.wait(); }
+
+        OpenGLContext::AsyncWorker::Ptr originalWorker;
+        WaitableEvent finishedSignal;
+    };
+
+    bool doWorkWhileWaitingForLock (bool contextIsAlreadyActive)
+    {
+        bool contextActivated = false;
+
+        for (OpenGLContext::AsyncWorker::Ptr work = workQueue.removeAndReturn (0);
+             work != nullptr && (! shouldExit()); work = workQueue.removeAndReturn (0))
+        {
+            if ((! contextActivated) && (! contextIsAlreadyActive))
+            {
+                if (! context.makeActive())
+                    break;
+
+                contextActivated = true;
+            }
+
+            NativeContext::Locker locker (*nativeContext);
+
+            (*work) (context);
+            clearGLError();
+        }
+
+        if (contextActivated)
+            OpenGLContext::deactivateCurrentContext();
+
+        return shouldExit();
+    }
+
+    void execute (OpenGLContext::AsyncWorker::Ptr workerToUse, bool shouldBlock, bool calledFromDestructor = false)
+    {
+        if (calledFromDestructor || destroying.get() == 0)
+        {
+            BlockingWorker* blocker = (shouldBlock ? new BlockingWorker (static_cast<OpenGLContext::AsyncWorker::Ptr&&> (workerToUse)) : nullptr);
+            OpenGLContext::AsyncWorker::Ptr worker = (blocker != nullptr ? blocker : static_cast<OpenGLContext::AsyncWorker::Ptr&&> (workerToUse));
+            workQueue.add (worker);
+
+            messageManagerLock.abort();
+            context.triggerRepaint();
+
+            if (blocker != nullptr)
+                blocker->block();
+        }
+        else
+        {
+            jassertfalse; // you called execute AFTER you detached your openglcontext
+        }
+    }
+
     //==============================================================================
     static CachedImage* get (Component& c) noexcept
     {
@@ -498,6 +613,7 @@ public:
     }
 
     //==============================================================================
+    friend class NativeContext;
     ScopedPointer<NativeContext> nativeContext;
 
     OpenGLContext& context;
@@ -506,20 +622,28 @@ public:
     OpenGLFrameBuffer cachedImageFrameBuffer;
     RectangleList<int> validArea;
     Rectangle<int> viewportArea, lastScreenBounds;
-    double scale;
+    double scale = 1.0;
+    AffineTransform transform;
    #if JUCE_OPENGL3
-    GLuint vertexArrayObject;
+    GLuint vertexArrayObject = 0;
    #endif
 
     StringArray associatedObjectNames;
     ReferenceCountedArray<ReferenceCountedObject> associatedObjects;
 
     WaitableEvent canPaintNowFlag, finishedPaintingFlag, repaintEvent;
-    bool shadersAvailable, hasInitialised;
-    Atomic<int> needsUpdate;
-    uint32 lastMMLockReleaseTime;
+   #if JUCE_OPENGL_ES
+    bool shadersAvailable = true;
+   #else
+    bool shadersAvailable = false;
+   #endif
+    bool hasInitialised = false;
+    Atomic<int> needsUpdate { 1 }, destroying;
+    uint32 lastMMLockReleaseTime = 0;
 
     ScopedPointer<ThreadPool> renderThread;
+    ReferenceCountedArray<OpenGLContext::AsyncWorker, CriticalSection> workQueue;
+    MessageManager::Lock messageManagerLock;
 
    #if JUCE_IOS
     iOSBackgroundProcessCheck backgroundProcessCheck;
@@ -545,9 +669,17 @@ public:
         detach();
     }
 
+    void detach()
+    {
+        auto& comp = *getComponent();
+        stop();
+        comp.setCachedComponentImage (nullptr);
+        context.nativeContext = nullptr;
+    }
+
     void componentMovedOrResized (bool /*wasMoved*/, bool /*wasResized*/) override
     {
-        Component& comp = *getComponent();
+        auto& comp = *getComponent();
 
         if (isAttached (comp) != canBeAttached (comp))
             componentVisibilityChanged();
@@ -555,10 +687,10 @@ public:
         if (comp.getWidth() > 0 && comp.getHeight() > 0
              && context.nativeContext != nullptr)
         {
-            if (CachedImage* const c = CachedImage::get (comp))
+            if (auto* c = CachedImage::get (comp))
                 c->handleResize();
 
-            if (ComponentPeer* peer = comp.getTopLevelComponent()->getPeer())
+            if (auto* peer = comp.getTopLevelComponent()->getPeer())
                 context.nativeContext->updateWindowPosition (peer->getAreaCoveredBy (comp));
         }
     }
@@ -571,7 +703,7 @@ public:
 
     void componentVisibilityChanged() override
     {
-        Component& comp = *getComponent();
+        auto& comp = *getComponent();
 
         if (canBeAttached (comp))
         {
@@ -598,12 +730,22 @@ public:
     }
    #endif
 
+    void update()
+    {
+        auto& comp = *getComponent();
+
+        if (canBeAttached (comp))
+            start();
+        else
+            stop();
+    }
+
 private:
     OpenGLContext& context;
 
-    static bool canBeAttached (const Component& comp) noexcept
+    bool canBeAttached (const Component& comp) noexcept
     {
-        return comp.getWidth() > 0 && comp.getHeight() > 0 && isShowingOrMinimised (comp);
+        return (! context.overrideCanAttach) && comp.getWidth() > 0 && comp.getHeight() > 0 && isShowingOrMinimised (comp);
     }
 
     static bool isShowingOrMinimised (const Component& c)
@@ -611,7 +753,7 @@ private:
         if (! c.isVisible())
             return false;
 
-        if (Component* p = c.getParentComponent())
+        if (auto* p = c.getParentComponent())
             return isShowingOrMinimised (*p);
 
         return c.getPeer() != nullptr;
@@ -624,47 +766,51 @@ private:
 
     void attach()
     {
-        Component& comp = *getComponent();
-        CachedImage* const newCachedImage = new CachedImage (context, comp,
-                                                             context.openGLPixelFormat,
-                                                             context.contextToShareWith);
+        auto& comp = *getComponent();
+        auto* newCachedImage = new CachedImage (context, comp,
+                                                context.openGLPixelFormat,
+                                                context.contextToShareWith);
         comp.setCachedComponentImage (newCachedImage);
-        newCachedImage->start(); // (must wait until this is attached before starting its thread)
-        newCachedImage->updateViewportSize (true);
 
-        startTimer (400);
+        start();
     }
 
-    void detach()
+    void stop()
     {
         stopTimer();
 
-        Component& comp = *getComponent();
+        auto& comp = *getComponent();
 
        #if JUCE_MAC
         [[(NSView*) comp.getWindowHandle() window] disableScreenUpdatesUntilFlush];
        #endif
 
-        if (CachedImage* const oldCachedImage = CachedImage::get (comp))
+        if (auto* oldCachedImage = CachedImage::get (comp))
             oldCachedImage->stop(); // (must stop this before detaching it from the component)
+    }
 
-        comp.setCachedComponentImage (nullptr);
-        context.nativeContext = nullptr;
+    void start()
+    {
+        auto& comp = *getComponent();
+
+        if (auto* cachedImage = CachedImage::get (comp))
+        {
+            cachedImage->start(); // (must wait until this is attached before starting its thread)
+            cachedImage->updateViewportSize (true);
+
+            startTimer (400);
+        }
     }
 
     void timerCallback() override
     {
-        if (CachedImage* const cachedImage = CachedImage::get (*getComponent()))
+        if (auto* cachedImage = CachedImage::get (*getComponent()))
             cachedImage->checkViewportBounds();
     }
 };
 
 //==============================================================================
 OpenGLContext::OpenGLContext()
-    : nativeContext (nullptr), renderer (nullptr), currentRenderScale (1.0),
-      contextToShareWith (nullptr), versionRequired (OpenGLContext::defaultGLVersion),
-      imageCacheMaxSize (8 * 1024 * 1024),
-      renderComponents (true), useMultisampling (false), continuousRepaint (false)
 {
 }
 
@@ -706,6 +852,11 @@ void OpenGLContext::setPixelFormat (const OpenGLPixelFormat& preferredPixelForma
     openGLPixelFormat = preferredPixelFormat;
 }
 
+void OpenGLContext::setTextureMagnificationFilter (OpenGLContext::TextureMagnificationFilter magFilterMode) noexcept
+{
+    texMagFilter = magFilterMode;
+}
+
 void OpenGLContext::setNativeSharedContext (void* nativeContextToShareWith) noexcept
 {
     // This method must not be called when the context has already been attached!
@@ -736,13 +887,18 @@ void OpenGLContext::attachTo (Component& component)
     if (getTargetComponent() != &component)
     {
         detach();
-        attachment = new Attachment (*this, component);
+        attachment.reset (new Attachment (*this, component));
     }
 }
 
 void OpenGLContext::detach()
 {
-    attachment = nullptr;
+    if (auto* a = attachment.get())
+    {
+        a->detach(); // must detach before nulling our pointer
+        attachment.reset();
+    }
+
     nativeContext = nullptr;
 }
 
@@ -758,7 +914,7 @@ Component* OpenGLContext::getTargetComponent() const noexcept
 
 OpenGLContext* OpenGLContext::getContextAttachedTo (Component& c) noexcept
 {
-    if (CachedImage* const ci = CachedImage::get (c))
+    if (auto* ci = CachedImage::get (c))
         return &(ci->context);
 
     return nullptr;
@@ -773,7 +929,7 @@ OpenGLContext* OpenGLContext::getCurrentContext()
 
 bool OpenGLContext::makeActive() const noexcept
 {
-    OpenGLContext*& current = currentThreadActiveContext.get();
+    auto& current = currentThreadActiveContext.get();
 
     if (nativeContext != nullptr && nativeContext->makeActive())
     {
@@ -798,7 +954,7 @@ void OpenGLContext::deactivateCurrentContext()
 
 void OpenGLContext::triggerRepaint()
 {
-    if (CachedImage* const cachedImage = getCachedImage())
+    if (auto* cachedImage = getCachedImage())
         cachedImage->triggerRepaint();
 }
 
@@ -830,7 +986,7 @@ void* OpenGLContext::getRawContext() const noexcept
 
 OpenGLContext::CachedImage* OpenGLContext::getCachedImage() const noexcept
 {
-    if (Component* const comp = getTargetComponent())
+    if (auto* comp = getTargetComponent())
         return CachedImage::get (*comp);
 
     return nullptr;
@@ -838,7 +994,7 @@ OpenGLContext::CachedImage* OpenGLContext::getCachedImage() const noexcept
 
 bool OpenGLContext::areShadersAvailable() const
 {
-    CachedImage* const c = getCachedImage();
+    auto* c = getCachedImage();
     return c != nullptr && c->shadersAvailable;
 }
 
@@ -846,7 +1002,7 @@ ReferenceCountedObject* OpenGLContext::getAssociatedObject (const char* name) co
 {
     jassert (name != nullptr);
 
-    CachedImage* const c = getCachedImage();
+    auto* c = getCachedImage();
 
     // This method must only be called from an openGL rendering callback.
     jassert (c != nullptr && nativeContext != nullptr);
@@ -860,7 +1016,7 @@ void OpenGLContext::setAssociatedObject (const char* name, ReferenceCountedObjec
 {
     jassert (name != nullptr);
 
-    if (CachedImage* const c = getCachedImage())
+    if (auto* c = getCachedImage())
     {
         // This method must only be called from an openGL rendering callback.
         jassert (nativeContext != nullptr);
@@ -891,6 +1047,46 @@ void OpenGLContext::setAssociatedObject (const char* name, ReferenceCountedObjec
 void OpenGLContext::setImageCacheSize (size_t newSize) noexcept     { imageCacheMaxSize = newSize; }
 size_t OpenGLContext::getImageCacheSize() const noexcept            { return imageCacheMaxSize; }
 
+void OpenGLContext::execute (OpenGLContext::AsyncWorker::Ptr workerToUse, bool shouldBlock)
+{
+    if (auto* c = getCachedImage())
+        c->execute (static_cast<OpenGLContext::AsyncWorker::Ptr&&> (workerToUse), shouldBlock);
+    else
+        jassertfalse; // You must have attached the context to a component
+}
+
+void OpenGLContext::overrideCanBeAttached (bool newCanAttach)
+{
+    if (overrideCanAttach != newCanAttach)
+    {
+        overrideCanAttach = newCanAttach;
+
+        if (auto* a = attachment.get())
+            a->update();
+    }
+}
+
+//==============================================================================
+struct DepthTestDisabler
+{
+    DepthTestDisabler() noexcept
+    {
+        glGetBooleanv (GL_DEPTH_TEST, &wasEnabled);
+
+        if (wasEnabled)
+            glDisable (GL_DEPTH_TEST);
+    }
+
+    ~DepthTestDisabler() noexcept
+    {
+        if (wasEnabled)
+            glEnable (GL_DEPTH_TEST);
+    }
+
+    GLboolean wasEnabled;
+};
+
+//==============================================================================
 void OpenGLContext::copyTexture (const Rectangle<int>& targetClipArea,
                                  const Rectangle<int>& anchorPosAndTextureSize,
                                  const int contextWidth, const int contextHeight,
@@ -903,6 +1099,8 @@ void OpenGLContext::copyTexture (const Rectangle<int>& targetClipArea,
     glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
     glEnable (GL_BLEND);
 
+    DepthTestDisabler depthDisabler;
+
     if (areShadersAvailable())
     {
         struct OverlayShaderProgram  : public ReferenceCountedObject
@@ -986,13 +1184,13 @@ void OpenGLContext::copyTexture (const Rectangle<int>& targetClipArea,
             Params params;
         };
 
-        const GLshort left   = (GLshort) targetClipArea.getX();
-        const GLshort top    = (GLshort) targetClipArea.getY();
-        const GLshort right  = (GLshort) targetClipArea.getRight();
-        const GLshort bottom = (GLshort) targetClipArea.getBottom();
+        auto left   = (GLshort) targetClipArea.getX();
+        auto top    = (GLshort) targetClipArea.getY();
+        auto right  = (GLshort) targetClipArea.getRight();
+        auto bottom = (GLshort) targetClipArea.getBottom();
         const GLshort vertices[] = { left, bottom, right, bottom, left, top, right, top };
 
-        const OverlayShaderProgram& program = OverlayShaderProgram::select (*this);
+        auto& program = OverlayShaderProgram::select (*this);
         program.params.set ((float) contextWidth, (float) contextHeight, anchorPosAndTextureSize.toFloat(), flippedVertically);
 
         GLuint vertexBuffer = 0;
@@ -1000,7 +1198,7 @@ void OpenGLContext::copyTexture (const Rectangle<int>& targetClipArea,
         extensions.glBindBuffer (GL_ARRAY_BUFFER, vertexBuffer);
         extensions.glBufferData (GL_ARRAY_BUFFER, sizeof (vertices), vertices, GL_STATIC_DRAW);
 
-        const GLuint index = (GLuint) program.params.positionAttribute.attributeID;
+        auto index = (GLuint) program.params.positionAttribute.attributeID;
         extensions.glVertexAttribPointer (index, 2, GL_SHORT, GL_FALSE, 4, 0);
         extensions.glEnableVertexAttribArray (index);
         JUCE_CHECK_OPENGL_ERROR
@@ -1028,25 +1226,33 @@ void OpenGLContext::NativeContext::surfaceCreated (jobject holder)
 {
     ignoreUnused (holder);
 
-    if (juceContext != nullptr)
+    if (auto* cachedImage = CachedImage::get (component))
     {
-        if (OpenGLContext::CachedImage* cachedImage = juceContext->getCachedImage())
-            cachedImage->resume();
-
-        juceContext->triggerRepaint();
+        if (auto* pool = cachedImage->renderThread.get())
+        {
+            if (! pool->contains (cachedImage))
+            {
+                cachedImage->resume();
+                cachedImage->context.triggerRepaint();
+            }
+        }
     }
 }
 
 void OpenGLContext::NativeContext::surfaceDestroyed (jobject holder)
 {
     ignoreUnused (holder);
-    // unlike the name suggets this will be called just before the
-    // surface is destroyed. We need to pause the render thread.
 
-    if (juceContext != nullptr)
+    // unlike the name suggests this will be called just before the
+    // surface is destroyed. We need to pause the render thread.
+    if (auto* cachedImage = CachedImage::get (component))
     {
-        if (OpenGLContext::CachedImage* cachedImage = juceContext->getCachedImage())
-            cachedImage->pause();
+        cachedImage->pause();
+
+        if (auto* threadPool = cachedImage->renderThread.get())
+            threadPool->waitForJobToFinish (cachedImage, -1);
     }
 }
 #endif
+
+} // namespace juce
