@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 extern void (*clearOpenGLGlyphCache)(); // declared in juce_graphics
 
 namespace OpenGLRendering
@@ -254,10 +259,9 @@ struct Target
 };
 
 //==============================================================================
-class PositionedTexture
+struct PositionedTexture
 {
-public:
-    PositionedTexture (OpenGLTexture& texture, const EdgeTable& et, const Rectangle<int>& clipRegion)
+    PositionedTexture (OpenGLTexture& texture, const EdgeTable& et, Rectangle<int> clipRegion)
         : clip (clipRegion.getIntersection (et.getMaximumBounds()))
     {
         if (clip.contains (et.getMaximumBounds()))
@@ -272,7 +276,7 @@ public:
         }
     }
 
-    PositionedTexture (GLuint texture, const Rectangle<int> r, const Rectangle<int> clipRegion) noexcept
+    PositionedTexture (GLuint texture, Rectangle<int> r, Rectangle<int> clipRegion) noexcept
         : textureID (texture), area (r), clip (clipRegion)
     {}
 
@@ -294,7 +298,7 @@ private:
             : area (et.getMaximumBounds().withSize (nextPowerOfTwo (et.getMaximumBounds().getWidth()),
                                                     nextPowerOfTwo (et.getMaximumBounds().getHeight())))
         {
-            data.calloc ((size_t) (area.getWidth() * area.getHeight()));
+            data.calloc (area.getWidth() * area.getHeight());
             et.iterate (*this);
         }
 
@@ -323,6 +327,24 @@ private:
             memset (currentLine + x, 255, (size_t) width);
         }
 
+        void handleEdgeTableRectangle (int x, int y, int width, int height, int alphaLevel) noexcept
+        {
+            while (--height >= 0)
+            {
+                setEdgeTableYPos (y++);
+                handleEdgeTableLine (x, width, alphaLevel);
+            }
+        }
+
+        void handleEdgeTableRectangleFull (int x, int y, int width, int height) noexcept
+        {
+            while (--height >= 0)
+            {
+                setEdgeTableYPos (y++);
+                handleEdgeTableLineFull (x, width);
+            }
+        }
+
         HeapBlock<uint8> data;
         const Rectangle<int> area;
 
@@ -334,9 +356,8 @@ private:
 };
 
 //==============================================================================
-class ShaderPrograms  : public ReferenceCountedObject
+struct ShaderPrograms  : public ReferenceCountedObject
 {
-public:
     ShaderPrograms (OpenGLContext& context)
         : solidColourProgram (context),
           solidColourMasked (context),
@@ -404,7 +425,7 @@ public:
               screenBounds (program, "screenBounds")
         {}
 
-        void set2DBounds (const Rectangle<float>& bounds)
+        void set2DBounds (Rectangle<float> bounds)
         {
             screenBounds.set (bounds.getX(), bounds.getY(), 0.5f * bounds.getWidth(), 0.5f * bounds.getHeight());
         }
@@ -436,7 +457,7 @@ public:
               maskBounds  (program, "maskBounds")
         {}
 
-        void setBounds (const Rectangle<int>& area, const Target& target, const GLint textureIndex) const
+        void setBounds (Rectangle<int> area, const Target& target, GLint textureIndex) const
         {
             maskTexture.set (textureIndex);
             maskBounds.set (area.getX() - target.bounds.getX(),
@@ -487,11 +508,11 @@ public:
               matrix (program, "matrix")
         {}
 
-        void setMatrix (const Point<float> p1, const Point<float> p2, const Point<float> p3)
+        void setMatrix (Point<float> p1, Point<float> p2, Point<float> p3)
         {
-            const AffineTransform t (AffineTransform::fromTargetPoints (p1.x, p1.y,  0.0f, 0.0f,
-                                                                        p2.x, p2.y,  1.0f, 0.0f,
-                                                                        p3.x, p3.y,  0.0f, 1.0f));
+            auto t = AffineTransform::fromTargetPoints (p1, Point<float>(),
+                                                        p2, Point<float> (1.0f, 0.0f),
+                                                        p3, Point<float> (0.0f, 1.0f));
             const GLfloat m[] = { t.mat00, t.mat01, t.mat02, t.mat10, t.mat11, t.mat12 };
             matrix.set (m, 6);
         }
@@ -632,15 +653,13 @@ public:
               imageLimits (program, "imageLimits")
         {}
 
-        void setMatrix (const AffineTransform& trans,
-                        const int imageWidth, const int imageHeight,
+        void setMatrix (const AffineTransform& trans, int imageWidth, int imageHeight,
                         float fullWidthProportion, float fullHeightProportion,
-                        const float targetX, const float targetY,
-                        const bool isForTiling) const
+                        float targetX, float targetY, bool isForTiling) const
         {
-            const AffineTransform t (trans.translated (-targetX, -targetY)
-                                        .inverted().scaled (fullWidthProportion / imageWidth,
-                                                            fullHeightProportion / imageHeight));
+            auto t = trans.translated (-targetX, -targetY)
+                          .inverted().scaled (fullWidthProportion / imageWidth,
+                                              fullHeightProportion / imageHeight);
 
             const GLfloat m[] = { t.mat00, t.mat01, t.mat02, t.mat10, t.mat11, t.mat12 };
             matrix.set (m, 6);
@@ -655,8 +674,7 @@ public:
         }
 
         void setMatrix (const AffineTransform& trans, const TextureInfo& textureInfo,
-                        const float targetX, const float targetY,
-                        bool isForTiling) const
+                        float targetX, float targetY, bool isForTiling) const
         {
             setMatrix (trans,
                        textureInfo.imageWidth, textureInfo.imageHeight,
@@ -823,13 +841,13 @@ struct StateHelpers
             srcFunction = dstFunction = 0;
         }
 
-        template <class QuadQueueType>
+        template <typename QuadQueueType>
         void setPremultipliedBlendingMode (QuadQueueType& quadQueue) noexcept
         {
             setBlendFunc (quadQueue, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
         }
 
-        template <class QuadQueueType>
+        template <typename QuadQueueType>
         void setBlendFunc (QuadQueueType& quadQueue, GLenum src, GLenum dst)
         {
             if (! blendingEnabled)
@@ -848,7 +866,7 @@ struct StateHelpers
             }
         }
 
-        template <class QuadQueueType>
+        template <typename QuadQueueType>
         void disableBlend (QuadQueueType& quadQueue) noexcept
         {
             if (blendingEnabled)
@@ -859,8 +877,8 @@ struct StateHelpers
             }
         }
 
-        template <class QuadQueueType>
-        void setBlendMode (QuadQueueType& quadQueue, const bool replaceExistingContents) noexcept
+        template <typename QuadQueueType>
+        void setBlendMode (QuadQueueType& quadQueue, bool replaceExistingContents) noexcept
         {
             if (replaceExistingContents)
                 disableBlend (quadQueue);
@@ -874,42 +892,54 @@ struct StateHelpers
     };
 
     //==============================================================================
-    template <class QuadQueueType>
+    template <typename QuadQueueType>
     struct EdgeTableRenderer
     {
-        EdgeTableRenderer (QuadQueueType& q, const PixelARGB c) noexcept
+        EdgeTableRenderer (QuadQueueType& q, PixelARGB c) noexcept
             : quadQueue (q), colour (c)
         {}
 
-        void setEdgeTableYPos (const int y) noexcept
+        void setEdgeTableYPos (int y) noexcept
         {
             currentY = y;
         }
 
-        void handleEdgeTablePixel (const int x, const int alphaLevel) noexcept
+        void handleEdgeTablePixel (int x, int alphaLevel) noexcept
         {
-            PixelARGB c (colour);
+            auto c = colour;
             c.multiplyAlpha (alphaLevel);
             quadQueue.add (x, currentY, 1, 1, c);
         }
 
-        void handleEdgeTablePixelFull (const int x) noexcept
+        void handleEdgeTablePixelFull (int x) noexcept
         {
             quadQueue.add (x, currentY, 1, 1, colour);
         }
 
-        void handleEdgeTableLine (const int x, const int width, const int alphaLevel) noexcept
+        void handleEdgeTableLine (int x, int width, int alphaLevel) noexcept
         {
-            PixelARGB c (colour);
+            auto c = colour;
             c.multiplyAlpha (alphaLevel);
             quadQueue.add (x, currentY, width, 1, c);
         }
 
-        void handleEdgeTableLineFull (const int x, const int width) noexcept
+        void handleEdgeTableLineFull (int x, int width) noexcept
         {
             quadQueue.add (x, currentY, width, 1, colour);
         }
 
+        void handleEdgeTableRectangle (int x, int y, int width, int height, int alphaLevel) noexcept
+        {
+            auto c = colour;
+            c.multiplyAlpha (alphaLevel);
+            quadQueue.add (x, y, width, height, c);
+        }
+
+        void handleEdgeTableRectangleFull (int x, int y, int width, int height) noexcept
+        {
+            quadQueue.add (x, y, width, height, colour);
+        }
+
     private:
         QuadQueueType& quadQueue;
         const PixelARGB colour;
@@ -918,14 +948,14 @@ struct StateHelpers
         JUCE_DECLARE_NON_COPYABLE (EdgeTableRenderer)
     };
 
-    template <class QuadQueueType>
+    template <typename QuadQueueType>
     struct FloatRectangleRenderer
     {
-        FloatRectangleRenderer (QuadQueueType& q, const PixelARGB c) noexcept
+        FloatRectangleRenderer (QuadQueueType& q, PixelARGB c) noexcept
             : quadQueue (q), colour (c)
         {}
 
-        void operator() (const int x, const int y, const int w, const int h, const int alpha) noexcept
+        void operator() (int x, int y, int w, int h, int alpha) noexcept
         {
             if (w > 0 && h > 0)
             {
@@ -954,8 +984,8 @@ struct StateHelpers
             zeromem (currentTextureID, sizeof (currentTextureID));
         }
 
-        template <class QuadQueueType>
-        void setTexturesEnabled (QuadQueueType& quadQueue, const int textureIndexMask) noexcept
+        template <typename QuadQueueType>
+        void setTexturesEnabled (QuadQueueType& quadQueue, int textureIndexMask) noexcept
         {
             if (texturesEnabled != textureIndexMask)
             {
@@ -986,20 +1016,20 @@ struct StateHelpers
             }
         }
 
-        template <class QuadQueueType>
+        template <typename QuadQueueType>
         void disableTextures (QuadQueueType& quadQueue) noexcept
         {
             setTexturesEnabled (quadQueue, 0);
         }
 
-        template <class QuadQueueType>
+        template <typename QuadQueueType>
         void setSingleTextureMode (QuadQueueType& quadQueue) noexcept
         {
             setTexturesEnabled (quadQueue, 1);
             setActiveTexture (0);
         }
 
-        template <class QuadQueueType>
+        template <typename QuadQueueType>
         void setTwoTextureMode (QuadQueueType& quadQueue, GLuint texture1, GLuint texture2)
         {
             JUCE_CHECK_OPENGL_ERROR
@@ -1021,7 +1051,7 @@ struct StateHelpers
             JUCE_CHECK_OPENGL_ERROR
         }
 
-        void setActiveTexture (const int index) noexcept
+        void setActiveTexture (int index) noexcept
         {
             if (currentActiveTexture != index)
             {
@@ -1031,13 +1061,13 @@ struct StateHelpers
             }
         }
 
-        void bindTexture (const GLuint textureID) noexcept
+        void bindTexture (GLuint textureID) noexcept
         {
             jassert (currentActiveTexture >= 0);
 
-            if (currentTextureID [currentActiveTexture] != textureID)
+            if (currentTextureID[currentActiveTexture] != textureID)
             {
-                currentTextureID [currentActiveTexture] = textureID;
+                currentTextureID[currentActiveTexture] = textureID;
                 glBindTexture (GL_TEXTURE_2D, textureID);
                 JUCE_CHECK_OPENGL_ERROR
             }
@@ -1052,7 +1082,7 @@ struct StateHelpers
         }
 
     private:
-        GLuint currentTextureID [3];
+        GLuint currentTextureID[3];
         int texturesEnabled, currentActiveTexture;
         const OpenGLContext& context;
 
@@ -1062,9 +1092,7 @@ struct StateHelpers
     //==============================================================================
     struct TextureCache
     {
-        TextureCache() noexcept
-            : activeGradientIndex (0), gradientNeedsRefresh (true)
-        {}
+        TextureCache() noexcept {}
 
         OpenGLTexture* getTexture (ActiveTextures& activeTextures, int w, int h)
         {
@@ -1076,7 +1104,8 @@ struct StateHelpers
 
             for (int i = 0; i < numTexturesToCache - 2; ++i)
             {
-                const OpenGLTexture* const t = textures.getUnchecked(i);
+                auto* t = textures.getUnchecked(i);
+
                 if (t->getWidth() == w && t->getHeight() == h)
                     return textures.removeAndReturn (i);
             }
@@ -1107,7 +1136,7 @@ struct StateHelpers
                 }
 
                 JUCE_CHECK_OPENGL_ERROR;
-                PixelARGB lookup [gradientTextureSize];
+                PixelARGB lookup[gradientTextureSize];
                 gradient.createLookupTable (lookup, gradientTextureSize);
                 gradientTextures.getUnchecked (activeGradientIndex)->loadARGB (lookup, gradientTextureSize, 1);
             }
@@ -1120,20 +1149,19 @@ struct StateHelpers
     private:
         enum { numTexturesToCache = 8, numGradientTexturesToCache = 10 };
         OwnedArray<OpenGLTexture> textures, gradientTextures;
-        int activeGradientIndex;
-        bool gradientNeedsRefresh;
+        int activeGradientIndex = 0;
+        bool gradientNeedsRefresh = true;
     };
 
     //==============================================================================
     struct ShaderQuadQueue
     {
-        ShaderQuadQueue (const OpenGLContext& c) noexcept
-            : context (c), numVertices (0)
+        ShaderQuadQueue (const OpenGLContext& c) noexcept  : context (c)
         {}
 
         ~ShaderQuadQueue() noexcept
         {
-            static_jassert (sizeof (VertexInfo) == 8);
+            static_assert (sizeof (VertexInfo) == 8, "Sanity check VertexInfo size");
             context.extensions.glBindBuffer (GL_ARRAY_BUFFER, 0);
             context.extensions.glBindBuffer (GL_ELEMENT_ARRAY_BUFFER, 0);
             context.extensions.glDeleteBuffers (2, buffers);
@@ -1142,6 +1170,16 @@ struct StateHelpers
         void initialise() noexcept
         {
             JUCE_CHECK_OPENGL_ERROR
+
+           #if JUCE_ANDROID || JUCE_IOS
+            int numQuads = maxNumQuads;
+           #else
+            GLint maxIndices = 0;
+            glGetIntegerv (GL_MAX_ELEMENTS_INDICES, &maxIndices);
+            auto numQuads = jmin ((int) maxNumQuads, (int) maxIndices / 6);
+            maxVertices = numQuads * 4 - 4;
+           #endif
+
             for (int i = 0, v = 0; i < numQuads * 6; i += 6, v += 4)
             {
                 indexData[i] = (GLushort) v;
@@ -1158,22 +1196,22 @@ struct StateHelpers
             JUCE_CHECK_OPENGL_ERROR
         }
 
-        void add (const int x, const int y, const int w, const int h, const PixelARGB colour) noexcept
+        void add (int x, int y, int w, int h, PixelARGB colour) noexcept
         {
             jassert (w > 0 && h > 0);
 
-            VertexInfo* const v = vertexData + numVertices;
+            auto* v = vertexData + numVertices;
             v[0].x = v[2].x = (GLshort) x;
             v[0].y = v[1].y = (GLshort) y;
             v[1].x = v[3].x = (GLshort) (x + w);
             v[2].y = v[3].y = (GLshort) (y + h);
 
            #if JUCE_BIG_ENDIAN
-            const GLuint rgba = (GLuint) ((colour.getRed() << 24) | (colour.getGreen() << 16)
-                                        | (colour.getBlue() << 8) |  colour.getAlpha());
+            auto rgba = (GLuint) ((colour.getRed() << 24) | (colour.getGreen() << 16)
+                                | (colour.getBlue() << 8) |  colour.getAlpha());
            #else
-            const GLuint rgba = (GLuint) ((colour.getAlpha() << 24) | (colour.getBlue() << 16)
-                                        | (colour.getGreen() << 8) |  colour.getRed());
+            auto rgba = (GLuint) ((colour.getAlpha() << 24) | (colour.getBlue() << 16)
+                                | (colour.getGreen() << 8) |  colour.getRed());
            #endif
 
             v[0].colour = rgba;
@@ -1183,40 +1221,40 @@ struct StateHelpers
 
             numVertices += 4;
 
-            if (numVertices > numQuads * 4 - 4)
+            if (numVertices > maxVertices)
                 draw();
         }
 
-        void add (const Rectangle<int>& r, const PixelARGB colour) noexcept
+        void add (Rectangle<int> r, PixelARGB colour) noexcept
         {
             add (r.getX(), r.getY(), r.getWidth(), r.getHeight(), colour);
         }
 
-        void add (const Rectangle<float>& r, const PixelARGB colour) noexcept
+        void add (Rectangle<float> r, PixelARGB colour) noexcept
         {
             FloatRectangleRenderer<ShaderQuadQueue> frr (*this, colour);
             RenderingHelpers::FloatRectangleRasterisingInfo (r).iterate (frr);
         }
 
-        void add (const RectangleList<int>& list, const PixelARGB colour) noexcept
+        void add (const RectangleList<int>& list, PixelARGB colour) noexcept
         {
-            for (const Rectangle<int>* i = list.begin(), * const e = list.end(); i != e; ++i)
-                add (*i, colour);
+            for (auto& i : list)
+                add (i, colour);
         }
 
-        void add (const RectangleList<int>& list, const Rectangle<int>& clip, const PixelARGB colour) noexcept
+        void add (const RectangleList<int>& list, Rectangle<int> clip, PixelARGB colour) noexcept
         {
-            for (const Rectangle<int>* i = list.begin(), * const e = list.end(); i != e; ++i)
+            for (auto& i : list)
             {
-                const Rectangle<int> r (i->getIntersection (clip));
+                auto r = i.getIntersection (clip);
 
                 if (! r.isEmpty())
                     add (r, colour);
             }
         }
 
-        template <class IteratorType>
-        void add (const IteratorType& et, const PixelARGB colour)
+        template <typename IteratorType>
+        void add (const IteratorType& et, PixelARGB colour)
         {
             EdgeTableRenderer<ShaderQuadQueue> etr (*this, colour);
             et.iterate (etr);
@@ -1235,13 +1273,19 @@ struct StateHelpers
             GLuint colour;
         };
 
-        enum { numQuads = 256 };
+        enum { maxNumQuads = 256 };
 
         GLuint buffers[2];
-        VertexInfo vertexData [numQuads * 4];
-        GLushort indexData [numQuads * 6];
+        VertexInfo vertexData[maxNumQuads * 4];
+        GLushort indexData[maxNumQuads * 6];
         const OpenGLContext& context;
-        int numVertices;
+        int numVertices = 0;
+
+       #if JUCE_ANDROID || JUCE_IOS
+        enum { maxVertices = maxNumQuads * 4 - 4 };
+       #else
+        int maxVertices = 0;
+       #endif
 
         void draw() noexcept
         {
@@ -1259,10 +1303,9 @@ struct StateHelpers
     //==============================================================================
     struct CurrentShader
     {
-        CurrentShader (OpenGLContext& c) noexcept
-            : context (c), activeShader (nullptr)
+        CurrentShader (OpenGLContext& c) noexcept  : context (c)
         {
-            const char programValueID[] = "GraphicsContextPrograms";
+            auto programValueID = "GraphicsContextPrograms";
             programs = static_cast<ShaderPrograms*> (context.getAssociatedObject (programValueID));
 
             if (programs == nullptr)
@@ -1277,7 +1320,7 @@ struct StateHelpers
             jassert (activeShader == nullptr);
         }
 
-        void setShader (const Rectangle<int>& bounds, ShaderQuadQueue& quadQueue, ShaderPrograms::ShaderBase& shader)
+        void setShader (Rectangle<int> bounds, ShaderQuadQueue& quadQueue, ShaderPrograms::ShaderBase& shader)
         {
             if (activeShader != &shader)
             {
@@ -1319,7 +1362,7 @@ struct StateHelpers
         ShaderPrograms::Ptr programs;
 
     private:
-        ShaderPrograms::ShaderBase* activeShader;
+        ShaderPrograms::ShaderBase* activeShader = nullptr;
         Rectangle<int> currentBounds;
 
         CurrentShader& operator= (const CurrentShader&);
@@ -1327,9 +1370,8 @@ struct StateHelpers
 };
 
 //==============================================================================
-class GLState
+struct GLState
 {
-public:
     GLState (const Target& t) noexcept
         : target (t),
           activeTextures (t.context),
@@ -1370,7 +1412,7 @@ public:
     }
 
     void setShaderForGradientFill (const ColourGradient& g, const AffineTransform& transform,
-                                   const int maskTextureID, const Rectangle<int>* const maskArea)
+                                   int maskTextureID, const Rectangle<int>* maskArea)
     {
         JUCE_CHECK_OPENGL_ERROR
         activeTextures.disableTextures (shaderQuadQueue);
@@ -1391,12 +1433,12 @@ public:
             textureCache.bindTextureForGradient (activeTextures, g);
         }
 
-        const AffineTransform t (transform.translated (0.5f - target.bounds.getX(),
-                                                       0.5f - target.bounds.getY()));
-        Point<float> p1 (g.point1.transformedBy (t));
-        const Point<float> p2 (g.point2.transformedBy (t));
-        const Point<float> p3 (Point<float> (g.point1.x + (g.point2.y - g.point1.y),
-                                             g.point1.y - (g.point2.x - g.point1.x)).transformedBy (t));
+        auto t = transform.translated (0.5f - target.bounds.getX(),
+                                       0.5f - target.bounds.getY());
+        auto p1 = g.point1.transformedBy (t);
+        auto p2 = g.point2.transformedBy (t);
+        auto p3 = Point<float> (g.point1.x + (g.point2.y - g.point1.y),
+                                g.point1.y - (g.point2.x - g.point1.x)).transformedBy (t);
 
         ShaderPrograms* const programs = currentShader.programs;
         const ShaderPrograms::MaskedShaderParams* maskParams = nullptr;
@@ -1471,7 +1513,7 @@ public:
     }
 
     void setShaderForTiledImageFill (const TextureInfo& textureInfo, const AffineTransform& transform,
-                                     const int maskTextureID, const Rectangle<int>* const maskArea, bool isTiledFill)
+                                     int maskTextureID, const Rectangle<int>* maskArea, bool isTiledFill)
     {
         blendMode.setPremultipliedBlendingMode (shaderQuadQueue);
 
@@ -1535,33 +1577,30 @@ private:
 };
 
 //==============================================================================
-class SavedState  : public RenderingHelpers::SavedStateBase<SavedState>
+struct SavedState  : public RenderingHelpers::SavedStateBase<SavedState>
 {
     typedef RenderingHelpers::SavedStateBase<SavedState> BaseClass;
 
-public:
-    SavedState (GLState* const s)
-        : BaseClass (s->target.bounds), state (s), isUsingCustomShader (false)
+    SavedState (GLState* s)  : BaseClass (s->target.bounds), state (s)
     {}
 
     SavedState (const SavedState& other)
-        : BaseClass (other), font (other.font),
-          state (other.state), isUsingCustomShader (false),
+        : BaseClass (other), font (other.font), state (other.state),
           transparencyLayer (other.transparencyLayer),
-          previousTarget (other.previousTarget.createCopy())
+          previousTarget (createCopyIfNotNull (other.previousTarget.get()))
     {}
 
     SavedState* beginTransparencyLayer (float opacity)
     {
-        SavedState* const s = new SavedState (*this);
+        auto* s = new SavedState (*this);
 
         if (clip != nullptr)
         {
-            const Rectangle<int> clipBounds (clip->getClipBounds());
+            auto clipBounds = clip->getClipBounds();
 
             state->flush();
             s->transparencyLayer = Image (OpenGLImageType().create (Image::ARGB, clipBounds.getWidth(), clipBounds.getHeight(), true));
-            s->previousTarget = new Target (state->target);
+            s->previousTarget.reset (new Target (state->target));
             state->target = Target (state->target.context, *OpenGLImageType::getFrameBufferFrom (s->transparencyLayer), clipBounds.getPosition());
             s->transparencyLayerAlpha = opacity;
             s->cloneClipIfMultiplyReferenced();
@@ -1580,10 +1619,10 @@ public:
 
             state->flush();
             state->target = *finishedLayerState.previousTarget;
-            finishedLayerState.previousTarget = nullptr;
+            finishedLayerState.previousTarget.reset();
 
             state->target.makeActive();
-            const Rectangle<int> clipBounds (clip->getClipBounds());
+            auto clipBounds = clip->getClipBounds();
 
             clip->renderImageUntransformed (*this, finishedLayerState.transparencyLayer,
                                             (int) (finishedLayerState.transparencyLayerAlpha * 255.0f),
@@ -1599,8 +1638,7 @@ public:
         {
             if (trans.isOnlyTranslation() && ! transform.isRotated)
             {
-                GlyphCacheType& cache = GlyphCacheType::getInstance();
-
+                auto& cache = GlyphCacheType::getInstance();
                 Point<float> pos (trans.getTranslationX(), trans.getTranslationY());
 
                 if (transform.isOnlyTranslated)
@@ -1614,7 +1652,8 @@ public:
                     Font f (font);
                     f.setHeight (font.getHeight() * transform.complexTransform.mat11);
 
-                    const float xScale = transform.complexTransform.mat00 / transform.complexTransform.mat11;
+                    auto xScale = transform.complexTransform.mat00 / transform.complexTransform.mat11;
+
                     if (std::abs (xScale - 1.0f) > 0.01f)
                         f.setHorizontalScale (xScale);
 
@@ -1623,10 +1662,10 @@ public:
             }
             else
             {
-                const float fontHeight = font.getHeight();
+                auto fontHeight = font.getHeight();
 
-                AffineTransform t (transform.getTransformWith (AffineTransform::scale (fontHeight * font.getHorizontalScale(), fontHeight)
-                                                                               .followedBy (trans)));
+                auto t = transform.getTransformWith (AffineTransform::scale (fontHeight * font.getHorizontalScale(), fontHeight)
+                                                                     .followedBy (trans));
 
                 const ScopedPointer<EdgeTable> et (font.getTypeface()->getEdgeTableForGlyph (glyphNumber, t, fontHeight));
 
@@ -1646,7 +1685,7 @@ public:
 
     //==============================================================================
     template <typename IteratorType>
-    void renderImageTransformed (IteratorType& iter, const Image& src, const int alpha,
+    void renderImageTransformed (IteratorType& iter, const Image& src, int alpha,
                                  const AffineTransform& trans, Graphics::ResamplingQuality, bool tiledFill) const
     {
         state->shaderQuadQueue.flush();
@@ -1659,14 +1698,14 @@ public:
     }
 
     template <typename IteratorType>
-    void renderImageUntransformed (IteratorType& iter, const Image& src, const int alpha, int x, int y, bool tiledFill) const
+    void renderImageUntransformed (IteratorType& iter, const Image& src, int alpha, int x, int y, bool tiledFill) const
     {
         renderImageTransformed (iter, src, alpha, AffineTransform::translation ((float) x, (float) y),
                                 Graphics::lowResamplingQuality, tiledFill);
     }
 
     template <typename IteratorType>
-    void fillWithSolidColour (IteratorType& iter, const PixelARGB colour, bool replaceContents) const
+    void fillWithSolidColour (IteratorType& iter, PixelARGB colour, bool replaceContents) const
     {
         if (! isUsingCustomShader)
         {
@@ -1685,7 +1724,7 @@ public:
         state->shaderQuadQueue.add (iter, fillType.colour.getPixelARGB());
     }
 
-    void fillRectWithCustomShader (OpenGLRendering::ShaderPrograms::ShaderBase& shader, const Rectangle<int>& area)
+    void fillRectWithCustomShader (OpenGLRendering::ShaderPrograms::ShaderBase& shader, Rectangle<int> area)
     {
         state->setShader (shader);
         isUsingCustomShader = true;
@@ -1699,7 +1738,7 @@ public:
     //==============================================================================
     Font font;
     GLState* state;
-    bool isUsingCustomShader;
+    bool isUsingCustomShader = false;
 
 private:
     Image transparencyLayer;
@@ -1710,15 +1749,14 @@ private:
 
 
 //==============================================================================
-class ShaderContext   : public RenderingHelpers::StackBasedLowLevelGraphicsContext<SavedState>
+struct ShaderContext   : public RenderingHelpers::StackBasedLowLevelGraphicsContext<SavedState>
 {
-public:
     ShaderContext (const Target& target)  : glState (target)
     {
         stack.initialise (new SavedState (&glState));
     }
 
-    void fillRectWithCustomShader (ShaderPrograms::ShaderBase& shader, const Rectangle<int>& area)
+    void fillRectWithCustomShader (ShaderPrograms::ShaderBase& shader, Rectangle<int> area)
     {
         static_cast<SavedState&> (*stack).fillRectWithCustomShader (shader, area);
     }
@@ -1728,9 +1766,8 @@ public:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ShaderContext)
 };
 
-class NonShaderContext   : public LowLevelGraphicsSoftwareRenderer
+struct NonShaderContext   : public LowLevelGraphicsSoftwareRenderer
 {
-public:
     NonShaderContext (const Target& t, const Image& im)
         : LowLevelGraphicsSoftwareRenderer (im), target (t), image (im)
     {}
@@ -1738,7 +1775,7 @@ public:
     ~NonShaderContext()
     {
         JUCE_CHECK_OPENGL_ERROR
-        const GLuint previousFrameBufferTarget = OpenGLFrameBuffer::getCurrentFrameBufferTarget();
+        auto previousFrameBufferTarget = OpenGLFrameBuffer::getCurrentFrameBufferTarget();
 
        #if ! JUCE_ANDROID
         target.context.extensions.glActiveTexture (GL_TEXTURE0);
@@ -1798,7 +1835,7 @@ LowLevelGraphicsContext* createOpenGLGraphicsContext (OpenGLContext& context, in
 
 LowLevelGraphicsContext* createOpenGLGraphicsContext (OpenGLContext& context, OpenGLFrameBuffer& target)
 {
-    return OpenGLRendering::createOpenGLContext (OpenGLRendering::Target (context, target, Point<int>()));
+    return OpenGLRendering::createOpenGLContext (OpenGLRendering::Target (context, target, {}));
 }
 
 LowLevelGraphicsContext* createOpenGLGraphicsContext (OpenGLContext& context, unsigned int frameBufferID, int width, int height)
@@ -1817,7 +1854,7 @@ struct CustomProgram  : public ReferenceCountedObject,
 
     static CustomProgram* get (const String& hashName)
     {
-        if (OpenGLContext* c = OpenGLContext::getCurrentContext())
+        if (auto* c = OpenGLContext::getCurrentContext())
             return static_cast<CustomProgram*> (c->getAssociatedObject (hashName.toRawUTF8()));
 
         return nullptr;
@@ -1825,13 +1862,12 @@ struct CustomProgram  : public ReferenceCountedObject,
 
     static CustomProgram* getOrCreate (LowLevelGraphicsContext& gc, const String& hashName, const String& code, String& errorMessage)
     {
-        if (CustomProgram* c = get (hashName))
+        if (auto* c = get (hashName))
             return c;
 
-        if (OpenGLRendering::ShaderContext* sc = dynamic_cast<OpenGLRendering::ShaderContext*> (&gc))
+        if (auto* sc = dynamic_cast<OpenGLRendering::ShaderContext*> (&gc))
         {
             ReferenceCountedObjectPtr<CustomProgram> c (new CustomProgram (*sc, code));
-
             errorMessage = c->lastError;
 
             if (errorMessage.isEmpty())
@@ -1874,7 +1910,7 @@ OpenGLShaderProgram* OpenGLGraphicsContextCustomShader::getProgram (LowLevelGrap
     return nullptr;
 }
 
-void OpenGLGraphicsContextCustomShader::fillRect (LowLevelGraphicsContext& gc, const Rectangle<int>& area) const
+void OpenGLGraphicsContextCustomShader::fillRect (LowLevelGraphicsContext& gc, Rectangle<int> area) const
 {
     String errorMessage;
 
@@ -1892,3 +1928,5 @@ Result OpenGLGraphicsContextCustomShader::checkCompilation (LowLevelGraphicsCont
 
     return Result::fail (errorMessage);
 }
+
+} // namespace juce
