@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 OSCArgument::OSCArgument (int32 value) noexcept
     : type (OSCTypes::int32), intValue (value)
 {
@@ -49,7 +54,7 @@ String OSCArgument::getString() const noexcept
         return stringValue;
 
     jassertfalse; // you must check the type of an argument before attempting to get its value!
-    return String();
+    return {};
 }
 
 int32 OSCArgument::getInt32() const noexcept
@@ -86,7 +91,7 @@ const MemoryBlock& OSCArgument::getBlob() const noexcept
 class OSCArgumentTests  : public UnitTest
 {
 public:
-    OSCArgumentTests() : UnitTest ("OSCArgument class") {}
+    OSCArgumentTests() : UnitTest ("OSCArgument class", "OSC") {}
 
 
     MemoryBlock getMemoryBlockWithRandomData (size_t numBytes)
@@ -216,17 +221,6 @@ public:
                 assignment = copy;
                 expect (assignment.getType() == OSCTypes::blob);
                 expect (assignment.getBlob() == blob);
-
-               #if JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
-                OSCArgument move = std::move (arg);
-                expect (move.getType() == OSCTypes::blob);
-                expect (move.getBlob() == blob);
-
-                OSCArgument moveAssignment ("this will be overwritten!");
-                moveAssignment = std::move (copy);
-                expect (moveAssignment.getType() == OSCTypes::blob);
-                expect (moveAssignment.getBlob() == blob);
-               #endif
            }
         }
     }
@@ -235,3 +229,5 @@ public:
 static OSCArgumentTests OSCArgumentUnitTests;
 
 #endif // JUCE_UNIT_TESTS
+
+} // namespace juce
