@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 OSCBundle::OSCBundle()
 {
 }
@@ -109,7 +114,7 @@ const OSCBundle& OSCBundle::Element::getBundle() const
 class OSCBundleTests  : public UnitTest
 {
 public:
-    OSCBundleTests() : UnitTest ("OSCBundle class") {}
+    OSCBundleTests() : UnitTest ("OSCBundle class", "OSC") {}
 
     void runTest()
     {
@@ -190,15 +195,15 @@ private:
         expect (! bundle[1].isBundle());
 
         int numElementsCounted = 0;
-        for (OSCBundle::Element* element = bundle.begin(); element != bundle.end(); ++element)
+        for (auto& element : bundle)
         {
-            expect (element->isMessage());
-            expect (! element->isBundle());
+            expect (element.isMessage());
+            expect (! element.isBundle());
             ++numElementsCounted;
         }
         expectEquals (numElementsCounted, 2);
 
-        OSCBundle::Element* e = bundle.begin();
+        auto* e = bundle.begin();
         expect (e[0].getMessage().size() == 1);
         expect (e[0].getMessage().begin()->getInt32() == testInt);
         expect (e[1].getMessage().size() == 2);
@@ -212,7 +217,7 @@ static OSCBundleTests OSCBundleUnitTests;
 class OSCBundleElementTests  : public UnitTest
 {
 public:
-    OSCBundleElementTests() : UnitTest ("OSCBundle::Element class") {}
+    OSCBundleElementTests() : UnitTest ("OSCBundle::Element class", "OSC") {}
 
     void runTest()
     {
@@ -235,3 +240,5 @@ public:
 static OSCBundleElementTests OSCBundleElementUnitTests;
 
 #endif // JUCE_UNIT_TESTS
+
+} // namespace juce
