@@ -2,27 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-//==============================================================================
+namespace juce
+{
+
 OSCMessage::OSCMessage (const OSCAddressPattern& ap) noexcept  : addressPattern (ap)
 {
 }
@@ -49,7 +53,12 @@ bool OSCMessage::isEmpty() const noexcept
     return arguments.isEmpty();
 }
 
-OSCArgument& OSCMessage::operator[] (const int i) const noexcept
+OSCArgument& OSCMessage::operator[] (const int i) noexcept
+{
+    return arguments.getReference (i);
+}
+
+const OSCArgument& OSCMessage::operator[] (const int i) const noexcept
 {
     return arguments.getReference (i);
 }
@@ -83,7 +92,7 @@ void OSCMessage::addArgument (OSCArgument arg)      { arguments.add (arg); }
 class OSCMessageTests  : public UnitTest
 {
 public:
-    OSCMessageTests() : UnitTest ("OSCMessage class") {}
+    OSCMessageTests() : UnitTest ("OSCMessage class", "OSC") {}
 
     void runTest()
     {
@@ -143,7 +152,6 @@ public:
         }
 
 
-       #if JUCE_COMPILER_SUPPORTS_VARIADIC_TEMPLATES && JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
         beginTest ("Initialisation with argument list (C++11 only)");
         {
             int testInt = 42;
@@ -188,10 +196,11 @@ public:
                 expectEquals (msg[4].getInt32(), testInt);
             }
         }
-       #endif
     }
 };
 
 static OSCMessageTests OSCMessageUnitTests;
 
 #endif // JUCE_UNIT_TESTS
+
+} // namespace juce
