@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_OSCMESSAGE_H_INCLUDED
-#define JUCE_OSCMESSAGE_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -52,7 +53,6 @@ public:
     OSCMessage (const OSCAddressPattern& ap) noexcept;
 
 
-   #if JUCE_COMPILER_SUPPORTS_VARIADIC_TEMPLATES && JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
     /** Constructs an OSCMessage object with the given address pattern and list
         of arguments.
 
@@ -67,7 +67,6 @@ public:
     */
     template <typename Arg1, typename... Args>
     OSCMessage (const OSCAddressPattern& ap, Arg1&& arg1, Args&&... args);
-   #endif
 
     /** Sets the address pattern of the OSCMessage.
 
@@ -92,7 +91,8 @@ public:
         This method does not check the range and results in undefined behaviour
         in case i < 0 or i >= size().
     */
-    OSCArgument& operator[] (const int i) const noexcept;
+    OSCArgument& operator[] (const int i) noexcept;
+    const OSCArgument& operator[] (const int i) const noexcept;
 
     /** Returns a pointer to the first OSCArgument in the OSCMessage object.
         This method is provided for compatibility with standard C++ iteration mechanisms.
@@ -107,7 +107,6 @@ public:
     /** Removes all arguments from the OSCMessage. */
     void clear();
 
-
     //==============================================================================
     /** Creates a new OSCArgument of type int32 with a given value
         and adds it to the OSCMessage object.
@@ -142,7 +141,6 @@ public:
 private:
 
     //==============================================================================
-   #if JUCE_COMPILER_SUPPORTS_VARIADIC_TEMPLATES && JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
     template <typename Arg1, typename... Args>
     void addArguments (Arg1&& arg1, Args&&... args)
     {
@@ -151,7 +149,6 @@ private:
     }
 
     void addArguments() {}
-   #endif
 
     //==============================================================================
     OSCAddressPattern addressPattern;
@@ -160,13 +157,11 @@ private:
 
 
 //==============================================================================
-#if JUCE_COMPILER_SUPPORTS_VARIADIC_TEMPLATES && JUCE_COMPILER_SUPPORTS_MOVE_SEMANTICS
- template <typename Arg1, typename... Args>
- OSCMessage::OSCMessage (const OSCAddressPattern& ap, Arg1&& arg1, Args&&... args)
-     : addressPattern (ap)
- {
-     addArguments (std::forward<Arg1> (arg1), std::forward<Args> (args)...);
- }
-#endif
+template <typename Arg1, typename... Args>
+OSCMessage::OSCMessage (const OSCAddressPattern& ap, Arg1&& arg1, Args&&... args)
+    : addressPattern (ap)
+{
+    addArguments (std::forward<Arg1> (arg1), std::forward<Args> (args)...);
+}
 
-#endif // JUCE_OSCMESSAGE_H_INCLUDED
+} // namespace juce
