@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 namespace
 {
     //==============================================================================
@@ -55,7 +60,7 @@ namespace
         size_t getDataSize() const noexcept         { return input.getDataSize(); }
 
         /** Returns the current position of the stream. */
-        uint64 getPosition()                         { return uint64 (input.getPosition()); }
+        uint64 getPosition()                        { return uint64 (input.getPosition()); }
 
         /** Attempts to set the current position of the stream. Returns true if this was successful. */
         bool setPosition (int64 pos)                { return input.setPosition (pos); }
@@ -172,7 +177,7 @@ namespace
                 typeList.add (type);
             }
 
-            size_t bytesRead = (size_t) typeList.size() + 2;
+            auto bytesRead = (size_t) typeList.size() + 2;
             readPaddingZeros (bytesRead);
 
             return typeList;
@@ -203,15 +208,19 @@ namespace
 
             OSCMessage msg (ap);
 
-            for (OSCType* type = types.begin(); type != types.end(); ++type)
-                msg.addArgument (readArgument (*type));
+            for (auto& type : types)
+                msg.addArgument (readArgument (type));
 
             return msg;
         }
 
         //==============================================================================
-        OSCBundle readBundle()
+        OSCBundle readBundle (size_t maxBytesToRead = std::numeric_limits<size_t>::max())
         {
+            // maxBytesToRead is only passed in here in case this bundle is a nested
+            // bundle, so we know when to consider the next element *not* part of this
+            // bundle anymore (but part of the outer bundle) and return.
+
             if (input.getNumBytesRemaining() < 16)
                 throw OSCFormatError ("OSC input stream exhausted while reading bundle");
 
@@ -220,9 +229,18 @@ namespace
 
             OSCBundle bundle (readTimeTag());
 
-            while (! isExhausted())
+            size_t bytesRead = 16; // already read "#bundle" and timeTag
+            auto pos = getPosition();
+
+            while (! isExhausted() && bytesRead < maxBytesToRead)
+            {
                 bundle.addElement (readElement());
 
+                auto newPos = getPosition();
+                bytesRead += (size_t) (newPos - pos);
+                pos = newPos;
+            }
+
             return bundle;
         }
 
@@ -230,12 +248,12 @@ namespace
         OSCBundle::Element readElement()
         {
             if (input.getNumBytesRemaining() < 4)
-                throw OSCFormatError("OSC input stream exhausted while reading bundle element size");
+                throw OSCFormatError ("OSC input stream exhausted while reading bundle element size");
 
-            const size_t elementSize = (size_t) readInt32();
+            auto elementSize = (size_t) readInt32();
 
             if (elementSize < 4)
-                throw OSCFormatError("OSC input stream format error: invalid bundle element size");
+                throw OSCFormatError ("OSC input stream format error: invalid bundle element size");
 
             return readElementWithKnownSize (elementSize);
         }
@@ -246,7 +264,7 @@ namespace
             if ((uint64) input.getNumBytesRemaining() < elementSize)
                 throw OSCFormatError ("OSC input stream exhausted while reading bundle element content");
 
-            const char firstContentChar = static_cast<const char*> (getData()) [getPosition()];
+            auto firstContentChar = static_cast<const char*> (getData()) [getPosition()];
 
             if (firstContentChar == '/')  return OSCBundle::Element (readMessageWithCheckedSize (elementSize));
             if (firstContentChar == '#')  return OSCBundle::Element (readBundleWithCheckedSize (elementSize));
@@ -273,8 +291,10 @@ namespace
 
         OSCBundle readBundleWithCheckedSize (size_t size)
         {
-            const size_t begin = (size_t) getPosition();
-            OSCBundle bundle (readBundle());
+            auto begin = (size_t) getPosition();
+            auto maxBytesToRead = size - 4; // we've already read 4 bytes (the bundle size)
+
+            OSCBundle bundle (readBundle (maxBytesToRead));
 
             if (getPosition() - begin != size)
                 throw OSCFormatError ("OSC input stream format error: wrong element content size encountered while reading");
@@ -284,7 +304,7 @@ namespace
 
         OSCMessage readMessageWithCheckedSize (size_t size)
         {
-            const size_t begin = (size_t) getPosition();
+            auto begin = (size_t) getPosition();
             OSCMessage message (readMessage());
 
             if (getPosition() - begin != size)
@@ -301,9 +321,7 @@ namespace
 struct OSCReceiver::Pimpl   : private Thread,
                               private MessageListener
 {
-    Pimpl()
-      : Thread ("Juce OSC server"),
-        formatErrorHandler (nullptr)
+    Pimpl()  : Thread ("JUCE OSC server")
     {
     }
 
@@ -313,13 +331,12 @@ struct OSCReceiver::Pimpl   : private Thread,
     }
 
     //==============================================================================
-    bool connectToPort (int portNum)
+    bool connectToPort (int portNumber)
     {
         if (! disconnect())
             return false;
 
-        portNumber = portNum;
-        socket = new DatagramSocket (false);
+        socket.setOwned (new DatagramSocket (false));
 
         if (! socket->bindToPort (portNumber))
             return false;
@@ -328,25 +345,38 @@ struct OSCReceiver::Pimpl   : private Thread,
         return true;
     }
 
+    bool connectToSocket (DatagramSocket& newSocket)
+    {
+        if (! disconnect())
+            return false;
+
+        socket.setNonOwned (&newSocket);
+        startThread();
+        return true;
+    }
+
     bool disconnect()
     {
         if (socket != nullptr)
         {
             signalThreadShouldExit();
-            socket->shutdown();
+
+            if (socket.willDeleteObject())
+                socket->shutdown();
+
             waitForThreadToExit (10000);
-            socket = nullptr;
+            socket.reset();
         }
         return true;
     }
 
     //==============================================================================
-    void addListener (Listener<MessageLoopCallback>* listenerToAdd)
+    void addListener (OSCReceiver::Listener<MessageLoopCallback>* listenerToAdd)
     {
         listeners.add (listenerToAdd);
     }
 
-    void addListener (Listener<RealtimeCallback>* listenerToAdd)
+    void addListener (OSCReceiver::Listener<RealtimeCallback>* listenerToAdd)
     {
         realtimeListeners.add (listenerToAdd);
     }
@@ -363,12 +393,12 @@ struct OSCReceiver::Pimpl   : private Thread,
         addListenerWithAddress (listenerToAdd, addressToMatch, realtimeListenersWithAddress);
     }
 
-    void removeListener (Listener<MessageLoopCallback>* listenerToRemove)
+    void removeListener (OSCReceiver::Listener<MessageLoopCallback>* listenerToRemove)
     {
         listeners.remove (listenerToRemove);
     }
 
-    void removeListener (Listener<RealtimeCallback>* listenerToRemove)
+    void removeListener (OSCReceiver::Listener<RealtimeCallback>* listenerToRemove)
     {
         realtimeListeners.remove (listenerToRemove);
     }
@@ -439,7 +469,7 @@ private:
             if (threadShouldExit())
                 return;
 
-            const size_t bytesRead = (size_t) socket->read (buffer, (int) sizeof (buffer), false);
+            auto bytesRead = (size_t) socket->read (buffer, (int) sizeof (buffer), false);
 
             if (bytesRead >= 4)
                 handleBuffer (buffer, bytesRead);
@@ -450,10 +480,10 @@ private:
     template <typename ListenerType>
     void addListenerWithAddress (ListenerType* listenerToAdd,
                                  OSCAddress address,
-                                 Array<std::pair<OSCAddress, ListenerType*> >& array)
+                                 Array<std::pair<OSCAddress, ListenerType*>>& array)
     {
-        for (int i = 0; i < array.size(); ++i)
-            if (address == array.getReference (i).first && listenerToAdd == array.getReference (i).second)
+        for (auto& i : array)
+            if (address == i.first && listenerToAdd == i.second)
                 return;
 
         array.add (std::make_pair (address, listenerToAdd));
@@ -462,7 +492,7 @@ private:
     //==============================================================================
     template <typename ListenerType>
     void removeListenerWithAddress (ListenerType* listenerToRemove,
-                                    Array<std::pair<OSCAddress, ListenerType*> >& array)
+                                    Array<std::pair<OSCAddress, ListenerType*>>& array)
     {
         for (int i = 0; i < array.size(); ++i)
         {
@@ -481,9 +511,9 @@ private:
     //==============================================================================
     void handleMessage (const Message& msg) override
     {
-        if (const CallbackMessage* callbackMessage = dynamic_cast<const CallbackMessage*> (&msg))
+        if (auto* callbackMessage = dynamic_cast<const CallbackMessage*> (&msg))
         {
-            const OSCBundle::Element& content = callbackMessage->content;
+            auto& content = callbackMessage->content;
 
             callListeners (content);
 
@@ -498,9 +528,15 @@ private:
         typedef OSCReceiver::Listener<OSCReceiver::MessageLoopCallback> Listener;
 
         if (content.isMessage())
-            listeners.call (&Listener::oscMessageReceived, content.getMessage());
+        {
+            auto&& message = content.getMessage();
+            listeners.call ([&] (Listener& l) { l.oscMessageReceived (message); });
+        }
         else if (content.isBundle())
-            listeners.call (&Listener::oscBundleReceived, content.getBundle());
+        {
+            auto&& bundle = content.getBundle();
+            listeners.call ([&] (Listener& l) { l.oscBundleReceived (bundle); });
+        }
     }
 
     void callRealtimeListeners (const OSCBundle::Element& content)
@@ -508,50 +544,43 @@ private:
         typedef OSCReceiver::Listener<OSCReceiver::RealtimeCallback> Listener;
 
         if (content.isMessage())
-            realtimeListeners.call (&Listener::oscMessageReceived, content.getMessage());
+        {
+            auto&& message = content.getMessage();
+            realtimeListeners.call ([&] (Listener& l) { l.oscMessageReceived (message); });
+        }
         else if (content.isBundle())
-            realtimeListeners.call (&Listener::oscBundleReceived, content.getBundle());
+        {
+            auto&& bundle = content.getBundle();
+            realtimeListeners.call ([&] (Listener& l) { l.oscBundleReceived (bundle); });
+        }
     }
 
     //==============================================================================
     void callListenersWithAddress (const OSCMessage& message)
     {
-        typedef OSCReceiver::ListenerWithOSCAddress<OSCReceiver::MessageLoopCallback> Listener;
-
-        for (std::pair<OSCAddress, Listener*>* entry = listenersWithAddress.begin();
-             entry != listenersWithAddress.end();
-             ++entry)
-        {
-            if (Listener* listener = entry->second)
-                if (message.getAddressPattern().matches (entry->first))
+        for (auto& entry : listenersWithAddress)
+            if (auto* listener = entry.second)
+                if (message.getAddressPattern().matches (entry.first))
                     listener->oscMessageReceived (message);
-        }
     }
 
     void callRealtimeListenersWithAddress (const OSCMessage& message)
     {
-        typedef OSCReceiver::ListenerWithOSCAddress<OSCReceiver::RealtimeCallback> Listener;
-
-        for (std::pair<OSCAddress, Listener*>* entry = realtimeListenersWithAddress.begin();
-             entry != realtimeListenersWithAddress.end();
-             ++entry)
-        {
-            if (Listener* listener = entry->second)
-                if (message.getAddressPattern().matches (entry->first))
+        for (auto& entry : realtimeListenersWithAddress)
+            if (auto* listener = entry.second)
+                if (message.getAddressPattern().matches (entry.first))
                     listener->oscMessageReceived (message);
-        }
     }
 
     //==============================================================================
-    ListenerList<OSCReceiver::Listener<OSCReceiver::MessageLoopCallback> > listeners;
-    ListenerList<OSCReceiver::Listener<OSCReceiver::RealtimeCallback> >    realtimeListeners;
+    ListenerList<OSCReceiver::Listener<OSCReceiver::MessageLoopCallback>> listeners;
+    ListenerList<OSCReceiver::Listener<OSCReceiver::RealtimeCallback>>    realtimeListeners;
 
-    Array<std::pair<OSCAddress, OSCReceiver::ListenerWithOSCAddress<OSCReceiver::MessageLoopCallback>* > > listenersWithAddress;
-    Array<std::pair<OSCAddress, OSCReceiver::ListenerWithOSCAddress<OSCReceiver::RealtimeCallback>* > >    realtimeListenersWithAddress;
+    Array<std::pair<OSCAddress, OSCReceiver::ListenerWithOSCAddress<OSCReceiver::MessageLoopCallback>*>> listenersWithAddress;
+    Array<std::pair<OSCAddress, OSCReceiver::ListenerWithOSCAddress<OSCReceiver::RealtimeCallback>*>>    realtimeListenersWithAddress;
 
-    ScopedPointer<DatagramSocket> socket;
-    int portNumber;
-    OSCReceiver::FormatErrorHandler formatErrorHandler;
+    OptionalScopedPointer<DatagramSocket> socket;
+    OSCReceiver::FormatErrorHandler formatErrorHandler { nullptr };
     enum { oscBufferSize = 4098 };
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Pimpl)
@@ -564,7 +593,7 @@ OSCReceiver::OSCReceiver()   : pimpl (new Pimpl())
 
 OSCReceiver::~OSCReceiver()
 {
-    pimpl = nullptr;
+    pimpl.reset();
 }
 
 bool OSCReceiver::connect (int portNumber)
@@ -572,12 +601,17 @@ bool OSCReceiver::connect (int portNumber)
     return pimpl->connectToPort (portNumber);
 }
 
+bool OSCReceiver::connectToSocket (DatagramSocket& socket)
+{
+    return pimpl->connectToSocket (socket);
+}
+
 bool OSCReceiver::disconnect()
 {
     return pimpl->disconnect();
 }
 
-void OSCReceiver::addListener (Listener<MessageLoopCallback>* listenerToAdd)
+void OSCReceiver::addListener (OSCReceiver::Listener<MessageLoopCallback>* listenerToAdd)
 {
     pimpl->addListener (listenerToAdd);
 }
@@ -630,7 +664,7 @@ void OSCReceiver::registerFormatErrorHandler (FormatErrorHandler handler)
 class OSCInputStreamTests  : public UnitTest
 {
 public:
-    OSCInputStreamTests() : UnitTest ("OSCInputStream class") {}
+    OSCInputStreamTests() : UnitTest ("OSCInputStream class", "OSC") {}
 
     void runTest()
     {
@@ -1151,3 +1185,5 @@ public:
 static OSCInputStreamTests OSCInputStreamUnitTests;
 
 #endif // JUCE_UNIT_TESTS
+
+} // namespace juce
