@@ -2,28 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_OSCRECEIVER_H_INCLUDED
-#define JUCE_OSCRECEIVER_H_INCLUDED
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -52,6 +54,14 @@ public:
     */
     bool connect (int portNumber);
 
+    /** Connects to a UDP datagram socket that is already set up,
+        and starts listening to OSC packets arriving on this port.
+        Make sure that the object you give it doesn't get deleted while this
+        object is still using it!
+        @returns true if the connection was successful; false otherwise.
+    */
+    bool connectToSocket (DatagramSocket& socketToUse);
+
     //==============================================================================
     /** Disconnects from the currently used UDP port.
         @returns true if the disconnection was successful; false otherwise.
@@ -188,11 +198,7 @@ public:
         The arguments passed are the pointer to and the data of the buffer that
         the OSCReceiver has failed to parse.
     */
-   #if JUCE_COMPILER_SUPPORTS_LAMBDAS
     typedef std::function<void (const char* data, int dataSize)> FormatErrorHandler;
-   #else
-    typedef void (*FormatErrorHandler) (const char* data, int dataSize);
-   #endif
 
     /** Installs a custom error handler which is called in case the receiver
         encounters a stream it cannot parse as an OSC bundle or OSC message.
@@ -213,5 +219,4 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (OSCReceiver)
 };
 
-
-#endif // JUCE_OSCRECEIVER_H_INCLUDED
+} // namespace juce
