@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 namespace
 {
     //==============================================================================
@@ -32,7 +37,7 @@ namespace
 
         This class implements the Open Sound Control 1.0 Specification for
         the format in which the OSC data will be written into the buffer.
-     */
+    */
     struct OSCOutputStream
     {
         OSCOutputStream() noexcept {}
@@ -134,14 +139,14 @@ namespace
 
             OSCTypeList typeList;
 
-            for (OSCArgument* arg = msg.begin(); arg != msg.end(); ++arg)
-                typeList.add (arg->getType());
+            for (auto& arg : msg)
+                typeList.add (arg.getType());
 
             if (! writeTypeTagString (typeList))
                 return false;
 
-            for (OSCArgument* arg = msg.begin(); arg != msg.end(); ++arg)
-                if (! writeArgument (*arg))
+            for (auto& arg : msg)
+                if (! writeArgument (arg))
                     return false;
 
             return true;
@@ -155,8 +160,8 @@ namespace
             if (! writeTimeTag (bundle.getTimeTag()))
                 return false;
 
-            for (OSCBundle::Element* element = bundle.begin(); element != bundle.end(); ++element)
-                if (! writeBundleElement (*element))
+            for (auto& element : bundle)
+                if (! writeBundleElement (element))
                     return false;
 
             return true;
@@ -201,7 +206,7 @@ namespace
 //==============================================================================
 struct OSCSender::Pimpl
 {
-    Pimpl() noexcept : targetPortNumber (0) {}
+    Pimpl() noexcept  {}
     ~Pimpl() noexcept { disconnect(); }
 
     //==============================================================================
@@ -210,20 +215,31 @@ struct OSCSender::Pimpl
         if (! disconnect())
             return false;
 
-        socket = new DatagramSocket (true);
+        socket.setOwned (new DatagramSocket (true));
         targetHostName = newTargetHost;
         targetPortNumber = newTargetPort;
 
         if (socket->bindToPort (0)) // 0 = use any local port assigned by the OS.
             return true;
 
-        socket = nullptr;
+        socket.reset();
         return false;
     }
 
+    bool connectToSocket (DatagramSocket& newSocket, const String& newTargetHost, int newTargetPort)
+    {
+        if (! disconnect())
+            return false;
+
+        socket.setNonOwned (&newSocket);
+        targetHostName = newTargetHost;
+        targetPortNumber = newTargetPort;
+        return true;
+    }
+
     bool disconnect()
     {
-        socket = nullptr;
+        socket.reset();
         return true;
     }
 
@@ -231,15 +247,17 @@ struct OSCSender::Pimpl
     bool send (const OSCMessage& message, const String& hostName, int portNumber)
     {
         OSCOutputStream outStream;
-        outStream.writeMessage (message);
-        return sendOutputStream (outStream, hostName, portNumber);
+
+        return outStream.writeMessage (message)
+            && sendOutputStream (outStream, hostName, portNumber);
     }
 
     bool send (const OSCBundle& bundle, const String& hostName, int portNumber)
     {
         OSCOutputStream outStream;
-        outStream.writeBundle (bundle);
-        return sendOutputStream (outStream, hostName, portNumber);
+
+        return outStream.writeBundle (bundle)
+            && sendOutputStream (outStream, hostName, portNumber);
     }
 
     bool send (const OSCMessage& message)   { return send (message, targetHostName, targetPortNumber); }
@@ -266,9 +284,9 @@ private:
     }
 
     //==============================================================================
-    ScopedPointer<DatagramSocket> socket;
+    OptionalScopedPointer<DatagramSocket> socket;
     String targetHostName;
-    int targetPortNumber;
+    int targetPortNumber = 0;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Pimpl)
 };
@@ -282,7 +300,7 @@ OSCSender::OSCSender()   : pimpl (new Pimpl())
 OSCSender::~OSCSender()
 {
     pimpl->disconnect();
-    pimpl = nullptr;
+    pimpl.reset();
 }
 
 //==============================================================================
@@ -291,6 +309,11 @@ bool OSCSender::connect (const String& targetHostName, int targetPortNumber)
     return pimpl->connect (targetHostName, targetPortNumber);
 }
 
+bool OSCSender::connectToSocket (DatagramSocket& socket, const String& targetHostName, int targetPortNumber)
+{
+    return pimpl->connectToSocket (socket, targetHostName, targetPortNumber);
+}
+
 bool OSCSender::disconnect()
 {
     return pimpl->disconnect();
@@ -310,7 +333,7 @@ bool OSCSender::sendToIPAddress (const String& host, int port, const OSCBundle&
 class OSCBinaryWriterTests  : public UnitTest
 {
 public:
-    OSCBinaryWriterTests() : UnitTest ("OSCBinaryWriter class") {}
+    OSCBinaryWriterTests() : UnitTest ("OSCBinaryWriter class", "OSC") {}
 
     void runTest()
     {
@@ -633,4 +656,212 @@ public:
 
 static OSCBinaryWriterTests OSCBinaryWriterUnitTests;
 
+//==============================================================================
+class OSCRoundTripTests  : public UnitTest
+{
+public:
+    OSCRoundTripTests() : UnitTest ("OSCRoundTripTests class", "OSC") {}
+
+    void runTest()
+    {
+        beginTest ("Empty OSC message");
+        {
+            OSCMessage outMessage ("/test/empty");
+
+            OSCOutputStream output;
+            output.writeMessage (outMessage);
+
+            OSCInputStream input (output.getData(), output.getDataSize());
+            OSCMessage inMessage = input.readMessage();
+
+            expectEquals (inMessage.size(), 0);
+        }
+
+        beginTest ("OSC message with single argument");
+        {
+            OSCMessage outMessage ("/test/one_arg", 42);
+
+            OSCOutputStream output;
+            output.writeMessage (outMessage);
+
+            OSCInputStream input (output.getData(), output.getDataSize());
+            OSCMessage inMessage = input.readMessage();
+
+            expectEquals (inMessage.size(), 1);
+            expectEquals (inMessage[0].getInt32(), 42);
+        }
+
+        beginTest ("OSC message with multiple arguments");
+        {
+            OSCMessage outMessage ("/test/four_args", 42, 0.5f, String ("foo"), String ("bar"));
+
+            OSCOutputStream output;
+            output.writeMessage (outMessage);
+
+            OSCInputStream input (output.getData(), output.getDataSize());
+            OSCMessage inMessage = input.readMessage();
+
+            expectEquals (inMessage.size(), 4);
+            expectEquals (inMessage[0].getInt32(), 42);
+            expectEquals (inMessage[1].getFloat32(), 0.5f);
+            expectEquals (inMessage[2].getString(), String ("foo"));
+            expectEquals (inMessage[3].getString(), String ("bar"));
+        }
+
+        beginTest ("Empty OSC bundle");
+        {
+            OSCBundle outBundle;
+
+            OSCOutputStream output;
+            output.writeBundle (outBundle);
+
+            OSCInputStream input (output.getData(), output.getDataSize());
+            OSCBundle inBundle = input.readBundle();
+
+            expectEquals (inBundle.size(), 0);
+        }
+
+        beginTest ("OSC bundle with single message");
+        {
+            OSCMessage outMessage ("/test/one_arg", 42);
+            OSCBundle outBundle;
+            outBundle.addElement (outMessage);
+
+            OSCOutputStream output;
+            output.writeBundle (outBundle);
+
+            OSCInputStream input (output.getData(), output.getDataSize());
+            OSCBundle inBundle = input.readBundle();
+
+            expectEquals (inBundle.size(), 1);
+
+            OSCMessage inMessage = inBundle[0].getMessage();
+
+            expectEquals (inMessage.getAddressPattern().toString(), String ("/test/one_arg"));
+            expectEquals (inMessage.size(), 1);
+            expectEquals (inMessage[0].getInt32(), 42);
+        }
+
+        beginTest ("OSC bundle with multiple messages");
+        {
+            OSCMessage outMessage1 ("/test/empty");
+            OSCMessage outMessage2 ("/test/one_arg", 42);
+            OSCMessage outMessage3 ("/test/four_args", 42, 0.5f, String ("foo"), String ("bar"));
+
+            OSCBundle outBundle;
+            outBundle.addElement (outMessage1);
+            outBundle.addElement (outMessage2);
+            outBundle.addElement (outMessage3);
+
+            OSCOutputStream output;
+            output.writeBundle (outBundle);
+
+            OSCInputStream input (output.getData(), output.getDataSize());
+            OSCBundle inBundle = input.readBundle();
+
+            expectEquals (inBundle.size(), 3);
+
+            {
+                OSCMessage inMessage = inBundle[0].getMessage();
+
+                expectEquals (inMessage.getAddressPattern().toString(), String ("/test/empty"));
+                expectEquals (inMessage.size(), 0);
+            }
+            {
+                OSCMessage inMessage = inBundle[1].getMessage();
+
+                expectEquals (inMessage.getAddressPattern().toString(), String ("/test/one_arg"));
+                expectEquals (inMessage.size(), 1);
+                expectEquals (inMessage[0].getInt32(), 42);
+            }
+            {
+                OSCMessage inMessage = inBundle[2].getMessage();
+
+                expectEquals (inMessage.getAddressPattern().toString(), String ("/test/four_args"));
+                expectEquals (inMessage.size(), 4);
+                expectEquals (inMessage[0].getInt32(), 42);
+                expectEquals (inMessage[1].getFloat32(), 0.5f);
+                expectEquals (inMessage[2].getString(), String ("foo"));
+                expectEquals (inMessage[3].getString(), String ("bar"));
+            }
+        }
+
+        beginTest ("OSC bundle containing another bundle");
+        {
+            OSCBundle outBundleNested;
+            outBundleNested.addElement (OSCMessage ("/test/one_arg", 42));
+
+            OSCBundle outBundle;
+            outBundle.addElement (outBundleNested);
+
+            OSCOutputStream output;
+            output.writeBundle (outBundle);
+
+            OSCInputStream input (output.getData(), output.getDataSize());
+            OSCBundle inBundle = input.readBundle();
+
+            expectEquals (inBundle.size(), 1);
+            expect (inBundle[0].isBundle());
+            OSCBundle inBundleNested = inBundle[0].getBundle();
+            expectEquals (inBundleNested.size(), 1);
+            expect (inBundleNested[0].isMessage());
+
+            OSCMessage msg = inBundleNested[0].getMessage();
+
+            expectEquals (msg.getAddressPattern().toString(), String ("/test/one_arg"));
+            expectEquals (msg.size(), 1);
+            expectEquals (msg[0].getInt32(), 42);
+        }
+
+        beginTest ("OSC bundle containing multiple other bundles");
+        {
+            OSCBundle outBundleNested1;
+            outBundleNested1.addElement (OSCMessage ("/test/empty"));
+            OSCBundle outBundleNested2;
+            outBundleNested2.addElement (OSCMessage ("/test/one_arg", 42));
+
+            OSCBundle outBundle;
+            outBundle.addElement (outBundleNested1);
+            outBundle.addElement (outBundleNested2);
+
+            OSCOutputStream output;
+            output.writeBundle (outBundle);
+
+            OSCInputStream input (output.getData(), output.getDataSize());
+            OSCBundle inBundle = input.readBundle();
+
+            expectEquals (inBundle.size(), 2);
+
+            {
+                expect (inBundle[0].isBundle());
+                OSCBundle inBundleNested = inBundle[0].getBundle();
+                expectEquals (inBundleNested.size(), 1);
+                expect (inBundleNested[0].isMessage());
+
+                OSCMessage msg = inBundleNested[0].getMessage();
+
+                expectEquals (msg.getAddressPattern().toString(), String ("/test/empty"));
+                expectEquals (msg.size(), 0);
+            }
+            {
+                expect (inBundle[1].isBundle());
+                OSCBundle inBundleNested = inBundle[1].getBundle();
+                expectEquals (inBundleNested.size(), 1);
+                expect (inBundleNested[0].isMessage());
+
+                OSCMessage msg = inBundleNested[0].getMessage();
+
+                expectEquals (msg.getAddressPattern().toString(), String ("/test/one_arg"));
+                expectEquals (msg.size(), 1);
+                expectEquals (msg[0].getInt32(), 42);
+            }
+        }
+    }
+};
+
+static OSCRoundTripTests OSCRoundTripUnitTests;
+
+//==============================================================================
 #endif // JUCE_UNIT_TESTS
+
+} // namespace juce
