@@ -2,26 +2,31 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
+namespace juce
+{
+
 const OSCTimeTag OSCTimeTag::immediately;
 
 static const uint64 millisecondsBetweenOscAndJuceEpochs = 2208988800000ULL;
@@ -40,9 +45,6 @@ OSCTimeTag::OSCTimeTag (Time time) noexcept
 {
     const uint64 milliseconds = (uint64) time.toMilliseconds() + millisecondsBetweenOscAndJuceEpochs;
 
-    // something went seriously wrong if the line above didn't render the time nonnegative!
-    jassert (milliseconds >= 0);
-
     uint64 seconds = milliseconds / 1000;
     uint32 fractionalPart = uint32 (4294967.296 * (milliseconds % 1000));
 
@@ -78,7 +80,7 @@ bool OSCTimeTag::isImmediately() const noexcept
 class OSCTimeTagTests  : public UnitTest
 {
 public:
-    OSCTimeTagTests() : UnitTest ("OSCTimeTag class") {}
+    OSCTimeTagTests() : UnitTest ("OSCTimeTag class", "OSC") {}
 
     void runTest()
     {
@@ -100,7 +102,7 @@ public:
             expect (! copyTag.isImmediately());
         }
 
-        beginTest ("Conversion to/from Juce Time");
+        beginTest ("Conversion to/from JUCE Time");
 
         {
             Time time;
@@ -148,3 +150,5 @@ public:
 static OSCTimeTagTests OSCTimeTagUnitTests;
 
 #endif
+
+} // namespace juce
