@@ -2,29 +2,30 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-#ifndef JUCE_OSCTIMETAG_H_INCLUDED
-#define JUCE_OSCTIMETAG_H_INCLUDED
-
+namespace juce
+{
 
 //==============================================================================
 /**
@@ -56,14 +57,14 @@ public:
     /** Constructs an OSCTimeTag object from a raw binary OSC time tag. */
     OSCTimeTag (uint64 rawTimeTag) noexcept;
 
-    /** Constructs an OSCTimeTag object from a Juce Time object. */
+    /** Constructs an OSCTimeTag object from a juce::Time object. */
     OSCTimeTag (Time time) noexcept;
 
-    /** Returns a Juce Time object representing the same time as the OSCTimeTag.
+    /** Returns a juce::Time object representing the same time as the OSCTimeTag.
 
         If the OSCTimeTag has the special value representing "immediately", the
-        resulting Juce Time object will represent an arbitrary point of time (but
-        guaranteed to be in the past), since Juce Time does not have such a special value.
+        resulting juce::Time object will represent an arbitrary point of time (but
+        guaranteed to be in the past), since juce::Time does not have such a special value.
     */
     Time toTime() const noexcept;
 
@@ -81,5 +82,4 @@ private:
     uint64 rawTimeTag;
 };
 
-
-#endif // JUCE_OSCTIMETAG_H_INCLUDED
+} // namespace juce
