@@ -2,36 +2,49 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2015 - ROLI Ltd.
+   Copyright (c) 2017 - ROLI Ltd.
 
-   Permission is granted to use this software under the terms of either:
-   a) the GPL v2 (or any later version)
-   b) the Affero GPL v3
+   JUCE is an open source library subject to commercial or open-source
+   licensing.
 
-   Details of these licenses can be found at: www.gnu.org/licenses
+   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
+   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
+   27th April 2017).
 
-   JUCE is distributed in the hope that it will be useful, but WITHOUT ANY
-   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-   A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+   End User License Agreement: www.juce.com/juce-5-licence
+   Privacy Policy: www.juce.com/juce-5-privacy-policy
 
-   ------------------------------------------------------------------------------
+   Or: You may also use this code under the terms of the GPL v3 (see
+   www.gnu.org/licenses).
 
-   To release a closed-source product which uses JUCE, commercial licenses are
-   available: visit www.juce.com for more information.
+   JUCE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES, WHETHER
+   EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR PURPOSE, ARE
+   DISCLAIMED.
 
   ==============================================================================
 */
 
-CameraDevice::CameraDevice (const String& nm, int index, int minWidth, int minHeight, int maxWidth, int maxHeight,
-                            bool highQuality)
-   : name (nm), pimpl (new Pimpl (name, index, minWidth, minHeight, maxWidth, maxHeight, highQuality))
+namespace juce
+{
+
+#if JUCE_MAC || JUCE_IOS
+ #include "../native/juce_mac_CameraDevice.h"
+#elif JUCE_WINDOWS
+ #include "../native/juce_win32_CameraDevice.h"
+#elif JUCE_ANDROID
+ #include "../native/juce_android_CameraDevice.h"
+#endif
+
+//==============================================================================
+CameraDevice::CameraDevice (const String& nm, int index, int minWidth, int minHeight, int maxWidth, int maxHeight, bool useHighQuality)
+   : name (nm), pimpl (new Pimpl (name, index, minWidth, minHeight, maxWidth, maxHeight, useHighQuality))
 {
 }
 
 CameraDevice::~CameraDevice()
 {
     stopRecording();
-    pimpl = nullptr;
+    pimpl.reset();
 }
 
 Component* CameraDevice::createViewerComponent()
@@ -79,14 +92,14 @@ StringArray CameraDevice::getAvailableDevices()
 CameraDevice* CameraDevice::openDevice (int index,
                                         int minWidth, int minHeight,
                                         int maxWidth, int maxHeight,
-                                        bool highQuality)
+                                        bool useHighQuality)
 {
-    ScopedPointer<CameraDevice> d (new CameraDevice (getAvailableDevices() [index], index,
-                                                     minWidth, minHeight, maxWidth, maxHeight,
-                                                     highQuality));
-
-    if (d->pimpl->openedOk())
-        return d.release();
+    if (ScopedPointer<CameraDevice> d = new CameraDevice (getAvailableDevices() [index], index,
+                                                          minWidth, minHeight, maxWidth, maxHeight, useHighQuality))
+        if (d->pimpl->openedOk())
+            return d.release();
 
     return nullptr;
 }
+
+} // namespace juce
